; $Id: dcs_cryst_align.pro,v 1.17.2.2 2004/05/28 19:11:04 azuah Exp $
; This program plots crystal alignment files
; Yiming Qiu (Yiming.Qiu@nist.gov) 4/4/2003

;circle symbol
pro dcs_cryst_circle,radius=radius,fill=fill,thick=thick
    if n_elements(radius) eq 0 then radius=1
    if n_elements(fill)   eq 0 then fill=0
    if n_elements(thick)  eq 0 then thick=1
    x = findgen(16)*(!pi*2/15)
    usersym,radius*cos(x),radius*sin(x),fill=fill,thick=thick
end

pro dcs_cryst_align_e2lambda_event,event
    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=state

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
        ok = dialog_message(dialog_parent=state.tlb,!error_state.msg,/error)
        catch,/cancel
        return
    end

    case event.id of
        state.eTxt:     dcs_cryst_align_e2lambda_update,state,/E
        state.lTxt:     dcs_cryst_align_e2lambda_update,state,/lambda
        state.plotBut:  launch_DCSToolsPerformance,event
        state.exitBut:  widget_control,event.handler,/destroy
        else:
    endcase
end

pro dcs_cryst_align_e2lambda_update,state,E=E,lambda=lambda
    eng = !values.f_nan & wav = !values.f_nan   ;initialization
    ;retrieve values from text boxes
    if keyword_set(E) then begin
        widget_control,state.eTxt,get_value=str
        tmp = dm_to_number(str)
        if finite(tmp) then begin
            eng = tmp
            if eng gt 0 then begin
                wav = sqrt(81.8042/eng)
                widget_control,state.lTxt,set_value=dm_to_string(wav)
            endif
        endif
    endif
    if keyword_set(lambda) then begin
        widget_control,state.lTxt,get_value=str
        tmp = dm_to_number(str)
        if finite(tmp) then begin
            wav = tmp
            if wav gt 0 then begin
                eng = 81.8042/wav/wav
                widget_control,state.eTxt,set_value=dm_to_string(eng)
            endif
        endif
    endif
    ;find out the energy resolution and flux
    if finite(eng) and finite(wav) then begin
        f_low  = interpol((*(state.flPtr))[1,*],(*(state.flPtr))[0,*],wav)/state.fmax*100.0
        f_med  = interpol((*(state.fmPtr))[1,*],(*(state.fmPtr))[0,*],wav)/state.fmax*100.0
        flux   = 'low:'+strtrim(string(0>(f_low)<100,format='(f6.2)'),2)+', '+$
                 'med:'+strtrim(string(0>(f_med)<100,format='(f6.2)'),2)
        widget_control,state.fxTxt,set_value=flux
        er_low = 13220./wav^2.947
        er_med = 3948./wav^2.656
        eresl  = 'low:'+strtrim(string(er_low,format='(f9.1)'),2)+', '+$
                 'med:'+strtrim(string(er_med,format='(f9.1)'),2)
        widget_control,state.erTxt,set_value=eresl
    endif else begin
        widget_control,state.fxTxt,set_value=''
        widget_control,state.erTxt,set_value=''
        if keyword_set(E) then  widget_control,state.lTxt,set_value=''
        if keyword_set(lambda) then widget_control,state.eTxt,set_value=''
    endelse
end

pro dcs_cryst_align_e2lambda_Exit,tlb
    widget_control,tlb,get_uvalue=state
    ptr_free,state.flPtr,state.fmPtr
    widget_control,tlb,/destroy
end

;E to Lambda conversion
pro dcs_cryst_align_e2lambda,event,lambda=lambda
    state={group_leader:    0L, $   ;group leader
           tlb:             0L, $   ;top level base
           eTxt:            0L, $   ;E text box
           lTxt:            0L, $   ;lambda text box
           erTxt:           0L, $   ;E resolution text box
           fxTxt:           0L, $   ;flux text box
           plotBut:         0L, $   ;start performance button
           exitBut:         0L, $   ;exit button
           fmax:            0e, $   ;maximum flux
           flPtr:    ptr_new(), $   ;low resl mode flux
           fmPtr:    ptr_new()  $   ;medium resl mode flux
          }
    registerName = 'dcs_cryst_align_e2lambda'
    if xregistered(registerName) then begin ;only allow one copy running
        id = LookupManagedWidget(registername)
        widget_control,id,iconify=0
        return
    endif
    if n_elements(lambda) eq 0 then lambda=!values.f_nan
    if n_elements(event) ne 0 then begin
      state.group_leader = event.top
      state.tlb = widget_base(title='Convert Energy to Wavelength',/col,kill_notify='dcs_cryst_align_e2lambda_Exit',$
                  tlb_frame_attr=1,group_leader=event.top)
    endif else $
      state.tlb = widget_base(title='Convert E to Wavelength',/col,kill_notify='dcs_cryst_align_e2lambda_Exit',$
                  tlb_frame_attr=1)
    row1        = widget_base(state.tlb,/row,/grid_layout)
    col1        = widget_base(row1,/col,/grid_layout)
    col2        = widget_base(row1,/col,/grid_layout)
    void        = widget_label(col1,value='E (meV)',/align_center)
    state.eTxt  = widget_text(col1,/editable,/all_events)
    void        = widget_label(col2,value='Lambda ('+string('c5'xb)+')',/align_center)
    state.lTxt  = widget_text(col2,/editable,/all_events,value=dm_to_string(lambda))
    void        = widget_label(col1,value='FWHM ('+string('b5'xb)+'eV)',/align_center)
    state.erTxt = widget_text(col1)
    void        = widget_label(col2,value='Flux (% of maximum)',/align_center)
    state.fxTxt = widget_text(col2)
    row2        = widget_base(state.tlb,/row,/align_center)
    state.plotBut = widget_button(row2,value='  Plot  ',sensitive=xregistered('dave',/noshow))
    void          = widget_label(row2,value= '   ')
    state.exitBut = widget_button(row2,value='  Exit  ')
    ;flux info
    state.flPtr = ptr_new([[1.25,357], [1.3,1799.4], [1.35,30630], [1.36,44460], [1.38,69780], [1.4,86595], [1.42,86820], [1.44,59370], [1.45,57570], [1.46,73200],$
                    [1.48,117900], [1.5,137498], [1.52,143280], [1.54,145380], [1.55,135750], [1.56,106530], [1.58,33600], [1.6,34440], [1.62,122010], [1.64,177630],$
                    [1.65,172500], [1.66,158280], [1.68,167760], [1.7,212850], [1.72,165270], [1.74,68310], [1.75,51450], [1.76,71100], [1.78,186870], [1.8,266385],$
                    [1.82,265980], [1.84,216420], [1.85,177360], [1.86,131010], [1.88,43860], [1.9,4959], [1.95,3192], [1.96,7602], [1.98,32730], [2,87915],$
                    [2.02,116220], [2.04,60900], [2.05,28993.5], [2.06,10188], [2.08,820.8], [2.1,167.22], [2.15,385.5], [2.2,196260], [2.25,541500], [2.3,720400],$
                    [2.4,803100], [2.5,875400], [2.6,905700], [2.7,912600], [2.8,902400], [2.9,898500], [3,901350], [3.1,888300], [3.2,876600], [3.25,855900],$
                    [3.27,826800], [3.3,760800], [3.33,390900], [3.35,346200], [3.38,731100], [3.4,831900], [3.42,839400], [3.45,846000], [3.5,861300], [3.6,870300],$
                    [3.7,877800], [3.8,859500], [3.85,844500], [3.9,832200], [3.95,807000], [3.98,781200], [4,763800], [4.03,734700], [4.05,787800], [4.08,850500],$
                    [4.1,872400], [4.15,857400], [4.2,847200], [4.3,828900], [4.4,798300], [4.5,765900], [4.6,738000], [4.62,719400], [4.65,716400], [4.67,746400],$
                    [4.7,780000], [4.72,777900], [4.75,772800], [4.78,766200], [4.8,763950], [5,721650], [5.5,645900], [6,558000], [6.3,508200], [6.4,487500],$
                    [6.45,466500], [6.5,453600], [6.55,405600], [6.6,297180], [6.65,33900], [6.7,24780], [6.75,279690], [6.8,384600], [6.85,402600], [6.9,413700],$
                    [6.95,406200], [7,404850], [7.5,349800], [8,292040], [9,199770], [10,142250], [12,69260], [14,34890], [15,25104], [16,17742],$
                    [18,9459], [20,5184]])
    state.fmPtr = ptr_new([[1.35,3894], [1.4,11083.5], [1.45,4354.5], [1.5,17070], [1.55,17355], [1.6,1409.55], [1.65,22560], [1.7,27450], [1.75,3330], [1.8,34275],$
                    [1.85,22425], [1.9,207.45], [1.95,322.2], [2,11304], [2.05,1801.5], [2.1,130.185], [2.15,150.9], [2.2,28005], [2.25,72900], [2.3,92160],$
                    [2.4,102165], [2.5,112905], [2.6,117765], [2.7,120540], [2.8,117645], [2.9,119475], [3,122535], [3.2,117090], [3.25,114390], [3.3,103335],$
                    [3.35,31950], [3.4,112305], [3.5,116445], [3.6,120810], [3.7,120810], [3.8,119025], [3.85,118320], [3.9,115035], [3.95,111540], [4,105735],$
                    [4.05,106050], [4.1,123960], [4.15,121905], [4.2,121185], [4.3,117795], [4.4,113085], [4.6,103245], [4.8,110460], [5,103703], [5.5,91095],$
                    [6,77842.5], [6.5,59820], [6.6,36780], [6.7,952.5], [6.8,49335], [6.9,53190], [7,54127.5], [7.5,43965], [8,36540], [9,24315],$
                    [10,16657.5], [12,7679.25], [14,3831], [16,1926], [18,1006.58], [20,554.55]])
    state.fmax = max((*(state.flPtr))[1,*])
    dm_center_kid,state.tlb,state.group_leader
    widget_control,state.tlb,/realize
    dcs_cryst_align_e2lambda_update,state,/lambda
    widget_control,state.tlb,set_uvalue=state
    xmanager,registerName,state.tlb,cleanup='dcs_cryst_align_e2lambda_Exit',/no_block
end

;plotting
pro dcs_cryst_align_Plot,pState,chars=chars,thick=thick
    xtit = ['Rotation (!6!Eo!n)','Height (mm)','X-translation (mm)','Y-translation (mm)','X-tilt (!6!Eo!n)','Y-tilt (!6!Eo!n)',$
            'File Index','T (K)']
    xtit = xtit[(*pState).xaxis]
    ytit = 'Neutron Counts/'+strtrim(string((*pState).duration,format='(i5)'),2)+' Secs'
    tit  = '!6!u'+(*pState).det_tit+', '+(*pState).tch_tit+'!n'
    n_file = n_elements((*(*pState).yPtr)[0,*])
    y = fltarr(n_file) & dy = y
    for i=0,n_file-1 do begin
       y[i]  = total((*(*pState).yPtr)[*(*pState).detPtr,i])
       dy[i] = sqrt(total((*(*pState).dyPtr)[*(*pState).detPtr,i]^2))
    endfor
    xmax = max((*(*pState).xPtr)[(*pState).xaxis,*],min=xmin)
    ymax = max(y+dy,imax) & imax = imax[0]
    ymin = min(y-dy)
    if total(finite((*pState).xran)) ne 2 then begin
       (*pState).xran = [xmin,xmax]+[-0.05,0.05]*(xmax-xmin+abs(xmax+1)/10.)
       (*pState).yran = [ymin,ymax]+[-0.05,0.05]*(ymax-ymin)
    endif
    if n_elements(thick) eq 0 then thick = 1.5
    if n_elements(chars) eq 0 then chars = 1.2
    dcs_cryst_circle,thick=thick
    ;adjust xmargin according to the ticktext
    otmp = obj_new('idlgrAxis',1,/exact,range=(*pState).yran)
    otmp->getproperty,ticktext=otick
    otick->getproperty,strings=ticktext
    max_len = max(dm_strlen(ticktext))
    obj_destroy,otmp
    xmargin = [max_len+5,2.5]
    ymargin = [4,2.5]
    plot,(*(*pState).xPtr)[(*pState).xaxis,*],y,xrange=(*pState).xran,yrange=(*pState).yran,/xst,/yst,$
       title=tit,xtitle=xtit,ytitle=ytit,linestyle=0,thick=thick,psym=8,xthick=thick,ythick=thick,xminor=2,yminor=2,$
       chars=chars,xmargin=xmargin,ymargin=ymargin
    ;the following two lines are for multiple dcs_cryst_align running at the same time
    *(*pState).xaxisPtr = !x
    *(*pState).yaxisPtr = !y
    if float(!version.release) ge 5.6 then $
       errplot,(*(*pState).xPtr)[(*pState).xaxis,*],y-dy,y+dy,thick=thick $
    else $
       errplot,(*(*pState).xPtr)[(*pState).xaxis,*],y-dy,y+dy
    index = uniq((*(*pState).xPtr)[(*pState).xaxis,*],sort((*(*pState).xPtr)[(*pState).xaxis,*]))
    if ((*pState).fit eq 1) and (n_elements(index) ne 1) and (n_elements(y) gt 4)then begin
       fwhm = total(y-ymin)/(ymax-ymin)*$
              abs((*(*pState).xPtr)[(*pState).xaxis,1]-(*(*pState).xPtr)[(*pState).xaxis,0])
       estimates    = [ymax,(*(*pState).xPtr)[(*pState).xaxis,imax],fwhm/sqrt(8.*alog(2)),ymin]
       estimates[2] = estimates[2]/2.
       result = gaussfit((*(*pState).xPtr)[(*pState).xaxis,*],y,params,estimates=estimates,nterms=n_elements(estimates))
       xx = xmin+findgen(100)/99*(xmax-xmin)
       yy = params[0]*exp(-0.5*((xx-params[1])/params[2])^2)+params[3]
       oplot,xx,yy,lines=0,thick=thick
       xyouts,(*pState).xran[0]+((*pState).xran[1]-(*pState).xran[0])*0.05,$
            (*pState).yran[0]+((*pState).yran[1]-(*pState).yran[0])*0.9,$
         'peak!l !n='+strtrim(string(params[1],format='(f8.2)'),2),chars=chars/1.2
       xyouts,(*pState).xran[0]+((*pState).xran[1]-(*pState).xran[0])*0.05,$
            (*pState).yran[0]+((*pState).yran[1]-(*pState).yran[0])*0.81,$
         'fwhm='+strtrim(string(params[2]*sqrt(8.*alog(2)),format='(f9.3)'),2),chars=chars/1.2
    endif
end

;calculate q length from direct crystal lattice parameters
function dcs_cryst_align_qlength,hkl,cryst
    ddtor   = !dpi/(180d)
    sinabg  = sin(cryst[3:5]*ddtor)
    cosabg  = cos(cryst[3:5]*ddtor)
    v       = cryst[0]*cryst[1]*cryst[2]*sqrt((1d)+(2d)*cosabg[0]*cosabg[1]*cosabg[2]-total(cosabg[0:2]^2))
    rec_abc = (2d)*!dpi/v*[cryst[1]*cryst[2]*sinabg[0],cryst[0]*cryst[2]*sinabg[1],cryst[0]*cryst[1]*sinabg[2]]
    tmp     = (4d)*!dpi*!dpi/v/v*cryst[0]*cryst[1]*cryst[2]
    aadotbb = tmp*cryst[2]*(cosabg[0]*cosabg[1]-cosabg[2])
    bbdotcc = tmp*cryst[0]*(cosabg[1]*cosabg[2]-cosabg[0])
    ccdotaa = tmp*cryst[1]*(cosabg[2]*cosabg[0]-cosabg[1])
    return, sqrt(total((hkl*rec_abc)^2)+(2d)*(hkl[0]*hkl[1]*aadotbb+hkl[1]*hkl[2]*bbdotcc+hkl[2]*hkl[0]*ccdotaa))
end

pro dcs_cryst_align_zoom,event
    widget_control,event.top,get_uvalue = pState
    case event.type of
       0:    begin   ; button press
             (*pState).mouse = event.press
             if (*pState).mouse eq 1 then begin
                (*pState).xbox[*] = event.x
                (*pState).ybox[*] = event.y
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                empty
                widget_control,(*pState).plotDraw,/draw_motion_events
             endif
         end
       1:    begin  ; button release
             widget_control,(*pState).plotDraw,draw_motion_events = 0
             if (*pState).mouse eq 1 then begin
                xll = (*pState).xbox[0] < (*pState).xbox[1]
                yll = (*pState).ybox[0] < (*pState).ybox[1]
                w   = abs((*pState).xbox[1] - (*pState).xbox[0])
                h   = abs((*pState).ybox[1] - (*pState).ybox[0])
                if (w eq 0) or (h eq 0) then break
                xur = xll + w
                yur = yll + h
                ;the following two lines are for multiple dcs_cryst_align running at the same time
                !x = *(*pState).xaxisPtr
                !y = *(*pState).yaxisPtr
                ll = convert_coord(xll,yll,/device,/to_data)
                ur = convert_coord(xur,yur,/device,/to_data)
                if (ll[0] eq ur[0]) or (ll[1] eq ur[1]) then break
                (*pState).xran = [ll[0],ur[0]]
                (*pState).yran = [ll[1],ur[1]]
                wset,(*pState).winPix
                dcs_cryst_align_plot,pState
                wset,(*pState).winVis
                device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                (*pState).mouse = 0B
                (*pState).zoom  = 1B
             endif
             if (*pState).mouse eq 4 then begin
                (*pState).mouse = 0B
                if (*pState).zoom then begin
                    (*pState).zoom = 0B
                    (*pState).xran[*] = !values.f_nan
                    (*pState).yran[*] = !values.f_nan
                    wset,(*pState).winPix
                    dcs_cryst_align_plot,pState
                    wset,(*pState).winVis
                    device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                endif else $
                    widget_displaycontextmenu,event.id,event.x,event.y,(*pState).popMenu
             endif
         end
       2:    begin  ; mouse motion
              if (*pState).mouse eq 1 then begin
                (*pState).xbox[1] = event.x
                (*pState).ybox[1] = event.y
                xc = [(*pState).xbox[0],event.x,event.x,(*pState).xbox[0],(*pState).xbox[0]]
                yc = [(*pState).ybox[0],(*pState).ybox[0],event.y,event.y,(*pState).ybox[0]]
                wset,(*pState).winVis
                device,copy=[0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
                plots,xc,yc,/device,color=256L*255
                empty
              endif
         end
       else:
    endcase
end

pro dcs_cryst_align_event,event
    compile_opt IDL2    ;,strictarrsubs
                        ;idl2=defint32,strictarr
                        ;strictarr:   [] to index array
                        ;strictarrsubs: error when out-of-range indices,IDL5.6 or after
    WIDGET_CONTROL,/HOURGLASS
    widget_control,event.handler,get_uvalue=pState
    device,decomposed=1

    if (*pState).enotebook then $
        IF DAVE_SET_FOCUS(EVENT) THEN RETURN

    widget_type = widget_info(event.id,/type)
    case widget_type of
       8: tmp_value = widget_info(event.id,/droplist_select)  ;droplist
       12:tmp_value = event.index                             ;combobox
       else: tmp_value = 0
    endcase

    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       ok=dialog_message(dialog_parent=(*pState).tlb,!error_state.msg,/error)
       catch,/cancel
       return
    end

    case (event.id) of
       (*pState).tlb:    widget_control,event.id,scr_xsize=(*pState).geom[0],scr_ysize=(*pState).geom[1]
       (*pState).loadBut: begin
          (*pState).fit = 0
          (*pState).filesel->getproperty,path=open_path,file=open_file
          if n_elements(open_file) eq 0 then begin
              ok = dialog_message('Please select DCS files first.',/error,dialog_parent=event.handler)
              break
          endif
          e_range = (*pState).e_ran
          dm_load_dcsdata,open_path,open_file,ang=x,qty=y,dqty=dy,t_chan=(*pState).t_chan,e_range=e_range,$
              parent=(*pState).tlb,duration=duration,/raw
          if n_elements(e_range) ne 0 then (*pState).e_ran = e_range
          *(*pState).xPtr    = temporary(x)
          *(*pState).yPtr    = temporary(y)
          *(*pState).dyPtr   = temporary(dy)
          (*pState).duration = fix(duration,type=3)
          (*pState).xran[*]  = !values.f_nan
          (*pState).yran[*]  = !values.f_nan
          wset,(*pState).winPix
          dcs_cryst_align_plot,pState
          wset,(*pState).winVis
          device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
         end
       (*pState).fitBut:  begin
          if n_elements(*(*pState).yPtr) le 4 then break
          (*pState).fit = 1
          wset,(*pState).winPix
          dcs_cryst_align_plot,pState
          wset,(*pState).winVis
          device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
         end
       (*pState).saveBut: begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          mydevice = !d.name
          set_plot,'ps',/copy
          file = dm_choose_file('ps',/write,dialog_parent=(*pState).tlb,deviceKeywords=deviceKeywords)
          widget_control,/hourglass    ;busy signal
          if strlen(file) ne 0 then begin
              deviceKeywords.color = 0 ;black and white
              device,_extra=deviceKeywords
              dcs_cryst_align_plot,pState,chars=1.6
              device,/close_file
          endif
          set_plot,mydevice
         end
       (*pState).dataBut:  begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          file = dm_choose_file('txt',/write,dialog_parent=(*pState).tlb,title='Save plot data as an ASCII file.')
          if strlen(file) eq 0 then break
          openw,unit,file,/get_lun,error=openerr
          if openerr ne 0 then begin
            ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=(*pState).tlb)
          endif else begin
            n_file = n_elements((*(*pState).yPtr)[0,*])
            widget_control,/hourglass  ;busy signal
            for i=0,n_file-1 do begin
                x  = (*(*pState).xPtr)[(*pState).xaxis,i]
                y  = total((*(*pState).yPtr)[*(*pState).detPtr,i])
                dy = sqrt(total((*(*pState).dyPtr)[*(*pState).detPtr,i]^2))
                printf,unit,x,' ',y,' ',dy
            endfor
            free_lun,unit
          endelse
         end
       (*pState).printBut: begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          ok = dialog_printersetup(dialog_parent=(*pState).tlb)
          if not ok then break
          mydevice = !d.name
          set_plot,'printer'
          device,xsize=7,ysize=5,xoffset=0.75,/inch
          dcs_cryst_align_plot,pState,chars=1.6
          device,/close_document
          set_plot,mydevice
         end
       (*pState).exitBut: begin
          widget_control,event.handler,/destroy
          return
         end
       (*pState).detList: begin
          old = *(*pState).detPtr
          case tmp_value of
              0: begin
                 *(*pState).detPtr = indgen((*pState).ndet)
                 (*pState).det_tit = '!6det=all'
                 end
              1: begin
                 n_index = n_elements(*(*pState).detPtr)
                 if n_index eq (*pState).ndet then default='1:'+strtrim(string((*pState).ndet),2) $
                 else default=string(*(*pState).detPtr+1,format='('+string(n_index)+'(i3,","))')
                 if strmid(default,0,/reverse_offset) eq ',' then default=strmid(default,0,strlen(default)-1)
                 tmp = dm_dialog_input(dialog_parent=event.handler,'Det num:',default=default,title='Specifying detectors',$
                   info='Detector number starts from 1. eg:371,374 or 370:374 c',xsize=200)
                 (*pState).det_tit = '!6det='+tmp
                 if strlen(tmp) eq 0 then det_tmp=-1 $
                 else begin
                   status = ''
                   index  = strpos(tmp,'c',/reverse_search)
                   if index[0] ne -1 then status='c'
                   index = strpos(tmp,'C',/reverse_search)
                   if index[0] ne -1 then status='c'
                   index = strpos(tmp,'u',/reverse_search)
                   if index[0] ne -1 then status='u'
                   index = strpos(tmp,'U',/reverse_search)
                   if index[0] ne -1 then status='u'
                   index = strpos(tmp,'l',/reverse_search)
                   if index[0] ne -1 then status='l'
                   index = strpos(tmp,'L',/reverse_search)
                   if index[0] ne -1 then status='l'

                   ind0  = strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex)
                   ind1  = strsplit(tmp,':-')
                   index = -1L
                   for i=1,n_elements(ind1)-1 do begin
                    ind2 = where(ind0 eq ind1[i],count)
                    if count ne 0 then index=[index,ind2]
                   endfor
                   n_index = n_elements(index)
                   if n_index ne 1 then index=index[1:n_index-1]-1
                   tmp = strsplit(tmp,'[a-zA-Z '+string(9b)+',:-]',/regex,/extract)
                   det_tmp = -1L
                   for i=0,n_elements(tmp)-1 do begin
                    tmp1 = dm_to_number(tmp[i],/int)
                    if finite(tmp1) then begin
                        ind0 = where(index eq i,count)
                        if count ne 0 then begin
                           i = i+1
                           tmp2 = dm_to_number(tmp[i],/int)
                           if finite(tmp2) then det_tmp=[det_tmp,tmp1+findgen(abs(tmp2-tmp1)+1)] $
                           else det_tmp=[det_tmp,tmp1]
                        endif else det_tmp=[det_tmp,tmp1]
                    endif
                   endfor
                   index = where(det_tmp gt 0,count)
                   if count ne 0 then det_tmp=fix(det_tmp[index],type=3)-1 $
                   else det_tmp = indgen((*pState).ndet)
                   case status of
                    'c': det_tmp = dm_common(det_tmp,dm_mask_detbanks(0),/no_copy)
                    'l': det_tmp = dm_common(det_tmp,dm_mask_detbanks(-1),/no_copy)
                    'u': det_tmp = dm_common(det_tmp,dm_mask_detbanks(1),/no_copy)
                    else:det_tmp = dm_common(det_tmp,indgen((*pState).ndet),/no_copy)
                   endcase
                 endelse
                 if (det_tmp[0] eq -1) or (n_elements(det_tmp) eq (*pState).ndet) then begin
                   *(*pState).detPtr = indgen((*pState).ndet)
                   (*pState).det_tit = '!6det=all'
                   dm_set_droplist,(*pState).detList,select=0
                 endif else begin
                   *(*pState).detPtr = det_tmp
                   tth  =   strtrim(string(mean(abs((*(*pState).tthPtr)[det_tmp])),format='(f9.2)'),2)
                   if n_elements(det_tmp) gt 1 then tth='(<2!7h!6>='+tth+'!9%!6)' $
                   else tth='(2!7h!6='+tth+'!9%!6)'
                   (*pState).det_tit = (*pState).det_tit+tth
                 endelse
                 end
              2: begin
                 *(*pState).detPtr = dm_mask_detbanks(0)
                 (*pState).det_tit = '!6det=central'
                 end
              3: begin
                 *(*pState).detPtr = dm_mask_detbanks(1)
                 (*pState).det_tit = '!6det=upper'
                 end
              4: begin
                 *(*pState).detPtr = dm_mask_detbanks(-1)
                 (*pState).det_tit = '!6det=lower'
                 end
              else:begin
                 *(*pState).detPtr = tmp_value-5
                 tth = strtrim(dm_to_string(abs((*(*pState).tthPtr)[tmp_value-5])),2)
                 (*pState).det_tit = '!6det='+strtrim(string(tmp_value-4),2)+'(2!7h!6='+tth+'!9%!6)'
                 end
          endcase
          n_old = n_elements(old)
          n_new = n_elements(*(*pState).detPtr)
          comm  = dm_common(old,*(*pState).detPtr)
          same  = (n_old eq n_new) and (n_old eq n_elements(comm)) and (comm[0] ne -1)
          old   = -1 & comm = -1
          if (n_elements(*(*pState).yPtr) ne 0) and (1-same) then begin   ;reload data
              (*pState).xran[*] = !values.f_nan
              (*pState).yran[*] = !values.f_nan
              wset,(*pState).winPix
              dcs_cryst_align_plot,pState
              wset,(*pState).winVis
              device,copy=[0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
          endif
         end
       (*pState).timeList:   if ((*pState).t_chan ne tmp_value) or (tmp_value eq 2) then begin
          (*pState).t_chan = tmp_value
          if (*pState).t_chan eq 0 then (*pState).tch_tit='tchan=all' $
          else if (*pState).t_chan eq 1 then (*pState).tch_tit='tchan=elastic' $
          else begin
              default = dm_to_string((*pState).e_ran)
              new     = dm_dialog_input(['from:','to:'],title='Energy range',xsize=100,default=default,/float,$
                        dialog_parent=(*pState).tlb,info='In units of meV')
              (*pState).t_chan = 1
              (*pState).e_ran[*] = !values.f_nan
              if total(finite(new)) eq 2 then $
                if new[1] gt new[0] then begin
                    (*pState).e_ran   = new
                    (*pState).t_chan  = 2
                    (*pState).tch_tit = 'E=['+dm_to_string(new[0])+','+dm_to_string(new[1])+']meV'
                 endif
              if (*pState).t_chan eq 1 then begin
                ok = dialog_message('Invalid energy range. Elastic peak option is used.',dialog_parent=(*pState).tlb)
                dm_set_droplist,event.id,select=1
                (*pState).tch_tit = 'tchan=elastic'
              endif
          endelse
          (*pState).filesel->getproperty,file=open_file
          if n_elements(open_file) ne 0 then begin ;reload data
              tlb  = (*pState).tlb
              load = (*pState).loadBut
              dcs_cryst_align_event,{WIDGET_BUTTON,ID:load,TOP:tlb,HANDLER:tlb,SELECT:1}
              return
          endif
         endif
       (*pState).xaxisList:begin
          (*pState).xaxis = tmp_value
          (*pState).fit   = 0
          if n_elements(*(*pState).xPtr) ne 0 then begin
              (*pState).xran[*] = !values.f_nan
              (*pState).yran[*] = !values.f_nan
              wset,(*pState).winPix
              dcs_cryst_align_plot,pState
              wset,(*pState).winVis
              device,copy=[0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
          endif
         end
       (*pState).plotDraw:begin
          if n_elements(*(*pState).xPtr) eq 0 then break
          dcs_cryst_align_zoom,event
         end
       (*pState).xranBut:begin
          old = dm_to_string((*pState).xran)
          new = dm_dialog_input(['from:','to:'],title='Set X Range',xsize=100,default=old,dialog_parent=(*pState).tlb)
          if (old[0] ne new[0]) or (old[1] ne new[1]) then begin
            new = dm_to_number(new)
            if total(finite(new)) eq 2 then $
              if new[1] gt new[0] then begin
                (*pState).xran = new
                wset,(*pState).winPix
                dcs_cryst_align_plot,pState
                wset,(*pState).winVis
                device,copy=[0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
              endif
          endif
         end
       (*pState).yranBut:begin
          old = dm_to_string((*pState).yran)
          new = dm_dialog_input(['from:','to:'],title='Set Y Range',xsize=100,default=old,dialog_parent=(*pState).tlb)
          if (old[0] ne new[0]) or (old[1] ne new[1]) then begin
            new = dm_to_number(new)
            if total(finite(new)) eq 2 then $
              if new[1] gt new[0] then begin
                (*pState).yran = new
                wset,(*pState).winPix
                dcs_cryst_align_plot,pState
                wset,(*pState).winVis
                device,copy=[0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
              endif
          endif
         end
       (*pState).calctthBut:begin
         default = float(*(*pState).infoPtr)
         info    = dm_dialog_input(['lambda','a','b','c','alpha','beta','gamma','H','K','L'],$
                    default=default,/float,dialog_parent=(*pState).tlb,xsize=80,title='Calculate 2theta')
         if total(finite(info)) eq n_elements(default) then begin
            *(*pState).infoPtr=double(info)
            ki = 2.*!dpi/info[0]
            q  = dcs_cryst_align_qlength(info[7:9],info[1:6])
            if q/2. lt ki then begin
                mesg = ''
                tth  = asin(q/2./ki)*2/!dtor
                idet = where(*(*pState).tthPtr ge tth,count)
                if count ne 0 then begin
                    if idet[0]-1 gt 0 then begin
                        if abs((*(*pState).tthPtr)[idet[0]-1]-tth) lt abs((*(*pState).tthPtr)[idet[0]]-tth) then $
                             idet[0]=idet[0]-1
                    endif
                    tmp  = [idet[0],(912<(idet[0]+1)),(0>(idet[0]-1))]
                    idet = dm_common(tmp,dm_mask_detbanks(0),count)
                    if count eq 0 then $
                        mesg = 'The corresponding detector number is '+dm_to_string(tmp[0]+1,/int)+'.'  $
                    else $
                        mesg = 'The corresponding central bank detector number is '+dm_to_string(idet[0]+1,/int)+'.'
                endif else $
                    mesg = 'It is beyond the detector range.'
                tth  = strtrim(string(tth,format='(f9.2)'),2)+string('b0'xb)
                mesg = ['Two theta angle for ['+dm_to_string(info[7])+','+dm_to_string(info[8])+','+$
                       dm_to_string(info[9])+'] is '+tth+'.',mesg]
            endif else begin
                lmax = strtrim(string(4.*!dpi/q,format='(f9.2)'),2)+string('c5'XB)
                mesg = 'Incident wavelength must be smaller than '+lmax+' to view ['+dm_to_string(info[7])+','+$
                        dm_to_string(info[8])+','+dm_to_string(info[9])+'] reflection.'
            endelse
            ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result')
         endif
         end
       (*pState).calclattBut:begin
         default = strpos((*pState).det_tit,'!9%')
         if default[0] ne -1 then begin
            default = strsplit((*pState).det_tit,'=',/extract)
            if n_elements(default) ge 3 then $
                default = dm_to_number(default[2],/float)    $
            else default = !values.f_nan
         endif else $
            default = !values.f_nan
         default = [default,float((*(*pState).infoPtr)[[0,7,8,9]])]
         info = dm_dialog_input(['2theta','lambda','H','K','L'],title='Calculate a',$
                    default=default,/float,dialog_parent=(*pState).tlb,xsize=80)
         if total(finite(info)) eq 5 then begin
            info = double(info)
            a  = sqrt(total(info[2:4]^2))*info[1]/(2d)/sin(info[0]/(2d)*!dpi/(180d))
            ok = dialog_message("The lattice paramter for a cubic lattice is "+dm_to_string(float(a))+string('c5'xB)+'.',$
                /info,dialog_parent=(*pState).tlb,title='Result')
         endif
         end
       (*pState).calca2But:begin
         default = strpos((*pState).det_tit,'!9%')
         if default[0] ne -1 then begin
            default = strsplit((*pState).det_tit,'=',/extract)
            if n_elements(default) ge 3 then $
                default = dm_to_number(default[2],/float)    $
            else default = !values.f_nan
         endif else $
            default = !values.f_nan
         tth = dm_dialog_input('two theta',default=default,info='A2=90-theta',xsize=80,$
                dialog_parent=(*pState).tlb,title='Calculate A2')
         if finite(tth) then begin
            mesg = 'Set A2 to '+strtrim(string(90.-tth/2.,format='(f9.2)'),2)+string('b0'xb)+'.'
            ok = dialog_message(mesg,/info,dialog_parent=(*pState).tlb,title='Result')
         endif
         end
       (*pState).e2lambdaBut: dcs_cryst_align_e2lambda,event,lambda=float((*(*pState).infoPtr)[0])
       else:begin
         ;widget_context events
         widget_type = tag_names(event,/structure)
         if strlowcase(widget_type) eq 'widget_context' then $
             widget_displaycontextmenu,event.id,event.x,event.y,(*pState).contextMenu
         end
    endcase
    device,decomposed=(*pState).decompose
end

pro dcs_cryst_align_Exit,tlb
    widget_control,tlb,get_uvalue=pState,/no_copy
    tvlct,(*(*pState).rgb).r,(*(*pState).rgb).g,(*(*pState).rgb).b ;restore entry color map
    device,decomposed=(*pState).decompose  ;restore entry decompose info
    obj_destroy,(*pState).filesel
    cd,(*pState).root
    ptr_free,(*pState).detPtr,(*pState).xPtr,(*pState).yPtr,(*pState).dyPtr,(*pState).rgb
    ptr_free,(*pState).xaxisPtr,(*pState).yaxisPtr,(*pState).tthPtr,(*pState).infoPtr
    wdelete,(*pState).winPix
    ptr_free,pState
    widget_control,tlb,/destroy
end

;initparms=[a,b,c,alpha,beta,gamma,uh,uk,ul]
pro dcs_cryst_align,event,initparms
    state={group_leader:                 0L, $  ;group leader
           tlb:                          0L, $  ;top level base
           rgb:     ptr_new(/allocate_heap), $
           geom:                    [0L,0L], $  ;geometry
           filesel:               obj_new(), $  ;file selector
           root:                         '', $  ;starting directory
           infoPtr: ptr_new(/allocate_heap), $  ;store the info for calculating 2theta
                                             $  ;[lambda,a,b,c,alpha,beta,gamma,h,k,l]
           contextMenu:                  0L, $  ;context sensitive menu base
           calctthBut:                   0L, $  ;calculate 2theta button
           calclattBut:                  0L, $  ;calculate cubic lattice parameter
           calca2But:                    0L, $  ;calculate A2 button
           e2lambdaBut:                  0L, $  ;E to lambda conversion button
           loadBut:                      0L, $  ;load file button
           fitBut:                       0L, $  ;gauss fit button
           fit:                          0b, $  ;flag for fit
           saveBut:                      0L, $  ;save plot button
           dataBut:                      0L, $  ;save data button
           printBut:                     0L, $  ;print plot button
           exitBut:                      0L, $  ;exit button
           popMenu:                      0L, $  ;popup menu
           xranBut:                      0L, $  ;x range button
           yranBut:                      0L, $  ;y range button
           ndet:                         0L, $  ;number of detectors
           detList:                      0L, $  ;detector droplist
           tthPtr:  ptr_new(/allocate_heap), $  ;save the two theta value
           detPtr:  ptr_new(/allocate_heap), $  ;pointer to detector array
           timeList:                     0L, $  ;time channel droplist
           t_chan:                       1s, $  ;0-all 1-elastic 2-specify
           e_ran:                   [0e,0e], $  ;energy range for t_chan=2
           xaxisList:                    0L, $  ;x-axis droplist
           xaxis:                        0s, $  ;xaxis number
           xPtr:    ptr_new(/allocate_heap), $  ;pointer to x data
           yPtr:    ptr_new(/allocate_heap), $  ;pointer to y data
           dyPtr:   ptr_new(/allocate_heap), $  ;pointer to dy data
           xaxisPtr:ptr_new(/allocate_heap), $  ;pointer to store !x
           yaxisPtr:ptr_new(/allocate_heap), $  ;pointer to store !y
           winVis:                       0L, $  ;visible window
           winPix:                       0L, $  ;buffer invisible window
           plotDraw:                     0L, $  ;draw widget
           mouse:                        0b, $  ;mouse state
           xbox:                    [0e,1e], $  ;zoom box
           ybox:                    [0e,1e], $  ;zoom box
           zoom:                         0b, $  ;zoom flag
           xran:                  fltarr(2), $  ;xrange
           yran:                  fltarr(2), $  ;yrange
           duration:                     0L, $  ;duration of measurement in secs
           det_tit:                      '', $  ;output detector string
           tch_tit:                      '', $  ;output time channel string
           workDir:                      '', $  ;work directory
           datDir:                       '', $  ;data directory
           decompose:                    0l, $
           enotebook:                    0b  $  ;use Rob's enotebook
    }
    state.e_ran  = [!values.f_nan,!values.f_nan]
    registerName = 'dcs_cryst_align'
;   if xregistered(registerName) then return  ;only allow one copy to be running at one time
    ;save the entry decompose information
    device,get_decomposed=tmp & if n_elements(tmp) eq 1 then state.decompose=(0>(tmp)<1)
    device,decomposed=1
    tvlct,r,g,b,/get
    *state.rgb = {r:temporary(r),g:temporary(g),b:temporary(b)}  ;save entry color map
    loadct,0,/silent
    cd,current = current
    state.root = current
    state.workDir = current
    state.datDir  = current

    *state.infoPtr= dblarr(10)
    if n_elements(initparms) eq 9 then $
         (*state.infoPtr)[*] = [!values.d_nan,initParms] $
    else (*state.infoPtr)[*] = !values.d_nan
    ;get the detector information
    alldetPtr     = dm_load_detpos()
    *state.tthPtr = (*alldetPtr).two_theta
    if n_elements(event) ne 0 then begin
       state.group_leader = event.top
       state.tlb  = widget_base(title='DCS Single Crystal Alignment Utility',/row,kill_notify='dcs_cryst_align_Exit',/tlb_size_event,$
         group_leader=event.top)
    endif else $
       state.tlb  = widget_base(title='DCS Single Crystal Alignment Utility',/row,kill_notify='dcs_cryst_align_Exit',/tlb_size_event)
    left = widget_base(state.tlb,/col)
    state.filesel = obj_new('dm_filesel',parent=left,xsize=160,ysize=20,/frame,filter='.dcs')
    defsysv,'!dave_defaults',exists=exists
    if exists then $
       if ptr_valid(!dave_defaults) then begin
         state.datDir  = (*!dave_defaults).datDir
         state.workDir = (*!dave_defaults).workDir
       endif
    state.filesel->set_path,state.datDir
    if !version.os_family eq 'unix' then $
        center = widget_base(state.tlb,/grid_layout,/col,/align_bottom,xpad=0,ypad=0,space=0) $
    else $
        center = widget_base(state.tlb,/grid_layout,/col,/align_bottom)

    ;construct the detector droplist values
    state.ndet = n_elements((*alldetPtr).two_theta)
    tmp   = string((*alldetPtr).two_theta,format='(f9.2)')
    tmp   = string(indgen(state.ndet)+1,format='(i3)')+tmp
    index = dm_mask_detbanks(0)
    tmp[index] = tmp[index]+'  c' & index=-1  ;central bank
    index = dm_mask_detbanks(1)
    tmp[index] = tmp[index]+'  u' & index=-1  ;upper bank
    index = dm_mask_detbanks(-1)
    tmp[index] = tmp[index]+'  l' & index=-1  ;lower bank
    tmp   = ['All','Specify','Central Bank','Upper Bank','Lower Bank',tmp]
    ptr_free,alldetPtr
    *state.detPtr  = indgen(state.ndet)  ;default all detectors
    state.det_tit  = 'det=all'
    state.tch_tit  = 'tchan=elastic'
    void = widget_label(center,value='Det. Number')
    state.detList  = dm_widget_droplist(center,value=tmp)
    void = widget_label(center,value='Time Channel')
    state.timeList = dm_widget_droplist(center,value=['All','Elastic','Specify'],select=1)
    void = widget_label(center,value='X-axis')
    state.xaxisList= dm_widget_droplist(center,value=['Rotation','Height','X-translation','Y-translation','X-tilt','Y-tilt','File Index','Temperature'])
    tmpbase = widget_base(center,/row,/context_events)
    void    = widget_label(tmpbase,value='')

    ;context sensitive menu
    state.contextMenu = widget_base(tmpbase,/context_menu)
    state.calctthBut  = widget_button(state.contextMenu,value='Calculate 2theta')
    state.calclattBut = widget_button(state.contextMenu,value='Calculate Cubic Lattice Parameter')
    state.calca2But   = widget_button(state.contextMenu,value='Calculate A2 Reset Value')
    state.e2lambdaBut = widget_button(state.contextMenu,value='E to Wavelength Conversion',/separator)

    state.loadBut  = widget_button(center,value='Load Data')
    state.fitBut   = widget_button(center,value='Gauss Fit')
    state.saveBut  = widget_button(center,value='Save Plot')
    state.dataBut  = widget_button(center,value='Save Data')
    state.printBut = widget_button(center,value='Print')
    state.exitBut  = widget_button(center,value='Exit')

    geom  = widget_info(state.exitBut,/geometry)
    widget_control,state.detList,xsize=geom.scr_xsize
    widget_control,state.timeList,xsize=geom.scr_xsize
    widget_control,state.xaxisList,xsize=geom.scr_xsize

    geom  = widget_info(left,/geometry)
    ysize = geom.scr_ysize
    xsize = fix(1.3*ysize,type=3)
    state.plotDraw = widget_draw(state.tlb,xsize = xsize,ysize = ysize,/button_events)

    ;popup menu
    state.popMenu  = widget_base(state.plotDraw,/context_menu)
    state.xranBut  = widget_button(state.popMenu,value='X Range')
    state.yranBut  = widget_button(state.popMenu,value='Y Range')

    dm_center_kid, state.tlb   ;centering the top level base
    widget_control,state.tlb,/realize

    widget_control,state.plotDraw,get_value = winVis
    window,/free,/pixmap,xsize = xsize,ysize = ysize
    winPix = !d.window
    state.winVis = winVis
    state.winPix = winPix

    geom  = widget_info(state.tlb,/geometry)
    state.geom = [geom.scr_xsize,geom.scr_ysize]

    device,decomposed=state.decompose
    pState = ptr_new(state,/no_copy)
    widget_control,(*pState).tlb,set_uvalue=pState

    if xregistered('dave',/noshow) and (n_elements(event) ne 0) then begin
        (*pState).enotebook = 1b
        RET = DAVE_SET_FOCUS((*pState).tlb)
    endif

    xmanager,registerName,(*pState).tlb,cleanup='dcs_cryst_align_Exit',/no_block
end