; $Id: dm_det_spurion.pro,v 1.1 2004/02/17 21:15:10 qiuym Exp $
; plot spurion caused by the scattering from detectors

pro dcs_mslice::dm_det_spurion,debug=debug
    dPtr    =   self.dataStrPtr
    pPtr    =   self.projStrPtr
    tPtr    =   self.detPosPtr
    kPtr    =   self.kfactor
    tth_all =   (*tPtr).two_theta
    fwhm    =   dcs_jcc_resn((*dPtr).info.ch_wl,(*dPtr).info.ch_wl,(*dPtr).info.ch_res,(*dPtr).info.ch_ms)/1000.    ;in meV
    index   =   where(((*dPtr).e gt -1.2*fwhm) and ((*dPtr).e lt 1.2*fwhm), count1)  ;elastic channels
    if count1 eq 0 then return
    qty     =   (*dPtr).qty[index,*]
    en      =   (*dPtr).e[index]
    index   =   where((*tPtr).psi eq 0,count2)
    qty     =   qty[*,index]
    tth     =   ((*tPtr).two_theta)[index]

    a_det   =   3.594 ;lattice parameter for fcc stainless steel
    q_det   =   sqrt([3,4,8,11,12])*2.*!pi/a_det  ;[1,1,1],[2,0,0],[2,2,0],[3,1,1],[2,2,2]
    i_det   =   [0.005,0.003,0.003,0.003,0.003]   ;relative intensity

    tth_all =   tth[sort(tth)]
    max_en  =   max((*dPtr).e)
    max_tt1 =   max(tth_all[where(tth_all gt 0)],min=min_tt1)
    max_tt2 =   max(tth_all[where(tth_all lt 0)],min=min_tt2)


    ext_qty =   0.0
    ext_en  =   0.0
    ext_tth =   0.0

    current = systime(/sec)

    ei      =   (*dPtr).ei
    d_sd    =   4010.;   sample to detector distance
    conv    =   81.8042/4./!pi^2    ;2.072
    hpi     =   !pi/2.
    for i=0,count1-1 do begin
        ki  =   sqrt((ei-en[i])/conv)
        for j=0,n_elements(q_det)-1 do begin
            if q_det[j]/2. lt ki then begin
                dtth        =   2.*asin(q_det[j]/2./ki)
                if dtth gt hpi then begin
                    new_en  =   ei-(ki*d_sd/(d_sd-2*d_sd*cos(dtth)))^2*conv
                    if new_en lt max_en then begin
                        d_tth   =   2*dtth/!dtor-180.
                        for k=0,count2-1 do begin
                            new_tth =   tth[k]+d_tth
                            if (new_tth lt max_tt1 and new_tth gt min_tt1) or (new_tth lt max_tt2 and new_tth gt min_tt2) then begin
                                ext_tth =   [ext_tth,new_tth]
                                ext_en  =   [ext_en,new_en]
                                ext_qty =   [ext_qty,qty[i,k]*i_det[j]]
                            endif
                            new_tth =   tth[k]-d_tth
                            if (new_tth lt max_tt1 and new_tth gt min_tt1) or (new_tth lt max_tt2 and new_tth gt min_tt2) then begin
                                ext_tth =   [ext_tth,new_tth]
                                ext_en  =   [ext_en,new_en]
                                ext_qty =   [ext_qty,qty[i,k]*i_det[j]]
                            endif
                        endfor
                    endif
                endif
            endif
        endfor
    endfor

    if keyword_set(debug) then print,'Spurion calculation is done is ',systime(/sec)-current,' secs.'

    n_ext   =   n_elements(ext_qty)
    if n_ext eq 1 then begin
        ok  =   dialog_message("There's no spurion caused by the detectors.",dialog_parent=self.tlb,/info)
        return
    endif

    en      =   en#(fltarr(count2)+1)
    tth     =   (fltarr(count1)+1)#tth
    qty     =   [qty[*],ext_qty[1:n_ext-1]]
    en      =   [en[*], ext_en[1:n_ext-1]]
    tth     =   [tth[*],ext_tth[1:n_ext-1]]
    max_en  =   max(en,min=min_en)
    itmp    =   where((*dPtr).e le min_en,count)
    min_en  =   (*dPtr).e[itmp[count-1]]
    itmp    =   where((*dPtr).e ge max_en)
    max_en  =   (*dPtr).e[itmp[0]]
    uniq_en =   (*dPtr).e[where((*dPtr).e ge min_en and (*dPtr).e le max_en)]
    dm_stepgrid_bin,0.0,0.0,tth,en,qty,uniq_xval=tth_all,uniq_yval=uniq_en,/leave_blank
    qty     =   transpose(qty)

    ;replace dataStrPtr, projStrPtr, and detPosPtr by the new spurion data
    titl_lab        =   self.titl_lab
    self.titl_lab   =   'Spurion'
    self.dataStrPtr =   ptr_new({qty:qty,err:0*qty,e:en,weight:0,ei:ei,info:(*dPtr).info})
    self.detPosPtr  =   ptr_new({two_theta:tth,dtwo_theta:0*tth,psi:0*tth,dpsi:0*tth})
    self.projStrPtr =   ptr_new()
    self.kfactor    =   ptr_new(sqrt(ei/(ei-en)))
    self->dm_calc_projection    ;recalculate projection
    self->dm_plot_slice         ;plot the spurion

    ;return to initial state
    self.titl_lab   =   titl_lab
    ptr_free,self.dataStrPtr,self.projStrPtr,self.detPosPtr,self.kfactor
    self.dataStrPtr =   dPtr
    self.projStrPtr =   pPtr
    self.detPosPtr  =   tPtr
    self.kfactor    =   kPtr
end