; $Id: dm_proj_spec_samp.pro,v 1.6 2003/10/10 14:14:07 qiuym Exp $
; transform data points in the form (detector_angles, energy)->(Qx,Qy,Qz)
; in the spectrometer frame where [1,0,0] is along ki, [0,1,0] in the horizontal scattering plane
; and [0,0,1] pointing upward
; then into the sample reference frame by a rotation of the coordinate system
; parameters:
;   ei:       incident energy
;   en:       energy transfer: [ne] for powder and single crystal type, scalor for diffuse scattering type
;   det_tt:   two_theata:[ndet]
;   det_dtt:  dtwo_theta:[ndet]
;   det_psi:  psi:[ndet]
;   det_dpsi: dpsi:[ndet]
; keywords:
;   psi:      scalor for single crystal type, [nfile] array for diffuse scattering type; positive if ccw rotation of u from ki
;   type:     0:single crystal 1:powder 2:diffuse scattering
;   is_spe:   for matlab style spe file
function dm_proj_spec_samp,ei,en,det_tt,det_dtt,det_psi,det_dpsi,psi=psi,type=type,debug=debug,is_spe=is_spe
    if keyword_set(debug) then begin
       print,'##proj spec samp...'
       current=systime(/sec)
    endif
    E_ksq  =  2.0721246  ;E_ksq=!dcs_hsq2mn/4./!dpi/!dpi;=2.0721246
    ki     =  sqrt(ei/E_ksq)
    kf     =  sqrt((ei-en)/E_ksq)
    nen    =  n_elements(kf)  & ndet= n_elements(det_tt) & nfile=n_elements(psi)
    if n_elements(type) eq 0 then type=1      ;default-powder type
    if type ne 2 then $
       ki  = temporary(ki)*(fltarr(nen,ndet)+1)
    case type of
       0:begin  ;single crystal type -> Q[3,nen,ndet]
         Q       = fltarr(3,nen,ndet,/nozero)
         dettt   = det_tt*!dtor
         detpsi  = det_psi*!dtor
         cos2th  = cos(dettt)
         sin2th  = sin(temporary(dettt))
         sinpsi  = sin(detpsi)
         cospsi  = cos(temporary(detpsi))
         Q[0,*,*]= temporary(ki)-kf#temporary(cos2th)
         if keyword_set(is_spe) then begin  ;spe style definition of psi
             Q[1,*,*]= -kf#(sin2th*temporary(cospsi))
             Q[2,*,*]= -temporary(kf)#(temporary(sin2th)*temporary(sinpsi))
         endif else begin                   ;dcs defininiton of psi (7.6285 for upper bank, -7.6285 for lower bank) and 2th(+/-)
             Q[1,*,*]= kf#(sin2th*sqrt(1-(sinpsi/sin2th)^2))
             Q[2,*,*]= -temporary(kf)#temporary(sinpsi)
         endelse
         sinpsi  = sin(psi[0]*!dtor)
         cospsi  = cos(psi[0]*!dtor)
         R       = [[cospsi,-sinpsi,0.0],[sinpsi,cospsi,0.0],[0.0,0.0,1.0]]
         ;old and slower code
         ;tmp     = Q
         ;Q[0,*,*]= tmp[0,*,*]*R[0,0]+tmp[1,*,*]*R[0,1]+tmp[2,*,*]*R[0,2]
         ;Q[1,*,*]= tmp[0,*,*]*R[1,0]+tmp[1,*,*]*R[1,1]+tmp[2,*,*]*R[1,2]
         ;Q[2,*,*]= tmp[0,*,*]*R[2,0]+tmp[1,*,*]*R[2,1]+tmp[2,*,*]*R[2,2]
         Q       = reform(Q,3,nen*ndet)
         Q       = temporary(Q)##temporary(R)
         Q       = reform(Q,3,nen,ndet)
         end
       1:begin  ;powder type -> Q[nen,ndet]
         Q       = sqrt(ki*ki+(kf*kf)#(fltarr(ndet)+1)-2.*ki*(kf#cos(det_tt*!dtor)))
         end
       2:begin  ;diffuse scattering type -> Q[3,ndet, nfile]
         Q0      = fltarr(3,ndet,/nozero)
         dettt   = det_tt*!dtor
         detpsi  = det_psi*!dtor
         cos2th  = cos(dettt)
         sin2th  = sin(temporary(dettt))
         sinpsi  = sin(detpsi)
         cospsi  = cos(temporary(detpsi))
         Q0[0,*] = temporary(ki)-kf*temporary(cos2th)
         Q0[1,*] = kf*(sin2th*sqrt(1-(sinpsi/sin2th)^2))
         Q0[2,*] = -temporary(kf)*temporary(sinpsi)
         R       = fltarr(3,3,nfile)
         sinpsi  = sin(psi*!dtor)
         cospsi  = cos(psi*!dtor)
         R[0,0,*]= cospsi
         R[1,0,*]= -sinpsi
         R[0,1,*]= temporary(sinpsi)
         R[1,1,*]= temporary(cospsi)
         R[2,2,*]= 1.0
         Q       = fltarr(3,ndet,nfile,/nozero)
         for i=0,nfile-1 do $
            Q[*,*,i] = Q0##R[*,*,i]
         end
       else:
    endcase
    if keyword_set(debug) then begin
       print,'##proj spec samp finished in ',systime(/sec)-current,' sec.'
    endif
    return, Q
end