; $Id: dm_step_bin.pro,v 1.7.4.1 2004/05/28 17:52:01 azuah Exp $
; bin data according to given step
; parameter:
;   in_xstep: step of binning
;   xdat:     array of xdat
; keyword:
;   ydat:     array of ydat, using the same binning as ydat
;   yerr:     array of yerr, using the same binning as xdat
;   uniq_val: if set, then use the given uniq values instead of the xstep to determine the bins
;   avgsum:   0:average 1:sum
;   ewid:     energy width, used when e-integrated intensity option is choosed
;             input ydat=sum(I*ewid)
;             output ydat=sum(I*ewid)/sum(ewid)
pro dm_step_bin,in_xstep,xdat,ydat=ydat,yerr=yerr,uniq_val=uniq_val,debug=debug,avgsum=avgsum,ewid=ewid
    if n_elements(avgsum) eq 0 then avgsum=0   ;default using average in binning
    if keyword_set(debug) then current=systime(/sec)
    xstep = abs(in_xstep)
    if xstep eq 0 then xstep=1.
    nxdat    = n_elements(xdat)
    ok_ydat  = (n_elements(ydat) eq nxdat)
    ok_yerr  = (n_elements(yerr) eq nxdat)
    ok_eint  = (n_elements(ewid) eq nxdat)
    tmp_xmax = max(xdat,min=tmp_xmin)
    xmax  = temporary(tmp_xmax)+0.5*xstep
    xmin  = temporary(tmp_xmin)-0.5*xstep
    xsize = double(xmax-xmin)/xstep
    if xsize gt fix(xsize,type=3) then xsize=(2.>(xsize+1.))
    xsize = fix(xsize,type=3)
    if n_elements(uniq_val) ne 0 then begin
       index = where((uniq_val ge xmin) and (uniq_val lt xmax),count)
       if count ne 0 then uniq_val=uniq_val[index]
       index = -1
       xlookup = uniq_val[uniq(uniq_val,sort(uniq_val))]
       xlookup = [xlookup,xmax]
    endif else $
       xlookup = xmin+findgen(xsize+1)*xstep
    xsize = n_elements(xlookup)
    tmp_x = fltarr(xsize)
    if ok_ydat then tmp_y = fltarr(xsize)
    if ok_yerr then tmp_e = fltarr(xsize)
    if ok_eint then tmp_w = fltarr(xsize)
    num_x = lonarr(xsize)

    for i=0L,nxdat-1L do begin
        ix = value_locate(xlookup , xdat[i])
        tmp_x[ix] = tmp_x[ix]+xdat[i]
        if ok_ydat then tmp_y[ix] = tmp_y[ix]+ydat[i]
        if ok_yerr then tmp_e[ix] = tmp_e[ix]+yerr[i]*yerr[i]
        if ok_eint then tmp_w[ix] = tmp_w[ix]+ewid[i]
        num_x[ix] = num_x[ix]+1
    endfor
    index = where(num_x gt 0,count)
    if count ne 0 then begin
       tmp_x = tmp_x[index]
       num_x = num_x[index]
       if ok_ydat then tmp_y = tmp_y[index]
       if ok_yerr then tmp_e = tmp_e[index]
       if ok_eint then tmp_w = tmp_w[index]
       index = -1
       xdat  = tmp_x/num_x
       if avgsum eq 1 then num_x = 1
       if ok_eint then num_x = tmp_w
       if ok_ydat then ydat  = tmp_y/num_x
       if ok_yerr then yerr  = sqrt(tmp_e)/num_x
    endif
    if keyword_set(debug) then print,'step_bin finished in ',systime(/sec)-current,' secs.'
end