; $Id: davedat1d__define.pro,v 1.5 2003/10/21 15:27:03 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat1D::cleanup
;
; PURPOSE:____________________
;   Cleanup method for the DaveDat1D class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat1D::cleanup

; call the base class cleanup method
self->DaveDat::cleanup

end


;===============================================================================
; NAME:_______________________
;   DaveDat1D::init
; 
; PURPOSE:
;   Initialization method for the DaveDat1D class
;   DaveDat1D is derived from DaveDat
;
; INPUTS:_____________________
;   KEYWORDS:
;     offset: An additive offset to be associated with an objects
;     data. It is mainlly utilized for display purposes
;
;     _EXTRA: is used to pass the folowing to the base class
;        filename: The name of the file from which the (DAVE format) data
;        should be retrieved and used for the construction of the object
;
;        davePtr: Specifies a ptr to a (DAVE) structure to be used in
;        constructing the current object
;
;        scalefactor: a multiplicative scalefactor to be assigned to the
;        object data.
;
; OUTPUTS:____________________
;     return value: 0 signifies failure and 1 success.
;
function DaveDat1D::init, offset=oset, _EXTRA=etc

on_error, 2
catch, theError
if (theError ne 0) then begin
    catch, /cancel
    message, /reissue_last
    return, 0
endif

; call the base class init method
if (self->DaveDat::init(_EXTRA=etc) ne 1) then return, 0

; ensure data is 1D and not 2D
if (ptr_valid(self.zvPtr)) then begin
    if ((size(*(self.zvPtr)))[0] gt 1) then begin
        *(self.zvPtr) = (*(self.zvPtr))[*,1]
    endif
endif
if (ptr_valid(self.zePtr)) then begin
    if ((size(*(self.zePtr)))[0] gt 1) then begin
        *(self.zePtr) = (*(self.zePtr))[*,1]
    endif
endif

if (n_elements(oset) gt 0) then begin
    self.offset = oset
endif

return, 1
end


;===============================================================================
; NAME:_______________________
;   DaveDat1D__define
;
; PURPOSE:____________________
;   Definition of the class structure for the DaveDat1D class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat1D__define

void = {DaveDat1D, $
        offset:0.0, $           ; additive offset for data
        inherits DaveDat $      ; inherits from DaveDat class
       }

end
