; $Id: davedat1d__get.pro,v 1.6.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat1D::get
; 
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     offset: Return the offset as a float. The data offset is used
;     when creating a multiplot containing other datasets
;
;     filename: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     davePtr: Return a _pointer_ to a _copy_ of the object's davePtr if
;     present
;
;     scalefactor: Return scalefactor as a float.
;
;     descriPtr: Return a _pointer_ to a _copy_ of the object's
;     descriPtr structure if present
;
;     instrument: Return the instrument name; an empty string is
;     returned if instrument is undefined
;
;     xVal: returns a float 1D vector containing the x-axis values.
;
;     xLabel: returns x-axis label as string
;
;     xType: returns x-axis type as string
;
;     xUnits: returns x-axis units as string
;
;     zVal: returns a float 1D/2D array containing the z-axis values.
;
;     zErr: returns a float 1D/2D array containing the z-axis errors.
;
;     zLabel: returns z-axis label as string
;
;     zUnits: returns z-axis units as string
;
;     treatment: returns a string (array) containing a record of
;     operations performed so far on the data.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;
; NOTE: DaveDat1D inherits from DaveDat and most of the functionality
; of DaveDat1D::get() is handled directly by DaveDat::get()

function DaveDat1D::get, _REF_EXTRA=etc, updDavePtr=updDavePtr, offset=offset, zVal=zVal

retVal = 1

; data offset
if (arg_present(offset)) then begin
    offset = self.offset
endif


; get davePtr with data updated to reflect current state of this object
if (arg_present(updDavePtr)) then begin
    if (ptr_valid(updDavePtr)) then heap_free, updDavePtr ; delete the heap variable if defined already
    if (ptr_valid(self.davePtr)) then dave_beast_ops,'dup',self.davePtr,updDavePtr
    ;; copy object's data to new davePtr
    if (ptr_valid(self.treatPtr)) then begin 
        (*(*(*updDavePtr).dataStrPtr).commonStr.treatmentPtr) = (*self.treatPtr)
    endif
    void = self->get(xval=xval,zval=zval,zerr=zerr)
    ; If the xaxis was reversed, undo it
    if (self.xDir eq 1) then begin
        xval = reverse(xval)
        zval = reverse(zval)
        zerr = reverse(zerr)
    endif
    ;if (n_elements(xval) le 0) xval = 0.0
    ;if (n_elements(zval) le 0) zval = 0.0
    ;if (n_elements(zerr) le 0) zerr = 0.0
    yval = 0.0
    (*(*(*updDavePtr).dataStrPtr).commonStr.histPtr) = {qty:zval,err:zerr,x:xval,y:yval}
    (*(*updDavePtr).descriPtr) = (*self.descriPtr)
endif

; zVal
if (arg_present(zval)) then begin
    if (not self->DaveDat::get(zval=zval)) then return, 0
    zval = zval + self.offset
endif

; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::get(_EXTRA=etc)
endif

return, retVal
end
;_______________________________________________________________________________
