; $Id: davedat2d__define.pro,v 1.2.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat2D::cleanup
;
; PURPOSE:____________________
;   Cleanup method for the DaveDat2D class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat2D::cleanup

if (ptr_valid(self.yvPtr)) then ptr_free, self.yvPtr

; call the base class cleanup method
self->DaveDat::cleanup

end


;===============================================================================
; NAME:_______________________
;   DaveDat2D::init
; 
; PURPOSE:
;   Initialization method for the DaveDat2D class
;   DaveDat2D is derived from DaveDat
;
; INPUTS:_____________________
;   KEYWORDS:
;     _EXTRA: is used to pass the folowing to the base class
;        filename: The name of the file from which the (DAVE format) data
;        should be retrieved and used for the construction of the object
;
;        davePtr: Specifies a ptr to a (DAVE) structure to be used in
;        constructing the current object
;
;        scalefactor: a multiplicative scalefactor to be assigned to the
;        object data.
;
; OUTPUTS:____________________
;     return value: 0 signifies failure and 1 success.
;
function DaveDat2D::init,  _EXTRA=etc

retVal = 1

; call the base class init method
if (self->DaveDat::init(_EXTRA=etc) ne 1) then return, 0

; initialize remainder of class variables if keywords
; 'davePtr' or 'filename' were passed.
if (n_elements(etc) gt 0) then begin
    isTrue1 = where(tag_names(etc) eq 'DAVEPTR')
    isTrue2 = where(tag_names(etc) eq 'FILENAME')
    ;; use /skipbase to skip base class resetData() since it was
    ;; already performed in call to base class init().
    if ((isTrue1 ge 0) or (isTrue2 ge 0)) then retVal = self->resetData(/skipbase) 
endif

return, retVal

end


;===============================================================================
; NAME:_______________________
;   DaveDat2D__define
;
; PURPOSE:____________________
;   Definition of the class structure for the DaveDat2D class
;
; INPUTS:_____________________
;   None.
;
pro DaveDat2D__define

void = {DaveDat2D, $
        yvPtr:ptr_new(), $      ; secondary independent axis values (y-axis)
        yLabel:'', $            ; y-axis label eg 'Scattering angle'
        yType:'', $             ; y-axis type 'points' or 'hist'
        yUnits:'', $            ; y-axis units eg 'degrees'
        yDir:0b, $              ; 0=original y was increasing; 1=y was decreasing with index
        inherits DaveDat $      ; inherits from DaveDat class
       }

end
