; $Id: davedat2d__get.pro,v 1.4.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat2D::get
; 
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     filename: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     davePtr: Return a _pointer_ to a _copy_ of the object's davePtr if
;     present
;
;     scalefactor: Return scalefactor as a float.
;
;     descriPtr: Return a _pointer_ to a _copy_ of the object's
;     descriPtr structure if present
;
;     instrument: Return the instrument name; an empty string is
;     returned if instrument is undefined
;
;     xVal: returns a float 1D vector containing the x-axis values.
;
;     xLabel: returns x-axis label as string
;
;     xType: returns x-axis type as string
;
;     xUnits: returns x-axis units as string
;
;     yVal: returns a float 1D vector containing the y-axis values.
;
;     yLabel: returns y-axis label as string
;
;     yType: returns y-axis type as string
;
;     yUnits: returns y-axis units as string
;
;     zVal: returns a float 1D/2D array containing the z-axis values.
;
;     zErr: returns a float 1D/2D array containing the z-axis errors.
;
;     zLabel: returns z-axis label as string
;
;     zUnits: returns z-axis units as string
;
;     treatment: returns a string (array) containing a record of
;     operations performed so far on the data.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;
; NOTE: DaveDat2D inherits from DaveDat and most of the functionality
; of DaveDat2D::get() is handled directly by DaveDat::get()

function DaveDat2D::get, _REF_EXTRA=etc, yVal=yval, yLabel=ylabel, yType=ytype, yUnits=yunits, $
  updDavePtr=updDavePtr, yDir=yDir

retVal = 1

; 
if (arg_present(yDir)) then begin
    yDir = self.yDir
endif

;  y-axis values
if (arg_present(yval)) then begin
    if (ptr_valid(self.yvPtr)) then yval = (*self.yvPtr)
endif

; y-axis label
if (arg_present(ylabel)) then begin
    ylabel = self.yLabel
endif

; y-axis type
if (arg_present(ytype)) then begin
    ytype = self.yType
endif

; y-axis units
if (arg_present(yunits)) then begin
    yunits = self.yUnits
endif

; get davePtr with data updated to reflect current state of this object
if (arg_present(updDavePtr)) then begin
    if (ptr_valid(updDavePtr)) then heap_free, updDavePtr ; delete the heap variable if defined already
    if (ptr_valid(self.davePtr)) then dave_beast_ops,'dup',self.davePtr,updDavePtr
    ;; copy object's data to new davePtr
    if (ptr_valid(self.treatPtr)) then begin 
        (*(*(*updDavePtr).dataStrPtr).commonStr.treatmentPtr) = (*self.treatPtr)
    endif
    void = self->get(xval=xval,yval=yval,zval=zval,zerr=zerr)
    ; If the xaxis was reversed, undo it
    if (self.xDir eq 1) then begin
        xval = reverse(xval)
        zval = reverse(zval)
        zerr = reverse(zerr)
    endif
    ; If the yaxis was reversed, undo it
    if (self.yDir eq 1) then begin
        yval = reverse(yval)
        zval = reverse(zval,2)
        zerr = reverse(zerr,2)
    endif
    ;tempX = (ptr_valid(self.xvPtr))? (*self.xvPtr) : 0.0
    ;tempY = (ptr_valid(self.yvPtr))? (*self.yvPtr) : 0.0
    ;tempZ = (ptr_valid(self.zvPtr))? (*self.zvPtr) : 0.0
    ;tempE = (ptr_valid(self.zePtr))? (*self.zePtr) : 0.0
    (*(*(*updDavePtr).dataStrPtr).commonStr.histPtr) = {qty:zval,err:zerr,x:xval,y:yval}
    (*(*updDavePtr).descriPtr) = (*self.descriPtr)
endif


; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::get(_EXTRA=etc)
endif

return, retVal
end
;_______________________________________________________________________________
