; $Id: davedat2d__resetdata.pro,v 1.3.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat2D::resetData
;
; PURPOSE:____________________
;   This function is used to 
;   ** set (initially) most of the DaveDat2D class variables or 
;   ** reset them (subsequestly).
;   The values to be set/reset are taken from the davePtr class variable
;   which, once set, would normally remain unaltered for the lifetime of
;   an object.
;
; INPUTS:_____________________
;   KEYWORDS:
;     skipbase : used to skip base class resetData() method during
;     class initialization. 
;
; OUTPUTS:____________________
;     return value: 0 signifies failure and 1 success.
;
function DaveDat2D::resetData, skipbase=skipbase

if (not keyword_set(skipbase)) then begin
    ;; first call the base class resetData method
    if (self->DaveDat::resetData() ne 1) then return, 0
endif 


; set/reset remaining class variables
if (ptr_valid(self.yvPtr)) then begin ; reset
    (*self.yvPtr) = (*(*(*self.davePtr).dataStrPtr).commonStr.histPtr).y
endif else begin                ; set
    self.yvPtr = ptr_new((*(*(*self.davePtr).dataStrPtr).commonStr.histPtr).y)
endelse

; Check if y-axis is not increasing with index then reverse it
self.yDir = 0
ny = n_elements((*self.yvPtr))
if ((*self.yvPtr)[ny-1] lt (*self.yvPtr)[0]) then begin
    self.yDir = 1
    (*self.yvPtr) = reverse((*self.yvPtr))
    (*self.zvPtr) = reverse((*self.zvPtr),2)    
    (*self.zePtr) = reverse((*self.zePtr),2)    
endif

self.yLabel = (*(*self.davePtr).dataStrPtr).commonStr.ylabel
self.yType = (*(*self.davePtr).dataStrPtr).commonStr.ytype
self.yUnits = (*(*self.davePtr).dataStrPtr).commonStr.yunits 

; yType must be defined!
if (strlen(strtrim(self.yType,2)) eq 0) then self.yType = 'POINTS'

return, 1
end 
;_______________________________________________________________________________



