; $Id: davedat2d__set.pro,v 1.2.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat2D::set
; 
; PURPOSE:
;   Main set method for class. Used to set/reset class variables (all
;   class variables are 'protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     filename: assign value (expects string) to object's filename variable
;
;     davePtr: assign value (expects a DAVE structure pointer) to object's davePtr variable
;
;     scalefactor: assign value (expects a number) to object's scalefactor variable.
;
;     descriPtr: assign value (expects a descriPtr structure pointer) to
;     object's descriPtr variable
;
;     instrument: assign value (expects scaler string) to object's
;     intrument variable
;
;     xVal: assign value (expects float array) to objects xVal (x-axis
;     values) variable 
;
;     xLabel: assign value (expects scaler string) to object's xLabel
;     (x-axis label) variable
;
;     xType: assign value (expects scaler string) to object's xType
;     (x-axis type) variable.
;
;     xUnits: assign value (expects scaler string) to object's xUnits
;     (x-axis units) variable.
;
;     yVal: assign value (expects float array) to objects yVal (y-axis
;     values) variable 
;
;     yLabel: assign value (expects scaler string) to object's yLabel
;     (y-axis label) variable
;
;     yType: assign value (expects scaler string) to object's yType
;     (y-axis type) variable.
;
;     yUnits: assign value (expects scaler string) to object's yUnits
;     (y-axis units) variable.
;
;     zVal: assign value (expects float array) to objects zVal (z-axis
;     values) variable
;
;     zErr: assign value (expects float array) to objects zErr (z-axis
;     values) variable.
;
;     zLabel: assign value (expects scaler string) to object's zLabel
;     (z-axis label) variable
;
;     zUnits: assign value (expects scaler string) to object's zUnits
;     (z-axis units) variable.
;
;     treatment: assign value (expects string array) to object's
;     treatPtr (treatment records) variable.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;
;
; NOTE: DaveDat2D inherits from DaveDat and most of the functionality
; of DaveDat2D::set() is handled directly by DaveDat::set()
;

function DaveDat2D::set, _EXTRA=etc, yVal=yval, yLabel=ylabel, yType=ytype, yUnits=yunits $
  ,yDir=yDir

; reset !error_state structure, ie clear all errors
message,/reset

retVal = 1

if (n_elements(yDir)) then self.yDir=yDir

;
if (n_elements(yval) gt 0) then begin
    yvalType = vartype(yval)
    if ( (yvalType eq 'FLOAT') or $
         (yvalType eq 'DOUBLE') or $
         (yvalType eq 'INT') or $
         (yvalType eq 'LONG') ) then begin
        if (ptr_valid(self.yvPtr)) then begin
            (*self.yvPtr) = yval ; assign contents to object 
        endif else begin
            self.yvPtr = ptr_new(yval)
        endelse 
    endif else begin
        !error_state.msg = 'DaveDat2D::set: yval keyword value is not a float'
        !error_state.code = -1
        retVal = 0
    endelse
endif

;
if (n_elements(ylabel) gt 0) then begin
    ;; make sure value is a _scaler string_
    if ( (vartype(ylabel) eq 'STRING') and $
         (size(ylabel,/n_dimensions) eq 0) ) then begin
        self.ylabel = ylabel 
    endif else begin
        !error_state.msg = 'DaveDat2D::set: ylabel keyword value is not a scaler string'
        !error_state.code = -1
        retVal = 0
    endelse
endif

;
if (n_elements(ytype) gt 0) then begin
    ;; make sure value is a _scaler string_
    if ( (vartype(ytype) eq 'STRING') and $
         (size(ytype,/n_dimensions) eq 0) ) then begin
        self.ytype = ytype
    endif else begin
        !error_state.msg = 'DaveDat2D::set: ytype keyword value is not a scaler string'
        !error_state.code = -1
        retVal = 0
    endelse
endif

;
if (n_elements(yunits) gt 0) then begin
    ;; make sure value is a _scaler string_
    if ( (vartype(yunits) eq 'STRING') and $
         (size(yunits,/n_dimensions) eq 0) ) then begin
        self.yunits = yunits 
    endif else begin
        !error_state.msg = 'DaveDat2D::set: yunits keyword value is not a scaler string'
        !error_state.code = -1
        retVal = 0
    endelse
endif

; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::set(_EXTRA=etc)
endif


return, retVal
end
;_______________________________________________________________________________
