; $Id: davedat__get.pro,v 1.5.4.1 2004/05/27 22:50:54 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveDat::get
;
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'private' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     filename: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     nameTag: A string name tag for object
;
;     davePtr: Return a pointer to a _copy_ of the object's davePtr if
;     present
;
;     scalefactor: Return scalefactor as a float.
;
;     descriPtr: Return a pointer to a _copy_ of the object's
;     descriPtr structure if present
;
;     instrument: Return the instrument name; an empty string is
;     returned if instrument is undefined
;
;     xVal: returns a float 1D vector containing the x-axis values.
;
;     xLabel: returns x-axis label as string
;
;     xType: returns x-axis type as string
;
;     xUnits: returns x-axis units as string
;
;     zVal: returns a float 1D/2D array containing the z-axis values.
;
;     zErr: returns a float 1D/2D array containing the z-axis errors.
;
;     zLabel: returns z-axis label as string
;
;     zUnits: returns z-axis units as string
;
;     treatment: returns a string (array) containing a record of
;     operations performed so far on the data.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveDat::get, filename=filename,davePtr=davePtr,descriPtr=descriPtr, $
                scalefactor=scalefactor, instrument=instrument, xVal=xval, xLabel=xlabel, $
                xType=xtype, xUnits=xunits, zVal=zval, zErr=zerr, zLabel=zlabel, zUnits=zunits, $
                treatment=treatment,nameTag=nameTag,nReferences=nReferences, xDir=xDir

; 
if (arg_present(xDir)) then begin
    xDir = self.xDir
endif

; filename keyword
if (arg_present(filename)) then begin
    filename = self.filename
endif

; Return name tag
if (arg_present(nameTag)) then begin
    nameTag = self.nameTag
endif

if (arg_present(nReferences)) then begin
    nReferences = self.nReferences
endif

; get original davePtr
if (arg_present(davePtr)) then begin
    if (ptr_valid(davePtr)) then heap_free, davePtr ; delete the heap variable if defined already
    if (ptr_valid(self.davePtr)) then dave_beast_ops,'dup',self.davePtr,davePtr
endif

;
if (arg_present(descriPtr)) then begin
    if (ptr_valid(descriPtr)) then ptr_free, descriPtr ; delete the heap variable if defined already
    if (ptr_valid(self.descriPtr)) then descriPtr = ptr_new(*self.descriPtr)
endif

;
if (arg_present(scalefactor)) then begin
    scalefactor = self.scalefactor
endif

;
if (arg_present(instrument)) then begin
    instrument = self.instrument
endif

;
if (arg_present(xval)) then begin
    if (ptr_valid(self.xvPtr)) then xval = (*self.xvPtr)
endif

;
if (arg_present(xlabel)) then begin
    xlabel = self.xLabel
endif

;
if (arg_present(xtype)) then begin
    xtype = self.xType
endif

;
if (arg_present(xunits)) then begin
    xunits = self.xUnits
endif

;
if (arg_present(zval)) then begin
    if (ptr_valid(self.zvPtr)) then zval = (*self.zvPtr)*(self.scalefactor)
endif

;
if (arg_present(zerr)) then begin
    if (ptr_valid(self.zePtr)) then zerr = (*self.zePtr)*(self.scalefactor)
endif

;
if (arg_present(zlabel)) then begin
    zlabel = self.zLabel
endif

;
if (arg_present(zunits)) then begin
    zunits = self.zUnits
endif

;
if (arg_present(treatment)) then begin
    if (ptr_valid(self.treatPtr)) then begin
        if (n_elements(*self.treatPtr) ne 0) then begin
            treatment = (*self.treatPtr)
        endif else begin
            treatment = ''
        endelse
    endif
endif

return, 1
end
