; $Id: davegr1d__resetmiscgrattribs.pro,v 1.1 2003/07/24 17:51:06 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr1D::resetMiscGrAttribs
;
; PURPOSE:____________________
;   Private method to initialize/update various axes graph attributes
;   so they are consistent with the data.
;
; INPUTS:_____________________
;   KEYWORDS
;     limits: update only the limits (minimum, maximum and major tick interval)
;   None.
;
pro DaveGr1D::resetMiscGrAttribs, limits=limits, index=index

message,/reset

nData = self->count()
if (nData le 0) then begin ; no data present
    xmin = 0.0
    xmax = 1.0
    zmin = 0.0
    zmax = 1.0
endif else begin
    if (not self->get(xmin=xmin,xmax=xmax,zmin=zmin,zmax=zmax)) then begin
        if (!debug) then print, !error_state.msg
        return
    endif
endelse

if (keyword_set(limits)) then begin
    xmin = xmin*self.xMiscAttribs.scaleFactor
    xmax = xmax*self.xMiscAttribs.scaleFactor
    zmin = zmin*self.xMiscAttribs.scaleFactor
    zmax = zmax*self.xMiscAttribs.scaleFactor
    ;; minimum
    self.xMiscAttribs.minimum = xmin
    self.xMiscAttribs.maximum = xmax
    ;; maximum
    self.zMiscAttribs.minimum = zmin
    self.zMiscAttribs.maximum = zmax   
    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.xMiscAttribs.tickInterval = (xmax-xmin)*0.25 ; 5 major ticks ie end points + 3 middle ones
    self.zMiscAttribs.tickInterval = (zmax-zmin)*0.25 ; 5 major ticks ie end points + 3 middle ones
    return
endif

if (not n_elements(index)) then index=-1


if (index ne 2) then begin ; do x-axis
    ;; scalefactor
    self.xMiscAttribs.scaleFactor = 1.0

    ;; min & max
    self.xMiscAttribs.minimum = xmin
    self.xMiscAttribs.maximum = xmax

    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.xMiscAttribs.tickInterval = (xmax-xmin)*0.25 ; 5 major ticks ie end points + 3 middle ones

    ;; flag to indicate whether custom user settings (min, max, tick
    ;; interval) should be used 
    self.xMiscAttribs.useCustomFlag = 0 ; don't use custom

    ;; log/linear axis
    self.xMiscAttribs.logFlag = 0 ; linear

    ;; Nos of decimals to display - 2 by default
    self.xMiscAttribs.decimalPlaces = 2

    ;; tick direction for primary axis- inwards/outwards
    self.xMiscAttribs.tickDir = 0 ; inwards

    ;; tick length (as percentage of axis length)
    self.xMiscAttribs.tickLen = 2.5

    ;; Nos of minor ticks between major ticks
    self.xMiscAttribs.minorTicks = 4
endif

if (index ne 0) then begin ; do z-axis (== y-axis for 1D data)
    ;; scalefactor
    self.zMiscAttribs.scaleFactor = 1.0

    ;; min and max
    self.zMiscAttribs.minimum = zmin
    self.zMiscAttribs.maximum = zmax

    ;; tick interval - set nos of major ticks to 5 (including end points) by default
    self.zMiscAttribs.tickInterval = (zmax-zmin)*0.25 ; 5 major ticks ie end points + 3 middle ones

    ;; flag to indicate whether custom user settings (min, max, tick
    ;; interval) should be used 
    self.zMiscAttribs.useCustomFlag = 0 ; don't use custom

    ;; log/linear axis
    self.zMiscAttribs.logFlag = 0 ; linear

    ;; Nos of decimals to display - 2 by default
    self.zMiscAttribs.decimalPlaces = 2

    ;; tick direction for primary axis- inwards/outwards
    self.zMiscAttribs.tickDir = 0 ; inwards

    ;; tick length (as percentage of axis length)
    self.zMiscAttribs.tickLen = 2.5

    ;; Nos of minor ticks between major ticks
    self.zMiscAttribs.minorTicks = 4
endif

end
