; $Id: davegr2d__define.pro,v 1.12 2003/10/21 15:27:03 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr2D::cleanup
;
; PURPOSE:____________________
;   Cleanup method for the DaveGr2D class
;
; INPUTS:_____________________
;   None.
;
pro DaveGr2D::cleanup

if (ptr_valid(self.plotAttribPtr)) then heap_free, self.plotAttribPtr
if (ptr_valid(self.dPlotAttribPtr)) then heap_free, self.dPlotAttribPtr

obj_destroy,self.palette

; call base class cleanup method
self->DaveGr::cleanup 

end

;===============================================================================
;   DaveGr2D::initContour
;   
;   private method which inializes default values for a contour
;   display
;
pro DaveGr2D::initContour

; handle plotType and plot attributes

paStr = {contourStr, $          ; contour attributes structure.
         fillFlag:1, $          ; fill flag to off(0)/on(1)
         nLevels:5, $           ; number of equally spaced contour levels
         shadeFlag:0 $         ; 0=>flat, 1=>Gouraud shading
        }

paStr.fillFlag = 1
paStr.nLevels = 10
paStr.shadeFlag = 0

self.plotType = 'CONTOUR'
self.plotAttribPtr = ptr_new(paStr)
self.dPlotAttribPtr = ptr_new(paStr)

end


;===============================================================================
;   DaveGr2D::initImage
;   
;   private method which inializes default values for an Image
;   display
;
pro DaveGr2D::initImage

paStr = {imageStr, $            ; image attributes structure.
         interpFlag:1b $      ; 1=>bilinear; 0=> nearest neighbor interpolation
        }

self.plotType = 'IMAGE'
self.plotAttribPtr = ptr_new(paStr)
self.dPlotAttribPtr = ptr_new(paStr)

end


;===============================================================================
;   DaveGr2D::initSurface
;   
;   private method which inializes default values for a Surface
;   display
;
pro DaveGr2D::initSurface

paStr = {surfaceStr, $          ; surface attributes structure.
         shadeFlag:1b, $        ; flat (0) or Gouraud (1) for styles[2,6].
         vertShadeFlag:1b, $    ; Use vertical/elevation color shading
         skirtFlag:0b, $        ; should a skirt be included? no (0), yes (1)
         skirtValue:0.0, $      ; z-value of skirt
         style:2b, $            ; 0=points,1=Wire,2=Filled,3=RuledXZ,4=RuledYZ,5=Lego,6=LegoFilled
         bottomColor:[0,0,0], $ ; the color to draw the bottom surface
         bottomFlag:0b, $       ; Should bottom be drawn using the specified bottom color
         hiddenLineFlag:1b, $   ; 0=disallow, 1=allow hidden line removal for surface
         ambLightInt:0.5, $     ; ambient light source intensity
         ambLightFlag:1, $      ; ambient light flag; 0/1 => Off/On
         posLightInt:0.5, $     ; positional light source intensity
         posLightLoc:[-1,0,1], $ ; positional light location
         posLightDir:[0,0,0], $ ; positional light direction
         posLightFlag:1 $       ; 0/1 => Off/On
        }

paStr.bottomColor = self.bgColor
self.plotType = 'SURFACE'
paStr.posLightDir = -(paStr.posLightLoc)
self.plotAttribPtr = ptr_new(paStr)
self.dPlotAttribPtr = ptr_new(paStr)

end



;===============================================================================
; NAME:_______________________
;   DaveGr2D::init
; 
; PURPOSE:
;   Initialization method for the DaveGr2D class
;
; INPUTS:_____________________
;   KEYWORDS:
;
;     dataObj: A DaveDat* object which contains the data
;
;     label: A variable (an ANNOTATE structure) specifying a text
;     label to be associated with the DaveGr object created.
;
;     plotType: The plotType specifies how the data should be
;     displayed. The options are 'contour', 'surface' and 'image' -
;     default is 'contour' if plotType keyword is not used.
;
; OUTPUTS:____________________
;
;     return value: 0 signifies failure and 1 success.
;
function DaveGr2D::init, plotType=plotType, yColor=yColor,yFont=yFont,yLineSize=yLineSize, $
  yFontSize=yFontSize,yLabel=yLabel, _EXTRA=etc

; reset !error_state structure, ie clear all errors
message,/reset

; if a dataObj keyword was passed make sure that it's value
; is a DaveDat2D object
; then call base class initialization after
if (n_elements(etc) gt 0) then begin
    res = where(tag_names(etc) eq 'DATAREF', dataRef_isSet)
    if (dataRef_isSet gt 0) then begin
        dataRef = etc.dataRef
        if (vartype((etc.dataRef)[0]) ne 'DAVEDAT2D') then begin
            !error_state.msg = 'DaveGr2D::init: invalid object reference (dataRef) passed'
            !error_state.code = -1
            return, 0
        endif
    endif 
endif
if (self->DaveGr::init(_EXTRA=etc) ne 1) then return, 0 ; base class initialization

if (n_elements(yFont) gt 0) then begin
    if (vartype(yFont) eq 'STRING') then self.yFont = yFont
endif else self.yFont = 'HELVETICA'

if (n_elements(yFontSize) gt 0) then begin
    self.yFontSize = yFontSize
endif else self.yFontSize = 12.0

if (n_elements(yLineSize) gt 0) then begin
    self.yLineSize = yLineSize
endif else self.yLineSize = 1.0

if (n_elements(yLabel) gt 0) then begin
    if (vartype(yLabel) eq 'STRING') then self.yLabel = yLabel
endif else begin
    if (dataRef_isSet) then begin        
        if((dataRef[0])->get(yLabel=yl,yUnits=yu)) then $
          self.yLabel = (strlen(yu) gt 0)? yl+' ('+yu+')' : yl
    endif
endelse

if (n_elements(yColor) gt 0) then begin
    dim = size(yColor)
    if ( (dim[0] eq 1) and (dim[1] eq 3)) then self.yColor = yColor
endif else self.yColor = [0b,0b,0b]

if (n_elements(yLabel) gt 0) then begin
    if (vartype(yLabel) eq 'STRING') then self.yLabel = yLabel
endif else if (n_elements(dataRef) gt 0) then begin
    valid = ['DAVEDAT2D']
    for i=0,n_elements(dataRef)-1 do begin
        res = where(valid eq vartype(dataRef[i]), count)
        if (count) then begin
            if (i eq 0) then begin
                if ((dataRef[i])->get(yLabel=yl,yUnits=yu)) then $
                  self.yLabel = (strlen(yu) gt 0)? yl+' ('+yu+')' : yl
            endif
        endif
    endfor
endif

; init the color palette
self.palette = obj_new('IDLgrPalette')
self.colorIndex = 13
(self.palette)->loadct,self.colorIndex

; Set bkgd and foreground colors
self.bgColor = [255b,255b,255b]
self.fgColor = not self.bgColor

if (n_elements(plotType) le 0) then plotType='' 

case strupcase(plotType) of    
    'IMAGE': self->initImage
    'SURFACE': self->initSurface
    'CONTOUR': self->initContour
    else: self->initContour
endcase

; Initialize miscellaneous axes graph attributes
self->resetMiscGrAttribs

return, 1

end


;===============================================================================
; NAME:_______________________
;   DaveGr2D__define
;
; PURPOSE:____________________
;   Definition of the class structure for the DaveGr2D class
;
; INPUTS:_____________________
;   None.
;
pro DaveGr2D__define

; Structure to represent miscellaneous axis graphic attributes and settings
void = {miscGrAttribs, $
        minimum:0.0, $          ; minimum value to be displayed
        maximum:0.0, $          ; maximum value to be displayed
        tickInterval:0.0, $     ; major tick interval
        useCustomFlag:0b , $    ; flag indicates if above limits should be used
        scaleFactor:0.0, $      ; a scale factor to be applied to axis
        logFlag:0, $            ; flag: 0==>linear; 1==>log10
        decimalPlaces:0b, $     ; number of decimal places to display
        tickDir:0b, $           ; tic direction: 0==>inwards; 1==>outwards
        tickLen:0.0, $          ; tick length as a percentage of the axis
        minorTicks:0b $         ; number of minor ticks to display
        }

void = {DaveGr2D, $
        plotType:'CONTOUR', $   ; type of 2D plot: 'contour','surface' or 'image'
        plotAttribPtr:ptr_new(), $ ; ptr to a struct defining attributes of the 2D plot
        dPlotAttribPtr:ptr_new(), $ ; default (fall back) value for the attributes
        colorIndex:0, $         ; color index in use
        palette:obj_new(), $    ;  color palette
        fgColor:[0b,0b,0b], $   ; foreground color - eg used for contours
        yLabel:'', $
        yColor:[0b,0b,0b], $
        yFont:'', $
        yFontSize:12.0, $
        yLineSize:1.0, $
        yMiscAttribs:{miscGrAttribs}, $ ;
        inherits DaveGr $       ; 
       }

end

