; $Id: davegr2d__get.pro,v 1.6 2003/10/21 15:27:03 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr2D::get
; 
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'private' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     label: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     inUse: Return inUse as integer.
;
;     plotType: Returns the (2D) plot type.
;
;     plotAttrib: Returns a structure containing the plot attributes
;     (the structure type depends on the plot type).
;
;     dataRef: Return a reference to the object's dataObj. Any
;     subsequent modifications to this reference will also alter the object.
;
;     dataObj: Return a _copy_ of the object's dataObj
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveGr2D::get, plotType=plotType $
  ,yFontVal=yFont,yColor=yColor,yFontSize=yFontSize,yLabel=yLabel,yLineSize=yLineSize,fgColor=fgColor $
  ,fillflag=fillflag, nlevels=nlevels,shadeflag=shadeflag,palette=palette,style=style $
  ,interpFlag=interpFlag,skirtFlag=skirtFlag,skirtValue=skirtValue,hiddenLineFlag=hlFlag $ 
  ,vertShadeFlag=vertShadeFlag,bottomFlag=bottomFlag,bottomColor=bottomColor,colorIndex=colorIndex $
  ,ambLightInt=alInt,posLightInt=plInt,posLightLoc=plLoc,posLightFlag=plFlag,ambLightFlag=alFlag $
  ,posLightDir=plDir,yMiscAttribs=yMiscAttribs $
  ,xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax,zmin=zmin,zmax=zmax,_REF_EXTRA=etc

retVal = 1

; Return plotType
if (arg_present(plotType)) then begin
    plotType = self.plotType
endif

; y axis Color
if (arg_present(yColor)) then begin
    yColor = self.yColor
endif

; y axis Font
if (arg_present(yFont)) then begin
    yFont = self.yFont
endif

; y axis FontSize
if (arg_present(yFontSize)) then begin
    yFontSize = self.yFontSize
endif

; y axis LineSize
if (arg_present(yLineSize)) then begin
    yLineSize = self.yLineSize
endif

; y axis Label
if (arg_present(yLabel)) then begin
    yLabel = self.yLabel
endif

; foreground Color
if (arg_present(fgColor)) then begin
    fgColor = self.fgColor
endif

; Return color Index
if (arg_present(colorIndex)) then begin
    colorIndex = self.colorIndex
endif

; color palette
if (arg_present(palette)) then begin
    palette = self.palette
endif

; Return fillflag
if (arg_present(fillflag)) then begin
    if (self.plotType eq 'CONTOUR') then $
      fillflag = (*self.plotAttribPtr).fillflag $
    else $
      retVal = 0
endif

; Return bottomflag
if (arg_present(bottomflag)) then begin
    if (self.plotType eq 'SURFACE') then $
      bottomflag = (*self.plotAttribPtr).bottomflag $
    else $
      retVal = 0
endif

; Return bottomColor
if (arg_present(bottomColor)) then begin
    if (self.plotType eq 'SURFACE') then $
      bottomColor = (*self.plotAttribPtr).bottomColor $
    else $
      retVal = 0
endif

; Return nlevels
if (arg_present(nlevels)) then begin
    if (self.plotType eq 'CONTOUR') then $
      nlevels = (*self.plotAttribPtr).nlevels $
    else $
      retVal = 0
endif

; Return shadeflag
if (arg_present(shadeflag)) then begin
    if (self.plotType ne 'IMAGE') then $
      shadeflag = (*self.plotAttribPtr).shadeflag $
    else $
      retVal = 0
endif

; Return vertShadeFlag
if (arg_present(vertShadeFlag)) then begin
    if (self.plotType eq 'SURFACE') then $
      vertShadeFlag = (*self.plotAttribPtr).vertShadeFlag $
    else $
      retVal = 0
endif

; Return interpFlag
if (arg_present(interpflag)) then begin
    if (self.plotType eq 'IMAGE') then $
      interpflag = (*self.plotAttribPtr).interpflag $
    else $
      retVal = 0
endif

; Return skirtflag
if (arg_present(skirtflag)) then begin
    if (self.plotType eq 'SURFACE') then $
      skirtFlag = (*self.plotAttribPtr).skirtFlag $
    else $
      retVal = 0
endif

; Return skirtValue
if (arg_present(skirtValue)) then begin
    if (self.plotType eq 'SURFACE') then $
      skirtValue = (*self.plotAttribPtr).skirtValue $
    else $
      retVal = 0
endif

; Return hiddenLineFlag
if (arg_present(hlFlag)) then begin
    if (self.plotType eq 'SURFACE') then $
      hlFlag = (*self.plotAttribPtr).hiddenLineFlag $
    else $
      retVal = 0
endif

; Return style
if (arg_present(style)) then begin
    if (self.plotType eq 'SURFACE') then $
      style = (*self.plotAttribPtr).style $
    else $
      retVal = 0
endif

; Return ambient light intensity
if (arg_present(alInt)) then begin
    if (self.plotType eq 'SURFACE') then $
      alInt = (*self.plotAttribPtr).ambLightInt $
    else $
      retVal = 0
endif

; Return ambient light Flag
if (arg_present(alFlag)) then begin
    if (self.plotType eq 'SURFACE') then $
      alFlag = (*self.plotAttribPtr).ambLightFlag $
    else $
      retVal = 0
endif

; Return Positional light intensity
if (arg_present(plInt)) then begin
    if (self.plotType eq 'SURFACE') then $
      plInt = (*self.plotAttribPtr).posLightInt $
    else $
      retVal = 0
endif

; Return positional light Flag
if (arg_present(plFlag)) then begin
    if (self.plotType eq 'SURFACE') then $
      plFlag = (*self.plotAttribPtr).posLightFlag $
    else $
      retVal = 0
endif

; Return Positional light location
if (arg_present(plLoc)) then begin
    if (self.plotType eq 'SURFACE') then $
      plLoc = (*self.plotAttribPtr).posLightLoc $
    else $
      retVal = 0
endif

; Return Positional light direction
if (arg_present(plDir)) then begin
    if (self.plotType eq 'SURFACE') then $
      plDir = (*self.plotAttribPtr).posLightDir $
    else $
      retVal = 0
endif

; Return miscellaneous graph attributes for z-axis
if (arg_present(yMiscAttribs)) then begin
    yMiscAttribs = self.yMiscAttribs
endif

; global xmin,xmax,ymin,ymax,zmin,zmax for all datasets
if (arg_present(xmin) or arg_present(xmax) or $
    arg_present(ymin) or arg_present(ymax) or $
    arg_present(zmin) or arg_present(zmax)) then begin
    ;; get all data
    dataRef = self->IDL_CONTAINER::get(count=nData,/all)

    ;; determine minima/maxima for combine datasets
    for i = 0,nData-1 do begin
        if (not (dataRef[i])->get(xval=xval,yval=yval,zval=zval,offset=offset,scalefactor=scalefactor)) then $
          return, 1
        if (i eq 0) then begin
            xmin = min(xval, max=xmax)
            ymin = min(yval, max=ymax)
            ;zmin = min(zval*scalefactor, max=zmax)
            zmin = min(zval, max=zmax)
        endif else begin
            xmini = min(xval, max=xmaxi) ; get xmin, xmax for current data
            ymini = min(xval, max=ymaxi) ; get ymin, ymax for current data
            ;zmini = min(zval*scalefactor, max=zmaxi) ; get zmin, zmax for current data
            zmini = min(zval, max=zmaxi) ; get zmin, zmax for current data
            xmin = (xmini < xmin) ; running xmin
            xmax = (xmaxi > xmax) ; running xmax
            ymin = (ymini < ymin) ; running xmin
            ymax = (ymaxi > ymax) ; running xmax
            zmin = (zmini < zmin) ;
            zmax = (zmaxi > zmax) ;
        endelse
    endfor
endif

                   
; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveGr::get(_EXTRA=etc)
endif

return, retVal

end
