; $Id: davegr__get.pro,v 1.8 2003/08/19 19:57:48 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr::get
; 
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'private' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     label: Return filename as string; an empty string is return
;     if the objects filename is undefined.
;
;     nameTag: Return string name tag for object
;
;     nData: Return nData as integer.
;
;     dataRef: Return a (array/scaler) reference(s) to the object's
;     dataObj. A NULL object will be return if no data is present. 
;
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveGr::get, label=label,nlabel=nlabel,position=pos,count=dataCount,dataRef=dataRef,nameTag=nameTag, $
  bgColor=bgColor,xColor=xColor,xFontVal=xFont,xFontSize=xFontSize,xLabel=xLabel,xLineSize=xLineSize, $
  zColor=zColor,zFontVal=zFont,zFontSize=zFontSize,zLabel=zLabel,zLineSize=zLineSize,title=title, $
  xMiscAttribs=xMiscAttribs,zMiscAttribs=zMiscAttribs,guiAttribsPtr=guiAttribsPtr, $
  all=all

retVal = 1

; return guiAttribsPtr
if (arg_present(guiAttribsPtr)) then begin
    guiAttribsPtr = self.guiAttribsPtr
endif

; Return title
if (arg_present(title)) then begin
    title = self.title
endif

; Return label
if (arg_present(label)) then begin
    if (self.nlabel gt 0) then $
      label = self.label[0:self.nlabel-1]
endif

; Return number labels present
if (arg_present(nlabel)) then begin
      nlabel = self.nlabel
endif

; Return name tag
if (arg_present(nameTag)) then begin
    nameTag = self.nameTag
endif

; Return miscellaneous graph attributes for x-axis
if (arg_present(xMiscAttribs)) then begin
    xMiscAttribs = self.xMiscAttribs
endif

; Return miscellaneous graph attributes for z-axis
if (arg_present(zMiscAttribs)) then begin
    zMiscAttribs = self.zMiscAttribs
endif

; bgColor
if (arg_present(bgColor)) then begin
    bgColor = self.bgColor
endif

; xColor
if (arg_present(xColor)) then begin
    xColor = self.xColor
endif

; xFont
if (arg_present(xFont)) then begin
    xFont = self.xFont
endif

; xFontSize
if (arg_present(xFontSize)) then begin
    xFontSize = self.xFontSize
endif

; xLineSize
if (arg_present(xLineSize)) then begin
    xLineSize = self.xLineSize
endif

; xLabel
if (arg_present(xLabel)) then begin
    xLabel = self.xLabel
endif

; zColor
if (arg_present(zColor)) then begin
    zColor = self.zColor
endif

; zFont
if (arg_present(zFont)) then begin
    zFont = self.zFont
endif

; zFontSize
if (arg_present(zFontSize)) then begin
    zFontSize = self.zFontSize
endif

; zLineSize
if (arg_present(zLineSize)) then begin
    zLineSize = self.zLineSize
endif

; zLabel
if (arg_present(zLabel)) then begin
    zLabel = self.zLabel
endif

; Return number of datasets
if (arg_present(nData)) then begin
    nData = self.nData
endif

; Return a _copy_ of the object's dataObj
;if (arg_present(dataObj)) then begin
;    if (obj_valid(dataObj)) then obj_destroy, dataObj     ; delete the object if already defined
;    dataObj = (self.dataObj)->clone()                     ; clone the object's dataObj
;    if (obj_valid(dataObj) ne 1) then return, 0           ; and make sure it succeeded
;endif

; Return a reference to the DaveDat objects.
; ** Essentially a wrapper around IDL_CONTEINER::get!
if (n_elements(etc) eq 0) then etc = {dummy:0} ;  create a dummy keyword in etc struct if necessary

if (arg_present(dataRef)) then begin
    ;; If position keyword is set, pass it on
    if (n_elements(pos) gt 0) then begin 
        etc = create_struct(etc, 'position',pos)
    endif else  etc = create_struct(etc, 'all',1)

    dataRef = self->IDL_CONTAINER::get(count=dataCount,_EXTRA=etc)
endif else begin
    if (keyword_set(all)) then begin
        retVal = self->IDL_CONTAINER::get(count=dataCount,/all)
    endif
endelse

return, retVal
end
