; $Id: davegr__set.pro,v 1.5 2003/07/11 19:25:42 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   DaveGr::set
; 
; PURPOSE:
;   Main set method for class. Used to set/reset class variables (all
;   class variables are 'private/protected' in IDL
;
; INPUT PARAMETERS:___________
;   KEYWORDS:
;
;     label: assign value (expects annotate structure) to object's label variable
;
;     nameTag: A string name tag for object
;
;     dataRef: assign value (expects DaveDat* object ref array/scaler) to object's dataRef variable.
;
;     addDataRef: assign value (expects DaveDat* object ref
;     array/scaler) to object's dataRef variable without replacing
;     existing data.
;
; OUTPUTS:____________________
;
;     return value: 0-failure,  1-success.
;

function DaveGr::set, label=label,title=title,nameTag=nameTag,bgColor=bgColor, $
  xColor=xColor,xFontVal=xFont,xFontSize=xFontSize,xLabel=xLabel,xLineSize=xLineSize, $
  zColor=zColor,zFontVal=zFont,zFontSize=zFontSize,zLabel=zLabel,zLineSize=zLineSize, $
  xMiscAttribs=xMiscAttribs,zMiscAttribs=zMiscAttribs

; reset !error_state structure, ie clear all errors
message,/reset

;use this to return a fail condition if at least one error is encountered
retVal = 1

; title
if (n_elements(title) gt 0) then begin
    ;; make sure title is a valid ANNOTATE structure
    if (vartype(title) eq 'ANNOTATE') then begin
        self.title = title
    endif
endif

; label keyword
if (n_elements(label) gt 0) then begin
    ;; make sure value is an ANNOTATE structure
    if(vartype(label) eq 'ANNOTATE') then begin
        len = n_elements(label) < self.maxLabel
        self.label[0:len-1] = label[0:len-1]
        self.nlabel = len
    endif else begin
        !error_state.msg = 'DaveGr::set: label keyword value should be an ANNOTATE structure'
        !error_state.code = -1
        return, 0
    endelse
endif

; set miscellaneous graph attributes for x-axis
if (n_elements(xMiscAttribs)) then begin
    ;; make sure value is a valid structure
    if(vartype(xMiscAttribs) eq 'MISCGRATTRIBS') then self.xMiscAttribs = xMiscAttribs
endif

; set miscellaneous graph attributes for z-axis
if (n_elements(zMiscAttribs)) then begin
    ;; make sure value is a valid structure
    if(vartype(zMiscAttribs) eq 'MISCGRATTRIBS') then self.zMiscAttribs = zMiscAttribs
endif

; name tag
if (n_elements(nameTag)) then begin
    if (vartype(nameTag) eq 'STRING') then self.nameTag = nameTag
endif

; xFont
if (n_elements(xFont) gt 0) then begin
    if (vartype(xFont) eq 'STRING') then self.xFont = xFont
endif

; xFontSize
if (n_elements(xFontSize) gt 0) then begin
    self.xFontSize = xFontSize
endif

; xLineSize
if (n_elements(xLineSize) gt 0) then begin
    self.xLineSize = xLineSize
endif

; xColor
if (n_elements(xColor) gt 0) then begin
    dim = size(xColor)
    if ( (dim[0] eq 1) and (dim[1] eq 3)) then self.xColor = xColor
endif

; zFont
if (n_elements(zFont) gt 0) then begin
    if (vartype(zFont) eq 'STRING') then self.zFont = zFont
endif

; zFontSize
if (n_elements(zFontSize) gt 0) then begin
    self.zFontSize = zFontSize
endif

; zLineSize
if (n_elements(zLineSize) gt 0) then begin
    self.zLineSize = zLineSize
endif

; zColor
if (n_elements(zColor) gt 0) then begin
    dim = size(zColor)
    if ( (dim[0] eq 1) and (dim[1] eq 3)) then self.zColor = zColor
endif

; bgColor
if (n_elements(bgColor) gt 0) then begin
    dim = size(bgColor)
    if ( (dim[0] eq 1) and (dim[1] eq 3)) then self.bgColor = bgColor
endif

; xLabel
if (n_elements(xLabel) gt 0) then begin
    if (vartype(xLabel) eq 'STRING') then self.xLabel = xLabel
endif

; zLabel
if (n_elements(zLabel) gt 0) then begin
    if (vartype(zLabel) eq 'STRING') then self.zLabel = zLabel
endif

return, retVal
end
