;$Id: db_display1d__define.pro,v 1.33 2004/04/02 19:10:41 azuah Exp $
;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Cleanup procedure for DB_Display1D
;
pro DB_Display1D::cleanup

; store/save current GUI/widget settings in the DaveGr1D object
void = (self.oDaveGr)->get(guiAttribsPtr=guiAttribsPtr)
(*guiAttribsPtr) = {winDim:self.winDim, $
                    xrangeFlag:self.xrangeFlag, $
                    xrange:self.xrange, $
                    yrangeFlag:self.yrangeFlag, $
                    yrange:self.yrange, $
                    dispType:self.dispType, $
                    dispError:self.dispError, $
                    dispTitle:self.dispTitle $
                   }

obj_destroy, self.oScene
obj_destroy, self.oDest
(self.o_registry)->remove,self      ; unregister self from Object Registry
(self.oActive1DCont)->remove, self.oDaveGr ; unregister DaveGr object from the active view list
device, decomposed=self.old_dec

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
;  Refresh (display data) method
pro DB_Display1D::refresh, _EXTRA=etc

; require color decomposition (for using rgb colors directly)
; return to previous setting at end of this procedure
device, get_decomposed = old_dec
device, decompose=1

switch self.renewScene of

    0: begin
        self.renewScene = 1  ; reset to default value of 1
        objArr = (self.oScene)->get(/all, count=nObj)
        ;; if Scene is empty, proceed to case '1' by not 'breaking'
        if (nObj gt 0) then begin
            (self.oDest)->draw, self.oScene
            break
        endif
    end

    1: begin
        ;; Modify tlb widget title so it includes the current name tag
        ;; for the DaveGr object being viewed
        void = (self.oDaveGr)->get(nameTag=title)
        widget_control, self.tlb,tlb_set_title='DATABROWSER - '+title

        ;; error bars
        etc = {dispError:self.dispError}

        ;; Title
        etc = create_struct(etc,'dispTitle',self.dispTitle)

        ;; Set display mode
        case self.dispType of
            0: etc = create_struct(etc,'dispType','POINTS')
            1: etc = create_struct(etc,'dispType','HISTOGRAMS')
        endcase

        ;; x-axis limits
        if (self.xrangeFlag) then begin
            etc = create_struct(etc,'xrange',self.xrange)
        endif
        ;; y-axis limits
        if (self.yrangeFlag) then begin
            etc = create_struct(etc,'yrange',self.yrange)
        endif

        (self.oDaveGr)->display, self.oDest, self.oScene, _EXTRA=etc
        ;Dg1Ddisplay, self.oDest,self.oScene,self.oDataCont, _EXTRA=etc

    end

endswitch

; return color decomposition to previous setting
device, decompose=old_dec

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Method for dealing with events generated by Context
; Sensitive Menus
;
pro DB_DISPLAY1D::handleConSenMenuEvent, event

;widget_control, event.top, get_uvalue = sPtr

; Get the selected CSM's name and value
uname = widget_info(event.id,/uname)
widget_control, event.id, get_uvalue=newValue
;newValue = newValueStr.val
;newIndex = newValueStr.ind

; Get the atom/model for which the CSM should act on
objRef = self.csmObjRef

switch uname of

    'STYPE': begin
        validAtomNames = ['SPLOT'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, symbol=symRef
                    symRef->setProperty, data=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (not daveGrRef->set(symType=newValue,position=datInd)) then return
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->getProperty, symbol=symRef
                symRef->setProperty, data=newValue
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                if (not daveGrRef->set(symType=newValue)) then return
                self->noRenew
            endif
        endelse
        break
    end

    'SSIZE': begin
        validAtomNames = ['SPLOT'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, symbol=symRef
                    symRef->getProperty, uvalue=symScale
                    symRef->setProperty, size=newValue*symScale
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (not daveGrRef->set(symSize=newValue,position=datInd)) then return
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->getProperty, symbol=symRef
                symRef->getProperty, uvalue=symScale
                symRef->setProperty, size=newValue*symScale
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                datInd = atomUval.datInd
                if (not daveGrRef->set(symSize=newValue,position=datInd)) then return
                self->noRenew
            endif
        endelse
        break
    end

    'LSTYLE': begin
        validAtomNames = ['LPLOT'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->setProperty,linestyle=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (not daveGrRef->set(lineType=newValue,position=datInd)) then return
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->setProperty,linestyle=newValue
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                datInd = atomUval.datInd
                if (not daveGrRef->set(lineType=newValue,position=datInd)) then return
                self->noRenew
            endif
        endelse
        ;; Update check mark against the item selected/deselected
        ;if (valid) then begin
        ;    widget_control,(*self.csblLSIDPtr)[self.csblLSIND],set_button=0
        ;    self.csblLSIND = newIndex
        ;    widget_control,(*self.csblLSIDPtr)[self.csblLSIND],set_button=1
        ;endif
        break
    end

    'THICK': begin
        validAtomNames = ['LPLOT','SPLOT','EPLOT','AXIS'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->setProperty,thick=newValue
                    ;if (atomName eq 'AXIS') then goto, SKIPAXIS0
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (atomName eq 'AXIS') then begin
                        case datInd of
                            0: kw_val = {xLineSize:newValue} ; x-axis
                            1: kw_val = {zLineSize:newValue} ; y-axis
                            else: kw_val = {dummy:0}
                        endcase
                        if (not daveGrRef->set(_EXTRA=kw_val)) then return
                        goto, SKIPAXIS0
                    endif
                    if (not daveGrRef->set(lineSize=newValue,position=datInd)) then return
                    ;; if a symbol plot, modify the symbols too
                    if (atomName eq 'SPLOT') then begin
                        objArr[nObj]->getProperty, symbol=symRef
                        symRef->setProperty, thick=newValue
                    endif
                    SKIPAXIS0:
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->setProperty,thick=newValue
                ;if (atomName eq 'AXIS') then goto, SKIPAXIS1
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                datInd = atomUval.datInd
                if (atomName eq 'AXIS') then begin
                    case datInd of
                        0: kw_val = {xLineSize:newValue} ; x-axis
                        1: kw_val = {zLineSize:newValue} ; y-axis
                        else: kw_val = {dummy:0}
                    endcase
                    if (not daveGrRef->set(_EXTRA=kw_val)) then return
                    goto, SKIPAXIS1
                endif
                if (not daveGrRef->set(lineSize=newValue,position=datInd)) then return
                ;; if a symbol plot, modify the symbols too
                if (atomName eq 'SPLOT') then begin
                    objArr[nObj]->getProperty, symbol=symRef
                    symRef->setProperty, thick=newValue
                endif
                SKIPAXIS1:
                self->noRenew
            endif
        endelse
        break
        break
    end

    'COLOR': begin
        validAtomNames = ['LPLOT','SPLOT','EPLOT','AXIS','LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->setProperty,color=newValue
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (atomName eq 'AXIS') then begin
                        case datInd of
                            0: kw_val = {xColor:newValue} ; x-axis
                            1: kw_val = {zColor:newValue} ; y-axis
                            else: kw_val = {dummy:0}
                        endcase
                        if (not daveGrRef->set(_EXTRA=kw_val)) then return
                        goto, SKIPAXIS2
                    endif
                    if (atomName eq 'LABEL') then begin
                        case datInd of
                            -1: begin ; title
                                if (not daveGrRef->get(title=title)) then return
                                title.color = newValue
                                if (not daveGrRef->set(title=title)) then return
                            end
                            else: begin ; misc label
                                if (not daveGrRef->get(label=label)) then return
                                label[datInd].color = newValue
                                if (not daveGrRef->set(label=label)) then return
                            end
                        endcase
                        goto, SKIPAXIS2
                    endif
                    if (not daveGrRef->set(color=newValue,position=datInd)) then return
                    ;; if a symbol plot, modify the symbols too
                    if (atomName eq 'SPLOT') then begin
                        objArr[nObj]->getProperty, symbol=symRef
                        symRef->setProperty, color=newValue
                    endif
                    SKIPAXIS2:
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->setProperty,color=newValue
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                datInd = atomUval.datInd
                if (atomName eq 'AXIS') then begin
                    case datInd of
                        0: kw_val = {xColor:newValue} ; x-axis
                        1: kw_val = {zColor:newValue} ; y-axis
                        else: kw_val = {dummy:0}
                    endcase
                    if (not daveGrRef->set(_EXTRA=kw_val)) then return
                    goto, SKIPAXIS3
                endif
                if (not daveGrRef->set(color=newValue,position=datInd)) then return
                ;; if a symbol plot, modify the symbols too
                if (atomName eq 'SPLOT') then begin
                    objArr[nObj]->getProperty, symbol=symRef
                    symRef->setProperty, color=newValue
                endif
                SKIPAXIS3:
                self->noRenew
            endif
        endelse
        break
    end

    'FONTSZ': begin
        validAtomNames = ['AXIS','LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    case atomName of
                        'AXIS': begin
                            objArr[nObj]->getProperty, title=titleRef, uvalue=atomUval, ticktext=ttRef
                            datInd = atomUval.datInd
                            case datInd of
                                0: kw_val = {xFontSize:newValue} ; x-axis
                                1: kw_val = {zFontSize:newValue} ; y-axis
                                else: kw_val = {dummy:0}
                            endcase
                            if (obj_valid(titleRef)) then begin
                                titleRef->getProperty, font=fontRef
                                fontRef->setProperty, size=newValue
                                ;ttRef->setProperty, size=newValue
                                daveGrRef = atomUval.objRef
                                if (not daveGrRef->set(_EXTRA=kw_val)) then return
                            endif
                        end
                        'LABEL': begin
                            objArr[nObj]->getProperty, font=fontRef, uvalue=atomUval
                            datInd = atomUval.datInd
                            daveGrRef = atomUval.objRef
                            fontRef->setProperty, size=newValue
                            case datInd of
                                -1: begin ; title
                                    if (not daveGrRef->get(title=title)) then return
                                    title.fontsize = newValue
                                    if (not daveGrRef->set(title=title)) then return
                                end
                                else: begin ; misc label
                                    if (not daveGrRef->get(label=label)) then return
                                    label[datInd].fontsize = newValue
                                    if (not daveGrRef->set(label=label)) then return
                                end
                            endcase
                        end
                    endcase
                    self->noRenew
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->getProperty, title=titleRef
                if (obj_valid(titleRef)) then begin
                    titleRef->getProperty, font=fontRef
                    fontRef->setProperty, size=newValue
                endif
                self->noRenew
            endif
        endelse

        break
    end

    'EDITLABLE' : begin
        validAtomNames = ['LABEL'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    DB_TxtLabel, datInd,objArr[nObj],daveGrRef,self,group_leader=event.top
                    self->noRenew
                endif
            endwhile
        endif
        break
    end

    'EDITAXIS' : begin
        validAtomNames = ['AXIS'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval, notext=emptyTitle
                    if (not emptyTitle) then begin
                        daveGrRef = atomUval.objRef
                        datInd = atomUval.datInd
                        ;datInd = (atomUval.datInd eq 0)? atomUval.datInd : 2 ; for 1D, zaxis == yaxis!!!
                        DB_AxisProp, datInd,objArr[nObj],daveGrRef,self,group_leader=event.top
                        self->noRenew
                    endif
                endif
            endwhile
        endif
        break
    end


    'LOGSCALE' : begin
        validAtomNames = ['AXIS'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval, notext=emptyTitle
                    if (not emptyTitle) then begin
                        daveGrRef = atomUval.objRef
                        case (atomUval.datInd) of
                            0: void=daveGrRef->get(xMiscAttribs=axisAttribs,xmax=amax)
                            1: void=daveGrRef->get(zMiscAttribs=axisAttribs,zmax=amax)
                        endcase
                        if (amax le 0.0) then begin
                            message = 'Cannot set log scale - whole range is negative!'
                            res = dialog_message(message,dialog_parent=self.tlb)
                            return
                        endif
                        axisAttribs.logFlag = 1 - axisAttribs.logFlag
                        case (atomUval.datInd) of
                            0: begin
                                void=daveGrRef->set(xMiscAttribs=axisAttribs)
                                if (self.xrangeFlag) then begin
                                    if (axisAttribs.logFlag) then begin
                                        if (self.xrange[0] le 0.0) then self.xrange[0] = 0.0001*amax
                                        self.xrange = alog10(self.xrange) 
                                    endif else self.xrange = 10.0^(self.xrange)
                                endif
                            end
                            1: begin 
                                void=daveGrRef->set(zMiscAttribs=axisAttribs)
                                if (self.yrangeFlag) then begin
                                    if (axisAttribs.logFlag) then begin
                                        if (self.yrange[0] le 0.0) then self.yrange[0] = 0.0001*amax
                                        self.yrange = alog10(self.yrange) 
                                    endif else self.yrange = 10.0^(self.yrange)
                                endif
                            end
                        endcase
                        self->Renew
                    endif
                endif
            endwhile
        endif
        break
    end

    'EDITLEGEND' : begin
        if (vartype(objRef) eq 'IDLGRLEGEND') then begin
            DB_legend, objRef,self,group_leader=event.top
            self->noRenew
        endif
        break
    end


    'SWITCHDISPMODE': begin
        validAtomNames = ['LPLOT','SPLOT'] ; ==> only atoms with name matching these will be affected
        objRefType = vartype(objRef) ; IDLgrModels will have multiple atoms
        if (objRefType eq 'IDLGRMODEL') then begin
            ;; loop through atoms in model and make changes
            objArr = objRef->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                objArr[nObj]->getProperty, name=atomName
                res = where(validAtomNames eq atomName,valid)
                if (valid) then begin
                    objArr[nObj]->getProperty, uvalue=atomUval
                    daveGrRef = atomUval.objRef
                    datInd = atomUval.datInd
                    if (not daveGrRef->set(lineSymFlag=newValue,position=datInd)) then return
                    self->renew ; reconstruct scene
                    break
                endif
            endwhile
        endif else begin
            objRef->getProperty, name=atomName
            res = where(validAtomNames eq atomName,valid)
            if (valid) then begin
                objRef->getProperty, uvalue=atomUval
                daveGrRef = atomUval.objRef
                datInd = atomUval.datInd
                if (not daveGrRef->set(lineSymFlag=newValue,position=datInd)) then return
                self->renew     ; reconstruct scene
            endif
        endelse                 ;end
        break
    end

    else: begin
        break
    end

endswitch

;(self.o_registry)->refresh      ;db_dispData, sPtr
print, 'renewscene = ',self.renewScene
self->refresh      ; refresh (NOT through registry in db_container!)


return



end





;===============================================================================
pro DB_Display1D::todoEvent, event

uname = widget_info(event.id,/uname)
print,'----------------------------------------------------------------'
print,'Handler for "',uname,'" is not yet implimented...'
widget_control,event.id, get_uvalue=uvalue
if (n_elements(uvalue) gt 0) then print, 'Uvalue: ',uvalue
help,event,/struct

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Display Context Sensitive Menus
pro DB_Display1D::dispConSenMenu, event

objArr = (self.oScene)->get(/all, count=nObj) ; get nos of objects in scene
while (nObj gt 0) do begin      ; loop through and
    nObj = nObj - 1
    if (vartype(objArr[nObj]) ne 'IDLGRVIEW') then continue
    oView = objArr[nObj]
    dims = [10,10]
    compArr = (self.oDest)->Select(oView,[event.x, event.y],dimensions = dims) ; grab model/atoms in view at mouse loc
    if (obj_valid(compArr[0]) ne 1) then continue
    count = n_elements(compArr)
    missionNotAccomplished = 1  ; don't want to display MORE THAN ONE CSM
    while ((count gt 0) and (missionNotAccomplished)) do begin
        count = count - 1
        ;; Analyse components
        (compArr[count])->getProperty, name=name
        self.csmObjRef = compArr[count]
        case name of
            'LMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBLINE')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
            'SMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBSYMB')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
            'LSMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBLSYM')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
            'AMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBAXES')
                wid = widget_info(csm, find_by_uname='LOGSCALE')
                objArr = self.csmObjRef->get(/all, count=nObj) ; get all atoms in model container
                objArr[0]->getProperty, uvalue=atomUval ; first one should be the axis - get its uvalue
                case (atomUval.datInd) of
                    0: void = (atomUval.objRef)->get(xMiscAttribs=axisAttribs)
                    1: void = (atomUval.objRef)->get(zMiscAttribs=axisAttribs) 
                endcase
                txt = (axisAttribs.logFlag eq 1)? 'Linear Scale' : 'Log Scale'
                widget_control, wid, set_value=txt
                widget_displaycontextmenu, event.id, event.x, event.y, csm

                ;csm = widget_info(event.top,find_by_uname='CSBAXES')
                ;widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
            'LABMODEL': begin
                csm = widget_info(event.top,find_by_uname='CSBLABEL')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
            'LEGEND': begin
                csm = widget_info(event.top,find_by_uname='CSBLEGEND')
                widget_displaycontextmenu, event.id, event.x, event.y, csm
                missionNotAccomplished = 0
            end
        endcase
    endwhile
endwhile


end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
;  Add Text method
; - Add a text label to the current plot
pro DB_Display1D::addText, event, _EXTRA=etc

; Retrieve required objects from the IDL object graphics heirarchy
oView = (self.oScene)->get(/all,isa='IDLgrView')
oMiscCont = (self.oScene)->get(isa='IDL_Container')
oModel = oView->getByName('DG1DMODEL')

; Get the viewplane rectangle
oView->getProperty, viewplane_rect=vprect

; Get nos of existing labels
if (not (self.oDaveGr)->get(nlabel=nlabel,label=labelArray)) then return

; New Label
newLabel = {annotate}
newLabel.text = 'New text'
newLabel.position = [0.5,0.5,0.0]
newLabel.alignment = 0.5
newLabel.color = [0,0,0]
newLabel.font = 'HELVETICA'
newLabel.fontsize = 10

; Create graphic atom using label
oLabelModel = obj_new('IDLgrModel',name='LABMODEL',/select_target) ; make it selectable
oModel->add, oLabelModel
xs = scale_coord([0,1],[vprect[0],vprect[0]+vprect[2]])
ys = scale_coord([0,1],[vprect[1],vprect[1]+vprect[3]])
oLabelFont = obj_new('IDLgrFont',name=newLabel.font,size=newLabel.fontsize)
oMiscCont->add,oLabelFont       ; so it can be properly cleaned up
oLabel = obj_new('IDLgrText',newLabel.text,name='LABEL',font=oLabelFont,/enable_formatting $
                 ,location=newLabel.position,alignment=newLabel.alignment,color=newLabel.color $
                 ,xcoord_conv=xs,ycoord_conv=ys,uvalue={objRef:(self.oDaveGr),datInd:nlabel})
oLabelModel->add,oLabel

; Add the new label to the DaveGr object
labelArray = (nlabel gt 0)? [labelArray,newLabel] : newLabel
void = (self.oDaveGr)->set(label=labelArray)

print,vprect

; Refresh display
self->noRenew
self->refresh

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
;  Print method
; - Print current display
; - Export to various file formats
pro DB_Display1D::print, event, _EXTRA=etc
uname = widget_info(event.id,/uname)

switch uname of
    'TIFF':
    'BMP':
    'PNG':
    'JPEG': begin
        ;; force a window refresh before capturing image
        self.renewScene = 0
        self->refresh
        ;; capture destination window contents
        oImage = (self.oDest)->read()
        oImage->getProperty, data=image
        obj_destroy, oImage
        title = 'DATABROWSER - Export '+uname+' Image'
        defFile = 'output.'+strlowcase(uname)
        ;print,self.workDir
        result = dialog_write_image(image,dialog_parent=event.top,path=self.workDir, $
                                    title=title,/warn_exist,type=uname,file=defFile)
        break
    end
    'POST': begin
        defFile = 'output.ps'
        title = 'DATABROWSER - Export Postscript'
        options = PSConfig(cancel=cancelled,group_leader=event.top,directory=self.workDir, $
                              filename=defFile)
        if (not cancelled) then begin
            if (options.landscape eq 1) then begin
                yoff = options.xoffset
                xoff = options.yoffset - options.xsize
            endif else begin
                xoff = options.xoffset
                yoff = options.yoffset
            endelse
            paperxsize = 2*xoff + options.xsize
            paperysize = 2*yoff + options.ysize
            units = (options.inches eq 1)? 1 : 2

            ;; Create a clipboard
            oClip = obj_new('IDLgrClipboard',units=units,dimensions=[paperxsize,paperysize])

            ;; Get the scene/view that is being displayed.
            oView = (self.oScene)->get(/all,isa='IDLgrView')
            oView->getProperty, location=winLoc,dimensions=winDim,units=winUnits
            oView->setProperty, units=units, $
                                dimensions=[options.xsize,options.ysize], $
                                location=[xoff,yoff]

            ;; Render the scene to the clipboard
            oClip->draw, self.oScene, file=options.filename,/postscript,/vector

            ;; Cleapup and revert view to previous settings
            obj_destroy, oClip
            oView->setProperty, location=winLoc,dimensions=winDim,units=winUnits
        endif

        break
    end
    'PRINT': begin
        oPrint = obj_new('IDLgrPrinter')
        title = 'DATABROWSER - Printer Setup'
        valid = dialog_printersetup(oPrint,dialog_parent=event.top,title=title)
        if (valid) then begin
            oPrint->draw, (self.oScene)
            oPrint->newDocument ; forces current document to be sent to printer
        endif
        obj_destroy, oPrint
        break
    end
endswitch


end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Handler for draw widget events
pro DB_Display1D::drawEvent, event

uname = widget_info(event.id,/uname)

possibleEventTypes = ['DOWN','UP','MOTION','SCROLL','EXPOSE']
eventType = possibleEventTypes[event.type]

case eventType of
    'DOWN': begin               ; Button press event
        self.renewScene = 0     ; ==> don't re-construct Scene
        if (event.press eq 4) then self->dispConSenMenu, event ; Right Mouse button was pressed; display Con Sen Menu
        if (event.press eq 1) then begin ;Left mouse button was pressed; zoomin functionality is required

            self.rbStart = [event.x,event.y] ; register the mouse location
            widget_control, event.id, draw_motion_events=1 ; switch on motion events

            ;; get properties of view which contains current plot
            objArr = (self.oScene)->get(/all, count=nObj)
            while (nObj gt 0) do begin
                nObj = nObj - 1
                if (vartype(objArr[nObj]) ne 'IDLGRVIEW') then continue
                (self.oDest)->getProperty, dimensions = winDims ; dimensions of destination window
                (objArr[nObj])->getProperty, all=allViewProps ; properties of view gr object
                ;; create a rubberband obj gr atom
                rbandColor = 255 - allViewProps.color ; set rub band color to reverse of view color
                vprect = allViewProps.viewplane_rect ; dimensions of viewplane
                vprect[2] = vprect[2]/float(winDims[0])
                vprect[3] = vprect[3]/float(winDims[1])
                self.rbScale = vprect
                rbandModel = obj_new('IDLgrModel',name='RBANDMODEL')
                self.oRband = obj_new('IDLgrPolyline',name='RBAND',color=rbandColor)
                (objArr[nObj])->add, rbandModel
                rbandModel->add, self.oRband
                nObj = 0
            endwhile
        endif
    end
    'UP': begin                 ; Button release event
        if (event.release eq 1) then begin ; Left mouse button released
            ;; cleanup rubberband obj gr model and atom
            if (obj_valid(self.oRband)) then begin
                (self.oRband)->getProperty, parent=rbandModel
                rbandModel->getProperty, parent=oView
                oView->remove, rbandModel
                obj_destroy, rbandModel ; should destroy rub band atom as well
            endif

            widget_control, event.id, draw_motion_events=0 ; switch off motion events
            widget_Control, event.id, /clear_events ; clear any motion events that may have occurred.
                                ;self.LMBisDown = 0

            if (not self.mouseMoved) then break ; Exit now if releasing from same spot that mouse was pressed

            ;; rubberband box is new zoom region
            self.xrangeFlag = 1
            xmin = self.rbStart[0]*self.rbScale[2] + self.rbScale[0]
            xmax = event.x*self.rbScale[2] + self.rbScale[0]
            self.xrange = (xmax gt xmin)? [xmin,xmax] : [xmax,xmin]
            self.yrangeFlag = 1
            ymin = self.rbStart[1]*self.rbScale[3] + self.rbScale[1]
            ymax = event.y*self.rbScale[3] + self.rbScale[1]
            self.yrange = (ymax gt ymin)? [ymin,ymax] : [ymax,ymin]

                                ;self.renewScene = 0 ; ==> don't re-construct Scene
            self->refresh       ; refresh plot scene via self

        endif
    end
    'MOTION': begin             ; Motion event
        self.mouseMoved = 1     ; register mouse move

        ;; determine rubber band box dimensions and modify the rub
        ;; band obj gr atom
        start = self.rbStart
        boxVertices = fltarr(2,5)
        boxVertices[0,*] = [start[0],event.x,event.x,start[0],start[0]]*self.rbScale[2] + self.rbScale[0]
        boxVertices[1,*] = [start[1],start[1],event.y,event.y,start[1]]*self.rbScale[3] + self.rbScale[1]
        (self.oRband)->setProperty, data=boxVertices

                                ;(self.oRband)->getProperty, parent=rbandModel
                                ;rbandModel->getProperty, parent=rbandView
        self->noRenew           ; ==> don't re-construct Scene
        self->refresh           ; refresh plot scene via self
                                ;(self.oDest)->draw, rbandView ; draw rubber band
    end
    'SCROLL':
    'EXPOSE': begin             ; Expose event, so repaint window
                                ;print,'Exposed Event generated...'
        self->noRenew           ; ==> don't re-construct Scene
        if (!debug) then print, 'EXPOSED EVENT: renescene = ',self.renewScene
        self->refresh           ; refresh via self
    end
endcase


end


;===============================================================================
; Exit (quit)
pro DB_Display1D::exitEvent, event

widget_control,event.top,/destroy

end


;===============================================================================
; Refresh graphics window
pro DB_Display1D::displayEvent, event

self->refresh      ; refresh via self

end

;===============================================================================
; Switch off scene re-construction
pro DB_Display1D::noRenew, event

self.renewScene = 0

end

;===============================================================================
; Switch on scene re-construction
pro DB_Display1D::renew, event

self.renewScene = 1

end

;===============================================================================
; Reset to default axis limits (ie data limits)
pro DB_Display1D::defLimEvent, event

self.xrangeFlag = 0
self.yrangeFlag = 0
self->refresh                   ; refresh via self

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Switch display mode (type)
pro DB_Display1D::dispTypeEvent, event

; switch off some items in the CSBLINE con. sen. menu
widget_control, self.csbline_symID, sensitive=self.dispType
widget_control, self.csbline_sylID, sensitive=self.dispType

; swap display type
self.dispType = self.dispType xor 1 ; swap setting (0 <--> 1)

;endcase

self->refresh                   ; refresh via registry

end


;===============================================================================
; Switch Error bars on/off
pro DB_Display1D::dispErrorEvent, event

self.dispError = self.dispError xor 1 ; swap setting (0 <--> 1)

self->refresh                   ; refresh via self

end

;===============================================================================
; Switch legend on/off
pro DB_Display1D::dispLegendEvent, event

if (not (self.oDaveGr)->get(legend=legend)) then return
legend.hideFlag = 1 - legend.hideFlag ; swap setting (0 <--> 1)
if (not (self.oDaveGr)->set(legend=legend)) then return

self->refresh                   ; direct refresh (not via registry)

end


;===============================================================================
; Switch main Title on/off
pro DB_Display1D::dispTitleEvent, event

self.dispTitle = 1 - self.dispTitle ; swap setting (0 <--> 1)

self->refresh                   ;  direct refresh (not via registry)

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Resize
pro DB_Display1D::tlbEvent, event

if dave_set_focus(event) then return

res = widget_info(event.top,/geometry)
new_x = event.x - self.dialog_x - 2*res.xpad
new_y = event.y - 2*res.ypad
new_x = new_x > self.min_draw_x
new_y = new_y > self.min_draw_y
(self.oDest)->setProperty, dimensions=[new_x,new_y]
(self.oDest)->getProperty, dimensions=dim
self.winDim = dim

;print,'Refreshing b/c of resize event...'
self->refresh                   ; refresh via self

end



;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Main (conventional) Event Handler for DB_Display1D
;
pro db_display1d_event, event
;if dave_set_focus(event) then return

; Retrieve the command structure embedded in the uvalue of the
; widget. The command structure is stored in the uvalue of either
; event.id or evend.handler
widget_control, event.id, get_uvalue = cmd

childID = event.id
while (vartype(cmd) ne 'DB_DISPLAY1D_CMD') do begin
    parentID = widget_info(childID,/parent)
    if (parentID eq 0) then break
    widget_control, parentID, get_uvalue = cmd
    childID = parentID
endwhile


if (vartype(cmd) eq 'DB_DISPLAY1D_CMD') then $
   call_method, cmd.method, cmd.object, event

end


;===============================================================================
; Richard Tumanjong Azuah
; NCNR, NIST
;
; Widget Cleanup procedure for DB_Display1D that would be called by
; XMANAGER when DB_display1D's tlb is destroyed
;
pro db_display1d_cleanup, tlb
widget_control, tlb, get_uvalue = cmd
; get the Widget Object's reference and destroy it
selfObject = cmd.object
obj_destroy, selfObject

end


;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST Feb, 2003
;
; NAME:_______________________
;   DB_Display1D::INIT
;
; PURPOSE:
;   Widget dialog program which handles the display DaveGr1D object(s).
;
; INPUT PARAMETERS:___________
;   PARAMETERS:
;
;
;   KEYWORDS:
;
;
; OUTPUTS:____________________
;
;     none
;
function DB_Display1D::init, group_leader=group_leader, objGrRef=oDaveGr, workDir=workDir

;
device, get_decomposed = old_dec
self.old_dec = old_dec
;device, decomposed=1

; create a structure to specify methods that should act as event
; handlers for widgets defined in DB_Display1D
cmdStr = {DB_DISPLAY1D_CMD, $
          object:self, $ ; reference to the widget object generating events
          method:'' $         ; the method that should handle the event
          ;parPtr:ptr_new() $  ; extra parameters to be passed to the method
         }



; Various tooltips
deTooltip = 'Togle on/off errorbars.'
pmTooltip = 'Switch plot mode. View data as points or histograms.'
refTooltip = 'Refresh the plot window'
resTooltip = 'Reset x,y-axis display limits to full range.'
exitTooltip = 'Close display and return to main DATABROWSER.'
ptTooltip = 'Set/modify plot type (as lines and/or symbols).'
lsTooltip = 'Set/modify plot linestyle.'
pthTooltip = 'Set/modify plot line thickness.'
symTooltip = 'Set/modify plot symbols.'
stTooltip = 'Set/modify symbol type.'
szTooltip = 'Set/modify symbol size.'
pcolTooltip = 'Set/modify plot color.'
acolTooltip = 'Set/modify axes color.'
atTooltip = 'Set/modify axis line thickness.'
addTextTip = 'Add a new text label to plot.'

; define the top-level-base
cmdStr.method = 'tlbEvent'
if (n_elements(group_leader) eq 0) then begin
    group_leader=0L
    modal=0
    self.dispType = 0b
    usingPrevGuiSettings = 0b
    self.tlb = widget_base(/row,title=title,/align_right,/tlb_size_events,uname='TLB' $
                           ,uvalue=cmdStr,group_leader=group_leader,mbar=menubar)
endif else begin
    modal=1
    widget_control, group_leader, get_uvalue=sPtr

    ;; Get the DaveGr object to be viewed
    if (n_elements(oDaveGr) gt 0) then begin
        self.oDaveGr = oDaveGr
    endif else begin
        oSelCont = (*sPtr).sel_cont
        self.oDaveGr = oSelCont->get(position=0)
    endelse

    ;; Can't allow multiple views of the same DaveGr1D object so make
    ;; some checks now
    self.oActive1DCont = (*sPtr).o_active_gr1D_cont
    if ((self.oActive1DCont)->count() gt 0) then begin
        ;; Exit immediately if DaveGr object is already being viewed
        if ((self.oActive1DCont)->isContained(self.oDaveGr)) then return, 0
    endif
    ;; register DaveGr object
    (self.oActive1DCont)->add, self.oDaveGr

    ; Retrieve ptr to previous GUI/widget settings
    if (not (self.oDaveGr)->get(guiAttribsPtr=guiAttribsPtr)) then return, 0
    usingPrevGuiSettings = (n_elements(*guiAttribsPtr) gt 0)? 1b : 0b

    ;; Get the first dataset in the DaveGr1D object and determine the
    ;; x-axis datatype
    if (not (self.oDaveGr)->get(dataRef=dataRef, position=0)) then return, 0
    if (not dataRef->get(xtype=xtype)) then return, 0
    xtype = strmid(strupcase(xtype),0,4)
    self.dispType = (xtype eq 'HIST')? 1b : 0b ; if xtype is not set, use 'points'
    if (usingPrevGuiSettings) then begin
        self.dispType = (*guiAttribsPtr).dispType
        self.xrangeFlag = (*guiAttribsPtr).xrangeFlag
        self.yrangeFlag = (*guiAttribsPtr).yrangeFlag
        self.xrange = (*guiAttribsPtr).xrange
        self.yrange = (*guiAttribsPtr).yrange
    endif

    void = (self.oDaveGr)->get(nameTag=title)
    title = 'DATABROWSER - '+title
    self.o_registry = (*sPtr).o_registry ; get the object registry
    (self.o_registry)->add, self    ; and register
    ;self.tlb = widget_base(group_leader=group_leader,/row,title=title,modal=modal $
    ;                 ,/align_right,/tlb_size_events,uname='TLB',uvalue=cmdStr)
    self.tlb = widget_base(group_leader=group_leader,/row,title=title $
                     ,/align_right,/tlb_size_events,uname='TLB',uvalue=cmdStr,mbar=menubar)
endelse

; Get working directory setting
if (n_elements(workDir) le 0) then $
  cd, current=workDir
self.workDir = workDir

; File Menu
filem = widget_button(menubar,value='File',/menu)
cmdStr.method='print'
expm = widget_button(filem,value='Export image as',/menu)
void = widget_button(expm,value='Postscript',uname='POST',uvalue=cmdStr)
void = widget_button(expm,value='Jpeg',uname='JPEG',uvalue=cmdStr,/separator)
void = widget_button(expm,value='Png',uname='PNG',uvalue=cmdStr)
void = widget_button(expm,value='Bmp',uname='BMP',uvalue=cmdStr)
void = widget_button(expm,value='Tiff',uname='TIFF',uvalue=cmdStr)
void = widget_button(filem,value='Print',uname='PRINT',uvalue=cmdStr)
cmdStr.method = 'exitEvent'
void = widget_button(filem,value='Close Window',/separator,uname='EXIT',uvalue=cmdStr)

; split TLB into:
;  1) an left pane for placing dialog buttons
;  2) a right pane for placing the draw window
cb1 = widget_base(self.tlb,/col)
cb2 = widget_base(self.tlb,/frame)

; Split the dialog base into several sections for grouping similar controls
cb11 = widget_base(cb1,/col,/align_center,/frame)
cb12 = widget_base(cb1,/col,/align_center,/frame)
cb13 = widget_base(cb1,/col,/align_center,/frame)
cb14 = widget_base(cb1,/col,/align_center)

; Now the buttons
;void = cw_bgroup(cb11,['Multiplot'],/col,/nonexclusive,frame=0,/return_index,uname='MULTIPLOT',set_value=[1])
;values=['Red','Green','Blue']
;void = widget_combobox(cb11,value=values,uname='DATASELECTION',sensitive=0)


plotTypes = ['Point Mode','Histogram Mode']
cmdStr.method = 'dispTypeEvent'
void = cw_bgroup(cb11,plotTypes,/col,/exclusive,/no_release,frame=0,set_value=self.dispType $
                 ,/return_index,uname='DISPTYPE',uvalue=cmdStr)

self.dispError = (usingPrevGuiSettings)? (*guiAttribsPtr).dispError : 0
cmdStr.method = 'dispErrorEvent'
void = cw_bgroup(cb11,['Display Error'],/col,/nonexclusive,frame=0,/return_index,uname='DISPERROR' $
                 ,set_value=self.dispError,uvalue=cmdStr)

res = widget_info(cb11,/geometry)
xsize = res.xsize - 2*res.xpad

self.dispTitle = (usingPrevGuiSettings)? (*guiAttribsPtr).dispTitle : 1
cmdStr.method = 'dispTitleEvent'
void = cw_bgroup(cb12,['Display Title'],/col,/nonexclusive,frame=0,/return_index,uname='DISPTITLE' $
                 ,set_value=self.dispTitle,uvalue=cmdStr)

cmdStr.method = 'dispLegendEvent'
void = (self.oDaveGr)->get(legend=legend)
void = cw_bgroup(cb12,['Display Legend'],/col,/nonexclusive,frame=0,/return_index,uname='DISPLEGEND' $
                 ,uvalue=cmdStr,set_value=(1 - legend.hideFlag))
cmdStr.method = 'addText'
void = widget_button(cb12,value='Add Text',uname='ADDTEXT',uvalue=cmdStr,tooltip=addTextTip)
cmdStr.method = 'todoEvent'
void = widget_button(cb12,value='Edit X-axis',uname='EXAXIS',xsize=xsize,uvalue=cmdStr,sensitive=0)
void = widget_button(cb12,value='Edit Y-axis',uname='EYAXIS',uvalue=cmdstr,sensitive=0)

cmdStr.method = 'displayEvent'
void = widget_button(cb13,value='Refresh',uname='DISPLAY',xsize=xsize,uvalue=cmdStr,tooltip=refTooltip)
cmdStr.method = 'defLimEvent'
void = widget_button(cb13,value='Reset Limits',uname='DEFLIM',xsize=xsize,uvalue=cmdStr,tooltip=resTooltip)
cmdStr.method = 'exitEvent'
void = widget_button(cb13,value='Close Window',uname='EXIT',uvalue=cmdstr,tooltip=exitTooltip)

; store useful widget dimensions in self (widget object)
res = widget_info(cb1,/geometry)
self.dialog_x = res.xsize
dialog_y = res.ysize
self.min_draw_y = fix(dialog_y*1.10)
self.min_draw_x = fix(self.min_draw_y*1.333)
self.winDim = (usingPrevGuiSettings)? (*guiAttribsPtr).winDim : [self.min_draw_x,self.min_draw_y]
cmdStr.method = 'drawEvent'
drawID = widget_draw(cb2,xsize=self.winDim[0],ysize=self.winDim[1],graphics_level=2,uname='DRAW' $
                     ,/button_events,/expose_events,uvalue=cmdStr)

;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Line Plots'.
cmdStr.method = 'handleConSenMenuEvent'
csb1 = widget_base(self.tlb, /context_menu,uname='CSBLINE',uvalue=cmdStr)
lstyle = widget_button(csb1,value='Line Style',/menu,tooltip=lsTooltip)
thick = widget_button(csb1,value='Line Thickness',/menu,tooltip=ltTooltip)
color = widget_button(csb1,value='Line Color',/menu,tooltip=pcolTooltip)
ptm = widget_button(csb1,value='Plot Type',/menu,tooltip=ptTooltip)
void = widget_button(ptm,value='Lines',uvalue=0,uname='SWITCHDISPMODE',/checked_menu)
widget_control, void, set_button=1
self.csbline_symID = widget_button(ptm,value='Symbols',uvalue=1,uname='SWITCHDISPMODE',/checked_menu)
self.csbline_sylID = widget_button(ptm,value='Lines+Symbols',uvalue=2,uname='SWITCHDISPMODE',/checked_menu)

void = widget_button(lstyle,value='Solid',uname='LSTYLE',uvalue=0)
void = widget_button(lstyle,value='Dotted',uname='LSTYLE',uvalue=1)
void = widget_button(lstyle,value='Dashed',uname='LSTYLE',uvalue=2)
void = widget_button(lstyle,value='Dash Dot',uname='LSTYLE',uvalue=3)
void = widget_button(lstyle,value='Dash Dot Dot Dot',uname='LSTYLE',uvalue=4)
void = widget_button(lstyle,value='Long Dashes',uname='LSTYLE',uvalue=5)

void = widget_button(thick,value='1.00',uname='THICK',uvalue=1.00)
void = widget_button(thick,value='1.25',uname='THICK',uvalue=1.25)
void = widget_button(thick,value='1.50',uname='THICK',uvalue=1.50)
void = widget_button(thick,value='1.75',uname='THICK',uvalue=1.75)
void = widget_button(thick,value='2.00',uname='THICK',uvalue=2.00)
void = widget_button(thick,value='2.25',uname='THICK',uvalue=2.25)
void = widget_button(thick,value='2.50',uname='THICK',uvalue=2.50)
void = widget_button(thick,value='2.75',uname='THICK',uvalue=2.75)
void = widget_button(thick,value='3.00',uname='THICK',uvalue=3.00)
void = widget_button(thick,value='3.25',uname='THICK',uvalue=3.25)
void = widget_button(thick,value='3.50',uname='THICK',uvalue=3.50)
void = widget_button(thick,value='3.75',uname='THICK',uvalue=3.75)
void = widget_button(thick,value='4.00',uname='THICK',uvalue=4.00)
void = widget_button(thick,value='4.25',uname='THICK',uvalue=4.25)
void = widget_button(thick,value='4.50',uname='THICK',uvalue=4.50)
void = widget_button(thick,value='4.75',uname='THICK',uvalue=4.75)
void = widget_button(thick,value='5.00',uname='THICK',uvalue=5.00)


void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Symbol Plots'
csb2 = widget_base(self.tlb, /context_menu,uname='CSBSYMB',uvalue=cmdStr)
stype = widget_button(csb2,value='Symbol Type',/menu,tooltip=stTooltip)
ssize = widget_button(csb2,value='Symbol Size',/menu,tooltip=szTooltip)
thick = widget_button(csb2,value='Line Thickness',/menu,tooltip=pthTooltip)
color = widget_button(csb2,value='Symbol Color',/menu,tooltip=pcolTooltip)
ptm = widget_button(csb2,value='Plot Type',/menu,tooltip=pttooltip)
void = widget_button(ptm,value='Lines',uvalue=0,uname='SWITCHDISPMODE',/checked_menu)
void = widget_button(ptm,value='Symbols',uvalue=1,uname='SWITCHDISPMODE',/checked_menu)
widget_control, void, set_button=1
void = widget_button(ptm,value='Lines+Symbols',uvalue=2,uname='SWITCHDISPMODE',/checked_menu)

void = widget_button(stype,value='Plus',uname='STYPE',uvalue=1)
void = widget_button(stype,value='Asterisk',uname='STYPE',uvalue=2)
void = widget_button(stype,value='Period',uname='STYPE',uvalue=3)
void = widget_button(stype,value='Diamond',uname='STYPE',uvalue=4)
void = widget_button(stype,value='Triangle',uname='STYPE',uvalue=5)
void = widget_button(stype,value='Square',uname='STYPE',uvalue=6)
void = widget_button(stype,value='Cross',uname='STYPE',uvalue=7)

void = widget_button(ssize,value='1.00',uname='SSIZE',uvalue=1.00)
void = widget_button(ssize,value='1.25',uname='SSIZE',uvalue=1.25)
void = widget_button(ssize,value='1.50',uname='SSIZE',uvalue=1.50)
void = widget_button(ssize,value='1.75',uname='SSIZE',uvalue=1.75)
void = widget_button(ssize,value='2.00',uname='SSIZE',uvalue=2.00)
void = widget_button(ssize,value='2.25',uname='SSIZE',uvalue=2.25)
void = widget_button(ssize,value='2.50',uname='SSIZE',uvalue=2.50)
void = widget_button(ssize,value='2.75',uname='SSIZE',uvalue=2.75)
void = widget_button(ssize,value='3.00',uname='SSIZE',uvalue=3.00)
void = widget_button(ssize,value='3.25',uname='SSIZE',uvalue=3.25)
void = widget_button(ssize,value='3.50',uname='SSIZE',uvalue=3.50)
void = widget_button(ssize,value='3.75',uname='SSIZE',uvalue=3.75)
void = widget_button(ssize,value='4.00',uname='SSIZE',uvalue=4.00)
void = widget_button(ssize,value='4.25',uname='SSIZE',uvalue=4.25)
void = widget_button(ssize,value='4.50',uname='SSIZE',uvalue=4.50)
void = widget_button(ssize,value='4.75',uname='SSIZE',uvalue=4.75)
void = widget_button(ssize,value='5.00',uname='SSIZE',uvalue=5.00)

void = widget_button(thick,value='1.00',uname='THICK',uvalue=1.00)
void = widget_button(thick,value='1.25',uname='THICK',uvalue=1.25)
void = widget_button(thick,value='1.50',uname='THICK',uvalue=1.50)
void = widget_button(thick,value='1.75',uname='THICK',uvalue=1.75)
void = widget_button(thick,value='2.00',uname='THICK',uvalue=2.00)
void = widget_button(thick,value='2.25',uname='THICK',uvalue=2.25)
void = widget_button(thick,value='2.50',uname='THICK',uvalue=2.50)
void = widget_button(thick,value='2.75',uname='THICK',uvalue=2.75)
void = widget_button(thick,value='3.00',uname='THICK',uvalue=3.00)
void = widget_button(thick,value='3.25',uname='THICK',uvalue=3.25)
void = widget_button(thick,value='3.50',uname='THICK',uvalue=3.50)
void = widget_button(thick,value='3.75',uname='THICK',uvalue=3.75)
void = widget_button(thick,value='4.00',uname='THICK',uvalue=4.00)
void = widget_button(thick,value='4.25',uname='THICK',uvalue=4.25)
void = widget_button(thick,value='4.50',uname='THICK',uvalue=4.50)
void = widget_button(thick,value='4.75',uname='THICK',uvalue=4.75)
void = widget_button(thick,value='5.00',uname='THICK',uvalue=5.00)

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])


;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Line + Symbol Plots'
csb3 = widget_base(self.tlb, /context_menu,uname='CSBLSYM',uvalue=cmdStr)
stype = widget_button(csb3,value='Symbol Type',/menu,tooltip=stTooltip)
ssize = widget_button(csb3,value='Symbol Size',/menu,tooltip=szTooltip)
lstyle = widget_button(csb3,value='Line Style',/menu,tooltip=lsTooltip)
thick = widget_button(csb3,value='Line/Symbol Thickness',/menu,tooltip=pthTooltip)
color = widget_button(csb3,value='Line/Symbol Color',/menu,tooltip=pcolTooltip)
ptm = widget_button(csb3,value='Plot Type',/menu,tooltip=ptTooltip)
void = widget_button(ptm,value='Lines',uvalue=0,uname='SWITCHDISPMODE',/checked_menu)
void = widget_button(ptm,value='Symbols',uvalue=1,uname='SWITCHDISPMODE',/checked_menu)
void = widget_button(ptm,value='Lines+Symbols',uvalue=2,uname='SWITCHDISPMODE',/checked_menu)
widget_control, void, set_button=1

void = widget_button(stype,value='Plus',uname='STYPE',uvalue=1)
void = widget_button(stype,value='Asterisk',uname='STYPE',uvalue=2)
void = widget_button(stype,value='Period',uname='STYPE',uvalue=3)
void = widget_button(stype,value='Diamond',uname='STYPE',uvalue=4)
void = widget_button(stype,value='Triangle',uname='STYPE',uvalue=5)
void = widget_button(stype,value='Square',uname='STYPE',uvalue=6)
void = widget_button(stype,value='Cross',uname='STYPE',uvalue=7)

void = widget_button(ssize,value='1.00',uname='SSIZE',uvalue=1.00)
void = widget_button(ssize,value='1.25',uname='SSIZE',uvalue=1.25)
void = widget_button(ssize,value='1.50',uname='SSIZE',uvalue=1.50)
void = widget_button(ssize,value='1.75',uname='SSIZE',uvalue=1.75)
void = widget_button(ssize,value='2.00',uname='SSIZE',uvalue=2.00)
void = widget_button(ssize,value='2.25',uname='SSIZE',uvalue=2.25)
void = widget_button(ssize,value='2.50',uname='SSIZE',uvalue=2.50)
void = widget_button(ssize,value='2.75',uname='SSIZE',uvalue=2.75)
void = widget_button(ssize,value='3.00',uname='SSIZE',uvalue=3.00)
void = widget_button(ssize,value='3.25',uname='SSIZE',uvalue=3.25)
void = widget_button(ssize,value='3.50',uname='SSIZE',uvalue=3.50)
void = widget_button(ssize,value='3.75',uname='SSIZE',uvalue=3.75)
void = widget_button(ssize,value='4.00',uname='SSIZE',uvalue=4.00)
void = widget_button(ssize,value='4.25',uname='SSIZE',uvalue=4.25)
void = widget_button(ssize,value='4.50',uname='SSIZE',uvalue=4.50)
void = widget_button(ssize,value='4.75',uname='SSIZE',uvalue=4.75)
void = widget_button(ssize,value='5.00',uname='SSIZE',uvalue=5.00)

void = widget_button(lstyle,value='Solid',uname='LSTYLE',uvalue=0)
void = widget_button(lstyle,value='Dotted',uname='LSTYLE',uvalue=1)
void = widget_button(lstyle,value='Dashed',uname='LSTYLE',uvalue=2)
void = widget_button(lstyle,value='Dash Dot',uname='LSTYLE',uvalue=3)
void = widget_button(lstyle,value='Dash Dot Dot Dot',uname='LSTYLE',uvalue=4)
void = widget_button(lstyle,value='Long Dashes',uname='LSTYLE',uvalue=5)

void = widget_button(thick,value='1.00',uname='THICK',uvalue=1.00)
void = widget_button(thick,value='1.25',uname='THICK',uvalue=1.25)
void = widget_button(thick,value='1.50',uname='THICK',uvalue=1.50)
void = widget_button(thick,value='1.75',uname='THICK',uvalue=1.75)
void = widget_button(thick,value='2.00',uname='THICK',uvalue=2.00)
void = widget_button(thick,value='2.25',uname='THICK',uvalue=2.25)
void = widget_button(thick,value='2.50',uname='THICK',uvalue=2.50)
void = widget_button(thick,value='2.75',uname='THICK',uvalue=2.75)
void = widget_button(thick,value='3.00',uname='THICK',uvalue=3.00)
void = widget_button(thick,value='3.25',uname='THICK',uvalue=3.25)
void = widget_button(thick,value='3.50',uname='THICK',uvalue=3.50)
void = widget_button(thick,value='3.75',uname='THICK',uvalue=3.75)
void = widget_button(thick,value='4.00',uname='THICK',uvalue=4.00)
void = widget_button(thick,value='4.25',uname='THICK',uvalue=4.25)
void = widget_button(thick,value='4.50',uname='THICK',uvalue=4.50)
void = widget_button(thick,value='4.75',uname='THICK',uvalue=4.75)
void = widget_button(thick,value='5.00',uname='THICK',uvalue=5.00)

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Axes'
csb4 = widget_base(self.tlb, /context_menu,uname='CSBAXES',uvalue=cmdStr)
thick = widget_button(csb4,value='Line Thickness',/menu)
color = widget_button(csb4,value='Axes Color',/menu)
afont = widget_button(csb4,value='Axes Font Size',/menu)
alog = widget_button(csb4,value='Log Scale',uname='LOGSCALE')
void = widget_button(csb4,value='Properties...',uname='EDITAXIS')

void = widget_button(thick,value='1.00',uname='THICK',uvalue=1.00)
void = widget_button(thick,value='1.25',uname='THICK',uvalue=1.25)
void = widget_button(thick,value='1.50',uname='THICK',uvalue=1.50)
void = widget_button(thick,value='1.75',uname='THICK',uvalue=1.75)
void = widget_button(thick,value='2.00',uname='THICK',uvalue=2.00)
void = widget_button(thick,value='2.25',uname='THICK',uvalue=2.25)
void = widget_button(thick,value='2.50',uname='THICK',uvalue=2.50)
void = widget_button(thick,value='2.75',uname='THICK',uvalue=2.75)
void = widget_button(thick,value='3.00',uname='THICK',uvalue=3.00)
void = widget_button(thick,value='3.25',uname='THICK',uvalue=3.25)
void = widget_button(thick,value='3.50',uname='THICK',uvalue=3.50)
void = widget_button(thick,value='3.75',uname='THICK',uvalue=3.75)
void = widget_button(thick,value='4.00',uname='THICK',uvalue=4.00)
void = widget_button(thick,value='4.25',uname='THICK',uvalue=4.25)
void = widget_button(thick,value='4.50',uname='THICK',uvalue=4.50)
void = widget_button(thick,value='4.75',uname='THICK',uvalue=4.75)
void = widget_button(thick,value='5.00',uname='THICK',uvalue=5.00)

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

void = widget_button(afont,value='8',uname='FONTSZ',uvalue=8)
void = widget_button(afont,value='10',uname='FONTSZ',uvalue=10)
void = widget_button(afont,value='12',uname='FONTSZ',uvalue=12)
void = widget_button(afont,value='14',uname='FONTSZ',uvalue=14)
void = widget_button(afont,value='16',uname='FONTSZ',uvalue=16)
void = widget_button(afont,value='18',uname='FONTSZ',uvalue=18)
void = widget_button(afont,value='20',uname='FONTSZ',uvalue=20)
void = widget_button(afont,value='22',uname='FONTSZ',uvalue=22)
void = widget_button(afont,value='24',uname='FONTSZ',uvalue=24)
void = widget_button(afont,value='26',uname='FONTSZ',uvalue=26)
void = widget_button(afont,value='28',uname='FONTSZ',uvalue=28)

;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Labels'
csb5 = widget_base(self.tlb, /context_menu,uname='CSBLABEL',uvalue=cmdStr)
color = widget_button(csb5,value='Color',/menu)
labfont = widget_button(csb5,value='Font Size',/menu)
void = widget_button(csb5,value='Properties...',uname='EDITLABLE')

void = widget_button(color,value='Aqua',uname='COLOR',uvalue=[0,255,255])
void = widget_button(color,value='Black',uname='COLOR',uvalue=[0,0,0])
void = widget_button(color,value='Blue',uname='COLOR',uvalue=[0,0,255])
void = widget_button(color,value='Fuchsia',uname='COLOR',uvalue=[255,0,255])
void = widget_button(color,value='Gray',uname='COLOR',uvalue=[128,128,128])
void = widget_button(color,value='Green',uname='COLOR',uvalue=[0,128,0])
void = widget_button(color,value='Lime',uname='COLOR',uvalue=[0,255,0])
void = widget_button(color,value='Maroon',uname='COLOR',uvalue=[128,0,0])
void = widget_button(color,value='Navy',uname='COLOR',uvalue=[0,0,128])
void = widget_button(color,value='Olive',uname='COLOR',uvalue=[128,128,0])
void = widget_button(color,value='Purple',uname='COLOR',uvalue=[128,0,128])
void = widget_button(color,value='Red',uname='COLOR',uvalue=[255,0,0])
void = widget_button(color,value='Silver',uname='COLOR',uvalue=[192,192,192])
void = widget_button(color,value='Teal',uname='COLOR',uvalue=[0,128,128])
void = widget_button(color,value='White',uname='COLOR',uvalue=[255,255,255])
void = widget_button(color,value='Yellow',uname='COLOR',uvalue=[255,255,0])

void = widget_button(labfont,value='8',uname='FONTSZ',uvalue=8)
void = widget_button(labfont,value='10',uname='FONTSZ',uvalue=10)
void = widget_button(labfont,value='12',uname='FONTSZ',uvalue=12)
void = widget_button(labfont,value='14',uname='FONTSZ',uvalue=14)
void = widget_button(labfont,value='16',uname='FONTSZ',uvalue=16)
void = widget_button(labfont,value='18',uname='FONTSZ',uvalue=18)
void = widget_button(labfont,value='20',uname='FONTSZ',uvalue=20)
void = widget_button(labfont,value='22',uname='FONTSZ',uvalue=22)
void = widget_button(labfont,value='24',uname='FONTSZ',uvalue=24)
void = widget_button(labfont,value='26',uname='FONTSZ',uvalue=26)
void = widget_button(labfont,value='28',uname='FONTSZ',uvalue=28)


;----------------------------------------------------------------------------
; Create Context Sensitive menus for 'Legend'
csb6 = widget_base(self.tlb, /context_menu,uname='CSBLEGEND',uvalue=cmdStr)
void = widget_button(csb6,value='Properties...',uname='EDITLEGEND')



;----------------------------------------------------------------------------
; realize widgets
widget_control, self.tlb,/realize

; add the destination object to self (widget object)
widget_control, drawID, get_value=oDest
self.oDest = oDest
oDest->getProperty, dimensions=dim
self.winDim = dim

; create and add a scene to self (widget object)
self.oScene = obj_new('IDLgrScene')

; assign self to uvalue of tlb
;widget_control, self.tlb, set_uvalue=self

; the graphics scene should be always reconstructed by default
self.renewScene = 1
registerName = 'db_display1d'
ret = dave_set_focus(self.tlb)
xmanager,registerName,self.tlb,/no_block,event_handler='db_display1d_event',cleanup='db_display1d_cleanup'

return,1
end


;===============================================================================
;
pro DB_Display1D__define

void = {DB_Display1D, $
        tlb:0L, $
        old_dec:0, $
        o_registry: obj_new(), $ ; object registry which handles communication with other programs
        oDest:obj_new(), $      ; the Destination object
        oScene:obj_new(), $     ; Scene to be drawn in the destination
        oDaveGr:obj_new(), $    ; the DaveGr object to be displayed
        oRband:obj_new(), $     ; a rubber band obj gr atom used for zooming
        oActive1DCont:obj_new(), $ ; contains a list of all DaveGr1D objects being actively viewed
        workDir:'', $           ; user's working directory
        csmObjRef:obj_new(), $  ; will contain the obj ref for object which triggers a con. sen menu
        renewScene:1b, $        ; flag to indicate whether the scene is to be completely reconstructed
        rbStart:[0,0], $        ; rubberband start point (for zooming)
        rbScale:[0.,0.,1.,1.], $ ; x,y scale factor to convert from device to data coordinates (for zooming)
        mouseMoved:0b, $        ; flag to indicate if mouse was moved while LMB is depressed
        xrangeFlag:0b, $        ; flag to indicate that user specified x-axis range should be used
        xrange:[0.,0.], $       ; user specified x-axis limits
        yrangeFlag:0b, $        ; flag to indicate that user specified y-axis range should be used
        yrange:[0.,0.], $       ; user specified y-axis limits
        csbline_symID:0, $      ; widget ID's for switch-to-symbols item in CSBLINE con. sen. base menu
        csbline_sylID:0, $      ; widget ID's for switch-to-symbols+lines item in CSBLINE con. sen. base menu
        min_draw_x:0, $         ; min dimensions for draw widget
        min_draw_y:0, $         ; min dimensions for draw widget
        dialog_x:0, $           ; min width for dialog portion of tlb
        winDim:[0,0], $        ; current draw window dimensions
        dispError:0b, $         ; flag to indicate display of error bars
        dispTitle:1b, $         ; flag to indicate if main title is to be displayed
        dispType:0b $           ; flag to indicate display mode; histogram/points
       }

end
