; $Id: db_editdescriptr.pro,v 1.6 2003/12/04 17:07:21 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   db_editDescriPtr
;
; PURPOSE:____________________
;   A small utility widget program to view/edit the contents of a
;   descriPtr (this a davePtr variable used to identify a dataset
;   within a series).
;
; INPUTS:_____________________
;   PARAMETER
;     descriPtr: a pointer to the descriPtr structure to be viewed/modified.
;   KEYWORDS
;     group_leader: the ID of the parent widget from which this dialog
;     was launched.
;
; OUTPUTS:____________________
;     return value: 
;        - 1 on success, 0 on failure 
;        - In addition, if successful, the descriPtr input parameter
;          will be modified to reflect the changes made in the dialog.
;
pro db_editDescriPtr_events, event

widget_control,event.top,get_uvalue = sPtr
uname = widget_info(event.id,/uname)

case uname of
    'ACCEPT': begin
        widID = widget_info(event.top,find_by_uname = 'NAME')
        widget_control, widID, get_value=value
        (*sPtr).name = strtrim(value,2)
        widID = widget_info(event.top,find_by_uname = 'LABEL')
        widget_control, widID, get_value=value
        (*sPtr).label = strtrim(value,2)
        widID = widget_info(event.top,find_by_uname = 'UNITS')
        widget_control, widID, get_value=value
        (*sPtr).units = strtrim(value,2)
        widID = widget_info(event.top,find_by_uname = 'VALUE')
        widget_control, widID, get_value=value
        (*sPtr).value = value
        widID = widget_info(event.top,find_by_uname = 'ERROR')
        widget_control, widID, get_value=value
        (*sPtr).error = value

        (*sPtr).cancel=0
        widget_control,event.top,/destroy
    end

    'CANCEL': begin
        (*sPtr).cancel=1
        widget_control,event.top,/destroy
    end

;    'NAME': begin
;        widget_control, event.id, get_value=value
;        (*sPtr).name = value
;    end
;
;    'LABEL': begin
;        widget_control, event.id, get_value=value
;        (*sPtr).label = value
;    end
;
;    'UNITS': begin
;        widget_control, event.id, get_value=value
;        (*sPtr).units = value
;    end
;
;    'VALUE': begin
;        widget_control, event.id, get_value=value
;        (*sPtr).value = value
;    end
;
;    'ERROR': begin
;        widget_control, event.id, get_value=value
;        (*sPtr).error = value
;    end

    else:

endcase

;print, 'Event handled...'

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function db_editDescriPtr, descriPtr, group_leader=group_leader

; A descriPtr parameter must be passed
if (n_params() ne 1) then return, 0

; descriPtr must be a valid ptr
if (ptr_valid(descriPtr) ne 1) then return, 0

; descriPtr must be a descriptrstr structure
if (vartype(*descriPtr) ne 'DESCRIPTRSTR') then return, 0

; The widgets
title = 'View/Edit Sample Information' 
if (n_elements(group_leader) eq 0) then begin
    group_leader = 0L
    modal = 0
    tlb = widget_base(/col,title=title,/align_center)
endif else begin
    modal = 1
    tlb = widget_base(group_leader=group_leader,/col,title=title,modal=modal,/align_center)
endelse

cb0 = widget_base(tlb,/col,/align_center)
void = widget_label(cb0,value='Modify the fields below to edit the Sample Information Tag.',/align_center)
void = widget_label(cb0,value="Name  - name of tag/variable without spaces, eg 'Temperature'",/align_left)
void = widget_label(cb0,value="Label - descriptive text, eg 'Sample Temperature'",/align_left)
void = widget_label(cb0,value="Units - the units of the tag/variable, eg 'K'",/align_left)
void = widget_label(cb0,value="Value - the value of the tag/variable, eg '200.0'",/align_left)
void = widget_label(cb0,value="Error - the error in the value, eg '0.50'",/align_left)

cb1 = widget_base(tlb,/col,/frame,/base_align_right)
xsize = 50
void = cw_field(cb1,/row,title='Name : ',value=(*descriPtr).name,/string,uname='NAME',/all_events,xsize=xsize)
void = cw_field(cb1,/row,title='Label: ',value=(*descriPtr).legend,/string,uname='LABEL',/all_events,xsize=xsize)
void = cw_field(cb1,/row,title='Units: ',value=(*descriPtr).units,/string,uname='UNITS',/all_events,xsize=xsize)
void = cw_field(cb1,/row,title='Value: ',value=(*descriPtr).qty,/float,uname='VALUE',/all_events,xsize=xsize)
void = cw_field(cb1,/row,title='Error: ',value=(*descriPtr).err,/float,uname='ERROR',/all_events,xsize=xsize)

cb2 = widget_base(tlb,/row,/align_center)
void = widget_button(cb2,value='Accept',uname='ACCEPT')
void = widget_button(cb2,value='Cancel',uname='CANCEL')

widget_control,tlb,/realize
sPtr = ptr_new({cancel:0, $
                name:'', $
                label:'', $
                units:'', $
                value:0.0, $
                error:0.0 $
               })

widget_control,tlb, set_uvalue=sPtr

registerName = 'db_editdescriptr'
xmanager,registerName,tlb,no_block=modal,event_handler='db_editdescriptr_events'

name = strtrim((*sPtr).name, 2)
if ((strlen(name) le 0) or (*sPtr).cancel) then begin
    ptr_free, sPtr
    return, 0
endif

(*descriPtr).name = (*sPtr).name
(*descriPtr).legend = (*sPtr).label
(*descriPtr).units = (*sPtr).units
(*descriPtr).qty = (*sPtr).value
(*descriPtr).err = (*sPtr).error

ptr_free, sPtr

return, 1

end
