; $Id: db_xspectra.pro,v 1.4 2003/10/21 15:30:38 azuah Exp $
;===============================================================================
;
; Richard Tumanjong Azuah
; NCNR, NIST
;
; NAME:_______________________
;   db_xSpectra
;
; PURPOSE:____________________
;   A small utility widget program to obtain information about which
;   spectra to extract from a 2D (DaveDat2D object) dataset.
;
; INPUTS:_____________________
;   PARAMETER
;     objRef: the object reference of the DaveDat2D object from which
;     the spectra are to be extracted.
;   KEYWORDS
;     group_leader: the ID of the parent widget from which this dialog
;     was launched.
;
; OUTPUTS:____________________
;     return value: a structure with information about the spectra to
;     be extracted, the axis to extract from and how the dialog was exited.
;

pro db_xSpectra_events, event

widget_control,event.top,get_uvalue = sPtr
uname = widget_info(event.id,/uname)

case uname of
    'ACCEPT': begin
        (*sPtr).cancel=0
        widget_control,event.top,/destroy
    end

    'CANCEL': begin
        (*sPtr).cancel=1
        widget_control,event.top,/destroy
    end

    'AXIS': begin
        (*sPtr).axis = (*sPtr).axis xor 1
        widget_control,(*sPtr).nslID, set_value=(*sPtr).label[(*sPtr).axis] 
    end

    'SPECTRA': begin
        ;; Only accept numbers [0-9] and delimiters - and , in the
        ;; text field. Also, don't allow delimiters at the beginning.
        if (event.type eq 0) then begin
            res = where((*sPtr).valid_achar eq event.ch, isaValidChar)
            if (isaValidChar) then begin
                res = where((*sPtr).valid_adel eq event.ch, isaDelimiter)
                if ((strlen(strtrim((*sPtr).spectra,2)) eq 0) and isaDelimiter) then begin
                    widget_control, event.id,set_value=(*sPtr).spectra
                    break
                endif
                widget_control, event.id,get_value=value
                (*sPtr).spectra = value
            endif else widget_control, event.id,set_value=(*sPtr).spectra
        endif
    end

    else:

endcase

;print, 'Event handled...'

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function db_xSpectra, objRef, group_leader=group_leader

returnStr = {cancel:1,axis:0,spectra:-1}

; An objRef parameter must be passed
if (n_params() ne 1) then return, returnStr

; objRef must be a valid object
if (obj_valid(objRef) ne 1) then return, returnStr

; ObjRef must be a DaveDat2D object
if (obj_class(objRef) ne 'DAVEDAT2D') then return, returnStr

if (not objRef->get(xVal=xval,yVal=yval,nameTag=nameTag)) then return, returnStr

nx = n_elements(xval)
ny = n_elements(yval)

if ((nx le 0) and (ny le 0)) then return, returnStr

xlabel = (nx gt 1)? 'There are '+strtrim(string(ny),2)+' spectra present' : $
                    'There is '+strtrim(string(ny),2)+' spectrum present'
if (nx eq 0) then xlabel = 'There are no spectra present'
ylabel = (ny gt 1)? 'There are '+strtrim(string(nx),2)+' spectra present' : $
                    'There is '+strtrim(string(nx),2)+' spectrum present'
if (ny eq 0) then ylabel = 'There are no spectra present'



; Widget definition module
title = 'DATABROWSER: Extract Spectra - '+nameTag 
if (n_elements(group_leader) eq 0) then begin
    group_leader = 0L
    modal = 0
    tlb = widget_base(/col,title=title,/align_center)
endif else begin
    modal = 1
    tlb = widget_base(group_leader=group_leader,/col,title=title,modal=modal,/align_center)
endelse

cb1 = widget_base(tlb,/col,/frame,/align_center)
extAxis = ['x','y']
void = cw_bgroup(cb1,extAxis,/row,/exclusive, $
	label_left = 'Desired independent axis for extracted spectra:',/no_release, $
	set_value=0,/return_index,uname='AXIS')
nspecLabelID = widget_label(cb1,value=xlabel)

cb2 = widget_base(tlb,/col,/frame,/align_center)
label1 = 'Please, specify spectra (>= 1) to extract below.'
label2 = 'Use comma as delimiter and dash for ranges.'
label3 = 'Thus 2-4,6,8 extracts spectra 2,3,4,6 and 8.'
void = widget_label(cb2,value=label1);,yoffset=10)
void = widget_label(cb2,value=label2);,yoffset=10)
void = widget_label(cb2,value=label3);,yoffset=10)

void = widget_text(cb2,/editable,/all_events,value='',uname='SPECTRA',xsize=60)

cb3 = widget_base(tlb,/row,/align_center)
void = widget_button(cb3,value='Accept',uname='ACCEPT')
void = widget_button(cb3,value='Cancel',uname='CANCEL')

widget_control,tlb,/realize
valid_ascii_chars = [44,45,48,49,50,51,52,53,54,55,56,57]
valid_ascii_delimiters = [44,45] 
sPtr = ptr_new({cancel:0, $
                axis:0, $
                spectra:'', $
                valid_achar:valid_ascii_chars, $
                valid_adel:valid_ascii_delimiters, $
                label:[xlabel,ylabel], $
                nslID:nspecLabelID $
               })

widget_control,tlb, set_uvalue=sPtr

registerName = 'db_xspectra'
xmanager,registerName,tlb,no_block=modal,event_handler='db_xspectra_events'

strspec = strtrim((*sPtr).spectra, 2)
if ((strlen(strspec) le 0) or (*sPtr).cancel) then begin
    cancel = (*sPtr).cancel
    axis = (*sPtr).axis
    ptr_free, sPtr
    return, {cancel:cancel,axis:axis,spectra:0}
endif

tokens = strsplit(strspec,',',/extract)
ntok = (size(tokens))[1]
intspec = -1
index = 0

for j = 0,ntok-1 do begin
    tok1 = strsplit(tokens[j],'-',/extract)
    if ((size(tok1))[1] eq 1) then begin
        i = fix(tok1[0])
        res = where(intspec eq i, isNotaNewValue)
        if (isNotaNewValue) then continue
        intspec = (index eq 0)? i : [intspec,i]
        index = index + 1
    endif else begin
        for i = fix(tok1[0]),fix(tok1[1]) do begin
            res = where(intspec eq i, isNotaNewValue)
            if (isNotaNewValue) then continue
            intspec = (index eq 0)? i : [intspec,i]
            index = index + 1
        endfor
    endelse       
endfor

returnStr = {cancel:(*sPtr).cancel,axis:(*sPtr).axis,spectra:intspec[sort(intspec)]}

ptr_free, sPtr

return, returnStr

end
