; $Id: ref_broad_three_fold_tunneling.pro,v 1.2 2003/05/20 20:47:14 dimeo Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function ref_broad_three_fold_tunneling,xvals,parms,			$
                    	parmnames = parmnames,					$
                    	init_guess = init_guess,				$
                    	qty = qty,								$
                    	single_parmnames = single_parmnames,	$
                    	multi_parmnames = multi_parmnames,		$
                    	res_ptr = res_ptr,						$
                    	step = step,							$
                    	all_y = all_y,							$
                    	_Extra = extra
; all_y keyword is relevant for this model
if n_elements(all_y) eq 0 then all_y = 1
nx = extra.nx & ny = extra.ny
x = xvals[0:nx-1] & y = xvals[nx:nx+ny-1]
ux = 1+bytarr(nx) & uy = 1+bytarr(ny)
parmnames = ['Radius','Tunnel energy','Gamma_t','Gamma_q', $
			'Intensity:'+strtrim(string(indgen(ny)),2), $
			'Center:'+strtrim(string(indgen(ny)),2)]
single_parmnames = parmnames[0:3]
multi_parmnames = ['Intensity','Center']
if (n_params() eq 0) or (n_params() eq 1) then begin
	if arg_present(init_guess) then begin
		intensity = max(qty,dimension = 1,index)
		qty_size = size(qty)
		for i = 0,qty_size[2]-1 do begin
			qty_max = max(qty[*,i],index)
			if i eq 0 then index_array = index else $
				index_array = [index_array,index]
		endfor
		max_index = (moment(index_array))[0]
		npeaks = 3	; number of peaks (assumed known)
		for i = 0,qty_size[2]-1 do begin
			xpeaks = get_peak_pos(x,qty[*,i],npeaks)
			if i eq 0 then peak_array = max(xpeaks) else $
				peak_array = [peak_array,max(xpeaks)]
		endfor

		radius = 1.0 & et = (moment(peak_array))[0]
		center = x[index_array]
		; Guess the 1/2 width at the highest y-value
		; Get the low value first
		amp = max(qty[*,qty_size[2]-1])
		ht = 0.5*amp
		wh_lo = where(abs(amp-ht) le min(abs(amp-ht)) and (x lt 0.0),count_lo)
		x_lo_lim = x[wh_lo[0]]
 		wh_hi = where(abs(amp-ht) le min(abs(amp-ht)) and (x gt 0.0),count_hi)
		x_hi_lim = x[wh_hi[0]]
		fwhm = x_hi_lim - x_lo_lim
		init_guess = [radius,et,fwhm,fwhm,intensity,center]
	endif
	if arg_present(step) then begin
		dx = x[1]-x[0]
		step = dblarr(4+2*ny)
	endif
	return,-1
endif

; parms = [radius,et,fwhm,fwhm,intensity,center]

radius = (abs(parms[0]))#uy
et = (parms[1])
gamma_t = parms[2] & gamma_q = parms[3]

intensity = ux#(parms[4:4+ny-1])
center = ux#(parms[4+ny:4+2*ny-1])

eisf = ux#((1.0+2.0*sph_bessel(y*radius*sqrt(3.0),0))/3.0)
xhi = max(x,min = xlo)
dx = x[1]-x[0] & dy = y[1]-y[0]
nshift = fix(0.5*((abs(xhi)-abs(xlo)))/dx)
ie_terms =	rains_lorentzian(x,area = 1.0,center = center,fwhm = 2.0*gamma_q) + $
			rains_lorentzian(x,area = 1.0,center = center+et,fwhm = 2.0*gamma_t) + $
			rains_lorentzian(x,area = 1.0,center = center-et,fwhm = 2.0*gamma_t)

ie_shape = scat_convolve(x,y,ie_terms,*res_ptr)

lineshape =	eisf*ref_delta(xvals,center,*res_ptr,nx = nx,ny = ny) + $
			((1.0-eisf)/3.0)*ie_shape

yout = intensity*lineshape
return,yout
end