; $Id: ref_random_jump.pro,v 1.1 2004/05/07 12:05:24 dimeo Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function ref_random_jump,xvals,parms,                       $
                     parmnames = parmnames,                 $
                     init_guess = init_guess,               $
                     qty = qty,                             $
                     single_parmnames = single_parmnames,   $
                     multi_parmnames = multi_parmnames,     $
                     res_ptr = res_ptr,                     $
                     all_y = all_y,                         $
                     lorentzian = lorentzian,               $
                     step = step,                           $
                     eisf = eisf,                           $
                     xunits = xunits,                       $
                     _Extra = extra
; all_y keyword is not relevant for this model
if n_elements(all_y) eq 0 then all_y = 1
nx = extra.nx & ny = extra.ny
x = xvals[0:nx-1] & y = xvals[nx:nx+ny-1]
ux = 1+bytarr(nx) & uy = 1+bytarr(ny)
parmnames = [  'RMS vibrational displacement (A)',          $
               'bond length (A)',                           $
               'translational diffusivity (A^2/ps)',        $
               'site residence time (ps)',                  $
               'rotational correlation time (ps)',          $
               'Intensity:'+strtrim(string(indgen(ny)),2),  $
               'Center:'+strtrim(string(indgen(ny)),2)      ]
nsingle = 5
nmulti = 2
single_parmnames = parmnames[0:nsingle-1]
multi_parmnames = ['Intensity','Center']

if strupcase(xunits) eq 'MEV' then $
   hbar = 0.6594 else $
   hbar = 659.4

if arg_present(step) then begin
   step = dblarr(nsingle+2*extra.ny)
endif
if (n_params() eq 0) or (n_params() eq 1) then begin
   if arg_present(init_guess) then begin
      intensity = max(qty,dimension = 1,index)
      qty_size = size(qty)
      for i = 0,qty_size[2]-1 do begin
         qty_max = max(qty[*,i],index)
         if i eq 0 then index_array = index else $
            index_array = [index_array,index]
      endfor
      ; Guess the 1/2 width at the highest y-value
      ; Get the low value first
      amp = max(qty[*,qty_size[2]-1])
      center = x[index_array]
      ht = 0.5*amp
      wh_lo = where(abs(amp-ht) le min(abs(amp-ht)) and (x lt 0.0),count_lo)
      x_lo_lim = x[wh_lo[0]]
      wh_hi = where(abs(amp-ht) le min(abs(amp-ht)) and (x gt 0.0),count_hi)
      x_hi_lim = x[wh_hi[0]]
      fwhm = x_hi_lim - x_lo_lim
      d = fwhm/(2.0*hbar*(y[ny-1]^2))
      tauo = 0.25 ; according to Sankar Nair, this is the approximate
                  ; residence time for bulk water at room temperature.
      u = 0.0
      tau = 0.1
      b = 1.0     ; initial guess for bond length
      init_guess = [u,b,d,tauo,tau,intensity,center]
      center = x[index_array]
      init_guess = [u,b,d,tauo,tau,intensity,center]
   endif
   return,-1
endif

u = parms[0]
b = parms[1]
d = parms[2]
tauo = parms[3]
tau = parms[4]
intensity = ux#(parms[nsingle:nsingle+ny-1])
center = ux#(parms[nsingle+ny:nsingle+2*ny-1])

jo = ux#(sph_bessel(y*b,0))
j1 = ux#(sph_bessel(y*b,1))
j2 = ux#(sph_bessel(y*b,2))
gamma = d*y^2/(1.0+d*tauo*y^2)
gamma = ux#gamma
dwf_term = ux#(exp(-(y*u)^2/3.0))


x = x#uy
x = x-center

term1 = (jo^2)*hbar*gamma/((hbar*gamma)^2+x^2)
term2 = 3.0*(j1^2)*hbar*(gamma+1.0/(3.0*tau))/((hbar*(gamma+1.0/(3.0*tau)))^2+x^2)
term3 = 5.0*(j2^2)*hbar*(gamma+1.0/(1.0*tau))/((hbar*(gamma+1.0/(1.0*tau)))^2+x^2)
sqe = (dwf_term/!dpi)*(term1+term2+term3)
resolution = *res_ptr
lineshape = scat_convolve(x,y,sqe,resolution)
yout = (intensity)*lineshape
return,yout
end