;$Id: create_gauss_res_fun.pro,v 1.3 2003/05/16 12:14:20 dimeo Exp $
;+
; NAME:
;       CREATE_GAUSS_RES_FUN
;
; PURPOSE:
;
;		This function creates a Gaussian resolution function based
;		on the input data (a required parameter of an object of the
;		class MODEL_DATA.
;
; AUTHOR:
;
;       Robert M. Dimeo, Ph.D.
;		NIST Center for Neutron Research
;       100 Bureau Drive
;		Gaithersburg, MD 20899
;       Phone: (301) 975-8135
;       E-mail: robert.dimeo@nist.gov
;       http://www.ncnr.nist.gov/staff/dimeo
;
; CATEGORY:
;
;       Utility, DAVE_REFINE application, DAVE project
;
; CALLING SEQUENCE:
;
;
; INPUT PARAMETERS (REQUIRED)
;
;		ODATA:		object of class MODEL_DATA
;
; COMMON BLOCKS:
;
;		None
;
; DEPENDENCIES:
;
;		TVIMAGE
;
; REQUIREMENTS:
;
;	Written using IDL 5.6.  I cannot ensure that it will work with previous
;	releases.
;
; DISCLAIMER
;
;		This software is provided as is without any warranty whatsoever.
;		Permission to use, copy, modify, and distribute modified or
;		unmodified copies is granted, provided this disclaimer
;		is included unchanged.
;
; MODIFICATION HISTORY:
;
;       Written by Rob Dimeo, April 14, 2003
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function cgrf_gaussian,x,y,area,center,fwhm
sig = (fwhm/2.354)
ny = n_elements(y) & uy = 1+bytarr(ny)
gauss = (area/sqrt(2.0*!dpi*sig^2))*exp(-0.5*((x#uy-center)/sig)^2)
return,gauss
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_display_plot,event
widget_control,event.top,get_uvalue = pstate
slider_id = widget_info(event.top,find_by_uname = 'GROUP_SLIDER')
widget_control,slider_id,get_value = val & val = val[0]
plot,*(*pstate).xptr,(*(*(*pstate).info_ptr).resptr)[*,val],psym = 0
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_refresh_plot,event
widget_control,event.top,get_uvalue = pstate
wset,(*pstate).plot_pix
cgrf_display_plot,event
wset,(*pstate).plot_vis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).plot_pix]
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_display_image,event
widget_control,event.top,get_uvalue = pstate
pos = [0.15,0.15,0.9,0.9]
xvalues = *(*pstate).xptr & yvalues = *(*pstate).yptr
erase
tvimage,bytscl(*(*(*pstate).info_ptr).resptr),position = pos
xrange = [min(xvalues),max(xvalues)]
yrange = [min(yvalues),max(yvalues)]
plot,xvalues,yvalues,xrange = xrange,yrange = yrange,/nodata,/noerase, $
	position = pos
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_refresh_image,event
widget_control,event.top,get_uvalue = pstate
wset,(*pstate).im_pix
cgrf_display_image,event
wset,(*pstate).im_vis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pstate).im_pix]
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_calc_resolution,event
widget_control,event.top,get_uvalue = pstate
x = *(*pstate).xptr
y = *(*pstate).yptr
fwhm_id = widget_info(event.top,find_by_uname = 'FWHM')
center_id = widget_info(event.top,find_by_uname = 'CENTER')
widget_control,fwhm_id,get_value = fwhm_string & fwhm_string = fwhm_string[0]
widget_control,center_id,get_value = center_string
center_string = center_string[0]
cen_expr = 'ycen='+center_string
wid_expr = 'ywid='+fwhm_string
rcen = execute(cen_expr,1)
if rcen ne 1 then begin
	strout = 'Error evaluating expression for center'
	void = dialog_message(strout,dialog_parent = event.top)
	return
endif
rwid = execute(wid_expr,1)
if rwid ne 1 then begin
	strout = 'Error evaluating expression for width'
	void = dialog_message(strout,dialog_parent = event.top)
	return
endif

nx = n_elements(x) & ny = n_elements(y)
ux = 1+bytarr(nx) & uy = 1+bytarr(ny)
scen = size(ycen)
if scen[0] eq 0 then begin
	ycen = ux#ycen#uy
endif
if scen[0] eq 1 then begin
	ycen = ux#ycen
endif
swid = size(ywid)
if swid[0] eq 0 then begin
	ywid = ux#ywid#uy
endif
if swid[0] eq 1 then begin
	ywid = ux#ywid
endif
res = cgrf_gaussian(x,y,ux#uy,ycen,ywid)
*(*(*pstate).info_ptr).resptr = res
cgrf_refresh_plot,event
cgrf_refresh_image,event
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cgrf_event,event
widget_control,event.top,get_uvalue = pstate
uname = widget_info(event.id,/uname)
case uname of
'FWHM':	$
	begin
		cgrf_calc_resolution,event
	end
'CENTER':	$
	begin
		cgrf_calc_resolution,event
	end
'ACCEPT':	$
	begin
		(*(*pstate).info_ptr).cancel = 0
		widget_control,event.top,/destroy
	end
'CANCEL':	$
	begin
		(*(*pstate).info_ptr).cancel = 1
		widget_control,event.top,/destroy
	end
'GROUP_SLIDER':	$
	begin
		cgrf_refresh_plot,event
	end
else:
endcase
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function create_gauss_res_fun,odata,	group_leader = group_leader,	$
										cancel = cancel
; Widget definition module
tvlct,r,g,b,/get
device,decomposed = 0
loadct,15,/silent
register_name = 'cgrf'
if n_params() eq 0 then return,0
if not obj_valid(odata) then return,0
ret = odata->get(nx = nx,ny = ny,xvals = x,yvals = y)
resolution = fltarr(nx,ny)
if n_elements(group_leader) eq 0 then begin
	group_leader = 0L
	modal = 0
endif else begin
	modal = 1
endelse
tlb = widget_base(/col,title = 'Gaussian Instrumental Resolution Generator',	$
	group_leader = group_leader,modal = modal,/tlb_frame_attr)
row = widget_base(tlb,/row)
col1 = widget_base(row,/col,/base_align_right)
void = cw_field(col1,title = 'FWHM(Y),e.g. 0.7+0.0*Y', $
	value = '0.7+0.0*Y',/return_events,uname = 'FWHM')
void = cw_field(col1,title = 'Center(Y),e.g. 0.0*Y', $
	value = '0.0*Y',/return_events,uname = 'CENTER')
col2 = widget_base(row,/col)
void = widget_button(col2,value = 'ACCEPT',uname = 'ACCEPT')
void = widget_button(col2,value = 'CANCEL',uname = 'CANCEL')
void = widget_slider(col2,title = 'Group',value = 0,min = 0,max = ny-1, $
	uname = 'GROUP_SLIDER')
plot_row = widget_base(tlb,/row)
xsize = 400 & ysize = 400
im_win = widget_draw(plot_row,xsize = xsize,ysize = ysize)
plot_win = widget_draw(plot_row,xsize = xsize,ysize = ysize)

widget_control,tlb,/realize
widget_control,im_win,get_value = im_vis
widget_control,plot_win,get_value = plot_vis
window,/free,/pixmap,xsize = xsize,ysize = ysize
im_pix = !d.window
window,/free,/pixmap,xsize = xsize,ysize = ysize
plot_pix = !d.window
info = {cancel:1,resptr:ptr_new(resolution)}
info_ptr = ptr_new(info)
state =	{	info_ptr:info_ptr,			$
			im_vis:im_vis,				$
			plot_vis:plot_vis,			$
			im_pix:im_pix,				$
			plot_pix:plot_pix,			$
			xptr:ptr_new(x),			$
			rptr:ptr_new(r),			$
			gptr:ptr_new(g),			$
			bptr:ptr_new(b),			$
			yptr:ptr_new(y)				}

pState = ptr_new(state)
widget_control,tlb,set_uvalue = pstate
pseudo_event = {top:tlb,id:void,handler:0L}
cgrf_calc_resolution,pseudo_event
xmanager,register_name,tlb,event_handler = 'cgrf_event'

; Get everything out and clean it up
tvlct,*(*pstate).rptr,*(*pstate).gptr,*(*pstate).bptr
info = *(*pstate).info_ptr
ptr_free,(*pstate).info_ptr
cancel = info.cancel
ptr_free,(*pstate).xptr,(*pstate).yptr
ptr_free,(*pstate).rptr,(*pstate).gptr,(*pstate).bptr
wdelete,(*pstate).im_pix
wdelete,(*pstate).plot_pix
ptr_free,pstate

return,info
end