; Written by J.R.D. Copley.
;************************************************************************************************
pro xsecsdb,xsecs,xslabels,database=database
;************************************************************************************************
;
; This routine generates and/or accesses the cross section database.
; If database ge 0 the database is generated.
; If database gt 0 it is saved in an IDL save file.
; If database lt 0 it is restored.
; Default is database=0.
;
compile_opt strictarr
;
if (n_elements(database) eq 0) then database=0
do_save=database gt 0
do_restore=database lt 0
;
if (do_restore) then begin
	if (lmgr(/vm)) then begin
		filename=sourceroot()+"xsecsdb.dat"
	endif else begin
		filename=!DAVE_AUXILIARY_DIR+'xsecsdb.dat'
	endelse
  restore,filename=filename
	return
endif
;
t0=systime(1)
readCrossSecs,xsecs,xslabels,getfromWeb=0,flagspecial=0
;print,string(systime(1)-t0,format='(f6.1)')+' s.'
;
t0=systime(1)
readAtWeights,atwts,atlabels,getfromWeb=0
;print,string(systime(1)-t0,format='(f6.1)')+' s.'
;
t0=systime(1)
n=n_elements(xsecs.(0))
for k=0,n-1 do begin
	atsym=xsecs.atsym[k]
	j=where(atsym eq atwts.isosym,count)
	if (count ne 1) then begin
		res=dialog_message('We have a problem in xsecsdb.pro')
	endif
	if (count eq 1) then begin
		xsecs.atwt[k]=atwts.isomass[j]
		xsecs.atno[k]=atwts.atno[j]
	endif
endfor
print,string(systime(1)-t0,format='(f6.1)')+' s.'
;
if (do_save)then begin
	save,filename='xsecsdb.dat',xsecs,xslabels
	print,'Cross sections database saved.'
endif
;
end
