;$Id: read_dcs.pro,v 1.2 2003/02/08 15:34:31 jmspark Exp $
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
; CONVERSION
;
;   s = rebin_dcs( s_in, dq = dq, dhw = dhw )
;
; READING
;
;	s = _read_dcs_one( file )
;	s = _read_dcs_files( files )
;	s = read_dcs( files )
;
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
pro histo2points_dcs, str

	q = (str.q[*,0:999] + str.q[*,1:1000])/2d
	hw = (str.hw[*,0:999] + str.hw[*,1:1000])/2d

	str.q = q
	str.hw = hw

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
; Plotting I vs hw curve from DCS data.
;
; 	raw = read_dcs( files, /positive )
;	ihw = ihw_dcs( raw, dhw = 0.02 )
;	plot_ihw_dcs, ihw, xrange = [ -0.2, 0.2 ]
;
pro plot_ihw_dcs, str, color = color, _extra = _extra

	plot, str.hw, str.f, color = color, _extra = _extra
	errplot, str.hw, str.f - str.e, str.f + str.e, color = color

end

pro plot_iq_dcs, str, color = color, _extra = _extra

	plot, str.q, str.f, color = color, _extra = _extra
	errplot, str.q, str.f - str.e, str.f + str.e, color = color

end

pro oplot_iq_dcs, str, color = color, _extra = _extra

	oplot, str.q, str.f, color = color, _extra = _extra
	errplot, str.q, str.f - str.e, str.f + str.e, color = color

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
pro pf2d_dcs, str, _extra = _extra

	pf2d, str.f, str.q, str.hw, _extra = _extra

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function rebin_dcs, str, dq = dq, dhw = dhw
	; The 'str' must contain the raw data instead of the normalized data.

	if n_elements( dq ) ne 1 then dq = 0.05
	if n_elements( dhw ) ne 1 then dhw = 0.05


	histo2points_dcs, str
	; q = (str.q[*,0:999] + str.q[*,1:1000])/2d
	; hw = (str.hw[*,0:999] + str.hw[*,1:1000])/2d

	f = str.f
	q = str.q
	hw = str.hw

	q_min = min( q, max = q_max )
	hw_min = min( hw, max = hw_max )

	n_q = fix( q_max / dq + 0.5 ) - fix( q_min / dq + 0.5 ) + 1
	q_v = ( dindgen( n_q ) + fix( q_min / dq + 0.5 ) ) * dq
	q_idx = fix( q / dq + 0.5 ) - fix( q_min / dq + 0.5 )

	n_hw = fix( hw_max / dhw + 0.5 ) - fix( hw_min / dhw + 0.5 ) + 1
	hw_v = ( dindgen( n_hw ) + fix( hw_min / dhw + 0.5 ) ) * dhw
	hw_idx = fix( hw / dhw + 0.5 ) - fix( hw_min / dhw + 0.5 )

	newf = dblarr( n_q, n_hw )
	newm = lonarr( n_q, n_hw )
	newq = q_v # ( dblarr( n_hw ) + 1d )
	newhw = ( dblarr( n_q ) + 1d ) # hw_v
	n_f = n_elements( f )
	for i = 0L, n_f - 1 do begin
		newf[ q_idx[i], hw_idx[i] ] = newf[ q_idx[i], hw_idx[i] ] + f[i]
		newm[ q_idx[i], hw_idx[i] ] = newm[ q_idx[i], hw_idx[i] ] + 1L
	endfor
	zero_m = newm eq 0
	newf = newf / ( newm + zero_m )
	newe = sqrt( newf ) / ( newm + zero_m )

	return, $
		{ f:newf $
		, e:newe $
		, q:newq $
		, hw:newhw $

		, monitor:str.monitor $
		, wavelength:str.wavelength $

		}
end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function iq_dcs, str, dq = dq, hw_range = hw_range
	; Start from the raw data.

	if n_elements( dq ) ne 1 then dq = 0.05
	if n_elements( hw_range ) ne 2 then hw_range = [ min( str.hw ), max( str.hw ) ]

	idx = where( str.hw ge hw_range[0] and str.hw le hw_range[1] )

	histo2points_dcs, str

	f = str.f[ idx ]
	q = str.q[ idx ]
	hw = str.hw[ idx ]

	q_min = min( q, max = q_max )

	n_q = fix( q_max / dq + 0.5 ) - fix( q_min / dq + 0.5 ) + 1
	q_v = ( dindgen( n_q ) + fix( q_min / dq + 0.5 ) ) * dq
	q_idx = fix( q / dq + 0.5 ) - fix( q_min / dq + 0.5 )

	newf = dblarr( n_q )
	newm = lonarr( n_q )
	newq = q_v
	newhw = total( hw_range ) / 2d

	n_f = n_elements( f )
	for i = 0L, n_f - 1 do begin
		newf[ q_idx[i] ] = newf[ q_idx[i] ] + f[i]
		newm[ q_idx[i] ] = newm[ q_idx[i] ] + 1L
	endfor

	zero_m = newm eq 0
	newe = sqrt( newf )
	newf = newf / ( newm + zero_m )
	newe = newe / ( newm + zero_m )

	return, $
		{ f:newf $
		, e:newe $
		, q:newq $
		, hw:newhw $

		, monitor:str.monitor $
		, wavelength:str.wavelength $

		}
end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function ihw_dcs, str, dhw = dhw, q_range = q_range
	; Start from the raw data.

	if n_elements( dhw ) ne 1 then dhw = 0.05
	if n_elements( q_range ) ne 2 then q_range = [ min( str.q ), max( str.q ) ]

	idx = where( str.q ge q_range[0] and str.q le q_range[1] )

	histo2points_dcs, str

	f = str.f[ idx ]
	q = str.q[ idx ]
	hw = str.hw[ idx ]

	hw_min = min( hw, max = hw_max )

	n_hw = fix( hw_max / dhw + 0.5 ) - fix( hw_min / dhw + 0.5 ) + 1
	hw_v = ( dindgen( n_hw ) + fix( hw_min / dhw + 0.5 ) ) * dhw
	hw_idx = fix( hw / dhw + 0.5 ) - fix( hw_min / dhw + 0.5 )

	newf = dblarr( n_hw )
	newm = lonarr( n_hw )
	newhw = hw_v
	newq = total( q_range ) / 2d

	n_f = n_elements( f )
	for i = 0L, n_f - 1 do begin
		newf[ hw_idx[i] ] = newf[ hw_idx[i] ] + f[i]
		newm[ hw_idx[i] ] = newm[ hw_idx[i] ] + 1L
	endfor

	zero_m = newm eq 0
	newe = sqrt( newf )
	newf = newf / ( newm + zero_m )
	newe = newe / ( newm + zero_m )

	return, $
		{ f:newf $
		, e:newe $
		, q:newq $
		, hw:newhw $

		, monitor:str.monitor $
		, wavelength:str.wavelength $

		}
end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function kikf_dcs, str
	; 'str' must contain the same sized f, e, q, and hw arrays as well as a scalar wavelength.


	f = str.f
	e = str.e
	hw = str.hw

	Ki = 2d * !dpi / str.wavelength
	Kf2 = ( 2.0717 * Ki^2 - hw ) / 2.0717
	Kf = sqrt( Kf2 )
	KiKf = Ki / Kf

	f = f * kikf
	e = e * kikf

	return, $
		{ f:f $
		, e:e $
		, q:str.q $
		, hw:str.hw $

		, monitor:str.monitor $
		, wavelength:str.wavelength $
		}

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
pro normal_dcs, str

	str.f = str.f / str.monitor
	str.e = str.e / str.monitor

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function sub_dcs, str1, str2
	; Use this function to subtract background.
	; The data are normalized here.

	f = str1.f / str1.monitor - str2.f / str2.monitor
	e = sqrt( ( str1.e / str1.monitor )^2 + ( str2.e / str2.monitor)^2 )

	return, $
		{ f:f $
		, e:e $
		, q:str1.q $
		, hw:str1.hw $

		, monitor:str1.monitor $
		, wavelength:str1.wavelength $
		}

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function _read_dcs_one, file, positive = positive

	dave_makeDavePtr, davePtr = davePtr
	dcs_defineDaveStructure,davePtr=davePtr

	; Read the DCS file.
	nofirstline = 0
	omithisto = 0
	dcs_read_binaryOctavefile $
		, file, nofirstline, omithisto $		; INPUT
		, variables_read, unknowns,error $		; OUTPUT
		, davePtr=davePtr

	; Specific Pointer.
	sPtr = (*(*davePtr).dataStrPtr).specificPtr

	; Data.
	; Note that *(sPtr).q_propsPtr only contains 1D data.
	; Thus it is not appropriate to use the provided Q values.
	f = (*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty
	angle = *(*sPtr).angle_propsPtr
	hw = *(*sPtr).energy_propsPtr
	DT = reform( (*sPtr).tchanlook[0,*] )

	; Find the temperature.
	Tidx = where( (*sPtr).temp_sample gt 0, nT )
	if nT ge 1 then temperature = total( ((*sPtr).temp_sample)[Tidx] ) / nT $
	else temperature = 0d

	; Monitor Counts, Duration, and wavelength
	monitor = long( (*sPtr).detsum[916] )
	duration = (*sPtr).duration
	wavelength = (*sPtr).ch_wl

	heap_free, davePtr

	return, $
		{ f:f $
		, angle:angle $
		, hw:hw $
		, DT:DT $

		, monitor:monitor $
		, duration:duration $
		, temperature:temperature $
		, wavelength:wavelength $
		}

end

function _read_dcs_files, files, positive = positive

	; Define system variables.
	defsysv, '!dave_defaults', exists=exists
	if not exists then begin
		cd, current=directory
		pdd=ptr_new({datDir:directory,workDir:directory,PDFReader:''})
		defsysv,'!dave_defaults',pdd
	endif
	davesysvars
	dcs_sysvars

	n_files = n_elements( files )
	f = 0d
	monitor = 0l
	duration = 0d
	sum_temperature = 0d
	for i = 0, n_files - 1 do begin

		s = _read_dcs_one( files[i] )
		f = f + s.f

		monitor = monitor + s.monitor
		duration = duration + s.duration
		sum_temperature = sum_temperature + s.temperature * s.monitor

	endfor
	temperature = sum_temperature / monitor

	; Sort and Select the useful part.
	sortkey = sort( s.angle )
	angle = s.angle[ sortkey ]
	f = transpose( f[0:999, sortkey ] )
	hw = s.hw[0:1000]
	DT = s.DT[0:999]

	if keyword_set( positive ) then begin

		idx = where( angle ge 0. )
		angle = angle[ idx ]
		f = f[ idx, * ]

	endif

	; Change to 2D matrix.
	yv = replicate( 1d, n_elements( hw ) )
	xv = replicate( 1d, n_elements( angle ) )
	angle = angle # yv
	hw = xv # hw
	DT = xv # DT

	; Angle to Q conversion.
	Ki = 2d * !dpi / s.wavelength
	Kf2 = ( 2.0717 * Ki^2 - hw ) / 2.0717
	Kf = sqrt( Kf2 )
	q = sqrt( Ki^2 + Kf2 - 2d * Ki * Kf * cos( !dtor * angle ) ) * $
		angle / abs( angle )

	return, $
		{ f:f $
		, angle:angle $
		, q:q $
		, hw:hw $
		, DT:DT $

		, monitor:double( monitor ) $
		, duration:duration $
		, temperature:temperature $
		, wavelength:s.wavelength $
		}

end

function read_dcs, files, positive = positive

	if n_elements( files ) eq 0 then files = dave_dialog_pickfile(/multiple_files, /read)
	if files[0] eq '' then return, 0
	s = _read_dcs_files( files, positive = positive )

	return, s

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************