; $Id: check_heap.pro,v 1.2 2003/07/25 18:43:22 tanya Exp $ 
;+
; NAME:
;      check_heap
;
; PURPOSE:
;
;	This is a generic routine help find memory leaks. 
; It reports which objects, pointers and file unit numbers are still on the 
; heap, after all objects supposedly have been destroyed. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Debugging of objects, pointers and file I/O.
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED:
;	Objects on the heap are assumed to have the following methods:
;	ncount = (lostall[i])->count()
;      	success = ((lostall[i])->get(everything=thisEverything))
;		The get method returns thisEverything as a summary string.
;
; EXAMPLE
;	...
;
; SEE ALSO:
;	<LI><A HREF="#AAA_README">AAA_README</A>
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;================================================================================
;*
pro check_heap, $
	skip_help=thisSkip_help
;/
; For listing all or one of the heap objects, intended to find memory leaks.

  infolabel =  'check_heap >> '

;  if (n_elements(strArray) EQ 0) then begin
    help, /routines, output = helpStr
;  endif
  sLen = n_elements(helpStr)
;Debug  print, ' help, /routines, output = helpStr'
;Debug  for i = 0, (sLen -1) do begin
;Debug    print, helpStr[i]
;Debug  endfor  
;Debug  print

;  print, infolabel, 'View the leftovers after obj_destroy'
  print
  print, infolabel, 'Verify file units numbers were handled properly. ',$
  		 '"help, /files"'
  help, /files
  print
;  print, infolabel, 'Inspect contents of heap. "help, /heap"'
;  help, /heap
;  print
   ; this creates an array with all the heap variables in it. Bad documentation.
  for heap_type = 0, 1 do begin  
;  help, lostall
   if (heap_type eq 0) then begin
     lostall = obj_valid()
     temp = size(lostall, /dim)
     len_lost = temp[0]
     out_string = "Objects: number lost = " + $
     		strcompress(string(len_lost),/remove_all)
   endif
   if (heap_type eq 1) then begin
     lostall = ptr_valid()
     temp = size(lostall, /dim)
     len_lost = temp[0]
     out_string = "Pointers: number lost = "+ $
     		strcompress(string(len_lost),/remove_all)
   endif

   if (len_lost gt 0) then print, infolabel, out_string
   for  i = 0,  (len_lost-1) do begin
    help, lostall[i]
    if (obj_valid(lostall[i])) then begin
         nmatch = 0
	 if (sLen gt 0) then begin
	    ncount = -1
            ncount = (lostall[i])->count()
	    if  (ncount eq -1) then begin
	      print, '  ', lostall[i], ' has no COUNT method. '
	    endif else begin
    	      print, '  ::COUNT() =[', strcompress(string(ncount), /remove_all), '] ' 
	    endelse

	    classStr =  string('*', obj_Class(lostall[i]), '::GET *')
	    matchArr = strmatch(helpStr, classStr, /fold_case)
;	    print, classStr, 'match=',  matchArr
	    for is = 0, (sLen-1) do begin
	        if (matchArr[is]) then begin
		  nmatch = nmatch + 1
		  thisEverything = ''
      		  success = ((lostall[i])->get(everything=thisEverything))
;     		  if (success) then begin
    		    print, '  ', thisEverything    
;    		    print, success, ' ', lostall[i], thisEverything    
;      		  endif
		endif		
	    endfor
	    if  (nmatch eq 0) then begin
	      print, '  ', lostall[i], ' has no GET method. "', classStr, '"'
	    endif
	 endif     
      
;      success = ((lostall[i])->get(everything=thisEverything))
;      if (success) then begin
;        print, success, ' ', lostall[i], thisEverything    
;      endif
    endif

    if (not keyword_set(thisSkip_help)) then begin
      if (ptr_valid(lostall[i])) then begin
        help, *(lostall[i])
	if (size(*(lostall[i]), /tname) NE 'UNDEFINED') then begin
          print, '  ', *(lostall[i])
	endif
      endif
      print
    endif else begin

      if (ptr_valid(lostall[i])) then begin
        help, *(lostall[i])
;	tempArray = *(lostall[i])
;	ntemp = n_elements(tempArray)
;	if (ntemp GT 6) then begin
;	  print, "Too large to print"
;;	  print, "Truncated [0:5] = ", tempArray[0:5]
;	endif else begin
;	  print, tempArray
;	endelse
      endif
      print
    endelse

   endfor
   print
  endfor 

end

