; $Id: cw_ow_binaryop.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_binaryOp
;
; PURPOSE:
;	This is a compound object widget which allows a binary operator to 
; act on two fields from an object of class specObj. User input values are
; allowed in lieu of the fields. The results are stored in the object.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_BINARYOP__DEFINE_arguments">CW_OW_BINARYOP__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
; <LI><A HREF="#CW_OW_RESULT__DEFINE_arguments">CW_OW_RESULT__DEFINE_arguments</A>
; <LI><A HREF="#CW_OW_OPER__DEFINE_arguments">CW_OW_OPER__DEFINE_arguments</A>
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_binaryOp::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_binaryOp::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_binaryOp::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_binaryOp::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_binaryOp::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_binaryOp::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_binaryOp::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
pro cw_ow_binaryOp::FreshenWhere, $ 
	inHeader = thisInHeader, $
	inData = thisInData

  self.where1WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  self.where2WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  
end
;==================================================================
;*
pro cw_ow_binaryOp_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj 

  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_binaryOp_event, event

;==================================================================
pro cw_ow_binaryOp::eventQuit, event=event

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_binaryOp::eventFreshen, event=event

  self -> freshenResults
  
end

;==================================================================
;*
pro cw_ow_binaryOp::eventProcess, event=event
;/

   (self.resultWObj) -> getProperty, $
   	label = resLabel, $ 
	name = resName, $
	units =resUnits

   (self.operWObj) -> getProperty, $
   	oper = theOper

   (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $	; Goes to the droplist
	value = value1	; Goes to the text entry box

    (self.where2WObj) -> getProperty, $
	from = From2, $	; which is sensitized 'input', 'header' or 'data'
	label = label2, $	; Goes to the droplist
	value = value2	; Goes to the text entry box

  if (from1 eq 'input') then begin
     tags1 = value1
   endif else begin
     tags1 = ['*', label1, '*'] 
   endelse
   
   if (from2 eq 'input') then begin
     tags2 = value2
   endif else begin
     tags2 = ['*', label2, '*'] 
   endelse
   
   OK = (self.SpecObj) -> binaryOp(theOper, $
   	/estimate, $
   	tags1 = tags1, $
	from1= from1, $
   	tags2 = tags2, $
	from2= from2, $
	resTags =  ['*', resLabel,  'cw_ow_binaryOp'], $
	name = resName, $
	units = resUnits)

   statusMsg = 'binaryOp successful'		; or an empty string
   if (NOT OK) then begin
      statusMsg = 'binaryOp FAILED'
   endif   
   self.parentWobj -> freshen, status=statusMsg
    
 if (OK) then begin 
  ;-----------------------------------------------------------------------
  treatString = "OK = specObj -> BinaryOp( '" + theOper + "' $"
  treatString = [treatString, ", /estimate  $"]
;  if (n_elements(tags1) GT 0) then begin
    if (size(tags1, /tname) EQ 'STRING') then begin
    treatString = [treatString, $
      string(", tags1 = ['", tags1[0], "', '", tags1[1], "', '", tags1[2], "'] $")]
    endif else begin
    treatString = [treatString, $
      string(", tags1 = ", tags1, " $")]
    endelse
;  endif
;  if (n_elements(from1) GT 0) then begin
    treatString = [treatString, $
    	string(", from1 = '", from1, "' $")]
;  endif
;  if (n_elements(tags2) GT 0) then begin
    if (size(tags2, /tname) EQ 'STRING') then begin
    treatString = [treatString, $
      string(", tags2 = ['", tags2[0], "', '", tags2[1], "', '", tags2[2], "'] $")]
    endif else begin
    treatString = [treatString, $
      string(", tags2 = ", tags2, " $")]
    endelse
;  endif
;  if (n_elements(from2) GT 0) then begin
    treatString = [treatString, $
    	string(", from2 = '", from2, "' $")]
;  endif
  treatString = [treatString, $
    	string(", resTags = ['*', '", resLabel,  "', 'cw_ow_binaryOp'] $")]
  treatString = [treatString, $
    	string(", name = '", resName,  "' $")]
  treatString = [treatString, $
    	string(", units = '", resUnits,  "' $")]
  treatString = [treatString, ")"]

  ;-----------------------------------------------------------------------
  OK = (self.SpecObj) -> appendField($
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
	qty= treatString ) 
  ;-----------------------------------------------------------------------
	  
   ; Might be in either place, so update both header and data areas.
   self.parentWobj -> freshen, /Where, /inHeader, /inData
 endif
  	  
end

;==================================================================
;*
pro cw_ow_binaryOp::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)

end

;=======================================================================
pro cw_ow_binaryOp::cleanup
 
  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  self.parentWObj = obj_new()	; Let old reference dangle.

  obj_destroy, [self.resultWObj, self.operWObj, $
  	self.where1WObj, self.where2WObj]
  
  message, /info, "..."
end


;==================================================================
pro cw_ow_binaryOp_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_binaryOp::cleanup
end

;==================================================================
pro cw_ow_binaryOp::freshenResults

   (self.operWObj) -> getProperty, $
   	oper = theOper

   (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $ ; Goes to the droplist
	value = value1	; Goes to the text entry box
;   print, "from1=", from1, " label1 =", label1, " value1 =", value1

   (self.where2WObj) -> getProperty, $
	from = From2, $	; which is sensitized 'input', 'header' or 'data'
	label = label2, $ ; Goes to the droplist
	value = value2	; Goes to the text entry box
;   print, "from2=", from2, " label2 =", label2, " value2 =", value2

   units1 = ''
   name1 = ''
   if (from1 eq 'input') then begin
     tags1 = value1
     units1 = strtrim( string(value1), 2)
     name1 = units1
     label1 = units1
   endif else begin
     tags1 = ['*', label1, '*'] 
     if (from1 eq 'header') then begin
       OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
    		   /inHeader)
     endif else begin
       OK = (self.specObj) -> getField(tags1, units=units1, name=name1,$
    		   /inData)
     endelse
     if (NOT OK) then message, /info, string($
     	"Failed to get name and units for label1=", label1) 
     	
   endelse

  units2 = ''
  name2 = ''
  if (from2 eq 'input') then begin
     tags2 = value2
     units2 = strtrim( string(value2), 2)
     name2 = units2
     label2 = units2
  endif else begin
     tags2 = ['*', label2, '*'] 
     if (from2 eq 'header') then begin
       OK = (self.specObj) -> getField(tags2, units=units2, name=name2,$
    		   /inHeader)
     endif else begin
       OK = (self.specObj) -> getField(tags2, units=units2, name=name2,$
    		   /inData)
     endelse
     if (NOT OK) then message, /info, string($
     	"Failed to get name and units for label2=", label2) 
     	
  endelse


   resName = string( Name1, ' ', theOpEr, ' ', Name2)
   reslabel = string( Label1, theOper, Label2)

   resUnits = ''
   if ( (from1 NE 'input') OR (from2 NE 'input') ) then begin
;;;   if ( NOT ( (from1 EQ 'input') AND (from2 EQ 'input')) ) then begin
    if (strlen(Units1) GT 0) OR (strlen(Units2) GT 0) then begin
     case (theOper) of 
      '+': begin
         resUnits = Units1 
     	 end
      '-': begin
         resUnits = Units1 
     	 end
      '*': begin
         resUnits = Units1 + ' ' + Units2
     	 end
      else: begin
         resUnits = Units1 + ' ' + theOper + ' ' + Units2
         end
      endcase
    endif
   endif
      
   (self.resultWObj) -> setProperty, $
   	label = resLabel, $ 
	name = resName, $
	units =resUnits   

end

;==================================================================
pro cw_ow_binaryOp::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_binaryOp_TLB", $
	title = 'cw_ow_binaryOp title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_binaryOp_event',$
;	/frame, $
	/row)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_binaryOp_TLB", $
	title = 'cw_ow_binaryOp title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_binaryOp_event',$
;	/frame, $
	/row)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

  buttonClusterID = widget_base(tlbID, /column, /frame)
  voidID = widget_button(buttonClusterID, value='FRESHEN', $
    		uvalue={wobject:self, method:'eventFreshen'})

  voidID = widget_button(buttonClusterID, value='PROCESS', $
    		uvalue={wobject:self, method:'eventProcess'})
  if (self.ParentID EQ 0L) then begin
    void = widget_button(buttonClusterID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
  endif


  ;-------------------------------------------------------------
  doItID = widget_base(tlbID, /row, /frame)

  self.resultWObj = obj_new('cw_ow_result', doItID)
  voidID = Widget_label(doItID, value="=")
 
  ;-------------------------------------------------------------
  self.where1WObj = obj_new('cw_ow_where', self.specObj, doItID,$
  	from = 'data')

  ;-------------------------------------------------------------
  listOp = (self.SpecObj) -> listBinaryOp()
  self.operWObj = obj_new('cw_ow_oper', doItID, operators = listOp, $
  	from ='operator')
  
  ;-------------------------------------------------------------
  self.where2WObj = obj_new('cw_ow_where', self.specObj, doItID,$
  	from = 'data')

  ;-------------------------------------------------------------
  self -> freshenResults

  ;-------------------------------------------------------------
  if (self.ParentID EQ 0L) then begin
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_binaryOp',  tlbID
  endif
   
end		;;  cw_ow_binaryOp::createWidgets


;==================================================================
;*
function cw_ow_binaryOp::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	destroy = thisDestroy, $	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	parentWobj = thisParentWobj ; parent object
;/
	
  debug = 1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)
 
  ;-------------------------------------------------------------
  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  ;-------------------------------------------------------------
  if (obj_valid(thisParentWObj)) then begin
;    help, thisParentWobj
;    thisParentWobj -> print_obj
    self.parentWObj = thisParentWobj
  endif
 
  ;-------------------------------------------------------------
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_binaryOp__define
  define = {cw_ow_binaryOp, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	parentID:0l, $
	parentWObj: obj_new(), $
	tlbID:0l, $
	resultWObj: obj_new(), $
	operWObj: obj_new(), $
	where1WObj: obj_new(), $
	where2WObj: obj_new() $
	}
;/

end
