; $Id: cw_ow_remove.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_remove
;
; PURPOSE:
;	This is a compound object widget which allows a field to be 
; selected from an object of class specObj. This field (and its data)
; is then removed from the object.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_REMOVE__DEFINE_arguments">CW_OW_REMOVE__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;-

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_remove::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_remove::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_remove::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_remove::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_remove::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_remove::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_remove::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_remove_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj 

  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_remove_event, event

;==================================================================
pro cw_ow_remove::eventQuit, event=event

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_remove::eventFreshen, event=event

  self -> freshenResults
  
end

;==================================================================
pro cw_ow_remove::FreshenWhere, $ 
	inHeader = thisInHeader, $
	inData = thisInData

  self.where1WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  
end

;==================================================================
;*
pro cw_ow_remove::eventProcess, event=event
;/

   (self.where1WObj) -> getProperty, $
	from = From1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $	; Goes to the droplist
	value = value1	; Goes to the text entry box

;   if (from1 eq 'input') then begin
;     tags1 = value1
;   endif else begin
     tags1 = ['*', label1, '*'] 
;   endelse
   
   inHeader = 0
   inData = 0
   if (from1 EQ 'header') then begin
     inHeader = 1
   endif
   if (from1 EQ 'data') then begin
     inData = 1
   endif 
   
   theErrMSg = ' '
   (self.SpecObj) -> remove, position = tags1, /destroy, $
     	inHeader = inHeader, inData=inData, $
	errMsg = theErrMsg

   statusMsg = 'remove successful'		; or an empty string
   if (strlen(theErrMsg) NE 0) then begin
      statusMsg = 'remove FAILED ' + theErrMsg
   endif   
   self.parentWobj -> freshen, status=statusMsg
	
   ;-----------------------------------------------------------------------
  treatString = "specObj -> remove,  $"
;    if (size(tags1, /tname) EQ 'STRING') then begin
      treatString = [treatString, $
      string(", position = ['", tags1[0], "', '", tags1[1], $
	"', '", tags1[2], "'] $")]
;    endif else begin
;      treatString = [treatString, string(", position  = ", tags1," $")]
;    endelse
    treatString = [treatString, ", /destroy $"]
    treatString = [treatString, $
    	string(", inHeader=", strtrim(string(inHeader),2), " $")]
    treatString = [treatString, $
    	string(", inData=", strtrim(string(inData),2), " $")]
;    treatString = [treatString, $
;    	string(", inHeader=", strtrim(string(inHeader),2), $
;	       ", inData=", strtrim(string(inData),2), " $")]
   treatString = [treatString, $
    	string(", errMsg = theErrMsg ")]

  ;-----------------------------------------------------------------------
  OK = (self.SpecObj) -> appendField($
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
	qty= treatString ) 
  ;-----------------------------------------------------------------------

   self.parentWobj -> freshen, /where, inHeader = inHeader, inData=inData
   
   if (strlen(theErrMsg) NE 0) then begin
     message, /info, theErrMsg
   endif	
   
end

;==================================================================
;*
pro cw_ow_remove::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)

end

;=======================================================================
pro cw_ow_remove::cleanup
 
  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  self.parentWObj = obj_new()	; Let old reference dangle.

  obj_destroy, [self.where1WObj]

  message, /info, "..."
  
end


;==================================================================
pro cw_ow_remove_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_remove::cleanup
end

;;==================================================================
;pro cw_ow_remove::freshenResults
;
;   (self.where1WObj) -> getProperty, $
;	from = From1, $	; which is sensitized 'input', 'header' or 'data'
;	label = label1, $ ; Goes to the droplist
;	value = value1	; Goes to the text entry box
;
;
;end

;==================================================================
pro cw_ow_remove::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_remove_TLB", $
	title = 'cw_ow_remove title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_remove_event',$
;	/frame, $
	/row)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_remove_TLB", $
	title = 'cw_ow_remove title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_remove_event',$
;	/frame, $
	/row)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

  buttonClusterID = widget_base(tlbID, /column, /frame)
;  voidID = widget_button(buttonClusterID, value='FRESHEN', $
;    		uvalue={wobject:self, method:'eventFreshen'})
;
  voidID = widget_button(buttonClusterID, value='REMOVE', $
    		uvalue={wobject:self, method:'eventProcess'})

  if (self.ParentID EQ 0L) then begin
    void = widget_button(buttonClusterID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
  endif

  ;-------------------------------------------------------------
  doItID = widget_base(tlbID, /row, /frame)

  ;-------------------------------------------------------------
  self.where1WObj = obj_new('cw_ow_where', self.specObj, doItID,$
  	from = 'data', /noInput)

  ;-------------------------------------------------------------
;  self -> freshenResults

  ;-------------------------------------------------------------
  if (self.ParentID EQ 0L) then begin
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_remove',  tlbID
  endif
   
end		;;  cw_ow_remove::createWidgets


;==================================================================
;*
function cw_ow_remove::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	destroy = thisDestroy, $ ; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	parentWobj = thisParentWobj ; parent object
;/
	
  debug = 1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)
 
  ;-------------------------------------------------------------
  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  ;-------------------------------------------------------------
  if (obj_valid(thisParentWObj)) then begin
    self.parentWObj = thisParentWobj
  endif
 
  ;-------------------------------------------------------------
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_remove__define
  define = {cw_ow_remove, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	parentID:0l, $
	parentWObj: obj_new(), $
	tlbID:0l, $
	where1WObj: obj_new() $
	}
;/

end
