; $Id: cw_ow_resultvalue.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_resultvalue
;
; PURPOSE:
;	This is a compound object widget which allows the quantity, errors, 
; and legend to be created for a new field in an object of class specObj. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#CW_OW_RESULTVALUE__DEFINE_arguments">CW_OW_RESULTVALUE__DEFINE_arguments</A>
;
;; OTHER OBJECT CLASSES AND ROUTINES USED:
;	FSC_FIELD
;
; MODIFICATION HISTORY:
;	Created July, 2003. Tanya Riseman
;	TMR. Aug 6, 2003. Changed file name from cw_ow_resultValue.pro to 
;	cw_ow_resultvalue.pro, to help RESOLVE_ALL in the BUILD process.
;-
; Note: in widget definitions, using
;	uvalue = {wobject:self}, $
; so that in the event handler 
; 	 widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.
; 	 self = uObj.wObject		; alias
; This way, if we add a method command, we use 
;	uvalue = {wobject:self, method:'quit'}, 
;

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_resultValue::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_resultValue::print_obj, $
  ; ThisField is parameter with Err of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_resultValue::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_resultValue::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_resultValue::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_resultValue::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_resultValue::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_resultValue_event, $
	event
;/


  widget_control, event.id, get_uvalue=uObj ; confusing to call it self here.

;  message, /info, string("Entered.  uObj.method = ", uObj.method)


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_resultValue_event, event


;==================================================================
pro cw_ow_resultValue::eventQuit, event=event

;  message, /info, "Entered"
  widget_control, self.tlbID, /destroy
  
end


;;==================================================================
;;*
;pro cw_ow_resultValue::event, $
;	event
;;/
;
;  uname = widget_info(event.id, /uname)
;  wErr = widget_info(event.id, /Err)
;
;  ;------------------------------------------
;   message, /info, string("Nothing done here. uname =", uname, " wErr=", wErr)
; 
;end



;=======================================================================
pro cw_ow_resultValue::cleanup
 
;  message, /info, "Entered"

  obj_destroy, self.QtyObj
  obj_destroy, self.ErrObj
  obj_destroy, self.NErrObj
  obj_destroy, self.LegendObj
  
end


;==================================================================
pro cw_ow_resultValue_cleanup, tlb

;    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_resultValue::cleanup

end

;==================================================================
pro cw_ow_resultValue::createWidgets

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_resultValue_TLB", $
	title = 'cw_ow_resultValue title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_resultValue_event',$
	/column)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_resultValue_TLB", $
	title = 'cw_ow_resultValue title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_resultValue_event',$
	/column)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

   QtyID = fsc_field(tlbID, $		; allows sensitive
        object=QtyObj, $
	uName = 'Qty', $
   	title = "Qty:", $
   	value = self.Qty, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_resultValue_event', $
	/string)	
   self.QtyID = QtyID
   self.QtyObj = QtyObj

   ;-------------------------------------------------------------
   ErrID = fsc_field(tlbID, $		; allows sensitive
        object=ErrObj, $
	uname = 'Err', $
   	title = "Err:", $
   	value = self.Err, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_resultValue_event', $
	/string)	
   self.ErrID = ErrID
   self.ErrObj = ErrObj

   ;-------------------------------------------------------------
   nErrID = fsc_field(tlbID, $		; allows sensitive
        object=nErrObj, $
	uname = 'nErr', $
   	title = "nErr:", $
   	value = self.nErr, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_resultValue_event', $
	/string)	
   self.nErrID = nErrID
   self.nErrObj = nErrObj

   ;-------------------------------------------------------------
   LegendID = fsc_field(tlbID, $		; allows sensitive
        object=LegendObj, $
	uname = 'Legend', $
   	title = "Legend:", $
   	value = self.Legend, $
	uvalue = {wobject:self, method:'event'}, $
	/cr_only, event_pro = 'cw_ow_resultValue_event', $
	/string)	
   self.LegendID = LegendID
   self.LegendObj = LegendObj

   ;-------------------------------------------------------------

  if (self.ParentID EQ 0L) then begin
    void = widget_button(self.tlbID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_resultValue',  tlbID
  endif
   
end		;;  cw_ow_resultValue::createWidgets

;==================================================================
;*
function cw_ow_resultValue::count
;/

  return, 1

end

;==================================================================
;*
pro cw_ow_resultValue::setProperty, $
	Qty = thisQty, $ 
	Err = thisErr, $
	nErr = thisnErr, $
	Legend = thisLegend
;/

  infolabel = "cw_ow_resultValue::setProperty >> "

  if (n_elements(thisQty) GT 0) then begin
;    print, infolabel, "thisQty = ", thisQty
    self.Qty = thisQty
     widget_control, self.QtyID, set_value = thisQty
;     OK = execute( string('self.Qty = ', numericQty)
;    if (OK) then begin
;      widget_control, self.QtyID, set_value = thisQty
;    endif else begin
;      errMsg = "Qty is not a numeric scalar or array"
;      message, /info, errMSg
;    endelse
  endif

  if (n_elements(thisErr) GT 0) then begin
    self.Err = thisErr
    widget_control, self.ErrID, set_value = thisErr
;    OK = execute( string('self.err = ', thisErr)
;    if (OK) then begin
;      widget_control, self.errID, set_value = thisErr
;    endif else begin
;      errMsg = "Err is not a numeric scalar or array"
;      message, /info, errMSg
;    endelse
  endif

  if (n_elements(thisNErr) GT 0) then begin
    self.nErr = thisnErr
    widget_control, self.nErrID, set_value = thisnErr
;    OK = execute( string('self.nerr = ', thisNerr)
;    if (OK) then begin
;      widget_control, self.nerrID, set_value = thisNerr
;    endif else begin
;      errMsg = "Nerr is not a numeric scalar or array"
;      message, /info, errMSg
;    endelse
  endif

  if (n_elements(thisLegend) GT 0) then begin
    self.Legend = thisLegend
    widget_control, self.LegendID, set_value = thisLegend
;    OK = execute( string('self.legend = ', thisLegend)
;    if (OK) then begin
;      widget_control, self.legendID, set_value = self.
;    endif else begin
;      errMsg = "Qty is not a numeric scalar or array"
;      message, /info, errMSg
;    endelse
  endif
	
end

;==================================================================
;*
pro cw_ow_resultValue::getProperty, $
	qty = thisQty, $ 
	err = thisErr, $
	nerr = thisNErr, $
	legend = thisLegend
;/

  infolabel = "cw_ow_resultValue::getProperty >> "

  if (arg_present(thisQty)) then begin
    widget_control, self.QtyID, get_value = thisQty	; refresh
    self.qty = thisQty
;    print, infolabel, "thisQty = ", thisQty
  endif

  if (arg_present(thisErr)) then begin
    widget_control, self.ErrID, get_value = thisErr	; refresh
    self.Err = thisErr
  endif

  if (arg_present(thisNErr)) then begin
    widget_control, self.nErrID, get_value = thisNErr	; refresh
    self.NErr = thisNErr
  endif

  if (arg_present(thisLegend)) then begin
    widget_control, self.LegendID, get_value = thisLegend	; refresh
    self.Legend = thisLegend
  endif

end

;==================================================================
;*
function cw_ow_resultValue::init, $
	thisParentID		; optional
;/
	
  debug = 1

  ;-------------------------------------------------------------

  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_resultValue__define
  define = {cw_ow_resultValue, $
;  	specObj:obj_new(), $	; don't destroy
;	destroy: 0, $
	parentID:0l, $
	tlbID:0l, $
	;---------------------------------------------------
	qty: '', $	
	qtyID :0L, $,
	qtyObj: obj_new(), $	; object returned from fsc_field.pro
	err: '', $	
	errID :0L, $,
	errObj: obj_new(), $	; object returned from fsc_field.pro
	nerr: '', $	
	nerrID :0L, $,
	nerrObj: obj_new(), $	; object returned from fsc_field.pro
	legend: '', $	
	legendID :0L, $
	legendObj: obj_new() $	; object returned from fsc_field.pro
	;---------------------------------------------------
	}
;/

end
