; $Id: cw_ow_resvol.pro,v 1.2 2003/09/12 15:25:48 tanya Exp $
;+
; NAME:
; 	cw_ow_resVol
;
; PURPOSE:
; This is a compound object widget which does a resolution volume correction.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE Widgets
;
; OBJECT DEFINTION
; <LI><A HREF="#cw_ow_resVol__DEFINE_arguments">cw_ow_resVol__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
; <LI><A HREF="#CW_OW_RESULT__DEFINE_arguments">CW_OW_RESULT__DEFINE_arguments</A>
; <LI><A HREF="#CW_OW_WHERE__DEFINE_arguments">CW_OW_WHERE__DEFINE_arguments</A>
; <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;	Created July-August, 2003. Tanya Riseman
;-
;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	cw_ow_resVol::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro cw_ow_resVol::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	cw_ow_resVol::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro cw_ow_resVol::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	cw_ow_resVol::helpclass
;
; PURPOSE:
;	Helper method for cw_ow_resVol::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro cw_ow_resVol::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass


;==================================================================
;*
pro cw_ow_resVol::FreshenWhere, $ 
	inHeader = thisInHeader, $
	inData = thisInData
;/

  self.where1WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  self.where2WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  self.where3WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  self.where4WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  self.where5WObj -> freshenWhere, inHeader = thisInHeader, inData = thisInData
  
end
;==================================================================
;*
pro cw_ow_resVol_event, $
	event
;/

  widget_control, event.id, get_uvalue=uObj 

  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event') then begin
      (uObj.wObject) -> event, event
    endif else begin
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; cw_ow_resVol_event, event

;==================================================================
pro cw_ow_resVol::eventQuit, event=event

  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro cw_ow_resVol::eventFreshen, event=event

  self -> freshenResults
  
end

;==================================================================
;*
pro cw_ow_resVol::eventProcess, event=event
;/

 infolabel = "cw_ow_resVol::eventProcess >> "
 tempSpecObj = (self.specObj)

 tempWhereWobj = (self.where1WObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from1, $	; which is sensitized 'input', 'header' or 'data'
	label = label1, $	; Goes to the droplist
	tags=tags1, $
	name=name1, $
	units=units1
 
 tempWhereWobj = (self.where2WObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from2, label = label2, tags=tags2, $
	name=name2, units=units2

 tempWhereWobj = (self.where3WObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from3, label = label3, tags=tags3, $
	name=name3, units=units3

 tempWhereWobj = (self.where4WObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from4, label = label4, tags=tags4, $
	name=name4, units=units4

 tempWhereWobj = (self.where5WObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from5, label = label5, tags=tags5, $
	name=name5, units=units5

   (self.resultWObj) -> getProperty, $
;   	tags = resTags, $
   	label = resLabel, $ 
	name = resName, $
	units =resUnits

  ; Change here!EDIT HERE

   tempSpecObj = (self.SpecObj)
   theErrMsg = ''
   print, infolabel, "E_in = tags2 = ", format_list(tags2)
   print, infolabel, "E_out = tags3 = ", format_list(tags3)
   
   OK =   func_icp_make_resvolume( tempSpecObj, $
   	rawIntensity=tag1, $
	E_in = tags2, $
	E_out = tags3, $
	dspacing_mon = tags4, $
	dspacing_ana = tags5, $
   	resTags = ['qtyObj', reslabel, '*'], $
	resName = resName, $
	resUnits =resUnits, $
	errMsg = theErrMsg)	; Treatment updated inside.

   statusMsg = 'resVol successful'		; or an empty string
   if (NOT OK) then begin
      statusMsg = ['resVol FAILED', theErrMsg]
   endif   

   self.parentWobj -> freshen, status=statusMsg
    
   ; Might be in either place, so update both header and data areas.
   self.parentWobj -> freshen, /Where, /inHeader, /inData
  	  
end

;==================================================================
;*
pro cw_ow_resVol::event, $
	event
;/

  uname = widget_info(event.id, /uname)
  wname = widget_info(event.id, /name)

  ;------------------------------------------
   message, /info, string("Should not be here. uname =", uname, " wname=", wname)

end

;=======================================================================
pro cw_ow_resVol::cleanup
 
  if (self.destroy) then begin 
;    message, /info, "Destroying self.specObj"
    obj_destroy, self.specObj
  endif else begin
;    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse

  self.parentWObj = obj_new()	; Let old reference dangle.

  obj_destroy, [self.resultWObj, $
  	self.where1WObj, self.where2WObj, self.where3WObj, $
	self.where4WObj, self.where5WObj]
  
  message, /info, "..."
end


;==================================================================
pro cw_ow_resVol_cleanup, tlb

    message, /info, "Entered'

    widget_control, tlb, get_uvalue=uObj
    obj_destroy, (uObj.wObject)		; calls cw_ow_resVol::cleanup
end

;==================================================================
;*
pro cw_ow_resVol::freshenResults
;/

 message, /info, 'Entered'
 help, (self.where1WObj)
 
 tempWhereWobj = (self.where1WObj)
 tempSpecObj = (self.specObj)
 helper_cw_ow_freshenResults, tempWhereWObj, tempSpecObj, $
 	from=from1, label = label1, tags=tags1, $
	name=name1, units=units1
 
; helper_cw_ow_freshenResults, (self.where2WObj), (self.specObj), $
; 	from=from2, label = label2, tags=tags2, $
;	name=name2, units=units2
 
; helper_cw_ow_freshenResults, (self.where3WObj), (self.specObj), $
; 	from=from3, label = label3, tags=tags3, $
;	name=name3, units=units3
 
; helper_cw_ow_freshenResults, (self.where4WObj), (self.specObj), $
; 	from=from4, label = label4, tags=tags4, $
;	name=name4, units=units4
 
; helper_cw_ow_freshenResults, (self.where5WObj), (self.specObj), $
; 	from=from5, label = label5, tags=tags5, $
;	name=name5, units=units5
 
  help,  Name1
   resName = string( Name1, " norm'd by resolution volume")
;   resName = string( Name1, ' normalized by resolution volume')
   reslabel = string( Label1, '_ResVol')
;   reslabel = string( Label1, '_NORMED')
   resUnits = units1

   (self.resultWObj) -> setProperty, $
   	label = resLabel, $ 
	name = resName, $
	units =resUnits   

end

;==================================================================
pro cw_ow_resVol::createWidgets
; This widget is a bit wide, so make it into 2 lines

 ;---------------------------------------------------  
  if (self.ParentID EQ 0L) then begin
    tlbID = widget_base($
  	uname = "cw_ow_resVol_TLB", $
	title = 'cw_ow_resVol title. No parent.' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_resVol_event',$
;	/frame, $
	/row)
  endif else begin
    tlbID = widget_base(self.ParentID, $
  	uname = "cw_ow_resVol_TLB", $
	title = 'cw_ow_resVol title. ' , $
	uvalue={wobject:self, method:'event'},$			; needed for kill, cleanup
	event_pro='cw_ow_resVol_event',$
;	/frame, $
	/row)
 endelse
 
 self.tlbID = tlbID

  ;-------------------------------------------------------------

  buttonClusterID = widget_base(tlbID, /column, /frame)
  voidID = widget_button(buttonClusterID, value='FRESHEN', $
    		uvalue={wobject:self, method:'eventFreshen'})

  voidID = widget_button(buttonClusterID, value='PROCESS', $
    		uvalue={wobject:self, method:'eventProcess'})

  if (self.ParentID EQ 0L) then begin
    void = widget_button(buttonClusterID, value='QUIT', $
    		uvalue={wobject:self, method:'eventQuit'})
  endif


  ;-------------------------------------------------------------
;Old  doItID = widget_base(tlbID, /row, /frame)
  ; Split into 2 lines because it is too wide now.
  doItID = widget_base(tlbID, /column, /frame)
  doItID1 = widget_base(doItID, /row)
  doItID2 = widget_base(doItID, /row)

  self.resultWObj = obj_new('cw_ow_result', doItID1)
  voidID = Widget_label(doItID1, value=$
"= f('Intensity-Bkgd', 'E_in', 'E_out', 'dspacing_mon', 'dspacing_ana')")
;  voidID = Widget_label(doItID2, value="F()"$
 
  ;-------------------------------------------------------------
  doItID2_1 = widget_base(doItID2, /column)
  voidID = Widget_label(doItID2_1, value="Intensity-Bkgd")
  self.where1WObj = obj_new('cw_ow_where', self.specObj, doItID2_1,$
  	from = 'data', $
	field = 'INTENSITY')
;	field = ['*', 'INTENSITY', '*'])

  ;-------------------------------------------------------------
;  voidID = Widget_label(doItID2, value="-")
  
  ;-------------------------------------------------------------
  doItID2_2 = widget_base(doItID2, /column)
  voidID = Widget_label(doItID2_2, value="E_in")
  OK = (self.specObj) -> getField(['*','E_in', '*'], /inData, qty=junk)
  if (OK) then begin
    self.where2WObj = obj_new('cw_ow_where', self.specObj, doItID2_2,$
  	from = 'data', $
	field = 'E_in')
  endif else begin
    OK = (self.specObj) -> getField(['*','E_in', '*'], /inHeader, qty=junk)
    if (OK) then begin
      self.where2WObj = obj_new('cw_ow_where', self.specObj, doItID2_2,$
  	from = 'header', $
	field = 'E_in')
    endif else begin
      self.where2WObj = obj_new('cw_ow_where', self.specObj, doItID2_2,$
  	from = 'input', $
	field = 1.0)
    endelse
  endelse
  
  ;-------------------------------------------------------------
;  voidID = Widget_label(doItID2, value="*")
  ;-------------------------------------------------------------
  doItID2_3 = widget_base(doItID2, /column)
  voidID = Widget_label(doItID2_3, value="E_out")
;  self.where3WObj = obj_new('cw_ow_where', self.specObj, doItID2_3,$
;  	from = 'data', $
;	field = 'E_out')
  OK = (self.specObj) -> getField(['*','E_out', '*'], /inData, qty=junk)
  if (OK) then begin
    self.where3WObj = obj_new('cw_ow_where', self.specObj, doItID2_3,$
  	from = 'data', $
	field = 'E_out')
  endif else begin
    OK = (self.specObj) -> getField(['*','E_out', '*'], /inHeader, qty=junk)
    if (OK) then begin
      self.where3WObj = obj_new('cw_ow_where', self.specObj, doItID2_3,$
  	from = 'header', $
	field = 'E_out')
    endif else begin
      self.where3WObj = obj_new('cw_ow_where', self.specObj, doItID2_3,$
  	from = 'input', $
	field = 1.0)
    endelse
  endelse
  

  ;-------------------------------------------------------------
  doItID2_4 = widget_base(doItID2, /column)
  voidID = Widget_label(doItID2_4, value="dspacing_mon")
  OK = (self.specObj) -> getField(['*','dspacing_mon', '*'], /inHeader, qty=junk)
  if (OK) then begin
    self.where4WObj = obj_new('cw_ow_where', self.specObj, doItID2_4,$
  	from = 'header', $
	field = 'dspacing_mon')
  endif else begin
    self.where4WObj = obj_new('cw_ow_where', self.specObj, doItID2_4,$
  	from = 'input', $
	field = 1.0)
  endelse
  
  ;-------------------------------------------------------------
  doItID2_5 = widget_base(doItID2, /column)
  voidID = Widget_label(doItID2_5, value="dspacing_ana")
  OK = (self.specObj) -> getField(['*','dspacing_ana', '*'], /inHeader, qty=junk)
  if (OK) then begin
    self.where5WObj = obj_new('cw_ow_where', self.specObj, doItID2_5,$
  	from = 'header', $
	field = 'dspacing_ana')
  endif else begin
    self.where5WObj = obj_new('cw_ow_where', self.specObj, doItID2_5,$
  	from = 'input', $
	field = 1.0)
  endelse

  ;-------------------------------------------------------------
  self -> freshenResults

;Nope.  self.parentWobj -> freshen, status='resVol waiting for PROCESS click.'

  ;-------------------------------------------------------------
  if (self.ParentID EQ 0L) then begin
    widget_control,tlbID,/realize		; make room for all these things.
    xmanager, 'cw_ow_resVol',  tlbID
  endif
   
end		;;  cw_ow_resVol::createWidgets


;==================================================================
;*
function cw_ow_resVol::init, $
	thisSpecObj, $		; required
	thisParentID, $		; optional
	destroy = thisDestroy, $	; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	parentWobj = thisParentWobj ; parent object
;/
	
  debug = 1

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, "Invalid object specObj."
    return, -1
  endif

  self.specObj = thisSpecObj
  
  help, self.specObj 

  self.destroy = keyword_set(thisDestroy)
 
  ;-------------------------------------------------------------
  if (n_elements(thisParentID) NE 0) then begin
    self.parentID = thisParentID
  endif
 
  ;-------------------------------------------------------------
  if (obj_valid(thisParentWObj)) then begin
;    help, thisParentWobj
;    thisParentWobj -> print_obj
    self.parentWObj = thisParentWobj
  endif
 
  ;-------------------------------------------------------------
  self -> createWidgets
  
  return, 1
  
end

;==================================================================
;*
pro cw_ow_resVol__define	; 
  define = {cw_ow_resVol, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $		; Whether ornot to destroy specObj.
	parentID:0l, $
	parentWObj: obj_new(), $
	tlbID:0l, $
	resultWObj: obj_new(), $	; Corrected intensity
	where1WObj: obj_new(), $	; Orig Intensity
	where2WObj: obj_new(), $	; E_in
	where3WObj: obj_new(), $	; E_out
	where4WObj: obj_new(), $	; dspacing_mon (in)
	where5WObj: obj_new() $		; dspacing_ana (out)
	}
;/

end
