; $Id: dave_tagcontainer__define.pro,v 1.3 2003/09/10 19:26:52 tanya Exp $ 
;+
; NAME:
;       dave_tagcontainer
;
; PURPOSE:
;
; A generalization of the IDL object class IDL_container,
; where items can be referred to by a "tag" string as an alternative
; to a positition integer.
; 
; This object holds other objects, particularly the classes 
; DAVE_tagContainer and qtyObj.
; Destroying a DAVE_tagContainer destroys any objects that 
; have been added to the container via the Add method.
;
; I have found that it is preferable to make a container class
; DAVE_tagContainer with basically two trees in parallel,
; one for the basic "data" using the class qtyObj and 
; another for I/O and GUI issues using the class tagObj.
; This is specifically to avoid creating a new class which inherits
; qtyObj and tagObj. This has several purposes:
; 	1. Avoid confusion by avoiding mixing the "pure" data and how the
;	   application manages it. 
;	2. Avoid as much as possible the use of EXTRA keywords parameters.
;	   Use the same methods and keywords as much as possible
;	   for different classes instead and use them explicitly.
;	   EXTRA and EXTRA_REF can not be mixed!
;	   EXTRA works fine for "input" keywords whose values are not modified.
;	3. Previously, I had problems with infinite loops in
; 	   GUIs from calling get/set with inherited keywords 
;	   in complex inherited classes. This would occur when a 
;	   typo created too few valid keywords. Then some sub-classes methods
;	   would be called with no valid keywords. 
;
; METHODS:
;	For a list of methods, either
; 	1. Use procedure "MK_HTML_HELPARG" to generate an HTML file
; 	containing documention extracted from this file.
;	or 2. Run the method "HELP_OBJ" to list the contents of the
;	class and all methods (with keywords in UPPERCASE and parameters
;	in lowercase. 
;
; Key methods: INIT, ADD, COUNT, GET.
;	CALLMETHOD could be used for running GUI-selected methods.
;	CALLFUNCTION and CALLPROCEDURE can be used to "add" external 
;	methods without changing the basic definition of this class 
;	DAVE_tagContainer. For example, instrument-specific initialization
;	routines. Of course, all functions and procedures can only use
;	methods get/set to access the private data. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE application programming.
;
; OBJECT DEFINTION
;<LI><A HREF="#DAVE_TAGCONTAINER__DEFINE_arguments"> DAVE_TAGCONTAINER__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	IDL_CONTAINER
;	<LI><A HREF="#TAGOBJ__DEFINE_arguments">TAGOBJ__DEFINE_arguments</A>
;
; OTHER ROUTINES USED:
;	...
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; MODIFICATION HISTORY:
;    Tanya Riseman, Spring, 2003. 
;    TMR. Aug 6, 2003. changed file name to all lowercase to help BUILD's RESOLVE_ALL
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;=============================================================================
;*
function DAVE_tagContainer::init
;/
;	myTags = thisMyTags		;;, $
;	parent = thisParent		;;, $
;	nodeStyle = thisNodeStyle, $	 
;	nodeTag = thisNodeTag 	 

;  self.nodeStyle =	obj_new('DAVE_primitiveString', thisNodeStyle)	
;  self.nodeTag  = 	obj_new('DAVE_primitiveString', thisNodeTag )
  self.tags = 		obj_new('IDL_Container') ; containing tagObj
  self.contents = 	obj_new('IDL_Container') ; containing DAVE_tagContainer
  						; or qtyObj.	

;  self.myTags = thisMyTags	; an object of class tagObj
;  self.parent = thisParent

  return, 1
end		;; DAVE_tagContainer::init,

;=============================================================================
;*
function DAVE_tagContainer::clone
;/
  infolabel =  "DAVE_tagContainer::clone >> "

  clone =obj_new('DAVE_tagContainer')

  ; linking of tags to fill clone.myTags is done by the add method.
  
  ncount = self -> count()
  for icount = 0, (ncount -1) do begin
    oneObj = self -> get(position = icount)
    if (NOT obj_valid(oneObj)) then begin
      print, infolabel, icount, $
      	" Unable to invoke method on NULL object reference:", $
        "ONEOBJ"
      print, infolabel, ncount, "= self -> count()"
;      self -> show, /recursive

      self -> getLabel, label=labelList	
      		;;, sizeString=sizeStringList	;;, count =countList
		
      print, infolabel, "Problem with i=", icount, " ", labelList[icount] 
;      for i = 0, (n_elements( labelList) -1) do begin
;;        print, i, ' ', labellist[i], " size=", sizeStringList[i]
;        print, i, ' ', labellist[i]	;;;, " count=", countList[i]
;      endfor       
      
    endif 

    oneTagObj = self -> getTag(position = icount)
    OK = oneTagObj -> get (tags = oneTags)
    clone -> add, tags=oneTags, contents = (oneObj -> clone()) 
;    print, infolabel, icount, " adding tags=", onetags
    
    ; Now clone the other fields in the tag object. 
    clone -> setTag, position = icount, tagObj=oneTagObj
;    print, infolabel, icount, " modifiying tags using obj ", obj_valid(oneTagObj)
;    onetagobj -> show 
;    print     
  endfor
  
  return, clone
end		;; DAVE_tagContainer::clone

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	DAVE_tagContainer::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro DAVE_tagContainer::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;+
; NAME:
; 	DAVE_tagContainer::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;-
;*
pro DAVE_tagContainer::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	DAVE_tagContainer::helpclass
;
; PURPOSE:
;	Helper method for DAVE_tagContainer::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro DAVE_tagContainer::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass

;========= End of methods common to all classes =============================
;=============================================================================
;*
function NumberOfTags, $ ; Returns the number of tags with form ['','','']
	thisTag		 ; An string array of size [3] or [nelem,3] 
;/
	
  tag_rank =  size(thisTag, /n_dimensions) 
  tag_dim =  size(thisTag, /dimensions) 
  if (tag_rank eq 1) then begin 
    nelem = 1			; tag_dim = [3]
  endif else begin
    nelem = tag_dim[0]		; tag_dim = [nelem, 3] hopefully
  endelse
;;;  nelem = n_elements(thisTag) 
	
  return, nelem

end		; NumberOfTags

	

;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::convert2Position
;
; PURPOSE:
; If the argument thisThing is a numeric type, assume that is already the 
; position and return it as came.
; Otherwise the argument thisThing is an array of type STRING and we assume
; that they are tags ['...','...','...'].
; Find the first matching tag in the container and
; return its position. Return negative number -1 if failure.
; If an array of tags, return an array of positions.
;-
;*
function DAVE_tagContainer::convert2Position, $; Returns an integer scalar or array.
	thisThing	; tag string  or string array
			; OR index integer or integer array	
;/

  infolabel = "DAVE_tagContainer::convert2Position >> "
  
  tname = size(thisThing, /tname)
  if (tname NE 'STRING') then begin
    ; assume that it is type 'INT' or similar and just return it as-is.
    return, thisThing
  endif

  nelem = numberOfTags(thisThing) 
  if (nelem LE 0) then begin
    print, infolabel, "Error: numberOftags(", thisThing,  ") =", nelem
    return, -1
  endif
  
;;  nelem = n_elements(thisThing) 
  ncount = (self.tags) -> count()

  if (nelem EQ 1) then begin
    currentPosition  = -1 		; Unreasonable value 
  
    ; Stop for loop early if there is a match.
    for i = 0, (ncount-1) do begin
      storedTagObj =  (self.tags) -> get(position=i)
;Wrong      OK = (self.tags) -> get(storedTag, position=i)
      if (obj_valid(storedTagObj)) then begin
        if (storedTagObj -> match(tags = thisThing)) then begin
          currentPosition = i
          goto, stop_forloop
	endif
      endif else begin
        print, infolabel, 'WARNING: Invalid object at position ', i
      endelse
    endfor
    stop_forloop:
  
  endif else begin		;; array of values
    currentPosition  = make_array(nelem, /int, value=-1)  ; Unreasonable value 
    ; thisThing = [['a', 'a', 'a'], ['b', 'b', 'b']]
    ; ['a', 'a', 'a'] == thisThing[*,0]
    for iTag = 0, (nelem-1) do begin
      ; Stop for loop early if there is a match.
      for i = 0, (ncount-1) do begin
;Old        OK = (self.tags) -> get(storedTag, position=i)
;Old        if (storedTag EQ thisThing[iTag]) then begin
        storedTagObj =  (self.tags) -> get(position=i)
	oneThing = thisThing[*, itag]
        if (storedTagObj -> match(tags = oneThing)) then begin
          currentPosition[i] = i
          goto, stop_forLoopArray
        endif
      endfor
      stop_forloopArray:
    endfor
  endelse
  
  return, currentPosition

end		;; DAVE_tagContainer::Convert2Position,
	


;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::convert2Tag
;
; PURPOSE:
; If the argument thisThing is a STRING type, assume that is already the 
; tag(s) and return it as came.
; Otherwise the argument thisThing is type INT or simular and we assume that
; it is the index or indices. Return it's corresponind tag(s).
; If an array of indices, return an array of tags.
;-
;*
function DAVE_tagContainer::convert2Tag, $ ; Returns a tag string ['','',''] 
					   ; or tag array
	thisThing 	; integer or integer array or 
			; tag string ['','',''] or tag array
;/
 
  tname = size(thisThing, /tname)
  if (tname EQ 'STRING') then begin
    ; assume that it is already the tag(s) and return it as-is.
    return, thisThing
  endif

  tagObj = (self.tags) -> get(position=thisThing)

  if (obj_valid(tagObj)) then begin		; should be class tagObj
    nelem = n_elements(tagObj) 
    if (nelem  eq 1) then begin
      OK = tagObj-> get(tags=currentTag)
      if (OK) then begin
        return, currentTag
      endif else begin
        return, ['', '', '']
      endelse   
    endif else begin
      OK = 1
      currentTag = make_array(nelem, 3, /string, value='')
      for i = 0, (nelem-1) do begin
        OK = OK AND tagObj[i]-> get(tags=oneTag)
        currentTag[i,0:2] = oneTag
      endfor
      return, currentTag
    endelse     
  endif else begin
    return, ['', '', '']		;; '<PROBLEM!!!>'
  endelse
  
end		;; DAVE_tagContainer::Convert2Tag,


;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::TagIsContained
;
; PURPOSE:
; 	Method returns 1/0 (true/false) for if all the tags in the 
;	argument thisTag have matches in the container class. 
; 	Optional keyword ncontained returns the number of matching tags.
;	Optional keyword postion returns an array containing the location 
;	in the container class of the tag(s) provided in the argument thisTags.
;	 The value -1 is given when a match was not found.  
; 	This method complements the method TagIsUnique.
;-
;*
function DAVE_tagContainer::TagIsContained, $	;Returns 0/1 for TRUE/FALSE
	thisTag, $			 ; tag string ['','',''] or tag array
	ncontained = thisNcontained, $	;Optional returned value. int
		; The number of tags which match the contents of database.
	position = thisPosition		;Optional returned val int or array
				; Position in database of matching items.
				; Items with no match provide a value of -1.
;/
  nelem = numberOfTags(thisTag)
;;;  nelem = n_elements(thisTag) 

  ncount = (self.tags) -> count()

  nContained = 0
  
  if (nelem EQ 1) then begin
    ; Stop for loop early if there is a match.
    for i = 0, (ncount-1) do begin
      storedTagObj =  (self.tags) -> get(position=i)
;      storedTag = storedTagObj -> get()
;      OK = storedTagObj -> get(tags = storedTag)
;      if (storedTag EQ thisTag) then begin
;        nContained = 1
;	position = i
;        goto, stop_forloop
;      endif
      if (storedTagObj -> match(tags = thisTag)) then begin
        nContained = 1
	position = i
        goto, stop_forloop
      endif
    endfor
    stop_forloop:
  
  endif else begin		;; array of values
    postion = make_array(nelem, /int, value=-1)
    for iTag = 0, (nelem-1) do begin
      ; Stop for loop early if there is a match.
      for i = 0, (ncount-1) do begin
        storedTagObj =  (self.tags) -> get(position=i)
;        storedTag = storedTagObj -> get()
;        if (storedTag EQ thisTag[iTag]) then begin
        if (storedTagObj -> match(tags = thisTag[iTag])) then begin
          nContained = nContained  + 1
	  position[iTag] = i
          goto, stop_forloopArray
        endif
      endfor
      stop_forloopArray:
    endfor
  endelse

  if (arg_present(thisNContained)) then begin
    thisNContained = nContained 
  endif   
  
  if (arg_present(thisPosition)) then begin
    thisPosition= position
  endif   
  
  success = (nContained EQ nelem)
  
  return, success

end		;; DAVE_tagContainer::TagIsContained,
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::TagIsUnique
;
; PURPOSE:
; 	Method returns 1/0 (true/false) for if all the tags in the argument 
;	thisTag are unique, i.e. not already present in the container class. 
; 	Optional keyword nNew returns the number of new tags.
;	Optional keyword isNew returns an array of length numberOfTags(thisTag)
;	filled with the boolean values 0/1, indicating which ones are unique.
;	
; 	This method complements the method TagIsContained.
;-
;*
function DAVE_tagContainer::TagIsUnique, $ ; Boolean return value for TRUE/FALSE
	thisTag, $		 ; tag string ['','',''] or tag array
		 		; string or string array
	nNew = thisNNew	, $	; Optional keyword. 
				; Returns # of tags which are new.
        isNew = thisIsNew	; Optional keyword. Returns boolean array
;/
	

  nelem = numberOfTags(thisTag)

  ncount = (self.tags) -> count()

  nContained = 0
  
  if (nelem EQ 1) then begin
    nNew = 0
    isContained = self -> tagIsContained(thisTag)
    if (NOT isContained) then begin
      nNew = 1
      isNew = 1
    endif
   
  endif else begin		;; array of values
    nNew = 0
    isNew = make_array(nelem, /int, value=0)
    for iTag = 0, (nelem-1) do begin
      isContained = self -> tagIsContained(thisTag[i])
      if (NOT isContained) then begin
        new = new + 1
	isNew[itag] = 1
      endif
    endfor
  endelse

  if (arg_present(thisNNew)) then begin
    thisNNew = nNew
  endif   

  if (arg_present(thisIsNew)) then begin
    thisIsNew = isNew
  endif     
  
  success = (nNew EQ nelem)
  
  return, success

end		;; DAVE_tagContainer::TagIsUnique
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::IsContained
;
; PURPOSE:
; 	Method returns 1/0 (true/false) for if the argument thisObj 
;	is stored in self.contents, which is a container object. 
;	Optional keyword postion returns an integer containing the location 
;	in the container class of the object provided in the argument thisObj.
;	The value -1 is given when a match was not found.  
; 	Specification is identical to the method IDL_Container::IsContained().
;-
;*
function DAVE_tagContainer::IsContained, $	; Returns 0/1 for true/false
	thisObj, $ 		; An object that might be in the container
	position = thisPosition ; Optional keyword. Returns integer
			; index position of object in the container.
			; Returns -1 if object is not in the container.  
;/

; Method returns 1/0 (true/false) for if thisObj is contained in the container. 

  if (n_elements(thisObj) EQ 0) then begin
    print, infolabel, 'FAILURE. Require argument thsiObj'
  endif
  
  infolabel = "DAVE_tagContainer::IsContained >> "
  if (arg_present(thisPosition)) then begin
;Debug    print, infolabel
;Debug    thisObj -> show
;Debug    self -> show
    success = (self.contents) -> IsContained(thisObj, position = thisPosition)
  endif else begin
    success = (self.contents) -> IsContained(thisObj)
  endelse

  return, success 
  
end		;; DAVE_tagContainer::IsContained,
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::IndexInRange
;
; PURPOSE:
; 	Method returns 1/0 (true/false) for if the scalar integer argument 
;	thisLocation is in the valid range of items stored in the container,
;	i.e. less than self -> count(). 
;	The argument thisLocation may also be an array of indices,
;	in which case the return value is true only if all elements
;	of thisLocation are valid indices.
;	In this case, the optional keyword inRange returns
;	a boolean array of the same length as thisLocation. 
;
;	 Use tagIsContained for searching by tag rather than location, 
;	if needed.
;-
;*
function DAVE_tagContainer::IndexInRange, $ ; Returns overall 0/1 for true/false
	thisLocation, $ 	; integer. indicating position in container
				; Might be scalar or array
  	verbose = thisVerbose, $;Optinal keyword.
				;  Complains to screen if there is a problem.
	inRange = thisInRange ; Optional keyword. 
			; Returns array of 0/1 (true/false)  
			; for individual elements of thisLocation.
;/


  infolabel = 'DAVE_tagContainer::IndexInRange >> '
  
  ncount = (self.tags) ->count()

  nelem = n_elements(thisLocation)
  if (nelem EQ 1) then begin
    OK = 1
    if NOT((thisLocation GE 0) AND (thisLocation LT ncount)) then begin
      if (keyword_set(thisVerbose)) then begin
        print, infolabel, " Location ",  $
      	  strcompress(string(thisLocation), /remove_all),  $
      	  " is out of range 0-", strcompress(string(ncount), /remove_all)
      endif
      OK = 0
    endif
    return, OK

  endif else begin
    OK = 1
    localInRange = make_array(nelem, /int, value=-1)
    for i = 0, (nelem-1) do begin
      if ((thisLocation[i] GE 0) AND (thisLocation[i] LT ncount)) then begin
         localInRange[i] = 1
      endif else  begin
        OK = 0
      endelse
    endfor
    if (arg_present(thisInRange)) then begin
      thisInRange = localInRange
    endif
    return, OK
  endelse  

end		;; DAVE_tagContainer::IndexInRange,
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::Move
;
; PURPOSE:
;	See IDL_container::Move.
;	The defintion has been extended so that tags of the form ['','','']
;	can be used instead of index positions for the two arguments
;	thisSource and thisDestination.
;
;-
;*
pro DAVE_tagContainer::Move, $
	thisSource, $	; either integer location or tag string array
	thisDestination	; either integer location or tag string array
;/

  infolabel = "DAVE_tagContainer::Move >> "

  localSource = 	self -> convert2Position(thisSource)  
  localDestination = 	self -> convert2Position(thisDestination)

  OK = 1
  
  if (not self->IndexInRange(localSource)) then begin
    print, infolabel, $
    	"Object(s) can not be moved because Source(s) not in container."
    OK = 0    
  endif

  if (not self->IndexInRange(localDestination)) then begin
    print, infolabel, $
    	"Object(s) can not be moved because Destination(s) not in container."
    OK = 0    
  endif
  
  if (NOT OK) then begin
    return
  endif

; Identical to IDL_Container::Move
 
  (self.contents) -> Move, localSource, localDestination
  (self.tags) -> Move, localSource, localDestination

  return
  
end		;; DAVE_tagContainer::Move
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::Remove
;
; PURPOSE:
;	See IDL_container::Remove.
;
;	Items in the container can be removed either by refering to their
;	position (keyword position) or passing the object itself as
;	an argument.
;
;	The defintion has been extended so that tags of the form ['','','']
;	can be used instead of index positions for the optional keyword 
;	position. 
;
;	Also, the optional boolean keyword destroy has been added.
;	if /destroy is used, then the object is destroyed when it is removed.
;	This is useful if the container contains the only reference to the
;	object. 
;
;-
;*
pro DAVE_tagContainer::Remove, $
	;--- Use either child_object parameter OR postion keyword OR all
	child_Object, $		; optional
	position = thisPosition, $; Optional. Tag string or index integer.
	all  = thisALL, $	; Optional boolean keyword.
			; Removed all objects from the container.
  	verbose = thisVerbose, $ ; Complains to screen if there is a problem.
	destroy = thisDestroy	; Optional boolean keyword.
			; Destroys objects as they are removed
			; Default behavior is is to simply remove the reference.
;/

  infolabel = "DAVE_tagContainer::Remove >> "

  ;------------------------------------------------------------
  if (keyword_set(thisAll)) then begin
    print, infolabel, ' keyword /ALL'
    ; Simplest case
    if (keyword_set(thisDestroy)) then begin
      obj_destroy, (self.contents)
      obj_destroy, (self.tags)
    endif else begin
      (self.contents) -> Remove, all=thisAll
      (self.tags) -> Remove, all=thisAll
    endelse
    return
  endif
  ;------------------------------------------------------------
  
;  ncount = (self.tags) -> count()

  if (n_elements(thisPosition) GT 0) then begin
    localPosition = self -> convert2Position(thisPosition)
    if (not self->IndexInRange(localPosition, verbose=thisVerbose)) then begin
      if (keyword_set(thisVerbose)) then begin
        print, infolabel, "Object(s) can not be removed because not in container."
      endif
      return
    endif    
  endif else begin

      ;; localPosition is not defined.
  endelse
  
  ;------------------------------------------------------------
  if ((n_elements(child_object) EQ 0) AND (NOT arg_present(child_object)) ) $
  		 then begin
;debug      print, infolabel, ' keyword position or no keywords'
     ; easy case
      if (keyword_set(thisDestroy)) then begin
        tag_object = (self.tags) -> get(position=localPosition) 
        child_object = (self.contents) -> get(position=localPosition) 
;Debug        print, infolabel, 'localPosition=', localPosition
;Debug	print, infolabel, "obj_valid(tag_object)=", obj_valid(tag_object)
;Debug	print, infolabel, "obj_valid(child_object)=", obj_valid(child_object)
      endif
      (self.contents) -> Remove, position = localPosition
      (self.tags) -> Remove, position = localPosition   
      if (keyword_set(thisDestroy)) then begin
;        print, infolabel, "Destroying position=", localPosition
;	help, child_object
;	help, tag_object
        obj_destroy, child_object
        obj_destroy, tag_object
      endif    

  endif else begin
    if (NOT obj_valid(child_object)) then begin
      if (keyword_set(thisVerbose)) then begin
        print, infolabel, "Object(s) can not be removed because not valid."
      endif
      return
    endif
    ; we need position so that we can remove the child_object's tag.
;Debug    print, infolabel, " by object reference"
;Debug    print, 'arg_present(child_object) = ', arg_present(child_object)
;Debug    help, child_object
;Debug    print, 'obj_valid(child_object) = ', obj_valid(child_object)
;Debug    child_object ->  show, /recursive
    OK = self -> IsContained(child_object, position=childPosition)
;;;    OK = (self.contents) -> IsContained(child_object, position=childPosition)
;Debug    print, infolabel, OK, 'isContained(',childPosition, ') =', OK 
    if (keyword_set(thisDestroy)) then begin
      tag_object = (self.tags) -> get(position=childPosition) 
    endif
    if (OK) then begin
      (self.contents) -> Remove, child_object
      (self.tags) -> Remove, position = childPosition   
      if (keyword_set(thisDestroy)) then begin
;       print, infolabel, "By child, Destroying position=", localPosition
;	help, child_object
;	help, tag_object
        obj_destroy, child_object
        obj_destroy, tag_object
      endif    
    endif else begin
      if (keyword_set(thisVerbose)) then begin
        print, infolabel, "Object(s) can not be removed because not in containier."
      endif
    endelse
  endelse

  return
  
end		;; DAVE_tagContainer::Remove
	
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::count
;
; PURPOSE:
;	Returns the number of items stored in the container class. 
;-
;*
function DAVE_tagContainer::Count	; Returns integer.
					; Returns # of objects in container.
;/
  if (obj_valid(self.tags)) then begin
    return, self.tags -> Count()	
  endif else begin
    return, 0	
  endelse

end		;; DAVE_tagContainer::Count
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::linkTags
;
; PURPOSE:
;	If a tree hierarchy is created using DAVE_tagContainer objects
;	the add method sets self.myTags to the tag object used to label
;	that object in the layer above.
;
;	For internal use.
;-
;*
pro DAVE_tagContainer::linkTags, $	
	thisMyTagObj			;  object of class tagObj.
;/
    self.myTags = thisMyTagObj 
end		;; DAVE_tagContainer::linkTags

;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::Add
;
; PURPOSE:
;	Similar to  IDL_container::Add.
;	
;	Each object added must have a tag string array associated with it,
;	['','',''].
;
;	The optional keyword position has been extended to accept a
;	tag string ['','',''] or an index integer.
;
;	The optional keyword Esrrmsg returns an error message upon
;	error, while returning a NULL string '' upon success.
;-
;*
pro DAVE_tagContainer::Add, $
	; require BOTH tag thisTag and an object thisObject.
	tags = thisTag, $	; Required keyword.
				; string array tag ['','',''] for thisObject
	contents=thisObject, $	 ; Required keyword. 
				; Object to be added to container.
	position = thisPosition, $ ; Optional location to place, 
			; type integer (index) or string array (tag ['','','']).  
			; Defaults to the end.
	errMsg = thisErrMsg	; Optional keyword String. Returns error message.
				; Null string '' if sucessful.
;/
;	OK = thisOK, $

  infolabel =  "DAVE_tagContainer::Add >> "

;  if (arg_present(thisOK)) then  thisOK = 0

  if (n_elements(thisPosition) GT 0) then begin 
    localPosition = self -> convert2Position(thisPosition)
  endif		; otherwise, localPosition not defined.
  
  ; Make sure that the new tag(s) are unique.
  OKtags = self -> TagIsUnique(thisTag)

  if (OKtags) then begin  
;Old    tagObj = obj_new('DAVE_primitiveString', thisTag)
;Old    self.tags -> Add, TagObj, position = localPosition
;Play    tempObj = obj_new('tagObj', tags=thisTag)
    self.tags -> Add, obj_new('tagObj', tags=thisTag),$
    		position = localPosition
    if (obj_isa(thisObject, 'DAVE_tagContainer')) then begin
      ncount = (self.tags) -> count()
      myTagObj = (self.tags) -> get(position = (nCount-1))
      thisObject -> LinkTags, myTagObj 
    endif
    self.contents -> Add, thisObject, position = localPosition
    if (arg_present(thisErrMsg)) then  thisErrMsg = ''
;    if (arg_present(thisOK)) then  thisOK = 1
 
  endif else begin
      anErrMsg =  infolabel + $
      '[' + thisTag + ']' + $
      " One or more of requested tag(s) are already being used"
    if (arg_present(thisErrMsg)) then begin
      thisErrMsg = anErrMsg
    endif else begin
      print, anErrMsg
    endelse
  endelse 
  
end		;; DAVE_tagContainer::Add,
	
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::Get
;
; PURPOSE:
;	Similar to  IDL_container::Get.
;	
;	Each object added must have a tag string array associated with it,
;	['','',''].
;
;	The optional keyword position has been extended to accept a
;	tag string ['','',''] or an index integer.
;
;	The optional keyword Esrrmsg returns an error message upon
;	error, while returning a NULL string '' upon success.
;-
;*
function DAVE_tagContainer::Get, $	; Returns object or 
				; returns 1 (when keyword /everything used.)
	; returns object or objects from self.contents.
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; String or string array keyword. 
		; Only objects that inherit from class name(s) are returned.
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; Keyword.
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	count = thisCount, $		; Keywords returns integer, 
			; # of objects selected
	everything = thisEverything, $	; Optional keyword which 
			; returns info string in keyword instead of the object
			; function returns 1 in this case.
	verbose = thisVerbose	; Optional keyword. Reports on progress.
;/
	
  infolabel = "DAVE_tagContainer::Get >> "
  
;;;  if (keyword_set(thisEverything)) then begin
  if (arg_present(thisEverything)) then begin
;Debug     print, infolabel, "arg_present thisEverything"
  ;	behaves incompatibly with all the other options for get.
  ;	returns 0/1, not objects.
        ncount = self -> count()
  	localClassSize = $	;;;'{' +  obj_class(self) +  '}' +  $
 		'[' + strcompress(string(ncount), /remove_all) + ']'
	myTagStrings = ''
	if (obj_valid(self.myTags)) then begin
          OK =  (self.myTags)-> get(tags=myTagStrings)
;Debug	  print, infolabel, OK, myTagStrings
	endif else begin
	  myTagsStrings = '<no associated tag object>'
	endelse
	thisEverything = [localClassSize, myTagStrings]
	return, 1			
  endif
  
  if (n_elements(thisPosition) GT 0) then begin 
    localPosition = self -> convert2Position(thisPosition)
;    if (keyword_Set(thisVerbose)) then begin
;      print, infolabel, "thisPosition = ", thisPosition
;      print, infolabel, "localPosition = ", localPosition
;    endif
    if (localPosition lt 0) then begin
      someTags = ['??', "?mytags?", '??']
      OK = (self.mytags) -> get(tags=sometags)
      if (keyword_set(thisVerbose)) then begin
       print, infolabel,  sometags, $
      		"Error finding index", localPosition, $
		" tag position '", thisPosition, "' " 
;        print, infolabel,  sometags, $
;      		"Error finding position '", thisPosition, "'"
;;        print, infolabel, localPosition, $
;;      		"Error finding position '", thisPosition, "'"
;        print, infolabel, "position's type (STRING, INT) = ", $
;		size(thisPosition, /tname)
      endif
     ; premature!
;opps      return, localPosition
    endif
  endif
  
 if (n_elements(localPosition) GT 0) then begin
   if (localPosition LT 0) then begin
       thisCount = -1
       return, obj_new()		; invalid empty object
   endif
 endif 

  if (arg_present(thisCount)) then begin
    ; thisCount is a named variable, not a constant
    ; Arg_present() function is not terribly robust.
      retObjs = self.contents -> get( position=localPosition, $
  	all=thisAll, ISA=thisISA, count=ThisCount)
;      return,  self.contents -> get( position=localPosition, $
;  	all=thisAll, ISA=thisISA, count=ThisCount)	
  endif else begin
      retObjs =  self.contents -> get( position=localPosition, $
  	all=thisAll, ISA=thisISA)
;      return, self.contents -> get( position=localPosition, $
;  	all=thisAll, ISA=thisISA)
  endelse

;  retType = size(retObjs, /tname)
;  message, /info, string( "retType = ", retType)
;  if (retType NE 'OBJREF') then begin
;    if (keyword_set(thisVerbose)) then begin
;      message, /info, string(retType, "Not type objRef")
;      print, retObjs
;      help, retObjs
;    endif
;    
;  endif else begin
   if (n_elements(retObjs) EQ 1) then begin
    if (NOT obj_valid(retObjs)) then begin
      if (keyword_set(thisVerbose)) then begin
        message, /info, $
		string("Invalid object. Requested position=", thisPosition)
        help, retObjs
      endif
     endif
   endif else begin
    if (NOT obj_valid(retObjs[0])) then begin
      if (keyword_set(thisVerbose)) then begin
        message, /info, string(n_elements(retObjs), " invalid objects")
        help, retObjs
      endif
     endif
   endelse
; endelse
 
  return, retObjs

  
end		;; DAVE_tagContainer::Get
	
;=============================================================================
;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::GetTag
;
; PURPOSE:
;	See also DAVE_tagContainer::GetTag.
;
;	Returns the tag object (class tagObj) for object stored at postion 
;	given by the keyword thisPosition.
;	Or it returns an array of tag objects when the keyword /all
;	is used. Note: the tag object contains information in addition
;	to just the tags, which is useful for use interface.
;-
;*
function DAVE_tagContainer::GetTag, $	; returns tag object(s) 
			; used to label objects stored in this container.
	all = thisAll, 	$ 	; Boolean keyword
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; keyword
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	count = thisCount	; Keywords returns integer, # of objects selected
;/
;; A little hard to do for now.
;;;	ISA = thisISA, $ 	; class names(S). string or string array
	
  infolabel =  "DAVE_tagContainer::GetTag >> "

  if (n_elements(thisPosition) GT 0) then begin 
    localPosition = self -> convert2Position(thisPosition)
  endif
  
  if (arg_present(thisCount)) then begin
    ; thisCount is a named variable, not a constant
    ; Arg_present() function is not terribly robust.
    storedTagObj =   (self.tags) -> get( position=localPosition, $
  	all=thisAll, $
;;;	ISA=thisISA, $
	count=ThisCount)	
  endif else begin
    storedTagObj = (self.tags) -> get( position=localPosition, $
  	all=thisAll)
;;;	, ISA=thisISA)	
  endelse

;  nelem = n_elements(storedTagObj)
;  if (nelem EQ 1) then begin
;    OK =  storedTagObj -> get(tags=storedTags)
;;;;    storedTags =  storedTagObj -> get()
;  endif else begin
;    ; this might not work. Might need
;    storedTags =  make_array(nelem, 3, /string)
;    for i = 0, (nelem-1) do begin
;      OK =  storedTagObj[i] -> get(tags=temptags)
;      storedTags[i, 0:2] = tempTags 
;;;      storedTags[i] = storedTagObj[i] -> get()
;    endfor
;  endelse
;  
;;??  obj_destroy, storedTagObj
  
  return, storedTagObj

end		;; DAVE_tagContainer::GetTag
	

;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::SetTag
;
; PURPOSE:
;	See also DAVE_tagContainer::GetTag.
;
;	Returns the tag object (class tagObj) for object stored at postion 
;	given by the keyword thisPosition.
;	Or it returns an array of tag objects when the keyword /all
;	is used. Note: the tag object contains information in addition
;	to just the tags, which is useful for use interface.
;-
;*
pro DAVE_tagContainer::setTag, $	; replaces tag object(s) 
			; used to label objects stored in this container.
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; keyword
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;------------- Use tagObj or the Keywords for tagbobj::set
	tagObj = thisTagObj, $	; New tag object
	;------------- Keywords for tagbobj::set
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $		;; Boolean. /annotate is OK.
	pendent = thisPendent		;; Will be C: constant, D: dependent, I: independent
;/
	
  infolabel =  "DAVE_tagContainer::SetTag >> "

;  print, infolabel, " position=", thisPosition
  
  if (n_elements(thisPosition) GT 0) then begin 
    localPosition = self -> convert2Position(thisPosition)
  endif
  
  leafNode = (self.tags) -> get( position=localPosition)

  if (n_elements(thisTagObj) GT 0) then begin
;    print, infolabel, "Keyword TagObj -> show"
;    thisTagObj -> show
    OK = thisTagObj -> get($
    	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent $	;; Will be C: constant, D: dependent, I: independent
	)
  endif 		;else begin
;    print, infolabel, " No keyword tagObj"
;  endelse
  

  OKfield = leafNode -> set( $
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent $	;; Will be C: constant, D: dependent, I: independent
	)


end		;; DAVE_tagContainer::SetTag
	

;=============================================================================
;+
; NAME:
; 	DAVE_tagContainer::getLabel
;
; PURPOSE:
;	Gets selected information about all the leaf nodes in the container.
; 	Arrays returned via optional keywords class, label, codeSource,
;	sizeString and count.
;-
;*
pro DAVE_tagContainer::GetLabel, $	
	; returns arrays of info for all objects in container.
  	argPresent = thisArgPresent, $	; Optional keyword, 
		; anon structure of boolean. For internal use. 
		; ARG_PRESENT() function does not work on 
   		; propagated keywords from other calling routines.
		; Needed for get* methods.
	;-------- use one or more of the following keywords ----
	class = thisClass, $		; Optional keyword, returns string array
					; of first string in tag ['','','']
	label =thisLabel, $		; Optional keyword, returns string array
					; for second string in tag ['','','']
	codeSource = thisCodeSource, $	; Optional keyword, returns string array
					; for third string in tag ['','','']
	sizeString = thisSizeString, $	; Optional keyword, returns string array
				; giving the size, based on rank and dimensions.
	count = thisCount		; Optional keyword, returns integer array
					; for # of objects in each leaf node.
;/

;	;----- use keyword position for indices or by tag ----------
;	position = thisPosition, $ ; 
;		; Integer scalar or Int Array. zero-based indices of the positions 
;	 	; or  String scalar or array of the tags.
;	rank = thisRank, $
;	dim = thisDim, $
;	all = thisAll, 	$ 	; Boolean keyword
	
  infolabel =  "DAVE_tagContainer::GetLabel >> "
  debug = 0

  if (n_elements(thisPosition) GT 0) then begin 
    localPosition = self -> convert2Position(thisPosition)
  endif

  ;------------------------------------------------------
  if (NOT arg_present(thisArgPresent)) then begin   
    ArgPresent = {$			; make this be anonymous structure
	class : arg_present(thisClass), $
	label : arg_present(thisLabel), $	
	codeSource : arg_present(thisCodeSource), $
	sizeString : arg_present(thisSizeString), $
	count : arg_present(thisCount) $
;	rank : arg_present(thisRank), $
;	dim : arg_present(thisDim) $
;	tags : arg_present(thisTags), $
;	hide : arg_present(thisHide), $
;	editable : arg_present(thisEditable), $
;	annotate : arg_present(thisAnnotate), $
;	everything: arg_present(thisEverything)
	}
  endif else begin
    ; ArgPresent had better include the appropriate tags.
    ArgPresent = thisArgPresent
  endelse
  ;------------------------------------------------------

  
  storedTagObj =  (self.tags) -> get(/all, count=ncount)
  if (ncount EQ 0) then begin
    message, /info, "Empty specObj? ncount = 0"
    return
  endif
  
  storedContentsObj =  (self.contents) -> get(/all)

  if (argpresent.class) then thisClass = make_array(ncount, /string)
  if (argpresent.label) then thisLabel = make_array(ncount, /string)
  if (argpresent.codesource) then thisCodeSource = make_array(ncount, /string)
  if (argpresent.sizeString) then thisSizeString = make_array(ncount, /string)
;  if (argpresent.rank) then thisLabel = make_array(ncount, /int)
;  if (argpresent.dim) then thisLabel = make_array(ncount, 4, /string)

  for i = 0, (ncount-1) do begin
     if (argpresent.class) then begin    
       OK =  storedTagObj[i] -> get(class = tempString)
       if (debug) then print, infolabel, i, OK, ' class'
       thisClass[i] = tempString
     endif
     if (argpresent.label) then begin    
       OK =  storedTagObj[i] -> get(label = tempString)
       if (debug) then print, infolabel, i, OK, ' label'
       thisLabel[i] = tempString
     endif
     if (argpresent.codeSource) then begin    
       OK =  storedTagObj[i] -> get(codeSource = tempString)
       if (debug) then print, infolabel, i, OK, ' codeSource'
       thisCodeSource[i] = tempString
     endif

     if (argpresent.sizeString) then begin    
       if (NOT obj_valid(storedContentsObj[i])) then begin
         message, /info, string(i, " Invalid object reference", $
		 storedContentsObj[i] )
         OK =  storedTagObj[i] -> get(class = classString, $
	 	label = labelString, codeSource = codeString)
	 message, /info, string("Assoc'ed tags = ", format_list($
	 	[classString, labelString, codeString]))         
       endif
       OK =  storedContentsObj[i] -> get(sizeString = tempString)
       if (debug) then print, infolabel, i, OK, ' sizeString'
       thisSizeString[i] = tempString
     endif
     
    endfor

   if (argPresent.count) then thisCount =   ncount
   
end		;; DAVE_tagContainer::Getlabel
	

;=======================================================================
;+
; NAME:
; 	DAVE_tagContainer::show
;
; PURPOSE:
;	Prints info to the screen or to a file unit number.
;-
;*
pro DAVE_tagContainer::show, $
  	lun=thisLun, $ ; Optionally, write to a file. If absent, to screen
	prefix = thisPrefix, $		; Optional keyword string.
	level = thisLevel, $		; Optional keyword. Integer. 
					; How deep in the show tree
	recursive = thisRecursive, $	; Boolean keyword
			; If keyword is set, then show nodes lower down in
			; the tree hierarchicy (if applicable). 
 	heapNum = thisHeapNum		; Boolean keyword.
			; Show the heap number via HELP for objects.
			; Useful when looking for memory leaks. 
;/

;;	showObj = thisShowObj, $	; Boolean Keyword
;DefaultNow	showClassSize = thisShowClassSize, $	; Boolean Keyword
;;;	maxLevel = thisMaxLevel		; Only show down so far in the tree.

;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (n_elements(thisLevel) EQ 0) then begin
    thisLevel = 0
  endif

  offset = ''
  for i = 1, thisLevel do begin
    offset = offset + '  '		; 2 spaces
  endfor

;Keep. 
  localPrefix = ''
  if (n_elements(thisPrefix) gt 0) then begin
    localPrefix = thisPrefix
  endif

  ncount = self -> count()

  localClassSize = ''
;  if (keyword_set(thisShowClassSize)) then begin
  	localClassSize = '{' +  obj_class(self) +  '}' +  $
 	'[' + strcompress(string(ncount), /remove_all) + ']'
; endif

;      myTagObj = (self.myTags) -> get(position = i)

;; Not needed. tag info passed using local prefix
;No    myTagString = ''
;   if obj_valid(self.myTags) then begin
;      OK = self.myTags -> get(tags=currentTag)
;      if (OK) then begin
;        myTagString = '.[' + currentTag[0] +  ', ' + $
;      		currentTag[1] +  ', ' +  currentTag[2] + ']= ' 
;      endif
;    endif  

  if (keyword_set(thisHeapNum)) then begin
      help, self, output = helpString
      printf, lun, helpString   
  endif
  printf, lun,$
;;;;	 strcompress(string((thisLevel)), /remove_all), ' ',  $
  	 offset, $
 	 localPrefix, $
;; Not needed. tag info passed using local prefix
;No	 myTagString, $
	 localClassSize			;, $

;  print, $
;  	 offset, $
;  	 localPrefix, $
;  	obj_class(self), '::', $
;;  	'<', (self.nodeStyle) -> get(), '> ', $
;;  	'{', (self.nodeTag) -> get(), '}', $
;	' tags= [[class,..], [label,...], [codeSource,...]] = '
;  print, self -> getTag(/all)
	
;  if (keyword_set(thisRecursive)) then begin
     for i = 0, (ncount -1) do begin

      tempObj = (self.contents) -> get(position = i)
      temp_ncount = tempObj -> count()
      tempTag = self -> convert2Tag(i)
      localPrefix = $
          string('.[', tempTag[0], ',', tempTag[1], ',',  tempTag[2], ']= ')

      localLevel = thisLevel + 1

      if (keyword_set(thisHeapNum)) then begin
    	help, tempObj, output = helpString
   	printf, lun, offset, localPrefix, '{DAVE_TAGCONTAINER}.(', $
		strcompress(string(i), /remove_all), ') ', helpString   
      endif

      if (keyword_set(thisRecursive)) then begin
        if (obj_valid(tempObj)) then begin
	  printf, lun, $ 
	 	offset, '  ', $		; extra offset.
	 	 localPrefix
          tempObj -> show, $
	  	lun = lun, $
		prefix = '  ', $
;Prettier?		prefix = localPrefix, $
		level= localLevel, $
;Default	      showClassSize = thisShowClassSize, $
	      recursive=thisRecursive
	endif else begin
   	  tempClassSize = '{' +  obj_class(tempObj) +  '}' + '[0]'
	  printf, lun, $ 
	 	offset, '  ', $		; extra offset.
	 	 localPrefix, $
		 tempClassSize 
	endelse
      endif else begin
;        temp_ncount = tempObj -> count()
   	tempClassSize = '{' +  obj_class(tempObj) +  '}' +  $
 	'[' + strcompress(string(temp_ncount), /remove_all) + ']'
	  printf, lun, $ 
	 	offset, '  ', $		; extra offset.
;	  	strcompress(string((thisLevel + 1)), /remove_all), ' ',  $
	 	 localPrefix, $
;No		 myTagString, $
		 tempClassSize 
      endelse
    endfor 
;  endif
  
;  if (keyword_set(thisRecursive)) then begin
;    for i = 0, (ncount -1) do begin
;      tempObj = (self.tags) -> get(position = i)
;;Wrong      tempTag = tempObj -> convert2Tag(i)
;      tempTag = self -> convert2Tag(i)
;      localPrefix = string(offset, '[', tempTag[0], ',',  $
;      	tempTag[1], ',',  tempTag[2], ']>> ')
;      tempObj -> show,  $
;        prefix = localPrefix, $
;;        prefix = tempTag, $
;      	level=(thisLevel+1), $
;	showObj=thisShowObj, /recursive
;    endfor
;  endif
  	
end

;================================================================================
;+
; NAME:
; 	DAVE_tagContainer::callFunction
;
; PURPOSE:
; 	Calls a function which MIGHT act on the self object, only via methods.
;  Functions are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The function's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	If used as a "friend" method, the second argument should probably be
; object itself. E.g., 
; 	result = myObject -> callFunction( 'SomeFunction', myObject, ...)
; Yes, that is a bit redundent, but means you can also call
; 	result = myObject -> callFunction( 'SomeFunction', anotherObject, ...)
; and have the function do NOTHING at all with myObject's data!
; This might be useful for widget objects.
;
;	The methods DAVE_tagContainer::callFunction, 
; DAVE_tagContainer::callProcedure and DAVE_tagContainer::callMethod 
; could be used to impliment an user interface which allows the user to 
; select functions, procedures and methods to apply. 
;
;-
;*
function DAVE_tagContainer::callFunction, $; returns whatever functionName returns.
  functionName, $	; A string.  don't include *.pro extension
  thisArg1, $		; Any type. Might be an object for "friend" methods
  thisArg2, $		; Any type. 
  thisArg3, $		; Any type.
  _EXTRA = extraExtra	; call by value keywords.
;/

  ; Calls a function which MIGHT act on the self object, only via methods.
  
  infolabel = "DAVE_tagContainer::callFunction >> "

  nparm = n_params() - 1	; Don't count function name!

  if (n_elements(extraExtra) gt 0) then begin
;DEBUG    print, infolabel, 'extraExtra =', extraExtra
    if (nparm eq 0) then begin
      ret = call_function( functionName, _EXTRA=extraExtra)
    endif
    if (nparm eq 1) then begin
      ret = call_function( functionName, thisArg1, _EXTRA=extraExtra)
    endif
    if (nparm eq 2) then begin
      ret = call_function( functionName, thisArg1, thisArg2, _EXTRA=extraExtra)
    endif
    if (nparm eq 3) then begin
      ret = call_function( functionName, thisArg1, thisArg2, thisArg3, $
      		 _EXTRA=extraExtra)
    endif
;    ret = call_function(functionName, _EXTRA=extraExtra)
;; 	self_tags		=self.tags, $		; optional
;;	self_contents		=self.contents, $	; optional
;;	_EXTRA=extraExtra)
  endif else begin
    if (nparm eq 0) then begin
      ret = call_function( functionName)
    endif
    if (nparm eq 1) then begin
      ret = call_function( functionName, thisArg1)
    endif
    if (nparm eq 2) then begin
      ret = call_function( functionName, thisArg1, thisArg2)
    endif
    if (nparm eq 3) then begin
      ret = call_function( functionName, thisArg1, thisArg2, thisArg3)
    endif
;    ret = call_function(functionName)
  endelse
  return, ret 
end		;; callFunction

;================================================================================
;+
; NAME:
; 	DAVE_tagContainer::callProcedure
;
; PURPOSE:
; 	Calls a procedure which MIGHT act on the self object, only via methods.
;  Procedures are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The procedures's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	If used as a "friend" method, the second argument should probably be
; object itself. E.g., 
; 	myObject -> callProcedure, 'SomeProcedure', myObject, ...
; Yes, that is a bit redundent, but means you can also call
; 	myObject -> callProcedure, 'SomeProcedure', anotherObject, ...
; and have the procedure do NOTHING at all with myObject's data!
; This might be useful for widget objects.
;
;	The methods DAVE_tagContainer::callFunction, 
; DAVE_tagContainer::callProcedure and
; DAVE_tagContainer::callMethod could be used to impliment an user interface 
; which allows the user to select functions, procedures and methods to
; apply. 
;
;-
;*
pro DAVE_tagContainer::callProcedure, $
  proName, $		; A string. don't include *.pro extension
  thisArg1, $		; Any type. Might be an object for "friend" methods.
  thisArg2, $		; Any type.
  thisArg3, $		; Any type.
  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
;/

;   arg1 = thisArg1, $
;   arg2 = thisArg2, $
;   arg3 = thisArg3, $
;   arg4 = thisArg4, $
;   arg5 = thisArg5, $
;   arg6 = thisArg6, $
;   arg7 = thisArg7, $

  ; Calls a procedure which MIGHT act on the self object, only via methods. 
  
  infolabel = "DAVE_tagContainer::callProcedure >> "

  nparm = n_params() - 1	; Don't count procedure name!

  if (n_elements(extraExtra) gt 0) then begin
;DEBUG    print, infolabel, 'extraExtra =', extraExtra
    if (nparm eq 0) then begin
      call_procedure, proName, _EXTRA=extraExtra
    endif
    if (nparm eq 1) then begin
      call_procedure, proName, thisArg1,  _EXTRA=extraExtra
    endif
    if (nparm eq 2) then begin
      call_procedure, proName, thisArg1, thisArg2,  _EXTRA=extraExtra
    endif
    if (nparm eq 3) then begin
      call_procedure, proName, thisArg1, thisArg2, thisArg3, _EXTRA=extraExtra
    endif
;    call_procedure, proName, _EXTRA=extraExtra
;;    	self_tags		=self.tags, $		; optional
;;	self_contents		=self.contents, $	; optional
;;	_EXTRA=extraExtra
  endif else begin
    if (nparm eq 0) then begin
      call_procedure, proName
    endif
    if (nparm eq 1) then begin
      call_procedure, proName, thisArg1
    endif
    if (nparm eq 2) then begin
      call_procedure, proName, thisArg1, thisArg2
    endif
    if (nparm eq 3) then begin
      call_procedure, proName, thisArg1, thisArg2, thisArg3
    endif
;    call_procedure, proName
;;    	self_tags		=self.tags, $		; optional
;;	self_contents		=self.contents
  endelse
  
  return
end		;; callProcedure

;================================================================================
;+
; NAME:
; 	DAVE_tagContainer::callMethod
;
; PURPOSE:
; 	Calls a method which acts on the "self" object. 
; If /isPro set, this function returns 1 (for true)
; If /isFunction is set, it returns the value that that function returns.
;
; 	Methods are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The method's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	The methods DAVE_tagContainer::callFunction, 
; DAVE_tagContainer::callProcedure and
; DAVE_tagContainer::callMethod could be used to impliment an user interface 
; which allows the user to select functions, procedures and methods to
; apply. 
;
;-
;*
function DAVE_tagContainer::callMethod, $	; if /isPro, returns 1.
			; if /isFunction, returns whatever methodName returns.
  methodName, $			; A string. Name of method to use.  
  	; The method is assumed to be a function unless otherwise indicated
	; with /isFunction. /isPro is provided for convenience.
  isFunction = thisIsFunction, $	; the default. 
  				; Indicates methodName has a return value.
  isPro = thisIsPro, $		; Indicates methodName has no return value.
  thisArg1, $			; Any type.
  thisArg2, $			; Any type.callMeth
  thisArg3, $			; Any type.
  _EXTRA = extraExtra		; call by value keywords. ONLY ONES CALL BY VALUE!
;/

  ; Calls a method which acts on the "self" object, which is selected
  ; from the choices thisObjRef, self.tags or self.contents or self.
  ; If /isPro set, this function returns 1 (for true)
  ; If /isFunction is set, it returns the value that that function returns.
    
    
  infolabel = "DAVE_tagContainer::callMethod >> " 
  hasReturnVal = 1
  if (keyword_set(thisIsFunction)) then hasReturnVal = 1
  if (keyword_set(thisIsPro)) then hasReturnVal = 0

  nparm = n_params() - 1	; Don't count method name!
  
;Debug  print, infolabel, "hasReturnVal =", hasReturnVal
;Debug  print, infolabel, "thisIsPro =", keyword_set(thisIsPro)
;Debug  print, infolabel, "thisIsFunction =", keyword_set(thisIsFunction)
;  print, infolabel, extraextra
  
  if (hasReturnVal) then begin
    ; default behavior
      if (n_elements(extraExtra) gt 0) then begin
        if (nparm eq 0) then begin
          ret = call_method(methodName,  self, _EXTRA=extraExtra)
	endif
        if (nparm eq 1) then begin
          ret = call_method(methodName,  self, thisArg1,_EXTRA=extraExtra)
	endif
        if (nparm eq 2) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, $
	  	_EXTRA=extraExtra)
	endif
        if (nparm eq 3) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, thisArg3, $
	  	_EXTRA=extraExtra)
	endif
      endif else begin
        if (nparm eq 0) then begin
          ret = call_method(methodName,  self)
	endif
        if (nparm eq 1) then begin
          ret = call_method(methodName,  self, thisArg1)
	endif
        if (nparm eq 2) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2)
	endif
        if (nparm eq 3) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, thisArg3)
	endif
      endelse
    return, ret 

  endif else begin

      if (n_elements(extraExtra) gt 0) then begin
        if (nparm eq 0) then begin
          call_method, methodName,  self, _EXTRA=extraExtra
	endif
        if (nparm eq 1) then begin
          call_method, methodName,  self, thisArg1,_EXTRA=extraExtra
	endif
        if (nparm eq 2) then begin
          call_method, methodName,  self, thisArg1, thisArg2, $
	  	_EXTRA=extraExtra
	endif
        if (nparm eq 3) then begin
          call_method, methodName,  self, thisArg1, thisArg2, thisArg3, $
	  	_EXTRA=extraExtra
	endif
      endif else begin
        if (nparm eq 0) then begin
          call_method, methodName,  self
	endif
        if (nparm eq 1) then begin
          call_method, methodName,  self, thisArg1
	endif
        if (nparm eq 2) then begin
          call_method, methodName,  self, thisArg1, thisArg2
	endif
        if (nparm eq 3) then begin
          call_method, methodName,  self, thisArg1, thisArg2, thisArg3
	endif
      endelse
    return, 1
    
  endelse
  
end		;; callMethod


;=======================================================================
pro DAVE_tagContainer::cleanup

  obj_destroy, (self.tags)
  obj_destroy, (self.contents)
;  obj_destroy, (self.nodeStyle)
;  obj_destroy, (self.nodeTag)

end

;=======================================================================
;*
pro DAVE_tagContainer__define
	; Both container classes will be the same length.
  define = {DAVE_tagContainer, $
	tags: obj_new(), $	; class tagObj 
	contents: obj_new(), $	; class IDL_Container (branching node) or 
				; class qtyObj (leaf node).
	myTags: obj_new() $	; class tagObj. Object reference of the tagObj
			; up one level in the tree heirarchy.
			; Void if the top of the tree.
	}
;/
end		;  DAVE_tagContainer__define
