; $Id: dave_undefine.pro,v 1.1 2003/08/06 17:21:04 tanya Exp $ 
;+
; NAME:
;       dave_undefine
;
; PURPOSE:
;	To make up to 8 existing variables undefined.
;	This is useful for optional passed keyword parameters in procedures,
;	functions and methods, where the behavior depends on which keywords
;	are used (tested via n_elements(). This is an alternative
;	to the "delete" command, which only works at the main program level.
;
; AUTHOR:
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;       DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED:
;	None.
;
; EXAMPLE
;       IDL> x = 3
;	IDL> y = 4
;	IDL> help, x, y
;	X               INT       =        3
;	Y               INT       =        4
;	IDL> DAVE_undefine, x, y, z
;	IDL> help, x
;	X               UNDEFINED = <Undefined>
;	IDL> myProcedure, keywdY=y, keywrdX=x	; Note: keywrdX is not used
;						; because x is undefined.
;
; MODIFICATION HISTORY:
;   Created July 16, 2003. Tanya Riseman
;   TMR. Aug 6, 2003. changed file name to all lowercase to help BUILD's RESOLVE_ALL
;
;-
;*
pro DAVE_undefine, $	; undefines scalar and vector variables. 
			; Not intended for pointers and objects.
	arg1, $		; If this is a defined variable, make it undefined.
	arg2, arg3, arg4, arg5, arg6, arg7, arg8 ; Ditto. optional arguments
;/	

 if (n_elements(arg1) GT 0) then begin
   junk =temporary(arg1)
 endif 

 if (n_elements(arg2) GT 0) then begin
   junk =temporary(arg2)
 endif 

 if (n_elements(arg3) GT 0) then begin
   junk =temporary(arg3)
 endif 

 if (n_elements(arg4) GT 0) then begin
   junk =temporary(arg4)
 endif 

 if (n_elements(arg5) GT 0) then begin
   junk =temporary(arg5)
 endif 

 if (n_elements(arg6) GT 0) then begin
   junk =temporary(arg6)
 endif 

 if (n_elements(arg7) GT 0) then begin
   junk =temporary(arg7)
 endif 

 if (n_elements(arg8) GT 0) then begin
   junk =temporary(arg8)
 endif 

end
