; $Id: daveptr_utils.pro,v 1.10 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
; 	davePtr_utils.pro
;
; PURPOSE:
; 	This is a collection of utilites that deal with the
; DAVE pointer data structure.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE utility routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created Jan 28-April 9, 2003. Tanya Riseman
;	Added routine davePtr_containsSpecObj. TMR Aug 5, 2003.
;	TMR. Aug 7, 2003. Removed routine davePtr_example.
;	TMR. Aug 7, 2003. Check for valid but empty pointer in davePtr_addTreatment
;	TMR. Aug 29, 2003. Updated filename handling in davePtr_restore.
;-


;================================================================================
;*
 function DavePtr_containsSpecObj, $ 	; returns 0/1 for false/true
 	DavePtr, $
	errMsg = thisErrMsg
;/

   if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = ''		; Null string for success
   endif

   if ( n_elements(DavePtr) EQ 0) then begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with (n_elements(DavePtr) EQ 0)"
     endif
     return, 0
   endif

   if ( n_elements(*DavePtr) EQ 0) then begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with (n_elements(*DavePtr) EQ 0)"
     endif
     return, 0
   endif

   if ( n_elements((*DavePtr).dataStrPtr ) EQ 0) then begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with (n_elements((*DavePtr).dataStrPtr ) EQ 0))"
     endif
     return, 0
   endif

   if (NOT ptr_valid((*DavePtr).dataStrPtr )) then begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with (NOT ptr_valid((*DavePtr).dataStrPtr ))"
     endif
     return, 0
   endif

   if ( n_elements((*(*DavePtr).dataStrPtr)) EQ 0) then begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with (n_elements((*(*DavePtr).dataStrPtr)) EQ 0)"
     endif
     return, 0
   endif
   

   if ( n_elements( ((*((*DavePtr).dataStrPtr)).SpecificPtr))  EQ 0) then  begin
;   if (n_elements( (*(*(*DavePtr).dataStrPtr).SpecificPtr)) EQ 0) then  begin
     if (n_elements(thisErrMsg) GT 0) then begin
;       thisErrMsg = "DavePtr_containsSpecObj failed with (n_elements( (*(*DavePtr).dataStrPtr).SpecificPtr) EQ 0)"
       thisErrMsg = "DavePtr_containsSpecObj failed with " + $
       	"( n_elements( (*((*((*DavePtr).dataStrPtr)).SpecificPtr))) EQ 0)"
     endif
     return, 0
   endif

;                       
   if (NOT ptr_valid( ((*((*DavePtr).dataStrPtr)).SpecificPtr) ) ) then  begin
;   if (NOT ptr_valid( (*(*(*DavePtr).dataStrPtr).SpecificPtr)) ) then  begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with " + $
       		 "(NOT ptr_valid( ((*((*DavePtr).dataStrPtr)).SpecificPtr)) )"
;       thisErrMsg = "DavePtr_containsSpecObj failed with (NOT ptr_valid((*(*(*DavePtr).dataStrPtr).SpecificPtr)) )"
     endif
     return, 0
   endif
   
   if (n_elements( (*((*((*DavePtr).dataStrPtr)).SpecificPtr))) EQ 0) then  begin
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with " + $
       		 "(n_elements( (*((*((*DavePtr).dataStrPtr)).SpecificPtr))) EQ 0)"
     endif
     return, 0
   endif
   
   SpecStrTags = tag_names( (*((*((*DavePtr).dataStrPtr)).SpecificPtr)) ) 
;   SpecStrTags = tag_names( (*(*(*DavePtr).dataStrPtr).SpecificPtr) )
   index1 = where( specStrTags EQ 'SPECOBJ',count)
   if (count NE 1) then begin
;     print, infolabel, "No specific object in SpecificPtr area.
     if (n_elements(thisErrMsg) GT 0) then begin
       thisErrMsg = "DavePtr_containsSpecObj failed with no specific object in SpecificPtr area"
     endif
     return, 0
   endif else begin
     return,  1	
   endelse

 end
 

;================================================================================
;*
 function DavePtr_getSpecObj, $ ; Returns an object of type specObj, if available
 	DavePtr, $
	from=thisFrom	; Optional string indicating the calling routine.
 
;/
   infolabel = "DavePtr_getSpecObj >> "
   debug = 0
;   debug = 1
   
   
   if (n_elements(thisFrom) EQ 0) then begin
     thisFrom = ''
   endif else begin
     thisFrom = '. From ' + thisFrom
   endelse

   theErrMsg = ''
   so_exists =  DavePtr_containsSpecObj( $
 	DavePtr, $
	errMsg = theErrMsg)
   
   if (NOT so_exists) then begin
     print, infolabel, string(theErrMsg, thisFrom) 
     return, obj_new()
   endif

   SpecStrTags = tag_names( (*((*((*DavePtr).dataStrPtr)).SpecificPtr)) ) 
   so = (*((*((*DavePtr).dataStrPtr)).SpecificPtr)).specobj
   if (obj_valid(so)) then begin
     return, so
   endif else begin
     message, /info, string("SpecObj exists, but is not a valid object.")
     return, obj_new() 
   endelse
   
;  ; Verify if it is an "empty" dave structure.
;  if (NOT ptr_valid( DavePtr)) then begin
;    message, /info, string("NULL DavePtr)", thisFrom)
;    return, obj_new()
;  endif
; 
;  if (n_elements( (* DavePtr)) EQ 0) then begin
;    message, /info,  string("empty DavePtr)", thisFrom)
;    return, obj_new()
;  endif
; 
  
;  if (ptr_valid( (((*((*DavePtr).dataStrPtr)).specificPtr)) )) then begin
;    if (n_elements( (*(((*((*DavePtr).dataStrPtr)).specificPtr))) )) then begin
;   	; It's an empty DavePtr.
;	message, /info, $
;	 string("empty (((*((*DavePtr).dataStrPtr)).specificPtr))", $
;	 thisFrom)
;       return, obj_new()
;    endif
;   
;;   if (ptr_valid( (*(*(*DavePtr).dataStrPtr).SpecificPtr)) ) then begin
;;     SpecStrTags = tag_names( (*(*(*DavePtr).dataStrPtr).SpecificPtr) )
;     if (debug) then begin
;       help, (((*((*DavePtr).dataStrPtr)).specificPtr))
;       help, (*(((*((*DavePtr).dataStrPtr)).specificPtr))) 
;       print, (*(((*((*DavePtr).dataStrPtr)).specificPtr))) 
;     endif
;     SpecStrTags = tag_names( (*(((*((*DavePtr).dataStrPtr)).specificPtr))) )     
;     index1 = where( specStrTags EQ 'SPECOBJ',count)
;     if (count NE 1) then begin
;       message, /info, string("No specific object in SpecificPtr area.", thisFrom)
;;       print, infolabel, "No specific object in SpecificPtr area."
;       return, obj_new()
;     endif else begin
;       return,  (*(((*((*DavePtr).dataStrPtr)).specificPtr)) ).specObj
;;       return,  (*(*(*DavePtr).dataStrPtr).SpecificPtr).specObj
;     endelse
;   endif else begin
;   	; It's an empty DavePtr.
;	message, /info, $
;	 string("NOT (ptr_valid((((*((*DavePtr).dataStrPtr)).specificPtr))))", $
;;	 string("NOT (ptr_valid((*(*(*DavePtr).dataStrPtr).SpecificPtr)))", $
;	 thisFrom)
;       return, obj_new()
;   endelse

 end
 

;================================================================================
;*
 pro DavePtr_nullSpecObj, $ ; Sets an null object of type specObj in the davePtr.
 	DavePtr, $
	from=thisFrom	; Optional string indicating the calling routine.
 
;/
   infolabel = "DavePtr_nullSpecObj >> "
   debug = 0
;   debug = 1
   
   
   if (n_elements(thisFrom) EQ 0) then begin
     thisFrom = ''
   endif else begin
     thisFrom = '. From ' + thisFrom
   endelse

   theErrMsg = ''
   so_exists =  DavePtr_containsSpecObj( $
 	DavePtr, $
	errMsg = theErrMsg)
   
   if (NOT so_exists) then begin
     print, infolabel, string(theErrMsg, thisFrom) 
   endif

   (*((*((*DavePtr).dataStrPtr)).SpecificPtr)).specobj = obj_new() ;null object
    
 end
 

;================================================================================
;*
 pro DavePtr_restore, $
   ; Note: this means the callling routine can give the dave Pointer
   ; any name it wants. Assume that it is saved with the name davePtr.
 	origDavePtr, $		; Ptr ID for dave data structure.
 	filename, $		; string
	ownerDavePtr = thisownerDavePtr ; Optional keyword. 'local' or 'main'. 
				; Defaults to 'main'
;/

  debug = 0
;  debug = 1
    
  if (NOT ptr_valid(origDavePtr)) then begin
    aString = "PROGRAMMER ERROR. DavePtr has become invalid somehow"
    message, /info, aString 
;    dialog_message, /info, aString
    return
  endif
  
  if (n_elements(filename) eq 0) then begin
;    print,  infolabel, $
;    	' No file keyword provided. Writing to daveptr.dave.'
;     message, /info, 'No file keyword provided. Attempting to restore daveptr.dave.'
;     filename = 'daveptr.dave'
    aString = "PROGRAMMER ERROR. No file keyword provided."
    message, /info, aString 
    return
  endif
 
   ; DavePtr returns filled properly
   restore, filename 		; './tests/cehex011_bt7.dave'
   
   (*origDavePtr) = (*davePtr)	; Ensure that the original ID is used.
   ptr_free, davePtr

   ; If the davePtr contains a specObj, update the davePtr ID there as well.
   ; TMR. Aug 20, 2003.
   tempSpecObj = davePtr_getSpecObj(origDavePtr, from='DavePtr_restore')
   if (obj_valid(tempSpecObj)) then  begin;
      if (n_elements(thisownerDavePtr) GT 0) then begin
        tempSpecObj -> SetownerDavePtr, thisownerDavePtr
      endif else begin
        tempSpecObj -> SetownerDavePtr, 'main'
      endelse

;     OK = (tempSpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
;     	qty=filename)
     tempFieldObj = tempSpecObj -> get( position=['*', 'so_filename', '*'], $
     	/inHeader)
     if (NOT obj_valid(tempFieldObj)) then begin
       max_filename_len = 0
       max_filename_len = max_filename_len > strlen(filename) ; keep track of length of filename
       pos = strpos(filename,!delimiter,/reverse_search) + 1
       length = strlen(filename) - pos
       short_filename = strmid(filename,pos,length)
       short_path = strmid(filename,0, (pos-1))
      (tempSpecObj) -> add, tags=['*', 'so_path', '*'], $
       		/inHeader, $
		contents = obj_new("qtyObj", qty=short_path) 
       (tempSpecObj) -> add, tags=['*', 'so_filename', '*'], $
       		/inHeader, $
		contents = obj_new("qtyObj", qty=short_filename) 
     endif
    (tempSpecObj) -> setDavePtrID, origDavePtr, /restore
    
   endif 


   if (debug) then begin   
     message, /info, string("DavePtr restored. ", $
   	"obj_valid(davePtr_getSpecObj())=", obj_valid(tempSpecObj) )
     daveptr_show, origDavePtr   
     if (obj_valid(tempSpecObj)) then  begin
       message, /info, string("restored DavePtr contains specObj. ID=", $
       		tempSpecObj)
       tempSpecObj -> show  ;, /recursive     
     endif
     
   endif


     
 end

;;================================================================================
;;*
; pro DavePtr_restore, DavePtr, $
; 	filename		; string
;   ; Note: this means the callling routine can give the dave Pointer
;   ; any name it wants. Assume that it is saved with the name davePtr.
;;/
;   
;  if (n_elements(filename) eq 0) then begin
;;    print,  infolabel, $
;;    	' No file keyword provided. Writing to daveptr.dave.'
;    filename = 'daveptr.dave'
;  endif
; 
;   ; DavePtr returns filled properly
;   restore, filename 		; './tests/cehex011_bt7.dave'
;   
;   
; end
 
;================================================================================
;*
 pro davePtr_save, DavePtr,$
   varName = thisVarName, $ ; Optional variable name. Defaults to 'DavePtr'
   file = thisFile	; Defaults to 'daveptr.dave'
; saves a DAVE pointer.
;/


  infolabel = "davePtr_save >> "  

  debug = 0
  
  if (n_elements(DavePtr) eq 0) then begin
    print,  infolabel, $
    	' Require a single DAVE pointer as input parameter'
    return
  endif
   
  if (debug) then print, infolabel

  if (n_elements(thisFile) eq 0) then begin
    print,  infolabel, $
    	' No file keyword provided. Writing to daveptr.dave.'
    thisFile = 'daveptr.dave'
  endif
  
  if (n_elements(thisVarName) EQ 0) then  begin
    thisVarName  = 'DavePtr'
    aString = "davePtr_save, DavePtr, file ='" +   thisFile + "'"
    davePtr_addTreatment, DavePtr, aString
    save, davePtr, file=thisFile
  endif else begin
    aString = "davePtr_save, " + thisVarName + ", file ='" + thisFile + "'"
    davePtr_addTreatment, DavePtr, aString
    ; now rename the variable davePtr
    rename_string = thisVarName + ' = ptr_new(*DavePtr)'
    OK = execute(rename_String)
    if (not OK) then print, 'Problem with command execute(', rename_string, ')'    
    
    exString = 'save, '+ thisVarName + ", file='" + thisFile +  "'"
    if (debug) then begin
      print, infolabel, 'command Execute(', string('help, ', thisVarName), ')'
      ok = execute(string('help, ', thisVarName))
      print, infolabel, 'command Execute(', $
      	string('help, *(', thisVarName,')') ,  ')'
      ok = execute(string('help, *(', thisVarName,')') )
      print, infolabel, 'command Execute(', exString, ')'
    endif
    OK = execute(exString)
    if (not OK) then begin
      print, infolabel, 'Execute failed on string:"', exString, "'"
    endif
    ok = execute(string('ptr_free, ', thisVarName) )
    if (not OK) then begin
      print, 'Problem with command execute(',$
    	string('ptr_free, ', thisVarName), ')'    
    endif
  endelse
end		; davePtr_save
;================================================================================
;*
 function davePtr_init, $
   DavePtr,$
   varName = thisVarName, $ ; Optional variable name. Defaults to 'DavePtr'
   file = thisFile, $	; Required. ICP triple axis data file to read in
   new = thisNew, $	; Optional Boolean keyword. Creates a new DavePtr.
   			; If not present, the current DavePtr contents are
			; cleaned out.
  ;-------------------------
  tag_descri = thistag_descri, $ ; For (*davePtr).descriPtr
  tag_descrLegend = thistag_descrLegend; For (*davePtr).descriPtr
;/

  ;-------------------------
;  ; Keywords passed on to icp_data::init:
;  tag_x = thisTag_x, $
;  tag_y = thisTag_y, $
;  tag_z = thisTag_z, $
;  onlyHeader = thisOnlyHeader, $ ; Read in only the header portion of the file.
;  noConstruct = thisNoConstruct, $; Don't construct columns of data 
;  				; from header info. The default is construct.
;  noPrune = thisNoPrune		; Don't prune constant (or any!) columns of data 
;	  			; The default is to prune.
  ;-------------------------
   
;  Creates or writes over a DAVE pointer, reading in data from an ICP 
; triple axis data file.
; Returns 0/1 for failure/success.

  infolabel = "davePtr_init >> "  
  debug = 0
  
  if ((n_elements(DavePtr) eq 0) and (not keyword_set(thisNew)) ) then begin
    print,  infolabel, $
    	' Require a single DAVE pointer as input parameter'
    return, 0
  endif
   
  if (debug) then print, infolabel

  if (n_elements(thisFile) eq 0) then begin
    print,  infolabel, $
    	' No file keyword provided. Need file to read data from'
    return, 0
  endif

  ;--------------------------------------------------
  success = 1
  dataObj = obj_new('icp_data', thisFile) 
;  	tag_x = thisTag_x, tag_y = thisTag_y, tag_z = thisTag_z, $
;  	onlyHeader = thisOnlyHeader, noConstruct = thisNoConstruct, $
;	noPrune = thisNoPrune )
;  if (debug) then begin
;    help, dataObj
;    dataObj -> show
;  endif
  if (not obj_valid(dataObj)) then begin
    emsg = infolabel +' Invalid object dataObj supposed to be made form file'+$
    	'"' + thisFile+  '"'
    goto, problem_xyz
  endif
  if (n_elements(thisTag_descri)  gt 0) then begin
    if (debug) then print, infolabel, 'thisTag_descri=', thisTag_descri
    OK = dataObj ->  set (tag_descri = thisTag_descri)
    if (not OK) then begin
      emsg = infolabel + 'Problem with dataObj->set(tag_descri=thisTag_descri)'
      goto, problem_xyz
    endif
  endif
  if (n_elements(thisTag_descriLegend)  gt 0) then begin
    if (debug) then print, infolabel, 'thisTag_descrLegend=', thisTag_descrLegend
    OK = OK and dataObj ->  set (tag_descrLegend = thisTag_descrLegend)
    if (not OK) then begin
      emsg = infolabel + $
      	'Problem with dataObj->set(tag_descrLegend = thisTag_descrLegend)'
      goto, problem_xyz
    endif
  endif
  
  ; Don't destroy dataObj at the the end of this routine because
  ; it is stored in the DavePtr.
  ;--------------------------------------------------
  ; structures
;  if (keyword_set(thisNew) then begin
    commonStr = {instrument:'Triple_axis_patchup', $
  	histPtr:ptr_new(), $
	xlabel:'', $
	ylabel:'', $
	xType:'points', $
	yType:'points', $
	xunits:'', $
	yunits:'', $
	histLabel:'', $	;z
	histUnits:'', $ ;z
	treatmentPtr:ptr_new(), $	; treatment history. string array.
	commonStrPtr:ptr_new()}		; used for linked lists.
;  endif
  
  if (debug) then dataObj-> show, /tags
  
  ;----- prepare the x tag ----------------------------
  OK = dataObj-> get(tag_x = onexcol, tag_y = oneycol, tag_z = onezcol)
  if (NOT OK) then begin
    emsg = infolabel + 'tags not found '
    print, infolabel,'tag_x,y,z=',onexcol,', ',oneycol,', ',onezcol
    goto, problem_xyz
  endif
  if (debug) then print, infolabel,'tag_x,y,z=',onexcol,', ',oneycol,', ',onezcol
  ok = OK and dataObj-> get(colLabel=onexcol, labels=oneLabel, units=oneunits, $
  	qty_array=oneQty, $
	n_cols = thisN_cols, n_rows = thisN_rows, $
	tname=oneTname)
  if (NOT OK) then begin
    emsg = infolabel + 'no x data found at all '
    goto, problem_xyz
  endif
  if (thisN_rows gt 0) then begin
    emsg = infolabel + 'Can not deal with 2-D data presently. size= ' + $
    	thisN_cols + thisN_rows
    goto, problem_xyz
  endif
  if (thisN_cols le 0) then begin
    emsg = infolabel + 'Can not have scalar data. size=' + thisN_cols
    goto, problem_xyz
  endif

  ;----------------------------------------------
  ; Create histogram sized correctly.
  if (oneycol eq '') then begin		; 1D data
    histStr = {qty:fltarr(thisN_cols), $ 
           err:fltarr(thisN_cols), $ 
           x:fltarr(thisN_cols), $
	   y: 0.0 }			; just so that the tag appears.

  endif else begin	; 2D data. This won't work with DaveDat2D.
    histStr = {qty:fltarr(thisN_cols), $ 
           err:fltarr(thisN_cols), $ 
           x:fltarr(thisN_cols) , $
           y:fltarr(thisN_cols)}
  endelse

  ;----------------------------------------------
  if (oneTname NE 'FLOAT') then begin
    emsg = infolabel + 'x variable is not type FLOAT: ' + oneTname
    goto, problem_xyz
  endif else begin
    if (n_elements(oneQty) gt 0) then begin
      histStr.x = oneQty
      commonStr.xLabel = oneLabel
      commonStr.xUnits = oneUnits
    endif else begin
      emsg = infolabel + 'no x data: '
      goto, problem_xyz
    endelse
  endelse
  
   ;---------------------------------------------------------------------
  ; Y is not required.
  OK = dataObj -> get(colLabel=oneycol, labels=oneLabel, units=oneunits, $
  	qty_array=oneQty, tname=oneTname)
  if (OK) then begin
    if (oneTname NE 'FLOAT') then begin
      emsg = infolabel + 'y variable is not type FLOAT: ' + oneTname
      goto, problem_xyz
    endif else begin
      if (n_elements(oneQty) gt 0) then begin
        histStr.y = oneQty
        commonStr.yLabel = oneLabel
        commonStr.yUnits = oneUnits
      endif else begin
        histStr.y = 0.0		; Stupid value when Y is not used.        
      endelse
    endelse
  endif
  
  ;---------------------------------------------------------------------
  ; Fill up the information. z=hist is required. 
  ; Is 2D if y is present, 1D otherwise.
  ok = dataObj -> get(colLabel=onezcol, labels=oneLabel, units=oneunits, $
  	qty_array=oneQty, err_array=oneErr, tname=oneTname)
  if (NOT OK) then begin
    emsg = infolabel + 'no HIST=z data found at all '
    goto, problem_xyz
  endif
  if (oneTname NE 'FLOAT') then begin
    emsg = infolabel + 'HIST=z variable is not type FLOAT: ' + oneTname
    goto, problem_xyz
  endif else begin
    if (n_elements(oneQty) gt 0) then begin
      histStr.qty = oneQty
      commonStr.histLabel = oneLabel
      commonStr.histUnits = oneUnits
      if (n_elements(oneErr) gt 0) then begin
        histStr.err = oneErr
      endif
    endif else begin
      emsg = infolabel + 'no HIST=z data: '
      goto, problem_xyz
    endelse
  endelse

;------------------------------------
 commonStr.histPtr = ptr_new(histStr, /no_copy) 
 if (( not keyword_set(thisNew)) and (not ptr_valid(DavePtr)) ) then begin
   print, infolabel, 'DavePtr_init called with invalid pointer DavePtr ', $
   	'and no /new keyword'
   print, infolabel, ' So setting keyword new=1.'
   print, infolabel, ' This will cause problems with the main DAVE program.'
   thisNew = 1
 endif

;---------------------------------
  goto, no_problem_xyz

  problem_xyz:
  print, emsg
  commonStr.histPtr = ptr_new() 
 
  no_problem_xyz:
;---------------------------------
  
 
 if (keyword_set(thisNew)) then begin
   dataStr = {commonStr:commonStr, $
  	specificPtr:ptr_new(dataObj)}
;   if (debug) then begin
;     help, dataStr
;     print, infolabel, dataStr
;   endif
   dataStrPtr = ptr_new(dataStr)
 
   DaveStr = {dataStrPtr:dataStrPtr, $
  	 descriPtr:ptr_new()}
   DavePtr = ptr_new(daveStr)


 endif else begin	; if (keyword_set(thisNew)) 
   ;-------------------------------------------------------------------------
   ; Points to a structure of strings and arrays
    ptr_free, (*(*DavePtr).dataStrPtr).commonStr.HistPtr
   ; Points to string array
    ptr_free, (*(*DavePtr).dataStrPtr).commonStr.TreatmentPtr
   ; points to a pointer for a linked list?
    ptr_free, (*(*DavePtr).dataStrPtr).commonStr.commonStrPtr

   ;.... load in the new values
   (*(*DavePtr).dataStrPtr).commonStr = commonStr

    ;.........................................................
    if (ptr_valid( $
            (*(*DavePtr).dataStrPtr).SpecificPtr )) then begin
       if ( obj_valid( *(*(*DavePtr).dataStrPtr).specificPtr) ) then begin
   	  obj_destroy, *(*(*DavePtr).dataStrPtr).specificPtr 
       endif
       ptr_free,  (*(*DavePtr).dataStrPtr).SpecificPtr 
    endif
    if (debug) then print, infolabel,  ' Laoding the specificPtr'
    (*(*DavePtr).dataStrPtr).specificPtr = ptr_new(dataObj)

    ;.........................................................
    ptr_free, (*DavePtr).descriPtr

 endelse	; if (keyword_set(thisNew)) 

  ;------------------------------------------------
  ; Initialize the descrip structure pointer if the info has been provided.
  if (n_elements(thisTag_descri) gt 0) then begin
   ok = dataObj->get(field=thistag_descri, legend_array=oneLegend, n_cols=thisn_cols)
  endif
  if (n_elements(oneLegend) eq 0) then oneLegend = ''
  if (n_elements(thisTag_descrLegend) EQ 0) then begin
    thistag_descrLegend = ''
  endif
  if (thistag_descrLegend NE '') then begin
    index1 = where( StrUpCase(thistag_descrLegend) EQ StrUpCase(onelegend),count)
    if (count eq 0) then begin	; Failure to find a match
      emsg = infolabel + 'Failed to find match for legend ' $
      		+ thistag_descrLegend
      print, emsg
      ;;; goto, problem_descri 
     (*davePtr).descriPtr = ptr_new(); invalid pointer 
    endif else begin
      iwhere = index1[0]
      ok = dataObj -> get(field=thistag_descri, oneRec = thisOneRec, icol=iwhere,irow=0)
     (*davePtr).descriPtr = ptr_new(thisoneRec)
    endelse
  endif else begin
    ok = dataObj -> get(field=thistag_descri, oneRec = thisOneRec, icol=0, irow=0)
     (*davePtr).descriPtr = ptr_new(thisoneRec)
  endelse

 ;.........................................................
 ;.... initialize the treatment pointer.
  if (n_elements(thisVarName) EQ 0) then  begin
    thisVarName  = 'DavePtr'
  endif
   astring = "OK = davePtr_init("+ thisVarName + ", file='" + thisFile + "'"
;   astring = "OK = davePtr_init(davePtr, file='" + thisFile + "'"
   if (n_elements(thistag_descri) gt 0) then begin
     astring = astring + ", tag_descri='" + thistag_descri + "'"
   endif
   if (n_elements(thistag_descrLegend) gt 0) then begin
     astring = astring + ", tag_descrLegend='" + thistag_descrLegend + "'"
   endif
   if (keyword_set(thisNew)) then begin
     astring = astring + ', /new)'
   endif else begin
     astring = astring + ')'
   endelse
   davePtr_addTreatment, DavePtr, aString
;;;;   (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr = ptr_new(astring)
   
;------------------------------------
 
  return, success

 ;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; Moved
;  problem_xyz:
;  print, emsg
;
;  obj_destroy, dataObj
;  return, 0		; invalid pointer

  
end		; davePtr_init

;================================================================================
;*
 pro davePtr_addTreatment, thisDavePtr,$
   strArray
;/
  infolabel = "davePtr_addtreatment >> "  
;  debug = 1  
;  if (debug) then begin
;    print, infolabel
;    help, thisdaveptr
;  endif

   allDone = 0  
   if (not ptr_valid((*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr)) $
   	then begin
     ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr
    (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr = ptr_new(strArray)    
    allDone = 1
    
   endif			; Now we have a valid pointer.

   if (NOT allDone) then begin
     if (n_elements((*(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr)) EQ 0) $
   	then begin
       ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr
      (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr = ptr_new(strArray)    
      allDone = 1
     endif
   endif			; Now we have a valid pointer with contents.
      
;   if (not ptr_valid((*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr)) $
;   	then begin
;     ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr
;    (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr = ptr_new(strArray)    
;   endif else begin
   if (NOT allDone) then begin
      holder_ptr = ptr_new( $
      	*((*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr) )
      ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr
      ; Array concatenation
      (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr = ptr_new($
      	[*(holder_ptr), strArray])
      ptr_free, holder_ptr 
      allDone = 1
;   endelse 
   endif			; Already have a pointer with contents.

 end		; pro davePtr_addTreatment


;================================================================================
;*
 pro davePtr_show, thisDavePtr,$
   varName = thisVarName, $ ; Optional variable name. Defaults to 'DavePtr'
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   file = thisFile, $	; Optionally, write to a file. If absent, to screen
   partial=thisPartial, $
   full=thisFull
; Shows a DAVE pointer and what's beneath it.
; Shows the contents in the same order that it is cleanup in the 
; proceedure davePtr_cleanup. Might seem a bit backwards.
;/

  infolabel = "davePtr_show >> "  

; From Help documentation:
;
;Set this keyword to display information about file units.
;If no arguments are supplied in the call to HELP, information 
;on all open file units (except the special units 0, -1, and -2) is displayed. 
;If arguments are provided, they are taken to be integer file unit numbers,
;and information on the specified file units is given.
;
;For example, the command:
;
;HELP, /FILES, -2, -1, 0
;
;gives information below about the default file units:
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
;
;The attributes column tells about the characteristics of the file. 
; For instance, the file connected to logical file unit 2 is called 
; "stderr" and is the standard error file. It is opened for write access 
;(Write), is a new file (Truncate), is a terminal (Tty), and cannot be 
;closed by the CLOSE command (Reserved).
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>  if (n_params() NE 1) then begin

  debug = 0
  
  if (n_elements(thisLun) eq 0) then begin
    if (n_elements(thisFile) eq 0) then begin
      lun= -1
    endif else begin
      openw, lun, thisFile, /get_lun     
      message, /info, string("Opening ", lun, " ", thisFile)
    endelse
  endif else begin
    lun= thisLun
    if (n_elements(thisFile) NE 0) then begin
      openw, lun, thisFile   
      message, /info, string("Opening ", lun, " ", thisFile)
    endif 
  endelse

  message, /info, string("lun=", lun)

  if (n_elements(thisVarName) EQ 0) then  begin
    VarName  = 'DavePtr'
  endif else begin
    varName = thisVarName
  endelse

  if (n_elements(thisDavePtr) eq 0) then begin
    printf, lun, infolabel, ' Requires "', VarName, $
		'"', ' to be a single DAVE pointer as input parameter'
    printf, lun
    goto, abort_routine
;    return
  endif
   
  if (not ptr_valid(thisDavePtr)) then begin
    printf, lun, infolabel, ' Requires "', VarName, $
    		" ID=", thisDavePtr, $
		'"', ' to be a valid DAVE pointer as input parameter'
    printf, lun
    goto, abort_routine
;    return
  endif
   
  if (n_elements(*thisDavePtr) EQ 0) then begin
    printf, lun, infolabel, ' Requires "', VarName, $
    		" ID=", thisDavePtr, $
		'"', ' to be a filled DAVE with pointer as input parameter'
    printf, lun
    goto, abort_routine
;    return
  endif
   
;  printf, lun
  if (debug) then printf, lun, infolabel, 'Entering. lun=', lun
  
;..............................................................................
; tags1 = tag_names((*thisDavePtr))
; printf, lun, 

; help, output=txtarray, (*thisDavePtr)
; printf, lun, txtarray

; subVarName = "(*" + varName + ")"
 singleton_show, (thisDavePtr), varName, /partial, lun=lun
 printf, lun
;; if (!debug) then printf, lun, '------------------------'
 
 tname = size((*thisDavePtr), /tname)
 top_tags = tag_names((*thisDavePtr) )

; printf, lun, '(*thisDavePtr) ='
; help, output=txtarray, $
;    	(*thisDavePtr)
; printf, lun, txtarray 
; printf, lun
    
 subVarName = "(*" + varName + ")"
 if (tname EQ 'STRUCT') then begin
   structure_show, (*thisDavePtr), subVarName, $		;; '(*DavePtr)', $
   	lun=lun, $
;   	/full, $	; one field per line
	/partial
;   full=thisFull, partial=thisPartial, $
	 	 
   printf, lun
 endif else begin
    singleton_show, lun=lun, /partial, (*thisDavePtr), subVarName ;; '(*thisDavePtr)'
    printf, lun, infolabel, ' Requires ', subVarName,  $
    	' to be a STRUCTURE'
    printf, lun
    goto, abort_routine
;    return
 endelse

; if (!debug) then printf, lun, '------------------------'
 ;==================================================

 subVarName = "((*" + varName + ").dataStrPtr)"
 tname = size( ((*thisDavePtr).dataStrPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = ptr_valid( ((*thisDavePtr).dataStrPtr) )
   if (OK) then begin
     if (n_elements( (*((*thisDavePtr).dataStrPtr)) ) EQ 0) then begin
       OK = 0
     endif
   endif
 endif 
 if (NOT OK) then begin
    if (!debug) then begin
      printf, lun, infolabel, ' Requires ', subVarName, $ ;;;'((*DavePtr).dataStrPtr)', $
    	' to be a valid POINTER with contents'
      printf, lun
    endif
     goto, skip_dataStrPtr
;    return
 endif
 singleton_show, ((*thisDavePtr).dataStrPtr), subVarName, $ ;;, '((*DavePtr).dataStrPtr)', $
 	/partial, lun=lun
 printf, lun

 
; if (!debug) then printf, lun, '------------------------'

 subVarName =  '(*((*' + varName + ').dataStrPtr))'
 tname = size( (*((*thisDavePtr).dataStrPtr)), /tname)
 OK = 0
 if (tname EQ 'STRUCT') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subvarname, $;; (*((*DavePtr).dataStrPtr))', $
    	' to be a valid STRUCTURE '
    printf, lun
  endif
    goto,  skip_datastrptr
    ; return
 endif
 structure_show, (*((*thisDavePtr).dataStrPtr)) , subvarname, $	;;'(*((*DavePtr).dataStrPtr))',$
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'

 subVarName =  '((*((*' + varName + ').dataStrPtr)).commonStr)'
 
 tname = size( ((*((*thisDavePtr).dataStrPtr)).commonStr), /tname)
 OK = 0
 if (tname EQ 'STRUCT') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subvarname, $ ;;((*((*DavePtr).dataStrPtr)).commonStr)', $
    	' to be a valid STRUCTURE '
    printf, lun
  endif
    goto,  skip_datastrptr
;    return
 endif
 structure_show, ((*((*thisDavePtr).dataStrPtr)).commonStr) , subVarName, $
; 	'((*((*DavePtr).dataStrPtr)).commonStr)',$
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'

 subVarName = '((*((*' + varName + ').dataStrPtr)).commonStr.histPtr)' 
 tname = size( ((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = ptr_valid( ((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr) )
   if (OK) then begin
     if (n_elements( ((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr) ) EQ 0) then begin
       OK = 0
     endif
   endif
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subVarName, $ ;;'((*((*DavePtr).dataStrPtr)).commonStr.histPtr)', $
    	' to be a valid POINTER  with contents'
    printf, lun
  endif
    goto,  skip_datastrptr
;    return
 endif
 singleton_show, ((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr) , subVarName, $ $
; 	'((*((*DavePtr).dataStrPtr)).commonStr.histPtr)',$
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'

;=========================================================================
 subVarName = '(*((*((*' + varName + ').dataStrPtr)).commonStr.histPtr))' 
 tname = size( (*((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr)), /tname)
 OK = 0
 if (tname EQ 'STRUCT') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subVarName, $ 
    	' to be a valid STRUCTURE '
    help, (*((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr))
    printf, lun
  endif
    goto,  skip_datastrptr
 endif
 structure_show, (*((*((*thisDavePtr).dataStrPtr)).commonStr.histPtr)) , subVarName, $ $
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'
;=========================================================================
 subVarName = '((*((*' + varName + ').dataStrPtr)).commonStr.treatmentPtr)'
 tname = size( ((*((*thisDavePtr).dataStrPtr)).commonStr.treatmentPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subVarName, $;;;((*((*DavePtr).dataStrPtr)).commonStr.treatmentPtr)', $
    	' to be a valid POINTER '
    printf, lun
  endif
    goto,  skip_datastrptr
;    return
 endif
 singleton_show, ((*((*thisDavePtr).dataStrPtr)).commonStr.treatmentPtr) , subVarName, $ $
; 	'((*((*DavePtr).dataStrPtr)).commonStr.treatmentPtr)',$
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'


 ;==============================================================================
; if (!debug) then printf, lun, '------------------------'
 subVarName = '((*((*' + varName + ').dataStrPtr)).commonStr.commonStrPtr)'
 tname = size( ((*((*thisDavePtr).dataStrPtr)).commonStr.commonStrPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subVarName, $;;;((*((*DavePtr).dataStrPtr)).commonStr.commonStrPtr)', $
    	' to be a valid POINTER '
    printf, lun
  endif
    goto,  skip_datastrptr
;    return
 endif
 singleton_show, ((*((*thisDavePtr).dataStrPtr)).commonStr.commonStrPtr) , subVarName, $
; 	'((*((*DavePtr).dataStrPtr)).commonStr.commonStrPtr)',$
 	/partial, lun=lun

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'

 ;============================================================================
 subVarName = '((*((*' + varName + ').dataStrPtr)).specificPtr)'
 tname = size( ((*((*thisDavePtr).dataStrPtr)).specificPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = ptr_valid( ((*((*thisDavePtr).dataStrPtr)).specificPtr) )
   if (OK) then begin
     if (n_elements( ((*((*thisDavePtr).dataStrPtr)).specificPtr) ) EQ 0) then begin
       OK = 0
     endif
   endif
 endif 
 if (NOT OK) then begin
  if (!debug) then begin
    printf, lun, infolabel, ' Requires ', subvarName, $	;;((*((*DavePtr).dataStrPtr)).specificPtr)', $
    	' to be a valid POINTER with contents'
    printf, lun
  endif
    goto,  skip_specificPtr
;    return
 endif
 
 wasNull = 0
 singleton_show, ((*((*thisDavePtr).dataStrPtr)).specificPtr) , subVarName, $
; 	'((*((*DavePtr).dataStrPtr)).specificPtr)',$
 	/partial, lun=lun, isNull=wasNull

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'

 ;============================================================================
 if (wasNULL) then goto, skip_specificPtr
 
 ;============================================================================
 subVarName = '(*(((*((*' + varName + ').dataStrPtr)).specificPtr)))'
; help, (*(((*((*thisDavePtr).dataStrPtr)).specificPtr)))
 tname = size( (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))), /tname)
 OK = 0
 if (tname EQ 'STRUCT') then begin
   OK = 1
 endif 
 if (NOT OK) then begin 
  if (!debug) then begin
    help, (*(((*((*thisDavePtr).dataStrPtr)).specificPtr)))
    printf, lun, infolabel, ' Requires ', subVarName, $ 
    	' to be a valid STRUCTURE '
    printf, lun
  endif
    goto,  skip_specificPtr
;    return
 endif
 structure_show, (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))) , subVarName, $
 	/partial, lun=lun	;;;, isNull=wasNull

 printf, lun

  ; if (!debug) then printf, lun, '------------------------' 
 
 ;============================================================================
 specTags = tag_names(  (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))) )
 index1 = where( (specTags EQ 'SPECOBJ') ,count)
 if (count EQ 1) then begin
   subVarName = '(*(((*((*' + varName + ').dataStrPtr)).specificPtr))).specObj'
   tname = size( (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))).specObj, /tname)
   OK = 0
   if (tname EQ 'OBJREF') then begin
     OK = obj_valid( (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))).specObj )
   endif
   if (NOT OK) then begin
    if (!debug) then begin
      printf, lun, infolabel, ' Requires ', subvarName, $
    	' to be a valid OBJREF of class SPECOBJ'
      printf, lun
    endif
      goto,  skip_specificPtr
   endif
 
   wasNull = 0
   singleton_show, (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))).specObj,$
   	 subVarName, $
 	/partial, lun=lun, isNull=wasNull
   printf, lun, "     ; obj_valid(SpecObj)= ", OK 
   printf, lun

   printf, lun,"; Check that DavePtr ID stored here is the", $
   	" same as at the top!"
   davePtrID = $
   (*(((*((*thisDavePtr).dataStrPtr)).specificPtr))).specObj -> getDavePtr()
   printf, lun, "(*(((*((*thisDavePtr).dataStrPtr)).specificPtr))).specObj", $
   	".davePtr = ", davePtrID
   printf, lun
 endif

 ;============================================================================
 skip_dataStrPtr:

 ;============================================================================
 skip_specificPtr:

 ;============================================================================
 index1 = where( (top_Tags EQ 'DESCRIPTR') ,count)
 if (count EQ 0) then begin
   goto,  skip_descriPtr
 endif 

 subVarName = '((*' + varName + ').descriPtr)'
 OK = ptr_valid( ((*thisDavePtr).descriPtr) )
 if (NOT OK) then begin
   goto,  skip_descriPtr
 endif
 
 if (n_elements( *((*thisDavePtr).descriPtr) ) eq 0) then begin
   goto,  skip_descriPtr
 endif
 
 tname = size( ((*thisDavePtr).descriPtr), /tname)
 OK = 0
 if (tname EQ 'POINTER') then begin
   OK = ptr_valid( ((*thisDavePtr).descriPtr) )
   if (n_elements( ((*thisDavePtr).descriPtr)) EQ 0) then OK = 0
   if (OK) then begin
     if (n_elements( (*((*thisDavePtr).descriPtr)) ) EQ 0) then begin
       OK = 0
     endif
   endif
 endif 
 if (NOT OK) then begin
    if (!debug) then begin
      printf, lun, infolabel, ' Requires ', subVarName, $ ;;((*DavePtr).descriPtr)', $
    	' to be a valid POINTER with contents'
      printf, lun
    endif
    goto,  skip_descriPtr
;    return
 endif
 
 singleton_show, ((*thisDavePtr).descriPtr) , subVarName, $
 	/partial, lun=lun		;;, isNull=wasNull

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'


 ;============================================================================
 subVarName = '(*((*' + varName + ').descriPtr))'
 tname = size( (*((*thisDavePtr).descriPtr)), /tname)
 OK = 0
 if (tname EQ 'STRUCT') then begin
   OK = 1
 endif 
 if (NOT OK) then begin
    printf, lun, infolabel, ' Requires ', subVarName, $  $
    	' to be a valid STRUCT '
    printf, lun
    goto,  skip_descriPtr
 endif
 structure_show, (*((*thisDavePtr).descriPtr)), subVarName, $
 	/partial, lun=lun		;;, isNull=wasNull

 printf, lun
 
; if (!debug) then printf, lun, '------------------------'
 ;============================================================================

 
 skip_descriPtr:

 ;============================================================================
 abort_routine:
 message, /info, "Now need to close up file unit #'s if necessary."
 
; the_end:
 ;-------------------------------------------------------------------------
  if (n_elements(thisFile) NE 0) then begin
    message, /info, string("Closing ", lun, " ", thisFile)
    close, lun
    free_lun, lun
  endif


; Not worthy of reporting! TMR. Aug 6, 2003.
; Also, if a bare bones (empty) Dave ptr, this will crash.
; ;-------------------------------------------------------------------------
;  if (n_elements(thisFile) NE 0) then begin
;;    if (n_elements(thisVarName) EQ 0) then  begin
;;      thisVarName  = 'DavePtr'
;;    endif
;    aString = "davePtr_show, " + VarName + ", file ='" + thisFile + '""
;;    aString = "davePtr_show, DavePtr, file ="',  thisFile, '""
;    if (keyword_set(thisFull)) then begin
;      aString = aString + ', /full'
;    endif
;    davePtr_addTreatment, thisDavePtr, aString
;  endif
  
  ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; davePtr_show

;================================================================================
;*
 pro davePtr_show_old, thisDavePtr,$
   varName = thisVarName, $ ; Optional variable name. Defaults to 'DavePtr'
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   file = thisFile, $	; Optionally, write to a file. If absent, to screen
   partial=thisPartial, $
   full=thisFull
; Shows a DAVE pointer and what's beneath it.
; Shows the contents in the same order that it is cleanup in the 
; proceedure davePtr_cleanup. Might seem a bit backwards.
;/

  infolabel = "davePtr_show >> "  

; From Help documentation:
;
;Set this keyword to display information about file units.
;If no arguments are supplied in the call to HELP, information 
;on all open file units (except the special units 0, -1, and -2) is displayed. 
;If arguments are provided, they are taken to be integer file unit numbers,
;and information on the specified file units is given.
;
;For example, the command:
;
;HELP, /FILES, -2, -1, 0
;
;gives information below about the default file units:
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
;
;The attributes column tells about the characteristics of the file. 
; For instance, the file connected to logical file unit 2 is called 
; "stderr" and is the standard error file. It is opened for write access 
;(Write), is a new file (Truncate), is a terminal (Tty), and cannot be 
;closed by the CLOSE command (Reserved).
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>  if (n_params() NE 1) then begin

  debug = 0
  
  if (n_elements(thisLun) eq 0) then begin
    if (n_elements(thisFile) eq 0) then begin
      lun= -1
    endif else begin
      openw, lun, thisFile, /get_lun     
    endelse
  endif else begin
    lun= thisLun
    if (n_elements(thisFile) NE 0) then begin
      openw, lun, thisFile   
    endif 
  endelse

  if (n_elements(thisVarName) EQ 0) then  begin
    thisVarName  = 'DavePtr'
  endif

  if (n_elements(thisDavePtr) eq 0) then begin
    printf, lun, infolabel, ' Requires "', thisVarName, $
    		" ID=", thisDavePtr, $
		'"', ' to be a single DAVE pointer as input parameter'
    printf, lun
    return
  endif
   
  if (not ptr_valid(thisDavePtr)) then begin
    printf, lun, infolabel, ' Requires "', thisVarName, $
    		" ID=", thisDavePtr, $
		'"', ' to be a valid DAVE pointer as input parameter'
    printf, lun
    return
  endif
   
;  printf, lun
  if (debug) then printf, lun, infolabel, 'Entering. lun=', lun
  
;..............................................................................

 ;-------------------------------------------------------------------------
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.instrument'
   help, output=txtarray, $
    	(*(*thisDavePtr).dataStrPtr).commonStr.instrument
   printf, lun, txtarray

  ;------------------------------------------------------------------------  
  ; Points to a structure of strings and arrays
    printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.HistPtr'
    help, output=txtarray, $
    	(*(*thisDavePtr).dataStrPtr).commonStr.HistPtr
    printf, lun, txtarray
    if (ptr_valid( (*(*thisDavePtr).dataStrPtr).commonStr.HistPtr)) then begin
      help, output=txtarray, $
    	(*(*(*thisDavePtr).dataStrPtr).commonStr.HistPtr)
      printf, lun, txtarray
      structure_show, lun=lun,  full=thisFull, $
      	(*(*(*thisDavePtr).dataStrPtr).commonStr.HistPtr)
    endif
    printf, lun

 ;-------------------------------------------------------------------------
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.xlabel=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.xlabel
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.ylabel=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.ylabel
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.xtype=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.xtype
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.ytype=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.ytype
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.xunits=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.xunits
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.yunits=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.yunits
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.histlabel=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.histlabel
   printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.histunits=', $
   	(*(*thisDavePtr).dataStrPtr).commonStr.histunits
   printf, lun

 ;-------------------------------------------------------------------------
 ; Points to string array
    printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr'
    help, output=txtarray, $
    	(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr
    printf, lun, txtarray
    if (ptr_valid( $
    (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr )) then begin
       strArray_show, (*(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr), $
       		lun=Lun, full=thisFull
;      help, output=txtarray, $
;    	(*(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr)
;      if (keyword_set(thisFull)) then begin
;        printf, lun, infolabel,   $
;         (*(*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr)
;      endif
    endif
    printf, lun

 ;-------------------------------------------------------------------------
 ; points to a pointer for a linked list?
    printf, lun, '(*(*thisDavePtr).dataStrPtr).commonStr.CommonStrPtr'
    help, output=txtarray, $
    	(*(*thisDavePtr).dataStrPtr).commonStr.commonStrPtr
    printf, lun, txtarray
    printf, lun

 ;-------------------------------------------------------------------------
 ; points to an an anonymous structure
 ; which may or maynot contain an Object of type specObj.
    printf, lun, '(*(*thisDavePtr).dataStrPtr).specificPtr'
    help, output=txtarray, $
    	(*(*thisDavePtr).dataStrPtr).specificPtr
    printf, lun, txtarray
    if (ptr_valid( $
            (*(*thisDavePtr).dataStrPtr).SpecificPtr )) then begin
      help, output=txtarray, $
    	(*(*(*thisDavePtr).dataStrPtr).specificPtr)
      printf, lun, txtarray
      ; Assume that it is an anonymous structure.
;      if ( obj_isa( *(*(*thisDavePtr).dataStrPtr).specificPtr, $
;	      'DATABASE') ) then begin
;	      ; Ensure that the method is available
;	printf, lun, "OBJ_ISA= 'DATABASE'.  '...-> show, /labels'= "
;        (*(*(*thisDavePtr).dataStrPtr).specificPtr) -> show, $
;		/labels, lun=lun, full=thisFull
;      endif
;      endif else begin
       tname = size(*(*(*thisDavePtr).dataStrPtr).specificPtr, /tname)
       if (tname EQ 'STRUCT') then begin
         structure_show, lun=lun, full=thisFull, partial=thisPartial, $
	 	 *(*(*thisDavePtr).dataStrPtr).specificPtr
       endif
;      endelse 
    endif
    printf, lun

 ; points to a structure
 ;-------------------------------------------------------------------------
    printf, lun, '(*thisDavePtr).dataStrPtr'
    help, output=txtarray, $
    	(*thisDavePtr).dataStrPtr
    printf, lun, txtarray
    if ( ptr_valid( (*thisDavePtr).dataStrPtr))  then begin
      structure_show, lun=lun,  $
      	(*(*thisDavePtr).dataStrPtr)
    endif
    printf, lun

 ;-------------------------------------------------------------------------
   someTags = tag_names( (*thisDavePtr))
   index1 = where( someTags EQ 'DESCRIPTR',count)
   if (count eq 1) then begin

 ;   points to a structure of strings and floats.
      printf, lun, '(*thisDavePtr).descriPtr'
      help, output=txtarray, $
    	 (*thisDavePtr).descriPtr
      printf, lun, txtarray
      if ( ptr_valid( (*thisDavePtr).descriPtr))  then begin
        help, output=txtarray, $
    	 (*(*thisDavePtr).descriPtr)
        printf, lun, txtarray
        structure_show, lun=lun, full=thisFull, $
      	(*(*thisDavePtr).descriPtr)
      endif
     printf, lun
   endif else begin
     printf, lun, '(*thisDavePtr).descriPtr is not defined'
     printf, lun
   endelse
   
 ;-------------------------------------------------------------------------
 ; points to a structre of pointers.
    printf, lun, 'thisDavePtr'
    help,  output=txtarray, $
    	thisDavePtr
    printf, lun, txtarray
    if ( ptr_valid( thisDavePtr))  then begin
      structure_show, lun=lun,  (*thisDavePtr)
    endif
   printf, lun

 ;-------------------------------------------------------------------------
  if (n_elements(thisFile) NE 0) then begin
    close, lun
    free_lun, lun
 endif

 ;-------------------------------------------------------------------------
  if (n_elements(thisFile) NE 0) then begin
;    if (n_elements(thisVarName) EQ 0) then  begin
;      thisVarName  = 'DavePtr'
;    endif
    aString = "davePtr_show, " + thisVarName + ", file ='" + thisFile + '""
;    aString = "davePtr_show, DavePtr, file ="',  thisFile, '""
    if (keyword_set(thisFull)) then begin
      aString = aString + ', /full'
    endif
    davePtr_addTreatment, thisDavePtr, aString
  endif
  
  ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; davePtr_show_old

;================================================================================
;*
 pro pstate_show, thisPState,$
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   full = thisFull	
; Shows a DAVE pState pointer and what's beneath it.
; Shows the pState pointer in the same order that it is cleanup in the 
; proceedure pstate_cleanup. Might seem a bit backwards.
;/

  infolabel = "pstate_show >> "  

; From Help documentation:
;
;Set this keyword to display information about file units.
;If no arguments are supplied in the call to HELP, information 
;on all open file units (except the special units 0, -1, and -2) is displayed. 
;If arguments are provided, they are taken to be integer file unit numbers,
;and information on the specified file units is given.
;
;For example, the command:
;
;HELP, /FILES, -2, -1, 0
;
;gives information below about the default file units:
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
;
;The attributes column tells about the characteristics of the file. 
; For instance, the file connected to logical file unit 2 is called 
; "stderr" and is the standard error file. It is opened for write access 
;(Write), is a new file (Truncate), is a terminal (Tty), and cannot be 
;closed by the CLOSE command (Reserved).
;
;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>  if (n_params() NE 1) then begin

  debug = 0
  
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (n_elements(thispState) eq 0) then begin
    printf, lun, infolabel, $
    	' Require a single DAVE pstate as input parameter'
    return
  endif
   
;  printf, lun
  if (debug) then printf, lun, infolabel, 'Entering. lun=', lun
  
;..............................................................................
  davePtr_show, (*thispstate).DavePtr, lun=thisLun, full=thisFull

 ;-------------------------------------------------------------------------
 ; points to an object (SPURION?) or a structure (?)
    printf, lun, '(*thispstate).ExtraPtr'
    help,  output=txtarray, $
    	(*thispstate).ExtraPtr
    printf, lun, txtarray
;    printf, lun, infolabel, *(*thispstate).ExtraPtr)
    if ( ptr_valid( (*thispstate).ExtraPtr))  then begin
      if ( obj_valid( (*(*thispstate).ExtraPtr) ) ) then begin
	printf, lun, "OBJ_VALID.  '...-> show'= "
        (*(*(*(*thispstate).DavePtr).dataStrPtr).extraPtr) -> show, $
		lun=lun, full=thisFull
      endif else begin
       structure_show, lun=lun, full=thisFull, (*(*thispstate).ExtraPtr)
      endelse 
    endif
   printf, lun


 ;-------------------------------------------------------------------------
    printf, lun, '*thispstate'
    help,  output=txtarray, (*thispstate)
    printf, lun, txtarray
    if ( ptr_valid( thispstate))  then begin
      structure_show, lun=lun, (*thispstate)
    endif

 ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; pstate_show


;================================================================================
;*
 pro structure_cleanup, $ ; Destroys a struture that is pointered to by specPtr
   specPtr,$
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   derefSpecific = thisDerefSpecific, $	;Boolean keyword. Default is destroy obj.
   verbose = thisVerbose, $
   skipSpecObj = thisSkipSpecObj
;/

  infolabel = "structure_cleanup >> "  

  debug = 0

  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse
  
  if (n_elements(specPtr) eq 0) then begin
    if (keyword_set(thisVerbose)) then begin
      printf, lun, infolabel, $
    	' Require a single structure pointer as input parameter'
      printf, lun
    endif
    return
  endif

  if (not ptr_valid(specPtr)) then begin
    if (keyword_set(thisVerbose)) then begin
      printf, lun,  infolabel, ' Requires a valid structure pointer as input parameter'
      printf, lun
    endif
    return
  endif
   
  if (debug) then printf, lun, infolabel, 'Entering'

 ;-------------------------------------------------------------------------
 ; points to a struture full of whatever the instrument scientist wanted
 ; to keep.

       tname = size( *SpecPtr, /tname)
       if (debug) then begin
         printf, lun, infolabel, 'SpecficPointer is pointing to a ', tname
       endif
       if (tname NE 'STRUCT') then begin
	 if (keyword_set(thisVerbose)) then begin
           printf, lun, infolabel, 'Non-standard. Should be to an anonymous structure.' 
           printf, lun, infolabel, 'No clean up of specPtr done.' 
	   printf, lun
	 endif
       endif else begin			;;  tname EQ STRUCT
         ; create a temporary pointer SpecPtr for ease of use
	 specTags = tag_names(*specPtr)
	 specStructName = tag_names(*specPtr, /structure_name)
         ntags = n_tags(*specPtr)
	 if (specStructName NE '') then begin
	   if (keyword_set(thisVerbose)) then begin
	     printf, lun, infolabel, "SpecPtr's structure has name '", $
	   	specStructName, "'"
	   endif
	 endif
	 for i = 0, (ntags-1) do begin
	   fieldTName = size((*specPtr).(i), /tname)
	   if (fieldTName EQ 'OBJREF') then begin
;  	     if (not keyword_set(thisderefSpecific)) then begin
	       if (debug) then begin
	         printf, lun, infolabel, 'obj_destroy, (*specPtr).(', i, ') =', $
	     		'(*specPtr).(', specTags[i], ')'
	       endif
	       if (obj_valid((*specPtr).(i)) ) then begin
		 if (obj_class((*specPtr).(i)) NE 'SPECOBJ') then begin
	           obj_destroy, (*specPtr).(i)
		 endif else begin
  		   if (NOT keyword_set(thisSkipSpecObj)) then begin
	             obj_destroy, (*specPtr).(i)	; , /skipDavePtr
		   endif
		 endelse
	       endif
;	     endif
	   endif
	   if (fieldTName EQ 'POINTER') then begin
;  	     if (not keyword_set(thisderefSpecific)) then begin
	       if (debug) then begin
	         printf, lun, infolabel, 'ptr_free, (*specPtr).(', i, ') =', $
	     		'(*specPtr).(', specTags[i], ')'
	       endif
	       if (ptr_valid((*specPtr).(i)) ) then begin
	       	 ptr_free, (*specPtr).(i)
	       endif
;	     endif
	   endif
	   if (fieldTName EQ 'STRUCT') then begin
	     if (keyword_set(thisVerbose)) then begin
	       printf, lun, infolabel, 'No code for deleting structures in structures.'
	     endif
	   endif
	   if NOT ((fieldTName EQ 'OBJREF') OR (fieldTName EQ 'POINTER') $
	   		OR (fieldTName EQ 'STRUCT')) then begin
	     if (debug) then begin
	       if (keyword_set(thisVerbose)) then begin
	         printf, lun, infolabel, "No cleanup needed for ", $
	     	 '(*specPtr).(', i, ') =', $
	     		'(*specPtr).(', specTags[i], ')'
	       endif
	     endif
	   endif
	 endfor
	 ptr_free, specPtr
       endelse

 
 ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; structure_cleanup

;========================================================================
;*
 pro DavePtr_destroy, $	; Destroys everything except the ptrID for thisDavePtr
	thisDavePtr, $
   	destroySpecObj = thisDestroySpecObj	; Optional keyword.
		
;/

; Destroys a DAVE pointer and what's beneath it.
; Note: davePtr_Show shows the Dave pointer in the same order that it is 
; cleanup in this proceedure davePtr_cleanup.

  infolabel = "davePtr_destroy >> "  

  message, /info, string("davePtr_containsSpecObj(thisDavePtr)=", $
	  davePtr_containsSpecObj(thisDavePtr))

;  tempObj = davePtr_containsSpecObj(thisDavePtr)
;  tempOK = obj_valid(tempObj)
;  print,  tempOK
;  message, /info, string(tempOK)		;; string("1 valid =", tempOK)
;  junkOK = obj_valid( davePtr_containsSpecObj(thisDavePtr)) 
;  print, "1 specObj valid = ", junkOK
;;  message, /info, string("1 SO valid =", junkOK)

  soExists = davePtr_containsSpecObj(thisDavePtr)
  message, /info, string(" contains SO=", soExists)
  if (soExists) then begin
    message, /info, string(" SO valid=", $
    	obj_valid(davePtr_getSpecObj(thisDavePtr, from='DavePtr_destroy')))
  endif  
  
  if (keyword_set(thisDestroySpecObj)) then begin
    if (davePtr_containsSpecObj(thisDavePtr)) then begin
	  ;heap_free doesn't destroy the object OK. 
	  tempObj = davePtr_getSpecObj(thisDavePtr, from='DavePtr_destroy')
	  obj_destroy, tempObj			; , /skipDavePtr
    endif
  endif else begin
    if (davePtr_containsSpecObj(thisDavePtr)) then begin
       (*(*(*thisDavePtr).dataStrPtr).SpecificPtr).specObj = obj_new()	; let objRef dangle
    endif
  endelse
  
;  junkOK = obj_valid( davePtr_containsSpecObj(thisDavePtr)) 
;  message, /info, string(junkOK)		;;string("2 SO valid =", junkOK)

  soExists = davePtr_containsSpecObj(thisDavePtr)
  message, /info, string(" contains SO=", soExists)
  if (soExists) then begin
    message, /info, string(" SO valid=", $
    	obj_valid(davePtr_getSpecObj(thisDavePtr, from='DavePtr_destroy')))
  endif  

  if (n_elements((*thisDavePtr)) GT 0) then begin
    heap_free, (*thisDavePtr)  	; Empty out the contents! Will be replaced.
  endif

;  junkOK = obj_valid( davePtr_containsSpecObj(thisDavePtr)) 
;  junkOK = obj_valid( davePtr_getSpecObj(thisDavePtr)) 
;  message, /info, string(junkOK)		;;("3 SO valid =" + string(junkOK))

  soExists = davePtr_containsSpecObj(thisDavePtr)
  message, /info, string(" contains SO=", soExists)
  if (soExists) then begin
    message, /info, string(" SO valid=", $
    	obj_valid(davePtr_getSpecObj(thisDavePtr, from='DavePtr_destroy')))
  endif  

end		; DavePtr_destroy

;========================================================================
;*
 pro DavePtr_empty, $	; partially destroys things within the dave structure
   DavePtr, $
   derefSpecObj = thisDerefSpecObj	;Boolean keyword. Default is destroy obj.
;/

  debug = 0
  debug2 = 0
  debug3 = 0
;  debug3 = 1
  
  infolabel = "DavePtr_empty >> "

  if (n_elements(DavePtr) EQ 0) then begin
    message, /info, string("DavePtr is undefined")
    return
  endif

  if (NOT ptr_valid(DavePtr)) then begin
    if (debug) then message, /info, string("DavePtr is alread invalid ID=", davePtr)
    return
  endif
  
  if (debug) then begin
    help, DavePtr   
    message, /info, string("ptr_valid(davePtr=", davePtr, $
    		") = ", ptr_valid(davePtr))
    if (ptr_valid(davePtr)) then begin
       help, *DavePtr 
    endif
  endif

  if (n_elements(*DavePtr) EQ 0) then begin
    if (debug) then message, string("*DavePtr has 0 elements. ID=", davePtr)
    return
  endif
  
  if (davePtr_containsSpecObj(DavePtr)) then begin
    if (!debug) then begin 
      message, /info, string("obj_destroy, specObj, /skipDavePtr; thisDerefSpecifics=", $
      	keyword_set(thisDerefSpecific))
    endif

    if (debug) then print, "n_elements(*davePtr) = ", n_elements(*davePtr)
    
    if (NOT keyword_set(thisDerefSpecific)) then begin
      ;heap_free doesn't destroy the object OK. 
      tempObj = davePtr_getSpecObj(DavePtr, from='DavePtr_empty')
      DavePtr_nullSpecObj, DavePtr	; Put a null object there. Sept 15, 2003.
      if (!debug) then message, /info, string(tempObj, "destroying specObj")
      obj_destroy, tempObj		; , /skipDavePtr
    endif else begin
      DavePtr_nullSpecObj, DavePtr	; Put a null object there. Sept 15, 2003.
    endelse 
    
    if (debug) then print, "n_elements(*davePtr) = ", n_elements(*davePtr)
    if (debug) then help, davePtr
  endif
  
  if (NOT ptr_valid(DavePtr)) then begin
    if (debug) then message, string("DavePtr is alread invalid ID=", davePtr)
;    return
  endif
  
  ; Too brutal.
 ;  heap_free, (*DavePtr)  	; Empty out the contents! Will be replaced.;

  if (debug2) then print, infolabel, DavePtr, " ptr_valid(DavePtr)=", ptr_valid(DavePtr)
  if (debug2) then print, infolabel, DavePtr, " n_elements(*davePtr)=", n_elements(*davePtr)
 
  ; The official way from DAVE.
  top_tags = tag_names((*DavePtr) )
  index1 = where( (top_Tags EQ 'DESCRIPTR') ,count)
  if (count EQ 1) then begin		; Might not be used.
    if (debug2) then print, infolabel, DavePtr, " ptr_valid(DavePtr)=", ptr_valid(DavePtr)
    if (debug2) then print, infolabel, DavePtr, " n_elements(*davePtr)=", n_elements(*davePtr)

    if (debug2) then print, infolabel, DavePtr, " ptr_valid(DavePtr)=", ptr_valid(DavePtr)
 
    if (debug2) then print, infolabel, DavePtr, (*DavePtr).descriPtr,  $
		"before NOT ptr_valid((*DavePtr).descriPtr)"
    if (debug3) then begin
      print
      davePtr_show, davePtr
      print
    endif
    
    if (ptr_valid((*DavePtr).descriPtr)) then begin
      if (n_elements((*DavePtr).descriPtr) GT 0) then begin
         ; for some reason heap_free makes davePtr invalid
        heap_free, ((*DavePtr).descriPtr)	; Seems to be doing heap_free, ((*DavePtr)
;        ptr_free, ((*DavePtr).descriPtr)		;sept 17, 2003
      endif else begin
        print, infolabel, DavePtr, (*DavePtr).descriPtr,  $
		"NOT ptr_valid((*DavePtr).descriPtr)"
      endelse
    endif else begin
       print, infolabel, DavePtr, (*DavePtr).descriPtr,  $
		"n_elements((*DavePtr).descriPtr) EQ 0"
    endelse
    
    ;For some reason davePtr became invalid here from the heap_free.
    if (debug2) then print, infolabel, DavePtr, " ptr_valid(DavePtr)=", ptr_valid(DavePtr)
    if (debug2) then print, infolabel, DavePtr, " n_elements(*davePtr)=", n_elements(*davePtr)
  endif 

  if (debug3) then begin
    print
    davePtr_show, davePtr
    print
  endif
  
  ;IF condition added. Aug 18, 2003. TMR
  if (debug) then begin
    help, DavePtr
    print, ptr_valid(daveptr)
    print, "n_elements(*davePtr) = ", n_elements(*davePtr)	; SOMETIMES HAS 0 ELEMENTS
    if (n_elements(*davePtr) GT 0) then begin
      help, (*DavePtr)
      help, (*DavePtr).dataStrPtr
      help, (*((*DavePtr).dataStrPtr))
      help, (*((*DavePtr).dataStrPtr)).commonStr
    endif
  endif

 if (debug2) then print, infolabel, DavePtr, " ptr_valid(DavePtr)=", ptr_valid(DavePtr)
 if (debug2) then print, infolabel, DavePtr, " n_elements(*davePtr)=", n_elements(*davePtr)
 
 if (n_elements(*davePtr) GT 0) then begin

  if (n_elements((*((*DavePtr).dataStrPtr)).commonStr) GT 0) then begin
    heap_free, ((*((*DavePtr).dataStrPtr)).commonStr)
  endif
  
  ;IF condition added. Aug 18, 2003. TMR
  if (n_elements((*((*DavePtr).dataStrPtr)).specificPtr) GT 0) then begin 
    heap_free, ((*((*DavePtr).dataStrPtr)).specificPtr)
  endif

  if (debug) then begin
    help, DavePtr   
    message, string("End of routine. ptr_valid(davePtr=", davePtr, $
    		") = ", ptr_valid(davePtr))
  endif

 endif   else begin
   message, /info, "n_elements(*davePtr) EQ 0. Not intended."
 endelse 

end		; DavePtr_empty

;========================================================================
;*
 pro DavePtr_cleanup, $	; Old fashioned cleanup.
   thisDavePtr,$
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   derefSpecific = thisDerefSpecific, $	;Boolean keyword. Default is destroy obj.
   verbose = thisVerbose, $
   skipSpecObj = thisSkipSpecObj
;/

; Destroys a DAVE pointer and what's beneath it.
; Note: davePtr_Show shows the Dave pointer in the same order that it is 
; cleanup in this proceedure davePtr_cleanup.
;/

  infolabel = "davePtr_cleanup >> "  

  debug = 0
  
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (n_elements(thisDavePtr) NE 1) then begin
    printf, lun, infolabel, $
    	' Require a single DAVE pointer as input parameter'
    printf, lun
    return
  endif

  if (not ptr_valid(thisDavePtr)) then begin
    printf, lun, infolabel, ' Requires a valid DAVE pointer as input parameter'
    printf, lun
    return
  endif
   
;  printf, lun
  if (debug) then printf, lun, infolabel, 'Entering. lun=', lun

 ;-------------------------------------------------------------------------
 ; Points to a structure of strings and arrays
    ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.HistPtr

 ;-------------------------------------------------------------------------
 ; Points to string array
    ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.TreatmentPtr

 ;-------------------------------------------------------------------------
 ; points to a pointer for a linked list?
    ptr_free, (*(*thisDavePtr).dataStrPtr).commonStr.commonStrPtr

 ;-------------------------------------------------------------------------
 ; points to a struture full of whatever the instrument scientist wanted
 ; to keep.
    structure_cleanup, (*(*thisDavePtr).dataStrPtr).SpecificPtr , $
   	lun = thisLun, $ ; Optionally, write to a file. If absent, to screen
   	derefSpecific = thisDerefSpecific, $	;Boolean keyword. Default is destroy obj.
	verbose = thisVerbose, $
   	skipSpecObj = thisSkipSpecObj

 ; points to a structure
 ;-------------------------------------------------------------------------
    ptr_free, (*thisDavePtr).dataStrPtr

 ;-------------------------------------------------------------------------
   someTags = tag_names( (*thisDavePtr))
   index1 = where( someTags EQ 'DESCRIPTR',count)
   if (count eq 1) then begin
     ; points to a structure of strings and floats.
      ptr_free, (*thisDavePtr).descriPtr
   endif

 ;-------------------------------------------------------------------------
 ; points to a structre of pointers.
     ptr_free, thisDavePtr

 ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; DavePtr_cleanup

; specStruc2SpecObj moved to specObj::convert, specObj::initDavePtr, and 
; 				specObj::initdavePtr
;;================================================================================
; pro specStruc2SpecObj, thisDavePtr,$
;;   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
;   replace = thisReplace	;Boolean keyword.
;   ;  Replaces structure's members with specObj. Default is to suppliment.
;; This routine creates an object of class SpecObj from the 
;; contents of the structure that specficPointer points to.
;; It then places that object at the end of the structure.
;; If the keyword "/replace" is used, the structure is only only {specObj:SpecObj}
;; and the original contents are removed.
;
;  infolabel = "specStruc2SpecObj >> "  
;
;  currentCodeSource = 'specStruc2SpecObj'
;  debug = 0
;  
;  if (n_elements(thisDavePtr) eq 0) then begin
;    print,  infolabel, $
;    	' Require a single DAVE pointer as input parameter'
;    print
;    return
;  endif
;
;  if (not ptr_valid(thisDavePtr)) then begin
;    print, infolabel, ' Requires a valid DAVE pointer as input parameter'
;    print
;    return
;  endif
;
;  specObject = obj_new('specObj', $
;  	label = 'generic_label', $	; needs a better label
;	codeSource = currentCodeSource, $
;	davePtr = thisDavePtr)
;
;  nsuc = 0
;  nfail = 0
;  nvar = 0
;
;  if (ptr_valid( (*(*thisDavePtr).dataStrPtr).CommonStr.treatmentPtr)) then begin
;   ; first put the treatment pointer in header portion of the specObj.
;    nsuc = nsuc + 1
;    nvar = nvar + 1
;    specObject -> add, /inHeader,$
;    	tags=['qtyObj', 'treatment', currentCodeSource], $
;   	contents=obj_new('qtyObj', name = 'Treatment history', $
;		ptr_qty=  (*(*thisDavePtr).dataStrPtr).CommonStr.treatmentPtr) 
;  endif
;  
; ; Copy the original contents of x, y, and hist/qty from CommonStr and histPtr
; ; Note: The use of the temporary pointer temp_ptr, allows the object specObject
; ; to get the data by reference, and not waste time copying it by value.
;
;  if (ptr_valid( (*(*thisDavePtr).dataStrPtr).CommonStr.histPtr)) then begin
;;Debug     help, (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr)
;;Debug     print, infolabel, $
;;Debug     	size( (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr), /tname)
;     histTags = tag_names( (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr) )
;     
;     currentXLabel = ''
;     index1 = where( histTags EQ 'X',count)
;     if (count eq 1) then begin
;       currentXLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.xlabel 
;;;;       currentLegend = (*(*thisDavePtr).dataStrPtr).CommonStr.xtype
;       currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.xunits
;        
;       ;Copy by value not reference, because the common area may get abused later.
;       if (n_elements( $
;       	 (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) NE 1) then begin
;         nsuc = nsuc + 1
;	 nvar = nvar + 1
;	 specObject -> add, /inData, $
;          tags=['qtyObj', currentXLabel, currentCodeSource], $
;	  contents=obj_new('qtyObj', name = currentXLabel, $
;		units = currentUnits,  $		;; no legend
;		qty =  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) 
;       endif else begin
;         nsuc = nsuc + 1
;	 nvar = nvar + 1
;	 specObject -> add, /inHeader, $
;          tags=['qtyObj', currentXLabel, currentCodeSource], $
;	  contents=obj_new('qtyObj', name = currentXLabel, $
;		units = currentUnits,  $		;; no legend
;		qty =  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) 
;	 ; this is presumably a cut.
;          specObject  -> setFocus, $ 
;	    cutTags = ['qtyObj', currentXLabel, currentCodeSource]
;       endelse
;     endif
;     
;     currentYLabel = ''
;     index1 = where( histTags EQ 'Y',count)
;     if (count eq 1) then begin
;       currentYLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.ylabel 
;;;;       currentLegend = (*(*thisDavePtr).dataStrPtr).CommonStr.ytype
;       currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.yunits
;       if (n_elements( $
;       	 (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y) NE 1) then begin
;         nsuc = nsuc + 1
;	 nvar = nvar + 1
;	 specObject -> add, /inData, $
;       	    tags=['qtyObj', currentYLabel, currentCodeSource], $
;   	    contents=obj_new('qtyObj', name = currentYLabel, $
;		units = currentUnits, $	;; no legend
;		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y ) 
;       endif else begin
;	 ; this is presumably a cut.
;          nsuc = nsuc + 1
;	  nvar = nvar + 1
;	  specObject -> add, /inHeader, $
;       	    tags=['qtyObj', currentYLabel, currentCodeSource], $
;   	    contents=obj_new('qtyObj', name = currentYLabel, $
;		units = currentUnits, $	;; no legend
;		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y ) 
;          specObject  -> setFocus, $ 
;	    cutTags = ['qtyObj', currentYLabel, currentCodeSource]
;       endelse
;     endif
;     
;     index1 = where( histTags EQ 'ERR',count)
;     if (count eq 1) then begin
;       useERR = 1
;     endif else begin
;       useErr = 0
;     endelse
;     nlen = n_elements((*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err)
;     if (nlen eq 1) then begin
;       if ((*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err EQ 0.0) $
;       		 then begin
;         useERR = 0
;       endif
;     endif
;     
;     currentZLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.histlabel 
;     currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.histUnits
;;;;     temp_ptr = ptr_new( (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).qty)
;     if (useERR) then begin
;;;;       tempErr_ptr = ptr_new( $
;;;;       			(*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err)
;       nsuc = nsuc + 1
;       nvar = nvar + 1
;       specObject -> add, /inData, $
;       	    tags=['qtyObj', currentZLabel, currentCodeSource], $
;   	    contents=obj_new('qtyObj', name = currentZLabel, $
;		units = currentUnits, $
;;;;		ptr_qty= temp_ptr, $
;;;;		ptr_err= tempErr_ptr, $
;		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).qty,$
;		err=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err )
;;;;       ptr_free, temp_ptr 
;;;;       ptr_free, tempErr_ptr 
;     endif else begin
;       nsuc = nsuc + 1
;       nvar = nvar + 1
;       specObject -> add, /inData, legend=currentZLabel, $
;       	    tags=['qtyObj', currentZLabel, currentCodeSource], $
;   	    contents=obj_new('qtyObj', name = currentZLabel, $
;		units = currentUnits, $
;;;;		ptr_qty= temp_ptr, $
;		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).qty)
;;;;       ptr_free, temp_ptr 
;     endelse
;   endif
;
;   ;; Focus defaults to the values originally stored in commonPtr.
;;    print, "specObject -> setFocus, xTags=..., zTags = ..." 
;    if ((currentXlabel NE '') AND (currentYlabel NE '')) then begin
;      specObject  -> setFocus, $ 
;   	xTags = ['qtyObj', currentXLabel, currentCodeSource], $
;   	yTags = ['qtyObj', currentYLabel, currentCodeSource], $
;   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
;    endif else begin
;
;      if (currentYlabel NE '') then begin
;        specObject  -> setFocus, $ 
;   	yTags = ['qtyObj', currentYLabel, currentCodeSource], $
;   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
;      endif  
;
;      if (currentXlabel NE '') then begin
;        specObject  -> setFocus, $ 
;   	XTags = ['qtyObj', currentXLabel, currentCodeSource], $
;   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
;      endif  
;
;    endelse    
;
; ;-------------------------------------------------------------------------
; ;-------------------------------------------------------------------------
;   someTags = tag_names( (*thisDavePtr))
;   index1 = where( someTags EQ 'DESCRIPTR',count)
;   if (count eq 1) then begin
;    if (ptr_valid( (*thisDavePtr).descriPtr)) then begin
;     currentDLabel = (*(*thisDavePtr).descriPtr).name
;     currentDLegend = (*(*thisDavePtr).descriPtr).legend
;       nsuc = nsuc + 1
;       nvar = nvar + 1
;       specObject -> add, /inData, legend=currentZLabel, $
;       	    tags=['qtyObj', currentDLabel, currentCodeSource], $
;   	    contents=obj_new('qtyObj', /inHeader, $
;	        name = currentDLabel, $
;		units = (*(*thisDavePtr).descriPtr).units, $
;		legend = currentDLegend, $
;		qty = (*(*thisDavePtr).descriPtr).qty, $
;		err = (*(*thisDavePtr).descriPtr).err) 
;      specObject  -> setFocus, $ 
;	descripTags = ['qtyObj', currentDLabel, currentCodeSource], $ 
;	descripLegend = currentDLegend
;    endif
;   endif 
;
; ;-------------------------------------------------------------------------
; ;-------------------------------------------------------------------------
; ; SpecPtr points to a struture full of whatever the instrument scientist wanted
; ; to keep. Copy the contents over into SpecObject.
;    if (ptr_valid( $
;            (*(*thisDavePtr).dataStrPtr).SpecificPtr )) then begin
;       tname = size( *(*(*thisDavePtr).dataStrPtr).SpecificPtr, /tname)
;       if (debug) then $
;       		print, infolabel, 'SpecficPointer is pointing to a ', tname
;       if (tname NE 'STRUCT') then begin
;         print, infolabel, 'Non-standard. Should be to an anonymous structure.' 
;         print, infolabel, 'No modification of (*(*thisDavePtr).dataStrPtr).specificPtr done.' 
;       endif else begin			;;  tname EQ STRUCT
;         ; create a temporary pointer SpecPtr for ease of use
;         specPtr = ptr_new(*(*(*thisDavePtr).dataStrPtr).SpecificPtr)
;	 specTags = tag_names(*specPtr)
;	 specStructName = tag_names(*specPtr, /structure_name)
;         ntags = n_tags(*specPtr)
;
;	 ;; Check to see if a specfic object already exists in the structure.
;	  specObj_preexists = 0
;	 index1 = where( (specTags EQ 'SPECOBJ'), count1)
;	 if (count1 NE 0) then begin
;	    print, infolabel, 'specificStructure already has a specObject field.'
;	    print, infolabel, 'Will destroy it and then re-create it.' 
;	    specObj_preexists = 1
;	    obj_destroy, (*specPtr).specObj
;	 endif
;
;	 if (specStructName NE '') then begin
;	   print, infolabel, "SpecificPtr's structure has name '", $
;	   	specStructName, "'"
;	 endif
;
;	 ; Fill up the rest of specObj using the fields of the original structure.
;	 for i = 0, (ntags-1) do begin
;	   fieldTName = size((*specPtr).(i), /tname)
;	   if (fieldTName EQ 'OBJREF') then begin
;	       print, infolabel, 'specObject skipping OBJREF ', $
;			 fieldTageName
;	   endif else begin
;	     if (fieldTName EQ 'POINTER') then begin
;	       temp_ptr = ptr_new( *((*specPtr).(i)) )
;	       if (debug) then print, infolabel, specTags[i], " is a ", fieldTName, $
;	       	 " of dim=", size(*(temp_ptr), /dim )
;	       if (debug) then print, infolabel, specTags[i], " is a ", $
;	       		fieldTName, " with. val[0]=", $
;	     		(*(temp_ptr))[0] 
;;	       print, infolabel, "fieldTName EQ 'POINTER'. size=", $
;;	     		size(*(temp_ptr) )
;	       print, infolabel, specTags[i],$
;	       	fieldTName, "=fieldTName EQ 'POINTER'. val[0]=", $
;	     		(*(temp_ptr))[0] 
;	     ; Note: add() decides where to place in (inHeader or in Data)
;	     ; based on the dimensions of the quantity ptr_qty=((*specPtr).(i)).
;	      nsuc = nsuc + 1
;	      specObject -> add, $
;	        tags = ['qtyObj', specTags[i], 'specStruc2SpecObj'], $
;   		contents=obj_new('qtyObj', name = specTags[i], $
;		  ptr_qty=temp_ptr) 
;;Wrong		  ptr_qty=((*specPtr).(i)) ) 
;		ptr_free, temp_ptr 
;	     endif else begin
;	       if (fieldTName EQ 'STRUCT') then begin
;	         ; avoid a bit of recursion, if not needed.
;	         print, infolabel, 'specObject skipping STRUCTURE ', $
;			 fieldTageName
;;;		 nfail = nfail + 1
;	       endif else begin
;	         temp_ptr = ptr_new(((*specPtr).(i)) )
;	         if (debug) then print, infolabel, specTags[i], " is a ARRAY ", fieldTName, $ 
;	       		 " of dim=", size(*(temp_ptr), /dim)
;	         if (debug) then print, infolabel, specTags[i], " is a ARRAY ", $
;	        	fieldTName, " with val[0]=", $
;	     		(*(temp_ptr))[0] 
;;	         print, infolabel, specTags[i], fieldTName, $
;;		 	"=fieldTName EQ array. val[0]=", $
;;	     		(*(temp_ptr))[0]
;	        ; Note: add() decides where to place in (inHeader or in Data)
;	        ; based on the dimensions of the quantity ptr_qty=temp_ptr.
;	      	 nsuc = nsuc + 1
;		 specObject -> add, $
;	           tags = ['qtyObj', specTags[i], 'specStruc2SpecObj'], $
;   		   contents=obj_new('qtyObj', name = specTags[i], $
;		  	ptr_qty=temp_ptr ) 
;;Wrong		        ptr_qty=ptr_new(((*specPtr).(i)) ) ) 
;		 ptr_free, temp_ptr 
;	       endelse
;	     endelse
;	   endelse
;	 endfor
;
;	 if (nsuc NE (ntags+nvar)) then begin
;	   print, infolabel, 'Success on only ', $
;	   	strcompress(string(nsuc), /remove_all), $
;	   	' out of ', strcompress(string(ntags), /remove_all),$
;		 ' fields in the structure plus ', $
;		 strcompress(string(nvar), /remove_all), $
;		 ' items in histPtr and descriPtr.'
;	 endif	 
;	 
;	 ; replace the old struture with a new anonymous one.
;	 if (keyword_set(thisReplace)) then begin
;	   structure_cleanup, specPtr
; 	   ptr_free,  (*(*thisDavePtr).dataStrPtr).SpecificPtr
;           (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
;      			ptr_new( {specObj,specObject})
;	 endif else begin
; 	   ptr_free,  (*(*thisDavePtr).dataStrPtr).SpecificPtr
;	   if (specObj_preexists) then begin
;	     (*specPtr).specObj= specObject 
;             (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
;      		ptr_new( (*specPtr) )
;	   endif else begin
;            (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
;      		ptr_new( create_struct((*(specPtr)), {specObj:specObject}) )
;	   endelse
;
;	 endelse
;	 ptr_free, specPtr
;       endelse
;
;    endif else begin
;      ; Nothing stored in specficPointer, but we already loaded some other 
;      ; infomation into specObj.
;      (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
;      		ptr_new( {specObj:specObject})
;      
;
;    endelse
;    
;  ;-------------------------------------------------
;    specObject -> setFocus, $ 
;        mainTitleTags = ['qtyObj', 'mainTitle', 'specObj_example1']
;    if (debug) then  print, "specObj -> setFocus, cutTags = ..."
;    specObject  -> setFocus, $ 
;	cutTags = ['qtyObj', 'specific_cut', 'specObj_example1']
;
; ;-------------------------------------------------------------------------
;  if (debug) then print, infolabel, 'Exiting'
;
;end		; specStruc2SpecObj


;================================================================================
;*
 pro pstate_cleanup, thisPState,$
   lun = thisLun, $	; Optionally, write to a file. If absent, to screen
   derefSpecific = thisDerefSpecific, $	;Boolean keyword. Default is destroy obj.
   derefExtra = thisDerefExtra		;Boolean keyword. Default is destroy obj.
; Destroys a DAVE pState pointer and what's beneath it.
; Note: pstate_Show shows the pState pointer in the same order that it is 
; cleanup in this proceedure pstate_cleanup.
;/

  infolabel = "pstate_cleanup >> "  
  debug = 0

  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (n_elements(thispState) eq 0) then begin
    printf, lun, infolabel, $
    	' Require a single DAVE pstate as input parameter'
    return
  endif
   
;  printf, lun
  if (debug) then printf, lun, infolabel, 'Entering. lun=', lun

;..............................................................................
  davePtr_cleanup, (*thispstate).DavePtr, $
  		lun=thisLun, derefSpecific = thisDerefSpecific

 ;-------------------------------------------------------------------------
 ; points to an object (SPURION?) or a structure (?)
    if ( ptr_valid( (*thispstate).ExtraPtr))  then begin
      if ( obj_valid( (*(*thispstate).ExtraPtr) ) ) then begin
        if (not keyword_set(thisderefExtra)) then begin
          obj_destroy, (*(*thispstate).ExtraPtr) 
        endif	; otherwise, reference the object, but don't destroy it.
      endif else begin
      endelse 
    endif
    ptr_free, (*thispstate).ExtraPtr

 ;-------------------------------------------------------------------------
  ptr_free, thispstate

 ;-------------------------------------------------------------------------
  if (debug) then printf, lun, infolabel, 'Exiting'

end		; pstate_cleanup

;==============================================================================
;*
pro structure_show,  $
	thisRec, $	; a structure
	theName, $	; string name of variable
	lun=thisLun, $	; Optionally, write to a file. If absent, to screen
	partial=ThisPartial, $
	full=ThisFull
;/
  infolabel = 'structure_show >> '
  
  if (n_elements(thisRec) eq 0) then return
 
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  theIndent = '    '
  
  tname = size(thisRec, /tname)
  if (n_elements(theName) EQ 0) then begin
     ; substitute name. 
	if (tname EQ 'STRUCT') then begin
          thename = '<<' + tag_names( thisRec, /structure_name) + '>>'
          if (theName eq '') then thename = '<<Anon>>'
	endif
  endif else begin
    theName = theName
  endelse
  
	if (tname EQ 'STRUCT') then begin
          ntags = n_tags( thisRec)
          thetags = tag_names(thisRec)
	  theStructName = tag_names( thisRec, /structure_name)
	  if (strLen(theStructName) EQ 0) then theStructName = '<ANON>'
	  
;          thename = '{' + tag_names( thisRec, /structure_name) + '}'
	  
	  printf, lun, theName, " = { ", theStructName, ','	  
	  
	  if (keyword_set(thisFull)) then begin
            for i = 0, (ntags-1) do begin
	      tname = size(thisrec.(i), /tname)
	      if (tname EQ 'POINTER') then begin
		if ( ptr_valid( (thisrec.(i)) ) ) then begin
                  printf, lun, theIndent, $
;		  	thename, '.', $
			thetags[i], ':',  $
			thisrec.(i)
		endif else begin
                  printf, lun, theIndent, $
;		  	thename, '.', $
			thetags[i], $
			': <UNDEFINED>'
		endelse
	      endif else begin
                printf, lun, theIndent, $
;			thename, '.', $
			thetags[i], ':',  $
			thisrec.(i)
	      endelse
;              printf, lun, thename, '.', thetags[i], '=',  $
;		thisrec.(i)
            endfor
	  endif else begin
	    if (keyword_set(thisPartial)) then begin
              for i = 0, (ntags-1) do begin
	        tname = size(thisrec.(i), /tname)
		tname_extra = ''
		if (tname EQ 'POINTER') then begin
		  if ( ptr_valid( (thisrec.(i)) ) ) then begin
		    tvalue = thisrec.(i)
;		    print, infolabel, 'WARNING! PTR'
	            tname_extra = '_' + size(*(thisrec.(i)), /tname)
		  endif else begin
		    tvalue = thisrec.(i)
;		    tvalue = '<UNDEFINED>'
	            tname_extra = '_<UNDEFINED>'
		  endelse
		endif else begin
		  tvalue = thisrec.(i)
		  if (tname EQ 'OBJREF') then begin
		    tname_extra = '_' + obj_class(thisrec.(i))
		  endif
		endelse
;Debug		print, infolabel, 'tname = ', tname

		if (n_elements(thisrec.(i)) LE 1) then  begin
		  if (tname EQ 'POINTER')   then begin
                    printf, lun, theIndent, $
;		    	thename, ':', $
			thetags[i], ':',  $
		      	tvalue, '=', $  
		      '<' + tname + tname_extra + '> ' $
;		      '{' + tname + tname_extra + '} ' $
		      + format_size(ptr_array=thisrec.(i))
		  endif else begin
		    if (tname EQ 'OBJREF') then begin
;		     message, /info, string("# of objects=", n_elements((thisrec.(i))))
;		     message, /info, string("valid objects=", obj_valid((thisrec.(i))))
                     printf, lun, theIndent, $
 ;		      thename, '.', $
		      thetags[i], ':',  $
		      tvalue
;NotNec		      '=',  '<' + tname + tname_extra +  '> '  $
;		      '=', '{' + tname + tname_extra +  '} '  $
		    endif else begin	; something ordinary
		      if (tname EQ 'STRUCT') then begin
                       printf, lun, theIndent, $
				thetags[i], ':',  $
		      		'<' + tname + '>'
;		      		'{' + tname + '}'
;		        subVarName = theName + '.' + thetags[i]
;		      	structure_show, (thisrec.(i)), subVarName, $
;				lun=thisLun, partial=ThisPartial, full=ThisFull
		      endif else begin
                        printf, lun, theIndent, $
;		      		thename, '.', $
				thetags[i], ':',  $
		      		'<' + tname + '> '  + $
;		      		'{' + tname + '} '  + $
		    		  strcompress(string(thisrec.(i)), /remove_all)
		      endelse
		    endelse
		  endelse 
		   
		endif else begin
		  ; don't actually show vector values, just the array length.
		  temp_ptr = ptr_new(thisrec.(i))
                  printf, lun, theIndent, $
;		  	thename, '.', $
			thetags[i], ':',  $
		    	'<' + tname+ tname_extra +  '> ' + $
;		    	'{' + tname+ tname_extra +  '} ' + $
		    		format_size(ptr_array=temp_ptr)
		  ptr_free, temp_ptr
		endelse
              endfor
	    endif else begin
	      printf, lun,  theIndent, 'tag_names(', thename, ') = ', theTags
	    endelse
	  endelse
	  printf, lun, "}"	  

	endif

end		; structure_show
;==============================================================================
;*
pro singleton_show,  $	; (opposite of structure_show)
	thisRec, $	; anything except a structure. 
	theName, $	; string name of thisrec
	lun=thisLun, $	; Optionally, write to a file. If absent, to screen
	partial=ThisPartial, $
	full=ThisFull, $
	isNULL =thisIsNull
;/
  infolabel = 'singleton_show >> '
  
  theIndent = '    '

  if (n_elements(thisRec) eq 0) then return
 
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (n_elements(thisIsNull) GT 0) then begin
    thisIsNull = 0
  endif

       tname = size(thisRec, /tname)

  if (n_elements(theName) EQ 0) then begin
    theName = '???'
  endif
  
  tname = size(thisRec, /tname)

;  if (!debug) then $
;  	message, /info, string(" lun=", lun, "name =", theName, " tname=", tname)

	if (tname NE 'STRUCT') then begin

	  if (keyword_set(thisFull)) then begin
	      if (tname EQ 'POINTER') then begin
		if ( ptr_valid( thisrec ) ) then begin
                  printf, lun, thename, '=',  $
			thisrec
		endif else begin
                  printf, lun, thename, '= <UNDEFINED>'
		  thisIsNull = 1
		endelse
	      endif else begin
		  if (tname EQ 'OBJREF') then begin
		    if ( NOT obj_valid( thisrec ) ) then begin
		      thisIsNull = 1
                      printf, lun, thename, '= <UNDEFINED>'
		    endif else begin
                     printf, lun, thename, '=',  $
			thisrec
		    endelse
		  endif else begin
                    printf, lun, thename, '=',  $
			thisrec
		  endelse
	      endelse

	  endif else begin
	    if (keyword_set(thisPartial)) then begin
		tname_extra = ''
		if (tname EQ 'POINTER') then begin
		  if ( ptr_valid( thisrec ) ) then begin
		    tvalue = thisrec
;		    print, infolabel, 'WARNING! PTR'
	            tname_extra = '_' + size(*(thisrec), /tname)
		  endif else begin
		    tvalue = thisrec
;		    tvalue = '<UNDEFINED>'
	            tname_extra = '_<UNDEFINED>'
		    thisIsNull = 1
		  endelse
		endif else begin
		  tvalue = thisrec
		  if (tname EQ 'OBJREF') then begin
		    tname_extra = '_' + obj_class(thisrec)
		    if ( NOT obj_valid( thisrec ) ) then begin
		      thisIsNull = 1
	              tname_extra = '_<UNDEFINED>'
		    endif
		  endif
		endelse
;Debug		print, infolabel, 'tname = ', tname

		if (n_elements(thisrec) LE 1) then  begin
		  if (tname EQ 'POINTER')   then begin
                    printf, lun, thename, '=',  $
		      tvalue, '=', $  
		      '<' + tname + tname_extra + '> ' $
;		      '{' + tname + tname_extra + '} ' $
		      + format_size(ptr_array=thisrec)
		  endif else begin
		    if (tname EQ 'OBJREF') then begin
;		      OK = thisRec -> get(everything=aString)
;		      if (OK) then begin
;                        printf, lun, thename,  '=',  $
;		       		 tvalue, '='
;			nlen = n_elements(aString)
;			if (nlen EQ 1) then begin
;			    printf, lun, theIndent, aString
;			endif else begin
;			  for i = 0, (nlen-1) do begin
;			    printf, lun, theIndent, aString[i]
;			  endfor
;			endelse
;		      endif else begin
                        printf, lun, thename,  '=',  $
		       		 tvalue
;		      endelse
		      
		    endif else begin	; something ordinary
                      printf, lun, thename, '=',  $
		      '<' + tname + '> '  + $
;		      '{' + tname + '} '  + $
		    	  strcompress(string(thisrec), /remove_all)
		    endelse
		  endelse 
		   
		endif else begin
		  temp_ptr = ptr_new(thisrec)
                  printf, lun, thename,  $
		    '<' + tname+ tname_extra +  '> ' + $
;		    '{' + tname+ tname_extra +  '} ' + $
		    		format_size(ptr_array=temp_ptr)
		  ptr_free, temp_ptr
		endelse

	    endif
	    
	  endelse
	endif

end		; singleton_show

;==============================================================================
;*
pro strArray_show,  thisStrArray, $
	lun=thisLun, $	; Optionally, write to a file. If absent, to screen
	full=ThisFull
;/

 if (n_elements(thisStrArray) eq 0) then return
 
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
;      print, infolabel, ' Oops. No lun. Setting to ', lun
  endif else begin
      lun = thisLun
  endelse

	  nlen = n_elements(thisStrArray)
 	  if (keyword_set(thisFull)) then begin
           for i = 0, (nlen-1) do begin
              printf, lun, thisStrArray[i]
            endfor
	  endif else begin
	    printf, lun,  '  String array of length ', nlen
	  endelse

end		; strArray_show

;==============================================================================
function convert_month,month
months =	['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct', $
			 'Nov','Dec']
month_num = 1+(where(strupcase(months) eq strupcase(month)))[0]
return,month_num
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*
function julian_date, $
	date_string
; This function converts the date string into a Julian date
; Decompose the date string into individual elements based on
; the separating white spaces.
; Is the primary delimiter a hyphen or a space?
;/

  hyphen = strpos(date_string,'-')
  if hyphen[0] eq -1 then begin	; blank is the delimiter
	wspos1 = strpos(date_string,' ')
	month_string = strmid(date_string,0,wspos1)
	trun_string1 = strtrim(strmid(date_string,wspos1),2)
	wspos2 = strpos(trun_string1,' ')
	day = fix(strtrim(strmid(trun_string1,0,wspos2),2))
	trun_string2 = strtrim(strmid(trun_string1,wspos2),2)
	wspos3 = strpos(trun_string2,' ')
	year = fix(strtrim(strmid(trun_string2,0,wspos3),2))
	time = strtrim(strmid(trun_string2,wspos3),2)
	colpos = strpos(time,':')
	hour = fix(strmid(time,0,colpos))
	minute = fix(strmid(time,colpos+1))
	month_num = convert_month(month_string)
  endif else begin
	wspos1 = strpos(date_string,'-')
	day = strmid(date_string,0,wspos1)
	trun_string1 = strtrim(strmid(date_string,wspos1+1),2)
	wspos2 = strpos(trun_string1,'-')
	month_string = strmid(trun_string1,0,wspos2)
	trun_string2 = strtrim(strmid(trun_string1,wspos2+1),2)
	wspos3 = strpos(trun_string2,' ')
	year = fix(strtrim(strmid(trun_string2,0,wspos3+1),2))
	time = strtrim(strmid(trun_string2,wspos3),2)
	colpos = strpos(time,':')
	hour = fix(strmid(time,0,colpos))
	minute = fix(strmid(time,colpos+1))
	month_num = convert_month(month_string)
  endelse
  julian_date = JULDAY(month_num, day, year, hour, minute)
  return,julian_date
end

;==============================================================================
; Examples are out of date:
;pro davePtr_example,  whichExample, $		; 0 means all
;  whichFile, $
;  partial = thisPartial, $
;  full = thisFull
;  infolabel = "davePtr_example >> "  
;
;end	;;davePtr_example
;==============================================================================
