; $Id: dialog_ow_menu.pro,v 1.10 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
; 	dialog_ow_menu
;
; PURPOSE:
;	Main menu for displaying and modifying a specObj.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created June, 2003. Tanya Riseman
;	Add self.statePtr and self.cleanup. July 23, TR
;	Added Dave Ptr open, save, review and empty buttons. Aug 20, 2003. TMR
;-
; dialog_ow_menu.pro

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	dialog_ow_menu::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro dialog_ow_menu::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	dialog_ow_menu::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro dialog_ow_menu::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	dialog_ow_menu::helpclass
;
; PURPOSE:
;	Helper method for dialog_ow_menu::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro dialog_ow_menu::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass

;==================================================================
;+
; NAME:
; 	dialog_ow_menu::freshen
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_menu can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_menu::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_menu 
;-
;*
pro dialog_ow_menu::freshen, $
	text=thisText
;/
	if (n_elements(thisText) GT 0) then begin
 	  widget_control, self.textID, set_value=thisText
	endif
end


;==================================================================
;*
pro dialog_ow_menu_event, $
	event
;/

  widget_control, event.top, get_uvalue=uObj 


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event_tlb') then begin
      ; Ignore base events.
 	 uname = widget_info(event.id, /uname)
 	 wname = widget_info(event.id, /name)
	 
	 call_method, uname,  (uObj.wObject), event = event

;	if (uname EQ 'eventQuit') then begin
;	  call_method, uname,  (uObj.wObject), event = event
;;          message, /info, 'call_method, uname,  (uObj.wObject), event = event'
;	endif else begin
;	  if (uname EQ 'eventSave') then begin
;	    call_method, uname,  (uObj.wObject), event = event
;	  endif else begin
;;           message, /info, 'Ignoring event_tlb'
;	  endelse
;	endelse
;;;;      (uObj.wObject) -> event, event
    endif else begin
        message, /info, 'Calling method uObj.method'    
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; dialog_ow_menu_event, event

;==================================================================
pro dialog_ow_menu::sensitive,  $
	thisSensitive	; 0 for no, 1 for yes, for key widget IDs.

  for i = 0, (self.nOnOff-1) do begin
    widget_control, self.onOffIDs[i],  sensitive=thisSensitive
  endfor

end

;==================================================================
pro dialog_ow_menu::eventAbout, event=event

  textArr = ['dialog_ow_menu is the master menu for viewing and modifying the ', $
  'specific section of the davePtr data structure, for all instrument types.', $
  'Triple axis data loaded using the TAS/ICP selection contains', $
  'the most complete information, thus far, as it includes units and errors.', $
  'Selecting the "Update DavePtrs common area with focus" button in the',$
  '"VIEW DATA" menu will update the dave data structure, making it available ',$
  'to other applications, including PAN, the DATA BROWSER and TRI-FENESTRE.', $
  '', $
  'Enjoy!', $
  '', $
  'Written by Tanya M. Riseman, NCNR, NIST, 2002-2003.']

  aboutWId = dialog_message(textArr,  $
  	dialog_parent = self.tlbID,$
  	/information, $
	title = 'About specific section', $
	display_name = 'About dialog_ow_menu')
;	title = 'About dialog_ow_menu, $
;	display_name = 'About specific section')

;  aboutWId = widget_text( $
;;  	self.tlbID, $
;  	groupLeader=self.groupLeaderID, $
;	value=textArr, $
;;	units = 20, $	; cm
;;	scr_xsize =15, $ 
;    	uname = 'eventAbout')

  
end

;==================================================================
pro dialog_ow_menu::eventViewDavePtr, event=event

  newFile = 'temporary_dialog_ow_menu_ViewDavePtr.txt'	; temporary file
;  newFile = 'dialog_ow_menu_eventViewDavePtr.txt'	; temporary file


;  if (obj_valid((self.specObj)) then begin
;    tempDavePtr = ((self.specObj) -> getDavePtr())
;  endif else begin
;        message, /info, 'Using stateptr instead to get davePtr'
;	help, self
;	help, (self.StatePtr)
;	print, tag_names(*(self.StatePtr))
	help, (*(self.statePtr)).davePtr
    tempDavePtr = (*(self.statePtr)).davePtr
;  endelse
  
;  davePtr_show, tempDavePtr, /partial

  davePtr_show, tempDavePtr, file=newFile, $
  	/partial
;	/full

  message, /info, string("Wrote to ", newFile)
   
  ; Do we need to free tempDavePtr?
  
  xdisplayfile, newFile, $
  	group = self.groupLeaderID, $
	title = "Dave Ptr contents"

  
end

;==================================================================
pro dialog_ow_menu::eventQuit, event=event
;  message, /info, '...'
;  print, event
  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro dialog_ow_menu::eventModify, event=event
;  message, /info, '...'
;  print, event

  ; a  blocking dialog widget
  modifyWId = obj_new('dialog_ow_tabmodify', (self.specObj), $
  	groupLeader=self.tlbID)
  
 if (obj_valid(modifyWID)) then begin 	; cleanup of dialog box
;     message, /info,  string("obj_destroy, of class ", obj_class(modifyWID))
     obj_destroy, modifyWID
   endif

end

;==================================================================
pro dialog_ow_menu::eventICP, event=event
;  message, /info, '...'
;  print, event

  ; a  blocking dialog widget
  icpWId = obj_new('dialog_ow_tabICP', (self.specObj), $
  	groupLeader=self.tlbID)
  
 if (obj_valid(ICPWID)) then begin 	; cleanup of dialog box
;     message, /info,  string("obj_destroy, of class ", obj_class(ICPWID))
     obj_destroy, ICPWID
   endif

end

;==================================================================
pro dialog_ow_menu::eventViewHeader, event=event
;  message, /info, '...'
;  print, event

  ; a  blocking dialog widget
;  viewHeaderId = dialog_viewHeader(specObj=(self.specObj))
  viewHeaderId = dialog_viewHeader(specObj=(self.specObj), $
  	group_leader = self.tlbID)

end

;==================================================================
pro dialog_ow_menu::eventViewData, event=event
;  message, /info, '...'
;  print, event

  ; a  blocking dialog widget
  ; blocking widget
;  viewHeaderId = dialog_viewData(specObj=(self.specObj))
  viewHeaderId = dialog_viewData(specObj=(self.specObj), $
  	group_leader = self.tlbID)

end

;==================================================================
pro dialog_ow_menu::eventSave, event=event
  message, /info, '...'
;  print, event

  OK = self.SpecObj -> getField(['*', 'so_filename', '*'], /inHeader, qty=so_File)
  if (OK) then begin
    name = so_file
  endif else begin
    OK = self.SpecObj -> getField(['*', 'filename', '*'], /inHeader, qty=dataFile)
    ii = strpos(datafile, '.')
    if (ii GT -1) then strput, datafile, '_', ii
    name = datafile + '.spo'	
  endelse
  
  message, /info, string(' Need pickfile for writing. Filename = ', name)
  oSpec = self.SpecObj		; alias, consistent with specObj__save routine

  save, oSpec, file=name

end


;==================================================================
pro dialog_ow_menu::eventRead, event=event
  message, /info, '...'
;  print, event

  widget_control, self.readNameID, get_value=name
    
  message, /info, string(' Need pickfile for writing. Filename = ', name)
  oSpec = self.SpecObj		; alias, consistent with specObj__save routine


  restore, name, /verbose		;=thisVerbose
  ; Should be restored as oSpec

  if (obj_valid(oSpec)) then  begin
    if (self.destroy) then begin
      obj_destroy, (self.specObj)
    endif		; Otherwise, let the old object reference dangle
    self.SpecObj = oSpec	; replace what is in the specific object
  endif
  
end

;==================================================================
pro dialog_ow_menu::event_davesave_open, event=event
  message, /info, '...'
;  print, event
  debug = 0
;  debug = 1

  compile_opt idl2

  ;----------------------------------------------- 
  ; file dialog for selecting files
  title = 'OPEN: DAVE save file selection for opening'

  datDir = (*!dave_defaults).datdir         ; get current data directory from DAVE env var

  cd, datDir[0], cur=curDir       ; cd to fans data dir

  ;----------------------------------------------------
  allowed_extensions = ['*.dave']	; or *.*

  long_fileList = dialog_pickfile(title=title,$
  	get_path=newDir, $
        filter=allowed_extensions, $
	/must_exist,$
	dialog_parent=(self.groupLeaderID))

  ;cd, curDir                      ; go back to previous dir

  nfiles = (strtrim(long_fileList[0],2) eq '') ? 0 : n_elements(long_fileList)

  if (nfiles le 0) then begin
      message, /info, "No file to open"
      ; return control to main DAVE
    return     ; no need to carry on!
  endif

  davePtr = (*(self.statePtr)).davePtr
  if (debug) then message, /info, string("orig davePtr ID=", davePtr)	; debug
  ;----------------------------------------------------
  ; Assuming that (nfiles EQ 1)
  davePtr_restore, davePtr, (long_fileList[0]), ownerDavePtr = 'main'

  if (debug) then message, /info, string("OPENED file=", (long_fileList[0]))	; debug
;  davePtr_show, davePtr		; debug
  message, /info, string("new davePtr ID=", davePtr)	; debug
  ;----------------------------------------------------

  tempObj = davePtr_getSpecObj(davePtr, from='dialog_ow_menu::event_davesave_open')
  if (obj_valid(tempObj)) then  begin
    message, /info, string("Valid specObj from ", long_fileList[0])
    if (self.destroy) then begin
      obj_destroy, (self.specObj)
    endif		; Otherwise, let the old object reference dangle
    message, /info, string("Old sepcObj ID =", self.specObj, ". New sepcObj ID =", tempObj)
    self.specObj = tempObj ; Swap in the new specific object.
    
     ; Now done in davePtr_restore.
;    (self.specObj) = tempObj
;     OK = (self.SpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
;     	qty=long_fileList[0])
;    (self.specObj) -> setDavePtrID, davePtr		; method does not exist.
    self -> sensitive, 1		; Sensitize certain buttons
  endif else begin
    ErrMsg = string("davePtr does not contain a specObj")
    message, /info, ErrMsg 
    self -> sensitive, 0		; Desensitize certain buttons
  endelse
  
end		; dialog_ow_menu::event_davesave_open

;==================================================================
pro dialog_ow_menu::event_davesave_save, event=event
  message, /info, '...'
;  print, event
  debug = 0
;  debug = 1

  compile_opt idl2

  ;----------------------------------------------- 
  ; file dialog for selecting files
  title = 'OPEN: DAVE save file selection for saving'

  workDir = (*!dave_defaults).workdir         ; get current data directory from DAVE env var

  cd, workDir[0], cur=curDir       ; cd to fans data dir

  ;----------------------------------------------------
;  suggestedName = ''	; variable suggestedName is undefined by default
  if (OBJ_VALID(self.SpecObj)) then begin
    ; Up date the stored name with the save file name.
    OK = (self.SpecObj) -> getField(['*', 'so_filename', '*'], /inHeader, $
	qty = suggestedName)
    periodPos = strpos( suggestedName, '.')
    print,  "suggestedname = ",  suggestedName, " periodPos =", periodPos 
    ; Won't work for "hidden" files starting with "."
    if (periodPos GE 1) then begin
      tempname = strmid(suggestedName, 0, periodPos) + '_' $
      	+ strmid(suggestedName, (periodPos+1), strlen(suggestedName))
;Wrong      suggestedName[periodPos:periodPos] = '_'	
;      	; instrument string .bt2 now _bt2.
    endif else begin
      tempName = suggestedName
    endelse
    
    suggestedName = tempName + '.dave'	; new suffix


;    OK = OK AND (self.SpecObj) -> setField(['*', 'so_path', '*'], /inHeader, $
;	qty = workdir)
;    OK = OK AND (self.SpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
;	qty = suggestedName)
;    if (OK) then begin
;      message, /info, "suggestedpath = ", workdir
;      message, /info, "suggestedName = ", suggestedName 
;      print, "suggestedName = ", suggestedName 
;    endif else begin
;      message, /info, " Fields so_path and so_filename failed to be updated."
;    endelse	

    message, /info, string("suggestedpath = ", (*!dave_defaults).workdir)
    message, /info,  string("suggestedName = ", suggestedName )
  endif
  

;  message, /info, string("suggestedName = ", suggestedName )
;  print, "suggestedName = ", suggestedName 

  ;----------------------------------------------------
  allowed_extensions = ['*.dave']	; or *.*

  long_fileList = dialog_pickfile(title=title,$
  	get_path=newDir, $
        filter=allowed_extensions, $
	file = suggestedName, $
	/write,$
	dialog_parent=(self.groupLeaderID))

  ;cd, curDir                      ; go back to previous dir

  nfiles = (strtrim(long_fileList[0],2) eq '') ? 0 : n_elements(long_fileList)

  if (nfiles le 0) then begin
      message, /info, "No file to save"
      ; return control to main DAVE
    return     ; no need to carry on!
  endif
  
  davePtr = (self.SpecObj) -> getDavePtr()

  if (debug) then message, /info, string("orig davePtr ID=", davePtr)	; debug

;  ;----------------------------------------------------
;  ; Update the name in storage.
;  OK = (self.SpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
;     	qty=long_fileList[0])
;  ; Assuming that (nfiles EQ 1)

  if (OBJ_VALID(self.SpecObj)) then begin
    OK =  (self.SpecObj) -> setField(['*', 'so_path', '*'], /inHeader, $
	qty = newdir)
    if (OK) then message, /info, string("Used path = ", newdir)
	
    pos = strpos(long_fileList[0],!delimiter,/reverse_search) + 1
    length = strlen(long_fileList[0]) - pos
    short_fileList = strmid(long_fileList[0],pos,length)
    OK = OK AND (self.SpecObj) -> setField(['*', 'so_filename', '*'], /inHeader, $
	qty = short_fileList)
    if (OK) then begin
      message, /info, string("Used path = ", newdir)
      message, /info, string("Used Name = ", short_fileList)
    endif else begin
      message, /info, string("Long Used Name = ", long_fileList[0])
      message, /info, " Fields so_path and/or so_filename failed to be updated."
    endelse	
  endif
  


;  ;----------------------------------------------------
  davePtr_save, davePtr, file=(long_fileList[0])

  if (debug) then message, /info, string("SAVED file=", (long_fileList[0]))	; debug
;  davePtr_show, davePtr		; debug
  ;----------------------------------------------------

end		;dialog_ow_menu::event_davesave_save

;==================================================================
pro dialog_ow_menu::event_davesave_empty, event=event

  message, /info, string("Old specObj ID =", self.specObj)

  davePtr = (self.SpecObj) -> getDavePtr()
  message, /info, string("Old Dave Ptr ID =", davePtr)

  DavePtr_empty, DavePtr

;  message, /info, "Automatically quit from menu now (nothing to work with)"
  
  self -> sensitive, 0		; Desensitize certain buttons
  
;  self -> eventQuit, event=event

end		;dialog_ow_menu::event_davesave_empty

;==================================================================
;;*
;pro dialog_ow_menu::event, $
;	event
;;/
;
;  uname = widget_info(event.id, /uname)
;  wname = widget_info(event.id, /name)
;
;  ;------------------------------------------
;   message, /info, string("Should not be here. uname =", uname, " wname=", wname)
;
;end

;=======================================================================
pro dialog_ow_menu::cleanup

  message, /info, "Entered. "		;"Resetting uvalue not needed"
   
  if (self.destroy) then begin 
    message, /info, "Destroying self.specObj"
    if (strlen(self.cleanup) GT 0) then begin
      self.statePtr = ptr_new()	; let the statePtr dangle and 
    				; be cleaned up by someone else
    endif
    obj_destroy, self.specObj
;;;    self.specObj = obj_new()		; new
  endif else begin
    message, /info, "Dangling self.specObj"
    self.specObj = obj_new()	; Let the old reference dangle.
  endelse
  
end

;==================================================================
pro dialog_ow_menu_cleanup, tlb

  message, /info, "Entered'

  widget_control, tlb, get_uvalue=uObj

  ; Grab this stuff before we clean the object up.  
  cleanup = (uObj.wObject) -> get(/cleanup)  
  statePtr = (uObj.wObject)-> get(/statePtr)
;  groupLeaderID = (*(self.statePtr)).notifyIDs[0]
  groupLeaderID = (uObj.wObject) -> get(/groupLeader)	; identical to above for now.
;  tlbID = self.tlbID

  print, "cleanup = ", cleanup
  print, "statePtr =", statePtr
  print, "groupLeaderID = ", groupLeaderID 
  
  if ((uObj.wObject) -> no_block()) then begin	
    obj_destroy, (uObj.wObject)		; calls dialog_ow_menu::cleanup
  endif

  if (n_elements(cleanup) GT 0) then begin
    message, /info, string("calling cleanup routine ", cleanup)
    ; Note: this cleans up self.statePtr
    widget_control, ((*statePtr).notifyIDs[0]), set_uvalue=statePtr
    call_procedure, cleanup, ((*statePtr).notifyIDs[0])	; uvalue should already be statePtr
;    widget_control, groupLeaderID, set_uvalue=statePtr
;    call_procedure, cleanup, groupLeaderID	; uvalue should already be statePtr
  endif 

  if (!debug) then begin
    ptr_free, statePtr
    message, /info, string("Exiting... ptr_valid(statePtr)=", ptr_valid(statePtr))
    if (ptr_valid(statePtr)) then begin
      help, statePtr
      print, (*statePtr)
    endif
  endif
end


;==================================================================
function dialog_ow_menu::get, $	; return value of class field as selected by keyword
  cleanup = boolCleanup, $
  statePtr = boolStatePtr, $
  groupLeader = boolGroupLeader, $
  no_block = boolNo_block
  
  if (keyword_set(boolCleanup)) then return, self.cleanup
  if (keyword_set(boolStatePtr)) then return, self.statePtr
  if (keyword_set(boolGroupLeader)) then return, self.GroupLeaderID
  if (keyword_set(boolNo_block)) then return, self.no_block
  
  return, 0		; error

end

;==================================================================
function dialog_ow_menu::no_block	; return 0/1 for no/yes
  return, self.no_block
end

;==================================================================
function dialog_ow_menu::createWidgets	; return 0/1 for failure/success

 skip_somebuttons = 1
 
 self.nOnOff = 0	; Number of buttons to toggle on/off when 
 			; DavePtr/SpecObj is empty

 ;---------------------------------------------------  
  if (self.groupLeaderID EQ 0L) then begin
     floating = 0
  endif else begin
    floating = 1
  endelse

 ;-------------------------------------------------------------  
  OK = self.SpecObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
;  OK = self.SpecObj -> getField(['*', 'filename', '*'], /inHeader, qty=dataFile)
  if (OK) then begin
      name = datafile
;    nlen = n_elements(dataFile)
;    if (nLen EQ 1) then begin 
;      name = datafile
;    endif else begin
;      name = string(datafile[0], ',..., ', datafile[(nlen-1)])
;    endelse
  endif else begin
    name = ''
  endelse

  aTitle = 'dialog_ow_menu' + ' ' + Name

  ;-------------------------------------------------------------
  if (ptr_valid(self.statePtr)) then begin
    tempStatePtr = self.statePtr
  endif else begin
    tempStatePtr = ptr_new()
  endelse

 ;---------------------------------------------------  
  if ((self.No_block) OR (self.groupLeaderID EQ 0L)) then begin
;    print, "((self.No_block) OR (self.groupLeaderID EQ 0L)) "
    message, /info, "not /modal"
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
 	uname = "dialog_ow_menu", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endif else begin
;    print, " NOT ((self.No_block) OR (self.groupLeaderID EQ 0L))"
    message, /info, "/modal"
    print,  n_elements(aTitle), "=len. aTitle=", aTitle
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
    	/modal, $
  	uname = "dialog_ow_menu", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endelse
;  print, "self.groupLeaderID =", self.groupLeaderID, $
;  	" self.No_block =", self.No_block
  self.tlbID = tlbID

  
  ;-------------------------------------------------------------
  voidID = widget_button(tlbID, value='VIEW SPECIFIC DATA', $
    	uname = 'eventViewData', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventViewData'})
;  viewDataId = dialog_viewData(specObj=(self.specObj))
;;  self.viewDataId= viewDataId
   self.nOnOff = self.nOnOff + 1			; 1
   self.onOffIDs[(self.nOnOff-1)] = voidID
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
  voidID = widget_button(tlbID, value='VIEW SPECIFIC HEADER', $
    	uname = 'eventViewHeader', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventViewHeader'})
;  viewHeaderId = dialog_viewHeader(specObj=(self.specObj))
;;  self.viewHeaderId= viewHeaderId
   self.nOnOff = self.nOnOff + 1			; 2
   self.onOffIDs[(self.nOnOff-1)] = voidID
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
  voidID = widget_button(tlbID, value='MODIFY SPECIFIC DATA', $
    	uname = 'eventModify', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventModify'})
;  modifyWId = obj_new('dialog_ow_tabmodify', groupLeader=tlbID, $
;	  specObj=(self.specObj))
;;  self.modifyWid = modifyWId 
   self.nOnOff = self.nOnOff + 1			; 3
   self.onOffIDs[(self.nOnOff-1)] = voidID
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
  voidID = widget_button(tlbID, value='ICP_MODIFY SPECIFIC DATA', $
    	uname = 'eventICP', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventICP'})
   self.nOnOff = self.nOnOff + 1			; 4
   self.onOffIDs[(self.nOnOff-1)] = voidID
  ;-------------------------------------------------------------

;  ;-------------------------------------------------------------
;  so_treatWId = obj_new('dialog_ow_textInfo', groupLeader=tlbID, dtree, $
;  	/so_treat)
;  self.so_textWid = so_treatWId
;
;  ;-------------------------------------------------------------
;  dave_treatWId = obj_new('dialog_ow_textInfo', groupLeader=tlbID, dtree, $
;  	/dave_treat)
;  self.dave_textWid = dave_treatWId
;  ;-------------------------------------------------------------
;
;  header_reportWId = obj_new('dialog_ow_textInfo', groupLeader=tlbID, dtree, $
;  	/inHeader, verbosity="everything")
;  self.header_reportWid = header_treatWId
;  ;-------------------------------------------------------------
;
;  data_reportWId = obj_new('dialog_ow_textInfo', groupLeader=tlbID, dtree, $
;  	/inHeader, verbosity="some")
;  self.header_reportWid = header_reportWId
;  ;-------------------------------------------------------------

;  holderHorizID0 =widget_base(tlbID, /row)


  ; Note: the next few widgets for eventSave and eventRead are 
  ; intended for the next version of DAVE which is fully objectified. - Tanya
  ;-------------------------------------------------------------
;    voidID = widget_button(tlbID, value='SAVE SPECIFIC OBJ', $	; dave specificObject
;    	uname = 'eventSave', $
;	event_pro='dialog_ow_menu_event',$
;   	uvalue={wobject:self, method:'eventSave'})
;    widget_control, voidID, sensitive=0		; Temporarily turn off

;  ;-------------------------------------------------------------
;    voidID = widget_text(tlbID, $
;;    	title='READ filename=', $	; dave specificObject
;    	/editable, $
;	value='', $
;	units = 2, $	; cm
;	scr_xsize =15, $ 
;    	uname = 'eventRead')
;    widget_control, voidID, sensitive=0		; Temporarily turn off
;    
;    voidID = widget_button(tlbID, value='READ SPECIFIC OBJ', $	; dave specificObject
;    	uname = 'eventRead', $
;	event_pro='dialog_ow_menu_event',$
;    	uvalue={wobject:self, method:'eventRead'})
;    widget_control, voidID, sensitive=0		; Temporarily turn off

  ;-------------------------------------------------------------
;        boxfilemenu = tlbID
        boxFrame = widget_base(tlbID, /column, frame=1) ; TMR Aug 6, 2003.
	void = widget_label(boxFrame, $
		value = 'DAVE data File options:')
        boxfilemenu = widget_base(boxFrame, /row) ; TMR Aug 6, 2003.

;	Major problems, because it destroys self.specobj!
;	void = widget_button(boxfilemenu, value = 'DAVE data open',$
;    		uname = 'event_davesave_open', $
;		event_pro='dialog_ow_menu_event',$
;    		uvalue={wobject:self, method:'vent_davesave_open'})
;        widget_control, void, sensitive=0		; Temporarily turn off

	void = widget_button(boxfilemenu, value = 'DAVE data save',$
    		uname = 'event_davesave_save', $
		event_pro='dialog_ow_menu_event',$
    		uvalue={wobject:self, method:'event_davesave_save'})
        self.nOnOff = self.nOnOff + 1			; 4
        self.onOffIDs[(self.nOnOff-1)] = voidID

	void = widget_button(boxfilemenu, value = 'DAVE Ptr review',$ ; duplicate
    		uname = 'eventVIEWDAVEPTR', $
		event_pro='dialog_ow_menu_event',$
    		uvalue={wobject:self, method:'eventVIEWDAVEPTR'})

;	Major problems, because it destroys self.specobj!
;	try putting it back. TMR. Sept 15, 2003.
	void = widget_button(boxfilemenu, value = 'DAVE ptr empty',$
    		uname = 'event_davesave_empty', $
		event_pro='dialog_ow_menu_event',$
    		uvalue={wobject:self, method:'event_davesave_empty'})
;        widget_control, void, sensitive=0		; Temporarily turn off

;  ;-------------------------------------------------------------
;	Now is 'DAVE Ptr review' button above.
;;    voidID = widget_button(tlbID, value='REVIEW DAVE PTR', $
;    	uname = 'eventVIEWDAVEPTR', $
;	event_pro='dialog_ow_menu_event',$
;    	uvalue={wobject:self, method:'eventVIEWDAVEPTR'})
;

  ;-------------------------------------------------------------
    voidID = widget_button(tlbID, value='ABOUT', $
    	uname = 'eventAbout', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventAbout'})


  ;-------------------------------------------------------------
    voidID = widget_button(tlbID, value='QUIT', $
;    voidID = widget_button(holderHorizID0, value='QUIT', $
    	uname = 'eventQuit', $
	event_pro='dialog_ow_menu_event',$
    	uvalue={wobject:self, method:'eventQuit'})


;  ;-------------------------------------------------------------
;    holderHorizID1 =widget_base(tlbID, /row)
;;    holderHorizID1 = tlbID
;    voidID = widget_label(holderHorizID1, value='Status:', $
;	units=2, $	; cm
;    	scr_xsize = 5)
;    
;    statusID = widget_text(holderHorizID1, value='', $
;    	uname = 'eventStatus', $
;	units=2, $	; cm
;	/scroll, $
;	scr_xsize = 20, $
;	scr_ysize = 2, $
;	event_pro='dialog_ow_menu_event',$
;    	uvalue={wobject:self, method:'eventStatus'})
;    self.statusID = statusID

  ;-------------------------------------------------------------
    widget_control,tlbID,/realize		; make room for all these things.
  ;-------------------------------------------------------------
    if (self.No_block) then begin
      xmanager, 'dialog_ow_menu', /no_block, tlbID, $
    	cleanup="dialog_ow_menu_cleanup"
    endif else begin
      xmanager, 'dialog_ow_menu',  tlbID, $
    	cleanup="dialog_ow_menu_cleanup"
    endelse

;   message, /info, "After xmanager"	; Don't see if it is a blocking widget

  return, 1  
end		;;  dialog_ow_menu::createWidgets


;==================================================================
;+
; NAME:
; 	dialog_ow_menu::init
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_menu can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_menu::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_menu. 
;
;	The programmer must obj_destroy the object upon return from init.
; The dialog widget is always blocking because the widgets it calls
; must also be blocking, since they are constructed on-the-fly from
; the contents of thisSpecObj, which is modified by the widgets.
;-
;*
function dialog_ow_menu::init, $	; returns 0/1 for failure/success
	thisSpecObj, $		; required
	groupLeader = thisGroupLeader, $
	no_block = thisNo_block, $	; Bad idea?.
	destroy = thisDestroy, $; destroy thisSpecObj when this GUI object is
				; destroyed. The default is not to.
	statePtr = thisStatePtr, $; An optional state pointer can be passed
				  ; to calling routine.
	cleanup = thisCleanup	  ; cleanup for the statePtr and calling 
				  ; routine reduceSpecObj.
;/
;	parentWobj = thisParentWObj, $	; required for freshening where
;	no_block = thisNo_block		; Bad idea.

;Obsolete idea:
; 	If the keyword no_block is set, the widget is non-blocking and
; automatically the object of class dialog_ow_menu is destroyed upon 
; closing the dialog box. If not set, the programmer must obj_destroy 
; the object upon return from init. Don't use /no_block
; simulataneously with dialog boxes which depend on the
; contents of the specific object. (/no_block may be broken. See top level 
; base definition.)
	
  debug = 0
;  debug = 1

  if (ptr_valid(thisStatePtr)) then begin
    (*thisStatePtr).GUIobj2 = self	
  endif

;  if (NOT obj_valid(thisSpecObj)) then begin
;    message, /info, string("Invalid object specObj. objRef ID=", thisSpecObj)
;    return, -1
;  endif

  self.specObj = thisSpecObj
  self.destroy = keyword_set(thisDestroy)

  ;-------------------------------------------------------------
  if (keyword_set(thisNo_block)) then begin
    self.no_block = thisNo_block
  endif
 
;  ;-------------------------------------------------------------
;  if (obj_valid(thisParentWobj)) then begin
;    self.parentWobj = thisParentWobj
;  endif 
  ;-------------------------------------------------------------
  if (n_elements(thisGroupLeader) NE 0) then begin
    self.groupLeaderID = thisGroupLeader
  endif else begin
     ; Must have a group_leader with blocking widget
     self.no_block = 0
  endelse 

  ;-------------------------------------------------------------
  if (ptr_valid(thisStatePtr) ) then begin
    if (n_elements(thisStatePtr) GT 0 ) then begin
      self.statePtr = thisStatePtr
    endif
  endif
  ;-------------------------------------------------------------
  
  ;-------------------------------------------------------------
  if (n_elements(thisCleanup) GT 0) then begin
    self.cleanup = thisCleanup
  endif
  ;-------------------------------------------------------------

  if (NOT obj_valid(thisSpecObj)) then begin
    message, /info, string("Invalid object specObj. objRef ID=", thisSpecObj)
    return, -1
  endif else begin
    OK = self -> createWidgets()
    return, OK
;     return, 1
  endelse

  
end

;==================================================================
;*
pro dialog_ow_menu__define
  define = {dialog_ow_menu, $
  	specObj:obj_new(), $	; don't destroy
	destroy: 0, $
	groupLeaderID : 0L, $
	no_block: 0, $
	tlbID:0l, $
	nOnOff: 0, $	; Number of widget IDS listed below in array.
	onOffIDs: [0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L], $
	readnameId: 0L, $
;	textID: 0L, $
	statePtr: ptr_new(), $
	cleanup: '' $
	}
;/

end
