; $Id: dialog_ow_selectfiletype.pro,v 1.3 2003/09/17 19:44:17 tanya Exp $
;+
; NAME:
; 	dialog_ow_selectFileType
;
; PURPOSE:
;	Menu for selecting the triple axis instrument, data reader and
;	the operator (single, concatentate, sum or zip, etc.).
;	This, combined with a dialog_pickfile, is in prepration for  
;	creating a specObj attatched to the DavePtr.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created June, 2003. Tanya Riseman
;	Add self.statePtr and self.cleanup. July 23, TR
;	TMR> Aug 29, 2003. Fixed instrument filter default value.
;-
; dialog_ow_selectFileType.pro

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	dialog_ow_selectFileType::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro dialog_ow_selectFileType::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	dialog_ow_selectFileType::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro dialog_ow_selectFileType::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	dialog_ow_selectFileType::helpclass
;
; PURPOSE:
;	Helper method for dialog_ow_selectFileType::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro dialog_ow_selectFileType::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass

;==================================================================
;*
pro dialog_ow_selectFileType_event, $
	event
;/

  widget_control, event.top, get_uvalue=uObj 


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event_tlb') then begin
      ; Ignore base events.
 	 uname = widget_info(event.id, /uname)
 	 wname = widget_info(event.id, /name)
         message, /info, string('Calling method uname =', uname)
	 call_method, uname,  (uObj.wObject), event = event

    endif else begin
        message, /info, string('Calling method uObj.method =', uObj.method)
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; dialog_ow_selectFileType_event, event
;==================================================================
pro dialog_ow_selectFileType::eventQuit, event=event

;Debug  message, /info, "Entered!"
   ;-------------------------------------------------------------
  self -> eventInstr, event=event  ; Refresh this in case clicking on it.
  self -> eventAction, event=event ; Refresh this in case clicking on it.
  ;-------------------------------------------------------------
 widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro dialog_ow_selectFileType::eventHelp, event=event

 message, /info, "Entered!"
;;  widget_control, self.tlbID, /destroy

;    textfile = 'dialog_ow_selectFileType_help.txt'
    strArray = [$
    'Help for dialog_ow_selectInst', $
    '  ', $
    'Raw or dave file filter: ', $
    '	Only "*.*" works under Linux for IDL V 5.6 and lower.', $
    '	Files loaded using "*.*" are assumed to be raw data files.', $
    '   Selections for raw data files are "*.*", "*.bt*", etc.', $
    '   Selections for dave data files are "*.dave" and "IN_MEMORY", etc.', $
    
    '  ', $
    'Data Reader: ', $
 '	Currently there is only one data reader for Triple Axis ICP data,', $
 '	"func_icp_data_init"', $
 '	Once there are other readers, for other instuments, ', $
 '	there will be more choices.', $
 '	', $
 '      However, "IN_MEMORY" and "*.dave" file selections will restore ', $
 '	a dave file and convert it to a "specObj" if necessary.', $
 '	This conversion data reader is pre-defined.', $ 
 '	Converted data might partially lack units.', $
 '  ', $
 '  ', $
 'Action:', $
 '	SINGLE --- This reads in a single raw (or dave) data file.', $
 '  ', $
 '	CONCATENATE --- This reads in multiple data files and concatenates them.', $
 '		This would be useful for when a planned scan was a bit short and ', $
 '		you wish to treat it and another run as if it were a single scan.', $
 '		All data fields are concatenated.', $
 '  ', $
 '	COMPACTCONCATENATE --- This performs CONCATENATE and then sorts and ', $
 '         	averages duplicate "x" entries for the dependent variable.', $
 ' 	   	This is only suitable for 1D data presently.', $
 '		It then sorts by x, so other parts of DAVE will work properly.', $
 '		You will be given the opportunity to over-ride the choice of x.', $
 '  ', $
 '	ZIP --- This joins N scans of length M into a single 3-D N X M data set. ', $
 '		All data fields which change between files will be zipped.', $
 '  ', $
 '	SUM --- This simply adds N scans of length M into a single 2-D data set of length N', $
 '		It assumes that the "z" data field to be summed is the COUNTS and the MIN.', $
 '		It produces a total intenisty normalized by the monitor. ', $
 '  ', $
 '      FRGD_BKGD --- This adds and subtracts the intensity field of two files.', $ 
 '		It assumes that the "z" data field to be added and subtracted is the INTENSITY,',$
 '		but you will have the opportunity to override it. ', $
 ' ', $
 ' ', $
 ' Note about CONCATENATE, ZIP, and SUM:', $
 '	All these multiple file routines check items in the header for whether', $
 '	they vary between the input files. If so, they become arrays.', $
 '	String arrays remain in the header section.', $
 '	Numeric arrays longer than length 4 are moved into the data section.', $
 ' ', $
 ' Note about CONCATENATE, and ZIP:', $
 '	Items in the data section remain their original size, if their values', $
 '	are the same (to within 1.0e-13) between files. Otherwise, they are', $
 "	either CONCATENATE'd or ZIPped. Note that this means that the user ", $
 '	does NOT have to indicate which data items to concatentate/zip nor how.', $
 '	This automation is useful for multi-column data files. ', $
 '	HOWEVER, THERE IS A PRICE FOR THIS AUTOMATION: Potential confusion when ', $
 '	the set of files are inappropriate for the requested operation.', $
 '	There is no attempt to check ahead of time whether the files ', $
 '	adequately matched. If not, an inordinate number of header items will ', $
 '	get moved over to the data section. Any item in the header or data sections', $
 '	which appears in only some of the files will be left out.', $
 ' ', $
 ' Note about SUM:', $
 '	SUM depends on certain items being present in the data and header sections, ', $
 '	unlike CONCATENATE and ZIP.', $
 '	the SUM method requires the keywords:', $
 "	  RawCounts (which defaults to ['qtyObj', 'COUNTS', '*'], suitable for Triple Axis data.)", $
 "	  RawTime (which defaults to ['qtyObj', 'min', '*'], suitable for Triple Axis data.)", $
 "	  RawMonitor (which defaults to  ['qtyObj', 'monitor', '*'] suitable for Triple Axis data.)", $
 '	A new data item with the name consisting of "Summed " in front of rawCount will appear.', $
 '	A new column for the intensity (counts normlaized by monitor) will also appear.', $
 '	', $
 '	', $
 '	Tanya Riseman, Ph.D. Sept, 2003', $
 '']
 
    temp_ptr = ptr_new(strArray)
    helpbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
	filename=textFile, $
	groupLeader = self.groupLeaderID)
    ptr_free, temp_ptr
    OK = helpbaseObj -> createWidgets()
  
    obj_destroy, self.helpFocusObj
    self.helpFocusObj = helpbaseObj
    
end

;==================================================================
pro dialog_ow_selectFileType::eventInstr, event=event

  widget_control, event.top, get_uvalue=uObj
  statePtr = (uObj.wObject)-> get(/statePtr)
  tempStr = (uObj.wObject) -> get(/instr)  
  (*statePtr).instr = tempStr 

	; do nothing until quit
end

;==================================================================
pro dialog_ow_selectFileType::eventDataReader, event=event

  widget_control, event.top, get_uvalue=uObj
  statePtr = (uObj.wObject)-> get(/statePtr)
  tempStr = (uObj.wObject) -> get(/dataReader)  
  (*statePtr).dataReader = tempStr

	; do nothing until quit
end

;==================================================================
pro dialog_ow_selectFileType::eventAction, event=event

  widget_control, event.top, get_uvalue=uObj
  statePtr = (uObj.wObject)-> get(/statePtr)
  tempStr = (uObj.wObject) -> get(/action)  
  (*statePtr).action = tempStr

	; do nothing until quit
end


;==================================================================
;;*
;pro dialog_ow_selectFileType::event, $
;	event
;;/
;
;  uname = widget_info(event.id, /uname)
;  wname = widget_info(event.id, /name)
;
;  ;------------------------------------------
;   message, /info, string("Should not be here. uname =", uname, " wname=", wname)
;
;end

;=======================================================================
pro dialog_ow_selectFileType::cleanup, $
	destroy = thisDestroy		; Ignored
  ; return the values in the GUI via the state pointer.
  
;  message, /info, "Entered. "		;"Resetting uvalue not needed"

  obj_destroy,  self.helpFocusObj
    
  ptr_free, self.instrOptions
  ptr_free, self.dataReaderOptions
  ptr_free, self.actionOptions
    
end

;==================================================================
pro dialog_ow_selectFileType_cleanup, tlb


  widget_control, tlb, get_uvalue=uObj

  ; Grab this stuff before we clean the object up.  
  cleanup = (uObj.wObject) -> get(/cleanup)  
  statePtr = (uObj.wObject)-> get(/statePtr)
  groupLeaderID = (uObj.wObject) -> get(/groupLeader)	; identical to above for now.

  ;--------------------------------------------------------------
  
  if ((uObj.wObject) -> no_block()) then begin	
    obj_destroy, (uObj.wObject)		; calls dialog_ow_selectFileType::cleanup
  endif

  if (strlen(cleanup) GT 0) then begin
;    message, /info, string("calling cleanup routine ", cleanup)
    ; Note: this cleans up self.statePtr
    widget_control, groupLeaderID, set_uvalue=statePtr
    call_procedure, cleanup, groupLeaderID	; uvalue should already be statePtr
  endif 

end


;==================================================================
function dialog_ow_selectFileType::get, $	; return value of class field as selected by keyword
  instr = boolInstr, $
  action = BoolAction, $
  dataReader = boolDataReader, $
  cleanup = boolCleanup, $
  statePtr = boolStatePtr, $
  groupLeader = boolGroupLeader, $
  no_block = boolNo_block
  
  if (keyword_set(boolInstr)) then begin
     ; hopefully will work both from an exclusive cw_bgroup and a widgt_text.
     widget_control, self.instrID, get_value=returnStr
     ; Either a string from widget_label or an index # frow cw_bgroup.
     if (size(returnStr, /tname) NE 'STRING') then begin
       position = returnStr
       returnStr = (*(self.instrOptions))[position]       
     endif 
     return, returnStr
  endif

  if (keyword_set(boolDataReader)) then begin
     ; hopefully will work both from an exclusive cw_bgroup and a widgt_text.
     widget_control, self.DataReaderID, get_value=returnStr
     ; Either a string from widget_label or an index # frow cw_bgroup.
     if (size(returnStr, /tname) NE 'STRING') then begin
       position = returnStr
       returnStr = (*(self.dataReaderOptions))[position]       
     endif 
     return, returnStr
  endif

  if (keyword_set(boolAction)) then begin
     ; hopefully will work both from an exclusive cw_bgroup and a widgt_text.
     widget_control, self.actionID, get_value=returnStr
     ; Either a string from widget_label or an index # frow cw_bgroup.
     if (size(returnStr, /tname) NE 'STRING') then begin
       position = returnStr
       returnStr = (*(self.actionOptions))[position]       
     endif 
     return, returnStr
  endif


  if (keyword_set(boolCleanup)) then return, self.cleanup
  if (keyword_set(boolStatePtr)) then return, self.statePtr
  if (keyword_set(boolGroupLeader)) then return, self.GroupLeaderID
  if (keyword_set(boolNo_block)) then return, self.no_block
  
  return, 0		; error

end

;==================================================================
function dialog_ow_selectFileType::no_block	; return 0/1 for no/yes
  return, self.no_block
end

;==================================================================
function dialog_ow_selectFileType::createWidgets	; return 0/1 for failure/success

 ;---------------------------------------------------  
  if (self.groupLeaderID EQ 0L) then begin
     floating = 0
  endif else begin
    floating = 1
  endelse

 ;-------------------------------------------------------------  
  aTitle = 'Select raw or dave file(s) and Action'

  ;-------------------------------------------------------------
  if (ptr_valid(self.statePtr)) then begin
    tempStatePtr = self.statePtr
  endif else begin
    tempStatePtr = ptr_new()
  endelse

 ;---------------------------------------------------  
  if ((self.No_block) OR (self.groupLeaderID EQ 0L)) then begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
 	uname = "dialog_ow_selectFileType", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/row)
  endif else begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
	    /modal, $		; Augst 1st, 2003
  	uname = "dialog_ow_selectFileType", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/row)
  endelse
  self.tlbID = tlbID

  
  ;-------------------------------------------------------------
  middleID = widget_base(tlbID, /col, frame=1)
  if (n_elements(*(self.instrOptions)) GT 1)  then begin
    defaultFilter = (*(self.statePtr)).instr
    pos_defFilter = where( (*(self.instrOptions)) EQ (*(self.statePtr)).instr)
    if (pos_defFilter LE 0) then pos_defFilter = 0
    self.instrID = cw_bgroup(middleID, (*(self.instrOptions)), $
    	/col, /exclusive, /no_release,$
	set_value=pos_defFilter, $
 	label_top='Raw or Dave File(s):', $
; 	label_top='Instrument', $
	button_uvalue=(*(self.instrOptions)), $
	uname = 'eventInstr', $
    	uvalue={wobject:self, method:'eventInstr'})

  endif else begin
    voidID = widget_label(middleID, $
    	value='Instrument: ',  $
	uname= 'eventInstr')
    self.instrID = widget_label(middleID, $
    	value=((*(self.instrOptions))[0]),  $
	uname= 'eventInstr')
  endelse
  ;-------------------------------------------------------------
  
  ;-------------------------------------------------------------
  middleID = widget_base(tlbID, /col, frame=1)
  if (n_elements(*(self.dataReaderOptions)) GT 1)  then begin
    self.dataReaderID = cw_bgroup(middleID, (self.dataReaderOptions), $
    	/col, /exclusive, /no_release, set_value=0,  $
 	label_top='Raw data Reader:', $
	button_uvalue=(self.dataReaderOptions), $
	uname = 'eventDataReader', $
    	uvalue={wobject:self, method:'eventDataReader'})

  endif else begin
    voidID = widget_label(middleID, $
    	value='Raw data Reader: ',  $
	uname= 'eventDataReader')
    self.dataReaderID = widget_label(middleID, $
    	value=((*(self.dataReaderOptions))[0]),  $
	uname= 'eventDataReader')
  endelse
  ;-------------------------------------------------------------

  ;-------------------------------------------------------------
  middleID = widget_base(tlbID, /col, frame=1)
  if (n_elements(*(self.actionOptions)) GT 1)  then begin
    self.actionID = cw_bgroup(middleID, (*(self.actionOptions)), $
    	/col, /exclusive,  /no_release, set_value=0,  $
 	label_top='Action:', $
	button_uvalue=(*(self.actionOptions)), $
	uname = 'eventAction', $
    	uvalue={wobject:self, method:'eventAction'})

  endif else begin
    void = widget_label(middleID, $
    	value='Action: ',  $
	uname= 'eventAction')
    self.actionID = widget_label(middleID, $
    	value= ((*(self.actionOptions))[0]),  $
	uname= 'eventAction')
  endelse
  ;-------------------------------------------------------------
  
  ;-------------------------------------------------------------
    voidID = widget_button(tlbID, $
    	value='HELP', $
    	uname = 'eventHelp', $
	event_pro='dialog_ow_selectFileType_event',$
    	uvalue={wobject:self, method:'eventHelp'})

  ;-------------------------------------------------------------
    voidID = widget_button(tlbID, $
    	value='CONTINUE', $
;    	value='DISMISS', $
    	uname = 'eventQuit', $
	event_pro='dialog_ow_selectFileType_event',$
    	uvalue={wobject:self, method:'eventQuit'})

  ;-------------------------------------------------------------
    widget_control,tlbID,/realize		; make room for all these things.
  ;-------------------------------------------------------------
    if (self.No_block) then begin
      xmanager, 'dialog_ow_selectFileType', /no_block, tlbID, $
    	cleanup="dialog_ow_selectFileType_cleanup"
    endif else begin
      xmanager, 'dialog_ow_selectFileType',  tlbID, $
    	cleanup="dialog_ow_selectFileType_cleanup"
    endelse


  return, 1  
end		;;  dialog_ow_selectFileType::createWidgets


;==================================================================
;+
; NAME:
; 	dialog_ow_selectFileType::init
;
; PURPOSE:
;	The compound widgets cw_ow_*.pro used by dialog_ow_selectFileType can create 
; new fields, or remove fields in the specific object self.specObj. 
; In order to update the droplists for all compound widgets, the 
; cw_ow* widgets call this method dialog_ow_selectFileType::freshen, which then 
; calls the methods cw_ow*::freshenWhere in all the compound widgets used 
; in dialog_ow_selectFileType. 
;
;	The programmer must obj_destroy the object upon return from init.
; The dialog widget is always blocking because the widgets it calls
; must also be blocking, since they are constructed on-the-fly from
; the contents of thisSpecObj, which is modified by the widgets.
;-
;*
function dialog_ow_selectFileType::init, $
	groupLeader = thisGroupLeader, $
	instrOptions = thisInstrOptions, $		; Optional.
	dataReaderOptions = thisDataReadOptions, $	; Optional.
	actionOptions = thisActionOptions, $		; Optional.
	statePtr = thisStatePtr, $; An optional state pointer can be passed
				  ; to calling routine.
	cleanup = thisCleanup	  ; cleanup for the statePtr and calling 
				  ; routine reduceSpecObj.
;/

  debug = 0 ; 1

  if (n_elements(thisInstrOptions) GT 0) then begin
    self.instrOptions = ptr_new(thisInstrOptions)
  endif else begin
    self.instrOptions = ptr_new([$
        'IN_MEMORY', '*.dave', $
	 '*.*', $	; Interpreted as a raw file
    	'*.bt*', '*.bt2', '*.bt7', '*.bt9'])
  endelse
  
  if (n_elements(thisDataReaderOptions) GT 0) then begin
    self.dataReaderOptions = ptr_new(thisDataReaderOptions)
  endif else begin
    self.dataReaderOptions = ptr_new(['func_icp_data_init'])
  endelse

  if (n_elements(thisactionOptions) GT 0) then begin
    message, /info, "thisactionOptions = ", thisactionOptions
    self.actionOptions = ptr_new(thisactionOptions)
  endif else begin
    self.actionOptions = ptr_new(['SINGLE', $
    	'CONCATENATE', 'COMPACTCONCATENATE', $
    	'ZIP', 'SUM', 'FRGD_BKGD'])	
  endelse

;  ;-------------------------------------------------------------
;  if (keyword_set(thisNo_block)) then begin
;    self.no_block = thisNo_block
;  endif
 
  ;-------------------------------------------------------------
  if (n_elements(thisGroupLeader) NE 0) then begin
    self.groupLeaderID = thisGroupLeader
  endif ;else begin
;     ; Must have a group_leader with blocking widget
;     self.no_block = 0
;  endelse 

  ;-------------------------------------------------------------
  if (ptr_valid(thisStatePtr) ) then begin
    if (n_elements(thisStatePtr) GT 0 ) then begin
      self.statePtr = thisStatePtr
    endif
  endif
  ;-------------------------------------------------------------
  
  ;-------------------------------------------------------------
  if (n_elements(thisCleanup) GT 0) then begin
    self.cleanup = thisCleanup
  endif
;  ;-------------------------------------------------------------
  
;Aug1st  OK = self -> createWidgets()
  
;  return, OK
  return, 1
  
end

;==================================================================
;*
pro dialog_ow_selectFileType__define
  define = {dialog_ow_selectFileType, $
	groupLeaderID : 0L, $
	no_block: 0, $
	tlbID:0l, $
	instrID: 0L, $
	instrOptions: ptr_new(), $;['bt2', 'bt7', 'bt9']
	dataReaderID: 0L, $
	dataReaderOptions: ptr_new(), $;['func_icp_data_init']
	actionID: 0L, $
	actionOptions: ptr_new(), $;['SINGLE', 'CONCATENATE', 
		; 'COMPACTCONCATENATE','ZIP', 'SUM']; method for specObj
	statePtr: ptr_new(), $
	helpFocusObj: obj_new(), $
	cleanup: '' $
	}
;/

;  	specObj:obj_new(), $	; don't destroy
;	destroy: 0, $
;	readnameId: 0L, $
;	textID: 0L, $

end
