; $Id: dialog_ow_textinfo.pro,v 1.3 2003/09/17 19:44:17 tanya Exp $
; NAME:
; 	dialog_ow_textinfo
;
; PURPOSE:
;
;  Allows the user to scroll through a write-protected text array and
;  optionally write it to disk.
; 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created Aug, 2003. Tanya Riseman
;-

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	dialog_ow_textInfo::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro dialog_ow_textInfo::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $          ; string.
  lun=thisLun, $        ; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs      ; Optionally print contents of pointers.
;/
          @generic__print_frag.pro
  
end             ; ::print_obj
;======== HELP method common to all classes ===============================
;;+
; NAME:
; 	dialog_ow_textInfo::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;;-
;;*
pro dialog_ow_textInfo::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;;/

          @generic__help_obj_frag.pro
end             ; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	dialog_ow_textInfo::helpclass
;
; PURPOSE:
;	Helper method for dialog_ow_textInfo::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro dialog_ow_textInfo::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

          @generic__helpClass_frag.pro
end             ; ::HelpClass

;==================================================================
;+
; NAME:
; 	dialog_ow_textInfo::freshen
;
; PURPOSE:
;	Updates the contents of the compound widgets used by dialog_ow_textInfo.
;	In principle, could be called by another widget or dialog. 
;	Currently, dialog_ow_textInfo is used as blocking dialog box.
;	Eventually, we could have a single box open with text in it,
;	with different routines updating it.
;-
;*
pro dialog_ow_textInfo::freshen, $
	text=thisText, $
	text_ptr=thisText_ptr, $
	status = thisStatus
;/
	if (n_elements(thisText) GT 0) then begin
 	  widget_control, self.textID, set_value=thisText
	endif

	if (n_elements(thisText_ptr) GT 0) then begin
 	  widget_control, self.textID, set_value= (*thisText_ptr)
	endif

	if (n_elements(thisStatus) GT 0) then begin
 	  widget_control, self.statusID, get_value= oldStatus
	  if (strlen(oldStatus) EQ 0) then begin
	    newStatus = thisStatus
	  endif else begin
	    newStatus = [oldStatus,thisStatus] 
	  endelse
 	  widget_control, self.statusID, set_value= newStatus
	endif

end


;==================================================================
;*
pro dialog_ow_textInfo_event, $
	event
;/

  widget_control, event.top, get_uvalue=uObj 


  if (obj_valid(uObj.wObject)) then begin
    if (uObj.method eq 'event_tlb') then begin
      ; Ignore base events.
 	 uname = widget_info(event.id, /uname)
 	 wname = widget_info(event.id, /name)	; BUTTON
;          message, /info, $
;	  	string('call_method, uname=', uname, ',  (uObj.wObject), event = event')
;          message, /info, $
;	  	string('call_method, wname=', wname)
	  call_method, uname,  (uObj.wObject), event = event
    endif else begin
        message, /info, 'Calling method uObj.method'    
      call_method, uObj.method, (uObj.wObject), event = event
    endelse
    
  endif else begin
    message, /info, "Error. Invalid object reference"
  endelse
  
  
end			;; dialog_ow_textInfo_event, event
;==================================================================
pro dialog_ow_textInfo::eventQuit, event=event

  ptr_free, self.text_ptr
  widget_control, self.tlbID, /destroy
  
end

;==================================================================
pro dialog_ow_textInfo::eventSave, event=event
  message, /info, '...'
;  print, event

  widget_control, self.textID, get_value=theText

  title = 'dialog_ow_textInfo ' + self.filename
  workDir = (*!dave_defaults).workdir	; [0] 
;  help,  (*!dave_defaults)
;  tags = tag_names( (*!dave_defaults))
;  print, "tag_names( (*!dave_defaults)) = ", tags
  help, workdir
  message, /info, string("workdir = ", workdir)
  
  newpath = workDir
  temp_filename =   string(workdir, self.filename) 
  long_fileList = dialog_pickfile($
;  	title=self.filename,$
  	get_path=newpath, $
;	filter = '*.*', $
        path=workdir, $
	file = temp_filename, $
	/write, $
	dialog_parent=self.groupLeaderID)
self.path = newpath 

  if (strlen(long_fileList) EQ 0) then begin
    self -> freshen, status = "No file selected"
    return
  endif else begin
;    self -> freshen, $
;    	status = string("Saved file ",long_FileList)
  endelse

;  self -> freshen, status = ""
  openw, lun, long_fileList, /get_lun , error = errCode    
  IF (errCode NE 0) then begin
    self -> freshen, $
    	status = !ERROR_STATE.MSG
;    	status = string(long_FileList, ".  OPEN Error. ",!ERROR_STATE.MSG)

  endif else begin

    printf, lun, theText

    close, lun, exit_status=exit_status
    free_lun, lun
    if (exit_status NE 0) then begin
      self -> freshen, $
     	status = string(long_FileList, ".  Error. CLOSE exit_status=", exit_status)
    endif else begin
      self -> freshen, $
    	status = string("Saved file ",long_FileList)
    endelse
  endelse 
   
end


;=======================================================================
pro dialog_ow_textInfo::cleanup
 
;  if (self.destroy) then begin 
;;    message, /info, "Destroying self.specObj"
;    obj_destroy, self.specObj
;  endif else begin
;;    message, /info, "Dangling self.specObj"
;    self.specObj = obj_new()	; Let the old reference dangle.
;  endelse

  ptr_free, self.text_ptr
  
end

;==================================================================
pro dialog_ow_textInfo_cleanup, tlb

  message, /info, "Entered'

  widget_control, tlb, get_uvalue=uObj
  if ((uObj.wObject) -> no_block()) then begin	
    obj_destroy, (uObj.wObject)		; calls dialog_ow_textInfo::cleanup
  endif
end


;==================================================================
function dialog_ow_textInfo::no_block	; return 0/1 for no/yes
  return, self.no_block
end

;==================================================================
function dialog_ow_textInfo::createWidgets	; return 0/1 for failure/success

 ;---------------------------------------------------  
  if (self.groupLeaderID EQ 0L) then begin
     floating = 0
  endif else begin
    floating = 1
  endelse

 ;-------------------------------------------------------------  
 name = self.filename
;  OK = self.SpecObj -> getField(['*', 'so_filename', '*'], /inHeader, qty=dataFile)
;  if (OK) then begin
;    name = datafile
;  endif else begin
;    name = ''
;  endelse

  aTitle = 'dialog_ow_textInfo' + ' ' + Name

 ;---------------------------------------------------  
  if ((self.No_block) OR (self.groupLeaderID EQ 0L)) then begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
  	uname = "dialog_ow_textInfo", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endif else begin
    tlbID = widget_base(group_leader=self.groupLeaderID, floating=floating, $
    	/modal, $
  	uname = "dialog_ow_textInfo", $
  	title = aTitle, $	
	uvalue={wobject:self, method:'event_tlb'}, $
  	/column)
  endelse
  self.tlbID = tlbID

  
;  holderHorizID0 =widget_base(tlbID, /row)
  ;-------------------------------------------------------------
  
  self.textID = widget_text(tlbID, $
	value= (*(self.text_ptr)), $
;	editable = 0, $
 	uname = 'eventText', $				; should never generate an event
	uvalue = {wobject:self, method:'eventText'}, $	; should never generate an event
	event_pro='dialog_ow_textInfo_event',$		; should never generate an event
	/scroll, $
	xsize = 100, $	; characters
;	xsize = 80, $	; characters
	ysize = 24 $	; lines
	)

  ;-------------------------------------------------------------
    voidID = widget_button(tlbID, value='SAVE', $
    	uname = 'eventSave', $
	event_pro='dialog_ow_textInfo_event',$
    	uvalue={wobject:self, method:'eventSave'})

  ;-------------------------------------------------------------
    holderHorizID0 =widget_base(tlbID, /column, frame=1)
;    holderHorizID0 =widget_base(tlbID, /row)
  
    void = widget_label( holderHorizID0,$
	 value='Status:')

    self.statusID = widget_text( holderHorizID0,$
	 value='', $
	/scroll, $
 	uname = 'eventStatus', $			; should never generate an event
	uvalue = {wobject:self, method:'eventStaus'}, $	; should never generate an event
	event_pro='dialog_ow_textInfo_event',$		; should never generate an event
	 xsize = 100, $
;	 xsize = 80, $
	 ysize = 5)

  ;-------------------------------------------------------------

;    voidID = widget_button(tlbID, value='QUIT', $
    voidID = widget_button(tlbID, value='DISMISS', $
    	uname = 'eventQuit', $
	event_pro='dialog_ow_textInfo_event',$
    	uvalue={wobject:self, method:'eventQuit'})

  ;-------------------------------------------------------------
;    holderHorizID1 =widget_base(tlbID, /row)
;;    holderHorizID1 = tlbID
;    voidID = widget_label(holderHorizID1, value='Status:', $
;	units=2, $	; cm
;    	scr_xsize = 5)
;    
;    statusID = widget_text(holderHorizID1, value='', $
;    	uname = 'eventStatus', $
;	units=2, $	; cm
;	/scroll, $
;	scr_xsize = 20, $
;	scr_ysize = 2, $
;	event_pro='dialog_ow_textInfo_event',$
;    	uvalue={wobject:self, method:'eventStatus'})
;    self.statusID = statusID

   ;-------------------------------------------------------------
    widget_control,tlbID,/realize		; make room for all these things.
  ;-------------------------------------------------------------
    if (self.No_block) then begin
      xmanager, 'dialog_ow_textInfo', /no_block, tlbID, $
    	cleanup="dialog_ow_textInfo_cleanup"
    endif else begin
      xmanager, 'dialog_ow_textInfo',  tlbID, $
    	cleanup="dialog_ow_textInfo_cleanup"
    endelse

;   message, /info, "After xmanager"	; Don't see if it is a blocking widget

  return, 1  
end		;;  dialog_ow_textInfo::createWidgets


;==================================================================
;+
; NAME:
; 	dialog_ow_textInfo::init
;
; PURPOSE:
;  Allows the user to scroll through a write-protected text array and
;  optionally write it to disk.
;
; The freshen method updates the contents of the compound widgets used by dialog_ow_textInfo.
;	In principle, could be called by another widget or dialog. 
;	Currently, dialog_ow_textInfo is used as blocking dialog box.
;	Eventually, we could have a single box open with text in it,
;	with different routines updating it.
;
; 	If the keyword no_block is set, the widget is non-blocking and
; automatically the object of class dialog_ow_textInfo is destroyed upon 
; closing the dialog box. If not set, the programmer must obj_destroy 
; the object upon return from init. Don't use /no_block
; simulataneously with dialog boxes which depend on the
; contents of the specific object.
;
; When this is a blocking widget:
; "Realize" this object dialog by call the method createWidgets()
; right after you  intialize the object. Save the object reference
; and destroy it once you are finished with it, if it is still valid,
; in case the cleanup routine dialog_ow_textInfo_cleanup was not called
; automatically by the window manager.
;
;-
;*
function dialog_ow_textInfo::init, $
	text_ptr= thisText_ptr, $
	filter = thisFilter, $
	path= thisPath, $
	filename=thisFilename, $
	groupLeader = thisGroupLeader
;/

;	thisSpecObj, $		; required for filename
;	parentWobj = thisParentWObj, $	; required for freshening where
;	destroy = thisDestroy	; destroy thisSpecObj when this GUI object is
;				; destroyed. The default is not to.
	
  debug = 0	;1

;  if (NOT obj_valid(thisSpecObj)) then begin
;    message, /info, "Invalid object specObj."
;    return, -1
;  endif

  if (n_elements(thisText_ptr) EQ 0 ) then begin
    message, /info, "Requires a text array. (n_elements(thisText_ptr) EQ 0 )"
    return, -1
  endif
  if (NOT ptr_valid(thisText_ptr)) then begin
    message, /info, "Requires a text array. (NOT ptr_valid(thisText_ptr))"
    return, -1
  endif
  if (n_elements((*thisText_ptr)) EQ 0 ) then begin
    message, /info, "Requires a text array.(n_elements((*thisText_ptr)) EQ 0)"
    return, -1
  endif
  
  ptr_free, self.text_ptr
  self.text_ptr = ptr_new(*(thisText_ptr))

  if (n_elements((*thisText_ptr)) EQ 0 ) then begin
    message, /info, "Requires a text array.(n_elements((*thisText_ptr)) EQ 0)"
    return, -1
  endif
  if (n_elements(thisFilter) GT 0) then self.filter = thisFilter
;  if (n_elements(thisPath) GT 0) then self.path = thisPath
  if (n_elements(thisFilename) GT 0) then self.filename = thisFilename
  
;  self.specObj = thisSpecObj
;  self.destroy = keyword_set(thisDestroy)

  ;-------------------------------------------------------------
  if (keyword_set(thisNo_block)) then begin
    self.no_block = thisNo_block
  endif
 
;  ;-------------------------------------------------------------
;  if (obj_valid(thisParentWobj)) then begin
;    self.parentWobj = thisParentWobj
;  endif 
  ;-------------------------------------------------------------
  if (n_elements(thisGroupLeader) NE 0) then begin
    self.groupLeaderID = thisGroupLeader
  endif else begin
     ; Must have a group_leader with blocking widget
     self.no_block = 0
  endelse 
  ;-------------------------------------------------------------

;  OK = self -> createWidgets() 
;  return, OK

  return, 1
  
end

;==================================================================
;*
pro dialog_ow_textInfo__define
  define = {dialog_ow_textInfo, $
	groupLeaderID : 0L, $
	no_block: 0, $
	tlbID:0l, $
	textID: 0L, $
	text_ptr: ptr_new(), $
	filter:'', $
	path:'', $
	filename: '', $
	statusID:0l $
	}
;/

end
