; $Id: dialog_viewheader.pro,v 1.4 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
; 	dialog_viewheader
;
; PURPOSE:
;	Menu for viewing the "header" section of a specObj.
;	Also allows the "focus" to be modified.
;	Is a traditional GUI, not a object widget.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE GUI main dialog routines
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	...
;
; MODIFICATION HISTORY:
;	Created April, 2003. Tanya Riseman
; NOTE: TMR. Sept 5, 2003. The legend button selection for DESCRIP=
; does not work properly in IDL v6.0 possibly due to an array of length 1 
; (v 6.0) is being returned instead of a scalar (v 5.6). The DESCRIP
; field in the DAVE common structure is only used by Richard Azuah's
; DAVE Browser and not by Tanya Riseman's SpecObj. If the DESCRIP 
; field is abandonned, this DESCRIP selection feature could be removed.
;
;-
; dialog_viewheader.pro
; Tanya Riseman, NIST Center for Neutron Research. April, 2003.


;===================================================================

pro dialog_viewheader_cleanup_event, event
;    infolabel = "dialog_viewheader_cleanup_event >> "  

    widget_control, event, get_uvalue=pInfo
 
    obj_destroy,  (*pInfo).textHeaderObj
    obj_destroy,  (*pInfo).textHeaderReportObj
    obj_destroy,  (*pInfo).textTreatDaveObj
    obj_destroy,  (*pInfo).textTreatSOObj
    
    ptr_free, pInfo
    
end
;===================================================================

pro dialog_viewheader_dismiss_event, event
    infolabel = "dialog_viewheader_dismiss_event >> "  
    widget_control, event.id, get_uvalue=pInfo

;    ptr_free, pInfo		; done in dialog_viewheader_cleanup_event
    
    widget_control, event.top, /destroy
end

;===================================================================

pro dialog_viewheader_tagleg_event, event
    infolabel = "dialog_viewheader_tagleg_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    widget_control, event.id, get_value=thistag_leg

    if (thisTag_leg EQ "<none>") then begin
      specObj -> setFocus, descripLegend = ''
    endif else begin
      specObj -> setFocus, descripLegend = thisTag_leg
    endelse
end
;===================================================================

pro dialog_viewheader_all_event, event
    infolabel = "dialog_viewheader_all_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
;    OK = specObj -> getField(['*', 'filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    strArray = specObj -> textInfo(/inHeader, /getAll, /showAnnotations)
;    strArray = specObj -> textInfo(/inHeader, /getEverything, /showAnnotations)

;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(strArray, /info, $
;	   	title=(datafile + ' complete header'))

    textfile = 'header_' + dataFile + '_txt'
    temp_ptr = ptr_new(strArray)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
;	filter = '*.*', $
;	path= thisPath, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()

    obj_destroy, (*pinfo).textHeaderObj
    (*pinfo).textHeaderObj = textbaseObj
end


;===================================================================

pro dialog_viewheader_annotated_event, event
    infolabel = "dialog_viewheader_annotated_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
;    OK = specObj -> getField(['*', 'filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    strArray = specObj -> textInfo(/inHeader, /getAll, /onlyAnnotations)

;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(strArray, /info, $
;	   	title=(datafile + ' annotated header'));

    textfile = 'headerReport_' + dataFile + '_txt'
    
    temp_ptr = ptr_new(strArray)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
;	filter = '*.*', $
;	path= thisPath, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()
    
    obj_destroy, (*pinfo).textHeaderReportObj
    (*pinfo).textHeaderReportObj = textbaseObj
    
end


;===================================================================

pro dialog_viewheader_treatment_event, event
    infolabel = "dialog_viewheader_annotated_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''


    OK = specObj -> getField(['*', 'DavePtrTreatment', '*'], /inHeader, qty=treatmentArr)
;    OK = specObj -> getField(['*', 'Treatment history', '*'], /inHeader, qty=treatmentArr)
    if (NOT OK) then  treatmentArr = 'No treatment history stored in header'

;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(treatmentArr, /info, $
;	   	title=(datafile + ' Dave Ptr treatment history'))

    textfile = 'treatDave_' + dataFile + '_txt'
    
    temp_ptr = ptr_new(treatmentArr)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()

    obj_destroy, (*pinfo).textTreatDaveObj
    (*pinfo).textTreatDaveObj = textbaseObj

end


;===================================================================

pro dialog_viewheader_SO_treatment_event, event
    infolabel = "dialog_viewheader_annotated_event >> "  

    widget_control, event.top, get_uvalue=pInfo

    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''


    OK = specObj -> getField(['*', 'SpecObjTreatment', '*'], /inHeader, qty=treatmentArr)
    if (NOT OK) then  begin
      treatmentArr = 'No treatment history stored in header'
    endif
    
;Old way. Aug 13, 2003 TMR
;    textBaseID = dialog_message(treatmentArr, /info, $
;	   	title=(datafile + ' Specific Object treatment history'))

    textfile = 'treatSO_' + dataFile + '_txt'
       
    temp_ptr = ptr_new(treatmentArr)
    textbaseObj = obj_new( 'dialog_ow_textInfo', $
	text_ptr= temp_ptr, $
	filename=textFile, $
	groupLeader = (*pinfo).group_leader)
    ptr_free, temp_ptr
    OK = textbaseObj -> createWidgets()

    obj_destroy, (*pinfo).textTreatSOObj
    (*pinfo).textTreatSOObj = textbaseObj
end


;===================================================================

pro dialog_viewheader_updateButtons, $
	event, $
	tabbed_ID, $
	tabbedNone_ID, $
;Silly	displayLabels = thisDisplayLabels		;;, $
;Silly	toggleDisplay = thisToggleDisplay, $
	bounceOff = thisBounceOff

    infolabel = "dialog_viewheader_updateButtons >> "
    debug = 0

    widget_control, event.top, get_uvalue=pInfo
    specObj =(*pinfo).specObj		; Just temporary alias

    whichDCT = widget_info(event.id, /uname); 'title', 'cut', 'descrip', 'info'.
    if (debug) then print, infolabel, "whichDCT= '", whichDCT, "'"

    whichTab = widget_info((*pInfo).wTab, /tab_current)
    if (debug) then print, infolabel , "whichTab = ", whichTab

    widName = widget_info(event.id, /name) 
    if (widName EQ 'TAB') then goto, skip_over

    selectBut = event.value
    iselectBut = where((*pinfo).menutxt EQ selectBut, count)
    if (debug) then print, infolabel, "selectBut = ", selectBut, " iselectBut = ", iselectBut

    inclusiveButtons =  (norm(tabbedNone_ID EQ 0L) GT 0)
    
    widget_control, event.id, get_value=arr_tabITag_DCT  ; array [0,1,0,0..]  

    inclusiveButtons =  (norm(tabbedNone_ID EQ 0L) GT 0)
    
;    itemStart = (*pInfo).arr_itemStart[whichTab]
;    itemEnd = (*pInfo).arr_itemEnd[whichTab]

    if (keyword_set(thisBounceOff) ) then begin
      itemStart = (*pInfo).arr_itemStart[whichTab]
      itemEnd = (*pInfo).arr_itemEnd[whichTab]
      arrVals = make_array(/int, value=0, (itemEnd -itemStart +1))
      widget_control, tabbed_ID(whichTab), set_value=arrVals
      return
    endif

;    if (n_elements(thisDisplayLabels) GT 0) then begin
;      itemStart = (*pInfo).arr_itemStart[whichTab]
;      itemEnd = (*pInfo).arr_itemEnd[whichTab]
;      widget_control, tabbed_ID(whichTab), get_name=arrButLabels
;      arrButLabels[(iselectBut -itemStart)] = thisToggleDisplay[buttonOn]
;      
;;      arrButLabels = make_array(/string, value='', (itemEnd -itemStart +1))
;      widget_control, tabbed_ID(whichTab), set_name=arrButLabels
;    endif
    
     npages = (*pinfo).npages
     if (NOT inclusiveButtons) then begin
       for iTab = 0, (NPAGES-1) do begin
    	 itemStart = (*pInfo).arr_itemStart[iTab]
    	 itemEnd = (*pInfo).arr_itemEnd[iTab]
	 arrVals = make_array(/int, value=0, (itemEnd -itemStart +1))
	 if (((itemStart LE iselectBut) AND (iselectBut LE itemEnd)) $
	 	AND (selectBut NE '<none>') )then begin
	   arrVals[ (iselectBut - itemStart)] = 1
	 endif
	 widget_control, tabbed_ID(iTab), set_value=arrVals
         if (selectBut NE '<none>') then begin
	   widget_control, tabbedNone_ID(iTab), set_value=0
	 endif
       endfor
     endif else begin
     endelse
     
   
  skip_over:
  
end		;;  dialog_viewheader_updateButtons

;===================================================================

pro dialog_viewheader_tagdct_event, event

    infolabel = "dialog_viewheader_tagdct_event >> "  
    debug = 0

  MAX_LEGENDS = 4

    widget_control, event.top, get_uvalue=pInfo
    specObj =(*pinfo).specObj		; Just temporary alias

    OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
    if (NOT OK) then  datafile = ''

    widName = widget_info(event.id, /name) 
;    if (debug) then print, infolabel, "widName = '", widName, "'"
    if (widName EQ 'TAB') then goto, skip_over
    
    whichDCT = widget_info(event.id, /uname); 'tag_t', 'tag_d', 'tag_c','tag_info'.
    if (debug) then print, infolabel, "whichDCT= '", whichDCT, "'"

    thisTag_DCT = event.value
    buttonOn = event.select
        
    if (debug) then print, infolabel, " event = ", event
    

      if (thisTag_DCT eq '<none>') then thisTag_DCT = ''
      case (whichDCT) of 
        ;---------------------------------------------------------------
        'report': begin
;        'annotate': begin
           OK = specObj -> setTagField( ['*', thisTag_DCT, '*'], /inHeader, $
	   		annotate=buttonOn)	
	   print, infolabel, thisTag_DCT, OK, "annotate = ", buttonOn
  	   dialog_viewheader_updateButtons, event, $
	  	(*pInfo).tabbed_aID, (*pInfo).tabbedNone_aID			;;, $
;Silly		displayLabels=(*pInfo).footlabels, $
;Silly		toggleDisplay=[' ', '*']
		
	   end
	   
        ;---------------------------------------------------------------
        'descrip': begin
	   specObj -> setFocus, descriptags =['*', thisTag_DCT, '*']
	   ; Update text widget on the top
	   widget_control, (*pInfo).infoDescrID, $
	     	set_value=thisTag_DCT
	   ; Update the legend selection buttons on the top.
	   ; Always default to no 
    	   itag_DCT = where((*pinfo).menutxt EQ thistag_DCT, count)

	   for j = 1, ((*pInfo).nlegendsArr[itag_DCT]) do begin
    	     aString = (*pInfo).legendTxt[itag_DCT, (j-1)]
    	     vString = aString + strmid('        ', 0, (8-n_elements(aString)))	; 8 spaces
	     widget_control, (*pInfo).legButID[j], $
	     	set_value=vstring
;	     	set_value=(*pinfo).legendtxt[itag_DCT, (j-1)]
	   endfor
      	   widget_control, (*pInfo).legButId[0], /set_button	; <none>
	   for j = ((*pInfo).nlegendsArr[itag_DCT] +1), (MAX_LEGENDS) do begin
	     widget_control, (*pInfo).legButID[j], $
	     	set_value='        '		; 8 spaces
;      	    widget_control, (*pInfo).legButId[j], get_button=butVal	; not valid
;	    if (butVal) then begin
;      	      widget_control, (*pInfo).legButId[0], /set_button	; <none>
;	    endif	     
	   endfor

	  dialog_viewheader_updateButtons, event, $
	  	(*pInfo).tabbed_dID, (*pInfo).tabbedNone_dID
	  end
	  
        ;---------------------------------------------------------------
        'cut': begin
	   specObj -> setFocus, cuttags =['*', thisTag_DCT, '*']
	    widget_control, (*pInfo).infoCutID, $
	     	set_value=thisTag_DCT

  	   dialog_viewheader_updateButtons, event, $
	  	(*pInfo).tabbed_cID, (*pInfo).tabbedNone_cID
	  end

        ;---------------------------------------------------------------
        'title': begin
	   specObj -> setFocus, maintitletags =['*', thisTag_DCT, '*']
	    widget_control, (*pInfo).infoTitleID, $
	     	set_value= thisTag_DCT
  	   dialog_viewheader_updateButtons, event, $
	  	(*pInfo).tabbed_tID, (*pInfo).tabbedNone_tID
	  end


        ;---------------------------------------------------------------
        'info': begin
	   if ((thisTag_DCT EQ '') OR (thisTag_DCT EQ '<NONE>')) then begin
	     strArray = specObj -> textInfo(/inHeader, /getAll, /showTags)
	   endif else begin
	       strArray = specObj -> textInfo(/inHeader, $
	       	  position=['*',thisTag_DCT,'*'], /getEverything, /showTags)
	   endelse 

	   textBaseID = dialog_message(strArray, /info, $
	   	 title=(dataFile + ' header.' + thisTag_DCT))
;	   	 title=(dataFile + ' partial header'))

  	   dialog_viewheader_updateButtons, event, $
	  	(*pInfo).tabbed_infoID, (*pInfo).tabbedNone_infoID, $
		/bounceOff
	   end



	 else: begin
	   OK = 0
	   print, infolabel, '1st Case, whichDCT= "', whichDCT, '"'
	   end
      endcase

      if (not OK) then begin
        print, infolabel, 'Failed set(', whichDCT,' = ', thisTag_DCT, ')'
	return 
      endif

  skip_over:
    
end


;=========================================================================
pro viewheader_tabButtons, $
	thisSelection, $	; Which button is selected. -1 for <none> or array
	thisTab, $		; Which tab is selected.
	wparentID, $		; Widget ID of the base widget
	itemStart, itemEnd, $	; range for the tab itab
	thisLabelTop, $
	stringLabels, $
	tabLabels, $
	displayLabels = thisDisplayLabels, $
	MAX_PER_TAB, $
	tabbed_ID, $		; array of widget ID's, one per tabbed page
	tabbedNone_ID, $	; array of widget ID's, one per tabbed page
	exclusive=thisExclusive, $
	nonexclusive = thisNonexclusive

  infolabel = "viewheader_TabButtons >> " 
  debug = 0

 toggleNonExcl = 0
 if (keyword_set(thisNonexclusive)) then begin
   toggleNonExcl = 1
 endif

;-----------------------------------------------------------------------
 arrBut = make_array(/int, (itemEnd-itemStart+1), value = 0)
 if (debug) then print, infolabel, "n_elements(arrBut) = ", n_elements(arrBut)
  
 nStrLabels = n_elements(stringLabels)
 nTabLabels = n_elements(tabLabels)
 
 localSelection = thisSelection
 somethingSelected = 0
 if (n_elements(thisSelection) gt 1) then begin
   somethingSelected =1
 endif else begin
   if (thisSelection GE 0) then somethingSelected =1
 endelse
 
; if (thisSelection EQ (itemEnd  + 1) ) then begin
;   localSelection = -1
; endif 

 if (somethingSelected) then begin	
; if (localSelection NE -1) then begin	

    if  (n_elements(thisSelection) gt 1) then begin 
      arrBut = thisSelection[itemStart:itemEnd]
    endif else begin
      
      ipageT = floor(float(localSelection) / float(MAX_PER_TAB))
      if (thisTab EQ ipageT) then begin	; select the button
        if (debug) then begin 
      	  print, infolabel, "(localSelection - itemStart) = ", (localSelection - itemStart), $
      	    localSelection, itemStart, itemEnd
        endif
        arrBut[(localSelection - itemStart)] = 1
      endif 		; else begin			; report which tab page its on.
    endelse

 endif
 
 if (n_elements(thisDisplayLabels) EQ 0) then begin
   ; Have to have something other than a null string, 
   ; otherwise the buttons come out tiny and don't line up with other columns.
   displayLabels = make_array(/string, (itemEnd-itemStart +1), value= ' ')
 endif else begin
   displayLabels = thisDisplayLabels[itemStart:itemEnd]
 endelse
 
 if (debug) then begin
   print, infolabel, "button_uvalue=", stringLabels[itemStart:itemEnd]
   print, infolabel, "# of buttons = ", $
 		n_elements(stringLabels[itemStart:itemEnd])
;   print, infolabel, "button_uvalue=", [stringLabels[itemStart:itemEnd], TabLabels]
;   print, infolabel, "# of buttons = ", $
; 		n_elements([stringLabels[itemStart:itemEnd], TabLabels])
   print, infolabel, "labeled as ", DisplayLabels
   print, infolabel, "# of displayed labels = ", n_elements( displayLabels)
;;   print, infolabel, "set_value = ", iValBut

; print, "iinfo = ", iinfo
 print, "tabLabels=", tabLabels
 print, "# of displayed tabs=",  n_elements( tabLabels)
 
 endif
 
 localParent = widget_base(wparentID, /col)

;  print, infolabel, "thisTab = ", thisTab

  if ( toggleNonExcl ) then begin
;    print, infolabel, " release
    tabbed_ID[thisTab] = cw_bgroup(localParent, $
 	/col,  $
	/nonexclusive,$
   	displayLabels,   $
 	label_top= thisLabelTop,$
;;;	/no_release,  $
	frame=1, $
	set_value = arrBut, $
	button_uvalue=[stringLabels[itemStart:itemEnd], TabLabels], $
	uname= thisLabelTop)

    tabbedNone_ID(thisTab) = 0L
  endif else begin
;    print, infolabel, " /No_release
    tabbed_ID[thisTab] = cw_bgroup(localParent, $
 	/col,  $
	/nonexclusive,$
   	displayLabels,   $
 	label_top= thisLabelTop, $
	/no_release, $
	frame=1, $
	set_value = arrBut, $
	button_uvalue=[stringLabels[itemStart:itemEnd], TabLabels], $
	uname= thisLabelTop)

       tabbedNone_ID[thisTab] = cw_bgroup( localParent, $
  	/col, /nonexclusive, $
	'<none>', $
  	set_value = (NOT somethingSelected), $
;;;  	set_value = (localSelection EQ -1), $
	button_uvalue='<none>', $
	uname= thisLabelTop)
  endelse
    
end			; pro viewheader_tabButtons
;=========================================================================

;=========================================================================
;*
function dialog_viewheader, $
	group_leader=group_leader, $
	davePtr = thisdavePtr, $	; Use either davePtr or specObj
	specObj = thisSpecObj
;/

;;;	file=datafile, $

;Assume that the DavePtr is already filled up with ICP triple axis data
; Non-modal widget. Copying nmocurve.pro.

  infolabel = "dialog_viewheader >> " 
  debug = 0 
  debug2 = 1

 MAX_LEGENDS = 4
; MAX_PER_TAB = 10	; If more than 16 items, use multiple tabs
 MAX_PER_TAB = 16	; If more than 16 items, use multiple tabs
 
  if ((n_elements(thisDavePtr) EQ 0) AND $
  	(n_elements(thisSpecObj) EQ 0)) then begin
    print, infolabel, "Require one of the 2 keywords: davePtr, specObj"
    return, -1
  endif

  if (n_elements(thisDavePtr) EQ 0) then begin
    davePtr = thisSpecObj -> getDavePtr()
  endif else begin
    davePtr = thisDavePtr
  endelse

  if (n_elements(thisSpecObj) EQ 0) then begin
    specObj = (*(*(*DavePtr).dataStrPtr).SpecificPtr).specObj
  endif else begin
    specObj = thisSpecObj
  endelse

  if (NOT obj_valid(specObj)) then begin
    print, infolabel, "Invalid object specObj."
    return, -1
  endif
  
  if (n_elements(group_leader) EQ 0) then begin
    group_leader = 0L
    floating = 0
  endif else begin
    floating = 1
  endelse

  inputOutputID = widget_base(group_leader=group_leader, floating=floating, /row)
  ;; Check if there is an instance of this program running
  registerName = 'dialog_viewheader' + strtrim(string(group_leader), 2)
  if (xregistered(registerName)) then begin
    print, infolabel, registerName,  $
    	' Already registered. Only single instance allowed.'
    return, -1
  endif else begin
    if (debug) then print, infolabel, registerName,  ' just registered.'
  endelse
 
 child = widget_base(inputOutputID, /column)
 datafile = ''

;Short if (not ptr_valid(DavePtr)) then DavePtr = ptr_new()
 OK = specObj -> getField(['*', 'SO_filename', '*'], /inHeader, qty=dataFile)
 print, infolabel, "datafile=", datafile
 help, datafile
 print, n_elements(datafile)
 
 if (NOT OK) then  datafile = 'UNKNOWN_FILENAME'

; tempObj =  specObj -> get(position=['*', 'filename', '*'] , /inHeader)
; if (obj_valid(tempObj)) then begin
;    OK = tempObj -> get(qty=datafile)
; endif else begin
;   datafile = 'UNKNOWN FILENAME'
; endelse

 nLabels = 0
 specObj -> getLabel, /inHeader, $
 	label=dataLabels, $
	sizeString=sizeStringArray, $
	count=nLabels

;--------------------------------------------------------------------------
 voidID = Widget_label(child, value=datafile)
 prerowID0 = widget_base(child, /row, $ ;;, modal=model, $
 	event_pro='dialog_viewheader_tagleg_event', uvalue=pInfo)
; voidID = Widget_label(prerowID0, value=datafile)
 voidID = Widget_Button(prerowID0, Value='View whole header', $
 	event_pro='dialog_viewheader_all_event')

 voidID = Widget_Button(prerowID0, Value='View header report', $
 	event_pro='dialog_viewheader_annotated_event')
; voidID = Widget_Button(prerowID0, Value='View annotated header', $
; 	event_pro='dialog_viewheader_annotated_event')

 voidID = Widget_Button(prerowID0, Value='DavePtr treatment', $
 	event_pro='dialog_viewheader_treatment_event')

 voidID = Widget_Button(prerowID0, Value='Specific Obj treatment', $
 	event_pro='dialog_viewheader_SO_treatment_event')

 modal = 1

;------------------------------------------------------------------------ 
 ; Put spacer values '......' in to get the widths big enough
 legbutID = make_array(/long, (MAX_LEGENDS+1), value=0)	; Up to 4 labels allowed

 prerowID = widget_base(child, /row, $ ;;, modal=model, $
 	event_pro='dialog_viewheader_tagleg_event', uvalue=pInfo)
 voidID = Widget_label(prerowID, value="Title=")
 infoTitleID = Widget_text(prerowID, value=string("............."))
 voidID = Widget_label(prerowID, value="Cut=")
 infoCutID = Widget_text(prerowID,   value=string("........"))

 prerowID2 = widget_base(child, /row, $ ;;, modal=model, $
 	event_pro='dialog_viewheader_tagleg_event', uvalue=pInfo)
 voidID = Widget_label(prerowID2, value="Descript=")
 infoDescrID = Widget_text(prerowID2, value=string("........"))
 legrowID = widget_base(prerowID2, /row, /exclusive,  $
 	event_pro='dialog_viewheader_tagleg_event', uvalue=pInfo)

 legButID[0] = Widget_Button(legRowID, Value='<none>')
 for j = 1, (MAX_LEGENDS) do begin
 	; When go from short legends to long legends, sometimes the
	; button labels get truncated. But dynamic_resize is very slow.
;     legButID[j] = Widget_Button(legRowID, Value='', /dynamic_resize)
     legButID[j] = Widget_Button(legRowID, Value='.....')
 endfor
 widget_control, legButID[0], /set_button
 
 max_nlabels  = nLabels

 workingLabels  = make_array(/string, (max_nlabels))
 InfoLabels  = make_array(/string, (max_nlabels))

;May22 workingLabels  = make_array(/string, (max_nlabels+1))
;May22 InfoLabels  = make_array(/string, (max_nlabels+1))

;; Temporary. WIll be stored somewhere eventually.
; a_array  = make_array(/int, (max_nlabels+1),  value = 0)
; a_array[1] = 1
; a_array[3] = 1
 
;Didn't work
  ; First legend is "<none>'
; leglabels = make_array(/string, (max_nlabels+1), (MAX_LEGENDS+1))	; Up to 4 labels allowed
 leglabels = make_array(/string, (max_nlabels), MAX_LEGENDS)	; Up to 4 labels allowed
 nlegendsArr = make_array(/int, (max_nlabels))
; leglabels = make_array(/string, (max_nlabels+1), MAX_LEGENDS)	; Up to 4 labels allowed
; nlegendsArr = make_array(/int, (max_nlabels+1))

 itaga = -1	; <none> selected option
 itagD = -1	; <none> selected option
 itagC = -1	; <none> selected option
 itagT = -1	; <none> selected option
 itagi = -1
; itagD = max_nLabels	; <none> selected option
; itagC = max_nLabels	; <none> selected option
; itagT = max_nLabels	; <none> selected option
; itagi = max_nLabels

;; if (debug) specObj -> show, /focus		;; looks fine!

; tagD = ['?d', '?d', '?d']
; tagC = ['?c', '?c', '?c']
;; tagT = ['?t', '?t', '?t']

 OK = specObj -> getFocus(descripTags=tagD)
 if (NOT OK) then begin
   print, infolabel, "Failed specObj -> getFocus(descripTags=tagD)
 endif else begin
   OK = specObj -> getFocus(descripLegend=tagLeg)
 endelse
 OK = specObj -> getFocus(cutTags=tagC)
 if (NOT OK) then begin
   print, infolabel, "Failed specObj -> getFocus(cutTags=tagC)
 endif
 OK = specObj -> getFocus(mainTitleTags=tagT)
 if (NOT OK) then begin
   print, infolabel, "Failed specObj -> getFocus(mainTitleTags=tagT)
 endif

; if (debug) then print, infolabel, 'tagD = ', tagD
; if (debug) then print, infolabel, 'tagC = ', tagC
; if (debug) then print, infolabel, 'tagT = ', tagT

 itagD = specObj -> getPosition(tagD, /inHeader)
 itagC = specObj -> getPosition(tagC, /inHeader)
 itagT = specObj -> getPosition(tagT, /inHeader)

; if (debug) then print, infolabel, 'itagD = ', itagD
; if (debug) then print, infolabel, 'itagC = ', itagC
; if (debug) then print, infolabel, 'itagT = ', itagT

; print, infolabel, 'sizeStringArray =', sizeStringArray
 
 workingLabels[0:(nlabels-1)] = dataLabels
; workingSize[0:(nlabels-1)] = sizeStringArray
;May workingLabels[nlabels] = '<none>'
; workingSize[nlabels] = '[?]'
; infoLabels[nlabels] = ' '

 
 ncount = specObj -> count(/inHeader)
;Wrong ncount = specObj -> get (/count, /inHeader) 

if (debug) then  print, "Count = ", ncount
; print, "nLabels = ", nLabels  
; print, "nelem(infolabels) = ", n_elements(infoLabels)

 for i = 0, (nLabels-1) do begin
   OK = specObj -> getTagField(i, /inHeader, label=aString) 
;Nope   tempObj = specObj -> get(position=i, /inHeader) 
;Nope   OK = tempObj -> get (name=aString)   	; Short. Only the name.
;Long   OK = tempObj -> get (all=aString)   	; Long. Summary of contents.
;   if (debug) then print, OK,  i, "all=", aString
;;   OK = tempObj -> get (everything=aString)   
;;   if (debug) then print, i, "everything=", aString

   if (OK) then begin
     InfoLabels[i] = aString
   endif else begin
     InfoLabels[i] = '??' 
   endelse

;Wrong   tempObj = specObj -> getField(i, /inHeader, legend = localLegend)
   OK = specObj -> getField(i, /inHeader, legend = localLegend)
   if (OK) then begin
     nLegends = n_elements(localLegend)
   endif else begin
     message, /info, string("LocalLegend ", format_list(localLegend), $
     	" not found. nLegends=0")
     nLegends = 0
   endelse
   legendRank = size(localLegend, /n_dimensions)
   if (legendRank GE 2) then begin
     ; Probably only going to have multi-dimensional legends
     ; when SUM'ming inappropriate files together.
     temp = localLegend[0:*, 0]		; just use the first row
     localLegend = temp	; just use the first row
     nLegends = n_elements(localLegend)
     legendRank = size(localLegend, /n_dimensions)
   endif
   nlegendsArr[i] = nLegends
;   if (debug2) then begin
;     help, legLabels
;     help, localLEgend
;     print, i, "=i. nLegends = ", nLegends
;     if (nlegends GT 3) then begin
;       print, "localLegend = ", localLegend
;     endif
;   endif
   for j = 0, (nLegends-1) do begin
     if (legendRank GE 2) then begin
       legLabels[i, j] = localLegend[j, 0]	; just use the first row
     endif else begin
       legLabels[i, j] = localLegend[j]
     endelse
;     legLabels[i, j] = localLegend[j]
   endfor
   if (debug) then print, infolabel, "nlegends=", nlegends, " MAX_LEGENDS=", MAX_LEGENDS
   if (nLegends GE 1) then begin
     if (debug) then print, infolabel, "legLabels[", i, " , 0:(nLegends-1)]=", $
     		legLabels[i, 0:(nLegends-1)] 
   endif
   for j = (nLegends), (MAX_LEGENDS-1) do begin
     legLabels[i, j] = ' '
   endfor
;   if (nLegends LT MAX_LEGENDS) then begin
;     print, infolabel, "legLabels[", i, " , nlegends:(MAX_LEGENDS-1)]=", $
;   	legLabels[i, nlegends:(MAX_LEGENDS-1)] 
;   endif
 endfor


; Use -1 for now.
; if (itagD eq -1) then itagD = max_nLabels	; <none> selected option
; if (itagC eq -1) then itagC = max_nLabels	; <none> selected option
; if (itagT eq -1) then itagT = max_nLabels	; <none> selected option
 
 if (debug) then print, infolabel, 'itagD,C,T =', itagD, itagC, itagT
 if (debug) then print, infolabel, "ncount = ", ncount
 
 npages = ceil(float(ncount) / float(MAX_PER_TAB))

 print, infolabel, " npages=", npages, " ncount=", ncount
 
 mapLabels = make_array(/string, (npages))
 mapBlanks = make_array(/string, (npages), value=' ')
; mapLabels = make_array(/string, (npages+1))
; mapBlanks = make_array(/string, (npages+1), value=' ')
 itemStart = 0
 for i = 0, (npages-1) do begin
   itemEnd = itemStart + MAX_PER_TAB -1
   mapLabels[i] = string("<TAB ", $
   	strcompress(string(itemStart, '-', itemEnd), /remove_all), ">")
   itemStart = itemStart + MAX_PER_TAB
 endfor
; mapLabels[npages] = "<NONE>"

;-----------------------------------------------------------------------
; cw_bground widget IDs in the tab environment
  tabbed_aID = make_array(/long, npages)    
  tabbed_tID = make_array(/long, npages)    
  tabbed_cID = make_array(/long, npages)    
  tabbed_dID = make_array(/long, npages)   
  tabbed_infoID = make_array(/long, npages)   
  tabbedNone_aID = make_array(/long, npages)    
  tabbedNone_tID = make_array(/long, npages)    
  tabbedNone_cID = make_array(/long, npages)    
  tabbedNone_dID = make_array(/long, npages)   
  tabbedNone_infoID = make_array(/long, npages)   	; new
  ; I don't think I need the infoID. 
;-------------------------------------------------------------------
 
 arr_itemStart = make_array(/int, npages)
 arr_itemEnd = make_array(/int, npages)


;Silly  footLabels = make_array(/string, ncount, value=' ')
  itaga_arr = make_array(/int, ncount, value=0)
  for j = 0, (ncount -1) do begin
     OK = specObj -> getTagField(j, /inHeader, annotate=isAnnotated)
     if (IsAnnotated) then begin
;Silly       footLabels[j] = '*'	
       itaga_arr[j] = 1
     endif
  endfor
;  print, infolabel, "footlabels = ", footLabels
 
 tabbyID = widget_base(child, /col, modal=model)
 wTab = widget_tab(tabbyID)
 itemStart = 0
 for i = 0, (npages-1) do begin
   itemEnd = itemStart + MAX_PER_TAB -1
   if (itemEnd GE ncount) then itemEnd = ncount -1
   arr_itemStart[i] = itemStart 
   arr_itemEnd[i] = itemEnd 
   if (debug) then print, infolabel, "tab=", i,$
   	 " start=", itemStart, " end=", itemEnd
   rowID = widget_base(wTab, /row, $
 	event_pro='dialog_viewheader_tagdct_event', $
	title=strcompress(string(itemStart, '-', itemEnd), /remove_all))
	
	if (debug) then print, infolabel, "iTagT = ", iTagT, " iTab=", i
	if (debug) then print, infolabel, "itemStart, itemEnd =", itemStart, itemEnd
	if (debug) then print, infolabel, "rowId = ", rowId 
	if (debug) then print, infolabel, "label string = ", 'title'
	if (debug) then print, infolabel, "workingLabels = ", workingLabels
	if (debug) then print, infolabel, "mapLabels = ", mapLabels
	if (debug) then print, infolabel, "MAX_PER_TAB =", MAX_PER_TAB
	if (debug) then print, infolabel, "tabbed_tID =", tabbed_tID


  print, infolabel, " entering viewheader_tabButtons: itaga=", itaga, " itab=", i
  viewheader_tabButtons, $
   	itaga_Arr, $			;; itaga, 	$		;; -1, $
	i, $
	rowId, $
	itemStart, itemEnd, $
	'report', $
;	'annotate', $
	workingLabels, $
	mapLabels, $
;Silly	displayLabels = footLabels, $
	MAX_PER_TAB, $
	tabbed_aID, $
	tabbedNone_aID, /nonexclusive

   viewheader_tabButtons, $
   	iTagT, $
	i, $
	rowId, $
	itemStart, itemEnd, $
	'title', $
	workingLabels, $
	mapLabels, $
;;	displayLabels = infoLabels, $	; Comment out later. Defaults to blank buttons
	MAX_PER_TAB, $
	tabbed_tID, $
	tabbedNone_tID, /exclusive

	
  viewheader_tabButtons, $
   	iTagC, $
	i, $
	rowId, $
	itemStart, itemEnd, $
	'cut', $
	workingLabels, $
	mapLabels, $
;;	displayLabels = infoLabels, $	; Comment out later. Defaults to blank buttons
	MAX_PER_TAB, $
	tabbed_cID, $
	tabbedNone_cID, /exclusive
		
   viewheader_tabButtons, $
   	iTagD, $
	i, $
	rowId, $
	itemStart, itemEnd, $
	'descrip', $
	workingLabels, $
	mapLabels, $
;;	displayLabels = infoLabels, $	; Comment out later. Defaults to blank buttons
	MAX_PER_TAB, $
	tabbed_dID, $
	tabbedNone_dID, /exclusive

		
   viewheader_tabButtons, $
   	iTagI, $
	i, $
	rowId, $
	itemStart, itemEnd, $
	'info', $
	infoLabels, $
	mapLabels, $
	displayLabels = infoLabels, $
	MAX_PER_TAB, $
	tabbed_infoID, $
	tabbedNone_infoID, /nonexclusive

   itemStart = itemStart + MAX_PER_TAB
 endfor
 

  if (group_leader EQ 0L) then begin
	local_leader = inputOutputID 
  endif else begin
	local_leader = group_leader
  endelse

  info = {group_leader:local_leader, specObj:specObj, $
  	menuTxt:workingLabels, $
	wTab:wTab, $	; ID of tab widget
	MAX_PER_TAB:MAX_PER_TAB, $
	tabbed_aID:tabbed_aID, $
	tabbed_tID:tabbed_tID, tabbed_cID:tabbed_cID, $
	tabbed_dID:tabbed_dID, tabbed_infoID:tabbed_infoID, $ 
	tabbedNone_aID:tabbedNone_aID, $
	tabbedNone_tID:tabbedNone_tID, tabbedNone_cID:tabbedNone_cID, $
	tabbedNone_dID:tabbedNone_dID, tabbedNone_infoID:tabbedNone_infoID, $ 
	npages:npages, $  
	arr_itemStart:arr_itemStart, $
	arr_itemEnd:arr_itemEnd, $
	infoTitleID:infoTitleID, $
	infoCutID: infoCutID, $
	infoDescrID: infoDescrID, $
	legButID:legButID, $
	legendTxt:legLabels, nlegendsArr:nlegendsArr, $
	textHeaderObj: obj_new(), $
	textHeaderReportObj: obj_new(), $
	textTreatDaveObj: obj_new(), $
	textTreatSOObj: obj_new() $
	}
;	, $
;	footLabels: footLabels}
  pInfo = ptr_new(info)

;------------------------------------------------------------------------ 
; Now put in proper values into the info boxes
 widget_control, (*pInfo).infoTitleID, set_value= TagT[1]
 widget_control, (*pInfo).infoCutID, set_value= TagC[1]
 widget_control, (*pInfo).infoDescrID, set_value= TagD[1]
 
 print, infolabel, "Updating legends. ItagD=", itagD
 if (iTagD GE 0) then begin
;   print, "# of legends=", ((*pInfo).nLegendsArr[iTagD]), " tagLeg = '", tagLeg, "'"
   for j = 1,((*pInfo).nLegendsArr[iTagD]) do begin
     aString = (*pInfo).legendTxt[itagD, (j-1)]
     vString = aString + strmid('        ', 0, (8-n_elements(aString)))	; 8 spaces
     print, infolabel, j, "vstring = '", vstring, "'"
     widget_control, (*pInfo).legButId[j], $
   	set_value= vString
     nlen = strlen(aString)
;     print, "strcmp( '", astring, "', '", tagLeg, "', ", nlen, ") =",$
;     		  strcmp(aString, tagLeg, nlen)
     if (strcmp(aString, tagLeg, nlen)) then begin
       widget_control, (*pInfo).legButId[j], /set_button
     endif
   endfor
 endif
 
 voidID = Widget_button(child, $
 	value="Dismiss",$
 	uname='Dismiss', $
 	event_pro='dialog_viewheader_dismiss_event', uvalue=pInfo)

  Widget_Control, inputOutputID, /Realize

  widget_control,  inputOutputID, set_uvalue=pInfo

  xmanager, registerName,  inputOutputID, $
  	event_handler= 'dialog_viewheader_tagdct_event', $ 
  	cleanup = 'dialog_viewheader_cleanup_event'
  
;--------------------------------------------------------------------------
  
  return, inputOutputID  
end			; function dialog_viewheader

