; $Id: focusobj__define.pro,v 1.3 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
;       focusObj
;
; PURPOSE:
;
;	There is a "basic" container class DAVE_tagContainer, based upon
; the class IDL_container. There is a DAVE-specific class called
; specObj, which has as its definition
;  define = {specObj, $
;  	target: obj_new(), $	; class specObj
;  	store: obj_new(), $	; class DAVE_tagContainer
;	focus: obj_new(), $	; class focusObj 
;	davePtr: ptr_new(), $ ; DAVE pointer, including this object in specificPtr.
;	wID: 0L $ 		; Widget ID associated with this object.
;	}
; specObj has a nearly unorganized container called self.store
; which is an object of class DAVE_tagContainer.
; specObj also has a focus object of class focusObj, which indicates 
;    1. which node of <specObj>.store is currently the "header" section,
;    2. which node of <specObj>.store is currently the "data" section,
;    3. which leafs in the "data" section are currently the x, y, and z arrays.
;    4. which leaf in the "header" section is currently the desciptor.
;    5. Some additional things (see definition of focusObj).
;
;  The idea is that the specObj stores ALL the information one might 
; need as data, log files, for calculations, etc., for a generic data set,
; while the focus object specifies the most relevent information needed
; to produce a single plot.
; 
; Note: There is NO cleanup routine for this class, even though it has object
; references. Cleanup should be done by the container class DAVE_tagContainer.
;
; Requires that the contained objects include the following methods:
;	get, getDimension, getOneValue, show
;
; Key methods: SET, GET. PLOT
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE aaplication programming.
;
; OBJECT DEFINTION
; <LI><A HREF="#FOCUSOBJ__DEFINE_arguments">FOCUSOBJ__DEFINE_arguments</A>
;
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	....
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; EXAMPLE
;
; SEE ALSO:
;	<LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; MODIFICATION HISTORY:
;   Created	February, 2003. Tanya Riseman
;   TMR> Aug 6, 2003. changed file name to all lowercase to help BUILD's RESOLVE_ALL
;
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;======================================
;*
function focusObj::init, $
   target = thisTarget, $		; class specObj
   container = thisContainer, $		; class DAVE_tagContainer
   headerObj = thisHeaderObj, $		; class DAVE_tagContainer
   dataObj = thisDataObj, $		; class DAVE_tagContainer
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $		; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags		; optional   
;/
;   descripObj = thisDescripObj, $	; optional.
;   xObj = thisXObj, $			; optional
;   yObj = thisYObj, $			; optional
;   zObj = thisZObj			; optional   
   
  infolabel = "focusObj::init >> "  

  self -> set, $
   target = thisTarget, $
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $	; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags			; optional   

  message, /info, string(" self.target = ", thisTarget)
  
  return, 1
  
end		;; focusObj::init

;======================================
;+
; NAME:
; 	focusObj::clone
;
; PURPOSE:
;	Copy the object into another object.
;	Note: the object references thisContainer, thisHeaderObj and
;	dataObj must be provided for the new cloned specObj.
;-
;*
function focusObj::clone, $
   target = thisTarget, $	; class specObj
   container = thisContainer, $	; class DAVE_tagsContainer
   headerObj = thisHeaderObj, $ ; class DAVE_tagsContainer
   dataObj = thisDataObj	; class DAVE_tagsContainer
;/
   
  infolabel = "focusObj::clone >> "  

  OK = self -> get( $
;   container = thisContainer, $
;   headerObj = thisHeaderObj, $
;   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $	; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags)			; optional   

  clone = obj_new('focusObj', $
   target = thisTarget, $
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $	; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags)			; optional   
      
  self -> set, $
;   container = thisContainer, $
;   headerObj = thisHeaderObj, $
;   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $	; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags			; optional   


  return, clone
  
end		;; focusObj::clone

;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	focusObj::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro focusObj::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $		; string.
  lun=thisLun, $	; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs	 ; Optionally print contents of pointers.
;/
	  @generic__print_frag.pro
  
end		; ::print_obj

;======== HELP method common to all classes ===============================
;+
; NAME:
; 	focusObj::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number 
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;-
;*
pro focusObj::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;/
;  ; Note: The following keywords are only used by internal recrusion call.
;  methodsOnly=methodsOnly,  $
;  helpProStr=helpProStr, helpFuncStr=helpFuncStr, class=thisClass, level=level, $
; 	lun=thisLun 	; Optionally, write to a file. If absent, to screen

	  @generic__help_obj_frag.pro
end		; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	focusObj::helpclass
;
; PURPOSE:
;	Helper method for tagObj::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro focusObj::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/
; ; Note: The following keywords are only used by internal recrusion call.
;   helpProStr=helpProStr, helpFuncStr=helpFuncStr, class=thisClass, level=level, $
; 	lun=thisLun 	; Optionally, write to a file. If absent, to screen

	  @generic__helpClass_frag.pro
end		; ::HelpClass

;========= End of methods common to all classes =============================

;================================================================================
;*
pro  focusObj::set, $
   target = thisTarget, $		; instance of class specObj
   container = thisContainer, $		; instance of class dave_tagContainer
   headerObj = thisHeaderObj, $		; instance of class dave_tagContainer
   dataObj = thisDataObj, $		; instance of class dave_tagContainer
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $		; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags			; optional   
;/

  infolabel = "focusObj::set >> "
  debug = 0
  
  localVerbose = 0
  if (debug) then begin
    localVerbose = 1
  endif
  
  xrank = -1
  yrank = -1
  zrank = -1
  
  if (n_elements(thisTarget) gt 0) then begin
    self.target = thisTarget
  endif

  if (n_elements(thisContainer) gt 0) then begin
    self.container = thisContainer
    self.headerObj = obj_new()
    self.dataObj = obj_new()
  endif

  if (n_elements(thisheaderObj) gt 0) then begin
    self.headerObj = thisHeaderObj
  endif

  if (n_elements(thisDataObj) gt 0) then begin
    self.dataObj = thisDataObj
  endif

  if (n_elements(thisMainTitleTags) gt 0) then begin
    self.mainTitleTags = thisMainTitleTags
  endif

  if (n_elements(thisDataObj) gt 0) then begin
    self.dataObj = thisDataObj
  endif

  if (n_elements(thisMainTitleTags) gt 0) then begin
    self.mainTitleTags = thisMainTitleTags
  endif

  if (n_elements(thisDescripTags) gt 0) then begin
    self.descripTags = thisDescripTags
  endif
  if (n_elements(thisDescripLegend) gt 0) then begin
    self.descripLegend = thisDescripLegend
  endif

  if (n_elements(thisCutTags) gt 0) then begin
    self.cutTags = thisCutTags
  endif

  if (n_elements(thiszTags) gt 0) then begin
    self.zTags = thiszTags
  endif

  if (n_elements(thisXTags) gt 0) then begin
    self.xTags = thisXTags
  endif
  if (n_elements(thisyTags) gt 0) then begin
    self.yTags = thisyTags
  endif

  if ( (n_elements(thiszTags) gt 0) OR (n_elements(thisXTags) gt 0) OR $
       (n_elements(thisyTags) gt 0)) then begin
    xrank = 0
    yrank = 0
    xdim = [0,0,0,0]
    ydim = [0,0,0,0]
    zOK = 0
;    zrank = 0
    if (debug) then print, infolabel, $
    	"tempZObj = self.dataObj -> get(position = self.zTags)"
    tempZObj = self.dataObj -> get(position = self.zTags, verbose=localVerbose)
    if (obj_valid(tempZObj)) then begin
      zOK = tempZObj -> getDimension(rank=zrank, dim=zdim)     
    endif 
    xOK = 0
    tempXObj = self.dataObj -> get(position = self.xTags, verbose=localVerbose)
    if (obj_valid(tempXObj)) then begin
      xOK = tempXObj -> getDimension(rank=xrank, dim=xdim)     
    endif 
    yOK = 0
    tempYObj = self.dataObj -> get(position = self.yTags, verbose=localVerbose)
    if (obj_valid(tempYObj)) then begin
      yOK = tempYObj -> getDimension(rank=yrank, dim=ydim)     
    endif 
 
    if (debug) then begin
      print, infolabel, "xOK, yOK, zOK = ", xOK, yOK, zOK 
      print, infolabel, "xrank, yrank, zrank=", xrank, yrank, zrank 
;      print, infolabel, "xdim, ydim, zdim=", xdim, ydim, zdim
    endif

    ;--------------------------------------------------------------
    ; Any "nonsense" combination of array sizes will produce
    ; a blank for self.xtype and/or self.ytype.
    if (NOT xOK) then self.xtype = '' 
    if (NOT yOK) then self.ytype = '' 
    if (n_elements(zrank) EQ 0) then begin		;; debug
      print, infolabel, 'obj_valid(tempZObj) = ', obj_valid(tempZObj), $
      	" zOK = ", zOK, " zrank=", zrank
      if (zOK) then print, infolabel, "zrank=", zrank, ' zdim=', zdim
    endif
    if (zrank eq 1) then begin
;;;;      if ( ((not xOK) OR (xrank EQ 0)) AND (yOK)) then begin
     if ((xrank EQ 0) AND (yOK)) then begin
	self.ytype = ""
        if (zdim[0] eq ydim) then self.ytype = "points"
        if ((zdim[0]+1) eq ydim) then self.ytype = "histo"
	xtype = ''
      endif else begin
;;;;        if ( ((not yOK) OR (xrank EQ 0)) AND (xOK)) then begin
        if ((yrank EQ 0) AND (xOK)) then begin
	  self.xtype = ""
          if (zdim[0] eq xdim) then self.xtype = "points"
          if ((zdim[0]+1) eq xdim) then self.xtype = "histo"
	  ytype = ''
        endif
      endelse
      
    endif else begin
      if (zrank eq 2) then begin
	self.xtype = ""
        if (zdim[0] eq xdim[0]) then self.xtype = "points"
        if ((zdim[0]+1) eq xdim[0]) then self.xtype = "histo"
	self.ytype = ""
        if (zdim[1] eq ydim[0]) then self.ytype = "points"
        if ((zdim[1]+1) eq ydim[0]) then self.ytype = "histo"
      endif else begin
      	; zrank higher than 2 not supported yet.
      endelse
    endelse    
  
  endif

end	; focusObj::set

;================================================================================
;*
function  focusObj::get, $	; returns 1 if any keywords used. 
				; Returns 0 if no known keywords used.
   argPresent = thisArgPresent, $; ARG_PRESENT() function does not work on 
   		;propagated keywords from other calling routines.
		; Needed for get* methods.
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $		; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags, $			; optional 
   xType = thisXType, $
   yType = thisYType, $
   everything = thisEverything
;/
    
; returns 1 if any keywords used. Returns 0 if no known keywords used.

  infolabel = "focusObj::get >> "  

  if (NOT arg_present(thisArgPresent)) then begin   
    ArgPresent = {$			; make this be anonymous structure
	container : arg_present(thisContainer), $
	dataObj : arg_present(thisDataObj), $	
	headerObj : arg_present(thisHeaderObj), $
	mainTitleTags : arg_present(thisMainTitleTags), $
	descripTags : arg_present(thisdescripTags), $
	descripLegend : arg_present(thisDescripLegend), $
	cutTags : arg_present(thisCutTags), $
	xTags : arg_present(thisXtags), $
	yTags : arg_present(thisYtags), $
	zTags : arg_present(thisZtags), $ 
	xType : arg_present(thisXtype), $
	yType : arg_present(thisYtype), $
	everything : arg_present(thisEverything)}
  endif else begin
    ; ArgPresent had better include the appropriate tags.
    ArgPresent = thisArgPresent
  endelse

  nsuc = 0  

  ;-------------------------------------------------------
  if (argPresent.everything) then begin
    thisEverything = string("title=", self.mainTitleTags)
    thisEverything = thisEverything + " x=[" + $
    	(self.xTags)[0] + ',' + (self.xTags)[1] + ',' + (self.xTags)[2] + ']'
    thisEverything = thisEverything + " y=[" + $
    	(self.yTags)[0] + ',' + (self.yTags)[1] + ',' + (self.yTags)[2] + ']'
    thisEverything = thisEverything + " x=[" + $
    	(self.zTags)[0] + ',' + (self.zTags)[1] + ',' + (self.zTags)[2] + ']'
    nsuc = nsuc + 1
  endif
  

  if (argPresent.container) then begin
    thisContainer = self.container
    nsuc = nsuc + 1
  endif
  
  if (argPresent.dataObj) then begin
    thisdataObj = self.dataObj
    nsuc = nsuc + 1
  endif
  
  if (argPresent.headerObj) then begin
    thisHeaderObj = self.headerObj
    nsuc = nsuc + 1
  endif
  
  if (argPresent.mainTitleTags) then begin
    thisMainTitleTags = self.mainTitleTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.descripTags) then begin
    thisDescripTags = self.descripTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.descripLegend) then begin
    thisDescripLegend = self.descripLegend
    nsuc = nsuc + 1
  endif
  
  if (argPresent.cutTags) then begin
    thisCutTags = self.cutTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.xTags) then begin
    thisXtags = self.xTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.yTags) then begin
    thisYTags = self.yTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.zTags) then begin
    thisZtags = self.zTags
    nsuc = nsuc + 1
  endif
  
  if (argPresent.xType) then begin
    thisXType = self.xType
    nsuc = nsuc + 1
  endif
  
  if (argPresent.yType) then begin
    thisYType = self.YType
    nsuc = nsuc + 1
  endif
  
  ;-----------------------------------------
  success = (nsuc gt 0)

  if (success EQ 0) then begin
     print, infolabel, ' No recognized keywords provided, so return self'
     return, self
  endif

  return, success 

end	; focusObj::get

;================================================================================
;*
pro focusObj::show, $		
				; By default, show everything.
  lun=thisLun, $		; Optionally, write to a file. If absent, to screen
  strArray = thisStrArray, $	; Optionally, write to an array instead of file 
  heapNum = thisHeapNum, $	; Show the heap number via HELP
  recursive = thisRecursive	; Ignore this keyword.
;/

  infolabel = "focusObj::show >> "

  strAr = ''
  localStrAr = ''
  if (n_elements(thisStrArray) GT 0) then begin
;    lun = -666		; Don't write to a file
  endif else begin
	;Unit Attributes Name
	;-2   Write, Truncate, Tty, Reserved   <stderr>
	;-1   Write, Truncate, Tty, Reserved   <stdout>
	;0   Read, Tty, Reserved              <stdin>
    if (n_elements(thisLun) eq 0) then begin
      lun= -1
    endif else begin
      lun = thisLun
    endelse
  endelse
  
  localClass = obj_class(self)

  OK = self.container -> get(everything=thisEverything)
    if (keyword_set(thisHeapNum)) then begin
      help, self.container, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.container= ', helpString)]
      ; printf, lun, '{FOCUSOBJ}.container= ', helpString   
    endif
  if (OK) then begin
    strAr = [strAr, string('.container =', thisEverything)]
    ; printf, lun, localClass, '.container =', thisEverything
  endif
  strAr = [strAr, ' ']
;debug  print, infolabel, "container strAr =", strAr
 ; printf, lun

  
;  printf, lun, 'Container = ', 
;  	localClassSize = '{' +  obj_class(self.container) +  '}' +  $
; 	'[' + strcompress(string(ncount), /remove_all) + ']'

  thisEverything = ''
  OK = self.headerObj -> get(everything=thisEverything)
    if (keyword_set(thisHeapNum)) then begin
      help, self.headerObj, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.headerObj= ', helpString)]  
      ; printf, lun, '{FOCUSOBJ}.headerObj= ', helpString   
    endif
  if (OK) then strAr = [strAr, string(localClass, '.header =', thisEverything)]
;  if (OK) then printf, lun, localClass, '.header =', thisEverything
  strAr = [strAr, ' ']
;  printf, lun
;  printf, lun
  
  if (NOT(eqArrays(self.mainTitleTags, ['', '', '']))) then begin
    strAr = [strAr, string(localClass, '.MainTitleTags =', self.mainTitleTags)]
;    printf, lun, localClass, '.MainTitleTags =', self.mainTitleTags
    tempObj = self.headerObj -> get(position = self.mainTitleTags)
    if (obj_valid(tempObj)) then begin
      localPrefix = localClass + '.mainTitle ='
      if (keyword_set(thisHeapNum)) then begin
        help, tempObj, output = helpString
       strAr = [strAr, string('{FOCUSOBJ}.headerObj.mainTitle= ', helpString)]
       ; printf, lun, '{FOCUSOBJ}.headerObj.mainTitle= ', helpString   
      endif
      tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
      strAr = [strAr, localStrAr] 
;      tempObj -> show, lun=lun,  prefix = localPrefix
    endif else begin
      strAr = [strAr, "{FOCUSOBJ}.headerObj.mainTitle NOT DEFINED"]
;      printf, lun, "{FOCUSOBJ}.headerObj.mainTitle NOT DEFINED"
    endelse
    strAr = [strAr, ' ']
;    printf, lun
;    printf, lun
    
  endif 
  
   if (NOT(eqArrays(self.descripTags, ['', '', '']))) then begin
;;   if (self.descripTags NE ['', '', '']) then begin
    strAr = [strAr, string( localClass, '.descripTags =', self.descripTags)]
    strAr = [strAr, string( localClass, '.descripLegend =', self.descripLegend)]
;    printf, lun, localClass, '.descripTags =', self.descripTags
;    printf, lun, localClass, '.descripLegend =', self.descripLegend
;  thisEverything = ''
;  OK = self.headerObj -> get(position = self.descripTags, $
;  	everything=thisEverything)
;  if (OK) then printf, lun, localClass, '.descrip =', thisEverything
    tempObj = self.headerObj -> get(position = self.descripTags)
    localPrefix = localClass + '.descrip ='
    if (keyword_set(thisHeapNum)) then begin
      help, tempObj, output = helpString
      strAr = [strAr, string( '{FOCUSOBJ}.headerObj.descrip= ', helpString)]
;      printf, lun, '{FOCUSOBJ}.headerObj.descrip= ', helpString   
    endif
    tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
    strAr = [strAr, localStrAr] 
;    tempObj -> show, lun=lun, prefix = localPrefix
    localLegend = strtrim(self.descripLegend, 2)
    OK = tempObj -> $
    	getOnevalue( forLegend = localLegend, descripTitle=dstring)
     if (OK) then begin
       strAr = [strAr, string(localClass, ".descripTitle ='", dstring, "'")]
;       printf, lun, localClass, ".descripTitle ='", dstring, "'"
     endif else begin
       strAr = [strAr, string(localClass, '.descripTitle =?? OK=',OK)]
;       printf, lun, localClass, '.descripTitle =?? OK=',OK
     endelse
     strAr = [strAr, ' ']
;     printf, lun
;     printf, lun
  endif
  
   if (NOT(eqArrays(self.cutTags, ['', '', '']))) then begin
     strAr = [strAr, string( localClass, '.cutTags =', self.cutTags)]
;    printf, lun, localClass, '.cutTags =', self.cutTags
    tempObj = self.headerObj -> get(position = self.cutTags)
    if (obj_valid(tempObj)) then begin
      localPrefix = localClass + '.cut ='
      if (keyword_set(thisHeapNum)) then begin
        help, tempObj, output = helpString
        strAr = [strAr, string( '{FOCUSOBJ}.headerObj.cut= ', helpString   )]
;        printf, lun, '{FOCUSOBJ}.headerObj.cut= ', helpString   
      endif
      tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
      strAr = [strAr, localStrAr] 
;      tempObj -> show, lun=lun, prefix = localPrefix
      ; This should be a scalar always. 
      cstring = "?cstring?"
      OK = tempObj -> $
    	getOnevalue( forLegend = '', descripTitle=cstring)
;      OK = tempObj -> $
;;;;    	getOnevalue(descripTitle=cstring)
;    	getOnevalue( forPos = 0, descripTitle=cstring)
       if (OK) then begin
         strAr = [strAr, string( localClass, ".cutTitle ='", cstring, "'")]
;         printf, lun, localClass, ".cutTitle ='", cstring, "'"
        endif else begin
         strAr = [strAr, string( localClass, '.cutTitle =?? OK=',OK)]
;         printf, lun, localClass, '.cutTitle =?? OK=',OK
       endelse
;;      printf, lun, localClass, ".cutTitle = '", cstring, "'"
     endif else begin
       strAr = [strAr, string("{FOCUSOBJ}.headerObj.cutTitle NOT DEFINED")]
;       printf, lun, "{FOCUSOBJ}.headerObj.cutTitle NOT DEFINED"
     endelse

     strAr = [strAr, ' ']
;     printf, lun
;     printf, lun
  endif
  
  thisEverything = ''
  OK = self.dataObj -> get(everything=thisEverything)
    if (keyword_set(thisHeapNum)) then begin
      help, self.dataObj, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.dataObj= ', helpString   )]
;      printf, lun, '{FOCUSOBJ}.dataObj= ', helpString   
    endif
  if (OK) then strAr = [strAr, string(localClass, '.data =', thisEverything)]
;  if (OK) then printf, lun, localClass, '.data =', thisEverything
  strAr = [strAr, ' ']
;  printf, lun
;  printf, lun

  if NOT (eqArrays(self.xTags, ['', '', ''])) then begin
    strAr = [strAr, string(localClass, '.xTags =', self.xTags)]
    strAr = [strAr, string(localClass, '.xtype=',self.xtype )]
;    strAr = [strAr, string(localClass, '.xTags =[', self.xTags, $
;    		'], .xtype=',self.xtype )]
;    printf, lun, localClass, '.xTags =[', self.xTags, '], .xtype=',self.xtype 
;    thisEverything = ''
;    OK = self.dataObj -> get(position = self.xTags, $
;  	everything=thisEverything)
;    if (OK) then printf, lun, localClass, '.x =', thisEverything
    tempObj = self.dataObj -> get(position = self.xTags)
    localPrefix = localClass + '.x ='
    if (keyword_set(thisHeapNum)) then begin
      help, tempObj, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.headerObj.xObj= ', helpString   )]
;      printf, lun, '{FOCUSOBJ}.headerObj.xObj= ', helpString   
    endif
    if (obj_valid(tempObj)) then begin
      tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
      strAr = [strAr, localStrAr] 
;      tempObj -> show, lun=lun, prefix = localPrefix
    endif else begin
      strAr = [strAr, string(localPrefix + ' FAILED' )]
;      printf, lun, localPrefix + ' FAILED' 
    endelse
  strAr = [strAr, ' ']
;    printf, lun
;    printf, lun
  endif
  
  if NOT (eqArrays(self.yTags, ['', '', ''])) then begin
    strAr = [strAr, string(localClass, '.yTags =', self.yTags)]
    strAr = [strAr, string(localClass, '.ytype=',self.ytype )]
;    strAr = [strAr, string(localClass, '.yTags =[', self.yTags, $
  ;  	'], .ytype=',self.ytype )]
;    printf, lun, localClass, '.yTags =[', self.yTags, '], .ytype=',self.ytype 
;    thisEverything = ''
;    OK = self.dataObj -> get(position = self.yTags, $
;  	everything=thisEverything)
;    if (OK) then printf, lun, localClass, '.y =', thisEverything
    tempObj = self.dataObj -> get(position = self.yTags)
    localPrefix = localClass + '.y ='
;    tempObj -> show, lun=lun, prefix = localPrefix
    if (keyword_set(thisHeapNum)) then begin
      help, tempObj, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.headerObj.yObj= ', helpString )]
;      printf, lun, '{FOCUSOBJ}.headerObj.yObj= ', helpString   
    endif
    if (obj_valid(tempObj)) then begin
      tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
      strAr = [strAr, localStrAr] 
;      tempObj -> show, lun=lun, prefix = localPrefix
    endif else begin
      strAr = [strAr, string(localPrefix + ' FAILED' )]
;      printf, lun, localPrefix + ' FAILED' 
    endelse
    strAr = [strAr, ' ']
;    printf, lun
;    printf, lun
  endif
  
  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    strAr = [strAr, string(localClass, '.zTags =', self.zTags)]
;    strAr = [strAr, string(localClass, '.zTags =[', self.zTags, ']')]
;    printf, lun, localClass, '.zTags =[', self.zTags, ']'
;    thisEverything = ''
;    OK = self.dataObj -> get(position = self.zTags, $
;  	everything=thisEverything)
;    if (OK) then printf, lun, localClass, '.z =', thisEverything
    tempObj = self.dataObj -> get(position = self.zTags)
    localPrefix = localClass + '.z ='
;    tempObj -> show, lun=lun, prefix = localPrefix
    if (keyword_set(thisHeapNum)) then begin
      help, tempObj, output = helpString
      strAr = [strAr, string('{FOCUSOBJ}.headerObj.zObj= ', helpString)]
;      printf, lun, '{FOCUSOBJ}.headerObj.zObj= ', helpString   
    endif
    if (obj_valid(tempObj)) then begin
      tempObj -> show, lun=lun, strArray=localStrAr, prefix = localPrefix
      strAr = [strAr, localStrAr] 
;      tempObj -> show, lun=lun, prefix = localPrefix
    endif else begin
      strAr = [strAr, string(localPrefix + ' FAILED' )]
;      printf, lun, localPrefix + ' FAILED' 
    endelse
    strAr = [strAr, ' ']
;    printf, lun
;    printf, lun
  endif
  
  ;----------------------------------------------
;Debug  print, infolabel, "n_elements(strAr) =", n_elements(strAr)
  if (arg_present(thisStrArray)) then begin
;Debug     print, infolabel, " Returning strAr"
     thisStrArray = strAr; 
;     ; Print to screen just for debug
;    nelm = n_elements(strAr)
;    for i = 0, (nelm-1) do begin
;      print, i, " DEBUG   ", strAr[i]
;    endfor 
  endif else begin
;Debug     print, infolabel, " Printing strArray to LUN=", lun
    nelm = n_elements(strAr)
    for i = 0, (nelm-1) do begin
      printf, lun, strAr[i]
    endfor 
  endelse  


end		;;  focusObj::show

;================================================================================
;*
function focusObj::makeMainTitleString	; Returns a string
;/

  ;------- mainTitle ------------------------------------------------------ 
  ; Might not have any mainTitle to report. That's OK.
  tstring = ''
  if NOT (eqArrays(self.mainTitleTags, ['', '', ''])) then begin
    tempTObj = self.headerObj -> get(position = self.mainTitleTags)
    if (obj_valid(tempTObj)) then begin
       OK = tempTObj -> get(ptr_qty = tptr_qty) 
       if (OK) then begin
         if (n_elements(*tptr_qty) eq 1) then begin
           tString = *( tptr_qty)
	 endif else begin
	   for i = 0, (n_elements(*tptr_qty) -1) do begin
	     tString = tstring + (*(tptr_qty))[i] + '  '
	   endfor
	 endelse
       endif	; (OK)
    endif
    if (ptr_valid(tptr_qty)) then ptr_free, tptr_qty	; mainTitle

  endif 


  return, tstring
  
end		;; focusObj::makeMainTitleString

;================================================================================
;*
function focusObj::makeDescripString	; Returns a string
;/
  ;------- descrip ------------------------------------------------------ 
  ; Might not have any descrip to report. That's OK.
  dstring = ''
  if NOT (eqArrays(self.descripTags, ['', '', ''])) then begin
    tempDObj = self.headerObj -> get(position = self.descripTags)
    if (obj_valid(tempDObj)) then begin
      localForLegend = '*'	; in case there is no legend specified, 
      				; e.g. in the case of a scalar.
      OK = tempDObj -> $
    	getOnevalue( forLegend = localForLegend, descripTitle=dstring)
    endif

    if (ptr_valid(dptr_qty)) then ptr_free, dptr_qty	; descriptor
    if (ptr_valid(dptr_err)) then ptr_free, dptr_err
    if (ptr_valid(dptr_legend)) then ptr_free, dptr_legend
  endif
  
  return, dstring
end		;; focusObj::makeDescripString

;================================================================================
;*
function focusObj::makeCutString	; returns a string
;/
  ;------- cut any descrip to report. That's OK.
  cstring = ''
  if NOT (eqArrays(self.cutTags, ['', '', ''])) then begin
    tempCObj = self.headerObj -> get(position = self.cutTags)
    if (obj_valid(tempCObj)) then begin
      ; cut should always be a scalar.
;     localForLegend = '*'	; in case there is no legend specified, 
;      				; e.g. in the case of a scalar.
;     print, " OK = tempCObj -> getOnevalue( forPos = 0, descripTitle=cstring)"
    OK = tempCObj -> $
    	getOnevalue( forLegend = '', descripTitle=cstring)
;Broken     OK = tempCObj -> getOnevalue( forPos = 0, descripTitle=cstring)
;;;      OK = tempCObj -> getOnevalue(descripTitle=cstring)
      cstring = '  cut=' + cstring 
    endif
  endif
  
  return, cstring
end		;; focusObj::makeCutString

;================================================================================
;*
pro focusObj::plot, $
    over = over, $		; Draws over previous plot if set
    indexNum = thisIndexNum, $	; 0 for plot. 1 for oplot. 
    				; Can color code and vary the symbols using indexNum
    _EXTRA = extraExtra 	; extra keywords passed to plot.
;/
  xrank = -1		; garbage values in case not defined later.
  yrank = -2
  zrank = -3
  xdim = [-1, -1, -1, -1]
  ydim = [-2, -2, -2, -2]
  zdim = [-1, -3, -3, -3]
  
  infolabel = "focusObj::plot >> "
  debug = 0
  
  if (debug) then begin
    print, infolabel, "focus.xTags =", format_list(self.xTags)
    print, infolabel, "focus.yTags =", format_list(self.yTags)
    print, infolabel, "focus.zTags =", format_list(self.zTags)
  endif
  
  nsuc = 0

  thisOver = keyword_set(over)
  if (n_elements(thisIndexNum) gt 0) then begin
    if (thisIndexNum GT 0) then thisOver = 1
  endif else begin
    thisIndexNum = 0
  endelse
   
  Zabsent = eqArrays(self.zTags, ['', '', ''])
  if (Zabsent) then begin
    print, infolabel, "Nothing to plot"
    return
  endif

  Xabsent = eqArrays(self.xTags, ['', '', ''])
  Yabsent = eqArrays(self.yTags, ['', '', ''])

  if (debug) then print, infolabel, "xAbsent=", xAbsent, " xtags =", self.xTags
  if (debug) then print, infolabel, "yAbsent=", yAbsent, " ytags =", self.yTags

  if (not Yabsent) then begin		; Double check
    tempYObj = self.dataObj -> get(position = self.yTags)
    if (NOT obj_valid(tempYobj)) then begin
      Yabsent = 1
    endif else begin
      OK  = tempYObj -> get(rank=yrank, dim=ydim)	; Sept 2, 2003. TMR
      if (OK) then begin
        Yabsent = 0
        if (debug) then tempyObj -> show
      endif else begin
        Yabsent = 1
      endelse
      
;      Yabsent = (1 EQ (tempYobj -> count()))
    endelse
  endif
  if (not Xabsent) then begin		; Double check
    tempXObj = self.dataObj -> get(position = self.xTags)
    if (NOT obj_valid(tempXobj)) then begin
      Xabsent = 1
    endif else begin
      OK  = tempXObj -> get(rank=xrank, dim=xdim)	; Sept 2, 2003. TMR  
      if (OK) then begin
        Xabsent = 0
        if (debug) then tempXObj -> show
      endif else begin
        Xabsent = 1
      endelse
;      if (debug) then tempXObj -> show
;      Xabsent = (1 EQ (tempXobj -> count()))
    endelse
  endif

  if (debug) then print, infolabel, "xAbsent=", xAbsent, " xtags =", self.xTags
  if (debug) then print, infolabel, "yAbsent=", yAbsent, " ytags =", self.yTags


  ;------- z ------------------------------------------------------ 
;;  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    tempzObj = self.dataObj -> get(position = self.zTags)
    if (obj_valid(tempZObj)) then begin
       OK = tempZObj -> get(rank = zrank, dim=zdim) 
       if (OK) then nsuc = nsuc + 1
    endif
;;  endif 
  ;------------------------------------------------------------- 

  if (zrank EQ 1) then begin
    if (Yabsent) then begin
      if ( Xabsent) then begin
        axesMatch = 1
      endif else begin
;        OK  = tempXObj -> get(rank=xrank, dim=xdim)	; Sept 2, 2003. TMR
        axesMatch = (zrank EQ xrank) AND (zdim[0] EQ xdim[0]) ; Sept 2, 2003. TMR
      endelse
      if (axesMatch) then begin
        self -> plotxz, xAbsent, thisOver, thisIndexNum, $
      		_EXTRA = extraExtra
      endif

    endif else begin
;      OK  = tempXObj -> get(rank=xrank, dim=xdim)	; Sept 2, 2003. TMR  
;      OK  = tempYObj -> get(rank=yrank, dim=ydim)	; Sept 2, 2003. TMR
      axesMatch = (zrank EQ xrank) AND (zrank EQ yrank)
      axesMatch = axesMatch AND (zdim[0] EQ xdim[0]) 
      axesMatch = axesMatch AND (zdim[0] EQ ydim[0]) 
      if axesMatch then begin				; Sept 2, 2003. TMR
        self -> scatterxyz, thisOver, thisIndexNum, _EXTRA = extraExtra
      endif
      ; else ignore silly request.
;      self -> plotsxyz, thisOver, thisIndexNum, _EXTRA = extraExtra
    endelse
  endif else begin
    if (zrank EQ 2) then begin
      axesMatch = 1
      if (NOT xAbsent) then axesMatch = axesMatch AND (zdim[0] EQ xdim[0]) 
      if (NOT yAbsent) then axesMatch = axesMatch AND (zdim[1] EQ ydim[0]) 
      if (axesMatch) then begin
        self -> plotxyz, xAbsent, Yabsent, thisOver, thisIndexNum, $
      		_EXTRA = extraExtra
      endif
    endif else begin
      print, infolabel, "Z's rank is not plotable currently, ", zrank 
    endelse
  endelse
  
end			; focusObj::plot


;================================================================================
;*
pro focusObj::plotxyz, $
    xAbsent, $ 		; Boolean
    Yabsent, $		; Boolean
    thisOver, $		; Boolean. Draws over previous plot if set=1
    thisIndexNum, $	; Integer.  0 for plot. 1 for oplot. 
    			; Can color code and vary the symbols using indexNum
    _EXTRA = extraExtra ; extra keywords passed to plot.
;/

  infolabel = "focusObj::plotxyz >> "

  nsuc = 0

  nlevels = 12.0		; number of contour levels
  fancyContour = 0		; 0 = stupid contour. 1 = fancy contour
  				; Might clash with extraExtra
  xrank = -1
  yrank = -1
  zrank = -1
  
    
  if (NOT thisOver) then begin  
    tstring = self -> makeMainTitleString()
    dstring = self -> makeDescripString()
    cstring = self -> makeCutString()
  endif
  
  nvar = 1	; z
  if (NOT xAbsent) then nvar = nvar + 1
  if (NOT yAbsent) then nvar = nvar + 1
  
  ;------- x ------------------------------------------------------ 
;;  if NOT (eqArrays(self.xTags, ['', '', ''])) then begin
  if (NOT xAbsent) then begin
    tempXObj = self.dataObj -> get(position = self.XTags)
    if (obj_valid(tempXObj)) then begin
       OK = tempXObj -> get(ptr_qty = xptr_qty, axisTitle = xaxisTitle, $
       		rank = xrank) 
       if (OK) then nsuc = nsuc + 1
    endif
  endif 
  
  ;------- y ------------------------------------------------------ 
;;;  if NOT (eqArrays(self.yTags, ['', '', ''])) then begin
  if (NOT yAbsent) then begin
    tempYObj = self.dataObj -> get(position = self.YTags)
    if (obj_valid(tempYObj)) then begin
       OK = tempYObj -> get(ptr_qty = yptr_qty, axisTitle = yaxisTitle, $
       		rank = yrank) 
       if (OK) then nsuc = nsuc + 1
    endif
  endif 
  
  ;------- z ------------------------------------------------------ 
;  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    tempzObj = self.dataObj -> get(position = self.zTags)
    if (obj_valid(tempZObj)) then begin
       OK = tempZObj -> get(ptr_qty = zptr_qty, ptr_err = zptr_err, $
       		axisTitle = zaxisTitle, rank = zrank) 
       if (OK) then nsuc = nsuc + 1
    endif
;  endif 

  ;------------------------------------------------------------- 
  if (nsuc NE nvar) then begin
    print, infolabel, "Error", "nsuc = ", nsuc, " when expecting ", nvar
    goto, skip_plot
  endif  
  
;  print,  'ztitle=', zaxisTitle
;  print,  'xtitle=', xaxisTitle
;debug  print, 'dstring = "', dstring, '"'
;debug  print, size(dstring)

  if (fancyContour) then begin
    TvLCT, 0, 255, 0, 3
    loadCT, 0
    loadCT, 4, Ncolors = nlevels, bottom=1
    step = (max(*(zptr_qty)) - min(*(zptr_qty))) / nlevels
    clevels = IndGen(nlevels) * step + min(*(zptr_qty))
    ccolors = indgen(nlevels) + 1
    boolfill = 1
    boolfollow = 1
    pcolor = !p.color
    pbackground = !P.background
  endif	; else undefined variables

  if ((NOT xAbsent) and (NOT yAbsent)) then begin
;Debug    print, infolabel, n_elements(*(zptr_qty)), $
;    	n_elements(*(xptr_qty)), n_elements(*(yptr_qty)) 
    dimMatch = ((n_elements(*(xptr_qty)) * n_elements(*(yptr_qty)) ) EQ  $
    	n_elements(*(zptr_qty)) )
;debug    print, infolabel, "dimMatch = ", dimMatch
    if (NOT thisOver) then begin
      if (dimMatch) then begin
        contour, *(zptr_qty), *(xptr_qty), *(yptr_qty),  $
           title = tstring, $
           subtitle = string(dstring, cstring), $
           xtitle=xaxisTitle,$
	   ytitle=YaxisTitle, $
;	   ytitle=ZaxisTitle, $
	   fill=boolfill, $		; color fill
	   follow = boolfollow, $	; label every other contour
	   levels=clevels, $		; Contours at which values of z
	   c_colors = ccolors, $	; The colors of the fill
	   background = pcolor,$	
	   color = Pbackground, $
          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
      endif else begin
        contour, *(zptr_qty),   $
           title = tstring, $
           subtitle = string(dstring, cstring), $
           xtitle=xaxisTitle, $
	   ytitle=YaxisTitle, $
;	   ytitle=ZaxisTitle, $
	   fill=boolfill, $		; color fill
	   follow = boolfollow, $	; label every other contour
	   levels=clevels, $	; Contours at which values of z
	   c_colors = ccolors, $; The colors of the fill
	   background = pcolor,$	
	   color = Pbackground, $
          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
      endelse
    endif			;;  else begin
;      ocontour, *(zptr_qty), *(xptr_qty), *(yptr_qty),  $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;    endelse
  endif else begin
    if ((NOT xAbsent) and (yAbsent)) then begin
    	; Skip this, causes an error.
;      if (NOT thisOver) then begin
;        contour, *(zptr_qty), *(xptr_qty),  $
;           title = tstring, $
;           subtitle = string(dstring, cstring), $
;           xtitle=xaxisTitle, $
;;	   ytitle=ZaxisTitle, $
;	   fill=boolfill, $		; color fill
;	   follow = boolfollow, $	; label every other contour
;	   levels=clevels, $	; Contours at which values of z
;	   c_colors = ccolors, $; The colors of the fill
;	   background = pcolor,$	
;	   color = Pbackground, $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;      endif    ;;;else begin
;        contour, *(zptr_qty), *(xptr_qty),  $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;      endelse
     endif else begin
      if (NOT thisOver) then begin
        contour, *(zptr_qty),  $
           title = tstring, $
           subtitle = string(dstring, cstring), $
;           xtitle=xaxisTitle, ytitle=ZaxisTitle, $
	   fill=boolfill, $		; color fill
	   follow = boolfollow, $	; label every other contour
	   levels=clevels, $	; Contours at which values of z
	   c_colors = ccolors, $; The colors of the fill
	   background = pcolor,$	
	   color = Pbackground, $
          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
        endif 			;;else begin
;          contour, *(zptr_qty),  $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;        endelse
     endelse
  endelse
    
 skip_plot:
 
  if (ptr_valid(xptr_qty)) then ptr_free, xptr_qty	; x
  if (ptr_valid(yptr_qty)) then ptr_free, yptr_qty	; y

  if (ptr_valid(zptr_qty)) then ptr_free, zptr_qty	; x
  if (ptr_valid(zptr_err)) then ptr_free, zptr_err	; z
  
end			; focusObj::plotxyz

;;================================================================================
;;*
;function focusObj::textInfo, $ 	; Returns a string array
;				; By default, /geteverything.
;  ;-----------------------------------------------
;;  position = thisPosition, $	; Optionally only ask for one item in the Header.
;;      ; Either integer position or tags position ['class', 'label', 'codeSource']
;  ; ------------- Select one of the 3 following keywords. --------------
;  getSome=thisGetSome, $; Boolean keyword.
;  			; 1 line summary of one qtyObj Object. No vectorvalues.
;  getAll=thisGetAll, $  ; Bolean keyword
;  			; 1 line summary of one qtyObj Object
;  getEverything=thisGetEverything	; Boolean keyword
;  		; String array containing all contents of one qtyObjObObject
;;/
;
;  infolabel = "focusObj::textInfo >> "
;  debug = 0
;
;  if ( (NOT keyword_set(thisGetSome)) AND (NOT keyword_set(thisGetAll)) $
;  	AND (NOT keyword_set(thisGetEverything)) ) then begin
;    thisGetEverything = 1
;  endif
;
;  oneString = ''
;  if (keyword_set( thisGetSome) ) then begin
;    thisSome = oneString	; Defined
;;    thisAll = nullString	; Undefined
;;    thisEverything = nullString	; Undefined
;  endif
;  
;  if (keyword_set( thisGetAll) ) then begin
;;    thisSome = nullString	; Defined
;    thisAll = oneString		; Undefined
;;    thisEverything = nullString	; Undefined
;  endif
;  
;  if (keyword_set( thisGetEverything) ) then begin
;;    thisSome = nullString	; Defined
;;    thisAll = nullString	; Undefined
;    thisEverything = oneString	; Undefined
;  endif
;  
;  strArray = ''
;
;  OK = self -> getFocus(mainTitleTags = tempTag)
;  OK = self -> getField(self.mainTitleTags, /inHeader, $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring]
;  
;  OK = self -> getFocus(cutTags = tempTag)
;  OK = self -> getField(self.cutTags, /inHeader, $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring]
;
;  OK = self -> getFocus(decsripTags = tempTag)
;  OK = self -> getField(self.decsripTags, /inHeader, $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring]
;  
;  
;  OK = self -> getFocus(xTags = tempTag)
;  OK = self -> getField(tempTag, /inData,  $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring];
;
;  OK = self -> getFocus(yTags = tempTag)
;  OK = self -> getField(tempTag, /inData,  $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring]
;
;  OK = self -> getFocus(zTags = tempTag)
;  OK = self -> getField(tempTag, /inData, $
;  	some=thisSome, all=thisAll, everything=thisEverything)
;  strArray = [strArray, onestring]
;  
;  
;  return, StrArray
;end		;; focuscObj::textInfo

;================================================================================
;*
pro focusObj::plotsxyz, $
    thisOver, $		; Boolean. Draws over previous plot if set=1
    thisIndexNum, $	; Integer. 0 for plot. 1 for oplot. 
    			; Can color code and vary the symbols using indexNum
    _EXTRA = extraExtra ; extra keywords passed to plot.
;/
;    xAbsent, $ 
;    Yabsent, $

  infolabel = "focusObj::plotsxyz >> "

  nsuc = 0
  nvar = 3 ;x, y, z
  
  if (NOT thisOver) then begin  
    tstring = self -> makeMainTitleString()
    dstring = self -> makeDescripString()
    cstring = self -> makeCutString()
  endif
  
;  nvar = 1	; z
;  if (NOT xAbsent) then nvar = nvar + 1
;  if (NOT yAbsent) then nvar = nvar + 1
  
  ;------- x ------------------------------------------------------ 
;;  if NOT (eqArrays(self.xTags, ['', '', ''])) then begin
;  if (NOT xAbsent) then begin
    tempXObj = self.dataObj -> get(position = self.XTags)
    if (obj_valid(tempXObj)) then begin
       OK = tempXObj -> get(ptr_qty = xptr_qty, axisTitle = xaxisTitle, $
       		rank = xrank) 
       if (OK) then nsuc = nsuc + 1
    endif
;  endif 
  
  ;------- y ------------------------------------------------------ 
;;;  if NOT (eqArrays(self.yTags, ['', '', ''])) then begin
;  if (NOT yAbsent) then begin
    tempYObj = self.dataObj -> get(position = self.YTags)
    if (obj_valid(tempYObj)) then begin
       OK = tempYObj -> get(ptr_qty = yptr_qty, axisTitle = yaxisTitle, $
       		rank = yrank) 
       if (OK) then nsuc = nsuc + 1
    endif
;  endif 
  
  ;------- z ------------------------------------------------------ 
;  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    tempzObj = self.dataObj -> get(position = self.zTags)
    if (obj_valid(tempZObj)) then begin
       OK = tempZObj -> get(ptr_qty = zptr_qty, ptr_err = zptr_err, $
       		axisTitle = zaxisTitle, rank = zrank) 
       if (OK) then nsuc = nsuc + 1
    endif
;  endif 

  ;------------------------------------------------------------- 
  if (nsuc NE nvar) then begin
    print, infolabel, "Error ", "nsuc = ", nsuc, " when expecting ", nvar
    goto, skip_plot
  endif  
  
;  print,  'ztitle=', zaxisTitle
;  print,  'xtitle=', xaxisTitle
;debug  print, 'dstring = "', dstring, '"'
;debug  print, size(dstring)
  
;  if (ptr_valid(xptr_qty) and ptr_valid(yptr_qty) $
;  		and ptr_valid(zptr_qty)) then begin
;    if ( (xrank EQ zrank) AND (yrank EQ zrank)) then begin
      if (NOT thisOver) then begin
        plots, *(xptr_qty), *(yptr_qty), *(zptr_qty), $
;Notallowed         title = tstring, $
;Not_allowed         subtitle = string(dstring, cstring), $
;        title=theTitle, subTitle = theSubTitle,$ 
;Notallowed         xtitle=xaxisTitle, ytitle=ZaxisTitle, $
        _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
       endif else begin
         plots, *(xptr_qty), *(yptr_qty), *(zptr_qty), $
	   /continue, $		;; Not sure if this is the way we want to do it.
          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
       endelse
       
;    endif else begin
;      if ( (xrank + yrank) EQ zrank)  then begin
;        contour, *(zptr_qty), *(xptr_qty), *(yptr_qty),  $
;           title = tstring, $
;           subtitle = string(dstring, cstring), $
;;          title=theTitle, subTitle = theSubTitle,$ 
;          xtitle=xaxisTitle, ytitle=ZaxisTitle, $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;      endif
;    endelse
;  endif

 skip_plot:
 
  if (ptr_valid(xptr_qty)) then ptr_free, xptr_qty	; x
  if (ptr_valid(yptr_qty)) then ptr_free, yptr_qty	; y

  if (ptr_valid(zptr_qty)) then ptr_free, zptr_qty	; x
  if (ptr_valid(zptr_err)) then ptr_free, zptr_err	; z
  
end			; focusObj::plotsxyz

;=============================================================================
;*
function focusObj::getMinMax, $	; Returns an integer array  [min, max] 
	; for the range available in the focus or in specified field
	; ------  Use 1 of the three following keywords ------------------
	x = thisX, $		; Boolean Keyword.
	y = thisY, $		; Boolean Keyword.
	z = thisZ			; Boolean Keyword.
;/

 infolabel = 'specObj::getMinMax >> '
 
 message, /info, string(" self.target = ", $
	obj_valid(self.target), " ", self.target)
 return, (self.target) -> getMinMax(x = thisX, y = thisY, z = thisZ)

end

;================================================================================
;*
pro focusObj::scatterxyz, $
    thisOver, $		; Boolean. Draws over previous plot if set=1
    thisIndexNum, $	; Integer. 0 for plot. 1 for oplot. 
    			; Can color code and vary the symbols using indexNum
    _EXTRA = extraExtra ; extra keywords passed to plot.
;/
;    xAbsent, $ 
;    Yabsent, $

  infolabel = "focusObj::scatterxyz >> "

  nsuc = 0
  nvar = 3 ;x, y, z
  
  if (NOT thisOver) then begin  
    tstring = self -> makeMainTitleString()
    dstring = self -> makeDescripString()
    cstring = self -> makeCutString()
  endif
  
;  nvar = 1	; z
;  if (NOT xAbsent) then nvar = nvar + 1
;  if (NOT yAbsent) then nvar = nvar + 1
  
  ;------- x ------------------------------------------------------ 
;;  if NOT (eqArrays(self.xTags, ['', '', ''])) then begin
;  if (NOT xAbsent) then begin
    tempXObj = self.dataObj -> get(position = self.XTags)
    if (obj_valid(tempXObj)) then begin
       OK = tempXObj -> get(ptr_qty = xptr_qty, axisTitle = xaxisTitle, $
       		rank = xrank) 
       if (OK) then begin
	 xRange = self -> getMinMax(/x)
         nsuc = nsuc + 1
       endif
    endif
;  endif 
  
  ;------- y ------------------------------------------------------ 
;;;  if NOT (eqArrays(self.yTags, ['', '', ''])) then begin
;  if (NOT yAbsent) then begin
    tempYObj = self.dataObj -> get(position = self.YTags)
    if (obj_valid(tempYObj)) then begin
       OK = tempYObj -> get(ptr_qty = yptr_qty, axisTitle = yaxisTitle, $
       		rank = yrank) 
       if (OK) then begin 
         nsuc = nsuc + 1
	 yRange = self -> getMinMax(/y)
       endif
    endif
;  endif 
  
  ;------- z ------------------------------------------------------ 
;  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    tempzObj = self.dataObj -> get(position = self.zTags)
    if (obj_valid(tempZObj)) then begin
       OK = tempZObj -> get(ptr_qty = zptr_qty, ptr_err = zptr_err, $
       		axisTitle = zaxisTitle, rank = zrank) 
       if (OK) then begin
	  zRange = self  -> getMinMax(/z)
         nsuc = nsuc + 1
       endif
    endif
;  endif 

  ;------------------------------------------------------------- 
  if (nsuc NE nvar) then begin
    print, infolabel, "Error ", "nsuc = ", nsuc, " when expecting ", nvar
    goto, skip_plot
  endif  
  
;  print, infolabel, "xrange=", xrange
;  print, infolabel, "yrange=", yrange
;  print, infolabel, "zrange=", zrange
  
;  print, infolabel, "x=", *(xptr_qty)
;  print, infolabel, "y=", *(yptr_qty)
;  print, infolabel, "z=", *(zptr_qty)

  
;  print,  'ztitle=', zaxisTitle
;  print,  'xtitle=', xaxisTitle
;debug  print, 'dstring = "', dstring, '"'
;debug  print, size(dstring)
  
;  if (ptr_valid(xptr_qty) and ptr_valid(yptr_qty) $
;  		and ptr_valid(zptr_qty)) then begin
;    if ( (xrank EQ zrank) AND (yrank EQ zrank)) then begin
      if (NOT thisOver) then begin
	plot, *(xptr_qty), *(yptr_qty)
        surface, /save, /noData, dist(10), $	; Dummy plot range
		charSize=1.5, $
		xTitle = xaxisTitle, $
		yTitle = yaxisTitle, $
		zTitle = zaxisTitle, $
	        title=theTitle, subTitle = theSubTitle, $
		Xrange = xrange, yRange=yrange, zRange=zrange
	plots, *(xptr_qty), *(yptr_qty), *(zptr_qty), $
		PSym=1, SymSize=2.0, /T3D, $
;Notallowed         title = tstring, $
;Not_allowed         subtitle = string(dstring, cstring), $
;        title=theTitle, subTitle = theSubTitle,$ 
;Notallowed         xtitle=xaxisTitle, ytitle=ZaxisTitle, $
        _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.

	npts = n_elements(*(xptr_qty))

	zcolors = BytScl(*(zptr_qty), Top=99) + 1B
	LoadCt, 22, Ncolors=100, Bottom =1
	for i = 0, (npts-1) do begin
	  plots, [ (*(xptr_qty))[i], (*(xptr_qty))[i]], $
	  	 [ (*(yptr_qty))[i], (*(yptr_qty))[i]], $
	  	 [ (*(zptr_qty))[i], 0.0], color=zcolors[i], /T3D
	endfor
	
       endif else begin
         plots, *(xptr_qty), *(yptr_qty), *(zptr_qty), $
	   PSym = 2, SymSize=2.0, /T3D,$
	   /continue, $		;; Not sure if this is the way we want to do it.
          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.

	npts = n_elements(*(xptr_qty))

	zcolors = BytScl(*(zptr_qty), Top=99) + 1B
	LoadCt, 22, Ncolors=100, Bottom =1
	for i = 0, (npts-1) do begin
	  plots, [ (*(xptr_qty))[i], (*(xptr_qty))[i]], $
	  	 [ (*(yptr_qty))[i], (*(yptr_qty))[i]], $
	  	 [ (*(zptr_qty))[i], 0.0], color=zcolors[i], /T3D
	endfor
	
       endelse
       
;    endif else begin
;      if ( (xrank + yrank) EQ zrank)  then begin
;        contour, *(zptr_qty), *(xptr_qty), *(yptr_qty),  $
;           title = tstring, $
;           subtitle = string(dstring, cstring), $
;;          title=theTitle, subTitle = theSubTitle,$ 
;          xtitle=xaxisTitle, ytitle=ZaxisTitle, $
;          _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;      endif
;    endelse
;  endif

 skip_plot:
 
  if (ptr_valid(xptr_qty)) then ptr_free, xptr_qty	; x
  if (ptr_valid(yptr_qty)) then ptr_free, yptr_qty	; y

  if (ptr_valid(zptr_qty)) then ptr_free, zptr_qty	; x
  if (ptr_valid(zptr_err)) then ptr_free, zptr_err	; z
  
end			; focusObj::scatterxyz

;================================================================================
;*
pro focusObj::plotxz, $
    xAbsent, $		; Boolean.
    thisOver, $		; Boolean. Draws over previous plot if set=1
    thisIndexNum, $	; Integer. 0 for plot. 1 for oplot. 
    			; Can color code and vary the symbols using indexNum
    _EXTRA = extraExtra ; extra keywords passed to plot.
;/
; Need to do error bars better!

  infolabel = "focusObj::plotxz >> "

  nsuc = 0
  nvar = 1	; z
  if (NOT xAbsent) then nvar = nvar + 1

  xrank = -1
  yrank = -1
  zrank = -1

  if (NOT thisOver) then begin  
    tstring = self -> makeMainTitleString()
    dstring = self -> makeDescripString()
    cstring = self -> makeCutString()
  endif
  
  ;------- x ------------------------------------------------------ 
  if NOT (eqArrays(self.xTags, ['', '', ''])) then begin
    tempXObj = self.dataObj -> get(position = self.XTags)
    if (obj_valid(tempXObj)) then begin
       OK = tempXObj -> get(ptr_qty = xptr_qty, axisTitle = xaxisTitle) 
       if (OK) then nsuc = nsuc + 1
    endif
  endif 
  
  ;------- z ------------------------------------------------------ 
;  if NOT (eqArrays(self.zTags, ['', '', ''])) then begin
    tempzObj = self.dataObj -> get(position = self.zTags)
    if (obj_valid(tempZObj)) then begin
       OK = tempZObj -> get(ptr_qty = zptr_qty, ptr_err = zptr_err, $
       		axisTitle = zaxisTitle) 
       if (OK) then nsuc = nsuc + 1
    endif
;  endif 

  ;------------------------------------------------------------- 
  if (nsuc NE nvar) then begin
    print, infolabel, "Error", "nsuc = ", nsuc, " when expecting ", nvar
    goto, skip_plot
  endif  
  
  if (NOT xAbsent) then begin
    if (NOT thisOver) then begin
      plot, *(xptr_qty), *(zptr_qty), $
       title = tstring, $
       subtitle = string(dstring, cstring), $
       xtitle=xaxisTitle, ytitle=ZaxisTitle, $
      _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;      if (ptr_valid(zptr_err)) then begin
;        ploterr, *(xptr_qty), *(zptr_qty), *(zptr_err)        
;      endif
    endif else begin
      oplot, *(xptr_qty), *(zptr_qty), $
      _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
    endelse
  endif else begin
    if (NOT thisOver) then begin
      plot,  *(zptr_qty), $
       title = tstring, $
       subtitle = string(dstring, cstring), $
       xtitle=xaxisTitle, ytitle=ZaxisTitle, $
      _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
    endif else  begin
      oplot,  *(zptr_qty), $
      _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
    endelse
  endelse
  
;  if (ptr_valid(xptr_qty) and ptr_valid(zptr_qty)) then begin
;    plot, *(xptr_qty), *(zptr_qty), $
;       title = tstring, $
;       subtitle = string(dstring, cstring), $
;;      title=theTitle, subTitle = theSubTitle,$ 
;      xtitle=xaxisTitle, ytitle=ZaxisTitle, $
;      _EXTRA = extraExtra ; add extra keywords here and oplot vs plot.
;  endif

 skip_plot:
 
  if (ptr_valid(xptr_qty)) then ptr_free, xptr_qty	; x
  if (ptr_valid(zptr_qty)) then ptr_free, zptr_qty	; y
  if (ptr_valid(zptr_err)) then ptr_free, zptr_err	; z
  
  if (ptr_valid(dptr_qty)) then ptr_free, dptr_qty	; descriptor
  if (ptr_valid(dptr_err)) then ptr_free, dptr_err
  if (ptr_valid(dptr_legend)) then ptr_free, dptr_legend

  if (ptr_valid(tptr_qty)) then ptr_free, tptr_qty	; mainTitle

end			; focusObj::plotxz

;=============================================================================
;*
function focusObj::count	; Returns integer value of 1 always.
;/
;; Just for consistency with other classes, esp. DAVE_tagContainer.
   return, 1   
end		; focusObj::count

;================================================================================
;*
pro focusObj__define

  infolabel = "focusObj__define >> "  
  define = {focusObj, $
  	target: obj_new(), $	; That specObj object that 
				; the focus is upon.
  	container: obj_new(), $	; That DAVE_tagContainer object that 
				; the focus is upon.
	;------------------------------------
	headerObj: obj_new(), $	; Current "header" section where to place 
				; short things.
	mainTitleTags: ['', '', ''], $	; class qtyObj. Short cut reference.
	descripTags: ['', '', ''], $	; class qtyObj. Short cut reference.
	DescripLegend: '', $	; Use if descrip is a vector to narrow down.
	cutTags: ['', '', ''], $	; class qtyObj. Short cut reference.
        ;------------------------------------
	dataObj: obj_new(), $	; Current "data" section where to place 
				; long things.
	xTags: ['', '', ''], $	; class qtyObj. Short cut reference.
	yTags: ['', '', ''], $	; class qtyObj. Short cut reference.
	zTags: ['', '', ''],  $	; class qtyObj. Short cut reference.
	; Size of z is (nx, ny).
	xType: '', $; "points" (size of x is nx) or "histo" (size of x is nx+1)
	yType: '' $; "points" (size of y is ny) or "histo" (size of y is ny+1)
}
;/

end
