; $Id: format_size.pro,v 1.2 2003/09/12 15:25:37 tanya Exp $
;+
; NAME:
;      format_size
;
; PURPOSE:
;	Returns a string of format "[#]", "[#,#]" , "[#,#,#]" , "[#,#,#,#]". 
; corresponding to the number of dimensions and the size of those
; dimensions for an array.
;
;	The keyword array is used with arrays (passed by value).
; Use this only for small arrays, to avoid efficiency problems.
;
;	The keyword ptr_array is used with pointers to arrays 
; (passed by reference). Use this for large arrays.
;
;	The keyword dim_array is used when one already knows the size of 
; array, from the IDL command size(). In this case, zeros on the end of the
; array are stripped off. 
;
;	Examples:
;      retString = 'OBJ' + format_size(array=thingy)
;      retString = '{PTR}'  +  format_size(ptr_array=thingy)
;      retString =   format_size(dim_array=[2,3,0,0])
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED: none
;
; EXAMPLE
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*
function format_size, $		; returns string of form [#] for 1D, [#,#] for 2D, etc.
  dim_array, $			; # or [#,#] or [#,#,#], etc.
  ptr_Array = ptr_Array,  $	; Pointer to an array for which we format the array size.
  Array = anArray		; An array for which we format the array size.
;/
;;  anArray,  $	; Array for which we format the array size.
;  showScalar=showScalar	; Make this the default.
; Taking result from size command vsize, return string [#] 1D, [#,#] 2D

   if (n_elements(dim_array) NE 0) then begin
     temp_dim = dim_array
     temp_ndim = n_elements(dim_array)
     temp = [temp_ndim, temp_dim]
   endif else begin
   
     if ((n_elements(ptr_Array) EQ 0) AND (n_elements(anArray) EQ 0)) then begin
;       return, '{NULL}'
       return, '[NULL]'
     endif 
 
     if (NOT ptr_valid(ptr_array)) then begin
;       return, '{NULL}'
       return, '[NULL]'
     endif
      
     if (n_elements(ptr_Array) gt 0) then begin
       temp = size(*(ptr_Array))
     endif 
     if (n_elements(anArray) gt 0) then begin
       temp = size(anArray)
     endif
   endelse
   
   if (temp[0] eq 0) then begin
;     if (keyword_set(showScalar)) then begin
;       return, '{0}'
       return, '[0]'
;     endif else begin
;       return, ''
;     endelse
   endif

;   astring = '{'
   astring = '['
   ;; temp[0]  is the number of dimensions
   for i = 1, temp[0] do begin
     astring = astring + strcompress(string(temp[i]), /remove_all)
     if (i NE temp[0]) then begin
       astring = astring + ','
     endif else begin
;       astring = astring + '}'
       astring = astring + ']'
     endelse
   endfor

   return, astring

end
