; $Id: func_icp_data_init.pro,v 1.4 2003/09/12 15:25:25 tanya Exp $
;+
; NAME:
;       func_icp_data_init
;
; PURPOSE:
;
; 	This is a fake method for the object self, of type specObj, 
;	which is the first parameter to func_icp_data_init
; 	Because it is not a true method, any operations performed upon 
; 	self must be done via's its methods within this code.
;
;	This code reads in the data columns for ICP data for triple axis data
;	into the object self.
;	This code also reads in the header information by calling
;	func_icp_header_init.
;	The initialization method for the calss specObj
;	can either initialize using a pre-exisiting Dave Pointer,
;	or it can call a fake method with a data file name.
;	In the case of triple axis data, the specObj is initialized by:
;
;     oSpec = obj_new('specObj', $	; create from a TAS data file 
;  	label=fileName, $
;	codeSource='Dimeo_example', $
;	instrument='bt7', $
;  	path = filePath, $
;	filename = fileName, $
;	datareader='func_icp_data_init', $
;	wID = wID)
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;       <LI><A HREF="#SPECOBJ::INIT,_arguments">SPECOBJ::INIT,_arguments</A>
; 
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Spring, 2003.
;	Check for eof(unit) then reading data columns. TMR. Sept 3, 2003.
;
;-
;================================================================================
 function icp_data_makeErrors_columns, $
 	self

  infolabel = "icp_data_makeErrors_columns >> "  
  debug = 0
  
  success = 1
 
;;  OK = self -> getField(['*', 'scanPts', '*'], /inHeader, qty=nrows)
  tempObj = self -> get( /inData, /all, count = ncols)
;  if (debug) then print, infolabel, "ncols,  nrows = ", ncols,  nrows

  for i = 0, (ncols -1) do begin
    OK = self -> getTagField(i, /inData, label=localLabel)
    if (localLabel EQ  'COUNTS') then begin
;;;    if (localLabel EQ  'INTENSITY') then begin
      OK = OK AND self -> getField(i, /inData, qty = tempArr)		; ptr_qty = tempPtr)
      etemp = sqrt(tempArr)
      etemp = etemp + (etemp EQ 0)	; Error on zero counts is 1.
;      etemp_ptr = ptr_new(sqrt(*(tempPtr)))
      OK = OK AND self -> setField(i, /inData, err = etemp, styleErr='poisson')
      if (not OK) then begin
    	print,  infolabel, 'Failure to create errors for column ', i
      endif
      success = success and  OK
    endif
    
  endfor

  return, success

 end		; makeErrors_columns

;================================================================================
function icp_data_read_columns, $
 	self, $
 	unit

  infolabel = "icp_data_read_columns >> "  
  debug = 0
  
  ncols = 0
  OK = self -> getField(['*', 'scanPts', '*'], /inHeader, qty=nrows)
  tempObj = self -> get( /inData, /all, count = ncols)
  if (debug) then print, infolabel, "ncols,  nrows = ", ncols,  nrows

  if ((NOT OK) OR (ncols EQ 0)) then  return, 0
  
  
  temp_array = make_array(ncols, nrows, /float) 
;Old  temp_array = make_array(nrows, ncols, /float) 
  temp1row = make_array(ncols, /float) 
  temp1Col = make_array(nrows, /float)


  aline = ''
;  for i = 0, (nrows -1) do begin		; Sept 3, 2003. TMR
;    readf, unit, temp1row
;    if (debug) then print, infolabel, i, "=row ", temp1row
;    temp_array[*, i] = temp1row
;;Old    temp_array[i, *] = temp1row
;  endfor					; Sept 3, 2003. TMR

  i = 0
  while (NOT eof(unit)) do begin	; Sept 3, 2003. TMR
    readf, unit, temp1row
    if (debug) then print, infolabel, i, "=row ", temp1row
    temp_array[*, i] = temp1row
    i = i + 1
    if (i EQ nrows) then goto, trunc_while
  endwhile
  trunc_while:
    
  if (i NE nrows) then begin			; Sept 3, 2003. TMR
    OK = dialog_message(string("File has prematurely truncated after ", $
    	strtrim(string(i), 2), " rows out of ", $
	strtrim(string(nrows), 2), $
	". The remainder is zero-filled."), /info)
  endif
  
  if (debug) then print, infolabel, "temp_array[",ncols,  nrows, "]="
  if (debug) then print, temp_array


;Debug  print, infolabel, "Copying data to tria_1column objects'
  success = 1
  for icol = 0, (ncols  -1) do begin
;    if (debug) then print, infolabel, "temp_array[", icol, ", *]=", $
;    	 temp_array[icol, *]
    if (debug) then print, infolabel, " Delay tatic"
;    temp1Col = transpose(temp_array[icol, *])
;    if (debug) then print, "temp1Col = ", temp1Col
    
    if (debug) then print, infolabel, " 2nd Delay tatic"
    tempPtr = ptr_new(transpose(temp_array[icol, *]))
;Debug    if (debug) then print, "*(tempPtr) = ", *(tempPtr)
     col_success  = self -> setField(icol, /inData, $
;     	 qty = temp1Col)
     	 ptr_qty = tempPtr)
     ptr_free, tempPtr 
     success = success and col_success
  endfor
  
  close, unit, exit_status=exit_status
  free_lun, unit
  if (exit_status NE 0) then begin
    print, infolabel, 'CLOSE exit_status=', exit_status
  endif
;  close, unit
  
  return, success
    
 end		; read_columns

;================================================================================
 function icp_data_prep_columns, $
	self, currentClass, currentCode, $
 	labels, $
	ncols, $
	nrows, $ 
;;;;;;;;
; Keywords for columns of stuff that might already be present in data files.
; These can be constructed from header info too. Useful for when we skip the
; "data" portion of the file and only read the header part.
  deltaEMissing = thisDeltaEMissing, $ 	; Boolean keyword. deltaE = E_in -E_out
  QxyzMissing = thisQxyzMissing, $	; Boolean keyword. 3 columns of Q(x), Q(y), Q(z).
  AnglesMissing = thisAnglesMissing	; Boolean keyword. Angle 3, 4 on triple axis.
	
; This prepares for typical columns of data found in triple axis ICP data files.

  infolabel = "icp_data_prep_columns >> "  

  thisDeltaEMissing = 1
  thisQxyzMissing = 1
  thisAnglesMissing = 1

  success = 1
  OK = self -> getField(['*', 'q_scan', '*'], /inheader, $
	  	err=q_increment, units=q_someunits)
  for i = 0, (ncols -1) do begin
    someUnits = ''
    someLabel = labels[i]			;;; strupcase(labels[i])
    someName = labels[i]
;    someLegend = ''
    someTname = 'FLOAT'
    somePendent = 'C'  ; Will be C: constant, D: dependent, I: independent 
    increment = 0.0
    case strupcase(labels[i]) of
      'Q(X)' : begin
          someLabel = 'Q(h)'
          someName = 'Q(h)'
;	  someLegend = 'h'
	  someUnits = q_someUnits
	  increment = q_increment[0]
  	  thisQxyzMissing = 0
	  end
      'Q(Y)' : begin
          someLabel = 'Q(k)'
          someName = 'Q(k)'
;	  someLegend = 'k'
	  someUnits = q_someUnits
	  increment = q_increment[1]
  	  thisQxyzMissing = 0
	  end
      'Q(Z)' : begin
          someLabel = 'Q(l)'
          someName = 'Q(l)'
;	  someLegend = 'l'
	  someUnits = q_someUnits
	  increment = q_increment[2]
  	  thisQxyzMissing = 0
	  end
      'E' :  begin
          OK = self -> getField(['*', 'e_scan', '*'], /inheader, $
	  	err=increment, units=someunits)
          OK = self -> getField(['*', 'which_fixed', '*'], /inheader, $
	  	qty=which_fixed)
	  someName =  'deltaE=E_in-E_out'
	  if (which_fixed EQ 'EA') then someName = someName + ', Analyzer fixed.'
	  if (which_fixed EQ 'EM') then someName = someName + ', Monitor fixed.'
;	  if (which_fixed EQ 'EA') then someName = 'Monitor Energy'
;	  if (which_fixed EQ 'EM') then someName = 'Analyzer Energy'
;;	  someLegend = someLegend + '_fixed'
;          someName = 'Energy' + someLegend + '_fixed'
	  thisDeltaEMissing = 0
	  end
      'T-ACT' :  begin
          OK = self -> getField(['*', 'temp_scan', '*'], /inheader, $
	  	err=increment, units=someunits)
	  someName = 'Measured Temperature'
	  end
      'TEMP' :  begin
          OK = self -> getField(['*', 'temp_scan', '*'], /inheader, $
	  	err=increment, units=someunits)
	  someName = 'Set Temperature'
	  end
      'MIN' :  begin
          someName = 'Raw Recorded Count Time'
;          someName = 'Recorded Time'
          someUnits = 'minutes'
	  end
      'COUNTS' :  begin
          someLabel = 'COUNTS'
          someName = 'Raw Counts'
;          someLabel = 'INTENSITY'
;          someName = 'Intensity'
          someUnits = 'Events'
	  somePendent = 'D'
	  someTname = 'ULONG'
	  end
      'INTENSITY' :  begin
          someLabel = 'COUNTS'
          someName = 'Raw Counts'
;          someLabel = 'INTENSITY'
;          someName = 'Intensity'
          someUnits = 'Events'
	  somePendent = 'D'
	  someTname = 'ULONG'
	  end
      'A3' :  begin
          someLabel = 'Ang3'
          someName = 'Angle 3'
;	  someLegend = '3'
          someUnits = 'Degrees'
	  somePendent = 'I'
	  thisAnglesMissing = 0
	  end
      '3' :  begin
          someLabel = 'Ang3'
          someName = 'Angle 3'
;	  someLegend = '3'
          someUnits = 'Degrees'
	  somePendent = 'I'
	  thisAnglesMissing = 0
	  end
      'A4' :  begin
          someLabel = 'Ang4'
          someName = 'Angle 4'
;	  someLegend = '4'
          someUnits = 'Degrees'
	  somePendent = 'I'
	  thisAnglesMissing = 0
	  end
      '4' :  begin
          someLabel = 'Ang4'
          someName = 'Angle 4'
;	  someLegend = '4'
          someUnits = 'Degrees'
	  somePendent = 'I'
	  thisAnglesMissing = 0
	  end
	else: begin
		print, infolabel, ' Label not recognised = "',labels[i], '"' 
		print, infolabel, ' Carry on anyway. Using label as the name.'
		print, infolabel, ' No idea what the units should be.'
;      		lab_success = ((*(self.oneDatum_ptr))[i]) -> set($
;		  name=strupcase(labels[i]), units='?') 
		end
    endcase

     if ((increment ne 0.0) and (somePendent NE 'D')) then begin
       somePendent = 'I'
     endif

    anErrMsg = ''
;debug    print, infolabel, "Adding qtyObj with name =",someName 
;    print, infolabel, "nrows=", nrows		; Aug 15, 2003 DEBUG
    self -> add, /inData, $
    	tags = [currentClass, somelabel , currentCode], $
	errMsg = anErrMsg, $
	contents = obj_new('qtyObj', name=someName, $
;;;		rank=1, dim=nrows, $
		rank=1, dim=[nrows,0,0,0], $
		tname= someTname, $
;		initValue=666, $
	  qty = make_array(nrows, type=mn_tname_2_tcode(someTname),value=666),$
		units=someunits)
    OK = (anErrMsg EQ '')
    if (OK) then begin
      OK = OK AND self -> setTagField(/inData, $
    	[currentClass, somelabel , currentCode],  $
    	pendent = somePendent)
    endif
    if (not OK) then begin
    	print,  infolabel, 'Failure to add or label column ', i, $
		' with label ', labels[i]		
    endif else begin
;Print      print, infolabel, OK, 'Added col=',i, $
;Print      	' tag = ', [currentClass, somelabel , currentCode]  
    endelse

    success = success and OK	
  endfor

  return, success
 end		; prep_columns

;================================================================================
 function icp_data_read_labels, $
;NotNeeded 	self, $
  	unit, $
	labels, $
	ncols

  maxcols  = 10			; hardcoded maximum number of columns
  labels = make_array(maxcols, /string)
 
  aline = ''
  readf, unit, aline
  theEnd = 0
  ipos = 0
  while (not theEnd) do begin
    aword = readword(aline, ipos)
    if (aword eq '') then begin
      theEnd = 1
    endif else begin
      labels[ipos] = aword
      ipos = ipos + 1
    endelse
  endwhile
  ncols = ipos
  
  if (ncols gt 0) then begin
    return, 1 
  endif else begin
    return, 0
  endelse
    
 end		; icp_data_read_labels


;================================================================================
pro icp_data_show, $
  self, $		; By default, everything.
  lun=thisLun, $ ; Optionally, write to a file. If absent, to screen
;  unit=thisUnit, $ ; Optionally, write to a file. If absent, to screen
;Moved  lab = thisLab, $
  header = thisHeader, $	; Only the header
  Tags = thisTags, $		; Only the tags, etc.
  Size = thisSize, $		
  Status = thisStatus, $		
  Labels = thisLabels, $	
  column = thisColumn, $		; Show only one column, with label thisColumn.
  full = thisFull		; Show everything  

;moved  showLab = keyword_set(thisLab)
  showHeader = keyword_set(thisHeader)
  showTags = keyword_set(thisTags)
  showSize = keyword_set(thisSize)
  showStatus = keyword_set(thisStatus)
  showLabels = keyword_set(thisLabels)
  showColumn = keyword_set(thisColumn)

  if (keyword_set(thisFull)) then begin
    showAll = thisFull
  endif else begin
    showAll = $			; (not showLab) and $
  	(not showHeader) and (not showTags) and (not showSize) and $
  	(not showStatus) and (not showLabels) and (not showColumn)
  endelse

;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse


;This info moved to header
;  if (showLab or ShowAll) then begin  
;;    if (n_elements(unit) eq 0) then begin
;;      print, 'Lab=', self.lab
;;      print, 'Instrument= ', self.instrument
;;      print, 'Flipper_code (a,b,c,d, or n for none)= "', self.flipper_code, '"'
;;    endif else begin
;;      printf, thisUnit, 'Lab=', self.lab
;;      printf, thisUnit,  'Instrument= ', self.instrument
;;      printf, thisUnit,  'Flipper_code (a,b,c,d, or n for none)= "', self.flipper_code, '"'
;;    endelse
;    printf, lun, 'Lab=', self.lab
;    printf, lun,  'Instrument= ', self.instrument
;    printf, lun,  'Flipper_code (a,b,c,d, or n for none)= "', self.flipper_code, '"'
;  endif

;  self->database::show, unit=thisUnit, $
  self->database::show, lun=lun, full=thisFull, $
  	header = thisHeader, Tags = thisTags,  Size = thisSize, $	
	Status = thisTags, Labels = thisLabels, column = thisColumn

end			; icp_data_show

;================================================================================
function icp_data_construct, $
  self, $
  deltaE = thisDeltaE, $ 	; Boolean keyword. deltaE = E_in -E_out
  Ein = thisEin, $ 	; Boolean keyword. Incoming Energy
  Eout = thisEout, $ 	; Boolean keyword. Outgoing Energy
  q_mag = thisQ_mag, $ 	; Boolean keyword. Magnitude of Q
  inPlane = thisInplane,$ ;Boolean keyword. in-scattering-plane zeta & eta coords
;  zeta = thisZeta, $ 	; Boolean keyword. in-scattering-plane zeta coordinates
;  eta = thisEta, $ 	; Boolean keyword. in-scattering-plane eta coordinates
  index = thisIndex, $ 	; Boolean keyword. scan point number (0- (scanPts-1))
;;;;;;;;
; Keywords for columns of stuff that might already be present in data files.
; These can be constructed from header info too. Useful for when we skip the
; "data" portion of the file and only read the header part.
  Qxyz = thisQxyz, $	; Boolean keyword. 3 columns of Q(x), Q(y), Q(z).
  Temp = thisTemp, $	; Boolean keyword. Set point temperature.
  Angles = thisAngles, $; Boolean keyword. Angle 3, 4 on triple axis.
; Keywords for columns of derived stuff.
  AvTemp = thisAvTemp, $; Boolean keyword. Average sample temperature and 
  			; standard deviation, calculated from data column, 
			; and stored in the header.
; Generic keywords
  all = thisAll, $ 	; Boolean keyword. Construct all of the above
  varying = thisVarying, $ 	; Boolean keyword. Construct only those above
  			; which are varying. Use to modify the other keywords.
  selectTags = thisSelectTags ; Optionally set the default plotting tags.
  
; Construct new columns of data based on header information and/or
; pre-existing data columns.

  infolabel = "icp_data_construct >> "  

  success = 1
  debug = 0
  
  currentClass = 'qtyObj'
  currentCode= 'icp_data_construct'
  OKpendent = 1		; C for constant, D for dependent, I for independent
  
  ;----------------------------------------------------------------------
	if (keyword_set(thisSelectTags)) then begin
	  newTag_x  = ''
;	  newTag_x  = 'INDEX'
	  newTag_y = ''
	  newTag_z = ''
	  newCodeSource = currentCode
;	  self.tag_x = ''
;	  self.tag_y = ''
;	  self.tag_z = ''
	endif
  ;----------------------------------------------------------------------

  doEin = 0
  doEout = 0
  doDeltaE = 0
  doQ_mag = 0
  doInPlane = 0
;  doZeta = 0
;  doEta = 0
  doIndex = 0
  DoQxyz = 0
  DoTemp = 0
  DoAngles = 0
  DoAvTemp = 0

  if (keyword_set(thisDeltaE) or keyword_set(thisAll)) then doDeltaE = 1  
  if (keyword_set(thisEin) or keyword_set(thisAll)) then doEin = 1  
  if (keyword_set(thisEout) or keyword_set(thisAll)) then doEout = 1
  if (keyword_set(thisQ_mag) or keyword_set(thisAll)) then doQ_mag = 1
  if (keyword_set(thisInPlane) or keyword_set(thisAll)) then doInPLane = 1
;  if (keyword_set(thisInPlane) or keyword_set(thisAll)) then doInPLane = 0
;TEMPORARY  if (keyword_set(thisInPlane) or keyword_set(thisAll)) then doInPLane = 1
;  if (keyword_set(thisZeta) or keyword_set(thisAll)) then doZeta = 1
;  if (keyword_set(thisEta) or keyword_set(thisAll)) then doEta = 1
  if (keyword_set(thisIndex) or keyword_set(thisAll)) then doIndex = 1
  if (keyword_set(thisQxyz) or keyword_set(thisAll)) then doQxyz = 1
  if (keyword_set(thisTemp) or keyword_set(thisAll)) then doTemp = 1
  if (keyword_set(thisAngles) or keyword_set(thisAll)) then doAngles = 1
  if (keyword_set(thisAvTemp) or keyword_set(thisAvTemp)) then doAvTemp = 1

  if (debug) then self -> show, /recursive

;debug  print, infolabel, "doQ_mag = ", doQ_mag
  if (debug) then begin
    print, infolabel, 'doDeltaE=', doDeltaE,$
    	'  doEin=', doEin, ' doEout=', doEout, $
    	' doQ_mag=', doQ_mag, ' doInPlane=', doInPlane, ' doIndex=', doIndex, $
	' doQxyz =', doQxyz, ' doTemp=', doTemp, ' doAngles=', doAngles, $
	' doAvTemp=', doAvTemp
    print, infolabel, 'keyword_set(thisAll)=', keyword_set(thisAll)
  endif

  ;---------------------------------------------------------------------
  OK = self -> getField(['*', 'scanPts', '*'], /inHeader, qty=nrows)

  ;---------------------------------------------------------------------
    ; If the information is not available (e.g. 'I' scan),
    ; do not construct the corresponding columns.
  if (doDeltaE or doEin or doEout) then begin	 ;Note:  StyleErr = 'center_step'
    OK = self -> getField(['*', 'e_scan', '*'], /inHeader, $
    		qty=ec, err=es, units=ec_units, hasStep = e_hasStep)
;;    OK = OK and (self.header)-> get('e_fixed', qty=e_fixed)
;;    OK = OK and (self.header)-> get('which_fixed', qty=which_fixed)
    OK = OK and self -> getField(['*', 'which_fixed', '*'], /inHeader, $
    		qty=which_fixed) 
    OK = OK and self -> getField(['*', 'e_fixed', '*'], /inHeader, qty=e_fixed)
    if (debug AND OK) then begin
      print, infolabel, 'ec, es, which_fixed=', $
      		ec, es, which_fixed
    endif
    doDeltaE = doDeltaE and OK 
    doEin = doEin and OK 
    doEout = doEout and OK 
  endif
  
  if (doQ_mag or doQxyz) then begin		 ;Note:  StyleErr = 'center_step'
    OK = self -> getField(['*', 'q_scan', '*'], /inHeader, $
    	qty=qc_3vector, err=qs_3vector, units=qc_units)	;, hasStep = q_hasStep)

;    OK = (self.header)-> get('q_step', qty_array=qs_3vector) 
;    OK = OK and (self.header)-> get('q_center', qty_array=qc_3vector)
    doQ_mag = doQ_mag and OK
    doQxyz = doQxyz and OK
  endif
  
;??  ts = 0.0
  if (doTemp) then begin		 	;Note:  StyleErr = 'start_step'
    OK = self -> getField(['*', 'temp_scan', '*'], /inHeader, $
    		qty=t0, err=ts, units=t_units, hasStep = t_hasStep)
;    OK = (self.header)-> get('temp_step', qty_array=tstep)
;    OK = OK and (self.header)-> get('temp_start', qty_array=t0)
    doTemp = doTemp and OK
    if (debug) then message, /info, string("doTemp=", doTemp, " OK=", OK)
   if (n_elements(ts) GT 0) then  begin
     if (debug) then message, /info, string(" ts=", ts)
   endif else begin
     ts = 0.0
   endelse
     
  endif
  
  if (doInPlane) then begin 
    OK = self -> getField(['*', 'scatPlane_u', '*'], /inHeader, $
    		qty=temp_uvec,  units=uv_units)
    OK = OK AND self -> getField(['*', 'scatPlane_v', '*'], /inHeader, $
    		qty=temp_vvec)		;; ,  units=v_units)
;    OK = (self.header) -> get('scatPlane_u',  qty_array=temp_uvec)
;    OK = OK and (self.header) -> get('scatPlane_v', qty_array=temp_vvec)
    doInPlane = doInPlane and OK
  endif

  if (doAngles) then begin		 	;Note:  StyleErr = 'start_step'
    OK = self -> getField(['*', 'mot_angle', '*'], /inHeader, $
    	qty=angle_start, err=angle_step, units=angle_units)
;    OK = (self.header)->get('mot_angle_start',qty_array=angle_start)
;    OK = OK and (self.header)->get('mot_angle_step', qty_array=angle_step)
    doAngles = doAngles and OK
  endif
  
  if (doAvTemp) then begin		 	;Note:  StyleErr = 'start_step'
    OK = self -> getField(['*', 'T-ACT', '*'], /inData, $
    		qty=tscan, units=tscan_units)
     if (debug and OK) then print, infolabel, " T-act =", tscan, tscan_units
     if (NOT OK) then begin
       OK = self -> getField(['*', 'TEMP', '*'], /inData, $
    		qty=tscan, units=tscan_units)     
       if (debug and OK) then print, infolabel, " TEMP =", tscan, tscan_units
     endif
     if (debug and (not OK)) then print, infolabel, "Failed to find temp column!"
     doAvTemp = doAvTemp and OK
  endif
  
  if (debug) then begin
    print, infolabel, 'After checking info exists'
    print,  'doDeltaE=', doDeltaE,$ 'doEin=', doEin, ' doEout=', doEout, $
    	' doQ_mag=', doQ_mag, ' doInPlane=', doInPlane, ' doIndex=', doIndex, $
	' doQxyz =', doQxyz, ' doTemp=', doTemp, ' doAngles=', doAngles, $
	' doAvTemp=', doAvTemp
    print, infolabel, 'keyword_set(thisAll)=', keyword_set(thisAll)
  endif

  ;---------------------------------------------------------------------
  if (keyword_set(thisVarying)) then begin
    doEin_varying = 1
    doEout_varying = 1
    doDeltaE_varying = 1
    if (doEin or DoEout or doDeltaE) then begin
;n      OK = (self.header)-> get('e_step', qty_array=es)
;n      OK = (self.header)-> get('which_fixed', qty_array=which_fixed)
;n      if (debug) then begin
;n        print, infolabel, 'doEin. es, which_fixed=', es, which_fixed
;n      endif
      if ((which_fixed eq 'EM') or (es EQ 0.0)) then doEin_varying = 0
      if ((which_fixed eq 'EA') or (es EQ 0.0)) then doEout_varying = 0
      if (es EQ 0.0) then doDeltaE_varying = 0
;      if ((which_fixed eq 'EM') or (es EQ 0.0)) then doEin = 0
;      if ((which_fixed eq 'EA') or (es EQ 0.0)) then doEout = 0
;      if (es EQ 0.0) then doDeltaE = 0
    endif


    if (doQ_mag) then begin
      doQ_mag_varying = 1
;n      OK = (self.header)-> get('q_step', qty_array=qs_3vector)
;      if (norm(qs_3vector) EQ 0.0) then doQ_mag = 0
      if (norm(qs_3vector) EQ 0.0) then doQ_mag_varying = 0
    endif

    if (doTemp) then begin
      doTemp_varying = 1
;Debug      message, /info, string("Pre-crash: doTemp=", doTemp)
;n      OK = (self.header)-> get('temp_step', qty_array=tstep)
      if (n_elements(ts) EQ 0) then begin
;        doTemp = 0
        doTemp_varying = 0
      endif else begin
;        if (ts EQ 0.0) then doTemp = 0
        if (ts EQ 0.0) then doTemp_varying = 0
      endelse
;Debug      message, /info, string("Post-crash: doTemp=", doTemp)
    endif

    ; Checks for Qxyz and Angles are done below. 
  endif
  
  ;----------------------------------------------------------------------
  if (doEin) then begin
;n    OK = (self.header)-> get('e_center', qty_array=ec, units=ec_units)
;n    OK = (self.header)-> get('e_step', qty_array=es)
;n    OK = (self.header)-> get('e_fixed', qty_array=e_fixed)
;n    OK = (self.header)-> get('which_fixed', qty_array=which_fixed)
      if (debug) then begin
        print, infolabel, 'doEin 2. es, which_fixed=', es, which_fixed
      endif

    str_status = 'C'
    aVector = fltarr(nrows) + e_fixed
    if (which_fixed eq 'EM') then begin
	; Do nothing
    endif else begin		;'EA'
      if (debug) then begin
        print, infolabel, 'doEin: ec, es, e_fixed=', ec, es, e_fixed
      endif

      if (es NE 0.0) then begin
        str_status = 'I'
      	aVector = aVector + ec +  $
	  es*(make_array(nrows,/float,/index)-$
	  (float(nrows)-1.0)/2.0)
;	if (keyword_set(thisSelectTags)) then begin
;	  newTag_x  = ''
;;	  self.tag_x = 'E_in'
;	endif
      endif else begin
      endelse	
    endelse
    
    i_index = where( aVector LT 0.0, count)
    if (count gt 0) then begin
      print, infolabel, 'E_in =', aVector
      print, infolabel, '/Ein ', $
      	'Non-positive energy at monochromator at scan points ', i_index
      success = 0
    endif

    if (doEin_varying) then begin
      self -> add, tags = [currentClass, 'E_in', currentCode], $
      	/inData, $
        contents = obj_new('qtyObj', name = 'Incoming Energy', $
		qty = aVector, units=ec_units)
      OKpendent = OKpendent AND self -> setTagField(/inData, $
    	[currentClass, 'E_in' , currentCode],  $
    	pendent = str_status)
	
    endif else begin
      self -> add, tags = [currentClass, 'E_in', currentCode], $
      	/inHeader, $
        contents = obj_new('qtyObj', name = 'Incoming Energy', $
		qty = aVector[0], units=ec_units)
      
    endelse
    

  endif			; (keyword_set(thisEin)

  ;----------------------------------------------------------------------
  if (doEout) then begin
;n    OK = (self.header)-> get('e_center', qty_array=ec, units=ec_units)
;n    OK = (self.header)-> get('e_step', qty_array=es)
;n    OK = (self.header)-> get('e_fixed', qty_array=e_fixed)
;n    OK = (self.header)-> get('which_fixed', qty_array=which_fixed)
      if (debug) then begin
        print, infolabel, 'doEout 2. es, which_fixed=', es, which_fixed
      endif
    str_status = 'C'
    aVector = fltarr(nrows) + e_fixed
    if (which_fixed eq 'EM') then begin
      if (debug) then begin
        print, infolabel, 'doEout: ec, es, e_fixed=', ec, es, e_fixed
      endif
      if (es NE 0.0) then begin
        str_status = 'I'
;	if (keyword_set(thisSelectTags)) then begin
;	  newTag_x  = ''
;;	  self.tag_x = 'E_out'
;	endif
      	aVector = aVector -1.0*ec  $
	  -es*(make_array(nrows,/float,/index)-$
	  (float(nrows)-1.0)/2.0)

;        aVector = aVector - es*(make_array(self.num_ntple, /float, /index) $
;		-(float(self.num_ntple)-1.0)/2.0)
      endif
    endif else begin		;'EA'
;;;      aVector = fltarr(self.num_ntple) + e_fixed
    endelse
    i_index = where( aVector LT 0.0, count)
    if (count gt 0) then begin
      print, infolabel, 'E_out =', aVector
      print, infolabel, '/Eout ', $
      	'Non-positive energy at analyser at scan points ', i_index
      success = 0
    endif

    if (doEout_varying) then begin
      self -> add, tags = [currentClass, 'E_out', currentCode], $
      	/inData, $
        contents = obj_new('qtyObj', name = 'Outgoing Energy', $
		qty = aVector, units=ec_units)
      OKpendent = OKpendent AND self -> setTagField(/inData, $
    	[currentClass, 'E_out' , currentCode],  $
    	pendent = str_status)
	
    endif else begin
      self -> add, tags = [currentClass, 'E_out', currentCode], $
      	/inHeader, $
        contents = obj_new('qtyObj', name = 'Outgoing Energy', $
		qty = aVector[0], units=ec_units)
    endelse
    
  endif			; (keyword_set(thisEout)

  ;----------------------------------------------------------------------        
  if (doDeltaE or doEin or doEout) then begin	 ;Note:  StyleErr = 'center_step'
      if (es NE 0.0) then begin
	if (keyword_set(thisSelectTags)) then begin
	  newTag_x  = 'E'		; use that which came in the data file
	  newCodeSource = 'func_icp_data_init'
	endif
        if (debug) then print, infolabel, "es=", es, " xtag = ", newtag_x
      endif
  endif

  if (doDeltaE) then begin		; construct 
;n    OK = (self.header)-> get('e_center', qty_array=ec, units=ec_units)
;n    OK = (self.header)-> get('e_step', qty_array=es)
;n    OK = (self.header)-> get('e_fixed', qty_array=e_fixed)
;n    OK = (self.header)-> get('which_fixed', qty_array=which_fixed)
      if (debug) then begin
        print, infolabel, 'doDeltaE 2. es, which_fixed=', es, which_fixed
      endif
    str_status = 'C'
      if (es NE 0.0) then begin
        str_status = 'I'
      	aVector = ec  $
	   + es*(make_array(nrows,/float,/index)-$
	  (float(nrows)-1.0)/2.0)
	if (keyword_set(thisSelectTags)) then begin
	  newTag_x  = 'deltaE'
	endif

      endif else begin
      	aVector =make_array(nrows,/float) +  ec 
      endelse

    if (doDeltaE_varying) then begin
      self -> add, tags = [currentClass, 'deltaE', currentCode], $
      	/inData, $
        contents = obj_new('qtyObj', name = 'deltaE=E_in-E_out', $
		qty = aVector, units=ec_units)
      OKpendent = OKpendent AND self -> setTagField(/inData, $
    	[currentClass, 'deltaE' , currentCode],  $
    	pendent = str_status)
    endif else begin
      self -> add, tags = [currentClass, 'deltaE', currentCode], $
      	/inHeader, $
        contents = obj_new('qtyObj', name = 'deltaE=E_in-E_out', $
		qty = aVector[0], units=ec_units)
    endelse
     
  endif			; (keyword_set(thisDeltaE)

;Debug    print, infolabel, "Halvies =", (float(nrows)-1.0)/2.0

   ;----------------------------------------------------------------------
  if (doQxyz) then begin
;;; missing the needed function.
;n    iwhere = intarr(3)
;n    iwhere[0] = self->idatum('Q(X)')	; does it already exist? If so, skip it.
;n    iwhere[1] = self->idatum('Q(Y)')
;n    iwhere[2] = self->idatum('Q(Z)')
;n    OK = (self.header)-> get('q_center', qty_array=qc_3vector, units=qc_units)
;n    OK = OK and (self.header)-> get('q_step', qty_array=qs_3vector)
    if (debug) then begin
      print, infolabel, ' doQxyz. ', qc_3vector, qs_3vector
;n      print, infolabel, ' doQxyz. iwhere =', iwhere, qc_3vector, qs_3vector
    endif
    aVector = make_array(nrows, /float)
    label3 = ['Q(h)', 'Q(k)', 'Q(l)']
    for i = 0, 2 do begin
;n      if (iwhere[i] lt 0) then begin
        if (qs_3vector[i] NE 0.0) then begin
          str_status = 'I'
        endif else begin
          str_status = 'C'
        endelse
;Debug	print, infolabel, 'self.num_datum=',self.num_datum, $
;Debug		 ' self.num_ntple=',self.num_ntple 
	if ((not keyword_set(thisVarying)) or (qs_3vector[i] NE 0.0)) then begin
          Avector = qc_3vector[i] + $
    		qs_3vector[i]*(make_array(nrows, /float, /index)$
    		- (float(nrows)-1.0)/2.0)
    	  self -> add, tags = [currentClass, label3[i], currentCode], $
      	     /inData, $
       	     contents = obj_new('qtyObj', name = label3[i], $
		qty = aVector, units=qc_units)
    	  OKpendent = OKpendent AND self -> setTagField(/inData, $
    		[currentClass, 'deltaE' , currentCode],  $
    		pendent = str_status)
;           OK = self->add_column( label=label3[i], name = label3[i], $
;	 	 status=str_status, $
;        	qty_array = aVector, units=qc_units)
;n	 endif
       endif
     endfor
  endif			; (keyword_set(thisQxyz)

   ;----------------------------------------------------------------------
  if (doQ_mag) then begin
;  if (keyword_set(thisQ_mag) or keyword_set(thisAll)) then begin
;n    OK = (self.header)-> get('q_center', qty_array=qc_3vector, units=qc_units)
;    print, infolabel, 'units=', qc_units	; also a 3 vector of strings.
;n    OK = (self.header)-> get('q_step', qty_array=qs_3vector)
    aVector = make_array(nrows, /float)
    if (norm(qs_3vector) gt 0.0) then begin
      str_status = 'I'
	if (keyword_set(thisSelectTags)) then begin
	  newTag_x  = '|Q|'   
;	  self.tag_x = 'Q_MAG'
	endif
        if (debug) then print, infolabel, "qs=", norm(qs_3vector), $
		" xtag = ", newtag_x
    endif else begin
      str_status = 'C'
    endelse
    for i = 0, (nrows-1) do begin
      q_vector = qc_3vector + qs_3vector*(i - ((float(nrows)-1.0)/2.0) )
;03July9Wrong     q_vector = qc_3vector + qs_3vector*((i - (float(nrows)-1.0))/2.0)
      aVector[i] =norm(q_vector)
;Debug      print, infolabel,  i, "q_vector=", q_vector, " norm=", aVector[i]
    endfor
;    print, 'aVector=',  aVector
    self -> add, tags = [currentClass, '|Q|', currentCode], $
      	     /inData, $
       	     contents = obj_new('qtyObj', name = 'Magnitude of Q', $
		qty = aVector, units=qc_units)
    OKpendent = OKpendent AND self -> setTagField(/inData, $
    		[currentClass, '|Q|' , currentCode],  $
    		pendent = str_status)
     
  endif			; (keyword_set(thisQ_mag)

  ;----------------------------------------------------------------------
    TisConstant = 0
    OK = self -> getField(['*', 'T-act', '*'], /inData, isConstant = TisConstant)

    if ((NOT TisConstant) AND OK ) then begin
	if (keyword_set(thisSelectTags) AND (newTag_x EQ '') ) then begin
	  newTag_x  = 'T-act'	; use the recorded temperature
	  newCodeSource = 'func_icp_data_init'
        endif
    endif
    if (debug) then print, infolabel, "TisConstant=", TisConstant, $
		" xtag = ", newTag_x  

  if (doTemp) then begin
;n    iwhere = self->idatum('T') ; does it already exist? If so, skip it.
;n    if (iwhere lt 0) then begin
;n      OK = (self.header)-> get('temp_start', qty_array=t0, units=t0_units)
;n     OK = OK and (self.header)-> get('temp_step', qty_array=ts)
      aVector = fltarr(nrows) + t0
      str_status = 'C'
      if ((ts NE 0.0) and OK) then begin
        str_status = 'I'
	if (keyword_set(thisSelectTags) AND (newTag_x EQ '') ) then begin
	  newTag_x  = 'T'	; use the recorded temperature
;	  self.tag_x = 'T'
        endif
      endif
      aVector = aVector + ts*(make_array(nrows, /float, /index))

;Debug	print, infolabel, 't vector=', avector	; debug

    if (dotemp_varying) then begin
      self -> add, tags = [currentClass, 'T', currentCode], $
      	     /inData, $
       	     contents = obj_new('qtyObj', name = 'Temp set point', $
		qty = aVector, units=t_units)
      OKpendent = OKpendent AND self -> setTagField(/inData, $
    		[currentClass, 'T' , currentCode],  $
    		pendent = str_status)
    endif else begin
      self -> add, tags = [currentClass, 'T', currentCode], $
      	     /inHeader, $
       	     contents = obj_new('qtyObj', name = 'Temp set point', $
		qty = aVector[0], units=t_units)
    endelse
    
  endif			; (keyword_set(thisTemp)

   ;----------------------------------------------------------------------
  if (doAngles) then begin
    ; Only really interested in A3 and A4. 
    ; Not all files have self.mot_angle_step defined, so might also
    ; write some code to calculate it.
;n    iwhere = make_array(6, /int, value=-1)
;n	; does it already exist? If so, skip it. -1 means not found.
;n    iwhere[3-1] = max([self->idatum('A3'), self->idatum('3')])
;n    iwhere[4-1] = max([self->idatum('A4'), self->idatum('4')])
;    print, infolabel, 'Doangles, iwhere=', iwhere
;n    OK = (self.header)->get('mot_angle_start',qty_array=angle_start,units=angle_units)
;n    OK = OK and (self.header)->get('mot_angle_step', qty_array=angle_step)
;n    if (OK) then begin
     ; Skip this construction of the angle step size is not defined.
     label6 = ['A1', 'A2', 'A3', 'A4', 'A5', 'A6']
     if (debug) then begin
;n       print, 'doAngles.  iwhere=', iwhere
       print, 'doAngles.  label6=', label6
     endif
     for i = (1-1), (6-1) do begin
      if (debug) then begin
        print, infolabel, 'doAngles,  Angle ', i+1, ' label=', label6[i]
      endif
;n      if (iwhere[i] lt 0) then begin
        if (angle_step[i] NE 0.0) then begin
          str_status = 'I'
	endif else begin
          str_status = 'C'
        endelse
	if ((not keyword_set(thisVarying)) and (angle_step[i] NE 0.0)) then begin
          Avector = angle_start[i] + $
    		angle_step[i]*make_array(nrows, /float, /index)
          self -> add, tags = [currentClass, 'T', currentCode], $
      	     /inData, $
       	     contents = obj_new('qtyObj', name = label6[iangle], $
		qty = aVector, units=angle_units)
          OKpendent = OKpendent AND self -> setTagField(/inData, $
    		[currentClass, label6[iangle] , currentCode],  $
    		pendent = str_status)
;          OK = self->add_column( label=label6[iangle], name = label6[iangle], $
;	 	 status=str_status, $
;        	qty_array = aVector, units=qc_units)
	 endif
;n       endif

 	  if (keyword_set(thisSelectTags) and (angle_step[i] NE 0.0)) then begin
	    ; possible that both A3 and A4 are varying.
	    if ((i eq (4-1)) and (newTag_x eq label6[3-1])) then begin
	      newTag_y  = label6[i]
;	      self.tag_y = label6[i]
	      if (debug) then begin
	        print, infolabel, ' Setting self.tag_y to ', newtag_y 
	      endif
	    endif else begin
	      newTag_x  = label6[i]
;	      self.tag_x = label6[i]
	      if (debug) then begin
	        print, infolabel, ' Setting self.tag_x to ', newtag_x 
	      endif
	    endelse
	  endif
            
     endfor
;n    endif		; if (OK)
  endif			; (keyword_set(thisAngles)
;Debug  print, infolabel, " doAvTemp = ", doAvTemp
  ;---------------------------------------------------------------------------
  if (doAvTemp) then begin
      if (n_elements(tscan) GT 1) then begin
        Tmean = mean(tscan)
        temp_Terr = stddev(tscan)
;Debug      print, infolabel, "Calculating average temp=", Tmean, " +- ", temp_terr 
        if (temp_Terr NE 0.0) then Terr = temp_Terr ; otherwise leave undefined.
      endif else begin
        if (n_elements(tscan) EQ 1) then begin
          Tmean = tscan
	  Terr = 0.0
	endif
      endelse
      
      self -> add, tags = [currentClass, 'AvTemp', currentCode], /inHeader, $
       	     contents = obj_new('qtyObj', name = 'Average Temperature', $
		qty = Tmean, err=Terr, units=tscan_units)
   endif			; (keyword_set(thisAvTemp)

  ;----------------------------------------------------------------------
  if (doInPlane) then begin
;  if (doZeta or doEta) then begin
    ; First make the irreducible pair uirr and virr from u and v:
    ; Just incase something has been changed.

    self -> remove, position=['*', 'scatPlane_uirr', '*'], /inHeader, /destroy
    self -> remove, position=['*', 'scatPlane_virr', '*'], /inHeader, /destroy
    triasubs_irreduc, temp_uvec, temp_vvec, temp_uirrvec, temp_virrvec

;    OK = self -> setField( /inHeader, $
;    	[currentClass, 'scatPlane_uirr', currentCode], $
;	qty = temp_uirrvec)
;    OK = self -> setField(  /inHeader, $
;    	[currentClass, 'scatPlane_virr', currentCode], $
;	qty = temp_virrvec)

    self -> add, /inHeader, $
    	tags = [currentClass, 'scatPlane_uirr', currentCode], $
	contents = obj_new('qtyObj', $
		name='scatPlane_uirr', $
		qty = temp_uirrvec, units = uv_units)
    self -> add, /inHeader, $
    	tags = [currentClass, 'scatPlane_virr', currentCode], $
	contents = obj_new('qtyObj', $
		name='scatPlane_virr', $
		qty = temp_virrvec, units = uv_units)

    Zeta_Vector = make_array(nrows, /float)
    Eta_Vector = make_array(nrows, /float)
    outOfPlane_Vector = make_array(nrows, /float)

    OK = self -> getfield(['*', 'bmat', '*'],  /inHeader, $
    	qty =  temp_bmat)  
    if (NOT OK) then begin
      print, infolabel, 'bmat has not been initialized.'
      goto, skip_inPlane
    endif ;else begin
    
    OK = self -> getField(['*', 'q_scan', '*'], /inHeader, $
    		qty=qc_3vector, $
		err=qs_3vector, $; This is really a step, 
				 ; due to styleErr = 'step_center'
		units=qc_units)

    
    OK_h = self -> getField(['*', 'Q(h)', '*'], /inData, qty = q_h)
    OK_k = self -> getField(['*', 'Q(k)', '*'], /inData, qty = q_k)
    OK_l = self -> getField(['*', 'Q(l)', '*'], /inData, qty = q_l)

    if (NOT OK_h) then begin
      q_h = qc_3vector[0] + $
    		qs_3vector[0]*(make_array(nrows, /float, /index)$
    		- (float(nrows)-1.0)/2.0)
    endif
    if (NOT OK_k) then begin
      q_k = qc_3vector[1] + $
    		qs_3vector[1]*(make_array(nrows, /float, /index)$
    		- (float(nrows)-1.0)/2.0)
    endif
    if (NOT OK_l) then begin
      q_l = qc_3vector[2] + $
    		qs_3vector[2]*(make_array(nrows, /float, /index)$
    		- (float(nrows)-1.0)/2.0)
    endif
    
    zeta_vector = q_h*temp_bmat[0,0] + q_k*temp_bmat[0, 1] + q_l*temp_bmat[0,2]
    eta_vector  = q_h*temp_bmat[1,0] + q_k*temp_bmat[1,1] + q_l*temp_bmat[1,2]
    outofplane_vector= q_h*temp_bmat[2,0] + q_k*temp_bmat[2,1] $
    		+ q_l*temp_bmat[1,2]
    iout_where  = where(abs(outofplane_vector) gt 1.1e-5)
    for i = 0, (nrows -1) do begin
      if (abs(outofplane_vector[i]) gt 1.1e-5) then begin
        ; Use this latter to draw "forbidden" areas on the plot.
        print, infolabel, 'DoInPlane: ', $
		'Reciprocal lattice point is out of the plane'
	print, i, zeta_vector[i], eta_vector[i], outofplane_vector[i]
      endif else begin
        ; Use this latter to draw "OK" areas on the plot.
        outofplane_vector[i] = 0.0	; Make it clearly zero
      endelse
    endfor
    
    if (norm(qs_3vector) eq 0.0) then begin
      status_str = 'C'		; constant
    endif else begin
      status_str = 'I'		; independent variable
    endelse

    self -> add, /inData, $
    	tags = [currentClass, 'Q(zeta)', currentCode], $
	contents = obj_new('qtyObj', $
		name='zeta. Scattering plane unit vector', $
		qty = zeta_Vector, units = uv_units)
    OKpendent = self -> setTagField(/inData, $
    		[currentClass, 'Q(zeta)' , currentCode],  $
    		pendent = status_str)

    self -> add, /inData, $
    	tags = [currentClass, 'Q(eta)', currentCode], $
	contents = obj_new('qtyObj', $
		name='eta. Scattering plane unit vector', $
		qty = eta_Vector, units = uv_units)
    OKpendent = self -> setTagField(/inData, $
    		[currentClass, 'Q(eta)' , currentCode],  $
    		pendent = status_str)

    
    if (norm(outOfPlane_Vector) NE 0.0) then begin
      self -> add, /inData, $
    	tags = [currentClass, 'Q_OUTOFPLANE', currentCode], $
	contents = obj_new('qtyObj', $
		name='OUTOFPLANE. Scattering plane unit vector', $
		qty = outOfPlane_Vector, units = uv_units)
      OKpendent = self -> setTagField(/inData, $
    		[currentClass, 'Q_OUTOFPLANE' , currentCode],  $
    		pendent = status_str)
    endif
    skip_inPlane:
  endif			; keyword_set(thisInPlane)

  ;----------------------------------------------------------------------
  if (doIndex) then begin
;  if (keyword_set(thisIndex) or keyword_set(thisAll)) then begin
    aVector = make_array(nrows, /float, /index)
      self -> add, tags = [currentClass, 'INDEX', currentCode], $
      	     /inData, $
       	     contents = obj_new('qtyObj', name = 'Scan index number', $
		qty = aVector)
;03May28		qty = aVector, units=t_units)
      OKpendent = OKpendent AND self -> setTagField(/inData, $
    		[currentClass, 'INDEX' , currentCode],  $
    		pendent = 'I')
;    OK = self->add_column( label='INDEX', name = 'Scan index number', $
;     	 status='I', qty_array = aVector)
    if (newTag_x EQ '') then begin
      newTag_x = 'INDEX'
    endif
     
  endif			; (keyword_set(thisIndex)

  ;----------------------------------------------------------------------
  ; Finished creating data. Let's finish up the tags.
  ;----------------------------------------------------------------------
  if (keyword_set(thisSelectTags)) then begin

	; set the dependent variable.
;    newtag_z = 'INTENSITY'  
;    OKintensity = self -> getTagField(['*','INTENSITY', '*'], $
;    	   /inData, codeSource = zCodeSource)

    newtag_z = 'COUNTS'  
    OKintensity = self -> getTagField(['*','COUNTS', '*'], $
    	   /inData, codeSource = zCodeSource)
    if (debug) then print, infolabel, "zCodeSource = ",  zCodeSource
    
     if (debug) then print, infolabel, 'xy,zlabels=', $
     	newTag_x, ', ', newTag_y, ', ', newTag_z
       if ( newTag_y EQ '') then begin
;;;     if (NOT debug) then  $
          self -> setFocus, xtag = [currentClass, newTag_x, newCodeSource], $
       		 ytag = ['', '', ''], $
		 ztag = [currentClass, newTag_z, zCodeSource]
	endif else begin
          self -> setFocus, xtag = [currentClass, newTag_x, newCodeSource], $
       		 ytag = [currentClass,  newTag_y, currentCode], $
		 ztag = [currentClass, newTag_z, zCodeSource]
	endelse
  endif
  ;----------------------------------------------------------------------
  return, success
  
end			; icp_data_construct

;====================================================================== 
;*
function func_icp_data_init,  $	; returns 0/1 for failure/success.
  self, $			; object of type specObj.
  thisFilename, $		; Required. Reads in NIST Triple axis data.
  onlyHeader = thisOnlyHeader, $ ; Read in only the header portion of the file.
  noConstruct = thisNoConstruct, $; Don't construct columns of data 
  				; from header info. The default is construct.
  noPrune = thisNoPrune, $ 	; Don't prune constant (or any!) columns of data 
  				; The default is to prune.
  showProgress = thisShowProgress
;\

;Nope  lun=thisLun, $		; returned from read().
;Nope  close = thisClose, $		; Passed to read()
;  tag_x = thisTag_x, $
;  tag_y = thisTag_y, $
;  tag_z = thisTag_z, $

  infolabel = "func_icp_data_init >> "  
  debug = 0
  debug2 = 0
  
  if (keyword_set(thisShowProgress)) then begin
    show_progress = 1
  endif else  begin 
    show_progress = 0
  endelse
;Debug  print, infolabel, "showProgress=", show_progress

;;---------------- Default plotting selections. --------------
;  if (n_elements(thisTag_x) gt 0) then self.tag_x = thisTag_x
;  if (n_elements(thisTag_y) gt 0) then self.tag_y = thisTag_y
;  if (n_elements(thisTag_z) gt 0) then self.tag_z = thisTag_z
  

  if (NOT obj_valid(self)) then begin
    self = obj_new('specObj', label=thisfilename, $
    	codeSource='func_icp_data_init')
  endif
  

;;------------- Initialize the header -----------------------
  if (show_progress) then begin
    print, infolabel, 'progress .... loading header ..........'
  endif

  localSkipInPlane = 0		; Preferred
;  localSkipInPlane = 1		; TEMPORARY  
  print, infolabel, "localSkipInPlane =", localSkipInPlane
  
;  lun = 10
  success =func_icp_header_init(self, $
		filename=thisFilename,$
;     		lun=Lun) 
     		lun=Lun, skipInPlane=localSkipInPlane) 
  if (debug2) then begin
  	print, infolabel, success, "=callFunction( 'func_icp_header_init'...), ", $
		"Lun = ", Lun
  endif
  
  if (not success) then begin
    print, infolabel, "Failed to read in the header for file", thisFilename
    goto, failed
  endif else begin
    if (debug) then self -> show, /recursive
  endelse

;;------------- Read the column labels and create info holders for column data  -----------------------
  if (show_progress) then begin
    print, infolabel, 'progress .... reading column labels ..........'
  endif
;Debug  print, infolabel, "n_elements(Lun) =", n_elements(Lun)
;Debug  print, infolabel, "thisLun =", Lun
  success = icp_data_read_labels( Lun, labels, ncols)	; Self not needed
  if (not success) then goto, failed
  
  OK = self -> getField(['*', 'scanPts', '*'], /inheader, qty = nrows)
  
  if (debug) then begin
    print, infolabel, 'labels=', labels
  endif
  
  if (show_progress) then begin
    print, infolabel, 'progress .... preparing columns ..........'
  endif

  currentClass = 'qtyObj'
  currentCode = 'func_icp_data_init'
  success = success and icp_data_prep_columns(self, currentClass, currentCode, $
  		labels, ncols, nrows, $
  	deltaEMissing = thisDeltaEMissing, $ 	; Boolean keyword. deltaE = E_in -E_out
  	QxyzMissing = thisQxyzMissing, $	; Boolean keyword. 3 columns of Q(x), Q(y), Q(z).
  	AnglesMissing = thisAnglesMissing)  	; Boolean keyword. Angle 3, 4 on triple axis.

  print
;debug  print, infolabel, 'ncols, nrows = ', ncols, nrows
  OK = self -> getField(['*', 'scanPts', '*'], /inHeader, qty=stored_nrows)
  tempObj = self -> get( /inData, count = stored_ncols)
;debug  print, infolabel, 'Stored ncols, nrows = ', stored_ncols, stored_nrows
  if (debug) then self -> show, /recursive
  
;;---------------------------------------------------------------
  if (keyword_set(thisOnlyHeader)) then begin
    close, Lun, exit_status=exit_status
    free_lun, Lun
    if (exit_status NE 0) then begin
      print, infolabel, 'CLOSE exit_status=', exit_status
    endif
    goto, skip_data
  endif
  
;;------------- Now read in the columns of data  -----------------------
  if (show_progress) then begin
    print, infolabel, 'progress .... reading columns of data ..........'
  endif
  col_success =  icp_data_read_columns(self, Lun)
  if (not col_success) then goto, failed
 
     if (not col_success) then begin
    	print,  infolabel, 'Failure to read columns of data '
    endif
    success = success and col_success

;;------------- Now put in errors for the 'COUNTS' column  -----------------------
  if (show_progress) then begin
    print, infolabel, 'progress ....making error for COUNTS column ..........'
  endif
  ecol_success =  icp_data_makeErrors_columns(self)
  if (not ecol_success) then begin
    	print,  infolabel, 'Failure to create errors for columns of data '
  endif
  success = success and ecol_success

;;------------- Assign default plotting/DAVE choices  -----------------------
;  self.tag_z = 'COUNTS'

;;------------------------------------------------------------------
   skip_data:
   
;;------------------------------------------------------------------
   if (not keyword_set(thisNoConstruct)) then begin
     if (show_progress) then begin
       print, infolabel, 'progress .... constructing extra columns of info ..........'
     endif
;     success = success and icp_data_construct(self, /all, /varying, /selectTags)
    success = success and icp_data_construct(self, $
	  deltaE = thisDeltaEMissing, $ 	; Boolean keyword. deltaE = E_in -E_out
	  Qxyz = thisQxyzMissing, $	; Boolean keyword. 3 columns of Q(x), Q(y), Q(z).
	  Angles = thisAnglesMissing, $	; Boolean keyword. Angle 3, 4 on triple axis.
  	  /temp, /avTemp, /Ein, /Eout, /q_mag, /inPlane, /index, $
	  /varying, $	; Construct only those above
  			; which are varying. Use to modify the other keywords.
     	/selectTags)
   endif
;;------------------------------------------------------------------
;   if (not keyword_set(thisNoPrune)) then begin
;     ; Prune all columns of data which are constant.
;     if (show_progress) then begin
;       print, infolabel, 'progress .... pruning unneeded columns ..........'
;     endif
;     success = success and icp_data_prune_columns(self, /except, /constant)	
;   endif
   
;;-------------- return -----------------------------------------------------
  if (show_progress) then begin
    print, infolabel, 'progress .... finished reading in file ..........'
  endif
   return, success 
   
;;;  if (arg_present(thisLun)) then thisLun = lun
  
;;------------------------------------
 failed:
  if (n_elements(lun) GT 0) then begin
    close, Lun, exit_status=exit_status
    free_lun, Lun
    if (exit_status NE 0) then begin
      print, infolabel, 'CLOSE exit_status=', exit_status
    endif
  endif
  
  return, 0

end		; func_icp_data_init


