; $Id: func_icp_header_init.pro,v 1.4 2003/09/12 15:25:25 tanya Exp $
;+
; NAME:
;       func_icp_header_init
;
; PURPOSE:
;
; 	This is a fake method for the object self, of type specObj, 
;	which is the first parameter to func_icp_header_init
; 	Because it is not a true method, any operations performed upon 
; 	self must be done via's its methods within this code.
;
;	This code reads in the ICP header for triple axis data into
;	the object self. This code is called by func_icp_data_init.
;	In the future, it may also be called by itself, to initialized 
;	a specObj with only the header infomation, as a planning tool.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;       <LI><A HREF="#SPECOBJ::INIT,_arguments">SPECOBJ::INIT,_arguments</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Spring, 2003.
;
;-
; func_icp_header_init.pro
; These are fake methods for the object self.
; Becuase they are not true methods, any operations performed upon 
; self must be done via's its methods.
;
; For example
; In this file: fake methods in traditional IDL code:
;   OK = icp_header_read_q_line10(self, lun)
;   OK = self -> set(tag_x = thisTag_x)
;
; Not in this file: Real methods in object-oriented IDL code
;   OK = self -> icp_header::read_q_line10( lun)
;   self.tag_x = thisTag_x
;   

;================================================================================
function ICP_header_read_success, $
 	ErrMessage, $	; Recommended output. Returns an error message. errMessage
	Prompt, $	; Optional input. Strign at begining of error message
;;;	wherePtr, $	; Place in object structure
	OKarr	; Input required. Array of booleans, each containing the 
;	OK	; Input required. Array of booleans, each containing the 
			; status of each item in line
; returns overall Success status
; Don't make this a method unless it can access self to get the name of the
; item which failed. Should be able to do it using self.(istart+i), if we can
; get the number of the starting point.


;;;  common cb_ICP_header, prototypeRec, protoType_tags, protoType_ntags		
  infolabel = "ICP_header_read_success >> "  

;  message_anyway = 1		For debugging
  message_anyway = 0
    
  temp = size(OKarr)
  ; array temp has [0] = number of dimensions. The next elements contain the
  ; size of each dimension, one element per dimension.
 ndim =temp[0]

 if (ndim eq 0) then begin
 
  if (message_anyway) then begin
  ;For debugging:
    help, okArr
    print, ndim
    print, infolabel, okArr
  endif
  
  success = okArr

  if (arg_present(errMessage)) then begin
    if (success and (not message_anyway)) then begin
      errMessage = ''
    endif else begin
      if (arg_present(prompt)) then begin
	errMessage = prompt
      endif else begin
	errMessage = ''
      endelse

	      if (OKarr GE 0) then begin
	        errMessage = errMessage $
		  +strcompress(string(fix(okarr[i1])),/remove_all) 
	      endif  else begin
	        errMessage = errMessage + '_'
	      endelse 

    endelse
  endif  
 endif
 
 ;------------------------------------------------------------------
 if (ndim eq 1) then begin
  dim1 = temp[1]
  if (message_anyway) then begin
   ;For debugging:
    help, okArr
    print, ndim
    print, dim1
    print, infolabel, okArr[0:*]
  endif
  
  success = 1
  for i1 = 0, (dim1-1) do begin 
      if (OKarr[i1] NE -1) then begin
        success = success and okarr[i1]
      endif      
  endfor

  if (arg_present(errMessage)) then begin
    if (success and (not message_anyway)) then begin
      errMessage = ''
    endif else begin
      if (arg_present(prompt)) then begin
	errMessage = prompt
      endif else begin
	errMessage = ''
      endelse

	  for i1 = 0, (dim1-1) do begin 
	      if (OKarr[i1] GE 0) then begin
	        errMessage = errMessage $
		  +strcompress(string(fix(okarr[i1])),/remove_all) + ' '
	      endif  else begin
	        errMessage = errMessage + '_'
	      endelse 
	  endfor

    endelse
  endif  
 endif
   
 ;------------------------------------------------------------------
 if (ndim eq 2) then begin
 dim1 = temp[1]
 dim2 = temp[2]
 
   if (message_anyway) then begin
    ;For debugging:
    help, okArr
    print, ndim
    print, dim1
    print, dim2
    for i1 = 0, (dim1-1) do begin 
      print, infolabel, transpose(okArr[i1, 0:*])
    endfor
   endif
  
  success = 1
  for i1 = 0, (dim1-1) do begin 
    for i2 = 0, (dim2-1) do begin 
      if (OKarr[i1, i2] NE -1) then begin
        success = success and okarr[i1, i2]
      endif      
    endfor
  endfor

  if (arg_present(errMessage)) then begin
    if (success and (not message_anyway)) then begin
      errMessage = ''
    endif else begin
      if (arg_present(prompt)) then begin
	errMessage = prompt
      endif else begin
	errMessage = ''
      endelse

	  for i1 = 0, (dim1-1) do begin 
	    for i2 = 0, (dim2-1) do begin 
	      if (OKarr[i1,i2] GE 0) then begin
	        errMessage = errMessage $
		  +strcompress(string(fix(okarr[i1, i2])),/remove_all) ;+' '
	      endif  else begin
	        errMessage = errMessage + '_'
	      endelse 
	      if (i2 eq (dim2-1)) then begin
	        errMessage = errMessage + ', '
	      endif else begin
	        errMessage = errMessage + ' '
	      endelse
	    endfor
	  endfor
    endelse
  endif  
 endif
 
 ;------------------------------------------------------------------
;Debug  
 if (message_anyway) then begin
   print, infolabel, 'success = ', success	;temp
   print,  errMessage
 endif

  return, success
end

;================================================================================
function ICP_header_read_q_line1, $
	self, currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 1 of ICP files from unit lun.

  infolabel = "ICP_header_read_q_line1 >> " 
  debug = 0 
  anErrMsg = ''
  
  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 8
  ok_wid = 1
  OKarr = make_array(ok_len, /int, value=-3)
;Silly  OKarr = make_array(ok_len, ok_wid, /int, value=0)
;03Apr  OKarr = make_array(ok_len, ok_wid, /int, value=-3)
;Debug  print, infolabel, "OKarr = make_array(ok_len, ok_wid, /int, value=-3)"

  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='filename'...)"
;  OKarr[0,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'filename', currentCode], $
  	contents=obj_new('qtyObj', name='filename', qty=readword(aline, 0))

;  outfilename = 'DAVE_' + readword(aline, 0)
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'filename', currentCode], $
;  	contents=obj_new('qtyObj', name='outfilename', qty=outfilename)

  if (anErrMsg EQ '') then OKarr[0] = 1
  
  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='Date'...)"
;  OKarr[1,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
  	tags = [currentClass, 'Date', currentCode], $
  	contents=obj_new('qtyObj', name='Date', qty=readword(aline, 1)) 
  if (anErrMsg EQ '') then OKarr[1] = 1
	
  if (debug) then begin
  	print, infolabel, " Calling OK=self->add, label='ScanStyle'...)" 
	print, "qty = readword(aline, 2) =", readword(aline, 2)
  endif	
;  OKarr[2,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'ScanStyle', currentCode], $
  	contents=obj_new('qtyObj', name='ScanStyle', qty=readword(aline, 2)) 
  if (anErrMsg EQ '') then OKarr[2] = 1
	
  if (debug) then begin
  	print, infolabel, " Calling OK=self->add, label='monPreset'...)"
	print, "qty = readword(aline, 2) =", readword(aline,3)
  endif
;  OKarr[3,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'monPreset', currentCode], $
  	contents=obj_new('qtyObj', name='monPreset', $
	 qty=float(readword(aline, 3)), units='Events') 
  if (anErrMsg EQ '') then OKarr[3] = 1
	
  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='monPrefactor'...)"
;  OKarr[4,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'monPrefactor', currentCode], $
  	contents=obj_new('qtyObj', name='monPrefactor', $
		qty=float(readword(aline, 4))) 

  monitorVal = float(readword(aline, 3)) * float(readword(aline, 4))
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'monitor', currentCode], $
  	contents=obj_new('qtyObj', name='monitor', $
		qty=monitorVal,  units='Events')

  if (anErrMsg EQ '') then OKarr[4] = 1
	
  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='base'...)"
;  OKarr[5,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'base', currentCode], $
  	contents=obj_new('qtyObj', name='base', qty=readword(aline, 5)) 
  if (anErrMsg EQ '') then OKarr[5] = 1
	
  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='scanPts'...)"
;  OKarr[6,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'scanPts', currentCode], $
  	contents=obj_new('qtyObj', name='scanPts', qty=float(readword(aline, 6))) 
  if (anErrMsg EQ '') then OKarr[6] = 1
	
  if (debug) then $
  	print, infolabel, " Calling OK=self->add, label='type'...)"
;  OKarr[7,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'type', currentCode], $
  	contents=obj_new('qtyObj', name='type', qty=readword(aline, 7)) 
  if (anErrMsg EQ '') then OKarr[7] = 1

  thisFilename = readword(aline, 0)
  aFlip = 'none'
  valFlip = complex(0.0, 0.0)
  anInstr = '??#'
  if (debug) then print, infolabel, "pre-Construct flipper code"
  if (n_elements(thisFileName) GT 0) then begin	
   ihere = strpos(thisFilename, '.', /reverse_search)
   if (ihere gt 0) then begin
     nlen = n_elements(thisFilename)
     anInstr = strmid(thisFilename,(ihere+1),3)	; get last 3 characters
     aFlip = strmid(anInstr, 1,1) 
     if (not ((aFlip EQ 'a') or (aFlip EQ 'b') or $
     	      (aFlip EQ 'c') or (aFlip EQ 'd'))) then begin
	 aFlip = 'none'
     endif else begin
       if (aFlip EQ 'a') then  valFlip = complex(0.0, 0.0)
       if (aFlip EQ 'b') then  valFlip = complex(1.0, 0.0)
       if (aFlip EQ 'c') then  valFlip = complex(0.0, 1.0)
       if (aFlip EQ 'd') then  valFlip = complex(1.0, 1.0)
     endelse    
    endif 
  endif
  
  if (debug) then print, infolabel, "post-Construct flipper code"
  ; If want to specify the equipment as well as the beamline, use an array
;   OK_instr = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'instrument', currentCode], $
  	contents=obj_new('qtyObj', name='instrument', qty=anInstr) 
  OK_instr = (anErrMsg EQ '') 


  ; Flipper code is a complex number. 
  ; Real part is the incident polarization provided by the polarizer (range -1.0 to 1.0)
  ; Imaginary part is the scattered polarization provided by the polarizer (range -1.0 to 1.0)
  ; the legend = 'a', 'b', 'c', 'd' for the standard combos.
  ; the legend = 'none' when not used.
;;  OK_flipper = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'flipper_code', currentCode], $
  	contents=obj_new('qtyObj', name='flipper_code', qty=valFlip, $
	  	legend = aFlip,  units='Fraction') 
  OK_flipper = (anErrMsg EQ '') 
  
  for i = 0, (ok_len -1) do begin ; debug
    if (not (OKarr[i,0])) then begin
      print, infolabel, i, ' ', readword(aline, i), ' OK=', okarr[i,0:*]
    endif
  endfor
  
  if (not OK_instr) then begin
    print, infolabel,   $
  	"OKarr_instr = self->add, label='instrument', ...)=", OK_instr
  endif
  if (not OK_flipper) then begin
    print, infolabel,   $
  	"self->add, label='flipper_code', ...)=", OK_flipper
   endif

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 
  if (not success) then print, errMessage
 
  if (debug) then print, infolabel, " End of routine"
 
  return, success
end

;================================================================================
function ICP_header_read_q_line3, $
	self, currentClass, currentCode,$
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 3 of ICP files from unit lun.

  infolabel = "ICP_header_read_q_line3 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 1
  ok_wid = 1
  OKarr = make_array(ok_len, ok_wid, /int, value=-3)

;;;  OKarr[0,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'title', currentCode], $
  	contents=obj_new('qtyObj', name='title', qty=aline) 
  OKarr[0,0] = (anErrMsg EQ '') 

  for i = 0, (ok_len - 1) do begin ; debug
    if (not (OKarr[i,0])) then begin
      print, infolabel, i, ' ', readword(aline, i), ' OK=', okarr[i,0]
    endif
  endfor
  
  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 
  if (not success) then print, errMessage
  
  return, success
end

;================================================================================
function ICP_header_read_q_line4, $
	self, currentClass, currentCode,$
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 4 of ICP files from unit lun.

  infolabel = "ICP_header_read_q_line4 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 5	
  OKarr = make_array(ok_len, /int, value=-3)

  i = 0
  temp_array = fltarr(4)
  for j = 0, 3 do begin		; length 4
    temp_array[j]=readword(aline, (i+j))
  endfor
;  OKarr[0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'collimation', currentCode], $
  	contents=obj_new('qtyObj', name='collimation', qty=temp_array)
  OKarr[0] = (anErrMsg EQ '') 

  i = i + n_elements(temp_array) 		

  ; Mosaic is unused and has a NULL object associated with it.
;Skip  for  j= 0, (n_elements(self.mosaic)-1) do begin		; length 3
;Skip    OKarr[1,j] = self->add, 'mosaic', $
;skip 		qty=obj_new('qtyObj', name='instrument', $
;skip		qty=readword(aline,(i+j)), specify_index=j)  )
;Skip  endfor
  i = i + 3 		; n_elements(self.mosaic) 	; UNUSED. skip it.

  temp_array = fltarr(3)
  for j = 0, 2 do begin		; length 3
    temp_array[j]=readword(aline, (i+j))
  endfor
;  OKarr[2] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'scatPlane_u', currentCode], $
  	contents=obj_new('qtyObj', name='scatPlane_u', $
		legend=['h', 'k', 'l'], units='r.l.u.?', qty=temp_array)  
  OKarr[2] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array)

  ; scatAngle is unused and has a NULL object associated with it.
;Skip  OKarr[3] = self->add, 'scatAngle', qty=readword(aline, (i)))  
  i = i + 1		; n_elements(self.scatAngle) 	; UNUSED. skip it.

 
  for j = 0, 2 do begin		; length 3
    temp_array[j]=readword(aline, (i+j))
  endfor
;  OKarr[4] =
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'scatPlane_v', currentCode], $
  	contents=obj_new('qtyObj', name='scatPlane_v', $
		legend=['h', 'k', 'l'], units='r.l.u.?', qty=temp_array)
  OKarr[4] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array)	
    
  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end


;================================================================================
function ICP_header_read_q_line6, $
	self, currentClass, currentCode,$
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 6 of ICP files from unit lun.

  infolabel = "ICP_header_read_q_line6 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 2	
  OKarr = make_array(ok_len, /int, value=-3)

  i = 0
  temp_array = fltarr(3)
  for j = 0, 2 do begin		; length 3
    temp_array[j]=readword(aline, (i+j))
  endfor
;  OKarr[0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'latDim', currentCode], $
  	contents=obj_new('qtyObj', name='latDim', legend = ['a','b','c'],$
	 units= '!dave_angstromSym', qty=temp_array) 
  OKarr[0] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array)	

  for j = 0, 2 do begin		; length 3
    temp_array[j]=readword(aline, (i+j))
  endfor
;;  OKarr[1] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'latAngle', currentCode], $
  	contents=obj_new('qtyObj', name='latAngle', $
	legend = ['alpha', 'beta', 'gamma'],  units= 'Degrees', $
		 qty=temp_array)
  OKarr[1] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array)	

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end

;================================================================================
function ICP_header_read_q_line8, $
	self, currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 8 of 'Q' ICP files from unit lun.

  infolabel = "ICP_header_read_q_line8 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 7	
  OKarr = make_array(ok_len, /int, value=-3)

;  OKarr[0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'e_scan', currentCode], $
;    	tags = [currentClass, 'e_center', currentCode], $
  	contents=obj_new('qtyObj', name='e_scan', units='meV', $
		qty=float(readword(aline, 0)), $
		err=float(readword(aline, 1)), styleErr='center_step')
  OKarr[0] = (anErrMsg EQ '') 
	
;;  OKarr[1] =
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'e_step', currentCode], $
;  	contents=obj_new('qtyObj', name='e_step', units='meV', $
;		qty=float(readword(aline, 1))) 
;  OKarr[1] = (anErrMsg EQ '') 
	
;  OKarr[2] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'e_fixed', currentCode], $
  	contents=obj_new('qtyObj', name='e_fixed', units='meV', $
		qty=float(readword(aline, 2))) 
  OKarr[2] = (anErrMsg EQ '') 
		
	
;  OKarr[3] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'dspacing_mon', currentCode], $
  	contents=obj_new('qtyObj', name='dspacing_mon',$
	 units='!dave_angstromSym', $
;Aug 18,2003	 units='!dave_invangstromSym', $
		qty=float(readword(aline, 3)))
  OKarr[3] = (anErrMsg EQ '') 

	
;  OKarr[4] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'dspacing_ana', currentCode], $
  	contents=obj_new('qtyObj', name='dspacing_ana', $
	units='!dave_angstromSym', $
;Aug 18,2003	units='!dave_invangstromSym', $
		qty=float(readword(aline, 4)))
  OKarr[4] = (anErrMsg EQ '') 
	
;  OKarr[5] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'temp_scan', currentCode], $
;    	tags = [currentClass, 'temp_start', currentCode], $
;  	contents=obj_new('qtyObj', name='temp_start', units='K', $
  	contents=obj_new('qtyObj', name='temp_scan', units='K', $
		qty=float(readword(aline, 5)), $
		err=float(readword(aline, 6)), styleErr='start_step')
  OKarr[5] = (anErrMsg EQ '') 
  
;;  OKarr[6] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'temp_step', currentCode], $
;  	contents=obj_new('qtyObj', name='temp_step', units='K', $
;		qty=float(readword(aline, 6))) )
;  OKarr[6] = (anErrMsg EQ '') 

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end

;================================================================================
function ICP_header_read_q_line9, $
	self, currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 9 of 'Q' ICP files from unit lun.

  infolabel = "ICP_header_read_q_line9 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 1	
  OKarr = make_array(ok_len, /int, value=-3)

;  OKarr[0,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'which_fixed', currentCode], $
  	contents=obj_new('qtyObj', name='which_fixed', qty=readword(aline, 4)) 
  OKarr[0] = (anErrMsg EQ '') 

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end

;================================================================================
function ICP_header_read_q_line10, $
	self, currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 10 of ICP files from unit lun.

  infolabel = "ICP_header_read_q_line10 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 3	
  OKarr = make_array(ok_len, /int, value=-3)

  i = 0

;  temp_array = fltarr(3)
;  for j = 0, 2 do begin		; length 3
;    temp_array[j]=readword(aline, (i+j))
;  endfor
;;  OKarr[0] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'q_center', currentCode], $
;  	contents=obj_new('qtyObj', $
;		name='q_center', units='r.l.u.', qty=temp_array) 
;  OKarr[0] = (anErrMsg EQ '') 
;  i = i + n_elements(temp_array) 		
;
;  for j = 0, 2 do begin		; length 3
;    temp_array[j]=readword(aline, (i+j))
;  endfor
;;  OKarr[1] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'q_step', currentCode], $
;  	contents=obj_new('qtyObj', $
;		 name='q_step', units='r.l.u.', qty=temp_array) 
;  OKarr[1] = (anErrMsg EQ '') 
;  i = i + n_elements(temp_array) 		

  temp_array = fltarr(3)
  etemp_array = fltarr(3)
  for j = 0, 2 do begin		; length 3
    temp_array[j]=readword(aline, (i+j))
  endfor

  i = i + n_elements(temp_array) 		
  for j = 0, 2 do begin		; length 3
    etemp_array[j]=readword(aline, (i+j))
  endfor

  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'q_scan', currentCode], $
  	contents=obj_new('qtyObj', $
		name='q_scan', units='r.l.u.', $
;		name='q_center', units='r.l.u.', $
		qty=temp_array, $
		err=etemp_array, styleErr = 'center_step') 
  OKarr[0] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array) 		

;  OKarr[2,0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'h_field', currentCode], $
  	contents=obj_new('qtyObj', name='Magnetic_Field', units='G?',$
		qty=float(readword(aline, (i+0)))) 
  OKarr[2] = (anErrMsg EQ '') 

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end


;================================================================================
function ICP_header_read_b_line12_17, $
	self,  currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in lines 12-17 of 'B' ICP files from unit lun.

  infolabel = "ICP_header_read_b_line12_17 >> "  

  num_mot = 6
  ok_len = 3
  OKarr = make_array(ok_len,  /int, value=-3)

  anErrMsg = ''
  aline = ''
  temp_start = fltarr(num_mot)
  temp_step = fltarr(num_mot)
  temp_stop = fltarr(num_mot)
  for j = 0, (num_mot-1) do begin
    readf, lun, aline
    temp_start[j] = readword(aline, 1)
    temp_step[j] = readword(aline, 2)
    temp_stop[j] = readword(aline, 3)
  endfor
;  OKarr[0] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'mot_angle_start', currentCode], $
;  	contents=obj_new('qtyObj', name='mot_angle_start', units='Degrees?',$ 
;		qty= temp_start ) 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'mot_angle', currentCode], $
  	contents=obj_new('qtyObj', name='mot_angle_start', units='Degrees?',$ 
		qty= temp_start, err=temp_step, styleErr='start_step' ) 
  OKarr[0] = (anErrMsg EQ '') 

;;  OKarr[1] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'mot_angle_step', currentCode], $
;  	contents=obj_new('qtyObj', name='mot_angle_step', units='Degrees?',$
;		qty= temp_step ) 
;  OKarr[1] = (anErrMsg EQ '') 

;  OKarr[2] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'mot_angle_stop', currentCode], $
  	contents=obj_new('qtyObj', name='mot_angle_stop', units='Degrees',$
		qty= temp_stop ) 
  OKarr[2] = (anErrMsg EQ '') 
    
  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
    
  return, (success*[1,1,1,1,1,1])

end

;================================================================================
function ICP_header_read_i_line4, $
	self,  currentClass, currentCode, $
	lun, $		; required parameter. Unit number of file to read
	errMessage	; Optional. Returned error message

; reads in line 4 of 'I' ICP files from unit lun.

  infolabel = "ICP_header_read_i_line4 >> "  

  anErrMsg = ''
  aline = ''
  readf, lun, aline

  ok_len = 7	
  OKarr = make_array(ok_len, /int, value=-3)

  i = 0
  temp_array = fltarr(4)
  for j = 0, 3 do begin		; length 4
    temp_array[j]=readword(aline, (i+j))
  endfor
;  OKarr[0] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'collimation', currentCode], $
  	contents=obj_new('qtyObj', name='collimation', qty=temp_array) 
  OKarr[0] = (anErrMsg EQ '') 
  i = i + n_elements(temp_array) 		

  ; Mosaic is unused and has a NULL object associated with it.
;Skip  for  j= 0, (n_elements(self.mosaic)-1) do begin		; length 3
;Skip    OKarr[1,j] = self->add, 'mosaic', qty=readword(aline, (i+j)), specify_index=j)  
;Skip  endfor
  i = i + 3 		; n_elements(self.mosaic) 	; UNUSED. skip it.

;  OKarr[2] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'wavelen', currentCode], $
  	contents=obj_new('qtyObj', $
		name='wavelen',  units='!dave_angstromSym', $
		qty=readword(aline, i))  
  OKarr[2] = (anErrMsg EQ '') 
  i = i + 1

;  OKarr[3] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'temp_scan', currentCode], $
  	contents=obj_new('qtyObj', $
		name='temp_scan',  units='K', $
;		name='temp_start',  units='K', $
		qty=readword(aline, i), $
		styleErr='start_step')  
  OKarr[3] = (anErrMsg EQ '') 
  i = i + 1

;;  OKarr[4] = 
;  self->add,  /inHeader, errMsg = anErrMsg,  $
;    	tags = [currentClass, 'temp_step', currentCode], $
;  	contents=obj_new('qtyObj', name='temp_step',  units='K', $
;		qty=readword(aline, i), $
;		styleErr='start_step') 
;  OKarr[4] = (anErrMsg EQ '') 
  i = i + 1

;  OKarr[5] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'h_field', currentCode], $
  	contents=obj_new('qtyObj', name='h_field',  units='G?', $
		qty=readword(aline, i)) 
  OKarr[5] = (anErrMsg EQ '') 
  i = i + 1

;  OKarr[6] = 
  self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = [currentClass, 'num_det', currentCode], $
  	contents=obj_new('qtyObj', name='num_det',  $
		qty=readword(aline, i)) 
  OKarr[6] = (anErrMsg EQ '') 

  prompt = infolabel + ' Success reading items (0=F,1=T): '
  success = ICP_header_read_success(ErrMessage, prompt, OKarr) 

  if (not success) then print, errMessage
  
  return, success
end

;================================================================================
function ICP_header_read_q, $
  self, currentClass, currentCode, $
  newFile, $		; Filename
  lun=Lun, $
  close = thisClose
; reads in 'Q' scan styled files.

  infolabel = "ICP_header_Read_q >> "  
  debug = 0
  
  success = 1

  anErrMsg = ''
  aline = ''
  suc_len = 10 
  sucarr= make_array(suc_len, /int, value = -1)	; true but not used yet.
  
  if (debug) then print, infolabel, "calling icp_header_read_q_line1"
  sucarr[1-1] = icp_header_read_q_line1(self, currentClass, currentCode, lun)
  				;======	line  1 of ICP header

  readf, lun, aline		;====== line  2 of ICP header. Skip.   

  if (debug) then print, infolabel, "calling icp_header_read_q_line3"
  sucarr[3-1] = icp_header_read_q_line3(self, currentClass, currentCode, lun)
  				;======	line  3 of ICP header

  if (debug) then print, infolabel, "calling icp_header_read_q_line4"
  sucarr[4-1] = icp_header_read_q_line4(self, currentClass, currentCode, lun)
  				;======	line  4 of ICP header

  readf, lun, aline		;====== line  5 of ICP header. Skip.   

  if (debug) then print, infolabel, "calling icp_header_read_q_line6"
  sucarr[6-1] = icp_header_read_q_line6(self, currentClass, currentCode, lun)
  				;======	line  6 of ICP header

  readf, lun, aline		;====== line  7 of ICP header. Skip. 
    
  if (debug) then print, infolabel, "calling icp_header_read_q_line8"
  sucarr[8-1] = icp_header_read_q_line8(self, currentClass, currentCode, lun)
  				;======	line  8 of ICP header

  if (debug) then print, infolabel, "calling icp_header_read_q_line9"
  sucarr[9-1] = icp_header_read_q_line9(self, currentClass, currentCode, lun)
  				;======	line  9 of ICP header

  if (debug) then print, infolabel, "calling icp_header_read_q_line10"
  sucarr[10-1] = icp_header_read_q_line10(self, currentClass, currentCode, lun)
  			        ;====== line  10 of ICP header
  readf, lun, aline		;====== line  11 of ICP header. Skip.   

;----------------------------------------------------------
  success = 1
  errStr = 'Read errors on line(s) '
  for i = 0, (suc_len - 1) do begin
    success = success and sucarr[i]
    if (not sucarr[i]) then errStr = errStr + $
    	strcompress(string(i+1), /remove_all) + ' '
  endfor
  if (not success) then begin
;    close, lun, exit_status=exit_status
;    free_lun, unit
;    if (exit_status NE 0) then begin
;      print, infolabel, 'CLOSE exit_status=', exit_status
;    endif
;    lun = 0
    print, infoLabel, errStr
  endif
  

  return, success

end

;================================================================================
function ICP_header_read_b, $
  self,  currentClass, currentCode, $
  newFile, $		; Filename
  lun=Lun, $
  close = thisClose
; reads in 'B' scan styled files. Bragg buffer.
; Close unit lun upon read error  or keyword close.

  infolabel = "ICP_header_Read_B >> "  

;  openr, lun, newFile, /get_lun, error = errCode
;  IF (errCode NE 0) then begin
;    ; the standard error file (logical unit -2):
;    PRINT,  !ERROR_STATE.MSG
;    ;; PRINTF, -2, !ERROR_STATE.MSG
;    if (arg_present(thisLun)) then thisLun = 0
;    return, 0
;  endif

  success = 1

  anErrMsg = ''
  aline = ''
  suc_len = 17 
  sucarr= make_array(suc_len, /int, value = -1)	; true but not used yet.
  
  sucarr[1-1] = icp_header_read_q_line1(self, currentClass, currentCode, lun)
  			;======	line  1 of ICP header
  readf, lun, aline	;====== line  2 of ICP header. Skip.   
  sucarr[3-1] = icp_header_read_q_line3(self, currentClass, currentCode, lun)
  			;======	line  3 of ICP header
  sucarr[4-1] = icp_header_read_q_line4(self, currentClass, currentCode, lun)
  			;======	line  4 of ICP header
  readf, lun, aline	;====== line  5 of ICP header. Skip.   
  sucarr[6-1] = icp_header_read_q_line6(self, currentClass, currentCode, lun)
  			;======	line  6 of ICP header
  readf, lun, aline	;====== line  7 of ICP header. Skip. 
    
  sucarr[8-1] = icp_header_read_q_line8(self, currentClass, currentCode, lun)
  			;======	line  8 of ICP header
  sucarr[9-1] = icp_header_read_q_line9(self, currentClass, currentCode, lun)
  			;======	line  9 of ICP header
  sucarr[10-1] = icp_header_read_q_line10(self, currentClass, currentCode, lun)
  		        ;====== line  10 of ICP header
  readf, lun, aline	;====== line  11 of ICP header. Skip.   

  sucarr[11:16] = icp_header_read_b_line12_17(self, $
  	currentClass, currentCode, lun)
  			;======	lines 12-17 of ICP header
  readf, lun, aline	;====== line  18 of ICP header. Skip.   

;----------------------------------------------------------
  success = 1
  errStr = 'Read errors on line(s) '
  for i = 0, (suc_len - 1) do begin
    success = success and sucarr[i]
    if (not sucarr[i]) then errStr = errStr + $
    	strcompress(string(i+1), /remove_all) + ' '
  endfor

  if (not success) then begin
;    close, lun, exit_status=exit_status
;    free_lun, lun
;    if (exit_status NE 0) then begin
;      print, infolabel, 'CLOSE exit_status=', exit_status
;    endif
;    lun = 0
    print, infoLabel, errStr
  endif
  


;----------------------------------------------------------
;  if (keyword_set(thisClose)) then begin
;    close, lun, exit_status=exit_status
;    free_lun, lun
;    if (exit_status NE 0) then begin
;      print, infolabel, 'CLOSE exit_status=', exit_status
;    endif
;    lun = 0
;  endif
;  if (arg_present(thisLun)) then thisLun = Lun
;----------------------------------------------------------

  return, success

end

;================================================================================
function ICP_header_read_i, $
  self,  currentClass, currentCode, $
  newFile, $		; Filename
  lun=Lun, $
  close = thisClose
; reads in 'I' scan styled files. Generic increment buffer.
; Close unit lun upon read error or keyword close.

;;;  common cb_ICP_header, prototypeRec, protoType_tags, protoType_ntags		

  infolabel = "ICP_header_Read_I >> "  

;  openr, lun, newFile, /get_lun, error = errCode
;  IF (errCode NE 0) then begin
;    ; the standard error file (logical unit -2):
;    PRINT,  !ERROR_STATE.MSG
;    ;; PRINTF, -2, !ERROR_STATE.MSG
;    if (arg_present(thisLun)) then thisLun = 0
;    return, 0
;  endif

  success = 1

  anErrMsg = ''
  aline = ''
  suc_len = 11
  sucarr= make_array(suc_len, /int, value = -1)	; true but not used yet.
  
  sucarr[1-1] = icp_header_read_q_line1(self,  currentClass, currentCode, lun)
  				;======	line  1 of ICP header
  readf, lun, aline		;====== line  2 of ICP header. Skip.   
  sucarr[3-1] = icp_header_read_q_line3(self,  currentClass, currentCode, lun)
  				;======	line  3 of ICP header
  sucarr[4-1] = icp_header_read_i_line4(self,  currentClass, currentCode, lun)
  				;======	line  4 of ICP header
  readf, lun, aline		;====== line  5 of ICP header. Skip.   
  sucarr[5:10] = icp_header_read_b_line12_17(self,  $
  	currentClass, currentCode, lun)
  				;======	lines 6-11 of 'I' ICP header
  readf, lun, aline		;====== line  12 of ICP header. Skip.   

;----------------------------------------------------------
  success = 1
  errStr = 'Read errors on line(s) '
  for i = 0, (suc_len - 1) do begin
    success = success and sucarr[i]
    if (not sucarr[i]) then errStr = errStr + $
    	strcompress(string(i+1), /remove_all) + ' '
  endfor

  if (not success) then begin
;    close, lun, exit_status=exit_status
;    free_lun, lun
;    if (exit_status NE 0) then begin
;      print, infolabel, 'CLOSE exit_status=', exit_status
;    endif
;    lun = 0
    print, infoLabel, errStr
  endif
  

;----------------------------------------------------------
;  if (keyword_set(thisClose)) then begin
;    close, lun, exit_status=exit_status
;    free_lun, lun
;    if (exit_status NE 0) then begin
;      print, infolabel, 'CLOSE exit_status=', exit_status
;    endif
;    lun = 0
;  endif
;  if (arg_present(thisLun)) then thisLun = lun

;----------------------------------------------------------
  return, success

end


;================================================================================
function ICP_header_read, $
  self, $
  newFile, $		; Filename
  lun=thisLun, $
  close = thisClose
; reads in 'Q' or 'B' or 'I'  scan styled files. 
; Close unit thisLun upon read error or keyword close.

  infolabel = "ICP_header_Read >> "  
  debug = 0

   style = icp_style(newfile)

 if (debug) then begin
   print, infolabel, "Entered"
    if (n_elements(thisfilename) gt 0) then  $
    	print, '	', infolabel, "thisFileName=", thisfilename
    if (n_elements(thislun) gt 0) then  $
    	print, '	', infolabel, "thislun=", thislun
    if (n_elements(thisClose) gt 0) then $
    	print, '	', infolabel, "thisClose = ",thisClose
    if (arg_present(thisLun)) then begin
      print, '	', infolabel, "Modifiable thisLun"
    endif else begin
      print, '	', infolabel, "Can not modify thisLun"
    endelse
 endif 

  openr, Lun, newFile, /get_lun, error = errCode
  IF (errCode NE 0) then begin
    ; the standard error file (logical unit -2):
    PRINT,  !ERROR_STATE.MSG
    ;; PRINTF, -2, !ERROR_STATE.MSG
    if (arg_present(thisLun)) then thisLun = 0
    return, 0
  endif


  OK = 0
  currentClass = 'qtyObj'
  Case (style) of
   'Q': OK = icp_header_read_q(self, currentClass, 'ICP_readQ', $
   		newFile, lun=Lun, close=thisClose)
   'B': OK = icp_header_read_b(self, currentClass, 'ICP_readB', $ 
   		newFile, lun=Lun, close=thisClose)
   'I': OK = icp_header_read_i(self, currentClass, 'ICP_readI', $
   		 newFile, lun=Lun, close=thisClose)
   else: begin
   	end
    endcase

;----------------------------------------------------------
  if (not OK) then begin
;    print, infolabel, OK, " Failed icp_header_read_", style
     
    ok_dialog = Dialog_Message(string('Triple axis ICP file read error. Style=',$
	style), /info, title="Triple axis ICP file read error.")
    close, lun, exit_status=exit_status
    free_lun, lun
    if (exit_status NE 0) then begin
      print, infolabel, 'CLOSE exit_status=', exit_status
    endif
    lun = 0
    return, 0	; failure
  endif

;------ debugging ------------------------------------------------------
;  nHeader = self -> count(/inHeader)
;  nData = self -> count(/inData)
;  message, /info, string("Header =", nHeader, " ndata=", nData)
;  self -> show, /recursive
  
;----------------------------------------------------------------------------
  OK = self -> getField(['*', 'filename', '*'],/inHeader, qty=localFilename)
  OK = OK AND self -> getField(['*', 'Date', '*'], $
  			/inHeader, qty=localDate)
  OK = OK AND self -> getField(['*', 'Title', '*'], $
  			/inHeader, qty=localTitle)
  if (OK) then begin
    mainTitleString = localFilename + ": " + $
    	strtrim(localTitle) + "	 " + localDate
    self->add,  /inHeader, errMsg = anErrMsg,  $
    	tags = ['qtyObj', 'mainTitle', 'ICP_header_read'], $
  	contents=obj_new('qtyObj', name='mainTitle', qty=mainTitleString)
   self -> setFocus, maintitletags =['qtyObj', 'mainTitle', 'ICP_header_read']
  endif
  
;----------------------------------------------------------
  if (arg_present(thisLun)) then begin
    thisLun = lun
;    print, infolabel, "Modifying thisLun to be ", thisLun
  endif else begin
    print, infolabel, "Can not modify thisLun. Lun=", lun
  endelse

  if (keyword_set(thisClose)) then begin
;    print, infolabel, "Before closing, lun=", Lun
    close, lun, exit_status=exit_status
    free_lun, lun
    if (exit_status NE 0) then begin
      print, infolabel, 'CLOSE exit_status=', exit_status
    endif
    lun = 0
  endif

    
 if (debug) then begin
   print, infolabel, "Exiting"
    if (n_elements(thisfilename) gt 0) then  $
    	print, '	', infolabel, "thisFileName=", thisfilename
    if (n_elements(thislun) gt 0) then  $
    	print, '	', infolabel, "thislun=", thislun
    if (n_elements(thisClose) gt 0) then $
    	print, '	', infolabel, "thisClose = ",thisClose
;    if (arg_present(thisLun)) then begin
      print, '	', infolabel, "Modifiable thisLun =" , thisLun
;    endif else begin
;      print, '	', infolabel, "Can not modify thisLun"
;    endelse
 endif 



  return, OK

end		; ICP_header_read


;================================================================================
function AAA_func_icp_header_init_uv,  $
	self
;;;;      subroutine init_vectors(a, aa, bmat, bmatuv, b, 
;;     &      uu, vv, uxv, uuv, vuv, u, v, uxvi, 
;;     &	    zetatitle, etatitle, c, angle, xstar, ystar)
; from file spurion_routines.f
;
; Returns 0/1 for failure, success

  infolabel = "AAA_func_ICP_header_init_uv >> " 
  currentCode = 'AAA_func_ICP_header_init_uv '
  debug = 0

  currentClass = 'qtyObj'
  currentCode = 'AAA_func_icp_header_init_uv'
  
  delta = 1.0E-8		; for numbers within delta of zero
  
  OK = 1
  OK_add = 1
  
  if (debug) then begin
    print, infolabel, 'Entered'
  endif

  OK = self -> getField(['*', 'scatPlane_u', '*'], /inHeader, $
    		qty=vec_u,  units=uv_units)
  if (debug) then begin
    print, infolabel, OK, "=self->getfield(['*', 'scatPlane_u', '*'],...)"
    if (OK) then print, infolabel,' units=', uv_units
    if (OK) then print, infolabel, OK, ' vec_u =', vec_u
  endif
  OK = OK AND self -> getField(['*', 'scatPlane_v', '*'], /inHeader, $
    		qty=vec_v)		;; ,  units=v_units)
;  OK = OK and self->getfield(/inHeader,'scatPlane_u', qty=vec_u, units=units_u)
;  OK = OK and self->getfield(/inHeader,'scatPlane_v', qty=vec_v, units=units_v)
  if (debug) then begin
    print, infolabel, OK, "=self->getfield(['*', 'scatPlane_v', '*'], ...)"
    if (OK) then print, infolabel, OK, ' vec_v =', vec_v
  endif
  
  if (not OK) then begin
    ; The information is not available. Skip this routine.
    goto, end_unit_uv
  endif
   
  if (0.0 EQ norm( crossp(vec_u, vec_v) )) then begin
    print, infolabel, 'Orientation vectors are unsuitable!', $
                   ' (zero cross product)'
    return, 0
  endif

;  ;----- ICP_header class does not initialize the following, becuse this
;  ;----- info is not stored in the data file ICP header.
;  ; call this routine  multiple times, the objects might already be created.
  temp_legends = ['zeta','eta']
  if (n_elements(vec_u) EQ 0) then begin
    self -> add, /inHeader, $
    	tags = [currentClass, 'scatPlane_u', currentCode], $
	contents = obj_new('qtyObj', $
		name='(NOT DEFINED) Scattering plane unit vector 1', $
		qty = [0.0,0.0,0.0], units = 'r.l.u.')
;      OK = OK and self->add (legend = 'u_unitvec',  $
;            qty=obj_new('qtyObj', name = 'Scattering plane unit vector 1', $
;	      legend=temp_legends, qty=[0.0,0.0], units='r.l.u.') )
  endif	  
      
  if (n_elements(vec_v) EQ 0) then begin
    self -> add, /inHeader, $
    	tags = [currentClass, 'scatPlane_v', currentCode], $
	contents = obj_new('qtyObj', $
		name='(NOT DEFINED) Scattering plane unit vector 2',$
		qty = [0.0,0.0,0.0], units = 'r.l.u.')
;      OK = OK and self->add ('v_unitvec',  $
;            qty=obj_new('qtyObj',$
;		 name = 'Scattering plane unit vector 2', $
;	      legend=temp_legends, qty=[0.0,0.0], units='r.l.u.') )
  endif
  
;    self -> add, /inHeader, $
;    	tags = [currentClass, 'angle_v_zeta', currentCode], $
;	contents = obj_new('qtyObj', $
;		name='Scattering plane unit vector 1', $
;		 qty = 0.0, units = 'degrees.')
		 
;      OK = OK and self -> add(legend='angle_v_zeta',  $
;            qty=obj_new('qtyObj', name='Angle between v vector and zeta (x)', $
;	    	qty=0.0, units='degrees' ) )
;  
;
;  if (debug) then begin
;    junkOK = self -> getfield(/inHeader,'angle_v_zeta', all=thisAll)
;    print, infolabel, junkOk, 'angle_v_zeta', thisAll
;  endif
;    
;; Why crashing here in SHOW?
;  if (not obj_valid( self.denom)) then begin
;    OK = OK and self -> add('denom', /unprune, $
;    name='sqrt(1-cosa[0]^2-cosa[1]^2-cosa[2]^2+2.0*cosa[0]*cosa[1]*cosa[2])',$
;   units='Unitless', qty=0.0)
;;Old     units='Unitless', useString='qty')
;  endif
;
;  if (debug) then begin
;    junkOK = self -> getfield(/inHeader,'denom', all=thisAll)
;    print, infolabel, junkOk, 'denom', thisAll
;  endif
;
;  if (not obj_valid( self.xstar)) then begin
;    OK = OK and self -> add(legend='xstar', /unprune, $
;    	name='Convert from inv-distance to r.l.u. zeta-axis',$
;        qty=0.0)
;;old     	useString='qty')
;  endif
;
;  if (debug) then begin
;    junkOK = self -> getfield(/inHeader,'xstar', all=thisAll)
;    print, infolabel, junkOk, 'xstar', thisAll
;  endif
;
;  if (not obj_valid( self.ystar)) then begin
;    OK = OK and self -> add(legend='ystar', /unprune, $
;    	name='Convert from inv-distance to r.l.u. eta-axis',$
;        qty=0.0)
;;old     	useString='qty')
;  endif
;  ;----------------------------------------------------------------------------
;  
;
  
  OK = OK and self ->getfield(['*', 'latDim', '*'], /inHeader, $
  	qty=vec_dim, units=units_dim)
  OK = OK and self ->getfield(['*', 'latAngle', '*'], /inHeader, $
  	qty=vec_angle, units=units_angle)	;units should be degrees

;  if (units_dim eq '!dave_angstromSym') then begin
;;Old  if (units_dim[0] eq '!dave_angstromSym') then begin
;    units_uv = '!dave_invAngstromSym'
;  endif else begin
;    units_uv = '1/' + units_dim
;;Old    units_uv = '1/' + units_dim[0]
;  endelse
;
;;C  Calculate the reciprocal lattice vectors in
;;C  terms of Cartesian coordinates.
  bmat_temp =  dblarr(3,3)
  OK = triasubs_generateb( vec_dim, vec_angle, denom, bmat_temp)
  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'bmat', currentCode], $
	contents = obj_new('qtyObj', name='bmat rotation matrix', $
		 qty = bmat_temp, units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'bmat', currentCode]
  endif
  
  ; Hoping that denom is a return value from this routine.
  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'denom', currentCode], $
	contents = obj_new('qtyObj',$
	name='sqrt(1-cosa[0]^2-cosa[1]^2-cosa[2]^2+2.0*cosa[0]*cosa[1]*cosa[2])', $
		 qty = denom, units = 'Unitless')
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'denom', currentCode]
  endif
  
;
;  if (debug) then begin
;    ok = self.bmat -> getfield(/inHeader, everything=thisEverything)
;    print, 'denom =', denom
;    print, 'bmat_temp =', bmat_temp
;    print, 'self.bmat =', thisEverything
;;  print, 'self.bmat =', self.bmat
;  endif

;C  Perform the rotational transformation to a coordinate
;C  system where u // +x, u x v // +/- z, and v.y > 0.
  bmatuv_temp =  dblarr(3,3)
  triasubs_transform, vec_u,vec_v, bmat_temp, vec_uuv,vec_vuv, bmatuv_temp

  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'bmatuv', currentCode], $
	contents = obj_new('qtyObj', name='bmatuv rotation matrix', $
		 qty = bmatuv_temp, units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'bmatuv', currentCode]
  endif
  
;  if (debug) then begin
;    ok = self.bmatuv -> getfield(/inHeader,everything=thisEverything)
;    print, 'self.bmatuv =', thisEverything
;;;;  print, infolabel, 'self.bmatuv=', self.bmatuv
;    print, infolabel, 'vec_uuv=', vec_uuv
;    print, infolabel, 'vec_vuv=', vec_vuv
;  endif
;    
  ; Note: When done correctly, the 'z' component is zero. 
  ; vec_uuv[2] = vec_vuv[2] = 0 .

  if (abs(vec_uuv[2])  GT delta) OR ( abs(vec_vuv[2]) GT delta) then begin
;  if (vec_uuv[2]  NE 0.0) OR (vec_vuv[2]  NE 0.0) then begin
    print, infolabel, "Error! The third component should be zero! Delta=", delta
    print, infolabel, "vec_uuv = ", vec_uuv
    print, infolabel, "vec_vuv = ", vec_vuv
  endif

  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'u_unitvec', currentCode], $
	contents = obj_new('qtyObj', $
		name='Unit vector 1 in scattering plane', $
		qty = vec_uuv[0:1], units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'u_unitvec', currentCode]
  endif
  
  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'v_unitvec', currentCode], $
	contents = obj_new('qtyObj', $
		name='Unit vector 2 in scattering plane', $
		qty = vec_vuv[0:1], units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'v_unitvec', currentCode]
  endif
  
  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'xstar', currentCode], $
	contents = obj_new('qtyObj', $
		name='Horizontal scale factor in scattering plane', $
		qty = vec_uuv[0], units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'xstar', currentCode]
  endif
  
  self -> add, /inHeader, $
  	errMsg = theErrmsg, $
    	tags = [currentClass, 'ystar', currentCode], $
	contents = obj_new('qtyObj', $
		name='Vertical scale factor in scattering plane', $
		qty = vec_vuv[1], units = units_uv)
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then  begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'ystar', currentCode]
  endif
  
;;Debug  print,' vec_uuv=', vec_uuv
;;Debug  help, vec_uuv
;;Debug  print,' vec_vuv=', vec_vuv
;;Debug  help, vec_vuv
;;Debug  print,' transpose(vec_vuv) = ', transpose(vec_vuv) 
;;Debug  help, transpose(vec_vuv)
;  
;    OK = OK and self ->add(legend='u_unitvec',  $
;      qty=obj_new('qtyObj', qty=vec_uuv[0:1], units=units_uv)  ) 
;    OK = OK and self ->add(legend='v_unitvec', $ 
;        qty=obj_new('qtyObj', qty=vec_vuv[0:1], units=units_uv) )
;
 ; convert from dble to single  precision
;  temp_angle = float(atan(vec_vuv[0]/vec_vuv[1]) / !radeg)
  temp_angle = atan(vec_vuv[0]/vec_vuv[1]) / !radeg
  
  if (abs(temp_angle) LT delta) then temp_angle = 0.0

  print, infolabel, "angle_v_zeta: temp_angle = ", temp_angle
  print, infolabel, "tags=",  [currentClass, 'angle_v_zeta', currentCode]
  theErrMsg = ''
  self -> add, /inHeader, $
  	errMsg = theErrMsg, $
    	tags = [currentClass, 'angle_v_zeta', currentCode], $
	contents = obj_new('qtyObj', $
		name='Angle of Bravais lattice relative to zeta vector?', $
		 qty=temp_angle, units='Degrees')
  OK_add = (theErrMsg EQ '')
  IF (NOT OK_add) then begin
    print, theErrMsg
    print, infolabel, "Failed add tags=", $
  	[currentClass, 'angle_v_zeta', currentCode]
  endif
;  if (debug) then self -> show, /recursive
    
  if (debug) then begin
    print, infolabel, "errmsg = ", theErrMsg
    OK = self -> getField( ['*', 'angle_v_zeta', '*'], $
  	everything = theEverything)
    IF (OK) then print, infolabel, theEverything
  endif

;;Debug  print, 'temp_angle=', temp_angle
;;Debug  help, temp_angle
;  OK = OK and self ->add('angle_v_zeta', qty=temp_angle, units='Degrees')
;
;  OK = OK and self ->add('denom', qty=float(denom), units='Unitless')
;
;  OK = OK and self ->add('xstar', qty=vec_uuv[1], units=units_uv)
;  OK = OK and self ->add('ystar', qty=vec_vuv[2], units=units_uv)
;
  end_unit_uv:  
  
  print, infolabel, "returning OK=", OK
  return, OK
;  
end			; AAA_func_ICP_header_init_uv

;=============================================================================
; func_icp_header_init.pro
; These are fake methods for the object self.
; Becuase they are not true methods, any operations performed upon 
; self must be done via's its methods.
;

;*
function func_icp_header_init, $	; returns 0/1 for failure/success
  self, $
  filename = thisFilename, $	; Optional. Reads in NIST Triple axis data.
  lun=thisLun, $		; returned from read().
  close = thisClose, $		; Passed to read()
  skipInPlane = thisSkipInPlane ; In-plane scattering vectors are automatically
  				; calculated UNLESS this keyword is set.
;\
					
  common cb_ICP_header, prototypeRec, protoType_tags, protoType_ntags		


  infolabel = "func_icp_header_init >> "  
  debug = 0

;   print, infolabel
  success = 1

 if (debug) then begin
   print, infolabel, "Entered"
    if (n_elements(thisfilename) gt 0) then  $
    	print, '	', infolabel, "thisFileName=", thisfilename
    if (n_elements(thislun) gt 0) then  $
    	print, '	', infolabel, "thislun=", thislun
    if (n_elements(thisClose) gt 0) then $
    	print, '	', infolabel, "thisClose = ",thisClose
    if (arg_present(thisLun)) then begin
      print, '	', infolabel, "Modifiable thisLun"
    endif else begin
      print, '	', infolabel, "Can not modify thisLun"
    endelse
 endif 

;--------------------------------------------------------------------------
  if (n_elements(thisFileName) GT 0) then begin	;; Read from a file
   success = success and icp_header_read(self, $
   	thisFilename, $
	lun=thisLun, $		; returned from read().
	close = thisClose)		; Passed to read()
   if (NOT success) then goto, theEnd
  endif
;--------------------------------------------------------------------------

  if (debug) then begin
    print, infolabel, 'keyword_set(thisskipInPlane)=', $
    		keyword_set(thisskipInPlane)
  endif
  
; Temporarily avoid calling this routine, so we can concentrate on other things 1st.
  if (NOT keyword_set(thisskipInPlane)) then begin	;; 
    ; Automatically calculate in-plane information unless asked not to. 
    ; This will not work for certain types of files (such as 'I' style)
    ; because the required information is not present in the header. 
   successInit = AAA_func_icp_header_init_uv(self)
  endif
;--------------------------------------------------------------------------

; if (n_elements(thislun) gt 0) then  $
;    	print, '	', infolabel, "thislun=", thislun, $
;		" arg_present(thisLun)=",arg_present(thisLun) 

 if (debug) then begin
   print, infolabel, "Exit"
    if (n_elements(thisfilename) gt 0) then  $
    	print, '	', infolabel, "thisFileName=", thisfilename
    if (n_elements(thislun) gt 0) then  $
    	print, '	', infolabel, "thislun=", thislun
    if (n_elements(thisClose) gt 0) then $
    	print, '	', infolabel, "thisClose = ",thisClose
    if (arg_present(thisLun)) then begin
      print, '	', infolabel, "Modifiable thisLun"
    endif else begin
      print, '	', infolabel, "Can not modify thisLun"
    endelse
 endif

 theEnd:
   
 if (NOT success) then begin
   print, infolabel, "failed func_icp_header_init"
 endif
 
 return, success
end		; function func_icp_header_init

 
