; $Id: func_icp_subs.pro,v 1.3 2003/09/12 15:25:25 tanya Exp $
;+
; NAME:
;       func_icp_subs
;
; PURPOSE:
;
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;	<LI><A HREF="#FUNC_ICP_HEADER_INIT">FUNC_ICP_HEADER_INIT</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Dec 18, 2002.
;	Split func_icp_subs.pro into separate files. TMR. Aug 1, 2003
;-
; File triasubs.pro
; Tanya Riseman, Dec 18, 2002


;;=======================================================================
;function triasubs_init_xystar, $
;	lat_dim, lat_angle,$ ; both type objarr[3] of DescripObj
;	bmat, bmatuv, $
;	scatPlane_u, scatPlane_v, $	; both type objarr[3] of DescripObj
;	uuv, vuv
;;;;      subroutine init_vectors(a, aa, bmat, bmatuv, b, 
;;     &      uu, vv, uxv, uuv, vuv, u, v, uxvi, 
;;     &	    zetatitle, etatitle, c, angle, xstar, ystar)
;; from file spurion_routines.f
;;
;; Returns 0/1 for failure, success
;
;  if (0.0 EQ norm( crossp(scatPlane_u, scatPlane_v) )) then begin
;    print, infolabel, 'Orientation vectors are unsuitable!', $
;                   ' (zero cross product)'
;    return, 0
;  endif
;
;  OK = lat_dim->get(qty=vec_dim, units=units_dim)
;  OK = lat_angle->get(qty=vec_angle, units=units_angle)	;units should be degrees
;  OK = scatPlane_u->get(qty=vec_u units=units_u)
;  OK = scatPlane_v->get(qty=vec_v, units=units_v)
;
;;C  Calculate the reciprocal lattice vectors in
;;C  terms of Cartesian coordinates.
;  OK = triasubs_generateb( vec_dim, vec_angle, denom, bmat)
;
;;C  Perform the rotational transformation to a coordinate
;;C  system where u // +x, u x v // +/- z, and v.y > 0.
;  OK = trissubs_transform(vec_u,vec_v,bmat,vec_uuv,vec_vuv,bmatuv)
;  
;  uuv = objarr(3)
;  vuv = objarr(3)
;  for i = 0, 2 do begin
;    uuv[i] = obj_new('DescripObj', useCode='qty', qty=)  
;    vuv[i] = obj_new('DescripObj', useCode='qty')  
;  endfor
;  
;
;end		; triasubs_init_uv

;=======================================================================
 function triasubs_generateb, $
   vec_dim, $	; 3-vector with units of Inverse Angstroms
   vec_angle, $	;  3-vector with units of degrees
;Not useful   b, $	; 3-vector not used elsewhere
   denom, $		; Norm?	Value updated. Used as relcircsize by spurion.
   bmat		; transformation matrix
;;;      subroutine generateb(a,aa,b,c,BB) from file generate.f
;C     Transformation taken from Acta. Cryst. (1967) 22, 457-464
;C     by William R. Busing and Henri A. Levy.

  infolabel = "triasubs_generateb>> "  
  debug = 0
  
  cosa = cos(double(vec_angle)/!radeg)		; a 3-vector
  sina = sin(double(vec_angle)/!radeg)		; a 3-vector
  if (debug) then begin
    print, infolabel, 'cosa = ', cosa
    print, infolabel, 'sina = ', sina
  endif  
;  if (not arg_present(bmat)) then begin
;     print, infolabel, ' Argument bmat required!'
;     return, 0 
;  endif
;  if (not arg_present(denom)) then begin
;     print, infolabel,'Argument denom requiredd even though not terribly useful!'
;     return, 0 
;  endif
  
; Check for stupid mistakes
   suitable_dim = 1
   suitable_angle = 1
   for i = 0, 2 do begin
     suitable_dim = suitable_dim and (vec_dim[i] GT 0.0)
     suitable_angle = suitable_angle and (sina[i] NE 0.0)
   endfor
   if (not suitable_dim) then begin
     print, infolabel, 'Lattice dimensions must be all positive: ', vec_dim
   endif
   if (not suitable_angle) then begin
     print, infolabel, 'Lattice angles must not be 0 or 180 degrees: ', $
     		 vec_angle
   endif

   denom = sqrt(1.0d0	-cosa[0]^2 -cosa[1]^2 -cosa[2]^2  $
	+ 2.0*cosa[0] *cosa[1] *cosa[2] )
   if (denom EQ 0.0) then suitable_angle = 0
   
   if (not suitable_dim) or (not suitable_angle)  then begin
     return, 0 
   endif

  b = make_array(3, /double)
  cosb = make_array(3, /double)
  sinb = make_array(3, /double)
  b = 2.0d0*!pi *sina/(double(vec_dim)*denom)		; 3-vector
  if (debug) then begin
    print, infolabel, 'b=', b
  endif
  for i = 0, 2 do begin
;    b[i] = 2.0d0*!pi 
;    	*sin(double(vec_angle[i])/!radeg) /(double(vec_dim[i])*denom)
    triasubs_ijk, i, j, k	; cycle the indices (0,1,2),(1,2,0),(2,0,1)
    cosb[i] = (cosa[j]*cosa[k] - cosa[i])/(sina[j]*sina[k])
  endfor
  sinb = sqrt(1.0d0 - cosb^2)
  if (debug) then begin
    print, infolabel, 'cosb = ', cosb
    print, infolabel, 'sinb = ', sinb
  endif
  
;Already made
;  bmat = make_array(3, 3, /double)
  bmat=[[b[0], 0.0d0, 0.0d0], $
  	[b[1] * cosb[2], b[1] * sinb[2], 0.0d0], $
	[ b[2] * cosb[1],  -b[2] * sinb[1] *cosa[0] , 2.0d0 *!pi /vec_dim[2]] ]
	  
;  bmat[0,0] = b[0]
;  bmat[1,0] = 0.0d0
;  bmat[2,0] = 0.0d0
;
;  bmat[0,1] = b[1] * cosb[2]  
;  bmat[1,1] = b[1] * sinb[2]  
;  bmat[2,1] =0.0d0
;
;  bmat[0,2] = b[2] * cosb[1]  
;  bmat[1,2] = -b[2] * sinb[1] *cosa[0]  
;  bmat[2,2] =2.0d0 *!pi /vec_dim[2]
 
  if (debug) then begin
    print, infolabel, bmat
  endif
    
 return, 1 
end			; triasubs_generateb

;=======================================================================
pro triasubs_ijk, i, j, k 
; Cycle the indices (0, 1, 2) 
;;;      subroutine ijk(i,j,k) from file generate.f

  case i of 
    0: begin
    	j = 1
	k = 2
    	end
    1: begin
    	j = 2
	k = 0
    	end
    2: begin
    	j = 0
	k = 1
    	end
  endcase

end			; triasubs_ijk

;=======================================================================
function triasubs_calcThetaRad, x, y
;;;      subroutine calctheta(x,y,theta) from file transform.f
;C     Find the angle theta from the +x-axis to vector
;C     xi + yj over a range of -pi to pi (an extended
;C     inverse trig function)

  debug = 0

      if ((x EQ 0.0) and (y EQ 0.0)) then begin
        return, 0.0
      endif

;C     use arccosine to avoid any problems with x=0

;C     Quadrants I and II are fine as is and theta values
;C     will range from 0 to pi
      
      if (y GT 0.0) then begin
        return, acos(x/sqrt(x^2 + y^2))
      endif else begin

;C     Quadrants III and IV will need a negative sign since transform
;C     would go to -x otherwise.

        return, -acos(x/sqrt(x^2 + y^2))

      endelse

end		; triasubs_calcThetaRad 

;=======================================================================
function triasubs_rotate, $
	thetaRad, $	; angle in radians
	i 		; ith axis
;;;      subroutine rotate(theta,i,r) from file transform.f
;C     Generates the passive transformation matrix for a 
;C     counterclockwise rotation of the axes by theta radians 
;C     about the i-th axis.

  debug = 0

    triasubs_ijk, i, j, k	; cycle the indices (0,1,2),(1,2,0),(2,0,1)
    rmatrix = make_array(3, 3, /float)
    rmatrix[i,i] = 1.0
;    rmatrix[i,j] = 0.0
;    rmatrix[i,k] = 0.0
;    rmatrix[j,i] = 0.0
    rmatrix[j,j] = cos(thetaRad)
    rmatrix[j,k] = -sin(thetaRad)
;    rmatrix[k,i] = 0.0
    rmatrix[k,j] = sin(thetaRad)
    rmatrix[k,k] = cos(thetaRad)
    
    return, rmatrix
end		;;  triasubs_rotate

;=======================================================================
pro triasubs_transform, ui,vi,bmat,u3,v3,b1mat
;scatPlane_u, scatPlane_v, bmat, uuv, vuv,  bmatuv
;;;      subroutine transform(ui,vi,bmat,u3,v3,b1mat) from file transform.f

  infolabel = "triasubs_transform>> "  
  debug = 0
  
;C     The desired transformation is to a coordinate system where
;C     x or zeta is perpendicular to uu, vv is a postive rotation
;C     of A3 away from uu, and z is hence parallel to uu x vv.
;C
;C     Currently b[1] is parallel to x and b[2] lies in the x-y
;C     plane because of the generating routine

;C     Generate u and v in the current coordinate system.

;   u = fltarr(3)
;   v = fltarr(3)
   u = ui[0]*bmat[*,0] + ui[1]*bmat[*,1] + ui[2]*bmat[*,2] 
   v = vi[0]*bmat[*,0] + vi[1]*bmat[*,1] + vi[2]*bmat[*,2] 
   if (debug) then begin
     print, infolabel, 'ui=', ui
     print, infolabel, 'vi=', vi
     print, infolabel, 'bmat =', bmat
     print, infolabel, 'u=', u
     print, infolabel, 'v=', v
   endif

;C     What is one of the transformation matrices that will align
;C     u and x?  Will take 2 steps.
;C     Rotation about z-axis with quadrant conventions.

   thetaRad = triasubs_calcThetaRad(u[0], u[1])
   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
   endif
   
;C     -theta appears in the transformation matix because we
;C     want the transformation rotate the coordinate axies
;C     so that u will lie in the x'-z' plane
   tmat = triasubs_rotate(-thetaRad,2)
   b1mat = tmat ## bmat	;Fortran: call mmmult(tmat,bmat,3,3,3,b1mat)
   u1 = tmat ## u		;Fortran: call mvmult(tmat,u,3,3,u1)
   v1 = tmat ## v		;Fortran: call mvmult(tmat,v,3,3,v1)

   if (debug) then begin
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b1mat=', b1mat
     print, infolabel, 'u1=', u1
     print, infolabel, 'v1=', v1
   endif
      
;C     Rotation about y1-axis
;C     This one is a bit different since with the y1-axis coming
;C     out of the page x1-z1 are left-handed.  Best plan is to
;C     choose theta such that it is correct for the +z1 axis
;C     and then rotate pi/2 further to +x1.

   thetaRad = triasubs_calcThetaRad(u1[2], u1[0])
;   temp = -thetaRad + !pi/2.0
   tmat = triasubs_rotate( (-thetaRad + !pi/2.0), 1)

   b2mat = tmat ## b1mat	;Fortran: call mmmult(tmat,b1mat,3,3,3,b2mat)
   u2 = tmat ## u		;Fortran: call mvmult(tmat,u,3,3,u2)
   v2 = tmat ## v		;Fortran: call mvmult(tmat,v,3,3,v2)

   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b2mat=', b2mat
     print, infolabel, 'u2=', u2
     print, infolabel, 'v2=', v2
   endif

;C     final rotation about x''-axis
   thetaRad = triasubs_calcThetaRad(v2[1], v2[2])
      
   tmat = triasubs_rotate( (-thetaRad), 0)
   b1mat = tmat ## b2mat	;Fortran: call mmmult(tmat,b2mat,3,3,3,b1mat)
   u3 = transpose(tmat ## u2)		;Fortran: call mvmult(tmat,u2,3,3,u3)
   v3 = transpose(tmat ## v2)		;Fortran: call mvmult(tmat,v2,3,3,v3)

   if (debug) then begin
     print, infolabel, 'thetaRad =', thetaRad
     print, infolabel, 'tmat =', tmat
     print, infolabel, 'b1mat=', b1mat
     print, infolabel, 'u3=', u3
     print, infolabel, 'v3=', v3
   endif


end			; triasubs_transform

;=======================================================================
function triasubs_remove_lcd,  $
	x, y, a, b
; Based on subroutine mult(x,y,a,b) in spurion file calclattice.f
;C find the solution to ax+by=0
;C with no common factor between a and b
; Return 0 if indeterminant, 1 if OK.

  debug = 0

      if ((x eq 0) and (y eq 0)) then begin
        a=0                   ; both a and b indeterminant so this
        b=0                   ; really a flag not to do any reduction
        return, 0
      endif
      
      if (x eq 0) then begin
        b=0                   ; a indeterminant
        a=1
        return, 0
      endif
      
      if (y eq 0) then begin
        a=0                   ; b indeterminant
        b=1
        return, 0
      endif

;C so a and b are determined up to some multiple and the solution
;C a=y, b=-x is fine, dump it into lcd to check for any common
;C factors
     n_vec =   [y, -x, 0]   
     triasubs_lcd, n_vec, fac
     a=n_vec[0]
     b=n_vec[1]  
     return, 1
     
end		; triasubs_remove_lcd

;=======================================================================
pro triasubs_irreduc, $
	uin, vin, uirr, virr
; Based on subroutine irreduc(u,v,uirr,virr) in spurion file calclattice.f
;C  generate irreducible pair of vectors uirr,virr as close
;C  to right angles as possible that are still in the
;C  same plane as u and v

  debug = 0

;C  first check for any common denominators using division by primes
   uirr = uin
   virr = vin
  ; the prime "factors" by which uirr, virr has been changed
   triasubs_lcd, uirr, factors
   triasubs_lcd, virr, factors

;C what is the possibility that an integer sum of the peaks will yield
;C zero for one of the indices?

   check_again:
   for j = 0, 2    do begin
     nonZero = triasubs_remove_lcd(uirr[j], virr[j], ucoeff, vcoeff)
     if (nonZero) then begin
       diff = ucoeff*uirr + vcoeff*virr		; 3-vector
      	 triasubs_lcd, diff, n
       if (norm(diff) lt norm(virr)) then begin
         virr = diff			; remove the common factor
	 goto, check_again
       endif
     endif		; if (nonZero)
   endfor

end		; triasubs_irreduc

;=======================================================================
pro triasubs_lcd, $
	uirr, fac
; Based on subroutine lcd(u,v,uirr,virr) in spurion file calclattice.f
;C  Find the lowest common denominator fac in the 3-vector uirr.

  debug = 0

   nprimes = 26		; more than we will ever need.
   primes_vec = primes(nprimes)
   
   check_again:
   if (norm(uirr) gt 0) then begin
   ; Alternative is stupid (e.g. all zeros)
     for i = 0, (nprimes -1) do begin
       comm = 1	; true. Common denominator is found using modulo operator
       for j = 0, 2 do begin
         comm = comm and ( (fix(uirr[j]) mod primes_vec[i]) EQ 0 )
       endfor
       if (comm) then begin
         for j = 0, 2 do begin
          uirr(j) = uirr[j] / primes_vec[i]
	 endfor
	 fac = fac * primes_vec[i]
	 goto, check_again	; might have multiples of the same prime factor
       endif
     endfor
   endif

;C include a quick check of negatives
   comm = 1	; true. Common denominator is found using modulo operator
   for j = 0, 2 do begin
      comm = comm and (abs(uirr[j]) eq -uirr[j])
   endfor
   if (comm) then begin
     for j = 0, 2 do begin
       uirr[j] = -uirr[j]
     endfor
   endif   

end		; triasubs_lcd

