; $Id: generic__print_frag.pro,v 1.1 2003/07/23 14:15:13 tanya Exp $ 
;+
; NAME:
;	 generic__print_frag.pro
;
; PURPOSE:
; 	Fragment of method code for all classes	 <class>::print
; 	Insert in your class defintion  
;
;	pro YourClass::print, $
;	  ; ThisField is parameter with name of object struct's desired tag.
;	  ; Only the data associated with the tag is printed, if present.
;  	  thisField, $	; string
;  	  lun=thisLun, $; Optionally, write to a file. If absent, to screen
;	  alsoPtrs=alsoPtrs ; Optionally print contents of pointers.
;
;	  @generic__print_frag.pro
;	end
;	
; 	This avoids cut and paste proliferation of code, perhaps.
;
;	Note: In IDL, classes using inheritance basically construct
; one huge structure including all the data. The inherited methods
; have access to ALL of the data in the structure, not just the parts that
; are relevent to the inherited class. Therefore it is best to use these
; generic code fragements for methods ::print_obj and ::help_obj
; ONLY FOR THE TOP MOST CLASS which has inherited classes.
;
; I have found that it is preferable to make a container class 
; DAVE_tagContainer with basically two trees in parallel,
; one for the basic "data" using the class qtyObj and 
; another for I/O and GUI issues using the class tagObj.
; This is specifically to avoid creating a new class which inherits
; qtyObj and tagObj. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE application programming.
;
; SEE ALSO:
;	<LI><A HREF="#AAA_README">AAA_README</A>
;
; MODIFICATION HISTORY:
;	 Written by Tanya Riseman, Feb 4, 2003
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;=============================================================================
;Frag pro generic_obj::print, $
;Frag;	  ; ThisField is parameter with name of object struct's desired tag.
;Frag;	  ; Only the data associated with the tag is printed, if present.
;Frag  thisField, $		; string.
;Frag  ;-----------------------
;Frag  lun=thisLun, $; Optionally, write to a file. If absent, to screen
;Frag  alsoPtrs=alsoPtrs ; Optionally print contents of pointers.
;Frag  ;----------
;Frag;;;;  tags = thisTags	; Optional boolean keyword, used to display the current tags.
;;;;    			; thisField is ignored.
			

  infolabel = "generic_obj::print >> "  

;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  OK = execute(string('junkRec = {', obj_class(self), '}'))
  theTagnames =   tag_names(junkRec)  
  ntags = n_tags(junkrec)

  if (keyword_set(thistags)) then begin
    printf, lun, 'Tags=', theTagNames
    printf, lun
    return
  endif

  if (n_elements(thisField) eq 0)  then begin
   i_start = 0
   I_stop = (ntags -1) 
  endif else begin
   i_index =  WHERE(theTagNames EQ StrUpCase(thisField), count)
   if (count eq 0) then begin	; Failure to find a match
     i_start = 0
     i_stop = -1
      printf, lun, ' Failed to find the field ', thisField, $
     	' in  object with  class ', obj_Class(self)
   endif else begin	; Should only be one match
     i_start = i_index[0]
     i_stop = i_index[0]
   endelse
   
  endelse

  for i = i_start, i_stop do begin
    dim = n_elements(self.(i))
    if (dim eq 1) then begin
      if (keyword_set(alsoPtrs)) and (ptr_valid(self.(i))) then begin
           printf, lun, 'self.(',strtrim(i,2), ') =	*(self.', $
	   	theTagNames[i], $
	   	') =	', *(self.(i))
      endif else begin
           printf, lun, 'self.(',strtrim(i,2), ') =	self.', $
	   	theTagNames[i], $
	   	'=	', self.(i)
      endelse
    endif else begin
      printf, lun, 'self.(',strtrim(i,2), ') =	self.', $
	   	theTagNames[i], $
	   	'=	', self.(i)
    endelse
  endfor
  
;Frag end
 
