; $Id: nicesort.pro,v 1.2 2003/09/12 15:25:09 tanya Exp $
;+
; NAME:
; 	nicesort
;
; PURPOSE:
;	This routine sorts an array or pointer to an array. It 
; returns indexing arrays via keywords. 
; It also provides additional indexing arrays to find the start and stop 
; ranges of duplicated entries. 
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;	DAVE utilites
;
; MODIFICATION HISTORY:
;	Created Sept, 2003. Tanya Riseman
;-
;*
pro nicesort, $
	; Requires EITHER array (qty) or pointer to array (ptr_qty)
	qty = thisQty, $	; Input parameter. The array to sort.
	ptr_qty = thisPtr_qty, $; Input. Pointer to the array to sort.
	index = thisIndex, $	; Output int array. Index for sorting the array.
				; Contains duplicates
	start = thisStart, $	; Output int array. Index for the index, giving
				; the first occurances.
	stop = thisStop, $	; Output int array. Index for the index, giving
				; the last occurances.
	dupl = thisDupl		; Output int array. Number of duplicates
;/

  if (n_elements(thisQty) GT 0) then begin
    ptrQ = ptr_new(thisQty)
  endif
  if (n_elements(thisPtr_Qty) GT 0) then begin
    ptrQ = ptr_new(*thisPtr_Qty)
  endif
  
  if (n_elements(ptrQ) EQ 0) then begin
    message, /info, "Require EITHER qty OR ptr_qty keywords"
    return
  endif

  thisIndex = sort(*ptrQ)
  thisStop = uniq( (*ptrq)[thisIndex])
  nlen = n_elements(thisStop)
 
  thisStart = [0, (thisStop[0:(nlen-2)] +1) ]

  thisDupl = thisStop - thisStart + 1

  ptr_free, ptrQ
end

;=====================
pro nicesort_example

  nx=25
  mx=12
  x=(1.0+mx*randomu(unknown,nx))
  x = [x, x[0:10], x[0:3]]		; Put some duplicates in

  ptrX = ptr_new(x)
  ; Can pass it either an array or a pointer to an array
  nicesort, ptr_qty=ptrX, $
	index = xIndex, $
	start = iStart, $
	stop = iStop, $	
	dupl = xDupl
  ptr_free, ptrX

;  nicesort, qty=x, $
;	index = xIndex, $
;	start = iStart, $
;	stop = iStop, $	
;	dupl = xDupl

  print, "sorting index=", xindex
  print
  
  print,"istop=", istop		; is correct 
  print, "n_elements(istop)=", n_elements(istop)
  print

  print, "istart = ", istart
  print

;  print, "duplicity = ", xdupl
;  print

  print,"Has only 14 unique values: x = ", x
  print, "n_elements(x)=", n_elements(x)
  print


  print, "sorted x =", x[xindex]
  print

;  print, "First occurances in sorted x =", x[xindex[istart]]
;  print
  
  
;  print, "last occurances in sorted x =", x[xindex[istop]]
;  print
  
;  print, "Duplicity in sorted x =", xdupl
;  print
  
  print, "Sorted duplcity, range between x's start and stop"
  for i = 0, (n_elements(xdupl)-1) do begin
   print, " dupl[", strtrim(string(i),2),  "]=", strtrim(string(xdupl[i]),2), $
   	x[xindex[istart[i]:istop[i]]]
	
  endfor  
  print
 	  
end
