; $Id: printargpresent.pro,v 1.2 2003/09/12 15:25:09 tanya Exp $
;+
; NAME:
;      printArgPresent
;
; PURPOSE:
;
;	Helper function for debuging the methods <objectClass>::get*
; which use the anonymous structure argPresent..
;
;	The IDL function arg_present(yourKeyword) returns reasonable 0/1
; values for the use of a keywords, but if the keyword is then passed
; down to other  methods, functions and procedures, it then becomes
; always true. This means that items are unnecessarily calculated when
; not needed, slowing down the code. In the case of objects which we 
; didn't expect to make, it creates a memory leak because they are 
; not destroyed.
;
;	The work around is to use a structure called ArgPresent, which
; is passed onto the lower routines. See example section.
;
;	This routine gets passed an anonymous structure ArgPresent. 
; The routine prints out the labels (which correspond to the names of the 
; keywords) for all members which have the value TRUE. This routine is NOT 
; dependent on knowing the names of the keywords or the field names of the 
; structure argPresent ahead of time.
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;       NIST Center for Neutron Research
;       100 Bureau Drive
;       Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;	DAVE utility programming.
;
; OTHER FILES AND ROUTINES USED: None
;
; EXAMPLE
;   WRONG WAY:
; 	Ok = topGet(key1 = name)
;
;	function topGet, key1=thisKey1, key2=thisKey2, key3=thisKey3
;	  OKleft = leftGet(key2=thisKey2)
;	  OKright = rightGet(key1=thisKey1, key3=thisKey3
;	  if (n_elements(thisKey1)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  if (n_elements(thisKey2)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  if (n_elements(thisKey3)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  ;; Note: All three keys now exists and will be printed.
;	  return (OKleft OR OKright)
;       end
;
;   RIGHT WAY:
; 	Ok = topGet(key1 = name)
;
;	function topGet, $
;	  	argPresent = thisArgPresent, $; ARG_PRESENT() function does not work on 
;   		;propagated keywords from other calling routines.
;		; Needed for get* methods.
;		key1=thisKey1, key2=thisKey2, key3=thisKey3
;
;	  if (NOT arg_present(thisArgPresent)) then begin  
;   		ArgPresent = {$	; make this be anonymous structure
;			key1 : arg_present(thisKey1), $
;			key2 : arg_present(thisKey2), $
;			key3 : arg_present(thisKey3)}
;	  endif else begin
;	    ArgPresent = thisArgPresent
;  	  endelse
;
;	  printArgPresent, ArgPresent
;
;	  if (argPresent.key2) then begin
;	    OKleft = leftGet(argPresent=argPresent, key2=thisKey2)
;	  endif
;	  if ((argPresent.key1) OR (argPresent.key3)) then begin
;	    OKright = rightGet(argPresent=argPresent, key1=thisKey1,key3=thisKey3
;	  endif
;	  if (n_elements(thisKey1)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  if (n_elements(thisKey2)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  if (n_elements(thisKey3)GT 0) then begin
;		print, "thisKey1 Created. ", thisKey1
;	  endif
;	  ;; Note:  Only key1 now exists and will be printed.
;	  return (OKleft OR OKright)
;       end
;
; MODIFICATION HISTORY:
;	Created Spring, 2003. Tanya Riseman
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;======================================================
;*
pro printArgPresent, $
	ArgPresent, $	; Anonymous structure containing fields corresponding
		; to keywords and boolean values (0/1) indicating if
		; those keywords are being used. 
	showValues=thisShowValues	; Boolean keyword. 
;/

     ntags = n_tags(ArgPresent)
     theTags = tag_names(ArgPresent)
;     temparray = strarr(ntags)
     astring = ''
     for i = 0, (ntags -1) do begin
;       temparray[i] = theTags[i] + ':' + $
;       	strcompress(string(ArgPresent.(i))) + ' '
       if ((ArgPresent.(i)) OR (keyword_set(thisShowValues))) then begin
         aString = aString + theTags[i]
	 if (keyword_set(thisShowValues)) then begin
	   aString = aString + '=' + $
	   	strcompress(string(ArgPresent.(i)), /remove_all) 
	 endif
         if (i NE (ntags -1)) then  aString = aString + ', '
       endif
     endfor
;     print, 'ArgPresent = ', tempArray
     print, 'ArgPresent: ', aString
end

