; $Id: triasubs_calcthetarad.pro,v 1.2 2003/09/12 15:24:50 tanya Exp $
;+
; NAME:
;       triasubs_calcThetaRad
;
; PURPOSE:
;
;;;      subroutine calctheta(x,y,theta) from file transform.f
;C     Find the angle theta from the +x-axis to vector
;C     xi + yj over a range of -pi to pi (an extended
;C     inverse trig function)
;
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;	<LI><A HREF="#FUNC_ICP_HEADER_INIT">FUNC_ICP_HEADER_INIT</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Dec 18, 2002.
;	Split func_icp_subs.pro into separate files. TMR. Aug 1, 2003
;-

;=======================================================================
;*
function triasubs_calcThetaRad, $	; returns angle theta
	x,  $				; cartesian x coord
	y				; cartesian y coord
;/
;;;      subroutine calctheta(x,y,theta) from file transform.f
;C     Find the angle theta from the +x-axis to vector
;C     xi + yj over a range of -pi to pi (an extended
;C     inverse trig function)

  debug = 0

      if ((x EQ 0.0) and (y EQ 0.0)) then begin
        return, 0.0
      endif

;C     use arccosine to avoid any problems with x=0

;C     Quadrants I and II are fine as is and theta values
;C     will range from 0 to pi
      
      if (y GT 0.0) then begin
        return, acos(x/sqrt(x^2 + y^2))
      endif else begin

;C     Quadrants III and IV will need a negative sign since transform
;C     would go to -x otherwise.

        return, -acos(x/sqrt(x^2 + y^2))

      endelse

end		; triasubs_calcThetaRad 

