; $Id: triasubs_generateb.pro,v 1.2 2003/09/12 15:24:51 tanya Exp $
;+
; NAME:
;       triasubs_generateb
;
; PURPOSE:
;
;;;      subroutine generateb(a,aa,b,c,BB) from file generate.f
;C     Transformation taken from Acta. Cryst. (1967) 22, 457-464
;C     by William R. Busing and Henri A. Levy.
;
; 	Based on Fortran routines used by spurion program.
;	These routines are used by func_icp_header_init and func_icp_data_init.
;
;.........................................................................
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; SEE ALSO:
;	<LI><A HREF="#FUNC_ICP_DATA_INIT">FUNC_ICP_DATA_INIT</A>
;	<LI><A HREF="#FUNC_ICP_HEADER_INIT">FUNC_ICP_HEADER_INIT</A>
;	
; MODIFICATION HISTORY:
;
;       Written by Tanya Riseman, Dec 18, 2002.
;	Split func_icp_subs.pro into separate files. TMR. Aug 1, 2003
;-

;=======================================================================
;*
 function triasubs_generateb, $		; returns 0/1 for failure/success
   vec_dim, $	; 3-vector with units of Inverse Angstroms
   vec_angle, $	;  3-vector with units of degrees
   denom, $		; Norm?	Value updated. Used as relcircsize by spurion.
   bmat		; transformation matrix
;/
;Not useful   b, $	; 3-vector not used elsewhere


  infolabel = "triasubs_generateb>> "  
  debug = 0
  
  cosa = cos(double(vec_angle)/!radeg)		; a 3-vector
  sina = sin(double(vec_angle)/!radeg)		; a 3-vector
  if (debug) then begin
    print, infolabel, 'cosa = ', cosa
    print, infolabel, 'sina = ', sina
  endif  
;  if (not arg_present(bmat)) then begin
;     print, infolabel, ' Argument bmat required!'
;     return, 0 
;  endif
;  if (not arg_present(denom)) then begin
;     print, infolabel,'Argument denom requiredd even though not terribly useful!'
;     return, 0 
;  endif
  
; Check for stupid mistakes
   suitable_dim = 1
   suitable_angle = 1
   for i = 0, 2 do begin
     suitable_dim = suitable_dim and (vec_dim[i] GT 0.0)
     suitable_angle = suitable_angle and (sina[i] NE 0.0)
   endfor
   if (not suitable_dim) then begin
     print, infolabel, 'Lattice dimensions must be all positive: ', vec_dim
   endif
   if (not suitable_angle) then begin
     print, infolabel, 'Lattice angles must not be 0 or 180 degrees: ', $
     		 vec_angle
   endif

   denom = sqrt(1.0d0	-cosa[0]^2 -cosa[1]^2 -cosa[2]^2  $
	+ 2.0*cosa[0] *cosa[1] *cosa[2] )
   if (denom EQ 0.0) then suitable_angle = 0
   
   if (not suitable_dim) or (not suitable_angle)  then begin
     return, 0 
   endif

  b = make_array(3, /double)
  cosb = make_array(3, /double)
  sinb = make_array(3, /double)
  b = 2.0d0*!pi *sina/(double(vec_dim)*denom)		; 3-vector
  if (debug) then begin
    print, infolabel, 'b=', b
  endif
  for i = 0, 2 do begin
;    b[i] = 2.0d0*!pi 
;    	*sin(double(vec_angle[i])/!radeg) /(double(vec_dim[i])*denom)
    triasubs_ijk, i, j, k	; cycle the indices (0,1,2),(1,2,0),(2,0,1)
    cosb[i] = (cosa[j]*cosa[k] - cosa[i])/(sina[j]*sina[k])
  endfor
  sinb = sqrt(1.0d0 - cosb^2)
  if (debug) then begin
    print, infolabel, 'cosb = ', cosb
    print, infolabel, 'sinb = ', sinb
  endif
  
;Already made
;  bmat = make_array(3, 3, /double)
  bmat=[[b[0], 0.0d0, 0.0d0], $
  	[b[1] * cosb[2], b[1] * sinb[2], 0.0d0], $
	[ b[2] * cosb[1],  -b[2] * sinb[1] *cosa[0] , 2.0d0 *!pi /vec_dim[2]] ]
	  
;  bmat[0,0] = b[0]
;  bmat[1,0] = 0.0d0
;  bmat[2,0] = 0.0d0
;
;  bmat[0,1] = b[1] * cosb[2]  
;  bmat[1,1] = b[1] * sinb[2]  
;  bmat[2,1] =0.0d0
;
;  bmat[0,2] = b[2] * cosb[1]  
;  bmat[1,2] = -b[2] * sinb[1] *cosa[0]  
;  bmat[2,2] =2.0d0 *!pi /vec_dim[2]
 
  if (debug) then begin
    print, infolabel, bmat
  endif
    
 return, 1 
end			; triasubs_generateb

