; $Id: specobj__define.pro,v 1.10 2003/09/23 13:30:17 tanya Exp $ 
;+
; NAME:
;        specObj
;
; PURPOSE:
;
;
; AUTHOR:
;
;       Tanya M. Riseman, Ph.D.
;               NIST Center for Neutron Research
;       100 Bureau Drive
;               Gaithersburg, MD 20899
;       Phone: (301) 975-8379
;       E-mail: tanya.riseman@nist.gov
;
; CATEGORY:
;
;       Objects. DAVE application programming.
;
; OBJECT DEFINTION
;	 <LI><A HREF="#SPECOBJ__DEFINE_arguments">SPECOBJ__DEFINE_arguments</A>
;
; 
; OTHER OBJECT CLASSES AND ROUTINES USED:
;	<LI><A HREF="#FOCUSOBJ__DEFINE_arguments">FOCUSOBJ__DEFINE_arguments</A>
;	<LI><A HREF="#DAVE_TAGCONTAINER__DEFINE_arguments"> DAVE_TAGCONTAINER__DEFINE_arguments</A>
;	<LI><A HREF="#TAGOBJ__DEFINE_arguments">TAGOBJ__DEFINE_arguments</A>
;	<LI><A HREF="#QTYOBJ__DEFINE_arguments">QTYOBJ__DEFINE_arguments</A> (usually)
;
; OTHER ROUTINES USED: 
;	...
;
; INCLUDE FILES:
;	<A HREF="#GENERIC__HELCLASS_FRAG.PRO">GENERIC__HELCLASS_FRAG.PRO</A>
;	<A HREF="#GENERIC__HELP_OBJ_FRAG.PRO">GENERIC__HELP_OBJ_FRAG.PRO</A>
;	<A HREF="#GENERIC__PRINT_FRAG.PRO">GENERIC__PRINT_FRAG.PRO</A>
;
; EXAMPLE
;	IDL> <A HREF="#TAGOBJ_EXAMPLE">SPECOBJ_EXAMPLE</A>
;
; MODIFICATION HISTORY:
;	Tanya Riseman, Spring, 2003. 
;	specobj::Append renamed specObj::Concatenate. July 23, 2003. TR
;	COMBO_OBJ_NEW routine written . TMR Aug 9, 2003
;	COMBO_RESTORE routine written . TMR Aug 20, 2003
;	COMPACT, MakeLists rotuiens written. TMR. Sep 2, 2003
;	::BinaryOP and ::UnitaryOP now call qtyObj_binaryOp and qtyObj_unitaryOp. TMR Sept 8, 2003
;	::FRGD_BKGD routine written.  TMR Sept 8, 2003
;	TMR. Sept 15, 2003. changed usingGUI (0/1) to usingGroupLeader (0/#)
;	TMR. Sept 17, 2003. Added self.ownerDavePtr to object definition. 
;		Only cleanup self.davePtr if self.ownerDavePtr = 'local'.
;
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;======== PRINT method common to all classes ===============================
;+
; NAME:
; 	specObj::print_obj
;
; PURPOSE:
;	Prints to the screen or file unit number the contents of the object.
; 	Uses @generic__print_frag.pro.
;-
;*
pro specObj::print_obj, $
  ; ThisField is parameter with name of object struct's desired tag.
  ; Only the data associated with the tag is printed, if present.
  thisField, $		; string.
  lun=thisLun, $	; Optionally, write to a file. If absent, to screen
  alsoPtrs=alsoPtrs	 ; Optionally print contents of pointers.
;/
	  @generic__print_frag.pro
  
end		; ::print_obj

;======== HELP method common to all classes ===============================
;+
; NAME:
; 	specObj::help_obj
;
; PURPOSE:
;	Prints to the screen or file unit number CO
; 1. The types of the data member of the object 
; 2. All the methods associated with the object
; 3. All the arguments and keywords associated with the object.
;
;	Uses @generic__help_obj_frag.pro.
;-
;*
pro specObj::help_obj, $
  methodsOnly=methodsOnly,  $	; Ignore. Keyword only used by recursion call
  helpProStr=helpProStr, $	; Ignore. Keyword only used by recursion call
  helpFuncStr=helpFuncStr, $	; Ignore. Keyword only used by recursion call
  class=thisClass, $		; Ignore. Keyword only used by recursion call
  level=level, $		; Ignore. Keyword only used by recursion call
  lun=thisLun 			; Optionally, write to a file. If absent, to screen
;/

	  @generic__help_obj_frag.pro
end		; ::help_obj

;======== internal method HELPCLASS for use by HELP method ===================
;+
; NAME:
; 	specObj::helpclass
;
; PURPOSE:
;	Helper method for specObj::help_obj. Not for external use.
;	Uses @generic__helpClass_frag.pro.
;-
;*
pro specObj::helpClass, $
   helpProStr=helpProStr, $
   helpFuncStr=helpFuncStr, $
   class=thisClass, $
   level=level, $
   lun=thisLun 	; Optionally, write to a file. If absent, to screen
;/

	  @generic__helpClass_frag.pro
end		; ::HelpClass

;========= End of methods common to all classes =============================

;+
; NAME:
; 	specObj::init
;
; PURPOSE:
;	Initialize the object.
;-
;*
function specObj::init, $		; Returns 0/1 for failure/success
   ;----------------------------------------------------------------
   ; Use keywords combos (davePtr,/convert) or 
   ; (instrument, path, filename, dataReader) BUT NOT COMBINED!
   ;----------------------------------------------------------------
   davePtr = thisDavePtr, $		; Optionally create specObj from DAVE data
   					; if filename not used. 
					; Otherwise this is the davePtr handed from
					; above.
   convert = thisConvert, $		; Boolean keyword. If true, convert
   					; the DavePtr. 
					; Must be used with davePtr keyword.
   ownerDavePtr = thisownerDavePtr, $	; If 'local', cleanup destroys self.davePtr.
   					; If 'main', cleanup skips self.DavePtr
					; Defaults to 'main'.
   ;----------------------------------------------------------------
   instrument=thisInstrument, $		; Optional string. 
   					; Only used w/out convert keyword
   path = thisPath, $			; String. path for ICP file.
   					; Only used w/out convert keyword.
   filename = thisFilename, $		; String. ICP file to read from.
   					; Only used w/out convert keyword.
   dataReader = thisDataReader, $	; String. Data reader function's name
   					; Only used w/out convert keyword.
					; If datareader is left out, 
					; then assume that files are DAVE 
					; files and not raw data.
   ;----------------------------------------------------------------
   Label = thisLabel, $			; String. label portion of tags.
   CodeSource = thisCodeSource, $	; String. CodeSource portion of tags.
   ;----------------------------------------------------------------
   wID = thisWID, $			; widget ID asscoated with object self
   ;----------------------------------------------------------------
   errMsg = thisErrMsg, $		; Optional keyword. 
   					; Error message returned here.
   					; null string if success.
					; Not used here yet.
   ;----------------------------------------------------------------
  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
  				; Passed on to dataReader routine.
				
;/
;  headerTags = thisHeaderTags, $
;  dataTags = thisDataTags
     
  infolabel = "specObj::init >> "  
  debug = 0	;1

  ;-----------------------------------------------------------------------
  treatString = '; ' + systime()

  treatString = [treatString, "specObj = obj_new('specObj' $"]
  if (n_elements(thisDavePtr) GT 0) then begin
    treatString = [treatString, $
    	string(", davePtr = davePtr??? $")]
  endif
  if (n_elements(thisConvert) GT 0) then begin
    treatString = [treatString, $
    	string(", convert = ", strtrim(string(thisConvert), 2), " $")]
  endif
  if (n_elements(thisownerDavePtr) GT 0) then begin
    treatString = [treatString, $
    	string(", ownerDavePtr = ", thisownerDavePtr, " $")]
  endif

  if (n_elements(thisLabel) GT 0) then begin
    treatString = [treatString, $
    	string(", label = '", thisLabel, "' $")]
  endif
  if (n_elements(thisCodeSource) GT 0) then begin
    treatString = [treatString, $
    	string(", codeSource = '", thisCodeSource, "' $")]
  endif
;  if (n_elements(thisDavePtr) GT 0) then begin	; Hum...
;    treatString = [treatString, $
;    	string(", davePtr = thisDavePtr $")]
;  endif
  if (n_elements(thiInstrument) GT 0) then begin
    treatString = [treatString, $
    	string(", instrument = '", thisInstrument, "' $")]
  endif
  if (n_elements(thisPath) GT 0) then begin
    treatString = [treatString, $
    	 string(", path = '", thisPath, "' $")]
  endif
  if (n_elements(thisFilename) GT 0) then begin
    treatString = [treatString, $
    	string(", filename = '", thisFilename, "' $")]
  endif
  if (n_elements(thisDatareader) GT 0) then begin
    treatString = [treatString, $
    	string(", datareader = '", thisDataReader, "' $")]
  endif
  if (n_elements(thisWid) GT 0) then begin
    treatString = [treatString, $
    	string(", wID = '", strtrim(string(thiswID), 2), "' $")]
  endif
  if (n_elements(extraExtra) GT 0) then begin	; Might not work.
    treatString = [treatString, $
    	string(",_extra = '", extraExtra, "' $")]
  endif
  treatString = [treatString, ")"]
  ;-----------------------------------------------------------------------
  
  if (!debug) then begin
    print, treatString
    print
  endif

  success = 1
  
  if (n_elements(thisDataReader) gt 0) then begin
    dataReader = thisDataReader
  endif else begin
    datareader = ''
;    dataReader = 'func_icp_data_init'		; Defaults to triple axis reader.
  endelse

  if (n_elements(thisInstrument) gt 0) then begin
    currentInstrument = thisInstrument
  endif else begin
    currentInstrument = ''
  endelse

  if (n_elements(thisPath) gt 0) then begin
    currentPath = thisPath
  endif else begin
    currentPath = ''
  endelse
  if (n_elements(thisFilename) gt 0) then begin
    currentFilename = thisFilename
  endif else begin
    currentFilename = ''
  endelse
  currentFile = currentPath +  currentFilename

  if (n_elements(thisLabel) gt 0) then begin
    localLabel = thisLabel
  endif else begin
    localLabel = 'anonymous'
;     ['DVheader', 'anon', 'specObj::init']
  endelse

  if (n_elements(thisCodeSource) gt 0) then begin
    localCodeSource = thisCodeSource
  endif else begin
    localCodeSource = dataReader 
    if (localCodeSource EQ '') then localCodeSource = 'specObj::init'
  endelse

  localHeaderTags = ['DVheader', localLabel, localCodeSource]
  localDataTags =   ['DVdata', localLabel, localCodeSource]

  ;-----------------------------------------------
  BOOL_converted = keyword_set(thisConvert)
  message, /info, string("BOOL_converted = keyword_set(thisConvert)=", $
  	 BOOL_converted )
  if (BOOL_converted) then begin
    theErrMsg = ''
    ReadySpecObj = DavePtr_containsSpecObj(thisDavePtr, errMsg = theErrMsg)
    if (NOT ReadySpecObj) then  begin
      message, /info, string("DavePtr_containsSpecObj(thisDavePtr)=", $
    	DavePtr_containsSpecObj(thisDavePtr, errMsg = theErrMsg))
      print, "returned msg = ", theErrMsg
;      message, /info, string("returned msg = ", theErrMsg)
;      if (!debug) davePtr_show, thisDavePtr		; Debug
    endif

    if (ReadySpecObj) then  begin
;    if (DavePtr_containsSpecObj(thisDavePtr)) then begin
      message, /info, "SpecObj is already contained in davePtr. WARNING!"
      self = davePtr_getSpecObj(thisDavePtr, from='specObj::init')
     ;-----------------------------------------------
     self.ownerDavePtr = 'main'	; default value	
     if (n_elements(thisownerDavePtr) GT 0) then begin
       self.ownerDavePtr = thisownerDavePtr
     endif

      
      return, 1		; Already converted. Nothing to do.
    endif
  endif
  ;------------------------------------------------

  message, /info, string("INIT from file or daveptr w/o specObj") 


;  if (n_elements(thisHeadertags) gt 0) then begin
;    localHeadertags = thisHeadertags
;  endif else begin
;    localHeadertags =  ['DVheader', 'anon', 'specObj::init']
;  endelse
 
       if (!debug) then begin
	print
	message, /info, $
		"At the start. storeObj = obj_new('DAVE_TAGCONTAINER')"
        help, thisDaveptr
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
       endif
 
  ; Create empty containers  
  storeObj = obj_new('DAVE_TAGCONTAINER')
;Useless  obj_destroy, self.store 		; get rid of null object
  self.store = storeObj		;;obj_new('DAVE_TAGCONTAINER')

  ; fill with null objects the header and the data sections in focus.
;Useless  obj_destroy, self.focus 		; get rid of null object
  self.focus = obj_new('focusObj', target=self, container=storeObj)
;;  self.focus = obj_new('focusObj', container=self.store)

  ; Create the DAVE "header" section
  tempObj = obj_new('DAVE_TAGCONTAINER')
  self.focus -> set, headerObj = tempObj
  self.store -> add, tags = localHeaderTags, $
;;;  self -> add, tags = ['DVheader', 'anon', 'specObj::init'], $
  	contents = tempObj

  ; Create the DAVE "data" section
  tempObj = obj_new('DAVE_TAGCONTAINER')
  self.focus -> set, dataObj = tempObj
  self.store -> add, tags = localDataTags, $
;;;  self -> add, tags = ['DVdata', 'anon', 'specObj::init'], $
  	contents = tempObj
   
  if (debug) then begin
    print, infolabel, 'help, self.store'
    help, self.store
    print, infolabel, 'help, self.focus'
    help, self.focus
    print, infolabel, 'Exiting...'
  endif
  
  ; Note: do not initialize the DAVE pointer at this point.

  if (n_elements(thisWID) gt 0) then begin
    self.wID = thisWID	; widget ID asscoated with object self
  endif 

;Too much
;  if (debug) then begin
;    check_heap
;    print, infolabel, "Entering ::convert."
;  endif
  

  OK = 1  		
  BOOL_converted = keyword_set(thisConvert)
  
  convert_label:
  if (debug) then message, /info, string("BOOL_converted =", BOOL_converted)
  
  if (BOOL_converted) then begin
      if (!debug) then begin
	print
	message, /info, "OK = self -> convert(thisDavePtr, localCodeSource)"
        help,  thisDaveptr
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
       endif

      OK = self -> convert(thisDavePtr, localCodeSource)
      if (NOT OK) then begin
       success = 0
       if (debug) then message, /info, string("Failed converted") 
       goto, skip_init
      endif

      
      if (!debug) then begin
	print
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
       endif
  endif
  if (!debug) then message, /info, string("BOOL_converted=", BOOL_converted )

  if (NOT BOOL_Converted) then begin

    if (ptr_valid(thisDavePtr) ) then begin
      if (debug) then begin
         print, infolabel, "NOT BOOL_Converted but there is a davePtr"
 	 help, /heap
      endif
      if (n_elements(thisDavePtr) GT 0) then begin
;	davePtr_destroy, thisDavePtr		; Saves only the top ID#
	tempObj = davePtr_getSpecObj(thisDavePtr, from='specObj::init')
	if (self NE tempObj) then begin	; compare objRef ID's
	  davePtr_destroy, thisDavePtr, /destroySpecObj		; Saves only the top ID#
	  if (debug) then begin
 	    help, /heap
	  endif
	endif else begin
	  davePtr_destroy, thisDavePtr		; Saves only the top ID#
	endelse
	heap_free, (*thisDavePtr)	; Empty the rest. TMR. Sept 15, 2003.
      endif
    endif

    ; Have to have a data reader to initialize the file.
    if (strlen(dataReader) NE 0) then begin 
       if (!debug) then begin
	print
	message, /info, $
		"OK = self -> callFunction( dataReader, ...)"
        help, thisDaveptr
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
       endif
       
      OK = self -> callFunction( dataReader, self, $
      	(currentPath+currentFilename), $
	 _EXTRA = extraExtra) 
	 
      if (NOT OK) then begin
       success = 0
       if (debug) then message, /info, string("Failed converted") 
       goto, skip_init
      endif

       if (!debug) then begin
	print
	message, /info, $
		"post OK = self -> callFunction( dataReader, ...)"
        help, thisDaveptr
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
       endif

      if (currentInstrument NE '') then begin
        self -> add, /inHeader, $
    	  tags = ['qtyObj', 'SO_instr', 'specObj::init'], $
  	  contents=obj_new('qtyObj', name='SpecObj_instrument', qty=currentInstrument)
      endif
      if (currentPath NE '') then begin
        self -> add, /inHeader, $
    	  tags = ['qtyObj', 'SO_path', 'specObj::init'], $
  	  contents=obj_new('qtyObj', name='SpecObj_path', qty=currentPath)
      endif
      if (currentFilename NE '') then begin
        self -> add, /inHeader, $
    	  tags = ['qtyObj', 'SO_filename', 'specObj::init'], $
  	  contents=obj_new('qtyObj', name='SpecObj_filename', qty=currentFilename)
      endif
 


      if (n_elements(thisdavePtr) NE 0) then begin		; Aug 15, 2003
        if (!debug) then begin
	  print
	  message, /info, $
		"OK = self -> initDavePtr(davePtr=thisDaveptr, instrument=currentInstrument)"
          help, thisDaveptr
          if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	  help, /heap, /brief
         endif

        OK = self -> initDavePtr(davePtr=thisDaveptr, instrument=currentInstrument, $
		ownerDavePtr = thisownerDavePtr)
;Redundant        self -> setDavePtrID,  thisDavePtr
;        self.davePtr = thisDavePtr
      endif 

;      if (n_elements(thisdavePtr) EQ 0) then begin
;        thisDavePtr = ptr_new(/allocate_heap)		; TMR Aug 9, 2003
;      endif 
;      OK = self -> initDavePtr(davePtr=thisDaveptr, instrument=currentInstrument)
;      self.davePtr = thisDavePtr		; Aug 7, 2003
      
      if (!debug) then begin
        print
        help,  thisDaveptr
        if (ptr_valid(thisDaveptr)) then $
       		message, /info, string("(*thisdavePtr)=", (*thisdavePtr))
	help, /heap, /brief
	print
      endif
      
      self -> add, /inHeader,$
    	tags=['qtyObj', 'DavePtrTreatment', 'specObj::init'], $
   	contents=obj_new('qtyObj', name = 'Treatment history', $
		qty= ' ') 
    endif else begin		; if (strlen(dataReader) NE 0)
      if (strlen(currentFile) NE 0) then begin
         message, /info, " Use combo_restore instead! Bad idea here!"
	 return, 0
;         davePtr_restore, thisDavePtr, currentFile
;	 if (NOT davePtr_containsSpecObj(thisDavePtr)) then begin
;           BOOL_converted =  1
;           if (debug) then message, /info, string("Jumping to convert_label" ) 
;	   goto, convert_label
;	 endif else begin
;	   ;Need to swap out the object reference ID's
;	   obj_destroy,  self.focus
;	   obj_destroy, self.store
;	   tempObj = davePtr_getSpecObj(thisDavePtr)
;	   self.focus = tempObj -> get(/focus)
;	 endelse
      endif ;else begin		; if (strlen(currentFile) NE 0) 
      
;        ; Things are too minimal!
;        if (n_elements(thisdavePtr) EQ 0) then begin
;          thisDavePtr = ptr_new(/allocate_heap)		; TMR Aug 9, 2003
;        endif 
;        ; No Focus yet!
;        ;;; OK = self -> initDavePtr(davePtr=thisDaveptr, instrument=currentInstrument)
;;Redundant        self -> setDavePtrID,  thisDavePtr	; Aug 15, 2003.
;;        self.davePtr = thisDavePtr		; Aug 7, 2003
;      endelse

    endelse		; if (ptr_valid(davePtr))) then begin
    
    	;; else you can -> add whatever you want by hand.

;    if (n_elements(thisDavePtr) gt 0) then begin
;        OK = self -> initDavePtr(instrument = thisInstrument)
;    endif

  endif	 ; if (NOT BOOL_Converted) then begin
	
;  endelse

  self -> add, /inHeader,$
     	tags=['qtyObj', 'SpecObjTreatment', 'specObj::init'], $
   	contents=obj_new('qtyObj', name = 'Object treatment history', $
		qty= treatString ) 

  
;Too much  
;  if (debug) then begin
;    check_heap
;    print, infolabel, "Exiting ::convert."
;  endif
  		
  OK = 1
  if (obj_valid(self)) then OK = self -> scaleByMonitor()		; Do this by default
        
  skip_init:

  ;-----------------------------------------------
  self.ownerDavePtr = 'main'	; default value
  if (n_elements(thisownerDavePtr) GT 0) then begin
    self.ownerDavePtr = thisownerDavePtr
  endif
  ;-----------------------------------------------

  
  if (NOT success) then begin
    obj_destroy, self.store
    obj_destroy, self.focus
  endif
  

  return, success
;  return, (success  and OK)
   
end		; specObj::init

;=============================================================================
;+
; NAME:
; 	combo_obj_new
;
; PURPOSE:
;	A fake object initialization method.
;	Initialize the object, using multiple raw data input files or mutiple
;	davePtrs, using a particular method.
;	Keywords based on those used with obj_new('specobj', ..). 
;
; SEE ALSO:
;	 <LI><A HREF="#SPECOBJ::INIT,_arguments">SPECOBJ::INIT,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::CONCATENATE,_arguments">SPECOBJ::CONCATENATE,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::ZIP,_arguments">SPECOBJ::ZIP,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::SUM,_arguments">SPECOBJ::SUM,_arguments</A>
;-
;*
function combo_obj_new, $	; Returns an object of type ThisClass.
   thisMultiMethod, $		; Required method for combining the objects.
				; defaults to 'SUM'. 
				; Possible ones are  CONCATENATE, ZIP, & SUM.
   thisClass, $			; Required class type, probably 'SpecObj' of
   				; both the result and the intermediate objects.
				; Defaults to 'specobj'.
   ;----------------------------------------------------------------
   usingGroupLeader = thisUsingGroupLeader, $; If (thisUsingGroupLeader NE 0L) then 
   		; MIGHT pop up an interactive diglaog box. Sept 15, 2003. TMR.
;   usingGUI = thisUsingGUI, $	; If keyword_set(thisUsingGUI) then 
;   			; MIGHT pop up an interactive diglaog box.
   ;----------------------------------------------------------------
   davePtr = thisDavePtr, $	; Optionally passed ID from above.
   ;----------------------------------------------------------------
   ; Use keywords combos (listDavePtr,/convert) or 
   ; (listInstrument, listPath, listFilename, dataReader, convert=0) BUT NOT COMBINED!
   ;----------------------------------------------------------------
   convert = thisConvert, $		; Boolean keyword. If true, convert
   					; the DavePtr. Otherwise, read files.
					; Must be used with davePtr keyword.
   ownerDavePtr = thisownerDavePtr, $	; If 'local', cleanup destroys self.davePtr.
   					; If 'main', cleanup skips self.DavePtr
					; Defaults to 'main'.
   ;----------------------------------------------------------------
;   listDavePtr = thisListDavePtr, $	; Optionally create specObj from DAVE data
;   					; if filename not used. 
;					; Otherwise this is the davePtr handed from
;					; above.
   ;------------------------------------------------------------------
   ;  All *list* variables are arrays or single string converted to an array.
   ; e.g. listInstruments = ['bt4'] converts to ['bt4','bt4','bt4', ...]
   listInstrument=thisListInstrument, $		; Optional string. 
   					; Only used w/out davePtr keyword
   listPath = thisListPath, $			; String. path for ICP file.
   					; Only used w/out davePtr keyword.
   listFilename = thisListFilename, $		; String. ICP file to read from.
   					; Only used w/out davePtr keyword.
   dataReader = thisDataReader, $	; String. Data reader function's name
   					; Only used w/out davePtr keyword.
   ;----------------------------------------------------------------
   depField = thisDepField, $		; Optional.  
   					; depField = ['qtyObj', 'AvTemp', '*']
   					; Used by 'ZIP' method. 
   ;----------------------------------------------------------------
;Unused   Label = thisLabel, $			; String. label portion of tags.
;Unused   CodeSource = thisCodeSource, $	; String. CodeSource portion of tags.
   ;------------------------------------------------------------------
   wID = thisWID, $			; widget ID asscoated with object self
   errMsg = thisErrMsg			; Optional error message string
   ;------------------------------------------------------------------
;/
; Too hard and not needed!
;  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
;  				; Passed on to dataReader routine.

  debug = 0	;1
  debug2 = 0	;1
  
  thisErrMsg = ''
  infolabel = "combo_obj_new >> "
  
  if (n_elements(thisMultiMethod) EQ 0) then begin
    thisMultiMethod = 'SUM'	; Default value
  endif	

  if (n_elements(thisClass) EQ 0) then begin
    thisClass = 'specobj'	; Default value
  endif	
  


  BOOL_converted = keyword_set(thisConvert)
  
  nfiles = n_elements(thisListFilename)
  frgdObjs = make_array(nfiles, /Obj)

;  treatString = ''
  treatString = '; ' + systime()

 ; NEED [,,,]
 ; Put long lists as separate commands for macro.
;  if (n_elements(thisListDavePtr) GT 0) then begin
;    treatString = [treatString, string("listDavePtr = ", format_list(thisListDavePtr))]
;;    treatString = [treatString, string("listDavePtr = ", thisListDavePtr)]
;  endif  

  if (n_elements(thisListInstrument) GT 0) then begin
    treatString = [treatString, $
    	string("listInstrument = ", format_list(thisListInstrument))]
;    treatString = [treatString, string("listInstrument = ", thisListInstrument)]
  endif  
  if (n_elements(thisListInstrument) EQ 1) then begin
    listInstrument = make_array(nfiles, /string, value=thisListInstrument)
  endif else begin
    listInstrument = thisListInstrument
  endelse
  
  if (n_elements(thisListPath) GT 0) then begin
    treatString = [treatString, string("listPath = ", format_list(thisListpath))]
;    treatString = [treatString, string("listPath = ", thisListpath)]
  endif  
  if (n_elements(thisListPath) EQ 1) then begin
    listPath = make_array(nfiles, /string, value=thisListPath)
  endif else begin
    listPath = thisListPath
  endelse
  
  if (n_elements(thisListFilename) GT 0) then begin
    treatString = [treatString, $
    	string("listFilename = ", format_list(thisListFilename))]
;    treatString = [treatString, ("listFilename = ", thisListFilename)]
  endif  
  if (n_elements(thisListFilename) EQ 1) then begin
    listFilename = make_array(nfiles, /string, value=thisListFilename)
  endif else begin
    listFilename = thisListFilename
  endelse
  
  if (BOOL_converted) then begin
    message, /info, string("~~~~ Option for passing array of DavePTRs is not implimented")
    ; Commented out Sept 15, 2003. TMR.
;    for ifile = 0, (nfiles-1) do begin
;      frgdObjs[ifile] = obj_new(thisClass, $	; create from a davePtr
;    	davePtr = thisListDavePtr[ifile], $
;	/convert, $
;	_EXTRA = extraExtra)    
;;      if (!debug) then print, "***** ", ifile, " davePtr=",  thisListDavePtr[ifile], $
;;      	" valid=", ptr_valid(thisListDavePtr[ifile])
;    endfor
  endif else begin
   
   for ifile = 0, (nfiles-1) do begin
    frgdObjs[ifile] = obj_new(thisClass, $	; create from a data file 
        ownerDavePtr = 'local', $
  	label=		ListFilename[ifile], $
	instrument=	ListInstrument[ifile], $
  	path = 		listPath[ifile], $
	filename = 	ListFilename[ifile], $
	datareader=	thisDataReader, $
	_EXTRA = extraExtra)    
    endfor
  endelse

  if (debug2) then begin
    print
    print, infolabel
    for ifile = 0, (nfiles-1) do begin
      tempPtrID = (frgdObjs[ifile]) -> getDavePtr()
      message, /info, string(ifile, " frgdObj=", frgdObjs[ifile], $
		" davePtr =", tempPtrId)
    endfor
    print
    print
  endif
    
  if (n_elements(thisCodeSource) gt 0) then begin
    localCodeSource = thisCodeSource
  endif else begin
    localCodeSource = 'combo_obj_new'
    if (n_elements(thisDataReader) GT 0) then begin
      localCodeSource =  localCodeSource + "_" + thisDataReader
    endif
  endelse

  if (debug2) then begin
    print, "About to call minimal obj_new"
    help, davePtr
  endif
  
  specObj = obj_new('specObj',$		; empty container.
  	label= thisMultiMethod,  $
      	davePtr = davePtr, $		; Does not seem to make any difference.
;Sept16      	davePtr = davePtr, $
	ownerDavePtr = thisownerDavePtr, $
	codeSource = localCodeSource)	

  if (debug2) then help, davePtr
  if (debug) then begin
    specObj -> show
  endif
  
;  message, /info, string("Empty init. obj_valid(specObj)=", obj_valid(specObj) )

  if (debug) then begin
     for ifile = 0, (nfiles-1) do begin
      help, frgdObjs[ifile] 
    endfor 
  endif
  
  OK = call_method(thisMultiMethod, specObj, $
  		usingGroupLeader = thisUsingGroupLeader, $
;  		usingGUI = thisUsingGUI, $
    		frgdObjs, $
    		errMsg = thisErrMsg, $
		/noTreatmentHistory, $		; Suppress 
    		depField = thisDepField $	;only used by zip.
		)
  obj_destroy, frgdObjs

   if (n_elements(thisErrmsg) EQ 1) then begin
     if (strlen(thisErrmsg) GT 0) then begin
       print, thisErrMsg
     endif
   endif else begin
     if (n_elements(thisErrmsg) GT 1) then begin
       print, infolabel, "n_elements(thisErrmsg) =",thisErrmsg
       help,  thisErrmsg
       for i = 0, n_elements(thisErrmsg) do begin
         print, thisErrMsg[i]
       endfor
     endif
   endelse
   
   message, /info, string(OK, "=OK. After ",thisMultiMethod,$
   	 ". obj_valid(specObj)=", obj_valid(specObj))

  
  if (NOT OK) then begin
       print, infolabel, thisErrMsg
;       message, /info, thisErrMsg
       treatString = [treatString, string(" combo_obj_new(...FAILED on method" + $
	       thisMultiMethod+  " ...)")]
  endif else begin

; Sept 17, 2003
;  OK_dave = 1
     if (n_elements(thisdavePtr) NE 0) then begin		; Aug 15, 2003
        OK_DAVE = specObj -> initDavePtr(davePtr=thisDaveptr, $
		instrument=currentInstrument, $
		ownerDavePtr = thisownerDavePtr)
;;Redundant        specObj -> setDavePtrID,  thisDavePtr
      endif 

    ; Update the focus	; Commented out Sept 15, 2003.
;    OK_DAVE = specObj -> initDavePtr(davePtr=thisDaveptr, $
;    	instrument=currentInstrument)

    if (NOT OK_DAVE) then begin
       message, /info, thisErrMsg
       treatString = [treatString, string("OK_DAVE = specObj -> initDavePtr(", $
       		" ...FAILED after method" + $
	       thisMultiMethod+  "...) ")]
       goto, skip_treatment
    endif
     
;       resultStr =  string(theMultiMethod, " files ", short_fileList[0], " - ", $
;	short_fileList[(nfiles-1)])
;        message, /info, resultStr
;;Temp       davePtr_addTreatment, DavePtr, $
;;    	string(theMultiMethod, " files ", short_fileList[0], " - ", $
;;	short_fileList[(nfiles-1)])

    treatString = [treatString ,$
      	 string("specObj = combo_obj_new('" + thisMultiMethod + $
	 "', ",   thisClass, " $")]
    if (n_elements(thisDavePtr) GT 0) then begin
        treatString = [treatString, ", davePtr=davePtr $"]	; note: not defined
    endif
    if (n_elements(thisownerDavePtr) GT 0) then begin
      treatString = [treatString, $
    	string(", ownerDavePtr = ", thisownerDavePtr, " $")]
    endif
    ; Note: Since macros, if used, won't be interactive, comment out
    ; value of usingGroupLedaer in the history.
    if (n_elements(thisUsingGroupLeader) GT  0) then begin
        treatString = [treatString, string(";, usingGroupLeader=$", $
		thisUsingGroupLeader)]	
    endif
;    if (keyword_set(thisUsingGUI)) then begin
;        treatString = [treatString, ";, /usingGUI $"]	
;    endif

    if (BOOL_converted) then begin
      if (n_elements(thislistDavePtr) GT 0) then begin
        treatString = [treatString, ", davePtr=daveListPtr $"]
      endif
;      treatString = [treatString, ", convert=1)"]

    endif else begin

      if (n_elements(thisListPath) GT 0) then begin
        treatString = [treatString, ", listPath=listPath $"]
      endif
      if (n_elements(thisListFilename) GT 0) then begin
        treatString = [treatString, ", listFilename=listFilename $"]
      endif
      if (n_elements(thisListInstrument) GT 0) then begin
        treatString = [treatString, ", listInstrument=listInstrument $"]
      endif
;      treatString = [treatString, ",convert=0)"]
    endelse

; ADD THESE HERE!    
;   ;------------------------------------------------------------------
;Skip   wID = thisWID, $			; widget ID asscoated with object self
;   ;------------------------------------------------------------------
;SKIP  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
; 				; Passed on to dataReader routine.

      if (n_elements(thisDepField) GT 0) then begin
        help, format_list(thisdepField)
        treatString = [treatString, $
		string(", depField=", format_list(thisdepField), "$")]
      endif

;Unused
;      if (n_elements(thisLabel) GT 0) then begin
;        treatString = [treatString, $
;		string(", label='",thisLabel, "' $")]
;      endif

;      if (n_elements(thisCodeSource) GT 0) then begin
;        treatString = [treatString, $
;		string(", codeSource='",thisCodeSource, "' $")]
;      endif


    if (BOOL_converted) then begin
      treatString = [treatString, ", convert=1)"]
    endif else begin
      treatString = [treatString, ", convert=0)"]
    endelse
    
   SpecObj -> add, /inHeader,$
     	tags=['qtyObj', 'SpecObjTreatment', 'combo_obj_new'], $
   	contents=obj_new('qtyObj', name = 'Object treatment history', $
		qty= treatString ) 

;   OKTreat = SpecObj -> appendField( /inHeader, $
;  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
;	qty= treatString ) 
    skip_treatment:
  endelse
  
  if (NOT OK) then begin
    obj_destroy, specObj		; , /skipDavePtr		; return invalid object.
    return, obj_new()	; return a null object
  endif else begin
 
;    OK = SpecObj -> appendField($
;  	['qtyObj', 'SpecObjTreatment', 'specObj::init'], /inHeader,$
;	qty= treatString ) 
  endelse
  
  return, specObj

end		; combo_obj_new

;=============================================================================
;+
; NAME:
; 	combo_restore
;
; PURPOSE:
;	A fake object initialization method.
;	Initialize the object, using multiple DAVE save input files, 
;       using a particular method.
;	Keywords based on those used with obj_new('specobj', ..). 
;
; SEE ALSO:
;	 <LI><A HREF="#SPECOBJ::INIT,_arguments">SPECOBJ::INIT,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::CONCATENATE,_arguments">SPECOBJ::CONCATENATE,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::ZIP,_arguments">SPECOBJ::ZIP,_arguments</A>
;	 <LI><A HREF="#SPECOBJ::SUM,_arguments">SPECOBJ::SUM,_arguments</A>
;-
;*
function combo_restore, $	; Returns an object of type ThisClass.
   thisMultiMethod, $		; Required method for combining the objects.
				; defaults to 'SUM'. 
				; Possible ones are  SINGLE, CONCATENATE, ZIP, & SUM.
   thisClass, $			; Required class type, probably 'SpecObj' of
   				; both the result and the intermediate objects.
				; Defaults to 'specobj'.
   ;----------------------------------------------------------------
   usingGroupLeader = thisUsingGroupLeader, $; If (thisUsingGroupLeader NE 0L) then 
   		; MIGHT pop up an interactive diglaog box. Sept 15, 2003. TMR.
;   usingGUI = thisUsingGUI, $	; If keyword_set(thisUsingGUI) then 
;   			; MIGHT pop up an interactive diglaog box.
   ;----------------------------------------------------------------
   davePtr = thisDavePtr, $	; Optionally passed ID from above.
   ;----------------------------------------------------------------
   ; Use keywords combos (listDavePtr,/convert) or 
   ; (listInstrument, listPath, listFilename, dataReader, convert=0) BUT NOT COMBINED!
   ;----------------------------------------------------------------
   convert = thisConvert, $		; Boolean keyword. If true, convert
   					; the DavePtr. Otherwise, read files.
					; Must be used with davePtr keyword.
   ownerDavePtr = thisownerDavePtr, $	; If 'local', cleanup destroys self.davePtr.
   					; If 'main', cleanup skips self.DavePtr
					; Defaults to 'main'.
   ;----------------------------------------------------------------
;   listDavePtr = thisListDavePtr, $	; Optionally create specObj from DAVE data
;   					; if filename not used. 
;					; Otherwise this is the davePtr handed from
;					; above.
   ;------------------------------------------------------------------
   ;  All *list* variables are arrays or single string converted to an array.
   ; e.g. listInstruments = ['bt4'] converts to ['bt4','bt4','bt4', ...]
   listInstrument=thisListInstrument, $		; Optional string. 
   					; Only used w/out davePtr keyword
   listPath = thisListPath, $			; String. path for ICP file.
   					; Only used w/out davePtr keyword.
   listFilename = thisListFilename, $		; String. ICP file to read from.
   					; Only used w/out davePtr keyword.
;   dataReader = thisDataReader, $	; String. Data reader function's name
;   					; Only used w/out davePtr keyword.
   ;----------------------------------------------------------------
   depField = thisDepField, $		; Optional.  
   					; depField = ['qtyObj', 'AvTemp', '*']
   					; Used by 'ZIP' method. 
   ;----------------------------------------------------------------
;Unused   Label = thisLabel, $			; String. label portion of tags.
   CodeSource = thisCodeSource, $	; String. CodeSource portion of tags.
   ;------------------------------------------------------------------
   wID = thisWID, $			; widget ID asscoated with object self
   errMsg = thisErrMsg			; Optional error message string
   ;------------------------------------------------------------------
;/
; Too hard and not needed!
;  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
;  				; Passed on to dataReader routine.

  infolabel = "combo_restore"

  debug = 0
  debug2 = 0
  
  thisErrMsg = ''

  if (n_elements(thisMultiMethod) EQ 0) then begin
    thisMultiMethod = 'SINGLE'	; Default value
;    thisMultiMethod = 'SUM'	; Default value
  endif	

  if (n_elements(thisClass) EQ 0) then begin
    thisClass = 'specobj'	; Default value
  endif	
  
  
  BOOL_converted = keyword_set(thisConvert)
  
  nfiles = n_elements(thisListFilename)

  treatString = '; ' + systime()
;  treatString = ''

 ; NEED [,,,]
 ; Put long lists as separate commands for macro.
;  if (n_elements(thisListDavePtr) GT 0) then begin
;    treatString = [treatString, string("listDavePtr = ", format_list(thisListDavePtr))]
;  endif  

  if (n_elements(thisListInstrument) GT 0) then begin
    treatString = [treatString, $
    	string("listInstrument = ", format_list(thisListInstrument))]
  endif  
  if (n_elements(thisListInstrument) EQ 1) then begin
    listInstrument = make_array(nfiles, /string, value=thisListInstrument)
  endif else begin
    listInstrument = thisListInstrument
  endelse
  
  if (n_elements(thisListPath) GT 0) then begin
    treatString = [treatString, string("listPath = ", format_list(thisListpath))]
  endif  
  if (n_elements(thisListPath) EQ 1) then begin
    listPath = make_array(nfiles, /string, value=thisListPath)
  endif else begin
    listPath = thisListPath
  endelse
  
  if (n_elements(thisListFilename) GT 0) then begin
    treatString = [treatString, $
    	string("listFilename = ", format_list(thisListFilename))]
;    treatString = [treatString, ("listFilename = ", thisListFilename)]
  endif  
  if (n_elements(thisListFilename) EQ 1) then begin
    listFilename = make_array(nfiles, /string, value=thisListFilename)
  endif else begin
    listFilename = thisListFilename
  endelse
  
  if (n_elements(thisCodeSource) gt 0) then begin
    localCodeSource = thisCodeSource
  endif else begin
    localCodeSource = 'combo_restore'
    if (n_elements(thisDataReader) GT 0) then begin
      localCodeSource =  localCodeSource + "_" + thisDataReader
    endif
  endelse

 ;------------- SINGLE -----------------------------------------
 if ((nfiles EQ 1) OR (thisMultiMethod eq 'SINGLE')) then begin
     currentFile = listPath[0] + ListFilename[0]
     if (debug2) then begin
       plost=ptr_valid(count=nptrs)
       message, /info, string("Pre-restore: nptrs=", nptrs) 
     endif	
     davePtr_restore, thisDavePtr, currentFile, ownerDavePtr = thisownerDavePtr
     if (debug2) then begin
       plost=ptr_valid(count=nptrs)
       message, /info, string("Post-restore: nptrs=", nptrs) 
     endif	
     bool_converted = davePtr_containsSpecObj(thisDavePtr)
     message, /info, string($
     	"bool_converted = davePtr_containsSpecObj(thisDavePtr)", $
	bool_converted )
     if (bool_converted) then begin
       specObj = davePtr_getSpecObj(thisDavePtr)
     endif else begin     
       specObj = obj_new(thisClass, $	; create from a davePtr
	ownerDavePtr = thisownerDavePtr, $
    	davePtr = thisDavePtr, $
	/convert, $
	_EXTRA = extraExtra)    
     endelse

    if (NOT obj_valid(specObj)) then begin
      thisErrMsg = "Failed to initialize mostly specObj for single input"
      goto, the_end
    endif

     if (debug2) then begin

       plost=ptr_valid(count=nptrs)
       message, /info, string("getSpecObj: nptrs=", nptrs ) 
     endif	

   if (NOT bool_converted) then begin
     if (n_elements(thisListPath) EQ 1) then begin
       specObj -> add, /inHeader, $
    	  tags = ['qtyObj', 'SO_path', 'combo_restore'], $
  	  contents=obj_new('qtyObj', name='SpecObj_path', qty=thisListPath)
     endif 
     if (n_elements(thisListFilename) EQ 1) then begin
       specObj -> add, /inHeader, $
    	  tags = ['qtyObj', 'SO_filename', 'combo_restore'], $
  	  contents=obj_new('qtyObj', name='SpecObj_filename', qty=thisListFilename)
       specObj -> add, /inHeader, $
      	  tags = ['qtyObj', 'mainTitle', 'combo_restore'], $
  	  contents=obj_new('qtyObj', name='mainTitle', qty=thisListFilename)
     endif  
   endif
   
   if (debug) then message, /info, string("nfiles=1. method=", thisMultiMethod)
   if (debug) then help, specObj
   
   OK = 1
   goto, update_treatment
 endif

 ;------------- MULTIPLE -----------------------------------------

  frgdObjs = make_array(nfiles, /Obj)
  frgddavePtrs = make_array(nfiles, /Ptr)
  

  for ifile = 0, (nfiles-1) do begin
     
     currentFile = listPath[ifile] + ListFilename[ifile]		; Sept 9, 2003. TMR
     frgdDavePtrs[ifile] = ptr_new(/allocate)
	  ; /allocate is new. Aug 25, 2003. TMR.
     davePtr_restore, frgdDavePtrs[ifile], currentFile, ownerDavePtr = 'local'
     if (debug) then help, frgdDavePtrs[ifile]
     if (debug) then print, ifile, " valid DavePtr = ", $
     	(ptr_valid(frgdDavePtrs[ifile]) )
     bool_converted = davePtr_containsSpecObj(frgdDavePtrs[ifile])
     message, /info, string($
     	"bool_converted = davePtr_containsSpecObj(frgdDavePtrs[ifile])", $
	bool_converted )
     if (bool_converted) then begin
       frgdObjs[ifile] = davePtr_getSpecObj(frgdDavePtrs[ifile])
;       frgdObjs[ifile] -> setownerDavePtr('local')
     endif else begin     
       frgdObjs[ifile] = obj_new(thisClass, $	; create from a davePtr
        ownerDavePtr = 'local', $
    	davePtr = frgdDavePtrs[ifile], $
	/convert, $
;	convert=bool_converted, $
	_EXTRA = extraExtra)    
     endelse
  endfor   

  if (debug2) then begin
    print
    print, infolabel
    for ifile = 0, (nfiles-1) do begin
      tempPtrID = (frgdObjs[ifile]) -> getDavePtr()
      message, /info, string(ifile, " davePtr =", frgdDavePtrs[ifile], $
      	tempPtrId)
    endfor
    print
    print
  endif
    
 
 ;------------- NOT SINGLE -----------------------------------------

  specObj = obj_new('specObj',$		; empty container.
  	label= thisMultiMethod,  $
	ownerDavePtr = thisownerDavePtr, $
      	davePtr = davePtr, $
	codeSource = localCodeSource)	
  if (NOT obj_valid(specObj)) then begin
    thisErrMsg = "Failed to initialize mostly empty specObj for multiple inputs"
    goto, the_end
  endif
  
;  message, /info, string("Empty init. obj_valid(specObj)=", obj_valid(specObj) )

  OK = call_method(thisMultiMethod, specObj, $
  		usingGroupLeader = thisUsingGroupLeader, $
;  		usingGUI = thisUsingGUI, $
    		frgdObjs, $
    		errMsg = thisErrMsg, $
		/noTreatmentHistory, $		; Suppress 
    		depField = thisDepField $	;only used by zip.
		)

  if (NOT obj_valid(specObj)) then begin
    thisErrMsg = [thisErrMsg, $
    	string("Failed call to ", thisMultiMethod)]
    goto, the_end
  endif
  
;  obj_destroy, frgdObjs		; Moved to the end
  
  message, /info, string(OK, "=OK. After ",thisMultiMethod,$
   	 ". obj_valid(specObj)=", obj_valid(specObj))

  if (NOT OK) then begin
       message, /info, thisErrMsg
       treatString = treatString + " combo_obj_new(...FAILED on method" + $
	       thisMultiMethod+  " ...)"
    thisErrMsg = [thisErrMsg, $
    	string(" combo_obj_new(...FAILED on method" + $
	       thisMultiMethod+  " ...)")]
       goto, the_end
  endif else begin
    ; Update the focus
    message, /info, "Update the focus"
    
    message, /info, "Update the davePtr"
    if (debug) then specobj -> show, /recursive

;   Sept 17, 2003
;    OK = 1
    OK = specObj -> initDavePtr(davePtr=thisDaveptr, $
    	instrument=currentInstrument, $
	ownerDavePtr = thisownerDavePtr)
;;Redundant    specObj -> setDavePtrID,  thisDavePtr


    message, /info, string("Finished updating the davePtr obj_valid(specObj)=",$
    	 obj_valid(specObj))
  endelse
  
  ;------------ GOTO here for both SINGLE and MULTIPLE! -----------------------
  update_treatment:

     if (debug2) then begin
       plost=ptr_valid(count=nptrs)
       message, /info, string("update_treatment: nptrs=", nptrs ) 
     endif	
  
  if (NOT OK) then begin
       message, /info, thisErrMsg
       treatString = treatString + " combo_obj_new(...FAILED on method" + $
	       thisMultiMethod+  " ...)"
      goto, the_end
  endif
     
;Aug25  update_treatment:
    treatString = [treatString ,$
      	 string("specObj = combo_restore('" + thisMultiMethod + $
	 "', ",   thisClass, " $")]
    if (n_elements(thisDavePtr) GT 0) then begin
        treatString = [treatString, ", davePtr=davePtr $"]	; note: not defined
    endif
    ; Note: Since macros, if used, won't be interactive, comment out
    ; value of usingGroupLedaer in the history.
    if (n_elements(thisUsingGroupLeader) GT  0) then begin
;    if (thisUsingGroupLeader NE 0L) then begin
        treatString = [treatString, string(";, usingGroupLeader=$", $
		thisUsingGroupLeader)]	
    endif
    if (n_elements(thisownerDavePtr) GT 0) then begin
      treatString = [treatString, $
    	string(", ownerDavePtr = ", thisownerDavePtr, " $")]
    endif

;    if (keyword_set(thisUsingGUI)) then begin
;        treatString = [treatString, ";, /usingGUI $"]	
;    endif

    if (BOOL_converted) then begin
;      if (n_elements(thislistDavePtr) GT 0) then begin
;        treatString = [treatString, ", davePtr=daveListPtr $"]
;      endif
;;      treatString = [treatString, ", convert=1)"]

    endif else begin

      if (n_elements(thisListPath) GT 0) then begin
        treatString = [treatString, ", listPath=listPath $"]
      endif
      if (n_elements(thisListFilename) GT 0) then begin
        treatString = [treatString, ", listFilename=listFilename $"]
      endif
      if (n_elements(thisListInstrument) GT 0) then begin
        treatString = [treatString, ", listInstrument=listInstrument $"]
      endif
;      treatString = [treatString, ",convert=0)"]
    endelse

; ADD THESE HERE!    
;   ;------------------------------------------------------------------
;Skip   wID = thisWID, $			; widget ID asscoated with object self
;   ;------------------------------------------------------------------
;SKIP  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
; 				; Passed on to dataReader routine.

      if (n_elements(thisDepField) GT 0) then begin
        help, format_list(thisdepField)
        treatString = [treatString, $
		string(", depField=", format_list(thisdepField), "$")]
      endif

;Unused
;      if (n_elements(thisLabel) GT 0) then begin
;        treatString = [treatString, $
;		string(", label='",thisLabel, "' $")]
;      endif

;      if (n_elements(thisCodeSource) GT 0) then begin
;        treatString = [treatString, $
;		string(", codeSource='",thisCodeSource, "' $")]
;      endif


    if (BOOL_converted) then begin
      treatString = [treatString, ", convert=1)"]
    endif else begin
      treatString = [treatString, ", convert=0)"]
    endelse
   
   message, /info, "specObj->remove, position = ['*','SpecObjTreatment','*']"
   ; remove this if it exists:
   specObj -> remove, $
   	position = ['*', 'SpecObjTreatment', '*'], $
	/inHeader, $
	/destroy

   ; replace it with up to date one:
   message, /info, "specObj->add ...tags=['qtyObj', 'SpecObjTreatment', 'combo_obj_new'] "
   theErrMsg = ''
   SpecObj -> add, /inHeader,$
     	tags=['qtyObj', 'SpecObjTreatment', 'combo_obj_new'], $
   	contents=obj_new('qtyObj', name = 'Object treatment history', $
		qty= treatString ), $
	errMsg = theErrmsg 
   if (n_elements(theErrMsg) GE 1) then begin
     if (strlen(theErrMsg[0]) NE 0) then begin
       print, infolabel, theErrmsg
     endif
   endif
   
  
  if (NOT OK) then begin
    obj_destroy, specObj	; return invalid object.
;Sept17    obj_destroy, specObj, /skipDavePtr		; return invalid object.
;    specObj = obj_new()			; Or return a null object?
  endif 	

     if (debug2) then begin
       plost=ptr_valid(count=nptrs)
       message, /info, string("end: nptrs=", nptrs ) 
     endif	

  the_end:  

  if (n_elements(frgdObjs) GT 0) then begin
    obj_destroy, frgdObjs
  endif

  return, specObj

end		; combo_restore

;=============================================================================
; NAME:
; 	specObj::clone
;
; PURPOSE:
;	Clone the object.
;-
;*
function specObj::clone		; Returns a copy of self
;/

				
  infolabel = "specObj::clone >> "  
  debug = 0
  

 clone = obj_new('specObj', WID=0L)
 newStore = ((self.store) -> clone())
 clone -> setStore, newStore


 ; Now do the focus.
 OK = self -> getFocus(headerObj = oldHeaderObj, dataObj = oldDataObj)
  if (OK) then begin
   OK = (self.store) -> IsContained(oldHeaderObj, position=posHeader)
   OK = (self.store) -> IsContained(oldDataObj, position=posDATA)
;   print, infolabel, " Header position=", posHeader
;   print, infolabel, " Data position=", posData
 endif else begin
   posHeader = -2
   posData = -2
 endelse
 
 if (posHeader GE 0) then begin 
   newHeaderObj = clone -> get(position = posHeader, /inStore)
;   print, infolabel, " New header obj = " 
;   newHeaderObj -> show
 endif else begin
   newHeaderObj = obj_new()
   print, infolabel, " Can't find Header location. Header position=", posHeader
 endelse
 
 if (posData GE 0) then begin 
   newDataObj = clone  -> get(position = posData, /inStore)
;   print, infolabel, " New data obj  = " 
;   newDataObj -> show
 endif else begin
   newDataObj = obj_new()
   print, infolabel, " Can't find Data location. Data position=", posData
 endelse
 
 
 focusClone = ((self.focus) -> clone($
 	target = clone, $		; Sept 2, 2003. TMR
 	container = newStore, $
 	headerObj = newHeaderObj, dataObj = newDataObj))
 
 clone -> setFocus, focusObj=focusClone
 
 OK = clone -> getField(['qtyObj', 'SO_instr', 'specObj::init'], qty=localInstrument)
 if (NOT OK) then localInstrument = ''
 
 ; Note: theDavePtr is not defined.
 theOwnerDavePtr = 'local'			; until otherwise over-ridden.
 OK = clone -> initDavePtr(davePtr = theDavePtr, instrument=localInstrument, $
	 ownerDavePtr = thisownerDavePtr)
;Redundant clone -> setDavePtrID,  theDavePtr

 return, clone
   
end		; specObj::clone

;=========================================================================================
;*
function specObj::concatenate, $		; Returns 0/1 for failure/success
	appendages, $			; Array of specObj.
	depField = thisdepField, $	; Optional keyword. Ignored.
	noTreatmentHistory = thisNoTreatmentHistory, $
 	usingGroupLeader = thisUsingGroupLeader, $;  IGNORED
;	usingGUI = thisUsingGUI, $	; IGNORED keyword_set(thisUsingGUI) for now
	errMsg = thisErrMsg		; Optional error message string.
;/


 infolabel = "specObj::concatenate >> "
 debug = 0	;1 
  
 nObjs = n_elements(appendages)	; How many objects to concatenate. Min is 2.
 if (nObjs LT 2) then begin
   thisErrMsg = string(infolabel, 'Need at least 2 objects. nObjs=',nObjs) 
   return, 0
 endif
 
 firstAppendage = appendages[0]
  
  ;------------------------------------------------------------------
;Wrong  theownerDavePtr = self.ownerDavePtr		; Probably 'main'. Sept 17, 2003.
;Wrong  theDavePtrID = self.davePtr			; Sept 17, 2003.
 obj_destroy, self
;Sept17 obj_destroy, self, /skipDaveptr
 self  = firstAppendage -> clone()	; Clone the object
;Wrong self -> setDavePtrID, theDavePtrID, ownerDavePtr=theownerDavePtr 	; Sept 17, 2003.
 ;------------------------------------------------------------------
      
 ;***** concatenate the info in the data section *****************************
 nData = appendages[0] -> count(/inData)
 mismatch_arr = make_array(nData, /int)
 firstAppendage -> getLabel, /inData, label=label_arr, $
 	class=class_arr, codeSource=codeSource_arr

 thisErrMsg = '' 
 for i = 0, (nData-1) do begin
   Ok = firstAppendage -> gettagfield(i, /inData, tags = nthTags)

   for iAppend = 1, (nObjs -1) do begin    
;     message, /info, string(iappend, "=nth concatenate i=", i)
     Ok = firstAppendage -> gettagfield(i, /inData, tags = nthTags)
;     print,' tags=', nthTags
     tempObj = (appendages[iAppend]) ->  get(position=nthTags, /inData)
;     tempObj = (appendages[iAppend]) ->  get(position=i, /inData)
     if (obj_valid(tempObj)) then begin
       OK = self ->  appendField(nthTags, /inData, obj=TempObj)
       obj_destroy, tempObj
     endif else begin
       mismatch_arr(i) = 1
       self -> remove, position=nthTags, /inData, /destroy
       ; Remove non-matching fields rather than failing entirely.
       thisErrMsg = [thisErrMsg, string ('Mismatch with ', $
       	'[', class_arr[i], ',', label_arr[i], ',', codeSource_arr[i], '] ')]
     endelse
   endfor
 endfor

 any_mismatch = (round(total(mismatch_arr)) gt 0) 
 
 if (any_mismatch) then begin
   thisErrMsg = infolabel + 'Mismatch with ' + thisErrMsg 
 endif
 

; thisErrMsg = infolabel
; if (any_mismatch) then begin
;;   obj_destroy, self
;;   self = obj_new()
;   newErrMsg = 'Mismatch with '
;   for i = 0, (nData-1) do begin
;     if (mismatch_arr[i]) then begin
;       thisErrMsg = newErrMsg + string ($
;       	'[', class_arr[i], ',', label_arr[i], ',', codeSource_arr[i], '] ')
;     endif
;   endfor
;;   return, 0
; endif
;  


 ;***** concatenate differing info in the header section, selectively *****************************
 nData = appendages[0] -> count(/inHeader)
 for i = 0, (nData-1) do begin
   Ok = firstappendage -> gettagfield(i, /inHeader, tags = nthTags)
;   print,' tags=', nthTags

   depLabel = nthTags[1]
      
   if ((depLabel EQ 'DavePtrTreatment') OR (depLabel EQ 'SpecObjTreatment')) then begin
     print, infolabel, "Destroying ", depLabel
     self -> remove, /inHeader, position =  nthTags, /destroy 
     goto, skip_treatment
   endif
   
   OK = firstappendage -> getField(i, /inHeader, $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	rank = depRank, dim=depDim, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
   depPendent = 'I'
;   depPendent = ' '
;   OK = firstappendage  -> getTagField(i, /inHeader, $
;   	pendent = depPendent)
;   if (depPendent EQ ' ') then begin
;     depPendent = 'I'	; independent
;;     print, infolabel, depPendent, "=depPendent. tags = ", nthTags
;   endif

  ;-- Create arrays for the concatenated/zipped dependent variable
  depQty = dep1Qty
  if (n_elements(dep1Err) GT 0) then begin
    depErr = dep1Err
  endif
  if (n_elements(dep1NErr) GT 0) then begin
    depNErr = dep1NErr
  endif
  if (n_elements(dep1Legend) GT 0) then begin
    depLegend = dep1Legend
  endif

   qty_const = 1
   for iZip = 1, (nObjs -1) do begin 
     dep1Qty_prev = dep1Qty
     OK = (appendages[iZip]) -> getField(i, /inHeader, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
;     print, infolabel, 'Header: ', i, izip
;     print, "Header: tot=", round(total( (dep1Qty_prev EQ dep1Qty))), $
;     	" n = ",  n_elements(dep1Qty)
     if (depTName EQ 'STRING') then begin
       if ( round(total( (dep1Qty_prev EQ dep1Qty)))  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endif else begin
        if ( round(total( (abs(dep1Qty_prev - dep1Qty) LE 1.0e-13)) )  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endelse
;     print, infolabel, 'Header: ', i, izip, depName, " qty_const = ", qty_const
     depQty = [[depQty], [dep1Qty]]
     if (n_elements(dep1Err) GT 0) then begin
       depErr = [[depErr], [dep1Err]]
     endif
     if (n_elements(dep1NErr) GT 0) then begin
       depNErr = [[depErr], [dep1NErr]]
     endif
     if (n_elements(dep1Legend) GT 0) then begin
       depLegend = [[depLegend], [dep1Legend]]
     endif
   endfor

   if (NOT qty_const) then begin
   ; Remove the original field from the header section
;     print, infolabel, "Removing orig field from header section ", $
;   	  nthTags	;;;Wrong ['qtyObj', depLabel, '*']
     self -> remove, /inHeader, position =  nthTags, /destroy 

     ;--- If the results is a 1 X N array, it needs to be transposed to be properly 1-D
     theDims =  size(depQty, /dimensions)
;     print, infolabel, depName, " dim = ", theDims, " rank=", size(depQty, /n_dimensions)
     if (size(depQty,/n_dimensions) EQ 2) then begin
       if (theDims[0] EQ 1) then begin
         depQty = transpose(depQty)
	 if (n_elements(depErr) GT 0) then begin
	   depErr = transpose(depErr)
	 endif
	 if (n_elements(depNErr) GT 0) then begin
	   depNErr = transpose(depNErr)
	 endif
	 if (n_elements(depLegend) GT 0) then begin
	   depLegend = transpose(depLegend)
	 endif
;         print, infolabel, depName, " dim = ", theDims, $
;	 	" Trans'd dim = ",  size(depQty, /dimensions)
       endif
     endif

     ; Add new vector to the data section as long as it is a plottable type. 
     ; Hope that there is no name clash with the concatenated/zipped one added previously! 
     depName = depName + '_concatenate'
     if (depTName EQ 'STRING') then begin
       self -> add, /inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::Concatenate'], $
;         tags = ['qtyObj', string(depLabel, '_concatenate'), 'specObj::Concatenate'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
      endif else begin
       self -> add, /inData, $
         tags = ['qtyObj', depLabel, 'specObj::Concatenate'], $
;         tags = ['qtyObj', string(depLabel, '_concatenate'), 'specObj::Concatenate'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
       OK = self  -> setTagField( /inData, $
      		['qtyObj', depLabel, 'specObj::Concatenate'], $
;      		['qtyObj', string(depLabel, '_Concatenate'), 'specObj::Concatenate'], $
   		pendent = depPendent)
      endelse

    endif else begin
      ; The Concatenated/zipped field is trivial, so not worth saving.
    endelse

   ;-------- Make temporary variables all undefined, so that 
   ;-------- add method only uses those keywords with defined variables.
   DAVE_undefine, dep1Qty, dep1Err, dep1NErr, dep1Legend
   DAVE_undefine, depQty, depErr, depNErr, depLegend

   skip_treatment:
   
 endfor

 ;*********************************************************************************
   self -> add, /inHeader, $
        tags = ['qtyObj', '#OfConcatenatedFiles', 'specObj::Concatenate'], $
     	contents=obj_new('qtyObj', name='#OfConcatenatedFiles', qty= nObjs )

   ;--- Store updated ICP scan type -----------------------------------
   ; Note: Only ICP-dependent command here.
;   if (OK_ICP) then begin
      OK = self -> setField(/inHeader, $
        ['qtyObj', 'type', '*'], qty = 'concatenate')	; for appended/zip 
;   endif

 ;---- Update the specObj history. Has already been removed from the
 ; ---- header section. Needs work. -------------------------

 if (NOT keyword_set(thisNoTreatmentHistory)) then begin
   treatString = "OK = specObj -> Concatenate(  $"
   treatString = [treatstring,  " [appendages], $"]	; Needs fixing
   treatString = [treatString, ")"]
   OK = SpecObj -> appendField( /inHeader, $
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
	qty= treatString ) 

;  self -> add, /inHeader,$
;  	tags = ['qtyObj', 'SpecObjTreatment', 'specObj::init'], $
;        contents=obj_new('qtyObj',  $
;	  name = 'Object treamtment history',$
;	  qty= treatString ) 

 endif
  
 
 OK = self -> getField( /inHeader, $
      ['qtyObj', 'filename', '*'], $
      qty=theFilenames)
 nlen = n_elements(theFilenames) 
 if (nlen EQ 0) then theName = "<IN_MEMORY>"
 if (nlen EQ 1) then begin
   theName = "<IN_MEMORY:" + theFilenames + ">"
 endif else begin
   theName = "<IN_MEMORY:" + theFilenames[0] + '...' + theFilenames[(nlen-1)] + ">"
 endelse
 
 OK = self -> setField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      name='SpecObj_filename', $	; over-riden, not concatenated
      qty=theName)
 OK = self -> setTagField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      codeSource='specObj::Concatenate')
      
; self -> add, /inHeader, $
;      tags = ['qtyObj', 'filename', 'specObj::Concatenate'], $
;      contents=obj_new('qtyObj',  name = 'filename', $
;      	qty="<IN_MEMORY>")

 ;----- Not bothering to change the focus. ---------------------- 
  
 return, 1
end		; specObj::Concatenate

;=========================================================================================
;*
function specObj::CompactConcatenate, $		; Returns 0/1 for failure/success
		; Combination of methods concatenate and compact.
	appendages, $			; Array of specObj.
	depField = thisdepField, $	; Optional keyword. Passed to compact
	noTreatmentHistory = thisNoTreatmentHistory, $
        usingGroupLeader = thisUsingGroupLeader, $; If (thisUsingGroupLeader NE 0L) then 
   		; MIGHT pop up an interactive diglaog box. Sept 15, 2003. TMR.
;	usingGUI = thisUsingGUI, $	; IGNORED keyword_set(thisUsingGUI) for now
	errMsg = thisErrMsg		; Optional error message string.
;/

  debug = 0	;1
 
   if (debug) then message, /info, string("Calling self -> Concatenate")
  thisErrMsg = ''
  OK = self -> Concatenate( $		; Returns 0/1 for failure/success
	appendages, $			; Array of specObj.
	depField = thisdepField, $	; Optional keyword. Ignored.
	noTreatmentHistory = thisNoTreatmentHistory, $
  	usingGroupLeader = thisUsingGroupLeader, $
;	usingGUI = thisUsingGUI, $	; IGNORED keyword_set(thisUsingGUI) for now
	errMsg = thisErrMsg)

  if (OK) then begin
    thisErrMsg2 = ''
    if (debug) then message, /info, string("Calling self -> compact")
    OK = OK AND self-> compact( $		; Returns 0/1 for failure/success
;	sortField = thisdepField, $	; Quasi-Required keyword. 
	noTreatmentHistory = thisNoTreatmentHistory, $
  	usingGroupLeader = thisUsingGroupLeader, $
;	usingGUI = thisUsingGUI, $	; IGNORED keyword_set(thisUsingGUI) for now
	errMsg = thisErrMsg2)		; Optional error message string.
    if (strlen(thisErrMsg2) GT 0) then begin
      thisErrMsg = [thisErrMsg, thisErrmsg2]
    endif
  endif

  return, OK
   
end		;specObj::CompactConcatenate 


;=========================================================================================
;*
function specObj::compact, $		; Returns 0/1 for failure/success
	; Sorts all fields in the data section by the values of sortField, if dimensions
	; match. Then averages all duplicate values.
	; Assumes triple axes data, with all fields in data section 1D and the same length.
	; Does not test for numbers that are very close, eq. no tolerance in the test.
	sortField = thisSortField, $	; Recommended keyword (not quite required!). 
			; If not provided, uses 1st data field with pendent='D'.
	noTreatmentHistory = thisNoTreatmentHistory, $
        usingGroupLeader = thisUsingGroupLeader, $; If (thisUsingGroupLeader NE 0L) then 
   		; MIGHT pop up an interactive diglaog box. Sept 15, 2003. TMR.
;	usingGUI = thisUsingGUI, $	; If keyword_set(thisUsingGUI) then 
;					; GUI pops up to modify thisIntensity
	errMsg = thisErrMsg		; Optional error message string.
;-


;;	noTreatmentHistory = thisNoTreatmentHistory, $
;;	rawCounts = thisRawCounts, $	; Defaults to ['qtyObj', 'COUNTS', '*']
;;	rawTime = thisRawTime, $	; Defaults to ['qtyObj', 'MIN', '*']
;;	rawMonitor = thisRawMonitor, $	; Defaults to ['qtyObj', 'MONITOR', '*'] in header
;;	thisByMonitor, $ 		; Optional new value of monitor. Defaults to 1.
;	newIntensity = thisNewIntensity, $ ; Defaults to ['qtyObj', 'INTENSITY', '*']
;	newTime = thisNewTime		; Defaults to ['qtyObj', 'TIME', '*']

  debug = 0
  debug2 = 1
  
  thisErrMsg = ''

  infolabel = "specObj::compact >> "
  
  OK = 1
   
  
  sortField_Used = 1
  if (n_elements(thisSortField) EQ 0) then begin
    ; Which variables are independent or dependent or constant. 
    self -> makeLists, $	; Returns lists of tags regarding fields in data section
	independList = theIndependentList, $	; Output keyword. 
					; List of independent fields in data section.
	nIndep = theNIndep, $		; Number in independent list.
	dependList = theDependentList, $; Output keyword. 
					; List of dependent fields in data section.
	nDep = theNDep, $		; Number in dependent list.
	constantList = theConstantList, $; Output keyword. 
					; List of constant fields in data section.
	nConst = theNConst, $		; Number in constant list.
	nOther = theNOther		; number not characterized by 'I', 'D' or 'C'.


    thisSortField = theIndependentList[0:2]	; pick off the first one.
    sortField_Used = 0
  endif

  ;------------------------------------------------------------------
  if (debug2) then begin
      print, '~~~~~~  ', infolabel, " n_elements(usingGroupLeader) = ", $
      	 n_elements(thisusingGroupLeader)
  endif

  if (n_elements(thisUsingGroupLeader) NE 0) then begin

    if (debug2) then begin
      print, '~~~~~~  ', infolabel, " usingGroupLeader = ",  thisusingGroupLeader
    endif

   if (thisUsingGroupLeader NE 0) then begin
;  if (keyword_set(thisUsingGUI)) then begin
    ; Allow the user to change the selection thisSortField from the GUI if in interactive mode.
    message, /info, string("Orig field = ", format_list(thisSortField))
    ; This updates the zfocus with the new selection
    whereGUIObj = obj_new('dialog_ow_where', self, $
        groupLeader = thisUsingGroupLeader, $
    	from='data', $
	label = thisSortField[1], $
	xyzFocus = 'x')
    OK = whereGUIObj -> createWidgets()

    message, /info, string("Old sort tags = ", $
    	format_list(thisSortField))
	
    ; Copy over the new zfocus to the string array thisIntensity
    OK = self -> getFocus(xtags = thisSortField)

    message, /info, string(OK, "=OK. New sorting tags and focus = ", $
    	format_list(thisSortField))

   if (obj_valid(whereGUIObj)) then begin 
     obj_destroy, whereGUIObj
   endif
  endif
 endif 

  if ((n_elements(thisSortField) EQ 0) OR $
  	(round(total(strcmp(thisSortField, ['', '', '']))) EQ 3) ) then begin
    thisErrMsg = "Requires one independent field to sort by"
    message, /info, "Requires one independent field to sort by"
    return, 0
  endif 

 ;----------------------------------------------------------------------------------- 
 
;  if (debug) then message, /info, string("thisSortField = ", format_list(thisSortField))
  
  OK = self -> getField(thisSortField, /inData, $
  	qty = sortqty, $
        rank = sortrank, $
	dim = sortdim)

  if (debug) then begin
    message, /info, string(sortField_Used, $
	"=used. sorting field = ", format_list(thisSortField))
   message, /info, string( OK, "=OK. Sorting rank, dim=", sortRank, $
	      ' ', format_list(sortDim)) 
   print
  endif
  


  if (NOT OK) then begin
    thisErrMsg = "Did not find the sorting field " + format_list(thisSortField)
    goto, the_end
  endif

  nicesort, qty=sortQty, $
	index = qIndex, $
	start = iStart, $
	stop = iStop, $	
	dupl = qDupl

  nuniq = n_elements( iStart)
     
  nFields= self-> count(/inData)  
  OK = 1
  for iField = 0, (nFields-1) do begin
    DAVE_undefine, targetQty, targetErr, targetNErr, targetLegend
    DAVE_undefine, newQty, newErr, newLegend	; newNerr not used.
    iOK = self -> getField(iField, /inData,$
  	qty = targetqty, $
        err=targeterr, nerr=targetnerr, $
	legend = targetlegend, $
        rank = targetrank, $
	dim = targetdim)    

    if (eqarrays(sortdim, targetdim)) then begin
      if (debug) then begin
        print, ifield, "Size matches. ", format_list(sortdim)
	print, "Sorted now, but before duplicity is checked:"
      endif
      ; This is useful if only need to sort but no duplicates. Otherwise,
      ; we will write over the arrays.
      newqty =  targetqty[qIndex[istart]]		; Copy all unique ones over, in sorted order
      if (n_elements(targeterr) GT 0) then begin 
        weight = make_array(/double, nuniq)
        ; new errors will be all symmetric. Don't define newnerr.
        if ((n_elements(targeterr) GT 0) AND (n_elements(targetnerr) EQ 0)) then begin 
          newerr =  targeterr[qIndex[istart]]		; Copy all unique ones over, in sorted order
	endif else begin
          if ((n_elements(targeterr) GT 0) AND (n_elements(targetnerr) GT 0)) then begin 
	    newerr = 0.5*(targeterr[qIndex[istart]] + targetnerr[qIndex[istart]]) 
	  endif
	endelse
        if (debug) then begin
          print, "newqty=", newqty, " +- ", newerr
        endif
      endif else begin
        if (debug) then begin
          print, "newqty=", newqty
        endif
      endelse
      if (n_elements(targetlegend) GT 0) then begin
        newlegend = targetlegend[qIndex[istart]]
      endif


      if (debug) then begin
        help, newqty
;	print, ifield, " newqty = ", newqty
	help, newerr
	help, newlegend
	print
      endif
      
      ;-----------------------------------------	  
      for i = 0, (nuniq-1) do begin
        ; Check for duplicates. Average duplicates as appropriate.
       if (qDupl[i] GT 1) then begin
	    i1 = istart[i]
	    i2 = istart[i] + (qDupl[i]-1)
            if (n_elements(targeterr) EQ 0) then begin 
	      tgterr_arr = make_array(/double, qDupl[i], value = 1.0d0)
	    endif else begin
              if ((n_elements(targeterr) GT 0) AND $
	      	(n_elements(targetnerr) EQ 0)) then begin 
	        tgterr_arr = targeterr[qindex[i1:i2]]
	      endif else begin
                if ((n_elements(targeterr) GT 0) AND (n_elements(targetnerr) GT 0)) then begin 
	          tgterr_arr = 0.5*(targeterr[qindex[i1:i2]] $
			+ targetnerr[qindex[i1:i2]]) 
		endif
	      endelse
	    endelse
	    weight_arr = 1.0d0 / (tgterr_arr^2)
	    newqty[i] = total(targetQty[qindex[i1:i2]] * weight_arr)
	    newqty[i] = newqty[i] / total(weight_arr)
	    if (n_elements(targeterr) GT 0) then begin
	      newerr[i] = sqrt(1.0d0 / total(weight_arr))
	    endif    
       endif		; if (qDupl[i] GT 1) then begin
      endfor 		; for i = 0, (nuniq-1) do begin

       print
     endif

     if (debug) then begin
       if (n_elements(newerr) GT 0) then begin
         astring = string(ifield, " qty=", format_list(newqty), $
		 " +- ", format_list(newerr))
       endif else begin
         astring = string(ifield, " qty=", format_list(newqty))
       endelse
	 if (n_elements(astring) GT 1) then begin
	   astring = astring[0] + '...'
	 endif
         message, /info, astring
     endif

    if (iOK) then begin
      if (n_elements(targetnerr) GT 0) then begin 
        ; Not sure if this will work properly to discard the use of nerr.
        iOK = iOK and self -> setField(iField, /inData,$
  	  qty = newqty, $
          err=newerr, $
	  ptr_nerr = ptr_new(), $	; void pointer. Don't use asymmetric errors.
	  styleErr = '', $	; Discard 'poisson'
	  legend = newlegend)
      endif else begin
     ; The vast majority of cases will come here, due to symmetric errors.
        iOK = iOK and self -> setField(iField, /inData,$
  	  qty = newqty, $
          err=newerr, $
	  styleErr = '', $	; Discard 'poisson'
	  legend = newlegend)
      endelse
    endif else begin
      message, /info, string("Problem with ", iField)
    endelse
    OK = OK AND iOK

  endfor

  ;-------------------------------------------
  if (NOT keyword_set(thisNoTreatmentHistory)) then begin
    treatString = "OK = specObj -> Compact(  $"
    if (sortFiel_Used) then begin
      treatString = [treatstring,  $
        string("sortField = ", format_list(thisSortField))]
    endif
    treatString = [treatString, ")"]
    OK = SpecObj -> appendField( /inHeader, $
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
	qty= treatString ) 

  endif

  ;-------------------------------------------
  the_end:
  return, OK
  
end	;

;=========================================================================================
;*
pro specObj::makeLists, $	; Returns lists of tags regarding fields in data section
	independList = thisIndependList, $	; Output keyword. 
					; List of independent fields in data section.
	nIndep = thisNIndep, $		; Number in independent list.
	dependList = thisDependentList, $; Output keyword. 
					; List of dependent fields in data section.
	nDep = thisNDep, $		; Number in dependent list.
	constantList = thisConstantList, $; Output keyword. 
					; List of constant fields in data section.
	nConst = thisNConst, $		; Number in constant list.
	nOther = thisNOther		; number not characterized by 'I', 'D' or 'C'.
;/
;	errMsg = thisErrMsg, $		; Optional error message string.

  debug = 0	;1
  infolabel = "specObj::makeLists >> "
  
  ; Which variables are independent or dependent or constant. 
  nFields = self->count(/inData)	; How many objects to check. 
 
  thisNIndep = 0
  thisNdep = 0
  thisNConst = 0
  thisNOther = 0

  for iField= 0, (nFields -1) do begin    
    OK = self-> getTagField(iField, pendent=fieldPendent, tags=fieldTags)
    nfound = 0
    if (strmatch(fieldPendent, 'I', /fold_case)) then begin
      if (n_elements(thisIndependList) EQ 0) then begin
        thisIndependList = fieldTags
      endif else begin
        thisIndependList = [thisIndependList, fieldTags]
      endelse
      thisNIndep = thisNIndep + 1
      nfound = nfound + 1
;      if (debug) then message, /info, string(iField, thisNIndep, " indep=", format_list(fieldTags))
    endif 

    if (strmatch(fieldPendent, 'D', /fold_case)) then begin
      if (n_elements(thisDependList) EQ 0) then begin
        thisDependList = fieldTags
      endif else begin
        thisDependList = [thisDependList, fieldTags]
      endelse
      thisNdep = thisNdep + 1
      nfound = nfound + 1
;      if (debug) then message, /info, string(iField, thisNdep, " dep=", format_list(fieldTags))
    endif 

    if (strmatch(fieldPendent, 'C', /fold_case)) then begin
      if (n_elements(thisConstantList) EQ 0) then begin
        thisConstantList = fieldTags
      endif else begin
        thisConstantList = [thisConstantList, fieldTags]
      endelse
      thisNConst = thisNConst + 1
      nfound = nfound + 1
;      if (debug) then message, /info, string(iField, thisNConst, " const=", format_list(fieldTags))
    endif 
    
    if (nfound EQ 0) then begin
      thisNother = thisNOther + 1
      if (debug) then message, /info, string(iField, thisNother, " ???=", format_list(fieldTags))
    endif

;    if (debug) then message, /info, string(" nOther =", thisNother)

  endfor

  v_size = size(thisIndependList, /dimensions)
  print, infolabel, "size= ", v_size
  
  if (debug) then begin
    print, infolabel, "thisNIndep =", thisNIndep 
    for i = 0, (thisNIndep -1) do begin
      print, i, ' ', format_list(thisIndependList[(i*3):(i*3+2)])
;Wrong      print, i, ' ', format_list(thisIndependList[ 0:2, i])
    endfor
    print

    print, infolabel, "thisNdep =", thisNdep 
    for i = 0, (thisNdep -1) do begin
      print, i, ' ', format_list(thisDependList[(i*3):(i*3+2)])
;      print, i, ' ', format_list(thisDependList[0:2, i])
    endfor
    print

    print, infolabel, "thisNconst =", thisNconst 
    for i = 0, (thisNconst -1) do begin
      print, i, ' ', format_list(thisconstantList[ (i*3):(i*3+2)])
;      print, i, ' ', format_list(thisconstantList[ 0:2, i])
    endfor
    print
    
    print, infolabel, "thisNother =", thisNother
  endif
end	;

;=========================================================================================
;*
function specObj::sum, $		; Returns 0/1 for failure/success
		; Adds together "raw counts" field of several data objects
	appendages, $			; Array of specObj.
	depField = thisdepField, $	; Optional keyword. Ignored.
	noTreatmentHistory = thisNoTreatmentHistory, $
	errMsg = thisErrMsg, $		; Optional error message string.
	rawCounts = thisRawCounts, $	; Defaults to ['qtyObj', 'COUNTS', '*']
	rawTime = thisRawTime, $	; Defaults to ['qtyObj', 'MIN', '*']
	rawMonitor = thisRawMonitor, $	; Defaults to ['qtyObj', 'MONITOR', '*'] in header
        usingGroupLeader = thisUsingGroupLeader ; IGNORED
;	usingGUI = thisUsingGUI		; IGNORED keyword_set(thisUsingGUI) for now
	
;/


  infolabel = "specObj::sum >> "
  debug = 0 
  
  if (n_elements(thisRawCounts) EQ 0) then begin
    thisRawCounts = ['qtyObj', 'COUNTS', '*']	
  endif

  if (n_elements(thisRawTime) EQ 0) then begin
    thisRawTime = ['qtyObj', 'min', '*']
  endif

  if (n_elements(thisRawMonitor) EQ 0) then begin
    thisRawMonitor = ['qtyObj', 'monitor', '*']
  endif

 nObjs = n_elements(appendages)	; How many objects to sum. Min is 2.
 if (nObjs LT 2) then begin
   thisErrMsg = string(infolabel, 'Need at least 2 objects. nObjs=',nObjs) 
   return, 0
 endif
 
 firstAppendage = appendages[0]
  
  ;------------------------------------------------------------------
;Wrong  theownerDavePtr = self.ownerDavePtr		; Probably 'main'
;Wrong  theDavePtrID = self.davePtr
  obj_destroy, self
;Sept17 obj_destroy, self, /skipDavePtr
  self  = firstAppendage -> clone()	; Clone the object	' firstAppendage.ownerDavePtr = 'local'
;Wrong  self -> setDavePtrID, theDavePtrID, ownerDavePtr=theownerDavePtr
 ;------------------------------------------------------------------
 if (debug) then begin
   print, infolabel, "Just cloned. Now self->show. Need COUNTS in data"
   self -> show, /recursive
 endif

 ;------------------------------------------------------------------
 Ok = self  -> getFocus(zTags = theZtag)
  new_zlabel = theztag[1]
;  new_zCodeSource = theztag[2]
  new_zCodeSource  = 'specObj::sum'
; theZtag[2] = '*'
      
 ;***** Verify that the info in the data section matches. 
 ;***** Sum the monitor. Remove the monPreset and monPrefactor.
 nData = firstAppendage -> count(/inData)

 self -> remove, position=['qtyObj', 'monPreset', '*'], /inHeader, /destroy 
 self -> remove, position=['qtyObj', 'monPreset', '*'], /inData, /destroy 
 self -> remove, position=['qtyObj', 'monPrefactor', '*'], /inHeader, /destroy 
 self -> remove, position=['qtyObj', 'monPrefactor', '*'], /inData, /destroy 
 
  ;------------------------------------------
 monInHeader = 1
 monInData = 0

  OK = self -> getField (thisRawMonitor, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
        /inHeader, $
	name = nameMonitor, $
        units=unitsMonitor, $
  	qty=MonitorTotal)  		; scalar or possibly a short array

  tempErrMsg = ''
  if (not OK) then begin
    monInHeader = 0
    monInData = 1
    message, /info, thisErrMsg
    tempErrMsg = string("Can not find RawMonitor=",$ 
 	'[', thisRawMonitor[0], ',', thisRawMonitor[1], ',', thisRawMonitor[2], $
	'] in header ')
    nameMonitor = "DebugNameMonitor"
    OK = self -> getField (thisRawMonitor, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
        /inData, $
	name = nameMonitor, $
        units=unitsMonitor, $
  	qty=MonitorTotal)  

    if (not OK) then begin
      tempErrMsg = tempErrMsg + string("Can not find RawMonitor=",$ 
 	'[', thisRawMonitor[0], ',', thisRawMonitor[1], ',', thisRawMonitor[2], $
	'] in data')
      thisErrMsg = tempErrMsg
      return, 0
    endif
  endif
 
 
  ;------------------------------------------
 OK = self -> getField (thisRawCounts,  $ 
 	/inData, $
	name = nameCounts, $
        units  = unitsCounts, $
	styleErr=styleErrCounts, $
  	qty=qtyCounts, err= perrCounts, nerr= nerrCounts)  

 if (NOT OK) then begin
   thisErrMsg = string("Can not find in data RawCounts=",$ 
 	'[', thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], ']')
   message, /info, thisErrMsg
   return, 0
 endif

  if ( NOT strcmp(unitsCounts, 'Events', /fold_case)) then begin
    message, /info, string('WARNING! [', $
    	thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], '] should have ', $
	'units of "Events" not ', unitsCounts)
  endif

  if ( NOT strcmp(styleErrCounts, 'poisson', /fold_case)) then begin
    message, /info, string('WARNING! [', $
    	thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], '] should have ', $
	'poisson statitics for appropriate for raw counts, not ', styleErrCounts)
  endif

  ;---------------------------------------------------------
  OK_time = self -> getField (thisRawTime, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
       /inData, $
       name = nameMin, $
        units=unitsMin, $
  	qty=qtyMin)  

  if (not OK_time) then begin
    tempErrMsg =  string("Can not find RawTime=",$ 
 	'[', thisRawTime[0], ',', thisRawTime[1], ',', thisRawTime[2], ']')
    ; Not a fatal error
    message, /info, tempErrMsg
;;;    return, 0
  endif
  ;---------------------------------------------------------

 
 mismatch_arr = make_array(nData, /int)
 firstAppendage -> getLabel, /inData, label=label_arr, $
 	class=class_arr, codeSource=codeSource_arr

 thisErrMsg = '' 
 for iAppend = 1, (nObjs -1) do begin    
   if (debug) then  print, "sum: iappend = ", iappend
   monObj = (appendages[iAppend]) ->  get(position=thisRawMonitor, $
   	inData=monInData, inHeader=monInHeader)
   if (NOT obj_valid(monObj)) then begin
     thisErrMsg = string("Can not find monitor field in obj#=", iAppend)
;     obj_destroy, self
     return, 0
   endif else begin
     OK = monObj -> get(ptr_qty= ptr_qtyMon)
     monitorTotal  = monitorTotal + (*ptr_qtyMon)
     ptr_free, ptr_qtyMon
     if (debug) then print, iAppend, " monitorTotal = ",  monitorTotal 
   endelse

   countsObj = (appendages[iAppend]) ->  get(position=thisRawCounts, /inData)
   if (NOT obj_valid(CountsObj)) then begin
     thisErrMsg = string("Can not find counts field in obj#=", iAppend)
;     obj_destroy, self
     return, 0
   endif else begin
     OK = countsObj -> get(ptr_qty= ptr_qtyCounts)
;;;;     	ptr_err= ptr_perrCounts, ptr_nerr= ptr_nerrCounts)
     qtyCounts = qtyCounts + (*ptr_qtyCounts)
     ptr_free, ptr_qtyCounts
;     if (ptr_valid(ptr_perrCounts)) then begin
;       perrCounts = perrCounts + (*ptr_perrCounts) 
;       ptr_free, ptr_perrCounts
;     endif
;     if (ptr_valid(ptr_nerrCounts)) then begin
;       nerrCounts = nerrCounts + (*ptr_nerrCounts) 
;       ptr_free, ptr_nerrCounts
;     endif
     if (debug) then print, iAppend, " counts = ", qtyCounts[0:3]
   endelse


   if (OK_time) then begin
     timeObj = (appendages[iAppend]) ->  get(position=thisRawTime, /inData)
     if (NOT obj_valid(timeObj)) then begin
       thisErrMsg = string("Can not find time field in obj#=", iAppend)
;       obj_destroy, self
       return, 0
     endif else begin
       OK = timeObj -> get(ptr_qty= ptr_qtyTime)
       qtyMin  = qtyMin + (*ptr_qtyTime)
       ptr_free, ptr_qtyTime
       if (debug) then print, iAppend, " time = ", qtyMin[0:3]
     endelse
   endif
    
 endfor
 perrCounts = sqrt(qtyCounts)
 
 ;---------------------------------------------------------------
 ; Now load up the new summarized fields and get rid of the old ones. 
 newRawCountsTags = ['qtyObj', thisRawCounts[1], new_zCodeSource]
 self -> remove, position=thisRawCounts, /destroy
 self -> add, /inData, $
      tags=newRawCountsTags, $
;      tags=thisRawCounts, $
      contents= obj_new('qtyObj', $
	name = string("Summed ", nameCounts), $
        units  = unitsCounts, $
	styleErr='poisson', $
 	qty=qtyCounts, err= perrCounts)  
   if (debug) then print, thisRawCounts, " OK =", Ok, " qtyCounts =", qtyCounts[0:3]
; OK = self ->setField( thisRawCounts, $
;       	units  = unitsCounts, $
; 	qty=qtyCounts, err= perrCounts, nerr= nerrCounts)  
 OK = self ->setField( thisRawMonitor, qty=monitorTotal, $
 	name = string("Summed ", nameMonitor), $
 	inHeader=monInHeader, inData=monInData)	; Either inData or inHeader 
 if (debug) then print, thisRawMonitor, " OK =", Ok, " monitorTotal =",monitorTotal 
 if (OK_time) then begin
   OK = self ->setField( thisRawTime, $
 	name = string("Summed ", nameMin), $0
 	qty=qtyMin, /inData)  
  if (debug) then  print, thisRawTime, " OK =", Ok, " qtyMin =", qtyMin[0:3]
 endif
 
 ;---------------------------------------------------------------
 ; Verify that unchanged fields in data section are all present, 
 ; so that only those in common are kept. 
 for i = 0, (nData-1) do begin

   OK = firstAppendage -> gettagfield(i, /inData, tags = nthTags)
   if (debug) then print, "sum: data field i = ", i, " tags=", nthTags

   for iAppend = 1, (nObjs -1) do begin    
     OK_iappend = (appendages[iAppend]) -> gettagfield(i, /inData, tags = nthTags)
     if (NOT OK_iappend) then begin
       mismatch_arr(i) = 1
       self -> remove, position=nthTags, /inData, /destroy
       ; Remove non-matching extra fields rather than failing entirely.
       thisErrMsg = [thisErrMsg, string ("Data section. Remove non-matching ", $
	       '[', class_arr[i], ',', label_arr[i], ',', codeSource_arr[i], '] ')]
       goto, mismatch_in_tags
     endif
   endfor
   mismatch_in_tags:
 endfor

 any_mismatch = (round(total(mismatch_arr)) gt 0) 
 
 ;***** append differing info in the header section, selectively *********
 ;**** Note: normally there should not be any significant differences.
 ;**** Keep in header. 
 nData = appendages[0] -> count(/inHeader)
 for i = 0, (nData-1) do begin
    if (debug) then print, "sum: header field i = ", i
   Ok = firstappendage -> gettagfield(i, /inHeader, tags = nthTags)
;   print,' tags=', nthTags

   depLabel = nthTags[1]
      
   if ((depLabel EQ 'DavePtrTreatment') OR (depLabel EQ 'SpecObjTreatment')) then begin
     print, infolabel, "Destroying ", depLabel
     self -> remove, /inHeader, position =  nthTags, /destroy 
     goto, skip_treatment
   endif
   
   OK = firstappendage -> getField(i, /inHeader, $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	rank = depRank, dim=depDim, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
   depPendent = 'I'

  ;-- Create arrays for the summed/appended/zipped dependent variable
  depQty = dep1Qty
  if (n_elements(dep1Err) GT 0) then begin
    depErr = dep1Err
  endif
  if (n_elements(dep1NErr) GT 0) then begin
    depNErr = dep1NErr
  endif
  if (n_elements(dep1Legend) GT 0) then begin
    depLegend = dep1Legend
  endif

   qty_const = 1
   for iZip = 1, (nObjs -1) do begin 
     dep1Qty_prev = dep1Qty
     OK = (appendages[iZip]) -> getField(i, /inHeader, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
     if (depTName EQ 'STRING') then begin
       if ( round(total( (dep1Qty_prev EQ dep1Qty)))  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endif else begin
        if ( round(total( (abs(dep1Qty_prev - dep1Qty) LE 1.0e-13)) )  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endelse
     depQty = [[depQty], [dep1Qty]]
     if (n_elements(dep1Err) GT 0) then begin
       depErr = [[depErr], [dep1Err]]
     endif
     if (n_elements(dep1NErr) GT 0) then begin
       depNErr = [[depErr], [dep1NErr]]
     endif
     if (n_elements(dep1Legend) GT 0) then begin
       depLegend = [[depLegend], [dep1Legend]]
     endif
   endfor

   if (NOT qty_const) then begin
   ; Remove the original field from the header section
;     print, infolabel, "Removing orig field from header section ", $
;   	  nthTags	;;;Wrong ['qtyObj', depLabel, '*']
     self -> remove, /inHeader, position =  nthTags, /destroy ;

     ;--- If the results is a 1 X N array, it needs to be transposed to be properly 1-D
     theDims =  size(depQty, /dimensions)
;     print, infolabel, depName, " dim = ", theDims, " rank=", size(depQty, /n_dimensions)
     if (size(depQty,/n_dimensions) EQ 2) then begin
       if (theDims[0] EQ 1) then begin
         depQty = transpose(depQty)
	 if (n_elements(depErr) GT 0) then begin
	   depErr = transpose(depErr)
	 endif
	 if (n_elements(depNErr) GT 0) then begin
	   depNErr = transpose(depNErr)
	 endif
	 if (n_elements(depLegend) GT 0) then begin
	   depLegend = transpose(depLegend)
	 endif
;         print, infolabel, depName, " dim = ", theDims, $
;	 	" Trans'd dim = ",  size(depQty, /dimensions)
       endif
     endif

     ; Add new vector to the data section as long as it is a plottable type. 
     ; Hope that there is no name clash with the sum/appended/zipped one added previously! 
     depName = depName + '_sum'
     if (depTName EQ 'STRING') then begin
       self -> add, $
         /inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::Sum'], $
;         tags = ['qtyObj', string(depLabel, '_sum'), 'specObj::Sum'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
      endif else begin
       self -> add, $
;       	/inData, $
       		/inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::Sum'], $
;         tags = ['qtyObj', string(depLabel, '_Sum'), 'specObj::Sum'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
       OK = self  -> setTagField( $
;       		/inData, $
       		/inHeader, $
      		['qtyObj', depLabel, 'specObj::Sum'], $
;      		['qtyObj', string(depLabel, '_Sum'), 'specObj::Sum'], $
   		pendent = depPendent)
         if ( strcmp(theZtag[1], depLabel, /fold_case)) then begin	; not safe!
            new_zlabel = theztag[1] 
   	   new_zCodeSource = 'specObj::Sum'
         endif
      endelse

    endif else begin
      ; The sum/appended/zipped field is trivial, so not worth saving.
    endelse

   ;-------- Make temporary variables all undefined, so that 
   ;-------- add method only uses those keywords with defined variables.
   DAVE_undefine, dep1Qty, dep1Err, dep1NErr, dep1Legend
   DAVE_undefine, depQty, depErr, depNErr, depLegend

   skip_treatment:
   
 endfor

 ;*********************************************************************************
   self -> add, /inHeader, $
        tags = ['qtyObj', '#OfSumFiles', 'specObj::Sum'], $
     	contents=obj_new('qtyObj', name='#OfSumFiles', qty= nObjs )

   ;--- Store updated ICP scan type -----------------------------------
   ; Note: Only ICP-dependent command here.
;   if (OK_ICP) then begin
      OK = self -> setField(/inHeader, $
        ['qtyObj', 'type', '*'], qty = 'sum')	; for appended/zip/sum
;   endif

 ;---- Update the specObj history. Has already been removed from the
 ; ---- header section. Needs work. -------------------------
 if (NOT keyword_set(thisNoTreatmentHistory)) then begin
   treatString = "OK = specObj -> Sum(  $"
   treatString = [treatstring,  " [appendages], $"]	; Needs fixing
   treatString = [treatString, ")"]
   OK = SpecObj -> appendField( /inHeader, $
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
	qty= treatString ) 


;   self -> add, /inHeader,$
;  	tags = ['qtyObj', 'SpecObjTreatment', 'specObj::init'], $
;        contents=obj_new('qtyObj',  $
;	  name = 'Object treamtment history',$
;	  qty= treatString ) 
  endif
  
 OK = self -> getField( /inHeader, $
      ['qtyObj', 'filename', '*'], $
      qty=theFilenames)
 nlen = n_elements(theFilenames) 
 if (nlen EQ 0) then theName = "<IN_MEMORY>"
 if (nlen EQ 1) then begin
   theName = "<IN_MEMORY:" + theFilenames + ">"
 endif else begin
   theName = "<IN_MEMORY:" + theFilenames[0] + '...' + theFilenames[(nlen-1)] + ">"
 endelse
 
 OK = self -> setField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      name='SpecObj_filename', $	; over-riden, not appended
      qty=theName)
 OK = self -> setTagField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      codeSource='specObj::Sum')

 ;----------------------------------------------------
 if (debug) then begin
   print, infolabel, "Relaoded stiff. About to self -> scaleByMonitor()" 
   self -> show, /recursive
 endif
 ;----------------------------------------------------


; if (not DEBUG) then begin      
; ; can't find [*, counts, *] in this routine!
 ;----- Monitor counts -> intensity ---------------------- 
  OK = self -> scaleByMonitor()		; Do this by default
; endif
 
 ;----- Not bothering to change the focus. ---------------------- 
  if ( strcmp(new_zlabel, theZtag[1], /fold_case)) then begin	
    self  -> setFocus, cutTags = ['', '', ''], $
     	zTags = ['qtyObj', new_zlabel, new_zCodeSource]
    if (debug) then begin
      self -> show, /focus	; debug
      print, infolabel, "Changing z focus from [",theztag, "] to [", $
  	 ['qtyObj', new_zlabel, new_zCodeSource]   , "]"
    endif
  endif
 ;--------------------------------------------------
  ; return, OK		; Sept 3, 2003. TMR
  return, 1
  
end		; specObj::sum


;=========================================================================================
;*
function specObj::FRGD_BKGD, $		; Returns 0/1 for failure/success
		; subtracts and adds together "intensity" field of two data objects
	appendages, $			; Array of specObj. (Assumed to be length 2!)
	depField = thisdepField, $	; Optional keyword. Ignored.
	noTreatmentHistory = thisNoTreatmentHistory, $
	errMsg = thisErrMsg, $		; Optional error message string.
        usingGroupLeader = thisUsingGroupLeader, $; If (thisUsingGroupLeader NE 0L) then 
   		; MIGHT pop up an interactive diglaog box. Sept 15, 2003. TMR.
;	usingGUI = thisUsingGUI, $	; If keyword_set(thisUsingGUI) then 
;					; GUI pops up to modify thisIntensity
	intensity = thisIntensity 	; Defaults to ['qtyObj', 'INTENSITY', '*']
	
;/


  infolabel = "specObj::FRGD_BKGD >> "
  debug = 0
  debug2 = 0
  debug3 = 0

  if (debug) then begin  
    print
    print
    message, /info, string("Entered. nappendages=", n_elements(appendages))
  endif 
  
  if (n_elements(thisIntensity) EQ 0) then begin
    thisIntensity = ['qtyObj', 'INTENSITY', '*']	
  endif


 nObjs = n_elements(appendages)	; How many objects to sum. Require 2.
 if (nObjs NE 2) then begin
   thisErrMsg = string(infolabel, 'Need exactly 2 objects. nObjs=',nObjs) 
   return, 0
 endif
 
  firstAppendage = appendages[0]

  ;------------------------------------------------------------------
  if (debug2) then begin
      print, '~~~~~~  ', infolabel, " n_elements(usingGroupLeader) = ", $
      	 n_elements(thisusingGroupLeader)
  endif

  if (n_elements(thisUsingGroupLeader) NE 0) then begin
    if (debug2) then begin
      print, '~~~~~  ', infolabel, " usingGroupLeader = ",  thisusingGroupLeader, $
      	" test=", (thisUsingGroupLeader NE 0L)
    endif

   if (thisUsingGroupLeader NE 0) then begin
;  if (keyword_set(thisUsingGUI)) then begin
    ; Allow the user to change the selection thisIntensity from the GUI if in interactive mode.
    message, /info, string("Orig field = ", format_list(thisIntensity))
    ; This updates the zfocus with the new selection
    whereGUIObj = obj_new('dialog_ow_where', firstAppendage, $
        groupLeader = thisUsingGroupLeader, $
    	from='data', $
	label = thisIntensity[1])
    OK = whereGUIObj -> createWidgets()

    message, /info, string("Old Intensity tags = ", $
    	format_list(thisIntensity))
	
    ; Copy over the new zfocus to the string array thisIntensity
    OK = firstAppendage -> getFocus(ztags = thisIntensity)

    if (NOT OK) then begin
      print, infolabel, string(OK, "=OK. New Intensity tags and focus = ", $
    	format_list(thisIntensity))
;      message, /info, string(OK, "=OK. New Intensity tags and focus = ", $
;    	format_list(thisIntensity))
    endif
    
   if (obj_valid(whereGUIObj)) then begin 
     obj_destroy, whereGUIObj
   endif
  endif
 endif
 
  ;------------------------------------------------------------------
;Wrong  theownerDavePtr = self.ownerDavePtr		; Probably 'main'
;Wrong  theDavePtrID = self.davePtr
 obj_destroy, self		;, /skipDavePtr
 self  = firstAppendage -> clone()	; Clone the object
;Wrong  self -> setDavePtrID, theDavePtrID, ownerDavePtr=theownerDavePtr
 ;------------------------------------------------------------------
; if (debug) then begin
;   print, infolabel, "Just cloned. Now self->show. Need COUNTS in data"
;   self -> show, /recursive
; endif

 ;------------------------------------------------------------------
 Ok = self  -> getFocus(zTags = theZtag)
  if (debug) then message, /info, string("z fcous = ", format_list(theZtag))
  new_zlabel = theztag[1]
  new_zCodeSource  = 'specObj::FRGD_BKGD'
      
 ;***** Verify that the info in the data section matches. 
 ;***** Sum the monitor. Remove the monPreset and monPrefactor.
 nData = firstAppendage -> count(/inData)

; self -> remove, position=['qtyObj', 'monPreset', '*'], /inHeader, /destroy 
; self -> remove, position=['qtyObj', 'monPreset', '*'], /inData, /destroy 
; self -> remove, position=['qtyObj', 'monPrefactor', '*'], /inHeader, /destroy 
; self -> remove, position=['qtyObj', 'monPrefactor', '*'], /inData, /destroy 
 
  ;------------------------------------------
 intensObj_0 = self -> get( position=thisIntensity,  $ 
 	/inData) 
 if (debug3) then begin
   message, /info, string("intensObj_0 = ", intensObj_0 )
 endif 
 
 OK = self -> getField (thisIntensity,  $ 
 	/inData, $
	name = nameIntensity)  
; OK = self -> getField (thisIntensity,  $ 
; 	/inData, $
;	name = nameIntensity, $
;        units  = unitsIntensity, $
;;	styleErr=styleErrIntensity, $
;  	qty=qtyIntensity, err= perrIntensity, nerr= nerrIntensity)  
 
 if (NOT OK) then begin
   thisErrMsg = string("Can not find in data Intensity=", format_list(thisIntenisty))
   message, /info, thisErrMsg
   return, 0
 endif

 ;-------------------------------------------------------------------------------
 
  ;-----------------------------------------------------
 mismatch_arr = make_array(nData, /int)
 firstAppendage -> getLabel, /inData, label=label_arr, $
 	class=class_arr, codeSource=codeSource_arr

 ;---------------------------------------------------------------
 ; Verify that unchanged fields in data section are all present, 
 ; so that only those in common are kept. Any fields that differ in content
 ; have the background one copied over. 
 if (debug3) then begin
   print
   message, /info, string("Verifying that unchanged fields in data section are",$
	   	" all present.")
 endif 
 for i = 0, (nData-1) do begin

   OK = firstAppendage -> gettagfield(i, /inData, tags = nthTags)
;   if (debug) then print, infolabel, "data field i = ", i, " tags=", nthTags

   for iAppend = 1, (nObjs -1) do begin    
     OK_iappend = (appendages[iAppend]) -> gettagfield(i, /inData, tags = nthTags)
;     if (debug) then begin
;       message, /info, string(i, " Match checking ", format_list(nthTags))
;     endif
     if (NOT OK_iappend) then begin
       mismatch_arr(i) = 1
       self -> remove, position=nthTags, /inData, /destroy
       ; Remove non-matching extra fields rather than failing entirely.
       thisErrMsg = [thisErrMsg, string ("Data section. Remove non-matching ", $
	       '[', class_arr[i], ',', label_arr[i], ',', codeSource_arr[i], '] ')]
       if (debug) then message, /info, string(i, "Mismatch with ", format_list(nthTags))
       goto, mismatch_in_tags
     endif else begin
       ; Now verfiy that they have the same contents. If not, put in the background field too.
       OK = firstAppendage -> getfield(i, /inData, ptr_qty=ptr_qty_0)
       if (debug3) then begin
         message, /info, string(i, "=ifield. ptr_qty_0 = ", ptr_qty_0)
       endif 

       OK_iappend = (appendages[iAppend]) -> getfield(i, /inData, $
       		ptr_qty=ptr_qty_i, $
		name = name_i)
       if (debug3) then begin
         message, /info, string(i, "=ifield. ptr_qty_i = ", ptr_qty_i, " ", name_i)
       endif 
;       if (debug) then begin
;         print, infolabel, i, " OK=", OK, " OK_iappend=", OK_iappend
;	 print, "qty_0=", (*ptr_qty_0)
;	 print, "qty_i=", (*ptr_qty_i)
;       endif
       OK_iappend = (appendages[iAppend]) -> gettagfield(i, /inData, $
       		label=label_i)
       if (NOT eqarrays(ptr_qty_0, ptr_qty_i)) then begin
	 IntensObj_i = (appendages[iAppend]) ->  get(position=i, /inData)
         if (debug3) then begin
           message, /info, string(i, "=ifield. IntensObj_0 = ", IntensObj_0)
         endif 
;         OK_iappend = IntensObj_i -> get( $
;       		ptr_qty=ptr_qty_i, $
;		name = name_i, label=label_i)
         if (debug) then begin
           message, /info, string(i, " ", format_list(nthTags), " not exactly the same")
         endif
	 addErrMsg = ''
;         self -> add, /inData, $
;           tags= ['qtyObj', string('Bkgd ', label_i), new_zCodeSource], $
;           contents= (IntensObj_i -> clone()),  $
;	   errMsg = addErrMsg
;        if (debug3) then begin
;           message, /info, string(i, "Cloned as BKGD:", label_i)
;         endif 
	 IntensObj_iclone = IntensObj_i -> clone()	; Make a copy
         self -> add, /inData, $
           tags= ['qtyObj', string('Bkgd ', label_i), new_zCodeSource], $
           contents= IntensObj_iclone,  $
	   errMsg = addErrMsg
         if (debug3) then begin
           message, /info, string(strtrim(string(i),2), " Cloned as BKGD:", label_i, $
;	   	". IntensObj_0 = ", IntensObj_0, $
		". IntensObj_iclone = ", IntensObj_iclone)
;           message, /info, string(strtrim(string(i),2), " Cloned as BKGD:", label_i, $
;	   	". IntensObj_0 = ", strtrim(string(IntensObj_0), 2), $
;		". IntensObj_iclone = ", strtrim(string(IntensObj_iclone), 2))
         endif 
	 IntensObj_iclone = obj_new()	; Let dangle. Now let self keep tract of it and
	 		; destroy it when appropriate.
         if (strlen(addErrMsg) NE 0) then begin
	   message, /info, string("Failed to add", $
	   	format_list(['qtyObj', string('Bkgd ', label_i), new_zCodeSource]))
	   print, addErrMsg
	 endif else begin
	   ; Update the name for the background run
	   OK = self -> setField(['qtyObj', string('Bkgd ', label_i), new_zCodeSource], $
	   	/inData, $
		name = string('BKGD ', name_i))	
	 endelse
       endif
       ptr_free, ptr_qty_i
       ptr_free, ptr_qty_0
     endelse
   endfor
   mismatch_in_tags:
 endfor

 any_mismatch = (round(total(mismatch_arr)) gt 0) 
 if (debug) then message, /info, string($
 	"number of mismatches = ", round(total(mismatch_arr)))

  ;--------------------------------------------------------
 thisErrMsg = '' 
 ; Put the background intenisty in the specObj.
 ; Then  add and subtract the foreground and the background intensities
 IntensObj_0 = (appendages[0]) ->  get(position=thisIntensity, /inData)
 if (debug3) then begin
   print
   message, /info, string("... Now working on 'thisIntensity':", $
	   format_list(thisIntensity))
   message, /info, string("IntensObj_0 = ", IntensObj_0)
 endif 
   ;-------------------------------
 if (NOT obj_valid(IntensObj_0)) then begin
     thisErrMsg = string(infolabel, "Can not find ", format_list(thisIntensity), $
     		" field in obj#=", 0)
     return, 0
 endif 
   
 for iAppend = 1, (nObjs -1) do begin    
   if (debug) then  print, "FRGD_BKGD: iappend = ", iappend

   IntensObj_i = (appendages[iAppend]) ->  get(position=thisIntensity, /inData)
   if (debug3) then begin
     message, /info, string("BKGD IntensObj_i = ", IntensObj_i)
   endif 
      
   ;-------------------------------
   if (NOT obj_valid(IntensObj_i)) then begin
     thisErrMsg = string(infolabel, "Can not find Intensity field in obj#=", iAppend)
     return, 0
   endif else begin
;;     self -> add, /inData, $
;;         tags= ['qtyObj', string('Bkgd ', thisIntensity[1]), new_zCodeSource], $
;;         contents= (IntensObj_i -> clone())
;;
;;     if (debug3) then begin
;;           message, /info, string(i, "thisIntensity. Cloned as BKGD:", thisIntensity[1])
;;     endif 

;; No need to clone. Already in specObj.
;     IntensObj_iclone = IntensObj_i -> clone()		; make a copy
;     if (debug3) then begin
;       message, /info, string("BKGD IntensObj_iclone = ", IntensObj_iclone)
;     endif 
;     self -> add, /inData, $
;         tags= ['qtyObj', string('Bkgd ', thisIntensity[1]), new_zCodeSource], $
;         contents= IntensObj_iclone		; Oops this leaves a dangling object!
;     IntensObj_iclone = obj_new()	; Let dangle. Now let self keep tract of it and
;	 		; destroy it when appropriate.
;     if (debug3) then begin
;       message, /info, string("After NULLING. BKGD IntensObj_iclone = ", $
;       	IntensObj_iclone)
;     endif 
 
     ;-------------------------------
     binErrMsg = ''
     IntensObj_sub = qtyObj__binaryOp(IntensObj_0, '-', IntensObj_i, $
     	name = string('Frgd-Bkgd ', nameIntensity),$
	errMsg = binErrMes)
     if (debug3) then begin
       message, /info, string("IntensObj_sub = ", IntensObj_sub)
     endif 
     if (obj_valid(IntensObj_sub)) then begin
       self -> add, /inData, $
         tags= ['qtyObj', string('Frgd-Bkgd ', thisIntensity[1]), new_zCodeSource], $
         contents= IntensObj_sub
	 new_zlabel = string('Frgd-Bkgd ', thisIntensity[1])
     endif else begin
       thisErrMsg = [binErrMes, $
       	  string(infolabel, $
       		"Failed to create field 'Frgd-Bkgd ",thisIntensity[1], "'")]
       if (debug) then print, thisErrMsg
       return, 0
     endelse

    ;-------------------------------
     binErrMsg = ''
     if (debug) then  begin
       message, /info, string("obj_valid(IntensObj_0) = ",  obj_valid(IntensObj_0))
       message, /info, string("obj_valid(IntensObj_i) = ",  obj_valid(IntensObj_i))
     endif
     IntensObj_sum = qtyObj__binaryOp(IntensObj_0, '+', IntensObj_i, $
     	name = string('Frgd+Bkgd ', nameIntensity), $
	errMsg = binErrMes)
        if (debug3) then begin
           message, /info, string("IntensObj_sum = ", IntensObj_sum)
        endif 
     if (obj_valid(IntensObj_sum)) then begin
       self -> add, /inData, $
         tags= ['qtyObj', string('Frgd+Bkgd ', thisIntensity[1]), new_zCodeSource], $
         contents= IntensObj_sum
     endif else begin
       thisErrMsg = [binErrMes, $
       	  string(infolabel, $
 	      	"Failed to create field 'Frgd+Bkgd ",thisIntensity[1], "'")]
       if (debug) then print, thisErrMsg
       return, 0
     endelse
   ;-------------------------------
     IntensObj_i = obj_new()	; Let dangle. Now let self keep tract of it and
	 		; destroy it when appropriate.
   	
   endelse
    
 endfor

  ;--------------------------------------------------------------------------
 ;***** append differing info in the header section, selectively *********
 ;**** Note: normally there should not be any significant differences.
 ;**** Keep in header. 
 nData = appendages[0] -> count(/inHeader)
 for i = 0, (nData-1) do begin
;    if (debug) then print, "FRGD_BKGD: header field i = ", i
   Ok = firstappendage -> gettagfield(i, /inHeader, tags = nthTags)
;   print,' tags=', nthTags

   depLabel = nthTags[1]
      
   if ((depLabel EQ 'DavePtrTreatment') OR (depLabel EQ 'SpecObjTreatment')) then begin
     print, infolabel, "Destroying ", depLabel
     self -> remove, /inHeader, position =  nthTags, /destroy 
     goto, skip_treatment
   endif
   
   if (nthTags[1] EQ 'mainTitle')  then begin
     ; Keep the foreground's title as is.
     goto, skip_treatment
   endif
   
   OK = firstappendage -> getField(i, /inHeader, $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	rank = depRank, dim=depDim, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
   depPendent = 'I'

  ;-- Create arrays for the FRGD_BKGDmed/summed/appended/zipped dependent variable
  depQty = dep1Qty
  if (n_elements(dep1Err) GT 0) then begin
    depErr = dep1Err
  endif
  if (n_elements(dep1NErr) GT 0) then begin
    depNErr = dep1NErr
  endif
  if (n_elements(dep1Legend) GT 0) then begin
    depLegend = dep1Legend
  endif

   qty_const = 1
   for iZip = 1, (nObjs -1) do begin 
     dep1Qty_prev = dep1Qty
     OK = (appendages[iZip]) -> getField(i, /inHeader, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
     if (depTName EQ 'STRING') then begin
       if ( round(total( (dep1Qty_prev EQ dep1Qty)))  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endif else begin
        if ( round(total( (abs(dep1Qty_prev - dep1Qty) LE 1.0e-13)) )  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endelse
     depQty = [[depQty], [dep1Qty]]
     if (n_elements(dep1Err) GT 0) then begin
       depErr = [[depErr], [dep1Err]]
     endif
     if (n_elements(dep1NErr) GT 0) then begin
       depNErr = [[depErr], [dep1NErr]]
     endif
     if (n_elements(dep1Legend) GT 0) then begin
       depLegend = [[depLegend], [dep1Legend]]
     endif
   endfor

   if (NOT qty_const) then begin
   ; Remove the original field from the header section
    print, infolabel, "Modifying field in header section to reflect BKGD run", $
   	  format_list(nthTags)	;;;Wrong ['qtyObj', depLabel, '*']
     self -> remove, /inHeader, position =  nthTags, /destroy ;

     ;--- If the results is a 1 X N array, it needs to be transposed to be properly 1-D
     theDims =  size(depQty, /dimensions)
;     print, infolabel, depName, " dim = ", theDims, " rank=", size(depQty, /n_dimensions)
     if (size(depQty,/n_dimensions) EQ 2) then begin
       if (theDims[0] EQ 1) then begin
         depQty = transpose(depQty)
	 if (n_elements(depErr) GT 0) then begin
	   depErr = transpose(depErr)
	 endif
	 if (n_elements(depNErr) GT 0) then begin
	   depNErr = transpose(depNErr)
	 endif
	 if (n_elements(depLegend) GT 0) then begin
	   depLegend = transpose(depLegend)
	 endif
;         print, infolabel, depName, " dim = ", theDims, $
;	 	" Trans'd dim = ",  size(depQty, /dimensions)
       endif
     endif

     ; Always add new vector to the header section, since only 2 appendages. 
     depName = depName + '_FRGD_BKGD'
     if (depTName EQ 'STRING') then begin
       self -> add, $
         /inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::FRGD_BKGD'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
      endif else begin
       self -> add, $
       		/inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::FRGD_BKGD'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
       OK = self  -> setTagField( $
       		/inHeader, $
      		['qtyObj', depLabel, 'specObj::FRGD_BKGD'], $
   		pendent = depPendent)
         if ( strcmp(theZtag[1], depLabel, /fold_case)) then begin	; not safe!
            new_zlabel = theztag[1] 
   	   new_zCodeSource = 'specObj::FRGD_BKGD'
         endif
      endelse

    endif else begin
      ; The FRGD_BKGD/sum/appended/zipped field is trivial, so not worth saving.
    endelse

   ;-------- Make temporary variables all undefined, so that 
   ;-------- add method only uses those keywords with defined variables.
   DAVE_undefine, dep1Qty, dep1Err, dep1NErr, dep1Legend
   DAVE_undefine, depQty, depErr, depNErr, depLegend

   skip_treatment:
   
 endfor

 ;*********************************************************************************
   self -> add, /inHeader, $
        tags = ['qtyObj', '#OfFRGD_BKGDFiles', 'specObj::FRGD_BKGD'], $
     	contents=obj_new('qtyObj', name='#OfFRGD_BKGDFiles', qty= nObjs )

   ;--- Store updated ICP scan type -----------------------------------
   ; Note: Only ICP-dependent command here.
;   if (OK_ICP) then begin
      OK = self -> setField(/inHeader, $
        ['qtyObj', 'type', '*'], qty = 'FRGD_BKGD')	; for appended/zip/sum/FRGD_BKGD
;   endif

 ;---- Update the specObj history. Has already been removed from the
 ; ---- header section. Needs work. -------------------------
 if (NOT keyword_set(thisNoTreatmentHistory)) then begin
   treatString = "OK = specObj -> FRGD_BKGD(  $"
   treatString = [treatstring,  " [appendages], $  ; Needs fixing"]	; Needs fixing
   treatString = [treatString, ")"]
   OK = SpecObj -> appendField( /inHeader, $
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
	qty= treatString ) 


;   self -> add, /inHeader,$
;  	tags = ['qtyObj', 'SpecObjTreatment', 'specObj::init'], $
;        contents=obj_new('qtyObj',  $
;	  name = 'Object treamtment history',$
;	  qty= treatString ) 
  endif
  
 OK = self -> getField( /inHeader, $
      ['qtyObj', 'filename', '*'], $
      qty=theFilenames)
 nlen = n_elements(theFilenames) 
 if (nlen EQ 0) then theName = "IN_MEMORY"
 if (nlen EQ 1) then begin
   theName = "IN_MEMORY:" + theFilenames + ""
 endif else begin
   theName = "IN_MEMORY:" + theFilenames[0] + '...' + theFilenames[(nlen-1)] + ">"
 endelse
 
 OK = self -> setField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      name='SpecObj_filename', $	; over-riden, not appended
      qty=theName)
 OK = self -> setTagField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      codeSource='specObj::FRGD_BKGD')

 ;----------------------------------------------------
; if (debug) then begin
;   print, infolabel, "Reloaded stuff. About to modify the focus" 
;   self -> show, /recursive
; endif
 ;----------------------------------------------------


    ;-------- Change the focus to FRGD-BKGD intensity
    self  -> setFocus, cutTags = ['', '', ''], $
     	zTags = ['qtyObj', new_zlabel, new_zCodeSource]
    if (debug) then begin
      self -> show, /focus	; debug
      print, infolabel, "Changing z focus from [",theztag, "] to [", $
  	 ['qtyObj', new_zlabel, new_zCodeSource]   , "]"
    endif

;  if ( strcmp(new_zlabel, theZtag[1], /fold_case)) then begin	
;    ;----- Not bothering to change the focus. ---------------------- 
;    self  -> setFocus, cutTags = ['', '', ''], $
;     	zTags = ['qtyObj', new_zlabel, new_zCodeSource]
;    if (debug) then begin
;      self -> show, /focus	; debug
;      print, infolabel, "Changing z focus from [",theztag, "] to [", $
;  	 ['qtyObj', new_zlabel, new_zCodeSource]   , "]"
;    endif
;  endif 
  
 ;--------------------------------------------------

 ;----------------------------------------------------
 if (debug) then begin
   print, infolabel, "Reloaded stuff. After modifying the focus" 
   self -> show, /recursive
 endif
 ;----------------------------------------------------

  ; return, OK		; Sept 3, 2003. TMR
  return, 1
  
end		; specObj::FRGD_BKGD


;=========================================================================================
;*
function specObj::zip, $		; Returns 0/1 for failure/success
	appendages, $			; Array of specObj.
	noTreatmentHistory = thisNoTreatmentHistory, $
	depField = thisdepField, $		; Optional keyword. 
					; Only sets the 'Y' focus 
        usingGroupLeader = thisUsingGroupLeader, $; IGNORED
;	usingGUI = thisUsingGUI, $	; IGNORED keyword_set(thisUsingGUI) for now
	errMsg = thisErrMsg		; Optional error message string.
;/
;;;	indepField = thisIndepField, $  ; optional. the tags for the data field.


 infolabel = "specObj::zip >> "
 debug = 0	;1 
  
 nObjs = n_elements(appendages)	; How many objects to zip. Min is 2.
 firstAppendage = appendages[0]

  
  ;------------------------------------------------------------------
;Wrong  theownerDavePtr = self.ownerDavePtr		; Probably 'main'
;Wrong  theDavePtrID = self.davePtr
 obj_destroy, self		; , /skipDavePtr
 self  = firstAppendage -> clone()	; Clone the object
;Wrong  self -> setDavePtrID, theDavePtrID, ownerDavePtr=theownerDavePtr

; print, infolabel, "Removing ", ['qtyObj', '#OfZipFiles', 'specObj::Zip']
; self -> remove, /inHeader, /destroy, $
;      position = ['qtyObj', '#OfZipFiles', 'specObj::Zip']
; print, infolabel, "Removing ", ['qtyObj', 'zipFilenames', 'specObj::Zip']
; self -> remove, /inHeader, /destroy, $
;      position = ['qtyObj', 'zipFilenames', 'specObj::Zip']
 ;------------------------------------------------------------------
      
;Unused nZip = 1
 startZip = 1

  Ok = self  -> getFocus(zTags = theZtag)
  new_zlabel = theztag[1]
  new_zCodeSource = theztag[2]

  new_ylabel = ''
  if (n_elements(thisDepField) GT 0) then begin
    new_ylabel = thisDepField[1]
  endif

 ;***** zip up info in the data section, selectively *****************************
 ;-- Now do the independent fields in the data section! ------  
 nData = appendages[0] -> count(/inData)
 for i = 0, (nData-1) do begin
   Ok = appendages[0] -> gettagfield(i, /inData, tags = nthTags)
;   print,' tags=', nthTags

   depLabel = nthTags[1]   
   OK = firstappendage  -> getField(i, /inData, $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	rank = depRank, dim=depDim, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
   depPendent = ' '
   OK = firstappendage  -> getTagField(nthTags,  /inData, pendent = depPendent)
   if (depPendent NE 'D') then begin
     depPendent = 'I'	; independent
   endif
;
;   depPendent = 'I'
;;   depPendent = ' '
;;   OK = firstappendage  -> getTagField(nthTags,  /inData, pendent = depPendent)
;;   if (depPendent EQ ' ') then begin
;;     deppendent = 'I'	; independent
;;;     print, infolabel, depPendent, "=depPendent. tags = ", nthTags
;;   endif

  ;-- Create arrays for the zipped dependent variable
  depQty = dep1Qty
  if (n_elements(dep1Err) GT 0) then begin
    depErr = dep1Err
  endif
  if (n_elements(dep1NErr) GT 0) then begin
    depNErr = dep1NErr
  endif
  if (n_elements(dep1Legend) GT 0) then begin
    depLegend = dep1Legend
  endif

   qty_const = 1
   for iZip = 1, (nObjs -1) do begin 
     dep1Qty_prev = dep1Qty
     OK = (appendages[iZip]) -> getField(i, /inData, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
;     print, infolabel, 'Data: ', i, izip
;     print, "Data: tot=", round(total( (dep1Qty_prev EQ dep1Qty))), $
;     	" n = ",  n_elements(dep1Qty)
;     if ( round(total( (dep1Qty_prev EQ dep1Qty)))  $
;     		NE n_elements(dep1Qty)) then begin
     if ( round(total( (abs(dep1Qty_prev - dep1Qty) LE 1.0e-13)) )  $
     		NE n_elements(dep1Qty)) then begin
	qty_const = qty_const AND 0
     endif
;     print, infolabel, 'Data: ', i, izip, depName, " qty_const = ", qty_const
     depQty = [[depQty], [dep1Qty]]
     if (n_elements(dep1Err) GT 0) then begin
       depErr = [[depErr], [dep1Err]]
     endif
     if (n_elements(dep1NErr) GT 0) then begin
       depNErr = [[depErr], [dep1NErr]]
     endif
     if (n_elements(dep1Legend) GT 0) then begin
       depLegend = [[depLegend], [dep1Legend]]
     endif
   endfor

   if (NOT qty_const) then begin
     ; Remove the original field from the data section
;     print, infolabel, "Removing orig field from data section ", $
;   	  nthTags	;;;Wrong ['qtyObj', depLabel, '*']
     self -> remove, /inData, position =  nthTags, /destroy 
     
	
     ; Add new vector to the data section. 
     ; Hope that there is no name clash with the zipped one added previously! 
     depName = depName + 'zipped'
     self -> add, /inData, $
      tags = ['qtyObj', depLabel, 'specObj::Zip'], $
;      tags = ['qtyObj', string(depLabel, '_zip'), 'specObj::Zip'], $
      contents=obj_new('qtyObj',  $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	qty=depQty,$
	err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	legend=depLegend)
     OK = self  -> setTagField( /inData, $
      		['qtyObj', depLabel, 'specObj::Zip'], $
;      		['qtyObj', string(depLabel, '_zip'), 'specObj::Zip'], $
   		pendent = depPendent)
	

      ;--- modify the focus if necessary
;      if (debug) then begin 
;        print, infolabel, strcmp(theZtag[1], depLabel, /fold_case), $
;     		 "=strcmp(", theZtag[1], ", ", depLabel, ", /fold_case)"
;      endif

      if ( strcmp(theZtag[1], depLabel, /fold_case)) then begin	; not safe!
        new_zlabel = theztag[1] 
;        new_zlabel = theztag[1] + '_zip'
	new_zCodeSource = 'specObj::Zip'
      endif
      
    endif else begin
      ; The zipped field is trivial, so not worth saving.
    endelse

   ;-------- Make temporary variables all undefined, so that 
   ;-------- add method only uses those keywords with defined variables.
   DAVE_undefine, dep1Qty, dep1Err, dep1NErr, dep1Legend
   DAVE_undefine, depQty, depErr, depNErr, depLegend
;   if (n_elements(dep1Qty) GT 0) then begin
;     junk = temporary(dep1Qty)	; Make del1Qty undefined for a bit.
;   endif
;   if (n_elements(dep1Err) GT 0) then begin
;      junk = temporary(dep1Err)	; Make del1Err undefined for a bit.
;   endif
;   if (n_elements(dep1Nerr) GT 0) then begin
;      junk = temporary(dep1NErr)	; Make del1NErr undefined for a bit.
;   endif
;   if (n_elements(dep1Legend) GT 0) then begin
;      junk = temporary(dep1Legend)	; Make del1legend undefined for a bit.
;   endif

;   if (n_elements(depQty) GT 0) then begin
;     junk = temporary(depQty)	; Make delQty undefined for a bit.
;   endif
;   if (n_elements(depErr) GT 0) then begin
;      junk = temporary(depErr)	; Make delErr undefined for a bit.
;   endif
;   if (n_elements(depNerr) GT 0) then begin
;      junk = temporary(depNErr)	; Make delNErr undefined for a bit.
;   endif
;   if (n_elements(depLegend) GT 0) then begin
;      junk = temporary(depLegend)	; Make dellegend undefined for a bit.
;   endif

 endfor

 ;***** zip up info in the header section, selectively *****************************
 ;-- Now do the independent fields in the header section! ------  
 nData = appendages[0] -> count(/inHeader)
 for i = 0, (nData-1) do begin
   Ok = appendages[0] -> gettagfield(i, /inHeader, tags = nthTags)
;   print,' tags=', nthTags

   depLabel = nthTags[1]   
   if ((depLabel EQ 'DavePtrTreatment') OR (depLabel EQ 'SpecObjTreatment')) then begin
     self -> remove, /inHeader, position =  nthTags, /destroy 
     goto, skip_treatment
   endif

   OK = (appendages[0])  -> getField(i, /inHeader, $
	name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	rank = depRank, dim=depDim, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
   depPendent = 'I'

;   depPendent = ' '
;   OK = (appendages[0])  -> getTagField(i, /inHeader, $
;   	pendent = depPendent)
;   if (depPendent EQ ' ') then begin
;     depPendent = 'I'	; independent
;;     print, infolabel, depPendent, "=depPendent. tags = ", nthTags
;   endif

  ;-- Create arrays for the zipped dependent variable
  depQty = dep1Qty
  if (n_elements(dep1Err) GT 0) then begin
    depErr = dep1Err
  endif
  if (n_elements(dep1NErr) GT 0) then begin
    depNErr = dep1NErr
  endif
  if (n_elements(dep1Legend) GT 0) then begin
    depLegend = dep1Legend
  endif

   qty_const = 1
   for iZip = 1, (nObjs -1) do begin 
     dep1Qty_prev = dep1Qty
     OK = (appendages[iZip]) -> getField(i, /inHeader, $
 	qty=dep1Qty, err=dep1Err, nerr=dep1Nerr, $
 	legend=dep1Legend)
;     print, infolabel, 'Header: ', i, izip
;     print, "Header: tot=", round(total( (dep1Qty_prev EQ dep1Qty))), $
;     	" n = ",  n_elements(dep1Qty)
     if (depTName EQ 'STRING') then begin
       if ( round(total( (dep1Qty_prev EQ dep1Qty)))  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endif else begin
        if ( round(total( (abs(dep1Qty_prev - dep1Qty) LE 1.0e-13)) )  $
     		NE n_elements(dep1Qty)) then begin
	  qty_const = qty_const AND 0
        endif
     endelse
;     print, infolabel, 'Header: ', i, izip, depName, " qty_const = ", qty_const
     depQty = [[depQty], [dep1Qty]]
     if (n_elements(dep1Err) GT 0) then begin
       depErr = [[depErr], [dep1Err]]
     endif
     if (n_elements(dep1NErr) GT 0) then begin
       depNErr = [[depErr], [dep1NErr]]
     endif
     if (n_elements(dep1Legend) GT 0) then begin
       depLegend = [[depLegend], [dep1Legend]]
     endif
   endfor

   if (NOT qty_const) then begin
   ; Remove the original field from the header section
;     print, infolabel, "Removing orig field from header section ", $
;   	  nthTags	;;;Wrong ['qtyObj', depLabel, '*']
     self -> remove, /inHeader, position =  nthTags, /destroy 

     ;--- If the results is a 1 X N array, it needs to be transposed to be properly 1-D
     theDims =  size(depQty, /dimensions)
;     print, infolabel, depName, " dim = ", theDims, " rank=", size(depQty, /n_dimensions)
     if (size(depQty,/n_dimensions) EQ 2) then begin
       if (theDims[0] EQ 1) then begin
         depQty = transpose(depQty)
	 if (n_elements(depErr) GT 0) then begin
	   depErr = transpose(depErr)
	 endif
	 if (n_elements(depNErr) GT 0) then begin
	   depNErr = transpose(depNErr)
	 endif
	 if (n_elements(depLegend) GT 0) then begin
	   depLegend = transpose(depLegend)
	 endif
;         print, infolabel, depName, " dim = ", theDims, $
;	 	" Trans'd dim = ",  size(depQty, /dimensions)
       endif
     endif

     ; Add new vector to the data section as long as it is a plottable type. 
     ; Hope that there is no name clash with the zipped one added previously! 
     depName = depName + '_zipper'
     if (depTName EQ 'STRING') then begin
       self -> add, /inHeader, $
         tags = ['qtyObj', depLabel, 'specObj::Zip'], $
;         tags = ['qtyObj', string(depLabel, '_zipper'), 'specObj::Zip'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
      endif else begin
       self -> add, /inData, $
         tags = ['qtyObj', depLabel, 'specObj::Zip'], $
;         tags = ['qtyObj', string(depLabel, '_zipper'), 'specObj::Zip'], $
         contents=obj_new('qtyObj',  $
	   name = depName, units=depUnits, tname = depTname, styleErr=depStyleErr, $
	  qty=depQty,$
	  err=depErr, nerr=depNerr, $	; Note: undefined keywords are ignored
 	  legend=depLegend)
       OK = self  -> setTagField( /inData, $
      		['qtyObj', depLabel, 'specObj::Zip'], $
;      		['qtyObj', string(depLabel, '_zipper'), 'specObj::Zip'], $
   		pendent = depPendent)
       print, infolabel, depPendent, "=depPendent. inData. tags = ", $
       		['qtyObj', depLabel, 'specObj::Zip']
;       		['qtyObj', string(depLabel, '_zipper'), 'specObj::Zip']
	
      endelse

      ;--- modify the focus if necessary
;      if (debug) then begin 
;        print, infolabel, strcmp(theZtag[1], depLabel, /fold_case), $
;     		 "=strcmp(", theZtag[1], ", ", depLabel, ", /fold_case)"
;      endif

;      if ( strcmp(theZtag[1], depLabel, /fold_case)) then begin	; not safe!
;;      if (theZtag[1] EQ depLabel) then begin	; not safe!
;        new_zlabel = theztag[1] + '_zip'
;	new_zCodeSource = 'specObj::Zip'
;      endif
      
      if ( strcmp(new_ylabel, depLabel, /fold_case)) then begin	; not safe!
        new_ylabel = deplabel 
;        new_ylabel = deplabel  + '_zipper'
      endif
      
    endif else begin
      ; The zipped field is trivial, so not worth saving.
    endelse

   ;-------- Make temporary variables all undefined, so that 
   ;-------- add method only uses those keywords with defined variables.
   DAVE_undefine, dep1Qty, dep1Err, dep1NErr, dep1Legend
   DAVE_undefine, depQty, depErr, depNErr, depLegend
;   if (n_elements(dep1Qty) GT 0) then begin
;     junk = temporary(dep1Qty)	; Make del1Qty undefined for a bit.
;   endif
;   if (n_elements(dep1Err) GT 0) then begin
;      junk = temporary(dep1Err)	; Make del1Err undefined for a bit.
;   endif
;   if (n_elements(dep1Nerr) GT 0) then begin
;      junk = temporary(dep1NErr)	; Make del1NErr undefined for a bit.
;   endif
;   if (n_elements(dep1Legend) GT 0) then begin
;      junk = temporary(dep1Legend)	; Make del1legend undefined for a bit.
;   endif

;   if (n_elements(depQty) GT 0) then begin
;     junk = temporary(depQty)	; Make delQty undefined for a bit.
;   endif
;   if (n_elements(depErr) GT 0) then begin
;      junk = temporary(depErr)	; Make delErr undefined for a bit.
;   endif
;   if (n_elements(depNerr) GT 0) then begin
;      junk = temporary(depNErr)	; Make delNErr undefined for a bit.
;   endif
;   if (n_elements(depLegend) GT 0) then begin
;      junk = temporary(depLegend)	; Make dellegend undefined for a bit.
;   endif

   skip_treatment:
   
 endfor

 ;*********************************************************************************
   self -> add, /inHeader, $
        tags = ['qtyObj', '#OfZipFiles', 'specObj::Zip'], $
     	contents=obj_new('qtyObj', name='#OfZipFiles', qty= nObjs )

   ;--- Store updated ICP scan type -----------------------------------
   ; Note: Only ICP-dependent command here.
;   if (OK_ICP) then begin
      OK = self -> setField(/inHeader, $
        ['qtyObj', 'type', '*'], qty = 'ZIP')	; for zip 
;   endif

 ;---- Update the specObj history. Has already been removed from the
 ; ---- header section. Needs work. -------------------------
 if (NOT keyword_set(thisNoTreatmentHistory)) then begin
   treatString = "OK = specObj -> Zip(  $"
   treatString = [treatstring,  " [appendages], $"]	; Needs fixing

   if (n_elements(thisDepField) NE 0) then begin
     treatString = [treatstring,$
 	 string(" , depField=['", thisDepField[0], "', '", $
	 	 thisDepField[1], "', '",thisDepField[2], "] $")]
   endif

   treatString = [treatString, ")"]
 
   OK = SpecObj -> appendField( /inHeader, $
  	['qtyObj', 'SpecObjTreatment', 'specObj::init'],$
	qty= treatString ) 

;   self -> add, /inHeader,$
;  	tags = ['qtyObj', 'SpecObjTreatment', 'specObj::init'], $
;        contents=obj_new('qtyObj',  $
;	  name = 'Object treamtment history',$
;	  qty= treatString ) 
 endif
 
 OK = self -> getField( /inHeader, $
      ['qtyObj', 'filename', '*'], $
      qty=theFilenames)
 nlen = n_elements(theFilenames) 
 if (nlen EQ 0) then theName = "<IN_MEMORY>"
 if (nlen EQ 1) then begin
   theName = "<IN_MEMORY:" + theFilenames + ">"
 endif else begin
   theName = "<IN_MEMORY:" + theFilenames[0] + '...' + theFilenames[(nlen-1)] + ">"
 endelse
 
 OK = self -> setField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      name='SpecObj_filename', $	; over-riden, not zipped
      qty=theName)
 OK = self -> setTagField( /inHeader, $
      ['qtyObj', 'SO_filename', '*'], $
      codeSource='specObj::Zip')
      
; self -> add, /inHeader, $
;      tags = ['qtyObj', 'filename', 'specObj::Zip'], $
;      contents=obj_new('qtyObj',  name = 'filename', $
;      	qty="<IN_MEMORY>")


  ;**************************************************************
  ;--- Modify the focus --------------------
  ; Keep the old x focus.
  self  -> setFocus, cutTags = ['', '', ''], $
     	zTags = ['qtyObj', new_zlabel, new_zCodeSource]

  if (n_elements(thisDepField) GT 0) then begin
;    new_ylabel = thisdepField[1]
    self  -> setFocus, $ 
;   	yTags = ['qtyObj', thisdepField[1], 'specObj::Zip']
   	yTags = ['qtyObj', new_ylabel, 'specObj::Zip']
  endif 

  if (debug) then begin
    print, infolabel, "Changing z focus from [",theztag, "] to [", $
  	 ['qtyObj', new_zlabel, new_zCodeSource]   , "]"
    print, infolabel, "Changing y focus to [",  $
    	['qtyObj', new_ylabel, 'specObj::Zip'] , "]"
  endif

  if (n_elements(thisErrMsg) GT 0) then thisErrMsg = ''	; no problems
  
  return, OK
  
end		; specObj::zip

;=========================================================================================
; ICP-specific at the moment.
;*
function specObj::scaleByMonitor, $	; Returns 0/1 for failure/success
	thisByMonitor, $ 	; Optional new value of monitor. Defaults to 1.
	rawCounts = thisRawCounts, $	; Defaults to ['qtyObj', 'COUNTS', '*']
	rawTime = thisRawTime, $	; Defaults to ['qtyObj', 'MIN', '*']
	rawMonitor = thisRawMonitor, $	; Defaults to ['qtyObj', 'MONITOR', '*'] in header
;Old			; or ['qtyObj', 'MONITOR_append' or 
;Old			; ['qtyObj', 'MONITOR_ZIPPER', '*'] in data section
	newIntensity = thisNewIntensity, $ ; Defaults to ['qtyObj', 'INTENSITY', '*']
	newTime = thisNewTime, $		; Defaults to ['qtyObj', 'TIME', '*']
	noReplace = thisNoReplace, $ ; Optional Boolen keyword. Don't replace 
				; newIntensity and/or newTime if pre-exist.
        errMsg = thisErrMsg  	; Optional keyword String. Returns error message.
				; Null string '' if sucessful.
;/
;	thisByMonitor,$ 	; Optional new value of monitor. Defaults to 1.
;				; Also modified field 'min' if present
;				; and modified the header.


  infolabel = "specObj::scaleByMonitor >> "
  debug = 0
  debug2 = 1

  success = 1
  
  if (debug2) then begin
    listObjs = obj_valid(count=nobj)
  endif
  
   thisErrMsg = ''
;  if (debug) then MESSAGE,/INFO, noprint=keyword_set(thisSilent), 'Entered routine'

  if (n_elements(thisByMonitor) EQ 0) then begin
    thisByMonitor = 1.0
  endif

  if (n_elements(thisRawCounts) EQ 0) then begin
    thisRawCounts = ['qtyObj', 'COUNTS', '*']	
  endif

  if (n_elements(thisRawTime) EQ 0) then begin
    thisRawTime = ['qtyObj', 'min', '*']
  endif

  if (n_elements(thisRawMonitor) EQ 0) then begin
    thisRawMonitor = ['qtyObj', 'monitor', '*']
  endif

  if (n_elements(thisNewIntensity) EQ 0) then begin
    thisNewIntensity = ['qtyObj', 'INTENSITY', 'specObj::scaleByMonitor']	
  endif
  if (thisNewIntensity[2] EQ '*') then begin
    thisNewIntensity[2] = 'specObj::scaleByMonitor'
  endif
  name_newIntensity = 'Intensity scaled by monitor'
  if (thisByMonitor NE 1.0) then begin
    name_newIntensity = string(name_newIntensity , '=', $
    	strcompress(string(thisByMonitor),/remove_all))
  endif
;  print, infolabel, "Intensity name =", name_newIntensity 
  ; "Ratio of Signal to Monitor Intensities"

  if (n_elements(thisNewTime) EQ 0) then begin
    thisNewTime = ['qtyObj', 'TIME', 'specObj::scaleByMonitor']
  endif
  if (thisNewTime[2] EQ '*') then begin
    thisNewTime[2] = 'specObj::scaleByMonitor'
  endif
  name_newTime = 'Time scaled by monitor'
  if (thisByMonitor NE 1.0) then begin
    name_newTime = string(name_newTime, '=', $
    	strcompress(string(thisByMonitor), /remove_all))
  endif


  ;---------------------------------------------------------
  OK = self -> getField (thisRawCounts,  $ 
 	/inData, $
        units  = unitsCounts, $
	styleErr=styleErrCounts, $
  	ptr_qty=ptr_qtyCounts, ptr_err= ptr_perrCounts, ptr_nerr= ptr_nerrCounts)  

  if (not OK) then begin
    thisErrMsg = string("Can not find RawCounts=",$ 
 	'[', thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], ']')
	
    if (debug) then message, /info, thisErrMsg
    goto, the_end
;    return, 0
  endif
  
  if ( NOT strcmp(unitsCounts, 'Events', /fold_case)) then begin
    message, /info, string('WARNING! [', $
    	thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], '] should have ', $
	'units of "Events" not ', unitsCounts)
  endif

  if ( NOT strcmp(styleErrCounts, 'poisson', /fold_case)) then begin
    message, /info, string('WARNING! [', $
    	thisRawCounts[0], ',', thisRawCounts[1], ',', thisRawCounts[2], '] should have ', $
	'poisson statitics for appropriate for raw counts, not ', styleErrCounts)
  endif

  ;---------------------------------------------------------
  OK_time = self -> getField (thisRawTime, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
       /inData, $
        units=unitsMin, $
  	ptr_qty=ptr_qtyMin)  

  if (not OK_time) then begin
    thisErrMsg = string("Warning. Can not find RawTime=",$ 
 	'[', thisRawTime[0], ',', thisRawTime[1], ',', thisRawTime[2], ']')
	 
    if (debug) then message, /info, thisErrMsg
    ; Not a fatal error
;;;    return, 0
  endif

  ;---------------------------------------------------------
  OK = self -> getField (thisRawMonitor, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
        /inHeader, $
        units=unitsMonitor, $
  	qty=oldMonitor)  		; scalar or possibly a short array

  if (not OK) then begin
    if (debug) then message, /info, string("Can not find RawMonitor=",$ 
 	'[', thisRawMonitor[0], ',', thisRawMonitor[1], ',', thisRawMonitor[2], $
	'] in header ')
    OK = self -> getField (thisRawMonitor, $ ; gets it from either header or data
  		; section since neither inData nor inHeader is specified.
        /inData, $
        units=unitsMonitor, $
  	qty=OldMonitor)  

    if (not OK) then begin
      if (debug) then message, /info, string("Can not find RawMonitor=",$ 
 	'[', thisRawMonitor[0], ',', thisRawMonitor[1], ',', thisRawMonitor[2], $
	'] in data')
      thisErrMsg = [thisErrMsg,$
       string("Can not find RawMonitor=",$ 
 	'[', thisRawMonitor[0], ',', thisRawMonitor[1], ',', thisRawMonitor[2], $
	']')]
      goto, the_end
;      return, 0
    endif
  endif

 ;------------------------------------------------------------------------------
 ; Make the scaled values!
 qtyCounts = float( double(*(ptr_qtyCounts)) * thisByMonitor / oldMonitor) 
 ptr_free, ptr_qtyCounts
 if (ptr_valid(ptr_perrCounts)) then begin
    perrCounts = float( double(*(ptr_perrCounts)) * thisByMonitor / oldMonitor) 
    ptr_free, ptr_perrCounts
 endif
 if (ptr_valid(ptr_nerrCounts)) then begin
   nerrCounts = float( double(*(ptr_nerrCounts)) * thisByMonitor / oldMonitor) 
   ptr_free, ptr_nerrCounts
 endif

  if (debug2) then begin
    listObjsMid = obj_valid(count=nobjMid)
    print, infolabel, "Before remove. Total # objs=", nobjMid
  endif

 if (NOT keyword_set(thisNoReplace)) then begin		; double negative
   self -> remove, position =  thisNewIntensity, /destroy  
   self -> remove, position =  thisNewTime, /destroy  
   self -> remove, position = ['qtyObj', 'newMonitor', 'specObj::scaleByMonitor'], $
   	/destroy  
 endif

  if (debug2) then begin
    listObjsMid = obj_valid(count=nobjMid)
    print, infolabel, "After remove. Total # objs=", nobjMid
  endif

 addErrMsg = ''
 self -> add, /inData, $
 	tags=thisNewIntensity, $
	errMsg = addErrMsg, $
	contents = obj_new('qtyObj', $
      		name = name_newIntensity, $
		units = "Arbritary", $
		StyleErr = '', $		; Now normal gaussian errors.
  		qty=qtyCounts, err= perrCounts, nerr=nerrCounts)

 OK = self -> setTagField( thisNewIntensity, /inData, $
 	pendent = 'D')		; dependent variable
	
 if (addErrMsg NE '') then begin
   thisErrMsg = [thisErrMsg, addErrMsg]
 endif
 
  if (debug2) then begin
    listObjsMid = obj_valid(count=nobjMid)
    print, infolabel, "After add newIntensity. Total # objs=", nobjMid
  endif

 if (OK_time) then begin

   if (thisByMonitor EQ 1.0) then begin
    unitsMin = unitsMin + '/Events_monitor'
   endif 
   
   qty_min = float( double(*(ptr_qtyMin)) * thisByMonitor / oldMonitor) 
   ptr_free, ptr_qtyMin
   self -> add, /inData, $
   	tags=thisNewTime, $
	errMsg = addErrMsg, $
	contents = obj_new('qtyObj', $
      		name = name_newTime, $
		units = unitsMin, $
  		qty=qty_min)
   OK = self -> setTagField( thisNewTime, /inData, $
 	pendent = 'I')		; independent variable (hopefully not const!)

   if (addErrMsg NE '') then begin
     thisErrMsg = [thisErrMsg, addErrMsg]
   endif
 endif

  if (debug2) then begin
    listObjsMid = obj_valid(count=nobjMid)
    print, infolabel, "After add newTime. Total # objs=", nobjMid
  endif

 
 if (thisByMonitor NE 1.0) then begin
   self -> add, /inHeader, $
   	tags=['qtyObj', 'newMonitor', 'specObj::scaleByMonitor'], $
	errMsg = addErrMsg, $
	contents = obj_new('qtyObj', $
      		name = 'Artifical monitor used in scaling', $
		units = 'Events', $
  		qty=thisByMonitor)
   if (addErrMsg NE '') then begin
     thisErrMsg = [thisErrMsg, addErrMsg]
   endif
 endif

  if (debug2) then begin
    listObjsMid = obj_valid(count=nobjMid)
    print, infolabel, "After add newIMonitor. Total # objs=", nobjMid
  endif

 OK = 1

  if (debug2) then begin
    listObjs2 = obj_valid(count=nobj2)
    print, infolabel, "change in # of objects = ", (nobj2 -nobj), "Total#=", nobj2
  endif
  
 
 the_end:

 print, infolabel, OK, "=OK. errMsg = '", thisErrMsg, "' strlen=", strlen(thisErrmsg)
 return, OK

end		;; specObj::scaleByMonitor

;============================================================================
;+
; NAME:
; 	specObj::convert
;
; PURPOSE:
; 	This routine fills an object of class SpecObj from the information in davePtr.
;	This includes:
; 		1. The contents of the structure that specficPointer points to.
;		2. The contents of histPtr
; 	It replaces the specific pointer's anonymous structure 
;	with either one only containing the new object (if keyword /replace is used)
;	or it then places that object at the end of the structure.
; 	If the keyword "/replace" is used, the structure is only only {specObj:SpecObj}
; 	and the original contents are removed.
;-
;*
function  specObj::convert, $	; Returns boolean 0/1 for failure/success
  thisDavePtr, $
  currentCodeSource, $
  replace = thisReplace		; Optional boolean keyword.
;/
   ;  Replaces structure's members with specObj. Default is to suppliment.
   ;  based on specStruct2SpecObj in file davePtr_utils.pro
   infolabel = "specObj::convert >> "
   debug = 0	;1

  if (n_elements(thisDavePtr) eq 0) then begin
    print,  infolabel, $
    	' Require a single DAVE pointer as input parameter'
    print
    return, 0
  endif

  if (not ptr_valid(thisDavePtr)) then begin
    print, infolabel, ' Requires a valid DAVE pointer as input parameter'
    print
    return, 0
  endif

  if (n_elements( (*thisDavePtr)) EQ 0  ) then begin
    print, infolabel, ' Requires a valid DAVE pointer as input parameter'
    print
    return, 0
  endif
  
  if (NOT PTR_VALID( (*thisDavePtr).dataStrPtr)  ) then begin
    print, infolabel, ' Requires a valid data structure in DAVE pointer'
    print
    return, 0
  endif
  
  if (n_elements( (*(*thisDavePtr).dataStrPtr))  EQ 0 ) then begin
    print, infolabel, ' Requires a valid data structure in DAVE pointer'
    print
    return, 0
  endif
  
  if (n_elements( (*(*thisDavePtr).dataStrPtr).CommonStr)  EQ 0 ) then begin
    print, infolabel, ' Requires a valid common data structure in DAVE pointer'
    print
    return, 0
  endif
  
  if (n_elements( (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr))  EQ 0 ) then begin
    print, infolabel, ' Requires a valid data histogram structure in DAVE pointer'
    print
    return, 0
  endif
  
  if (debug) then begin
     tempPtrs = ptr_valid(count=nPtrs)
     message, /info, string("ptr_valid(self.davePtr) = ", $
  	ptr_valid(self.davePtr), " nPtrs=", nPtrs )
  endif
;  ; Added code, Sept 16, 2003. TMR        
;	tempObj = davePtr_getSpecObj(self.DavePtr, from='specObj::convert')
;	if (self NE tempObj) then begin	; compare objRef ID's
;	  davePtr_destroy, self.DavePtr, /destroySpecObj		; Saves only the top ID#
;	endif else begin
;	  davePtr_destroy, self.DavePtr		; Saves only the top ID#
;	endelse
;	heap_free, self.DavePtr		; Empty the rest, including top ptr. TMR. Sept 15, 2003.
;        ptr_free, self.DavePtr

  self.davePtr = thisDavePtr
  if (debug) then message, /info, string("ptr_valid(self.davePtr) = ", $
  	ptr_valid(self.davePtr))
  
  success = 1

  nsuc = 0
  nfail = 0
  nvar = 0

; Need to think about this
; ; ----------------------------------------------------------------------------  
  if (ptr_valid( (*(*thisDavePtr).dataStrPtr).CommonStr.treatmentPtr)) then begin
   ; first put the treatment pointer in header portion of the specObj.
    nsuc = nsuc + 1
    nvar = nvar + 1
    self -> add, /inHeader,$
    	tags=['qtyObj', 'DavePtrTreatment', currentCodeSource], $
   	contents=obj_new('qtyObj', name = 'Treatment history', $
		qty=  *((*(*thisDavePtr).dataStrPtr).CommonStr.treatmentPtr) ) 
  endif

 ; ----------------------------------------------------------------------------  
 ; Copy the original contents of x, y, and hist/qty from CommonStr and histPtr
 ; Note: The use of the temporary pointer temp_ptr, allows the object self
 ; to get the data by reference, and not waste time copying it by value.

  if (ptr_valid( (*(*thisDavePtr).dataStrPtr).CommonStr.histPtr)) then begin
     histTags = tag_names( (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr) )
  
     currentXLabel = ''
     index1 = where( histTags EQ 'X',count)
     if (count eq 1) then begin
       currentXLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.xlabel 
       currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.xunits
        
       ;Copy by value not reference, because the common area may get abused later.
       if (n_elements( $
       	 (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) NE 1) then begin
         nsuc = nsuc + 1
	 nvar = nvar + 1
	 self -> add, /inData, $
          tags=['qtyObj', currentXLabel, currentCodeSource], $
	  contents=obj_new('qtyObj', name = currentXLabel, $
		units = currentUnits,  $		;; no legend
		qty =  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) 
       endif else begin
         nsuc = nsuc + 1
	 nvar = nvar + 1
	 self -> add, /inHeader, $
          tags=['qtyObj', currentXLabel, currentCodeSource], $
	  contents=obj_new('qtyObj', name = currentXLabel, $
		units = currentUnits,  $		;; no legend
		qty =  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).x) 
	 ; this is presumably a cut.
          self  -> setFocus, $ 
	    cutTags = ['qtyObj', currentXLabel, currentCodeSource]
       endelse
     endif
     
     currentYLabel = ''
     index1 = where( histTags EQ 'Y',count)
     if (count eq 1) then begin
       currentYLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.ylabel 
;;;       currentLegend = (*(*thisDavePtr).dataStrPtr).CommonStr.ytype
       currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.yunits
       if (n_elements( $
       	 (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y) NE 1) then begin
         nsuc = nsuc + 1
	 nvar = nvar + 1
	 self -> add, /inData, $
       	    tags=['qtyObj', currentYLabel, currentCodeSource], $
   	    contents=obj_new('qtyObj', name = currentYLabel, $
		units = currentUnits, $	;; no legend
		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y ) 
       endif else begin
	 ; this is presumably a cut.
          nsuc = nsuc + 1
	  nvar = nvar + 1
	  self -> add, /inHeader, $
       	    tags=['qtyObj', currentYLabel, currentCodeSource], $
   	    contents=obj_new('qtyObj', name = currentYLabel, $
		units = currentUnits, $	;; no legend
		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).y ) 
          self  -> setFocus, $ 
	    cutTags = ['qtyObj', currentYLabel, currentCodeSource]
       endelse
     endif
     
     index1 = where( histTags EQ 'ERR',count)
     if (count eq 1) then begin
       useERR = 1
     endif else begin
       useErr = 0
     endelse
     nlen = n_elements((*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err)
     if (nlen eq 1) then begin
       if ((*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err EQ 0.0) $
       		 then begin
         useERR = 0
       endif
     endif
     
     currentZLabel = (*(*thisDavePtr).dataStrPtr).CommonStr.histlabel 
     currentUnits = (*(*thisDavePtr).dataStrPtr).CommonStr.histUnits
     if (useERR) then begin
       nsuc = nsuc + 1
       nvar = nvar + 1
       self -> add, /inData, $
       	    tags=['qtyObj', currentZLabel, currentCodeSource], $
   	    contents=obj_new('qtyObj', name = currentZLabel, $
		units = currentUnits, $
		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).qty,$
		err=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).err )
     endif else begin
       nsuc = nsuc + 1
       nvar = nvar + 1
       self -> add, /inData, legend=currentZLabel, $
       	    tags=['qtyObj', currentZLabel, currentCodeSource], $
   	    contents=obj_new('qtyObj', name = currentZLabel, $
		units = currentUnits, $
		qty=  (*(*(*thisDavePtr).dataStrPtr).CommonStr.histPtr).qty)
     endelse
   endif

   ;; Focus defaults to the values originally stored in commonPtr.
    if ((currentXlabel NE '') AND (currentYlabel NE '')) then begin
      self  -> setFocus, $ 
   	xTags = ['qtyObj', currentXLabel, currentCodeSource], $
   	yTags = ['qtyObj', currentYLabel, currentCodeSource], $
   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
    endif else begin

      if (currentYlabel NE '') then begin
        self  -> setFocus, $ 
   	yTags = ['qtyObj', currentYLabel, currentCodeSource], $
   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
      endif  

      if (currentXlabel NE '') then begin
        self  -> setFocus, $ 
   	XTags = ['qtyObj', currentXLabel, currentCodeSource], $
   	zTags = ['qtyObj', currentZLabel, currentCodeSource]
      endif  

    endelse    

   ;-------------------------------------------------------------------------
   someTags = tag_names( (*thisDavePtr))
   index1 = where( someTags EQ 'DESCRIPTR',count)
   if (count eq 1) then begin
    if (ptr_valid( (*thisDavePtr).descriPtr)) then begin
     currentDLabel = (*(*thisDavePtr).descriPtr).name
     currentDLegend = (*(*thisDavePtr).descriPtr).legend
       nsuc = nsuc + 1
       nvar = nvar + 1
       self -> add, /inHeader,  $
       	    tags=['qtyObj', currentDLabel, currentCodeSource], $
   	    contents=obj_new('qtyObj', $
	        name = currentDLabel, $
		units = (*(*thisDavePtr).descriPtr).units, $
		legend = currentDLegend, $
		qty = (*(*thisDavePtr).descriPtr).qty, $
		err = (*(*thisDavePtr).descriPtr).err) 
      self  -> setFocus, $ 
	descripTags = ['qtyObj', currentDLabel, currentCodeSource], $ 
	descripLegend = currentDLegend
    endif
   endif 

 ;-------------------------------------------------------------------------
 ; SpecPtr points to a struture full of whatever the instrument scientist wanted
 ; to keep. Copy the contents over into self.
    if (ptr_valid( $
            (*(*thisDavePtr).dataStrPtr).SpecificPtr )) then begin
       tname = size( *(*(*thisDavePtr).dataStrPtr).SpecificPtr, /tname)
       if (debug) then $
       		print, infolabel, 'SpecficPointer is pointing to a ', tname
       if (tname NE 'STRUCT') then begin
         print, infolabel, 'Non-standard. Should be to an anonymous structure.' 
         print, infolabel, 'No modification of (*(*thisDavePtr).dataStrPtr).specificPtr done.' 
	 success = 0
       endif else begin			;;  tname EQ STRUCT
         ; create a temporary pointer SpecPtr for ease of use
         specPtr = ptr_new(*(*(*thisDavePtr).dataStrPtr).SpecificPtr)
	 specTags = tag_names(*specPtr)
	 specStructName = tag_names(*specPtr, /structure_name)
         ntags = n_tags(*specPtr)

	 ;; Check to see if a specfic object already exists in the structure.
	  specObj_preexists = 0
	 index1 = where( (specTags EQ 'SPECOBJ'), count1)
	 if (count1 NE 0) then begin
	    print, infolabel, 'specificStructure already has a specObj field.'
	    print, infolabel, 'Will destroy it and then re-create it.' 
	    specObj_preexists = 1
	    obj_destroy, (*specPtr).specObj
	    success = 0
	 endif

	 if (specStructName NE '') then begin
	   print, infolabel, "SpecificPtr's structure has name '", $
	   	specStructName, "'"
	 endif

	 ; Fill up the rest of specObj using the fields of the original structure.
	 for i = 0, (ntags-1) do begin
	   fieldTName = size((*specPtr).(i), /tname)
	   if (fieldTName EQ 'OBJREF') then begin
	       print, infolabel, 'specObj skipping OBJREF ', $
			 specTags[i]
;06082003			 fieldTagTName
	   endif else begin
	     if (fieldTName EQ 'POINTER') then begin
	       temp_ptr = ptr_new( *((*specPtr).(i)) )
	       if (debug) then print, infolabel, specTags[i], " is a ", fieldTName, $
	       	 " of dim=", size(*(temp_ptr), /dim )
	       if (debug) then print, infolabel, specTags[i], " is a ", $
	       		fieldTName, " with. val[0]=", $
	     		(*(temp_ptr))[0] 
	     ; Note: add() decides where to place in (inHeader or in Data)
	     ; based on the dimensions of the quantity ptr_qty=((*specPtr).(i)).
	      nsuc = nsuc + 1
	      self -> add, $
	        tags = ['qtyObj', specTags[i], 'specObj::Convert'], $
   		contents=obj_new('qtyObj', name = specTags[i], $
		  ptr_qty=temp_ptr) 
		ptr_free, temp_ptr 
	     endif else begin
	       if (fieldTName EQ 'STRUCT') then begin
	         ; avoid a bit of recursion, if not needed.
	         print, infolabel, 'specObj skipping STRUCTURE ', $
			 fieldTageName
;;		 nfail = nfail + 1
	       endif else begin
	         temp_ptr = ptr_new(((*specPtr).(i)) )
	         if (debug) then print, infolabel, specTags[i], " is a ARRAY ", fieldTName, $ 
	       		 " of dim=", size(*(temp_ptr), /dim)
	         if (debug) then print, infolabel, specTags[i], " is a ARRAY ", $
	        	fieldTName, " with val[0]=", $
	     		(*(temp_ptr))[0] 
	        ; Note: add() decides where to place in (inHeader or in Data)
	        ; based on the dimensions of the quantity ptr_qty=temp_ptr.
	      	 nsuc = nsuc + 1
		 self -> add, $
	           tags = ['qtyObj', specTags[i], 'specObj::Convert'], $
   		   contents=obj_new('qtyObj', name = specTags[i], $
		  	ptr_qty=temp_ptr ) 
		 ptr_free, temp_ptr 
	       endelse
	     endelse
	   endelse
	 endfor

	 if (nsuc NE (ntags+nvar)) then begin
	   success = 0
	   print, infolabel, 'Success on only ', $
	   	strcompress(string(nsuc), /remove_all), $
	   	' out of ', strcompress(string(ntags), /remove_all),$
		 ' fields in the structure plus ', $
		 strcompress(string(nvar), /remove_all), $
		 ' items in histPtr and descriPtr.'
	 endif	 
	 
	 ; replace the old struture with a new anonymous one.
	 if (keyword_set(thisReplace)) then begin
	   structure_cleanup, specPtr
 	   ptr_free,  (*(*thisDavePtr).dataStrPtr).SpecificPtr
           (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
      			ptr_new( {specObj:self})
	 endif else begin
 	   ptr_free,  (*(*thisDavePtr).dataStrPtr).SpecificPtr
	   if (specObj_preexists) then begin
	     (*specPtr).specObj= self 
             (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
      		ptr_new( (*specPtr) )
	   endif else begin
            (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
      		ptr_new( create_struct((*(specPtr)), {specObj:self}) )
	   endelse

	 endelse
	 ptr_free, specPtr
       endelse

    endif else begin
      ; Nothing stored in specficPointer, but we already loaded some other 
      ; infomation into specObj.
      (*(*thisDavePtr).dataStrPtr).SpecificPtr = $
      		ptr_new( {specObj:self})
      

    endelse
   
   ; Remove useless items from container. 
   self -> remove,  /all, /ifDataConstant, /inData, /destroy
   
  ;-------------------------------------------------
    self -> setFocus, $ 
        mainTitleTags = ['qtyObj', 'mainTitle', 'specObj::Convert']
    if (debug) then  print, "specObj -> setFocus, cutTags = ..."
    self  -> setFocus, $ 
	cutTags = ['qtyObj', 'specific_cut', 'specObj::Convert']

 ;-------------------------------------------------------------------------
  if (debug) then print, infolabel, 'Exiting'
  return, success

end

;;================================================================================
;+
; NAME:
; 	specObj::callFunction
;
; PURPOSE:
; 	Calls a function which MIGHT act on the self object, only via methods.
;  Functions are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The function's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	If used as a "friend" method, the second argument should probably be
; object itself. E.g., 
; 	result = myObject -> callFunction( 'SomeFunction', myObject, ...)
; Yes, that is a bit redundent, but means you can also call
; 	result = myObject -> callFunction( 'SomeFunction', anotherObject, ...)
; and have the function do NOTHING at all with myObject's data!
; This might be useful for widget objects.
;
;	The methods specObj::callFunction, specObj::callProcedure and
; specObj::callMethod could be used to impliment an user interface 
; which allows the user to select functions, procedures and methods to
; apply. 
;
;-
;*
 function specObj::callFunction, $	; returns whatever functionName returns.
  functionName, $	; A string.  don't include *.pro extension
  thisArg1, $		; Any type. Might be an object for "friend" methods.
  thisArg2, $		; Any type. 
  thisArg3, $		; Any type.
  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
;/

  ; Calls a function which MIGHT act on the self object, only via methods.
;WRONG  _REF_EXTRA = refextraExtra	; parameters and keywords. ONLY ONES CALL BY REFERENCE!
    ;------ where to do it ----------------- 
;  inHeader = thisInHeader, $		; Boolean keyword
;  inData = thisInData, $ 		; Boolean keyword

  infolabel = "specObj::callFunction >> "  


  nparm = n_params() - 1	; Don't count function name!

  if (n_elements(extraExtra) gt 0) then begin
    if (nparm eq 0) then begin
      ret = call_function( functionName, $
      	_EXTRA=extraExtra)
    endif
    if (nparm eq 1) then begin
      ret = call_function( functionName, thisArg1,$
      	 _EXTRA=extraExtra)
    endif
    if (nparm eq 2) then begin
      ret = call_function( functionName, thisArg1, thisArg2,$
      	_EXTRA=extraExtra)
    endif
    if (nparm eq 3) then begin
      ret = call_function( functionName, thisArg1, thisArg2, thisArg3, $
      		 _EXTRA=extraExtra)
    endif
   endif else begin
    if (nparm eq 0) then begin
      ret = call_function( functionName)
    endif
    if (nparm eq 1) then begin
      ret = call_function( functionName, thisArg1)
    endif
    if (nparm eq 2) then begin
      ret = call_function( functionName, thisArg1, thisArg2)
    endif
    if (nparm eq 3) then begin
      ret = call_function( functionName, thisArg1, thisArg2, thisArg3)
    endif
  endelse
  return, ret 
end	;specObj::callFunction 	
	
;=========================================================================CO=======
;+
; NAME:
; 	specObj::callProcedure
;
; PURPOSE:
; 	Calls a procedure which MIGHT act on the self object, only via methods.
;  Procedures are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The procedures's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	If used as a "friend" method, the second argument should probably be
; object itself. E.g., 
; 	myObject -> callProcedure, 'SomeProcedure', myObject, ...
; Yes, that is a bit redundent, but means you can also call
; 	myObject -> callProcedure, 'SomeProcedure', anotherObject, ...
; and have the procedure do NOTHING at all with myObject's data!
; This might be useful for widget objects.
;
;	The methods specObj::callFunction, specObj::callProcedure and
; specObj::callMethod could be used to impliment an user interface 
; which allows the user to select functions, procedures and methods to
; apply. 
;
;-
;*
pro specObj::callProcedure, $
  proName, $		; A string. don't include *.pro extension
  thisArg1, $		; Any type. Might be an object for "friend" methods.
  thisArg2, $		; Any type.
  thisArg3, $		; Any type.
  _EXTRA = extraExtra	; call by value keywords. ONLY ONES CALL BY VALUE!
;/
  ; Calls a procedure which MIGHT act on the self object, only via methods. 
  
  infolabel = "specObj::callProcedure >> "

  nparm = n_params() - 1	; Don't count procedure name!

  if (n_elements(extraExtra) gt 0) then begin
    if (nparm eq 0) then begin
      call_procedure, proName, _EXTRA=extraExtra
    endif
    if (nparm eq 1) then begin
      call_procedure, proName, thisArg1,  _EXTRA=extraExtra
    endif
    if (nparm eq 2) then begin
      call_procedure, proName, thisArg1, thisArg2,  _EXTRA=extraExtra
    endif
    if (nparm eq 3) then begin
      call_procedure, proName, thisArg1, thisArg2, thisArg3, _EXTRA=extraExtra
    endif
  endif else begin
    if (nparm eq 0) then begin
      call_procedure, proName
    endif
    if (nparm eq 1) then begin
      call_procedure, proName, thisArg1
    endif
    if (nparm eq 2) then begin
      call_procedure, proName, thisArg1, thisArg2
    endif
    if (nparm eq 3) then begin
      call_procedure, proName, thisArg1, thisArg2, thisArg3
    endif
  endelse
  
  return
end		;; callProcedure

;================================================================================
;+
; NAME:
; 	specObj::callMethod
;
; PURPOSE:
; 	Calls a method which acts on the "self" object. 
; If /isPro set, this function returns 1 (for true)
; If /isFunction is set, it returns the value that that function returns.
;
; 	Methods are currenty limited to a maximum of 3 arguments 
;  (thisArg1, thisArg2, thisArg3). The method's keywords values can 
;  NOT be modified by function, a limitation of passing them using
;  the _EXTRA keyword. 
;
;	The methods specObj::callFunction, specObj::callProcedure and
; specObj::callMethod could be used to impliment an user interface 
; which allows the user to select functions, procedures and methods to
; apply. 
;
;-
;*
function specObj::callMethod, $	; if /isPro, returns 1.
			; if /isFunction, returns whatever methodName returns.
  methodName, $				; A string. Name of method to use.  
  	; The method is assumed to be a function unless otherwise indicated
	; with /isFunction. /isPro is provided for convenience.
  isFunction = thisIsFunction, $	; the default. 
  				; Indicates methodName has a return value.
  isPro = thisIsPro, $		; Indicates methodName has no return value.
  thisArg1, $			; Any type.
  thisArg2, $			; Any type.
  thisArg3, $			; Any type.
  _EXTRA = extraExtra		; call by value keywords. ONLY ONES CALL BY VALUE!
;/

;;  objref = thisObjRef, $	; optional.
  ; Calls a method which acts on the "self" object, which is selected
  ; from the choices thisObjRef, self.tags or self.contents or self.
  ; If /isPro set, this function returns 1 (for true)
  ; If /isFunction is set, it returns the value that that function returns.
    
    
  infolabel = "specObj::callMethod >> " 
  hasReturnVal = 1
  if (keyword_set(thisIsFunction)) then hasReturnVal = 1
  if (keyword_set(thisIsPro)) then hasReturnVal = 0

  nparm = n_params() - 1	; Don't count method name!
  
;    ;---------- if no instructions given via keywords "/using*"
    ; default behavior
  if (hasReturnVal) then begin
      if (n_elements(extraExtra) gt 0) then begin
        if (nparm eq 0) then begin
          ret = call_method(methodName,  self, _EXTRA=extraExtra)
	endif
        if (nparm eq 1) then begin
          ret = call_method(methodName,  self, thisArg1,_EXTRA=extraExtra)
	endif
        if (nparm eq 2) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, $
	  	_EXTRA=extraExtra)
	endif
        if (nparm eq 3) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, thisArg3, $
	  	_EXTRA=extraExtra)
	endif
      endif else begin
        if (nparm eq 0) then begin
          ret = call_method(methodName,  self)
	endif
        if (nparm eq 1) then begin
          ret = call_method(methodName,  self, thisArg1)
	endif
        if (nparm eq 2) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2)
	endif
        if (nparm eq 3) then begin
          ret = call_method(methodName,  self, thisArg1, thisArg2, thisArg3)
	endif
      endelse
    return, ret 

  endif else begin
    ;---------- if no instructions given via keywords "/using*"
      if (n_elements(extraExtra) gt 0) then begin
        if (nparm eq 0) then begin
          call_method, methodName,  self, _EXTRA=extraExtra
	endif
        if (nparm eq 1) then begin
          call_method, methodName,  self, thisArg1,_EXTRA=extraExtra
	endif
        if (nparm eq 2) then begin
          call_method, methodName,  self, thisArg1, thisArg2, $
	  	_EXTRA=extraExtra
	endif
        if (nparm eq 3) then begin
          call_method, methodName,  self, thisArg1, thisArg2, thisArg3, $
	  	_EXTRA=extraExtra
	endif
      endif else begin
        if (nparm eq 0) then begin
          call_method, methodName,  self
	endif
        if (nparm eq 1) then begin
          call_method, methodName,  self, thisArg1
	endif
        if (nparm eq 2) then begin
          call_method, methodName,  self, thisArg1, thisArg2
	endif
        if (nparm eq 3) then begin
          call_method, methodName,  self, thisArg1, thisArg2, thisArg3
	endif
      endelse
    return, 1
    
  endelse
  
end		;; callMethod

;;================================================================================
;+
; NAME:
; 	specObj::Add
;
; PURPOSE:
;	Similar to IDL_container::Add and DAVE_tagContainer::Add.
;
;	Each object added must have a tag string array associated with it,
;	['','',''].
;
;	The optional keyword position has been extended to accept a
;	tag string ['','',''] or an index integer.
;
;	The optional keyword Esrrmsg returns an error message upon
;	error, while returning a NULL string '' upon success.
;	
;	The first level of the hiarachical tree has at least 2 containers,
;	one of which is identified as the "header" and the other as the "data"
;	containers. This as been done by use of the focusObj in self.focus.
;	The object added by  DAVE_tagContainer::Add
;	automatically gets placed down one level.
;	If /inHeader is specified, the objects gets placed in the header node.
;	If /inData is specified, the objects gets placed in the data node.
;	If neither /inHeader not /inData is specified, the object gets
;	placed automatically, with "small" objects placed in the header node
;	and "larger" objects in the data node. The size cut-off is a size of
;	4 for the largest dimension. The keyword DataMinSize can be used to 
;	over-ride this cut-off criterion.
;-
;*
 pro specObj::add, $
	; The following keywords are passed to  DAVE_tagContainer::Add
  tags = thisTag, $	  	; Required keyword.
				; string array tag ['','',''] for thisObject
  contents=thisObject, $	; Required keyword. 
				; Object to be added to container.
  position = thisPosition, $  $ ; Optional location to place, 
				; type integer (index) or string array (tag ['','','']).  
			  	; Defaults to the end.
  errMsg = thisErrMsg, $  	; Optional keyword String. Returns error message.
				; Null string '' if sucessful.
  ;------ where to put it ----------------- 
  ; If neither keyword appears, decide on basis of the dimensions of contents.
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $ 		; Boolean keyword
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore, $			; Boolean keyword.
  		; If used, keywords inHeader and inData and DataMinSize ignored
  ;------ Criterion for where to put it.  ----------------- 
  DataMinSize = thisDataMinSize		; Optional keyword integer.
  			; If neither /inHeader or /inData appear,
			; over-ride criterion of 4 for placing the data. 
			; Small objects go into header.
;/

  infolabel = "specObj::add >> "  

  success = 1
  debug = 0

  ;-------------------------------------------------------------- 
  if (keyword_set(thisInStore)) then begin
     (self.store) -> add, $
     	tags = thisTag, $	  ; string array tag for thisObject
	contents=thisObject, $		
	position = thisPosition, $
	errMsg = thisErrMsg  
     return
  endif
  

  ;-------------------------------------------------------------- 
  if (not (keyword_set(thisInHeader) or keyword_set(thisInData))) then begin
    if (n_elements(thisDataMinSize) eq 0) then begin
      thisDataMinSize = 4
    endif
;Upgrade later
    biggest_dim = thisObject -> count()
;    biggest_dim = maxDim(thisqty)
    
    if (biggest_Dim LE thisDataMinSize) then begin
      thisInHeader = 1
      thisInData = 0
    endif else begin
      thisInHeader = 0
      thisInData = 1
    endelse
  endif else begin
    if (not (keyword_set(thisInHeader))) then  thisInHeader = 0
    if (not (keyword_set(thisInData))) then  thisInData = 0
  endelse

  ;-------------------------------------------------------------- 
  OK = 1
  if (thisInHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    if (OK) then begin
      tempObj -> add, $
     	tags = thisTag, $	  ; string array tag for thisObject
	contents=thisObject, $		
	position = thisPosition, $
	errMsg = thisErrMsg  	  
;   (self.focus.headerObj) -> add, $
;    	tags = thisTag, $	  ; string array tag for thisObject
;	contents=thisObject, $		
;	position = thisPosition	  
    endif else begin
      print, infolabel, "Failure to find headerObj"
    endelse
  endif
  
  if (thisInData) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
    if (OK) then begin
      tempObj -> add, $
     	tags = thisTag, $	  ; string array tag for thisObject
	contents=thisObject, $		
	position = thisPosition, $
	errMsg = thisErrMsg    
    endif else begin
      print, infolabel, "Failure to find dataObj"
    endelse
;    (self.focus.headerObj) -> add, $
;    	tags = thisTag, $	  ; string array tag for thisObject
;	contents=thisObject, $		
;	position = thisPosition	  
  endif
 
  if (debug) then begin
    print, '---------------------'
    print, infolabel, 'InData=', thisInData,  ' InHeader=', thisInHeader
    help, tempObj, output=helpString
    if (thisInData) then begin
      print, '{specObj}.store."data" =', helpString
    endif
    if (thisInHeader) then begin
      print, '{specObj}.store."header" =', helpString
    endif
;    tempObj -> show, /recursive
;    print, 'tempObj -> show, /recursive'
    tempObj -> show, /heapNum
    print, 'tempObj -> show, /heapNum '
    print, '---------------------'
    print 
  endif
end		; specObj::Add

;================================================================================
;+
; NAME:
; 	specObj::Remove
;
; PURPOSE:
;	Similar to IDL_container::Remove and DAVE_tagContainer::Remove.
;
;	Items in the container can be removed either by refering to their
;	position (keyword position) or passing the object itself as
;	an argument. The use of the two boolean keywords keywords 
;	/inHeader or /inData is suggested. Otehrwise, the object is removed
;	from wherever is is found.
;
;	The defintion has been extended so that tags of the form ['','','']
;	can be used instead of index positions for the optional keyword 
;	position. 
;
;	Also, the optional boolean keyword destroy has been added.
;	if /destroy is used, then the object is destroyed when it is removed.
;	This is useful if the container contains the only reference to the
;	object. 
;
;	If the boolean keyword /ifDataConstant is set, then only those objects
;	which are constant (e.g. all members of an array identical) are removed.
;	This is useful for "pruning" non-varying data.
;-
;*
 pro specObj::remove, $
  ; To specify what to remove, 
  ; select one of the keywords child_object, position, or all.
  child_Object, $		; Object. optional
  position = thisPosition, $	; Optional. 
  				; Tag string array ['', '', '']or index integer.
  all  = thisALL, $			; Optional.
  ;---------------------------------
  destroy = thisDestroy, $	; Boolean keyword. Optional. 
  			; Destroys objects as they are removed.
			; Otherwise, make sure that something else does
			; the cleanup as only the reference will be removed.
  ;------ where to remove from ----------------- 
  inHeader = thisInHeader, $	; Boolean keyword. (self.focus specifies the header.) 
  inData = thisInData, $ 	; Boolean keyword. (self.focus specifies the data.) 
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore, $			; Boolean keyword.
  		; If used, keywords inHeader and inData  and ifDataConstant ignored
  ;----------------------- 
  errMsg = thisErrMsg, $	; String. Returns error message.
				; Null string '' if sucessful.
  verbose = thisVerbose, $	; Complains to screen if there is a problem.
  ifDataConstant = thisIfDataConstant	; Only coded for now for a single object 
;/

;;;  ifConstant = thisIfConstant		; Only coded for now for a single object 
					; or for /inData if use /all
					  
  infolabel = "specObj::remove >> "  

  success = 1
  debug = 0
  
  anErrMsg = ''

  ;-------------------------------------------------------------- 
  if (keyword_set(thisInStore)) then begin
      if (n_elements(child_object) GT 0)  then begin
        (self.store) -> remove, $
	  child_Object, $		; optional
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
      endif else begin
        (self.store) -> remove, $
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
      endelse
     return
  endif
  ;-------------------------------------------------------------- 
  
  ; remove from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif
;debug  print, infolabel, 'doHeader=', doHeader, ' doData=', doData 

  ;-------------------------------------------------------------- 
  nsuc = 0
  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    if (OK) then begin

      if (n_elements(child_object) GT 0)  then begin
;        childIsConst = child_Object -> isConstant()
;	if (keyword_set(thisIfConstant) AND (NOT childIsConst)) then begin
;	  goto, skipHeaderRemove
;	endif 

        tempObj -> remove, $
	  child_Object, $		; optional
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
      endif else begin
        tempChildObj = self->get( position = thisPosition, /inHeader)
;        childIsConst = tempChildObj -> isConstant()
;	if (keyword_set(thisIfConstant) AND (NOT childIsConst)) then begin
;	  goto, skipHeaderRemove
;	endif 

        tempObj -> remove, $
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
      endelse
;      skipHeaderRemove:

;      (self.focus.headerObj) -> remove, $
;	child_Object, $		; optional
;	position = thisPosition, $	; Optional. Tag string or index integer.
;	all  = thisALL, $			; Optional.
;	destroy = thisDestroy	; Optional. Destroys objects as they are removed
    endif else begin
      if (keyword_set(thisVerbose)) then $
      		print, infolabel, "Failure to find headerObj"
      anErrMsg = infolabel + "Failure to find headerObj"
    endelse

  endif
  
  if (doData) then begin
     OK = (self.focus) -> get(dataObj=tempObj)

    if (OK) then begin
     nchildren = n_elements(child_object)
     nall = 0
     if (keyword_set(thisAll)) then nall = tempObj -> count()
     
     if (NOT keyword_set(thisIfDataConstant)) then begin
       if (n_elements(child_object) GT 0)  then begin
         tempObj -> remove, $
	  child_Object, $		; optional
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
       endif else begin
         tempObj -> remove, $
	  position = thisPosition, $	; Optional. Tag string or index integer.
	  all  = thisALL, $			; Optional.
	  verbose = thisVerbose, $
	  destroy = thisDestroy	; Optional. Destroys objects as they are removed
       endelse
       goto, dataFinished
     endif

     if (nall GT 0) then begin	; ignore position and child_object
       ;---------- /all keyword set
       ;------- Multiple child_object must be dealt with one at a time
;debug      print, infolabel, "nall = ", nall
      for i = (nall-1), 0, -1 do begin
            tempChildObj = self->get( position = i, /inData)
;debug 	    print, infolabel, i, obj_valid(tempChildObj)
;debug 	    help, tempChildObj
            childIsConst = (tempChildObj) -> isConstant()
	    if (NOT childIsConst) then begin
	      ; do nothing 
	    endif else begin
              tempObj -> remove, $
	  	verbose = thisVerbose, $
	      	position = i, $	
	      	destroy = thisDestroy	; Optional. Destroys objects as they are removed
	  endelse
       endfor
       goto, dataFinished
     endif

    if (nchildren GT 0) then begin	; ignore position
      ;---------- child_object present
       ;------- Multiple child_object must be dealt with one at a time
      for i = (nchildren-1), 0, -1 do begin
            childIsConst = (child_Object[i]) -> isConstant()
	    if (NOT childIsConst) then begin
	      ; do nothing 
	    endif else begin

              tempObj -> remove, $
	      (child_Object[i]), $		; 
;;;	      position = thisPosition, $; IGNORED BECAUSE Child_object present.
;;;	      all  = thisALL, $		; Will be false, so no need to pass
	      verbose = thisVerbose, $
	      destroy = thisDestroy	; Optional. Destroys objects as they are removed
	  endelse
       endfor
       goto, dataFinished
     endif else begin
      ;---------- position keyword present. Assume only 1 position, not an array.
            tempChildObj = self->get( position = thisPosition, /inData)
            childIsConst = (tempChildObj) -> isConstant()
	    if (NOT childIsConst) then begin
	      ; do nothing 
	    endif else begin
              tempObj -> remove, $
	      position = thisPosition, $
	      verbose = thisVerbose, $
	      destroy = thisDestroy	; Optional. Destroys objects as they are removed
	  endelse
     endelse
     
     dataFinished:

    endif else begin
      if (keyword_set(thisVerbose)) then $
      		      print, infolabel, "Failure to find dataObj"
      anErrMsg = infolabel + "Failure to find dataObj"
    endelse
  endif

 if (arg_present(thisErrMsg)) then thisErrmsg = anErrMsg
  
end		; specObj::Remove


;=============================================================================
;+
; NAME:
; 	specObj::GetDavePtr
;
; PURPOSE:
;	Returns a pointer to the traditional Dave data structure.
;
; 	The specObj class has a pointer to a DAVE data structure: self.davePtr
;	The DAVE data contains a "specific" pointer to anonymous structure,
;	which includes an object of class specObj. This is a circular 
;	data structure that allows you to either handle the data as
;	a traditional DAVE data strucutre or as a object-oriented specObj object.
;	Think of it as Dr. Jekyll and Mr. Hyde.
;
;	See also specObj::CreateDavePtr
;
;-
;*
function specObj::GetDavePtr	; returns pointer to the DAVE data structure. 
;/
  return, self.davePtr
end

;=============================================================================
;=============================================================================
;+
; NAME:
; 	specObj::SetDavePtrID
;
; PURPOSE:
;	Swaps the DAVE Ptr ID for the pointer to the traditional Dave 
; 	data structure.
;
;
;-
;*
pro specObj::SetDavePtrID, $	; Swaps the PTR ID for the DAVE data structure. 
	newDavePtr, $		; New ID to use instead.
	ownerDavePtr = thisownerDavePtr, $ Optional keyword. 'local' or 'main'. 
	restore = thisRestore	; self has been restored from file and needs its
;/

;	errMsg = thisErrMsg
  debug = 0	;1

;  thisErrMsg= ''
;  if (NOT ptr_valid(newDavePtr)) then begin
;    thisErrMsg = string("PtrID is not valid.", newDavePtr)
;    return, 0
;  endif
  
  if (NOT ptr_valid( newdavePtr)) then begin
      message, /info, "PANIC! NOT ptr_valid(newdavePtr) Do nothing."
      return
  end
 
  if (keyword_set(thisRestore))  then begin
    self.davePtr = newDavePtr			; The new Ptr ID has been swapped in.
    						; and the contents are valid.
  endif else begin
    if (NOT ptr_valid( (self.davePtr))) then begin
      message, /info, "PANIC! NOT ptr_valid( (self.davePtr). Do nothing."
    endif else begin
      dummyDavePtr = ptr_new( *(self.davePtr))
      
;      DavePtr_empty, self.DavePtr, /derefSpecObj	; Empty out most of davePtr,
;      			; but let reference to specObj = self dangle.	
;      if (debug) then begin
;        print
;	message, /info, "DavePtr_empty, self.DavePtr, /derefSpecObj"
;        davePtr_show, self.DavePtr
;	print
;      endif
;
;      heap_free, (self.davePtr) 		; recursive ptr_free. Utterly destroy.
;;      ptr_free, self.davePtr 

      self.davePtr = newDavePtr		; Recycle the Ptr ID number
      if (ptr_valid(newDavePtr)) then begin
        leftoversPtr = ptr_new(*newDavePtr)
        heap_free, leftoversPtr
      endif

      *(self.davePtr) = *(dummyDavePtr)	; "copy" the contents of the pointer.
      ptr_free, dummyDavePtr
    endelse
  endelse
  

  if (n_elements(thisownerDavePtr) GT 0) then begin
    self.ownerDavePtr = thisownerDavePtr
  endif
    
end

;=============================================================================
;=============================================================================
;+
; NAME:
; 	specObj::SetownerDavePtr
;
; PURPOSE:
;	Sets the ownership of the DAVE Ptr ID.
;
;
;-
;*
pro specObj::SetownerDavePtr, $ ; Sets the ownership of the DAVE Ptr ID.
	newownerDavePtr		; either 'main' or 'local'.
;/
	self.ownerDavePtr = newownerDavePtr
end

;=============================================================================
;=============================================================================
;+
; NAME:
; 	specObj::getField
;
; PURPOSE:
;		This method "jumps" down the heirarchical tree, to the leaf node
; 	specified by the argument thisPosition (either an integer or a string "tag")
;	in the section /inHeader or /inData. If neither /inHeader or /inData
;	is specified, it looks in both areas.  
;
;	The leaf node is assumed to be an object of class qtyObj (or have
;	the same interface or inherited qtyObj). One or more of the 
;	keywords (name, units, styleErr, acisTitle, sizeString, rank, dim, tname
;	ptr_qty, qty, ptr_err, err, ptr_nerr, nerr, ptr_legend, legend, isScalar,
;	isNumeric, isConstant, hasError, and  hasStep) are used to return 
;	information about the leaf node.
;	
;	Note: ptr_qty, ptr_err, ptr_nerr and ptr_legend are "pass by reference"
;	and should be used for large data sets where efficiency is desired.
;	Make sure you don't modify the contents by accident.
;	qty, err, nerr, and legend are "pass by value".
;	
;-
;*
function specObj::getField, $		; Returns 0/1 for failure/success
	; returns 0/1 from self.contents by calling qtyObj's get.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
  thisPosition, $ ; This is the "field"
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------from where in container class ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ;----------------------- 
   argPresent = thisArgPresent, $; ARG_PRESENT() function does not work on 
   	;propagated keywords to inherited classes.
  ;------------ qtyObj::GetString
  name = thisname, $
  units = thisunits, $
  StyleErr = thisStyleErr, $
  axisTitle = thisAxisTitle, $ 	; Returns string for plot axis label. 
	  	; Includes name, legend? and units.
  sizeString = thisSizeString, $ ; Returns string for dimensions. 
  			;; '[0]', '[4]', '[#,#]', etc.. 
  some=thisSome, $; 1 line summary of one qtyObj Object. No vectorvalues.
  all=thisAll, $  ; 1 line summary of one qtyObj Object
  everything=thisEverything, $	; String array containing all contents of one qtyObjObObject
  ;------------ qtyObj::GetDimension
  rank=thisrank, $ 		; Number of dimensions. 0 for scalar.
  				; 1 for 1-D, 2 for 2-D, etc.
  dim=thisDim, $ 		; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  tname = thisTName, $ 		; Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
  ;----- qtyObj::GetValue 
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $		; Optional error on the values
  err = thiserr, $			; Optional error on the values
  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
  nerr = thisnerr, $			; Optional negative error on the values
  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
  legend = thislegend, $		; shares memory using new pointer.
  ;--------------------------
  isScalar = thisIsScalar, $
  isNumeric = thisIsNumeric, $
  isConstant = thisIsConstant, $
  hasError = thisHasError, $
  hasStep = thisHasStep
;/

  infolabel = "specObj::getField >>"
  debug = 0
  thisVerbose = 0
  
  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$		;; use anonymous structure 
      ;--- qtyObj::GetString
	  name:		arg_present(thisName), $	
	  units:	arg_present(thisUnits), $	
	  StyleErr:	arg_present(thisStyleErr), $	
	  axisTitle:	arg_present(thisAxisTitle), $
	  sizeString:	arg_present(thisSizeString), $
	  some:		arg_present(thisSome), $
	  all:		arg_present(thisAll), $
	  everything:	arg_present(thisEverything), $
      ;--- qtyObj::GetDimension
	  rank:		arg_present(thisrank), $	
	  Dim:		arg_present(thisDim), $	
	  tname:	arg_present(thisTName), $
      ;--- qtyObj::GetValue
	  ptr_qty:	arg_present(thisptr_qty), $
	  qty:		arg_present(thisqty), $
	  ptr_err:	arg_present(thisptr_err), $
	  err:		arg_present(thiserr), $
	  ptr_nerr:	arg_present(thisptr_nerr), $
	  nerr:		arg_present(thisnerr), $
	  ptr_legend:	arg_present(thisptr_legend), $
	  legend:	arg_present(thislegend), $
       ;-------- qtyObj:is*
          isNumeric:	arg_present(thisIsNumeric), $
          isScalar:	arg_present(thisIsScalar), $
          isConstant:	arg_present(thisIsConstant), $
	  hasError:	arg_present(thisHasError), $
	  hasStep:	arg_present(thisHasStep) $
	  }

  endif else begin
    ArgPresent = thisArgPresent
  endelse

;Debug
;  if (size(thisPosition, /tname) EQ 'STRING') then begin
;    message, /info, string('[', thisPosition[0], ', ', thisPosition[1],$
;  	', ', thisPosition[2], ']', $
;  	 " Entered with argPresent=")
;  endif else begin
;    message, /info, string(thisPosition, $
;  	 " Entered with argPresent=")
;  endelse
;  printArgPresent, ArgPresent   
  
  OKfield = 0
  OKis = 0
  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
  endif else begin
    if (doData) then begin
      OK = (self.focus) -> get(dataObj=tempObj)
    endif
  endelse


  if (OK) then begin
    leafNode = tempObj -> get(position = thisPosition, verbose = thisVerbose)
    
;	    if (debug) then $
;    	print, infolabel, thisPosition
;    if (debug) then $
;	help, leafNode
    if (NOT obj_valid(leafNode)) then begin
      OKfield = 0
      message, /info, string("Invalid leaf node ", thisPosition)
    endif else begin
      OKfield = 0
      If (argPresent.name OR argPresent.units OR argPresent.StyleErr $
       OR argPresent.axisTitle OR argPresent.sizeString $
       OR argPresent.some OR argPresent.all OR argPresent.everything $
	 OR argPresent.rank OR argPresent.dim OR argPresent.tname $
	 OR argPresent.ptr_qty OR argPresent.qty $
	 OR argPresent.ptr_err OR argPresent.err $
	 OR argPresent.ptr_nerr OR argPresent.nerr $
	 OR argPresent.ptr_legend OR argPresent.legend $
	  ) then begin

        OKfield = leafNode -> get($
	  argPresent =ArgPresent, $; ARG_PRESENT() function does not work on 
	   	;propagated keywords to inherited classes.
	  ;------------ qtyObj::GetString
	  name = thisname, $
	  units = thisunits, $
	  StyleErr = thisStyleErr, $
	  axisTitle = thisAxisTitle, $ 	; Returns string for plot axis label. 
		  	; Includes name, legend? and units.
	  sizeString = thisSizeString, $ ; Returns string for dimensions. 
	  			;; '[0]', '[4]', '[#,#]', etc.. 
	  some=thisSome, $; 1 line summary of one qtyObj Object. No vectorvalues.
	  all=thisAll, $  ; 1 line summary of one qtyObj Object
	  everything=thisEverything, $	; String array containing all contents of one qtyObjObObject
	  ;------------ qtyObj::GetDimension
	  rank=thisrank, $ 		; Number of dimensions. 0 for scalar.
	  				; 1 for 1-D, 2 for 2-D, etc.
	  dim=thisDim, $ 		; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
	  tname = thisTName, $ 		; Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
	  ;----- qtyObj::GetValue 
	  ptr_qty = thisptr_qty, $
	  qty = thisqty, $
	  ptr_err = thisptr_err, $		; Optional error on the values
	  err = thiserr, $			; Optional error on the values
	  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
	  nerr = thisnerr, $			; Optional negative error on the values
	  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
	  legend = thislegend $		; shares memory using new pointer.
	)
	endif
     OKis = (argPresent.isScalar OR argPresent.isNumeric OR $
	    argPresent.isConstant OR $
	    argPresent.hasError OR argPresent.hasStep)
     if (OKis) then begin	      
      If (argPresent.isScalar) then begin
        thisIsScalar = leafNode -> isScalar()
      endif	
      If (argPresent.isNumeric) then begin
        thisIsNumeric = leafNode -> isNumeric()
      endif	
      If (argPresent.isConstant) then begin
        thisIsConstant = leafNode -> isConstant()
      endif	
      If (argPresent.hasError) then begin
        thisHasError = leafNode -> HasError()
      endif	
      If (argPresent.hasStep) then begin
        thishasStep = leafNode -> hasStep()
      endif	
     endif
    endelse	; invalid leafNode

  endif	

;Debug
;  if (size(thisPosition, /tname) EQ 'STRING') then begin 
;;    print, infolabel, "OKfield=", OKField
;;    print, infolabel, "OKis=", OKis
;;    junk = size(thisPosition, /dim)
;;    print, infolabel, "dims(thisPosition) =", junk
;    astring = ' '
;    nelem = n_elements(thisPosition)
;    if (nelem NE 3) then begin
;      message, /info, string("n_elements(thisPosition)=", nelem)
;    endif
;    astring = '['
;    if (nelem EQ 1) then begin
;      astring = astring + thisPosition
;    endif else begin
;      for i = 0, (nelem -1) do begin
;        astring = astring +  thisPosition[i]
;	if (i NE (nelem -1)) then astring = astring + ', '
;      endfor
;    endelse
;    astring = astring + ']'
;;    print, infolabel, astring, n_elements(astring),$
;;    	 n_elements((OKfield OR OKis)), n_elements(doHeader), $
;;	 n_elements(doData)
;    message, /info, string( strtrim(string((OKfield OR OKis)), 2), $
;    	"=OK. ",  astring, " doHeader=",  strtrim(string(doHeader), 2), $
;	" doData=", strtrim(string(doData), 2) )
;;    message, /info, string((OKfield OR OKis), $
;;    	"=OK. [", thisPosition[0], ", ",  thisPosition[1],  ", ",thisPosition[2], $
;;	"]", " doHeader=", doHeader,   " doData=", doData)
;  endif else begin
;    message, /info, string((OKfield OR OKis),$
;    	"=OK. ", thisPosition, $
;  		" doHeader=", doHeader,   " doData=", doData)
;  endelse

;Debug
;  if (argPresent.everything) then begin
;;    print, "obj_valid(OKField) = ", obj_valid(OKField) 
;    print, "OKField = ", OKField
;;    print, "OKis = ", OKis
;    print, infolabel, thisPosition, " everything=", thisEverything
;  endif
    
  if (obj_valid(OKField)) then return, 0
  
  return, (OKfield OR OKis)
  
end		;; specObj::GetField,

;=========================================================================================
;*
function specObj::appendField, $		; Returns 0/1 for failure/success
	; returns 0/1 from self.contents by calling qtyObj's append.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
  thisPosition, $ ; This is the "field"
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------from where in container class ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ;----- qtyObj::append
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $		; Optional error on the values
  err = thiserr, $			; Optional error on the values
  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
  nerr = thisnerr, $			; Optional negative error on the values
  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
  legend = thislegend, $		; shares memory using new pointer.
  obj=thisObj 			; Copy contents of same kind of object into self
;/

  infolabel = "specObj::getField >>"
  debug = 0
  thisVerbose = 1
  
  
  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  OKfield = 0
;  if (doHeader) then begin
;    OK = (self.focus) -> get(headerObj=tempObj)
;  endif else begin
;    if (doData) then begin
;      OK = (self.focus) -> get(dataObj=tempObj)
;    endif
;  endelse
  OK = 1

  if (OK) then begin
    leafNode = self -> get(position = thisPosition, $
    	inHeader=doheader, inData=doData, verbose=thisVerbose)
;    leafNode = tempObj -> get(position = thisPosition)
    
    if (debug) then print, infolabel, thisPosition
    if (NOT obj_valid(leafNode)) then begin
      OKfield = 0
;      if (keyword_set(thisVerbose)) then begin	
        message, /info, string("Invalid leaf node ", thisPosition)
        message, /info, string("OK. doHeader", doHeader, " doData=", doData)
        help, leafNode
;      endif
    endif else begin
      OKfield = leafNode -> append($
	  ;----- qtyObj::append 
	  ptr_qty = thisptr_qty, $
	  qty = thisqty, $
	  ptr_err = thisptr_err, $		; Optional error on the values
	  err = thiserr, $			; Optional error on the values
	  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
	  nerr = thisnerr, $			; Optional negative error on the values
	  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
	  legend = thislegend,  $		; shares memory using new pointer.
  	  obj=thisObj) 				; Copy contents of same kind of object into self	  
    endelse	; invalid leafNode

  endif	

  return, (OKfield)
  
end		;; specObj::appendField,


;=============================================================================
;+
; NAME:
; 	specObj::cloneField
;
; PURPOSE:
;		This method "jumps" down the heirarchical tree, to the leaf node
; 	specified by the argument thisPosition (either an integer or a string "tag")
;	in the section /inHeader or /inData. If neither /inHeader or /inData
;	is specified, it looks in both areas. It then clones that item,
;	overiding its  name and tags.
;	
;-
;*
function specObj::cloneField, $		; Returns 0/1 for failure/success
	; returns 0/1 from self.contents by calling qtyObj's get.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
  thisPosition, $ ; This is the "field". required.
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------from where in container class ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ;------------ qtyObj::GetString
  name = thisname, $	; Optional string. overrides the name
  tags = thisTags, $	; Optional string array ['', '', '']. overrides the tags
  label = thisLabel, $	; Optional string. overrides the label portion of the tags
  units = thisUnits; Optional string. overrides the units
;/

  infolabel = "specObj::cloneField >>"
  debug = 0
  
;  print, infolabel, "Adding pos=", thisPosition, " in header=", thisInHeader,$
;  	" in data =", thisInData
  leafNode = self -> get(position=thisPosition, $
  	inHeader = thisInHeader, inData = thisInData)
;	help, leafNode
  if (NOT obj_valid(leafNode)) then begin
      OKfield = 0
  endif else begin
      OKfield = 0
      OKfield = leafNode -> get(name = origName, units = origUnits)
      OKfield = OKField AND self -> gettagfield(thisPosition, tags = origTags, $
      	  inHeader = thisInHeader, inData = thisInData)
  endelse	; invalid leafNode

  if (NOT OKfield) then begin
;    print, infolabel, "Failed ... obj_valid(leafNode)=", obj_valid(leafNode)
    return, 0
  endif
  
  if (n_elements(thisName) gt 0) then begin
    cloneName = thisName 
  endif else begin
    cloneName = origName + '_clone'
  endelse
  
  if (n_elements(thisTags) gt 0) then begin
    cloneTags = thisTags 
  endif else begin
    cloneTags = origTags 
    cloneTags[1] = cloneTags[1] + '_clone'
  endelse
  
  if (n_elements(thisUnits) gt 0) then begin
    cloneUnits = thisUnits
  endif else begin
    cloneUnits = origUnits
  endelse
  
  if (n_elements(thisLabel) gt 0) then begin
    cloneTags[1] =thisLabel
  endif 
  
  cloneNode = leafNode -> clone() 
  OK = cloneNode -> set(name=cloneName, units=cloneUnits)
  
  self -> add, $
	inHeader = thisInHeader, inData = thisInData, $	
  	tags = clonetags, $
	contents = cloneNode    
;  print, infolabel, "Added ", clonetags, " in header=", thisInHeader,$
;  	" in data =", thisInData
	
  return, 1
  
end		;; specObj::CloneField

;=============================================================================
;+
; NAME:
; 	specObj::getTagField
;
; PURPOSE:
;		This method "jumps" down the heirarchical tree, to the leaf node
; 	specified by the argument thisPosition (either an integer or a string "tag")
;	in the section /inHeader or /inData. If neither /inHeader or /inData
;	is specified, it looks in both areas.  
;
;	This routine returns information in the "tag" object associated with
;	the specified leaf node. This information is intended for 
;	assisting with the user-interface.One or more of the 
;	keywords (class, label, codeSource, tags, hide, editable, annotate,
;	pendent and  everything) are used to return 
;	information about the leaf node's tag object.
;-
;*
function specObj::getTagField, $	; Returns 0/1 for failure/success
	; returns 0/1 success/failure from self.contents by calling tagObj's get.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
	thisPosition, $ ; This is the "field"
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------ where to remove from ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ;----------------------- 
   argPresent = thisArgPresent, $; ARG_PRESENT() function does not work on 
   	;propagated keywords to inherited classes.
  ;------------ tagObj::Get
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent, $	;; Will be C: constant, D: dependent, I: independent
	everything = thisEverything	;Keyword returns a string, which is a summary.
;/

  debug = 0
  
  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif
  if (debug) then message, /info, $
  	string("doHeader=", doHeader, " doData=", doData)

  if (NOT arg_present(thisArgPresent)) then begin   
   ArgPresent = {$		;; use anonymous structure 
 	  class:		arg_present(thisClass), $	
	  label:	arg_present(thislabel), $	
	  codeSource:	arg_present(thisCodeSource), $	
	  tags:	arg_present(thistags), $
	  hide:	arg_present(thisHide), $
	  editable:		arg_present(thisEditable), $
	  annotate:		arg_present(thisAnnotate), $
	  pendent:	arg_present(thisPendent), $
	  everything:	arg_present(thisEverything) $
	  }

  endif else begin
    ArgPresent = thisArgPresent
  endelse

  OKfield = 0
  foundInHeader = 0
;Wrong. August 15, 2003. TMR
;  foundInData = 0  
;  if (doHeader) then begin
;    OK = (self.focus) -> get(headerObj=tempObj)
;    foundInHeader = OK
;    if (debug) then message, /info, $
;  	string(OK, "=(self.focus) -> get(headerObj=tempObj). doHeader=", doHeader)
;
;  endif else begin
;    if (doData) then begin
;      OK = (self.focus) -> get(dataObj=tempObj)
;      foundInData = OK
;      if (debug) then message, /info, $
;  	string(OK, "=(self.focus) -> get(dataObj=tempObj). doData=", doData)
;    endif
;  endelse

  foundInData = 0  
  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    foundInHeader = OK
    if (debug) then message, /info, $
  	string(OK, "=(self.focus) -> get(headerObj=tempObj). doHeader=", doHeader)
    ;; if doHeader and doData are both requested, double check that the item is in storage.
    if (size(thisPosition, /tname) EQ 'STRING') then begin
      ; thisPosition = ['*', '*', '*']
      doHeader = (tempObj -> TagIsContained(thisPosition))
    endif else begin
      ; thisPosition = integer
      doHeader = (thisPosition LT tempObj -> count())
    endelse
    if (not DoHeader) then OK = 0
  endif

  if (doData) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
    foundInData = OK
    if (debug) then begin
      message, /info, $
      	string(OK, "=(self.focus) -> get(dataObj=tempObj). doData=", doData)
      message, /info, string(" thisPosition=", format_list(thisPosition))
    endif
    if (size(thisPosition, /tname) EQ 'STRING') then begin
      ; thisPosition = ['*', '*', '*']
      doData = (tempObj -> TagIsContained(thisPosition))
    endif else begin
      ; thisPosition = integer
      doData = (thisPosition LT tempObj -> count())
    endelse
    if (not doData) then OK = 0
  endif
  
  if (doData AND doHeader) then begin
   doHeader = 0		; Have to chose one or the other.    
  endif
  
  if (debug) then message, /info, $
  	string(OK, "=OK. found inHeader=", foundInHeader, " inData=", foundInData)
	
  if (doData OR doHeader) then begin
;  if (OK) then begin
    leafNode = tempObj -> getTag(position = thisPosition)
    OKfield = leafNode -> get($
	argPresent =ArgPresent, $; ARG_PRESENT() function does not work on 
	   	;propagated keywords to inherited classes.
	;-----------------
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent, $	;; Will be C: constant, D: dependent, I: independent
	everything = thisEverything $
	)
  endif	
  
  return, OKfield	
end		;; specObj::GetTagField,

;=============================================================================
;+
; NAME:
; 	specObj::setTagField
;
; PURPOSE:
;		This method "jumps" down the heirarchical tree, to the leaf node
; 	specified by the argument thisPosition (either an integer or a string "tag")
;	in the section /inHeader or /inData. If neither /inHeader or /inData
;	is specified, it looks in both areas.  
;
;	This routine modifies information in the "tag" object associated with
;	the specified leaf node. This information is intended for 
;	assisting with the user-interface.One or more of the 
;	keywords (class, label, codeSource, tags, hide, editable, annotate,
;	and pendent) are used to return 
;	information about the leaf node's tag object.
;-
;*
function specObj::setTagField, $	; Returns 0/1 for failure/success
	; returns 0/1 success/failure from self.contents by calling tagObj's set.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
	thisPosition, $				 ; This is the "field"
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------ where to remove from ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $		; Boolean keyword
  ;----------------------- 
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent		;; Will be C: constant, D: dependent, I: independent
;/

  infolabel = "specObj::setTagField >>"

  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  OKfield = 0
  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
  endif else begin
    if (doData) then begin
      OK = (self.focus) -> get(dataObj=tempObj)
    endif
  endelse

  if (OK) then begin
    leafNode = tempObj -> getTag(position = thisPosition)
;Debug    print, infolabel, thisPosition 
;Debug    help, leafNode
    OKfield = leafNode -> set( $
	class = thisClass, $		; string "DVHeader", "DVData", "qtyObj"
	label = thisLabel, $		; string "Counts"
	codeSource = thisCodeSource, $	;; string "ICP", "PAN"
	tags = thisTags, $		;; string array of class, label, codeSource.
	hide = thisHide, $		;; Boolean. /hide is OK.
	editable = thisEditable, $	;; Boolean. /editable is OK.
	annotate = thisAnnotate, $	;; Boolean. /annotate is OK.
	pendent = thisPendent $	;; Will be C: constant, D: dependent, I: independent
	)
  endif	
  
  return, OKfield	
end		;; specObj::SetTagField,


;=============================================================================
;+
; NAME:
; 	specObj::setField
;
; PURPOSE:
;		This method "jumps" down the heirarchical tree, to the leaf node
; 	specified by the argument thisPosition (either an integer or a string "tag")
;	in the section /inHeader or /inData. If neither /inHeader or /inData
;	is specified, it looks in both areas.  
;
;	The leaf node is assumed to be an object of class qtyObj (or have
;	the same interface or inherited qtyObj). One or more of the 
;	keywords (name, units, styleErr, acisTitle, sizeString, rank, dim, tname
;	ptr_qty, qty, ptr_err, err, ptr_nerr, nerr, ptr_legend, and legend) are 
; 	used to modify information about the leaf node.
;	
;	Note: ptr_qty, ptr_err, ptr_nerr and ptr_legend are "pass by reference"
;	and should be used for large data sets where efficiency is desired.
;	Make sure you don't modify the contents by accident.
;	qty, err, nerr, and legend are "pass by value".
;	
;-
;*
function specObj::setField, $		; Returns 0/1 for failure/success
	; returns 0/1 from self.contents by calling qtyObj's set.
	;----- use argument position for indices or
	;  by tag ['blahClass', 'blahName', 'blahCodeSource']--
	thisPosition, $ ; This is the "field"
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
  ;------ where to remove from ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ;------------ qtyObj::GetString
  name = thisname, $
  units = thisunits, $
  StyleErr = thisStyleErr, $
  ;------------ qtyObj::GetDimension
  rank=thisrank, $ 		; Number of dimensions. 0 for scalar.
  				; 1 for 1-D, 2 for 2-D, etc.
  dim=thisDim, $ 		; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
  tname = thisTName, $ 		; Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
  ;----- qtyObj::GetValue 
  ptr_qty = thisptr_qty, $
  qty = thisqty, $
  ptr_err = thisptr_err, $		; Optional error on the values
  err = thiserr, $			; Optional error on the values
  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
  nerr = thisnerr, $			; Optional negative error on the values
  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
  legend = thislegend		; shares memory using new pointer.
;/

;  if (ptr_valid(thisPtr_err)) then begin
;    message, /info, string("ptr_valid(thisPtr_err)=", (*thisptr_err)[0], "..")
;  endif
  
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  OKfield = 0
  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
  endif else begin
    if (doData) then begin
      OK = (self.focus) -> get(dataObj=tempObj)
    endif
  endelse

  if (OK) then begin
    leafNode = tempObj -> get(position = thisPosition)
    if (obj_valid(leafNode)) then begin
      OKfield = leafNode -> set($
	  ;------------ qtyObj::GetString
	  name = thisname, $
	  units = thisunits, $
	  StyleErr = thisStyleErr, $
	  ;------------ qtyObj::GetDimension
	  rank=thisrank, $ 		; Number of dimensions. 0 for scalar.
	  				; 1 for 1-D, 2 for 2-D, etc.
	  dim=thisDim, $ 		; 0, or ([#]or #?) or [#,#] or [#,#,#], etc.
	  tname = thisTName, $ 		; Data type ('INT', 'FLOAT', etc.) of *(self.ptr_qty)
	  ;----- qtyObj::GetValue 
	  ptr_qty = thisptr_qty, $
	  qty = thisqty, $
	  ptr_err = thisptr_err, $		; Optional error on the values
	  err = thiserr, $			; Optional error on the values
	  ptr_nerr = thisptr_nerr, $		; Optional negative error on the values
	  nerr = thisnerr, $			; Optional negative error on the values
	  ptr_legend = thisptr_legend, $	; shares memory using the same pointer.
	  legend = thislegend	$	; shares memory using new pointer.
	)
    endif else begin
      message, /info, string("Failed to find field ",  thisPosition)
    endelse
  endif	
  

  return, OKfield	
end		;; specObj::setField,


;=============================================================================
;+
; NAME:
; 	specObj::get
;
; PURPOSE:
;	Similar to DAVE_tagContainer::get and IDL_container::get.
;
;	Returns object or an array of objects from either the "header" section 
;	(use /inHeader) or the "data" section (use /inData) of the container.
;	The objects are identitified by either /all, isa or position keywords.
;-
;*
function specObj::Get, $	; returns an object or object array.
	; returns object or objects from self.contents.
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	count = thisCount, $		; # of objects selected
	everything = thisEverything, $	; Optional keywords, returns an info string
  ;------ where to get from ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $		; Boolean keyword
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore, $			; Boolean keyword.
  				; If used, keywords inHeader and inData ignored
  ;---------------------------------------------------------
  verbose = thisVerbose	; Optional keyword. Reports on progress.
;/

  infoLabel = "specObj::Get >> "
  

  ;--------------------------------------------------------------------------
  if (keyword_set(thisInStore)) then begin
       if (arg_present(thisEverything)) then begin
       return, (self.store) -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount, $		; # of objects selected
	everything = thisEverything)	; Return info string in keyword 
      endif else begin
       return, (self.store) -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount)	; Return info string in keyword 
      endelse
 
  endif 
  ;--------------------------------------------------------------------------
    

  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    if (NOT obj_valid(tempObj)) then begin
      print, infolabel, OK, " Header object not initialized in focus or mistake"
      OK = 0
    endif
    if (OK) then begin
      if (arg_present(thisEverything)) then begin
       return, tempObj -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount, $		; # of objects selected
	everything = thisEverything)	; Return info string in keyword 
      endif else begin
       return, tempObj -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount)	; Return info string in keyword 
      endelse
    endif else begin
      print, infolabel, "Failure to find headerObj"
      return, obj_new()
    endelse
      
  endif
  
  if (doData) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
    if (OK) then begin
      if (arg_present(thisEverything)) then begin
       message, /info,  "Debug data Boo everything"
       return, tempObj -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount, $		; # of objects selected
	everything = thisEverything)	; Return info string in keyword 
      endif else begin
       leafObj= tempObj -> get( $
	all = thisAll, 	$ 	; Boolean keyword
	ISA = thisISA, $ 	; class names(S). string or string array
	;----- use keyword position for indices or by tag ----------
	position = thisPosition, $ ; 
		; Integer scalar or Int Array. zero-based indices of the positions 
	 	; or  String scalar or array of the tags.
	;---------------------------------------------------------
	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
	count = thisCount)	; Return info string in keyword 
	
	leaf_tname = size(leafObj, /tname)
	if (leaf_tname NE 'OBJREF') then begin
;	if (NOT obj_valid(leafObj)) then begin
	  nelems = n_elements(leafObj)
	  message, /info, string(nelems, " items returned of type ", leaf_tname)
	  if (n_elements(thisPosition) GT 0) then begin
	    message, /info, string("Requested position = ", thisPosition)
	  endif else begin
	    ndata = self->count(/inData)
	    message, /info, string("Requested all ", ndata, $
	    		" objects in data section ")
	  endelse
	  message, /info, "Require 1 or array of defined objects."
	  
;	  message, /info, "Failure in data. Focus then leaf objects"
	  
;          message, /info,  "Debug data Boo"
;          help, tempObj
 	  help, leafObj
	  print 
	endif
	return, leafObj
	
;       return, tempObj -> get( $
;	all = thisAll, 	$ 	; Boolean keyword
;	ISA = thisISA, $ 	; class names(S). string or string array
;	;----- use keyword position for indices or by tag ----------
;	position = thisPosition, $ ; 
;		; Integer scalar or Int Array. zero-based indices of the positions 
;	 	; or  String scalar or array of the tags.
;	;---------------------------------------------------------
;	verbose = thisVerbose, $	; Optional keyword. Reports on progress.
;	count = thisCount)	; Return info string in keyword 
      endelse
    endif else begin
      print, infolabel, "Failure to find dataObj"
      return, obj_new()
    endelse
      
  endif
	
end		;; specObj::Get,


;=============================================================================
;+
; NAME:
; 	specObj::getStore
;
; PURPOSE:
;	Returns self.store, which is of type DAVE_tagContainer.
;	 Used by clone method.
;-
;*
function specObj::GetStore	; returns the self.store object 
;/

  return, (self.store)
	
end		;; specObj::GetStore

;=============================================================================
;+
; NAME:
; 	specObj::setStore
;
; PURPOSE:
;	Over-rides self.store, which is of type DAVE_tagContainer.
;	 Used by clone method.
;-
;*
pro specObj::setStore, $	; returns the self.store object 
	newContainer		; New object of type DAVE_tagContainer.
;/

  obj_destroy, self.store
  self.store = newContainer

end		;; specObj::SetStore


;=============================================================================
;+
; NAME:
; 	specObj::getLabel
;
; PURPOSE:
;	Gets selected information about all the leaf nodes specified by
;	either /inHeader or /inData 
; 	Arrays returned via optional keywords class, label, codeSource,
;	sizeString and count.
; 	
;-
;*
pro specObj::GetLabel, $	; returns string arrays of the labels, size, etc..
  	argPresent = thisArgPresent, $	; Optional keyword, anon structure of boolean.
		; For internal use. 
		; ARG_PRESENT() function does not work on 
   		;propagated keywords from other calling routines.
	class = thisClass, $		; Optional keyword, returns the string array
					; for first string in tag ['','','']
	label = thisLabel, $		; Optional keyword, returns the string array
					; for second string in tag ['','','']
	codeSource = thisCodeSource, $	; Optional keyword, returns the string array
					; for third string in tag ['','','']
	sizeString = thisSizeString, $; Optional keyword, returns the string array
				; giving the size, based on rank and dimensions.

	;---------------------------------------------------------
	count = thisCount, $		; Optional keyword, returns integer array.
					; for # of objects in aech leaf node in 
					; section selected
  ;------ where to remove from ----------------- 
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $			; Boolean keyword
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore			; Boolean keyword.
  				; If used, keywords inHeader and inData ignored
;/
;;	tags = thisTags, $

  ;----------------------- 

  ;------------------------------------------------------
  if (NOT arg_present(thisArgPresent)) then begin   
    ArgPresent = {$			; make this be anonymous structure
	class : arg_present(thisClass), $
	label : arg_present(thisLabel), $	
	codeSource : arg_present(thisCodeSource), $
	sizeString : arg_present(thisSizeString), $
	count : arg_present(thisCount) $
	}
  endif else begin
    ; ArgPresent had better include the appropriate tags.
    ArgPresent = thisArgPresent
  endelse
  ;------------------------------------------------------

  if (keyword_Set(thisInStore)) then begin
      (self.store) -> getLabel,  argPresent = argPresent, $
	class = thisClass, $
	label = thisLabel, $
	codeSource = thisCodeSource, $
	sizeString = thisSizeString, $
	count = thisCount
      return
  endif
  ;------------------------------------------------------
  
  ; -> method from where ever it is found by default.
  doHeader = 1
  doData = 1
  if ((not keyword_set(thisInHeader)) and keyword_set(thisInData) ) then begin
    doHeader = 0
  endif
  if (keyword_set(thisInHeader) and (NOT keyword_set(thisInData)) ) then begin
    doData = 0
  endif

  if (doHeader) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    if (OK) then begin
      tempObj -> getLabel,  argPresent = argPresent, $
	class = thisClass, $
	label = thisLabel, $
	codeSource = thisCodeSource, $
	sizeString = thisSizeString, $
	count = thisCount
      return
    endif else begin
      print, infolabel, "Failure to find headerObj"
    endelse
  endif
  
  if (doData) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
    if (OK) then begin
      tempObj -> getLabel, argPresent= argPresent, $
	class = thisClass, $
	label = thisLabel, $
	codeSource = thisCodeSource, $
	sizeString = thisSizeString, $
	count = thisCount
      return
    endif else begin
      print, infolabel, "Failure to find dataObj"
    endelse
      
  endif
	
end		;; specObj::GetLabel,


;=============================================================================
;+
; NAME:
; 	specObj::getFocus
;
; PURPOSE:
;	See focusObj::get and focusObj::define
;-
;*
function specObj::GetFocus, $	; Returns 0/1 for failure/success.
				; Returns focusObj if no keywords are provided.
   argPresent = thisArgPresent, $	; Optional keyword, 
		; anon structure of boolean. For internal use. 
		; ARG_PRESENT() function does not work on 
   		; propagated keywords from other calling routines.
		; Needed for get* methods.
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags, $			; optional   
   xType = thisXType, $
   yType = thisYType, $
   everything = thisEverything
;/   
  if (NOT arg_present(thisArgPresent)) then begin   
    ArgPresent = {$			; make this be anonymous structure
	container : arg_present(thisContainer), $
	dataObj : arg_present(thisDataObj), $	
	headerObj : arg_present(thisHeaderObj), $
	mainTitleTags : arg_present(thisMainTitleTags), $
	descripTags : arg_present(thisdescripTags), $
	descripLegend : arg_present(thisDescripLegend), $
	cutTags : arg_present(thisCutTags), $
	xTags : arg_present(thisXtags), $
	yTags : arg_present(thisYtags), $
	zTags : arg_present(thisZtags), $
	xType : arg_present(thisXtype), $
	yType : arg_present(thisYtype), $
	everything : arg_present(thisEverything)$
	 }
  endif else begin
    ; ArgPresent had better include the appropriate tags.
    ArgPresent = thisArgPresent
  endelse

;;  print, infolabel, 'ArgPresent =', ArgPresent
  return, (self.focus) -> get( $
   argPresent = ArgPresent, $; ARG_PRESENT() function does not work on 
   		;propagated keywords from other calling routines.
		; Needed for get* methods.
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $	; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags)			; optional   

end		; specObj::GetFocus

;=============================================================================
;+
; NAME:
; 	specObj::GetPosition
;
; PURPOSE:
; 	Converts tags ['blah_class', 'blah_name', 'blah_codeSource'] to the
; 	integer position in the container. This integer is returned.
; 	It has a value of -1 if the tags are not found.
; 	'*' wildcard is allowed in the tag. 
;-
;*
function specObj::GetPosition, $
   thisXTags, $			; required
   ;---- Only one keyword of the two is allowed: /inHeader or /inData.
  inHeader = thisInHeader, $		; Boolean keyword
  inData = thisInData, $	 	; Boolean keyword
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore			; Boolean keyword.
  				; If used, keywords inHeader and inData ignored
;/

  infolabel = "specObj::GetPosition >> " 
;  debug = 0	;1
  
  OK = 0
  if (keyword_set(thisInStore)) then begin
    tempObj = (self.store)
  endif
  
  if (keyword_set(thisInHeader)) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
  endif
  
  if (keyword_set(thisInData)) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
  endif

  xPosition = -1
  if (NOT OK) then begin
    return, xPosition
  endif
  
    xPosition = tempObj -> Convert2Position(thisxTags)
;    if (debug) then print, infolabel, 'x=', thisXTags, xPosition

   return, xPosition

end		; specObj::GetPosition


;=============================================================================
;+
; NAME:
; 	specObj::setFocus
;
; PURPOSE:
;	See focusObj::set and focusObj::define
;-
;*
pro specObj::SetFocus, $
   focusObj = thisFocusObj, $
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $		; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags			; optional   
;/

  if (n_elements(thisFocusObj) GT 0) then begin
    obj_destroy, self.focus
    self.focus = thisFocusObj
  endif
  
  ; replace piecemeal
 (self.focus) -> set,  $
   container = thisContainer, $
   headerObj = thisHeaderObj, $
   dataObj = thisDataObj, $
   mainTitleTags = thisMainTitleTags, $	; optional.
   descripTags = thisDescripTags, $	; optional.
   DescripLegend = thisDescripLegend, $	; optional.
   cutTags = thisCutTags, $		; optional.
   xTags = thisXTags, $			; optional
   yTags = thisYTags, $			; optional
   zTags = thisZTags			; optional   

end		; specObj::SetFocus

;=============================================================================
;*
pro specObj::setWID, $ 
	thisWID			; Widget ID associated with this object.
;/
   self.wID = thisWID
end

;=============================================================================
;*
function specObj::getWID	; Returns widget ID associated with this object.
;/
   return, self.wID
end

;=============================================================================
;*
function specObj::getMinMax, $	; Returns an integer array  [min, max] 
	; for the range available in the focus or in specified field
	; ------  Use 1 of the three following keywords ------------------
	x = thisX, $		; Boolean Keyword.
	y = thisY, $		; Boolean Keyword.
	z = thisZ, $			; Boolean Keyword.
	field = thisField,$		; String keyword. 
 	 ;------from where in container class. Use with field keyword --- 
 	 inHeader = thisInHeader, $	; Boolean keyword
 	 inData = thisInData		; Boolean keyword
;/

 infolabel = 'specObj::getMinMax >> '
 
 if (n_elements(thisField) GT 0) then begin 
    OK = self -> getField(thisField, ptr_qty=thePtr_Qty, $
    	inHeader = thisInHeader, inData = thisInData)

 endif else begin
   OK = 0
   if ( keyword_set(thisX)) then begin
     OK = self -> getFocus(xTags = theXYZtag)
   endif 	
   if ( keyword_set(thisY)) then begin
     OK = self -> getFocus(yTags = theXYZtag)
   endif 	
   if ( keyword_set(thisZ)) then begin
     OK = self -> getFocus(zTags = theXYZtag)
   endif 	
   if (OK) then begin
;     print, infolabel, theXYZtag, keyword_set(thisX), keyword_set(thisY), $
;     		keyword_set(thisZ)
     OK = self -> getField(theXYZtag, ptr_qty=thePtr_Qty, /inData)
   endif 
 endelse
 
 
 if (NOT OK)  then begin 
; if ((NOT OK) OR (NOT (ptr_valid(thePtr_qty))) )then begin 
   return,  [0,0]	; error, so return a stupid range
 endif else begin 
   vmax = max((*thePtr_Qty), min = vmin)
   ptr_free, thePtr_Qty
   return, [vmin, vmax]
 endelse 

end


;================================================================================
;*
pro specObj::plot, $
    over = thisOver, $		; 0 for plot, 1 for oplot.
    indexNum = thisIndexNum, $	; 0 for plot. 1, 2, etc. for oplot. 
    			; Eventually can use color code and vary the symbols 
			; using indexNum.
    altFocus = thisAltFocus, $ 	; alternative focus (a bit naughty for a method)
    _EXTRA = extraExtra ; extra keywords (constant) passed to plot.
;/   
   infolabel = 'specObj::plot >> "
   
;   if (n_elements(extraExtra) GT 0)) then begin
;     print, infolabel, extraExtra
;   endif
   if (n_elements(thisOver) eq 0) then begin
     if (n_elements(thisIndexNum) GT 0) then begin
       if (thisIndexNum GT 0) then thisOver = 1
     endif
   endif
      
   if (n_elements(thisAltFocus) GT 0) then begin
     thisAltFocus -> plot, over=thisOver, indexNum = thisIndexNum,_EXTRA = extraExtra 
   endif else begin
     (self.focus) -> plot, over=thisOver,indexNum = thisIndexNum, _EXTRA = extraExtra 
   endelse 
   
end		; specObj::plot


;================================================================================
;+
; NAME:
; 	specObj::updateDavePtr
;
; PURPOSE:
;	Modifies the DAVE pointer in <specObj>.davePtr to reflect the 
;	current focus. Returns 0/1 for failure/success.
;
; 	The specObj class has a pointer to a DAVE data structure: self.davePtr
;	The DAVE data contains a "specific" pointer to anonymous structure,
;	which includes an object of class specObj. This is a circular 
;	data structure that allows you to either handle the data as
;	a traditional DAVE data strucutre or as a object-oriented specObj object.
;	Think of it as Dr. Jekyll and Mr. Hyde.
;
;	See also specObj::getDavePtr, initDavePtr
;
;-
;*
 function specObj::updateDavePtr, $	; Returns 0/1 for failure/success.
 	sort = thisSort			; Sorts "x" axis and "y" axis in the focus if set.
;/
; 	sortByX = thisSortByX		; Sorts "x" axis in the focus if set.

  infolabel = "specObj::updateDavePtr >> "  
  debug = 0	;1
  
  success = 1
  currentFocus = self.focus

   if (NOT ptr_valid(self.DavePtr)) then begin	; should be NULL if not initialized.
     print, infolabel, "self.DavePtr is not initialized. Use initDavePtr instead"
     return, 0
   endif
   
  zTags = currentFocus.zTags
  tempZObj = currentFocus.dataObj -> get(position = zTags)
  OK = tempZobj -> get ( qty = zqty, $
  	err = zerr, $
	units = zunits, $
        rank = zrank, $
	dim = zdim, $
  	axisTitle = zaxisTitle)
  	
  xTags = currentFocus.xTags
  tempXObj = currentFocus.dataObj -> get(position = XTags)
  if (obj_valid(tempXObj)) then begin
    OK = tempXobj -> get ( qty = Xqty, units = xunits, $
        rank = xrank, $
	dim = xdim, $
    	axisTitle=xaxisTitle)
  endif else begin
    xTags = currentFocus.cutTags
    tempXObj = currentFocus.headerObj -> get(position = xTags)
    if (obj_valid(tempXObj)) then begin
      OK = tempXobj -> get ( qty = Xqty, units = Xunits, $
        rank = xrank, $
	dim = xdim, $
      	axisTitle=XaxisTitle)
    endif
  endelse

  yTags = currentFocus.yTags
  tempYObj = currentFocus.dataObj -> get(position = yTags)
  if (obj_valid(tempYObj)) then begin
    OK = tempYobj -> get ( qty = Yqty, $
        rank = yrank, $
	dim = ydim, $
    units = Yunits, axisTitle=YaxisTitle)
  endif else begin
    yTags = currentFocus.cutTags
    tempYObj = currentFocus.headerObj -> get(position = yTags)
    if (obj_valid(tempYObj)) then begin
      OK = tempYobj -> get ( qty = Yqty, $
        rank = yrank, $
	dim = ydim, $
      units = Yunits, axisTitle=YaxisTitle)
    endif
   endelse

  ;-----------------------------------------------------
  if (keyword_set(thisSort)) then begin
;  if (keyword_set(thisSortByX)) then begin
    ; Only sorting for the kinds of arrays combos that TRIPLE axis data reader
    ; currently creates.
      if (!debug) then begin
    	 message, /info, string("Sorting by x! xrank=", xrank, $
	 	" yrank=", yrank, " zrank=", zrank)
      endif
      ;--------------------------------------------------
      if ((xrank EQ 1) AND (zrank EQ 1)) then begin
	ixmax = zdim[0] -1
        indexsort = sort(xqty)
	if (debug) then begin
	  print, infolabel, "Sorting by x: orig, new ordering:"
	  for i = 0, ixmax do begin
	    print, xQty[i], xQty[indexsort[i]]
	  endfor
	  print
	endif
        xQty = xQty[indexsort]

        if (n_elements(zQty) GT 0) then begin
          zQty = zQty[indexsort]
        endif
          
        if (n_elements(zErr) GT 0) then begin
          zErr  = zErr[indexsort]
        endif
          
        if (n_elements(yQty) GT 0) then begin
          yQty = yQty[indexsort]
        endif          
      endif
      ;--------------------------------------------------
      if ((xrank EQ 1) AND (zrank EQ 2)) then begin
        ; sort by x, then by y
	ixmax = zdim[0] -1
	iymax = zdim[1] -1

        indexsort = sort(xqty)
	if (debug) then begin
	  print, infolabel, "Sorting by x: orig, new ordering:"
	  for i = 0, ixmax do begin
	    print, xQty[i], xQty[indexsort[i]]
	  endfor
	  print
	endif
         xQty = xQty[indexsort]

        tempArray = zQty
        if (n_elements(zErr) GT 0) then begin
          etempArray = zErr
	endif
	
        for iy = 0, iymax do begin
          if (n_elements(zQty) GT 0) then begin
            zQty[indexsort, iy]= tempArray[0:ixmax, iy]
          endif
          if (n_elements(zErr) GT 0) then begin
            zErr[indexsort, iy]= etempArray[0:ixmax, iy]
          endif
          
	endfor
      
        ; now sort the y axis.
        indexsort = sort(yqty)
	if (debug) then begin
	  print, infolabel, "Sorting by y: orig, new ordering:"
	  for i = 0, iymax do begin
	    print, yQty[i], yQty[indexsort[i]]
	  endfor
	  print
	endif
        yQty = yQty[indexsort]

        tempArray = zQty
        if (n_elements(zErr) GT 0) then begin
          etempArray = zErr
	endif
	
        for ix = 0, xmax do begin
          if (n_elements(zQty) GT 0) then begin
            zQty[ix, indexsort]= tempArray[ix, 0:iymax]
          endif
          if (n_elements(zErr) GT 0) then begin
            zErr[ix, indexsort]= etempArray[ix, 0:iymax]
          endif
          
	endfor
      endif

  endif

  ;-----------------------------------------------------

  dTags = currentFocus.descripTags
  dLegend = currentFocus.descripLegend
  tempDObj = currentFocus.headerObj -> get(position = dTags)
  if (obj_valid(tempDObj)) then begin
;Stupid    tempDObj = self.dataObj -> get(position = dTags)
    DeeLegend = strtrim(dLegend, 2)
    OK = tempDobj -> get (units = Dunits, name=dname, everything=debugEverything)
;    print, infolabel, "descrip=", debugEverything
;    print, infolabel, " getOneValue(forLegend= '",DeeLegend, "'"
    OK = tempDobj -> getOneValue(forLegend=DeeLegend, $
    	oneqty = dqty, oneerr = derr)
;    print, infolabel, "tempDObj OK=", OK
  endif

  ; tmr_create_dave_pointer:
  ; Creates a new DAVE data structure if newDavePtr is null. Don't want this.
  ; Updates an old DAVE data structure if newDavePtr is valid. We want this.
  
  if (obj_valid(tempDobj)) then begin
    if (n_elements(DeeLegend ) EQ 0) then DeeLegend = ' '
;    print, infolabel, ' Updating daveptr.descrip :', $
;  	'"', dname, '" "',  dunits,  '" "', DeeLegend, '" '
;    print, infolabel, ' Updating daveptr.descrip : qty=', dqty
;    if (n_elements(derr) GT 0) then  begin
;      print, infolabel, ' Updating daveptr.descrip : err=', derr
;    endif
    OK = tmr_create_dave_pointer(self.DavePtr, $
	instrument = 'Triple_axis_patchup', $
  	qty = zqty, err = zerr, qtunits=zunits, qtlabel=zaxisTitle , $ 
  	xvals = xqty, xunits=xunits, xlabel=xaxisTitle , $ 
  	yvals = yqty, yunits=yunits, ylabel=yaxisTitle , $ 
	dname = dname, dunits = dunits, dlegend=DeeLegend, dqty = dqty, derr=derr, $
	ermsg = ret_erMsg)
;    if (NOT OK) then begin
;      print, infolabel, ret_erMsg2
;    endif
  endif else begin
    OK = tmr_create_dave_pointer(self.DavePtr, $
	instrument = 'Triple_axis_patchup', $
  	qty = zqty, err = zerr, qtunits=zunits, qtlabel=zaxisTitle , $ 
  	xvals = xqty, xunits=xunits, xlabel=xaxisTitle , $ 
  	yvals = yqty, yunits=yunits, ylabel=yaxisTitle , $ 
;;;DON'T MODIFY	specificStr = specRec, $
	ermsg = ret_erMsg)
	
  endelse
  

  if (NOT OK) then begin
    print, infolabel, ret_erMsg
    return, 0
  endif 
  
  return, 1
    
end		; specObj::updateDavePtr

;================================================================================
;+
; NAME:
; 	specObj::InitDavePtr
;
; PURPOSE:
;	Creates a DAVE pointer and places it in <specObj>.davePtr.
;	Returns 0/1 for failure/success.
;
; 	The specObj class has a pointer to a DAVE data structure: self.davePtr
;	The DAVE data contains a "specific" pointer to anonymous structure,
;	which includes an object of class specObj. This is a circular 
;	data structure that allows you to either handle the data as
;	a traditional DAVE data strucutre or as a object-oriented specObj object.
;	Think of it as Dr. Jekyll and Mr. Hyde.
;
;	See also specObj::getDavePtr, updateDavePtr
;
;-
;*
 function specObj::initDavePtr, $	; Returns 0/1 for failure/success
	instrument = thisInstrument, $	; String.
	DavePtr = newDavePtr, $		; Optional.Dave pointer.  
				; empty ptr_new(/allocate_heap) or filled.
        ownerDavePtr = thisownerDavePtr	; If 'local', cleanup destroys self.davePtr.
   					; If 'main', cleanup skips self.DavePtr
;/

;	path = thisPath, $			; string
;	file = thisFile 			; string. Does not include path

  infolabel = "specObj::initDavePtr >> "  
  debug = 0	;1

  if (n_elements(thisInstrument) EQ 0) then begin
	instrument = 'Triple_axis_patchup'
  endif else begin
  	instrument = thisInstrument
  endelse
  
  currentFocus = self.focus

;   if (ptr_valid(self.DavePtr)) then begin	; should be NULL if not initialized.
;     if (n_elements(self.DavePtr) GT 0) then begin
;       print, infolabel, "self.DavePtr is already initialized. Use updateDavePtr instead"
;       return, 0
;     endif
;   endif
   
  zTags = currentFocus.zTags
  if (debug) then print, infolabel, "currentFocus.zTags=", zTags
  tempZObj = currentFocus.dataObj -> get(position = zTags)
  OK = tempZobj -> get ( qty = zqty, err = zerr, units = zunits, $
  	axisTitle = zaxisTitle)
  	
  xTags = currentFocus.xTags
  if (debug) then print, infolabel, "currentFocus.xTags=", xTags
  tempXObj = currentFocus.dataObj -> get(position = XTags)
  if (obj_valid(tempXObj)) then begin
    OK =  tempXobj -> get ( qty = Xqty, units = xunits, axisTitle=xaxisTitle)
  endif else begin
    xTags = currentFocus.cutTags
    tempXObj = currentFocus.headerObj -> get(position = xTags)
    if (obj_valid(tempXObj)) then begin
      OK = tempXobj -> get ( qty = Xqty, units = Xunits, axisTitle=XaxisTitle)
    endif
  endelse

  yTags = currentFocus.yTags
  if (debug) then print, infolabel, "currentFocus.yTags=", yTags
  tempYObj = currentFocus.dataObj -> get(position = yTags)
  if (obj_valid(tempYObj)) then begin
    OK = tempYobj -> get ( qty = Yqty, units = Yunits, axisTitle=YaxisTitle)
  endif else begin
    yTags = currentFocus.cutTags
    tempYObj = currentFocus.headerObj -> get(position = yTags)
    if (obj_valid(tempYObj)) then begin
      OK = tempYobj -> get ( qty = Yqty, units = Yunits, axisTitle=YaxisTitle)
    endif
   endelse

  dTags = currentFocus.descripTags
  dLegend = currentFocus.descripLegend
;;;    print, infolabel, "dTags = ", dTags
    tempDObj = currentFocus.headerObj -> get(position = dTags)
    if (obj_valid(tempDObj)) then begin
      DeeLegend = strtrim(dLegend, 2)
;Stupid      tempDObj = self.dataObj -> get(position = dTags)
      OK = tempDobj -> get (units = Dunits, name=dname)
      OK = tempDobj -> getOneValue(forLegend=DeeLegend, $
    	oneqty = dqty, oneerr = derr)
    endif
  
  specRec = {specObj:self}
  if (!debug) then begin
    message, /info, "Loaded up  specRec = {specObj:self}"
    help, specRec 
    help, specRec.specObj
  endif

  ; tmr_create_dave_pointer. Creates a new DAVE data structure if newDavePtr is null.
  ; Updates an old DAVE data structure if newDavePtr is valid.
  
  if (!debug) then begin
;    message, /info, string("!debug=", !debug)
    message, /info, "Pre- tmr_create_dave_pointer"
    help, /heap, /brief
    print
    message, /info, "Contentents of specificstr:" 
    help,specRec
    help, specRec.specObj
  endif
  
  if (obj_valid(tempDobj)) then begin
    if (n_elements(DeeLegend ) EQ 0) then DeeLegend = ' '
      ;derr might not be defined.
;    print, infolabel, ' Creating daveptr.descrip :', $
;  	dname,  dunits, DeeLegend, dqty, derr 

    OK = tmr_create_dave_pointer(dummyDavePtr, $
	instrument = instrument, $
  	qty = zqty, err = zerr, qtunits=zunits, qtlabel=zaxisTitle , $ 
  	xvals = xqty, xunits=xunits, xlabel=xaxisTitle , $ 
  	yvals = yqty, yunits=yunits, ylabel=yaxisTitle , $ 
	dname = dname, dunits = dunits, dlegend=DeeLegend, dqty = dqty, derr=derr, $
	specificStr = specRec, $
	ermsg = ret_erMsg)
  endif else begin
    OK = tmr_create_dave_pointer(dummyDavePtr, $
	instrument = instrument, $
  	qty = zqty, err = zerr, qtunits=zunits, qtlabel=zaxisTitle , $ 
  	xvals = xqty, xunits=xunits, xlabel=xaxisTitle , $ 
  	yvals = yqty, yunits=yunits, ylabel=yaxisTitle , $ 
;	dname = dname, dunits = dunits, dlegend=DeeLegend, dqty = dqty, derr=derr, $
	specificStr = specRec, $
	ermsg = ret_erMsg)
  endelse
 
  if (!debug) then begin
;    message, /info,   "daveptr_show, dummyDavePtr"
;    daveptr_show, dummyDavePtr, varName="dummyDavePtr"
  endif
 
  if (debug) then print, infolabel, OK, " = tmr_create_dave_pointer(...)"
  if (NOT OK) then begin
    message, /info, ret_errMsg
  endif else begin
  
;Debug    davePtr_show, dummyDavePtr, varName= 'dummyDavePtr'
;Debug    print
  
    ; Change the pointerID 
      if (!debug) then begin
        help, /heap, /brief
        print, "heap_free, *(self.davePtr) & ptr_free, self.davePtr"
        print
      endif
      
      message, /info, string("Before daveptr_destroy. obj_valid(self)=", obj_valid(self))
      if (ptr_valid(self.davePtr)) then begin
        if (debug) then begin
	  print, "self.davePtr=", self.davePtr, *(self.davePtr)
	  message, /info, "daveptr_destroy, (self.davePtr)"
	endif
	daveptr_destroy, (self.davePtr)		;;;, /destroySpecObj
;        heap_free, (*(self.davePtr))
      endif else begin
        message, /info, string("invalid ptr self.davePtr=", self.davePtr)
      endelse
      
      if (debug) then message, /info, string("After daveptr_destroy. obj_valid(self)=", obj_valid(self))
      ptr_free, self.davePtr

      if (!debug) then begin
;	message, /info, "davePtr_show, newdavePtr,	before swap"
;        davePtr_show, newdavePtr, varName= 'newdavePtr'	; null pointer. 
      endif
 
      if (ptr_valid(newDavePtr)) then begin
      
        if (!debug) then begin
	  help, /heap, /brief
          print, "self.davePtr = newDavePtr & *(self.davePtr) =  *(dummyDavePtr)"
          print
	endif
		
        self.davePtr = newDavePtr
        *(self.davePtr) =  *(dummyDavePtr)

;	if (!debug) then begin
;	  message, /info, "davePtr_show, varName= self.davePtr"
;	  help, newDavePtr
;	  help, self.DavePtr
;	  help, dummyDavePtr
;          davePtr_show, self.davePtr, varName= 'self.davePtr'
;	endif

	if (!debug) then help, /heap, /brief
	if (!debug) then print, "*(newDavePtr) = *(dummyDavePtr)"
	if (!debug) then print
	
;Bad!	davePtr_empty, newDavePtr
;Bad!	heap_free, (*newDavePtr)
	(*newDavePtr) = (*dummyDavePtr)	 ; new: update contents of "master" davePtr.

        ptr_free, dummyDavePtr
		
      endif else begin
        self.davePtr = dummyDavePtr
      endelse
 
	if (!debug) then help, /heap, /brief
	if (!debug) then print, "post swapping pointer contents."
	if (!debug) then print

     if (!debug) then begin
       message, /info, string("ptr_valid(newDavePtr)=",ptr_valid(newDavePtr))
       if ( ptr_valid(newDavePtr)) then help, (*newDavePtr)
       print
;       message, /info, "daveptr_show, newDavePtr"
;       daveptr_show, newDavePtr, varName ='newDavePtr'
;       print
;       message, /info,   "daveptr_show, self.DavePtr"
;       daveptr_show, self.DavePtr, varName ='self.DavePtr'
;       print
;       
     endif

  endelse

  if (n_elements(thisOwnerDavePtr) GT 0) then begin
    self.OwnerDavePtr = thisOwnerDavePtr
  endif

  return, OK
    
end		; specObj::initDavePtr


;================================================================================
;*
pro specObj::show, $		; By default, everything.
  lun=thisLun, $ ; Optionally, write to a file. If absent, to screen
  recursive = thisRecursive, $	;  Boolean keyword. Show all levels
  focus = thisFocus, $		; Only the focus, etc.
  heapNum = thisHeapNum		; Show the heap number via HELP
;/

;  header = thisHeader, $	; Only the header
;  data = thisData, $		; Only the data
;  full = thisFull		; Show everything  
    
  showHeader = keyword_set(thisHeader)
  showData = keyword_set(thisData)
  showFocus = keyword_set(thisFocus)

  infolabel = "specObj::show >> "  
  debug = 0

;  showAll = (not showFocus) AND (NOT showHeader) AND (NOT showData)
  showAll = (not showFocus)

;  if (keyword_set(thisFull)) then begin
;    showAll = thisFull
;  endif else begin
;    showAll = ((not showHeader) and (not showData) and  (not showFocus))
;  endelse

;Unit Attributes Name
;-2   Write, Truncate, Tty, Reserved   <stderr>
;-1   Write, Truncate, Tty, Reserved   <stdout>
;0   Read, Tty, Reserved              <stdin>
  if (n_elements(thisLun) eq 0) then begin
      lun= -1
  endif else begin
      lun = thisLun
  endelse

  if (debug) then begin
    print, infolabel, 'showAll=', showAll, ' showFocus=', showFocus, $
	' heapNum =', keyword_set(thisHeapNum)
  endif
  
  if (showFocus or ShowAll) then begin
;    if (keyword_set(thisHeapNum)) then begin
;      help, self.focus, output = helpString
;      printf, lun, '{specObj}.focus= ', helpString   
;    endif
    if (debug) then print, "self.focus -> show, ..."
    if (obj_valid(self.focus)) then begin
      self.focus -> show, lun=lun, heapNum=thisHeapNum 
    endif else begin
      printf, lun, "self.focus=", self.focus, " is INVALID"
    endelse
  endif

;  if (showHeader) then begin
;    printf, lun, 'Header contents:'  
;    (self.focus.header) -> show, lun=thisLun, prefix='self.focus.header', $
;	    recursive=thisrecursive
;  endif
  
;  if (showData) then begin  
;    printf, lun, 'Data contents:'  
;    (self.focus.data) -> show, lun=thisLun, prefix='self.focus.header', $
;	    recursive=thisrecursive
;  endif
  
  if (ShowAll) then begin  
    printf, lun, 'All contents:'  
    if (keyword_set(thisHeapNum)) then begin
      help, self.store, output = helpString
      printf, lun, '{specObj}.store= ', helpString   
    endif
    if (debug) then print, infolabel, 'lun=', lun
    if (debug) then print, "self.store -> show, lun=lun, ..."
 
    if (obj_valid(self.store)) then begin
      (self.store) -> show, lun=Lun, prefix='self.STORE = ', $
	    recursive=thisrecursive, heapNum = thisHeapNum
    endif else begin
      printf, lun, "self.store=", self.store, " is INVALID"
    endelse

;   (self.store) -> show, lun=Lun, prefix='self.STORE = ', $
;	    recursive=thisrecursive, heapNum = thisHeapNum
  endif

end			; specObj::show


;===========================================================================
;*
function specObj::helper_isTag, $	; returns 1 if thisTags1, 0 if it is a #
	thisTags1,$		; input
	from=thisFrom1, $		; input
	inHeader=inHeader1, $		; output
	inData=inData1, $		; output
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	errMSg = thisErrMsg		; output. If there has been an error
;-
	
  debug = 0	;1
  infolabel =  "specObj::helper_isTag >> "
  if (debug) then begin
    print, infolabel, "thisTags1 = ", thisTags1
  endif
    	
  what1 = size(thisTags1, /tname)
  nwhat1 = n_elements(thisTags1)
  if ((what1 eq 'STRING') AND (nWhat1 eq 3)) then begin
    isTag = 1
    ; assume that it is proper tags
    inHeader1 = 0
    inData1 = 1		; default location
    if (strcmp(strtrim(thisFrom1, 2),  'header', /fold_case)) then begin
      InHeader1 = 1
    endif   
    if (strcmp(strtrim(thisFrom1, 2),  'data', /fold_case)) then begin
      InData1 = 1
    endif   
    if (inHeader1 OR inData1) then begin
      OK = self -> getField ( thisTags1 ,  $
        ptr_qty=ptr_qty1, $
  	ptr_err=ptr_err1, ptr_nerr=ptr_nerr1, $
	ptr_legend = ptr_leg1, $
	units=units1, name=name1, $
;	rank = rank1, dim = dim1, $
  	inHeader= InHeader1, inData= InData1)  
      if (NOT OK) then begin
        thisErrMsg = string(infolabel,  "Can not find tag " , $
		string(format_list(thisTags1)), $ 
		" inHeader=", InHeader1, " inData=", InData1)
;        thisErrMsg = string(infolabel,  "Can not find tag ['" , $
;		thisTags1[0], "', '", thisTags1[1], "', '", thisTags1[2], "'] ")
	return, isTag
      endif else begin
        if (ptr_valid(ptr_err1) AND ptr_valid(ptr_nerr1)) then begin
        ; Make the errors symmetric	
	  (*ptr_err1) = ((*ptr_err1) + (*ptr_nerr1))/2.0
	  ptr_free, ptr_nerr1
	endif
      endelse
      
    endif
  endif else begin
    ; assume thisFrom1 = 'input'
    isTag = 0
    ptr_qty1 = ptr_new(thistags1)
    OK = 1

    name1 = strtrim(string(thistags1), 2)		;; ''
    units1 = strtrim(string(thisTags1), 2)		;; ''
  endelse


  if (NOT ptr_valid(ptr_qty1)) then begin
    errMsg = string(infolabel,  " ptr_qty is not defined. name=", name1) 
    message, /info, errMsg
  endif else begin
    if (n_elements((*ptr_qty1)) EQ 0) then begin
      message, /info, string("ptr_qty is is zero-length. name=", name1)
    endif
  endelse
  
  ; Use scalar zero for those errors not being used.
	if (NOT ptr_valid(ptr_err1)) then begin
	  if (n_elements((*ptr_qty1)) EQ 1) then begin
	    tempZero = (*ptr_qty1) - (*ptr_qty1) 
	  endif else begin
	    if (debug) then print, infolabel, "(*ptr_qty1)[0]=", (*ptr_qty1)[0]
	    tempZero = ((*ptr_qty1)[0]) - ((*ptr_qty1)[0]) 
	  endelse
;	  if (debug) then print, infolabel, "zero=", tempZero

	  ptr_err1 = ptr_new(tempZero)	; same type as (*ptr_qty1)
	endif

  if (ptr_valid(ptr_nerr1)) then ptr_free, ptr_nerr1

  return, isTag 

end
  
;=================================================================
;*
function specObj::listBinaryOp		; returns list of recognized operators
;/

;  return, ['+', '-', '*', '/', '^']
  return, qtyObj__listBinaryOp()
end					; specObj::listBinaryOp

;=================================================================
;*
function specObj::BinaryOp, $	; returns 0/1 for failure/success
	; Operates upon items stored at field1 and field2 using qtyObj__binaryOp. 
;;;;	self, $			; specObj class
	thisOp, $			; '+', '-', '*', '/'
	;-----------------------------------------
	tags1 = thistags1, $	; initial value. Text array ['','',''] or float.
	from1 = thisFrom1, $	; 'header', 'data', 'input'
	;-----------------------------------------
	tags2 = thistags2, $
	from2 = thisFrom2, $	; 'header', 'data', 'input'
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;---------------------------------------
	resTags = thisTags, $	; new tags. If not present, use tags1 modified.
	codeSource = thisCodeSource, $ ; Optional codesource for tag.
  	inHeader = thisinHeader,$	; Where place the  results
	inData = thisInData, $	; where place the results
	;----------------------------------------------
	estimate = thisEstimate, $	; Estimate the errors if operator not recognized
	errMsg = thisErrMsg		; Optional error message if failure
	;----------------------------------------------
;	verbose = thisVerbose
;/
	;----------------------------------------------

;  success = 1
  infolabel = "specObj::BinaryOp >> "
  debug = 0
  
  errMsg1 = ' '
  if (debug) then print, infolabel, "tag1=", thisTags1
  isTag1 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags1,$		; input
	from=thisFrom1, $		; input
	inHeader=inHeader1, $		; output
	inData=inData1, $		; output
	errMSg = ErrMsg1)		; output. If there has been an error

  if (n_elements(ErrMsg1) GT 1) then begin	; string array.
    print, infolabel, ErrMsg1
    thisErrMsg = ErrMsg1
    return, 0
  endif
  
  if (strlen(strtrim(ErrMsg1,2)) GT 0) then begin	; single string
;    if (keyword_set(thisVerbose)) then begin
       message, /info, errMsg1
;    endif
    thisErrMsg = ErrMsg1
    return, 0
  endif

  if (debug) then begin
    if (isTag1) then begin
       message, /info, string("item 1 is in databse under ", format_list(thisTags1), $
      	" is a tag")
    endif else begin
       message, /info, string("item 1 is a scalar(?) value ", thisTags1)     
    endelse
  endif
    
  
  ;---------------------------------------------------------------------------------
  errMsg2 = ' '
  isTag2 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags2,$		; input
	from=thisFrom2, $		; input
	inHeader=inHeader2, $		; output
	inData=inData2, $		; output
	errMSg = ErrMsg2)		; output. If there has been an error

  if (n_elements(ErrMsg2) GT 1) then begin
    print, infolabel, errMsg2		; message can't handle string arrays
    return, 0
  endif else begin
    if (strlen(strtrim(ErrMsg2,2)) GT 0) then begin
       message, /info, errMsg2
      return, 0
    endif
  endelse
  
  if (debug) then begin
    if (isTag2) then begin
       print, infolabel, string("item 2 is in databse under ", $
         format_list(thisTags2), " is a tag")
    endif else begin
       message, /info, string("item 2 is a scalar(?) value ", thisTags2)     
    endelse
  endif
  
  ;---------------------------------------------------------------  
  if (isTag1) then begin
    obj1 = self -> get( position = thistags1, $
    	inHeader=inHeader1, inData=inData1)
  endif else begin
    obj1 = thisTags1	; really a scalar
  endelse
  
  ;---------------------------------------------------------------  
  if (isTag2) then begin
    obj2 = self -> get( position = thistags2, $
    	inHeader=inHeader2, inData=inData2)
  endif else begin
    obj2 = thisTags2	; really a scalar
  endelse
  ;---------------------------------------------------------------  
  newObj = qtyObj__binaryOp(obj1, thisOp, obj2, $
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend)	; legend for new results
  	
  ;---------------------------------------------------------------  
  if (NOT obj_valid(newObj)) then begin		; New Aug 18, 2003. TMR.
     thisErrMsg = string("Object failed to be constructed.") 
;     	" name=",  resName, $
;  	" units=",  resUnits, $
;	" legend=",  resLegend, $
;	" qty=", qtyout, $ 
;	" ptr_err=", ptr_errout)
    return, 0
  endif else begin
    OK = newObj -> get(name=resName, units=ResUnits, legend=resLegend)
  endelse
 	     	     

  ;-----------------------------------------------------------  
  if (n_elements(thisTags) GT 0) then begin
    resTags = thisTags
  endif else begin
    resTags = ['qtyObj', '...', 'specObj::BinaryOp']
    if (isTag1) then begin
      label1 = thisTags1[1] 
    endif else begin
      label1 = strtrim(string(thisTags1), 2)	; really a scalar
;      label1 = name1
    endelse 
    if (isTag2) then begin
      label2 = thisTags2[1] 
    endif else begin
      label2 = strtrim(string(thisTags2), 2)	; really a scalar
;      label2 = name2
    endelse 
    resTags[1] = label1 + '_' + thisOp + '_' + label2

  endelse

    if (resTags[0] eq '*') then resTags[0] = 'qtyObj'
    codeSource = 'specObj::BinaryOp'
    if (n_elements(thisCodeSource) GT 0) then begin
      if (thisCodeSource NE '*') then begin
        codeSource = thisCodeSource
      endif
    endif
    resTags[2] = codeSource
  
  success = 1
  theErrMsg = ''
  self -> add, tags = resTags, contents = newObj, $
     inHeader = thisInHeader, inData = thisInData, $
     errMsg = theErrMsg
  if (strlen(strtrim(theErrMsg[0],2)) GT 0) then begin
     for i=0, (n_elements(errMSg) -1) do begin
       message, /info, errMsg[i]
     endfor
     
;     if (keyword_set(thisVerbose)) then begin
       message, /info, string("Replacing rather than adding ", $
      	format_list(resTags))
;       message, /info, string("Replacing rather than add ['", $
;      	resTags[0],"', '", resTags[1],"', '", resTags[2], "']")
;     endif

;     print, infolabel, 'Removing old', resTags
     self -> remove, /destroy, position=resTags, $
     		inHeader = thisInHeader, inData = thisInData, $
     		errMsg = theErrMsg
     success = (theErrMsg EQ '')
;     print, infolabel, 'Adding ', resTags
      
     self -> add, tags = resTags, contents = newObj, $
	     inHeader = thisInHeader, inData = thisInData, $
	     errMsg = theErrMsg
     success = success AND (strlen(theErrMsg) EQ 0)
;     success = success AND (theErrMsg EQ '')
  endif
      
  return, success
end		; specObj::BinaryOp

;=================================================================
;*
function specObj::BinaryOp_old, $	; returns 0/1 for failure/success
	; Operates upon items stored at field1 and field2. 
;;;;	self, $			; specObj class
	thisOp, $			; '+', '-', '*', '/'
	;-----------------------------------------
	tags1 = thistags1, $	; initial value. Text array ['','',''] or float.
	from1 = thisFrom1, $	; 'header', 'data', 'input'
	;-----------------------------------------
	tags2 = thistags2, $
	from2 = thisFrom2, $	; 'header', 'data', 'input'
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;---------------------------------------
	resTags = thisTags, $	; new tags. If not present, use tags1 modified.
	codeSource = thisCodeSource, $ ; Optional codesource for tag.
  	inHeader = thisinHeader,$	; Where place the  results
	inData = thisInData, $	; where place the results
	;----------------------------------------------
	estimate = thisEstimate, $	; Estimate the errors if operator not recognized
	errMsg = thisErrMsg		; Optional error message if failure
	;----------------------------------------------
;	verbose = thisVerbose
;/
	;----------------------------------------------

;  success = 1
  infolabel = "specObj::BinaryOp >> "
  debug = 0
  
  errMsg1 = ' '
  if (debug) then print, infolabel, "tag1=", thisTags1
  isTag1 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags1,$		; input
	from=thisFrom1, $		; input
	inHeader=inHeader1, $		; output
	inData=inData1, $		; output
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	errMSg = ErrMsg1)		; output. If there has been an error

  if (n_elements(ErrMsg1) GT 1) then begin	; string array.
    print, infolabel, ErrMsg1
    thisErrMsg = ErrMsg1
    return, 0
  endif
  
  if (strlen(strtrim(ErrMsg1,2)) GT 0) then begin	; single string
;    if (keyword_set(thisVerbose)) then begin
       message, /info, errMsg1
;    endif
    thisErrMsg = ErrMsg1
    return, 0
  endif

  if (debug) then begin
    if (isTag1) then begin
       message, /info, string("item 1 is in databse under ['", $
      	thisTags1[0],"', '", thisTags1[1],"', '",thisTags1[2], "'] is a tag")
    endif else begin
       message, /info, string("item 1 is a scalar(?) value ", thisTags1)     
    endelse
  endif
    
  
  ;---------------------------------------------------------------------------------
  errMsg2 = ' '
  isTag2 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags2,$		; input
	from=thisFrom2, $		; input
	inHeader=inHeader2, $		; output
	inData=inData2, $		; output
	ptr_qty=ptr_qty2, $		; output
	ptr_err=ptr_err2, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg2, $		; output
	units=units2, $ 		; output
	name=name2, $ 			; output
	errMSg = ErrMsg2)		; output. If there has been an error

  if (n_elements(ErrMsg2) GT 1) then begin
    print, infolabel, errMsg2		; message can't handle string arrays
    return, 0
  endif else begin
    if (strlen(strtrim(ErrMsg2,2)) GT 0) then begin
       message, /info, errMsg2
      return, 0
    endif
  endelse
  
;  if (strlen(strtrim(ErrMsg2,2)) GT 0) then begin
;;    if (keyword_set(thisVerbose)) then begin
;;       message, /info, errMsg2
;       print, infolabel, errMsg2
;;    endif
;    thisErrMsg = ErrMsg2
;    return, 0
;  endif

  if (debug) then begin
    if (isTag2) then begin
       print, infolabel, string("item 2 is in databse under ['", $
      	thisTags2[0],"', '", thisTags2[1],"', '",thisTags2[2], "'] is a tag")
;       message, /info, string("item 2 is in databse under ['", $
;      	thisTags2[0],"', '", thisTags2[1],"', '",thisTags2[2], "'] is a tag")
    endif else begin
       message, /info, string("item 2 is a scalar(?) value ", thisTags2)     
    endelse
  endif
  
  ;---------------------------------------------------------------  
;  errMsg = ''
  case (thisOp) of 
    '+': begin
  	 qtyout  = (*(ptr_qty1)) + (*(ptr_qty2))
    	 end
    '-': begin
 	 qtyout  = (*(ptr_qty1)) - (*(ptr_qty2))
     	 end
    '*': begin
 	 qtyout  = (*(ptr_qty1)) * (*(ptr_qty2))
     	 end
    '/': begin
 	 qtyout  = (*(ptr_qty1)) / (*(ptr_qty2))
     	 end
    '^': begin
 	 qtyout  = (*(ptr_qty1)) ^ (*(ptr_qty2))
     	 end
    else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr = 'qtyout = ' + "( (*ptr_qty1) )" + thisOp + "( (*ptr_qty2) )" 
	    OK = execute(exStr)
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' with qty")
	      thisErrMsg = string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' with qty")
	     return, 0
	     
	    endif
	    if (debug) then print, infolabel,  "estimated qtyout=", qtyout
	 endif else begin
           message, /info, string("Unrecognised binary operator '", thisOp, "'")
	   thisErrMsg = string("Unrecognised binary operator '", thisOp, "'")
	   return, 0
	 endelse
    	 end
   endcase
    
  ;---------------------------------------------------------------  
;  print, infolabel, "ptr_valid(ptr_err*)=",  ptr_valid(ptr_err1) , ptr_valid(ptr_err2)
;  print, infolabel, "(*ptr_err1)=", *(ptr_err1)
;  print, infolabel, "(*ptr_err2)=", *(ptr_err2)
;  if (ptr_valid(ptr_err1) OR ptr_valid(ptr_err2) ) then begin
 
  errIsUsed1 = 1
  if (n_elements(*ptr_err1) eq 1) then begin
    if ((*ptr_err1) eq 0.0) then begin
      errIsUsed1 = 0
    endif
  endif

  errIsUsed2 = 1
  if (n_elements(*ptr_err2) eq 1) then begin
    if ((*ptr_err2) eq 0.0) then begin
      errIsUsed2 = 0
    endif
  endif

  if ( errIsUsed1 OR  errIsUsed2 ) then begin
;  if ( ((*ptr_err1) GT 0.0) OR ((*ptr_err2) GT 0.0) ) then begin
   ; Unused errors are currently a scalar zero.
    case (thisOp) of 
      '+': begin
         errout = sqrt( (*(ptr_err1))^2 + (*ptr_err2)^2)
    	 end
      '-': begin
         errout = sqrt( (*(ptr_err1))^2 + (*ptr_err2)^2)
     	 end
      '*': begin
         errout = sqrt( ( (*ptr_err1) * (*ptr_qty2) )^2 + $
      		     ( (*ptr_err2) * (*ptr_qty1) )^2  ) 
     	 end
      '/': begin
         errout = sqrt( ( (*ptr_err1) / (*ptr_qty2) )^2 + $
       		     ( (*ptr_err2) * (*ptr_qty1) / ((*ptr_qty2)^2) )^2  ) 
     	 end
      '^': begin
         errout = sqrt( ( (*ptr_err1) * $
		(*ptr_qty2) * ((*ptr_err1) ^ (*ptr_qty2 - 1.0)) )^2 + $
       		( (*ptr_err2) * (alog((*ptr_qty1)) * (*ptr_qty1) ^(*ptr_qty2)) $
			 )^2  ) 
     	 end
      else: begin
         if (keyword_set(thisEstimate)) then begin
	    ; this is a hack. Need to discuss with others if this is a reasonable
	    ; approach to estimating the error.
	    OK = 1
	    exStr_pp = "errout_pp =  " $
	     + "( ((*ptr_qty1) + (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) + (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_pp)
	    if (NOT OK) then goto, ex_fail
            errout_pp = abs(qtyout - errout_pp )
	    
	    exStr_pm = "errout_pm =  " $
	     + "( ((*ptr_qty1) + (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) - (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_pm)	    
	    if (NOT OK) then goto, ex_fail
            errout_pm = abs( qtyout - errout_pm )
	    
	    exStr_mp = "errout_mp =  " $
	     + "( ((*ptr_qty1) - (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) + (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_mp)
	    if (NOT OK) then goto, ex_fail
            errout_mp = abs( qtyout - errout_mp )
	    
	    exStr_mm = "errout_mm =  " $
	     + "( ((*ptr_qty1) - (*ptr_err1) ))" + thisOp $
	     + "( ((*ptr_qty2) - (*ptr_err2) ))" 
	    OK = OK AND execute(exStr_mm)
	    if (NOT OK) then goto, ex_fail
            errout_mm = abs( qtyout - errout_mm )

 ex_fail:
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' on estimated errors")
	      thisErrMsg = string("Failure to EXECUTE binary operator'", $
	      		thisOp, "' on estimated errors")
	     return, 0
	    endif

            n_nonzero = 0
	    if (norm(errout_pp) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_pm) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_mp) GT 0.0) then  n_nonzero = n_nonzero + 1
	    if (norm(errout_mm) GT 0.0) then  n_nonzero = n_nonzero + 1

	    
	    errout = 0.0
	    if (n_nonzero GT 0) then begin
	     errout = (errout_pp + errout_pm + errout_mp + errout_mm)
	     errout = errout / n_nonzero
	    endif

	    if (debug) then begin
              print, infolabel, "errout_pp = ", errout_pp
              print, infolabel, "errout_pm = ", errout_pm
              print, infolabel, "errout_mp = ", errout_mp
              print, infolabel, "errout_mm = ", errout_mm
	      print, infolabel, " n_nonzero = ",  n_nonzero
	      print, infolabel,  "estimated errout=", errout 
	    endif

	 endif else begin
           message, /info, string("Unrecognised binary operator '", thisOp, "'")
	   thisErrMsg = string("Unrecognised binary operator '", thisOp, "'")
	   return, 0
	 endelse
   	 end
     endcase
  endif else begin
    if (debug) then  print, infolabel, " No errors"
    errout = 0.0
  endelse
  ;---------------------------------------------------------------  

  if (n_elements(errout) eq 1) then begin  
    if (errout eq 0.0) then begin
      ; ptr_errout is undefined
    endif else begin
      ptr_errout = ptr_new(errout)
    endelse
  endif else begin
    ptr_errout = ptr_new(errout)
  endelse

  ;---------------------------------------------------------------  
  if (n_elements(thisName) GT 0) then begin
    resName = thisName
  endif else begin
    if (n_elements(Name1) GT 0) then begin
      resName = Name1 + ' ' + thisOp + ' ' + Name2
    endif
  endelse

  if (n_elements(thisUnits) GT 0) then begin
    resUnits = thisUnits
  endif else begin
   if (n_elements(Units1) GT 0) then begin
     case (thisOp) of 
      '+': begin
         resUnits = Units1 
   	 end
      '-': begin
         resUnits = Units1 
     	 end
;      '^': begin
;         resUnits = Units1 
;     	 end
      '*': begin
         resUnits = Units1 + ' ' + Units2
     	 end
;      '/': begin
;         resUnits = Units1 + ' ' + thisOp + ' ' + Units2
;      	 end
       else: begin
         resUnits = Units1 + ' ' + thisOp + ' ' + Units2
;         resUnits = string(Units1, thisOp, Units2)
         end
      endcase
   endif
  endelse
;  if (debug) then print, infolabel, "resUnits = ", resUnits
;  if (debug) then print, infolabel, "Units1 = ", Units1
;  if (debug) then print, infolabel, "Units2 = ", Units2

  if (n_elements(thislegend) GT 0) then begin
    reslegend = thisLegend
  endif else begin
    if (ptr_valid(ptr_leg1)) then resLegend = (*ptr_leg1)
  endelse

  newObj = obj_new('qtyObj', name = resName, $
  	units = resUnits, legend = resLegend, $
	qty = qtyout, ptr_err=ptr_errout)	

  if (NOT obj_valid(newObj)) then begin		; New Aug 18, 2003. TMR.
     thisErrMsg = string("Object failed to be constructed.") 
;     	" name=",  resName, $
;  	" units=",  resUnits, $
;	" legend=",  resLegend, $
;	" qty=", qtyout, $ 
;	" ptr_err=", ptr_errout)
    return, 0
  endif
 	     	     

  ;-----------------------------------------------------------  
  if (n_elements(thisTags) GT 0) then begin
    resTags = thisTags
  endif else begin
    resTags = ['qtyObj', '...', 'specObj::BinaryOp']
    if (isTag1) then begin
      label1 = thisTags1[1] 
    endif else begin
      label1 = name1
    endelse 
    if (isTag2) then begin
      label2 = thisTags2[1] 
    endif else begin
      label2 = name2
    endelse 
    resTags[1] = label1 + '_' + thisOp + '_' + label2

;    resTags = thisTags1
;    resTags[1] = thisTags1[1] + '_' + thisOp + '_' + thisTags2[1]
;;    resTags[1] = resTags1[1] + '_' + thisOp
  endelse

    if (resTags[0] eq '*') then resTags[0] = 'qtyObj'
    codeSource = 'specObj::BinaryOp'
    if (n_elements(thisCodeSource) GT 0) then begin
      if (thisCodeSource NE '*') then begin
        codeSource = thisCodeSource
      endif
    endif
    resTags[2] = codeSource
  
  
  if (ptr_valid(ptr_qty1))  then ptr_free, ptr_qty1
  if (ptr_valid(ptr_qty2))  then ptr_free, ptr_qty2
  if (ptr_valid(ptr_err1))  then ptr_free, ptr_err1
  if (ptr_valid(ptr_err2))  then ptr_free, ptr_err2
  if (ptr_valid(ptr_leg1)) then ptr_free, ptr_leg1
  if (ptr_valid(ptr_leg2)) then ptr_free, ptr_leg2
   
  if (ptr_valid(ptr_errout))  then ptr_free, ptr_errout
		
  success = 1
  theErrMsg = ''
  self -> add, tags = resTags, contents = newObj, $
     inHeader = thisInHeader, inData = thisInData, $
     errMsg = theErrMsg
  if (strlen(strtrim(theErrMsg[0],2)) GT 0) then begin
     for i=0, (n_elements(errMSg) -1) do begin
       message, /info, errMsg[i]
     endfor
     
;     if (keyword_set(thisVerbose)) then begin
       message, /info, string("Replacing rather than adding ", $
      	format_list(resTags))
;       message, /info, string("Replacing rather than add ['", $
;      	resTags[0],"', '", resTags[1],"', '", resTags[2], "']")
;     endif

;     print, infolabel, 'Removing old', resTags
     self -> remove, /destroy, position=resTags, $
     		inHeader = thisInHeader, inData = thisInData, $
     		errMsg = theErrMsg
     success = (theErrMsg EQ '')
;     print, infolabel, 'Adding ', resTags
      
     self -> add, tags = resTags, contents = newObj, $
	     inHeader = thisInHeader, inData = thisInData, $
	     errMsg = theErrMsg
     success = success AND (strlen(theErrMsg) EQ 0)
;     success = success AND (theErrMsg EQ '')
  endif
      
  return, success
end		; specObj::BinaryOp_old


;=================================================================
;*
function specObj::listUnitaryOp		; returns list of recognized operators
;/

;  return, ['-', 'cos', 'sin', 'exp', 'alog']
  return, qtyObj__listUnitaryOp()	
end					; specObj::listUnitaryOp

;=================================================================
;*
function specObj::UnitaryOp, $	; returns 0/1 for failure/success
	; Operates upon item stored at field1 using qtyObj__unitaryOp.
	thisOp, $			;  '-', 'cos', 'sin', 'exp', 'alog'
	;-----------------------------------------
	tags1 = thistags1, $	; initial value. Text array ['','',''] or float.
	from1 = thisFrom1, $	; 'header', 'data', 'input'
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;---------------------------------------
	resTags = thisTags, $	; new tags. If not present, use tags1 modified.
	codeSource = thisCodeSource, $ ; Optional codesource for tag.
  	inHeader = thisinHeader,$	; Where place the  results
	inData = thisInData, $	; where place the results
	;-------------------------------------------------
	estimate = thisEstimate	; Estimate the errors if operator not recognized
	;----------------------------------------------
;	verbose = thisVerbose
;/
	;----------------------------------------------

;  success = 1
  infolabel = "specObj::UnitaryOp >> "
  debug = 0	;1
  
  errMsg1 = ' '
  if (debug) then print, infolabel, "tag1=", thisTags1
  isTag1 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags1,$		; input
	from=thisFrom1, $		; input
	inHeader=inHeader1, $		; output
	inData=inData1, $		; output
;	ptr_qty=ptr_qty1, $		; output
;	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
;	ptr_legend=ptr_leg1, $		; output
;	units=units1, $ 		; output
;	name=name1, $ 			; output
	errMSg = ErrMsg1)		; output. If there has been an error

  if (n_elements(ErrMsg1) GT 1) then begin	; string array.
    print, infolabel, ErrMsg1
    thisErrMsg = ErrMsg1
    return, 0
  endif
  
  if (strlen(strtrim(ErrMsg1,2)) GT 0) then begin
;    if (keyword_set(thisVerbose)) then begin
       message, /info, errMsg1
;    endif
    return, 0
  endif

  if (debug) then begin
    if (isTag1) then begin
       message, /info, string("item 1 is in databse under ", format_list(thisTags1), $
      	" is a tag")
;       message, /info, string("item 1 is in databse under ['", $
;      	" is a tag")
;;       message, /info, string("item 1 is in databse under ", format_list(thisTags1), $
;;      	" is a tag")
;;      	thisTags1[0],"', '", thisTags1[1],"', '",thisTags1[2], "'] is a tag")
    endif else begin
       message, /info, string("item 1 is a scalar(?) value ", thisTags1)     
    endelse
  endif
    
  if (isTag1) then begin
    obj1 = self -> get( position = thistags1, $
    	inHeader=inHeader1, inData=inData1)
  endif else begin
    obj1 = thisTags1	; really a scalar
  endelse
  
  ;---------------------------------------------------------------  
  newObj = qtyObj__unitaryOp(thisOp, obj1, $
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend)	; legend for new results
  	
  ;---------------------------------------------------------------  
  if (NOT obj_valid(newObj)) then begin		; New Aug 18, 2003. TMR.
     thisErrMsg = string("Object failed to be constructed.") 
;     	" name=",  resName, $
;  	" units=",  resUnits, $
;	" legend=",  resLegend, $
;	" qty=", qtyout, $ 
;	" ptr_err=", ptr_errout)
    return, 0
  endif else begin
    OK = newObj -> get(name=resName, units=ResUnits, legend=resLegend)
  endelse
 	     	     

  ;-----------------------------------------------------------  
  if (n_elements(thisTags) GT 0) then begin
    resTags = thisTags
  endif else begin
    resTags = ['qtyObj', '...', 'specObj::UnitaryOp']
    if (isTag1) then begin
      label1 = thisTags1[1] 
    endif else begin
      label1 = name1
    endelse 
    resTags[1] = thisOp + '_' + label1

;    resTags = thisTags1
;    resTags[1] = thisOp + '_' + thisTags1[1] 
  endelse

    if (resTags[0] eq '*') then resTags[0] = 'qtyObj'
    codeSource = 'specObj::UnitaryOp'
    if (n_elements(thisCodeSource) GT 0) then begin
      if (thisCodeSource NE '*') then begin
        codeSource = thisCodeSource
      endif
    endif
    resTags[2] = codeSource
  
  
  success = 1
  theErrMsg = ' '
  self -> add, tags = resTags, contents = newObj, $
     inHeader = thisInHeader, inData = thisInData, $
     errMsg = theErrMsg
  if (strlen(strtrim(theErrMsg[0],2)) GT 0) then begin
     for i=0, (n_elements(errMSg) -1) do begin
       message, /info, errMsg[i]
     endfor
     
;     if (keyword_set(thisVerbose)) then begin
       message, /info, string("Replacing rather than adding ", $
      	format_list(resTags))
;       message, /info, string("Replacing rather than add ['", $
;      	resTags[0],"', '", resTags[1],"', '", resTags[2], "']")
;     endif

;     print, infolabel, 'Removing old', resTags
     self -> remove, /destroy, position=resTags, $
     		inHeader = thisInHeader, inData = thisInData, $
     		errMsg = theErrMsg
     success = (theErrMsg EQ '')
;     print, infolabel, 'Adding ', resTags
      
     self -> add, tags = resTags, contents = newObj, $
	     inHeader = thisInHeader, inData = thisInData, $
	     errMsg = theErrMsg
     success = success AND (strlen(theErrMsg) EQ 0)
;     success = success AND (theErrMsg EQ '')
  endif
      
  return, success
end		; specObj::UnitaryOp

;=================================================================
;*
function specObj::UnitaryOp_old, $	; returns 0/1 for failure/success
	; Operates upon item stored at field1.
	thisOp, $			;  '-', 'cos', 'sin', 'exp', 'alog'
	;-----------------------------------------
	tags1 = thistags1, $	; initial value. Text array ['','',''] or float.
	from1 = thisFrom1, $	; 'header', 'data', 'input'
	;------------------------
	; If name, units or legend are not provided, they are taken from tags1.
	name = thisName, $	; Name of results
	units = thisUnits, $	; units for results
	legend = thisLegend, $	; legend for new results
	;---------------------------------------
	resTags = thisTags, $	; new tags. If not present, use tags1 modified.
	codeSource = thisCodeSource, $ ; Optional codesource for tag.
  	inHeader = thisinHeader,$	; Where place the  results
	inData = thisInData, $	; where place the results
	;-------------------------------------------------
	estimate = thisEstimate	; Estimate the errors if operator not recognized
	;----------------------------------------------
;	verbose = thisVerbose
;/
	;----------------------------------------------

;  success = 1
  infolabel = "specObj::UnitaryOp_old >> "
  debug = 0	;1
  
  errMsg1 = ' '
  if (debug) then print, infolabel, "tag1=", thisTags1
  isTag1 = self -> helper_isTag( $	; returns 1 if thisTags1, 0 if it is a #
	thisTags1,$		; input
	from=thisFrom1, $		; input
	inHeader=inHeader1, $		; output
	inData=inData1, $		; output
	ptr_qty=ptr_qty1, $		; output
	ptr_err=ptr_err1, $		; output. Returns symmetric errors.
	ptr_legend=ptr_leg1, $		; output
	units=units1, $ 		; output
	name=name1, $ 			; output
	errMSg = ErrMsg1)		; output. If there has been an error
  if (strlen(strtrim(ErrMsg1,2)) GT 0) then begin
;    if (keyword_set(thisVerbose)) then begin
       message, /info, errMsg1
;    endif
    return, 0
  endif

  if (debug) then begin
    if (isTag1) then begin
       message, /info, string("item 1 is in databse under ['", $
      	thisTags1[0],"', '", thisTags1[1],"', '",thisTags1[2], "'] is a tag")
    endif else begin
       message, /info, string("item 1 is a scalar(?) value ", thisTags1)     
    endelse
  endif
    
  ;---------------------------------------------------------------  
;  errMsg = ''
  case (thisOp) of 
    '-': begin
 	 qtyout  = -(*(ptr_qty1))
     	 end
    'cos': begin
 	 qtyout  = cos(*(ptr_qty1))
     	 end
    'sin': begin
 	 qtyout  = sin(*(ptr_qty1))
     	 end
    'exp': begin
 	 qtyout  = exp(*(ptr_qty1))
     	 end
    'alog': begin
 	 qtyout  = alog(*(ptr_qty1))
     	 end
    else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr = 'qtyout = ' + thisOp + "( (*ptr_qty1) )" 
	    OK = execute(exStr)
;	    if (debug) then begin 
;	      print, 'qty1=', *(ptr_qty1)
;	      print, infolabel, "exStr=", exStr
;	      print, "hardcoded tan=", tan( (*ptr_qty1) )
;	      print, 'qtyout=', qtyout
;	    endif
	    if (NOT OK) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "'")
	     return, 0
	    endif
	 endif else begin
           message, /info, string("Unrecognised unitary operator '", thisOp, "'")
	   return, 0
	 endelse
    	 end
   endcase
    
  ;---------------------------------------------------------------  
  errIsUsed1 = 1
  if (n_elements(*ptr_err1) eq 1) then begin
    if ((*ptr_err1) eq 0.0) then begin
      errIsUsed1 = 0
    endif
  endif


  if ( errIsUsed1 ) then begin
;  if (*ptr_err1 GT 0.0)  then begin
;  if (ptr_valid(ptr_err1) ) then begin
   ; Unused errors are currently a scalar zero.
    case (thisOp) of 
      '-': begin
         errout = *(ptr_err1)
     	 end
      'cos': begin
         errout = abs((*ptr_err1) *  sin(*(ptr_qty1)))
     	 end
      'sin': begin
         errout = abs((*ptr_err1) * cos(*(ptr_qyt1)))
     	 end
      'exp': begin
         errout = (*ptr_err1) * exp(*(ptr_qty1))
     	 end
      'alog': begin
         errout = (*ptr_err1) / (*(ptr_qty1))
     	 end
      else: begin
         if (keyword_set(thisEstimate)) then begin
	    exStr_plus = "errout_plus = " +  thisOp + $
	    	'( ( (*ptr_qty1) + (*ptr_err1) ) ) '
	    OK_plus = execute(exStr_plus)
	    if (NOT OK_plus) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "' on positive errors")
	     return, 0
	    endif
            errout_plus = abs( (*ptr_qty1) - errout_plus )
	    
	    exStr_minus = "errout_minus = " + thisOp + $
	    	'( ( (*ptr_qty1) - (*ptr_err1) ) )'
	    OK_minus = execute(exStr_minus)
	    errout = abs(errout_plus - errout_minus)/2.0
	    if (NOT OK_minus) then begin
              message, /info, string("Failure to EXECUTE unitary operator'", $
	      		thisOp, "' on negative errors")
	     return, 0
	    endif
            errout_minus = abs( (*ptr_qty1) - errout_minus )

	 endif else begin
           message, /info, string("Unrecognised unitary operator '", thisOp, "'")
	   return, 0
	 endelse
    	 end
     endcase
   endif else begin
    if (debug) then  print, infolabel, " No errors"
    errout = 0.0
  endelse
  ;---------------------------------------------------------------  

  if (n_elements(errout) eq 1) then begin  
    if (errout eq 0.0) then begin
      ; ptr_errout is undefined
    endif else begin
      ptr_errout = ptr_new(errout)
    endelse
  endif else begin
    ptr_errout = ptr_new(errout)
  endelse

  ;---------------------------------------------------------------  
  if (n_elements(thisName) GT 0) then begin
    resName = thisName
  endif else begin
    if (n_elements(Name1) GT 0) then begin
      resName = string(thisOp, '(', Name1, ')')
    endif
  endelse

  if (n_elements(thisUnits) GT 0) then begin
    resUnits = thisUnits
  endif else begin
   if (n_elements(Units1) GT 0) then begin
     case (thisOp) of 
      '-': begin
         resUnits = Units1 
     	 end
      else: begin
         resUnits = string(thisOp, '(', Units1, ')')
         end
      endcase
   endif
  endelse
;  if (debug) then print, infolabel, "resUnits = ", resUnits
  

  if (n_elements(thislegend) GT 0) then begin
    reslegend = thisLegend
  endif else begin
    if (ptr_valid(ptr_leg1)) then resLegend = (*ptr_leg1)
  endelse

  newObj = obj_new('qtyObj', name = resName, $
  	units = resUnits, legend = resLegend, $
	qty = qtyout, ptr_err=ptr_errout)	
  	     	     

  ;-----------------------------------------------------------  
  if (n_elements(thisTags) GT 0) then begin
    resTags = thisTags
  endif else begin
    resTags = ['qtyObj', '...', 'specObj::UnitaryOp']
    if (isTag1) then begin
      label1 = thisTags1[1] 
    endif else begin
      label1 = name1
    endelse 
    resTags[1] = thisOp + '_' + label1

;    resTags = thisTags1
;    resTags[1] = thisOp + '_' + thisTags1[1] 
  endelse

    if (resTags[0] eq '*') then resTags[0] = 'qtyObj'
    codeSource = 'specObj::UnitaryOp'
    if (n_elements(thisCodeSource) GT 0) then begin
      if (thisCodeSource NE '*') then begin
        codeSource = thisCodeSource
      endif
    endif
    resTags[2] = codeSource
  
  
  if (ptr_valid(ptr_qty1))  then ptr_free, ptr_qty1
  if (ptr_valid(ptr_err1))  then ptr_free, ptr_err1
  if (ptr_valid(ptr_leg1)) then ptr_free, ptr_leg1
   
  if (ptr_valid(ptr_errout))  then ptr_free, ptr_errout
		
  success = 1
  theErrMsg = ' '
  self -> add, tags = resTags, contents = newObj, $
     inHeader = thisInHeader, inData = thisInData, $
     errMsg = theErrMsg
  if (strlen(strtrim(theErrMsg[0],2)) GT 0) then begin
     for i=0, (n_elements(errMSg) -1) do begin
       message, /info, errMsg[i]
     endfor
     
;     if (keyword_set(thisVerbose)) then begin
       message, /info, string("Replacing rather than add ['", $
      	resTags[0],"', '", resTags[1],"', '", resTags[2], "']")
;     endif

;     print, infolabel, 'Removing old', resTags
     self -> remove, /destroy, position=resTags, $
     		inHeader = thisInHeader, inData = thisInData, $
     		errMsg = theErrMsg
     success = (theErrMsg EQ '')
;     print, infolabel, 'Adding ', resTags
      
     self -> add, tags = resTags, contents = newObj, $
	     inHeader = thisInHeader, inData = thisInData, $
	     errMsg = theErrMsg
     success = success AND (theErrMsg EQ '')
  endif
      
  return, success
end		; specObj::UnitaryOp_old


;================================================================================
;*
pro specObj__save, $. 		; "Friend" function, using IDL save
   oSpec, $			; object of class SpecObj
   file = thisFile		; Defaults to 'oSpec.osp'
;\

  infolabel = "specObj__save >> "  

  if (n_elements(thisFile) eq 0) then begin
    thisFile = 'oSpec.osp'
    print,  infolabel, $
    	' No file keyword provided. Writing to oSpec.osp.'
  endif

  save, oSpec, file=thisFile
  
end

;================================================================================
;*
pro specObj__restore, $ 	; "Friend" function, using IDL restore
   oSpec, $			; object of class SpecObj
   file = thisFile, $		; Defaults to 'oSpec.osp'
   verbose=thisVerbose
;\

  infolabel = "specObj__restore >> "

  if (n_elements(thisFile) eq 0) then begin
;    print,  infolabel, $
;    	' No file keyword provided. Writing to daveptr.dave.'
    filename = 'oSpec.osp'
  endif else begin
    filename = thisFile
  endelse
  
  restore, filename, verbose=thisVerbose

end

;================================================================================
;*
function specObj::textInfo, $ 	; Returns a string array
				; By default, /geteverything.
  ;----------- Requires one of the following two --------------
  inFocus = thisInFocus, $
  inHeader = thisInHeader, $
  inData = thisInData, $
  ;-----------------------------------------------
  position = thisPosition, $	; Optionally only ask for one item in the 
  		; Header or in data (not focus).
      ; Either integer position or tags position ['class', 'label', 'codeSource']
  ; ------------- Select one of the 3 following keywords. --------------
  getSome=thisGetSome, $; Boolean keyword.
  			; 1 line summary of one qtyObj Object. No vectorvalues.
  getAll=thisGetAll, $  ; Bolean keyword
  			; 1 line summary of one qtyObj Object
  getEverything=thisGetEverything, $	; Boolean keyword
  		; String array containing all contents of one qtyObjObObject
  showTags = thisShowTags, $
  showAnnotations = thisShowAnnotations, $
  showPendent = thisShowPendent, $
  onlyAnnotations = thisOnlyAnnotations
;/

  infolabel = "specObj::textInfo >> "
  debug = 0
  
  if ( (NOT keyword_set(thisGetSome)) AND (NOT keyword_set(thisGetAll)) $
  	AND (NOT keyword_set(thisGetEverything)) ) then begin
    thisGetEverything = 1
  endif

  inHeader = 1
  inHeader = keyword_set(thisInHeader)
  inData = keyword_set(thisInData)
  inFocus = keyword_set(thisInFocus)
  
  if (debug) then print, infolabel, "keyword_set(thisShowAnnotations) =",$
  	 keyword_set(thisShowAnnotations)
  if (debug) then print, infolabel, "keyword_set(thisOnlyAnnotations) =",$
  	 keyword_set(thisOnlyAnnotations)

  strArray = ''
  oneString = ''

  if (inFocus) then begin
;   print, infolabel, "n_elements(strArray) = ", n_elements(strArray)
   self.focus -> show, strArray=focusArray
;   print, infolabel, "n_elements(focusArray) = ", n_elements(focusArray)
;   print, infolabel, " focusArray=", focusArray
   return, focusArray
  endif
  
  if (n_elements(thisPosition) EQ 0) then begin			; Do all
    ncount = self -> count(inHeader=inHeader, inData=inData)
    oneString = ''
    if (debug) then print, infolabel, " ncount =", ncount
    if (keyword_set(thisGetSome)) then begin
      for i = 0, (ncount -1) do begin

        if (keyword_set(thisShowTags)) then begin
          OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
		class = theClass, label=theLabel, codeSource=theCodeSource)
          tagString = string('[', theClass, ', ', theLabel, ', ',  theCodeSource, ']')
	  strArray = [tagString, strArray]
        endif 
        if (debug) then print, infolabel, 'getSome ', i
	pStr = ''
	if (keyword_set(thisShowpendent) ) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			pendent=pStr)
	  pStr = pStr + ' '
	endif
	isAnnotated = 1
	if (keyword_set(thisShowAnnotations) OR $
		 keyword_set(thisOnlyAnnotations)) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			annotate=isAnnotated)
	endif
	if (keyword_set(thisShowAnnotations)) then begin
	  if (IsAnnotated) then anString = '*' else anString = ' '		
	endif else begin
	  anString = ''	  
	endelse
	if (isAnnotated OR keyword_set(thisShowAnnotations) ) then begin
          OK = self -> getField(i, inHeader=inHeader, inData=inData, $
			some=oneString) 
          strArray = [strArray, string(pStr, anString, onestring)]
	endif
      endfor
    endif 
    
    
    if (keyword_set(thisGetAll)) then begin
      for i = 0, (ncount -1) do begin
        if (keyword_set(thisShowTags)) then begin
          OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
		class = theClass, label=theLabel, codeSource=theCodeSource)
          tagString = string('[', theClass, ', ', theLabel, ', ',  theCodeSource, ']')
	  strArray = [tagString, strArray]
        endif 
        if (debug) then print, infolabel, 'getAll ', i
	pStr = ''
	if (keyword_set(thisShowpendent) ) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			pendent=pStr)
	  pStr = pStr + ' '
	endif
	isAnnotated = 1
	if (keyword_set(thisShowAnnotations) OR $
		 keyword_set(thisOnlyAnnotations)) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			annotate=isAnnotated)
	endif
	if (keyword_set(thisShowAnnotations)) then begin
	  if (IsAnnotated) then anString = '*' else anString = ' '		
	endif else begin
	  anString = ''	  
	endelse
	if (isAnnotated OR keyword_set(thisShowAnnotations) ) then begin
          OK = self -> getField(i, inHeader=inHeader, inData=inData, $
			all=oneString) 
          strArray = [strArray, string(pStr, anString, onestring)]
 	endif
      endfor
    endif 
    if (keyword_set(thisGetEverything)) then begin
      for i = 0, (ncount -1) do begin
        if (keyword_set(thisShowTags)) then begin
          OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
		class = theClass, label=theLabel, codeSource=theCodeSource)
          tagString = string('[', theClass, ', ', theLabel, ', ',  theCodeSource, ']')
	  strArray = [tagString, strArray]
        endif 
        if (debug) then print, infolabel, 'getEverything ', i
	pStr = ''
	if (keyword_set(thisShowpendent) ) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			pendent=pStr)
	  pStr = pStr + ' '
	endif
	isAnnotated = 1
	if (keyword_set(thisShowAnnotations) OR $
		 keyword_set(thisOnlyAnnotations)) then begin
	  OK = self -> getTagField(i, inHeader=inHeader, inData=inData, $
			annotate=isAnnotated)
	endif
	if (keyword_set(thisShowAnnotations)) then begin
	  if (IsAnnotated) then anString = '*' else anString = ' '		
	endif else begin
	  anString = ''	  
	endelse
	if (isAnnotated OR keyword_set(thisShowAnnotations) ) then begin
          OK = self -> getField(i, inHeader=inHeader, inData=inData, $
			everything=oneString) 
	  if (n_elements(oneString) EQ 1) then begin
	    oneString = string(pString, anString, oneString)
	  endif else begin
	    oneString[0] = string(pStr, anString, oneString[0])
	  endelse
        strArray = [strArray, onestring]
	endif
     endfor
    endif 

    ;---------------------------------------------------------------
  endif else begin	; Only want 1 item from the data list

    if (keyword_set(thisGetSome)) then begin
     OK = self -> getField(thisPosition, inHeader=inHeader, inData=inData, $
			some=StrArray)
    endif 
    if (keyword_set(thisGetAll)) then begin
     OK = self -> getField(thisPosition, inHeader=inHeader, inData=inData, $
			all=StrArray)
    endif 
    if (keyword_set(thisGetEverything)) then begin
     OK = self -> getField(thisPosition, inHeader=inHeader, inData=inData, $
			everything=StrArray)
    endif 

    if (keyword_set(thisShowTags)) then begin
     OK = self -> getTagField(thisPosition, inHeader=inHeader, inData=inData, $
		class = theClass, label=theLabel, codeSource=theCodeSource)
     StrArray = [string('[', theClass, ', ', theLabel, ', ',  $
     	theCodeSource, ']'),$
        StrArray]
    endif 
    
  endelse
  
  if (debug) then begin
    if (n_elements(thisPosition) GT 0) then begin
      print, infolabel, "position=", thisPosition
      print, strArray
    endif 
  endif
   
  return, StrArray
end		;; specObj::textInfo

;=============================================================================
;*
function specObj::Count, $	:Returns an integer with the # items in header or data.
  inHeader = thisInHeader,  $	; Optional boolean keyword
  inData = thisInData, $	; Optional boolean keyword
  ; Alternatively, get object from self.store (one level up).
  inStore = thisInStore		; Boolean keyword.
  				; If used, keywords inHeader and inData ignored
  	; Default is the count of DAVE_tagContainer objects in self.store
;/
  
  
  if (keyword_set(thisInHeader)) then begin
    OK = (self.focus) -> get(headerObj=tempObj)
    if (OK) then begin
      ncount =  tempObj -> count() 
      return, ncount   
    endif else begin
      return, -1
    endelse      

  endif

  if (keyword_set(thisInData)) then begin
    OK = (self.focus) -> get(dataObj=tempObj)
    if (OK) then begin
      ncount = tempObj -> count() 
      return, ncount
    endif else begin
      return, -1
    endelse      
  endif
  
  if (NOT keyword_set(thisInHeader)) and (NOT keyword_set(thisInData)) then begin
    ncount = self.store -> count()
    return, ncount
  endif
end		;; specObj::Count
 

;;================================================================================
;;+
;; NAME:
;; 	specObj__SubstantiateMethods:
;;
;; PURPOSE:
;;	This proceedure is intended to be run at the beginning of DAVE, so that 
;;	all the methods of specObj and its associated objects are
;;	loaded into IDL, by initializing soem onjects. Then, if data objects
;	are restored from disk, the methods are available.  
;;-;
;;*
;pro specObj__SubstantiateMethods
;;/
;
;  spec_dummy = obj_new('specObj')
;  ; specObj initializes dave_tagContainer, focusObj, and tagObj as well.
;  
;  qty_dummy = obj_new('qtyObj')
;
;;  check_heap
;
;  obj_destroy, spec_dummy
;  obj_destroy, qty_dummy
;
;;  check_heap
;
;end

;================================================================================
;*
pro specObj::Cleanup			; , $
;	skipDavePtr = thisSkipDavePtr		; Soon to be obsolete
;/

  infolabel = "specObj::cleanup >> "  
  debug = 0	;1
  
  obj_destroy, self.store
  obj_destroy, self.focus

  if (debug) then begin
    message, /info, string("self.ownerDavePtr=", self.ownerDavePtr) 
;    	, $ " keyword_set(skipDavePtr) =", keyword_set(skipDavePtr))
  endif
  
  if (self.ownerDavePtr eq 'local') then begin
    if (ptr_valid(self.DavePtr)) then begin
      DavePtr_cleanup, self.DavePtr, /skipSpecObJ	; Completely empty
    endif
  endif else begin
    ;;;; (self.ownerDavePtr eq 'main')
    if (ptr_valid(self.DavePtr)) then begin	; Don't touch davePtr at all.
      ; Place a null object reference there.
      if (DavePtr_containsSpecObj(self.DavePtr)) then begin
        (*(*(*(self.daveptr)).datastrptr).specificptr).specObj = obj_new()
      endif
    endif
  endelse

;  if (NOT keyword_set(thisSkipDavePtr)) then begin
;    if (ptr_valid(self.DavePtr)) then begin
;      DavePtr_empty, self.DavePtr, /derefSpecObj	; leave a sketelton.
;;      DavePtr_cleanup, self.DavePtr, /skipSpecObJ	; Completely empty
;    endif
;  endif else begin
;    if (ptr_valid(self.DavePtr)) then begin	; Don't touch davePtr at all.
;      ; Place a null object reference there.
;      if (DavePtr_containsSpecObj(self.DavePtr)) then begin
;        (*(*(*(self.daveptr)).datastrptr).specificptr).specObj = obj_new()
;      endif
;    endif
;  endelse

  
  ; the other objects are extra references to objects already destroyed above.

;Debug  print, infolabel, 'Entered cleanup'

end		; :: cleanup

;================================================================================
;*
pro specObj__define

  infolabel = "specObj__define >> "  
  define = {specObj, $
  	store: obj_new(), $	; class DAVE_tagContainer
	focus: obj_new(), $	; class focusObj 
	davePtr: ptr_new(), $ ; DAVE pointer, including this object in specificPtr.
	ownerDavePtr: '', $	; string with either 'local' or 'main' (from DAVE program) 
	wID: 0L $ 		; Widget ID associated with this object.
	}
;/

end
